Package: rfriend
Type: Package
Title: Provides Batch Functions and Visualisation for Basic Statistical
        Procedures
Version: 3.1.0
Authors@R: c(person("Sander H.", "van Delden", role = c("aut", "cre"), email = "plantmind@proton.me"))
Description: Designed to streamline data analysis and statistical testing, reducing the length of R 
    scripts while generating well-formatted outputs in 'pdf', 'Microsoft Word', and 'Microsoft Excel' 
    formats. In essence, the package contains functions which are sophisticated wrappers around 
    existing R functions that are called by using 'f_' (user f_riendly) prefix followed by the normal 
    function name. This third version of the 'rfriend' package focuses  primarily on data exploration, 
    including tools for creating summary tables, f_summary(), summary figures, f_scan(), 
    outlier detection and removal, f_outlier() and f_remove_outliers(), performing data transformations,
    f_boxcox() in part based on 'MASS/boxcox' and 'rcompanion', and f_bestNormalize() 
    which wraps and extends functionality from the 'bestNormalize' package. Furthermore, 'rfriend' 
    can automatically (or on request) generate visualizations such as boxplots, f_boxplot(), 
    QQ-plots, f_qqnorm(), histograms f_hist(), and density plots f_density(). Additionally, the package 
    includes several statistical test functions: f_aov(), f_chisq_test(), f_corplot(), f_kruskal_test(), 
    f_lmer(), f_glm(), f_t_test(), f_wilcox_test(),  for sequential testing and visualisation of the 
    similar named 'stats' functions. These functions, except for f_chisq_test(), support testing multiple
    response variables and predictors, while also handling assumption checks, data transformations, 
    and post hoc tests. Post hoc results are automatically summarized in a table using the compact letter 
    display (cld) format for easy interpretation. The package also provides a function to do model 
    comparison, f_model_comparison(), and several utility functions to simplify common R tasks. 
    For example, f_clear() clears the workspace and restarts R with a single command; f_setwd() sets 
    the working directory to match the directory of the current script; f_theme() quickly changes 
    'RStudio' themes; and f_factors() converts multiple columns of a data frame to factors, and much more. 
    If you encounter any issues or have feature requests, please feel free to contact me via email.
Note: When loading, both MuMIn and rstatix are imported. Since rstatix
        internally depends on broom, this may trigger a warning about
        S3 method overwrites, specifically for nobs.fitdistr and
        nobs.multinom. These warnings are harmless and do not affect
        functionality.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.4.0)
Imports: bestNormalize, crayon, DHARMa, dplyr, emmeans, ggplot2,
        grDevices, gridExtra, knitr, lme4, lmerTest, magick, multcomp,
        multcompView, MuMIn, nortest, pander, png, rlang, rmarkdown,
        rstatix, rstudioapi, this.path, tidyr, writexl, xfun
Suggests: MASS, nnet, pbkrtest, testthat (>= 3.0.0), tibble
Config/testthat/edition: 3
SystemRequirements: Pandoc (>= 3.2)
NeedsCompilation: no
Packaged: 2026-05-30 18:29:52 UTC; sander
Author: Sander H. van Delden [aut, cre]
Maintainer: Sander H. van Delden <plantmind@proton.me>
URL: https://delde001.github.io/rfriend/
Config/roxygen2/version: 8.0.0
Repository: CRAN
Date/Publication: 2026-05-30 19:10:02 UTC
Built: R 4.5.2; ; 2026-05-30 22:39:46 UTC; unix

Package: rfriend
Type: Package
Title: Provides Batch Functions and Visualisation for Basic Statistical
        Procedures
Version: 3.2.0
Authors@R: c(person("Sander H.", "van Delden", role = c("aut", "cre"), email = "plantmind@proton.me"))
Description: Designed to streamline data analysis and statistical testing, reducing the length of R 
    scripts while generating well-formatted outputs in 'pdf', 'Microsoft Word', and 'Microsoft Excel' 
    formats. In essence, the package contains functions which are sophisticated wrappers around 
    existing R functions that are called by using 'f_' (user f_riendly) prefix followed by the normal 
    function name. This third version of the 'rfriend' package focuses  primarily on data exploration, 
    including tools for creating summary tables, f_summary(), summary figures, f_scan(), 
    outlier detection and removal, f_outlier() and f_remove_outliers(), performing data transformations,
    f_boxcox() in part based on 'MASS/boxcox' and 'rcompanion', and f_bestNormalize() 
    which wraps and extends functionality from the 'bestNormalize' package. Furthermore, 'rfriend' 
    can automatically (or on request) generate visualizations such as boxplots, f_boxplot(), 
    QQ-plots, f_qqnorm(), and histograms f_hist(). Additionally, the package includes several 
    statistical test functions: f_aov(), f_chisq_test(), f_corplot(), f_kruskal_test(), 
    f_lmer(), f_glm(), f_t_test(), f_wilcox_test(),  for sequential testing and visualisation of the 
    similar named 'stats' functions. These functions, except for f_chisq_test(), support testing multiple
    response variables and predictors, while also handling assumption checks, data transformations, 
    and post hoc tests. Post hoc results are automatically summarized in a table using the compact letter 
    display (cld) format for easy interpretation. The package also provides a function to do model 
    comparison, f_model_comparison(), and several utility functions to simplify common R tasks. 
    For example, f_clear() clears the workspace and restarts R with a single command; f_setwd() sets 
    the working directory to match the directory of the current script; f_theme() quickly changes 
    'RStudio' themes; and f_factors() converts multiple columns of a data frame to factors, and much more. 
    If you encounter any issues or have feature requests, please feel free to contact me via email.
Note: When loading, both MuMIn and rstatix are imported. Since rstatix
        internally depends on broom, this may trigger a warning about
        S3 method overwrites, specifically for nobs.fitdistr and
        nobs.multinom. These warnings are harmless and do not affect
        functionality.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.4.0)
Imports: bestNormalize, crayon, car, DHARMa, dplyr, emmeans, ggplot2,
        grDevices, gridExtra, knitr, lifecycle, lme4, lmerTest, magick,
        MuMIn, nortest, pander, png, rlang, rmarkdown, rstatix,
        rstudioapi, this.path, tidyr, writexl, xfun
Suggests: MASS, nnet, pbkrtest, testthat (>= 3.0.0), tibble, readxl
Config/testthat/edition: 3
SystemRequirements: Pandoc (>= 3.2)
NeedsCompilation: no
Packaged: 2026-07-06 20:04:10 UTC; shvan
Author: Sander H. van Delden [aut, cre]
Maintainer: Sander H. van Delden <plantmind@proton.me>
URL: https://delde001.github.io/rfriend/
Config/roxygen2/version: 8.0.0
LazyData: true
RoxygenNote: 8.0.0
Repository: CRAN
Date/Publication: 2026-07-06 20:30:02 UTC
Built: R 4.6.0; ; 2026-07-06 23:53:47 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(f_boxplot,data.frame)
S3method(f_boxplot,formula)
S3method(f_boxplot,integer)
S3method(f_boxplot,numeric)
S3method(f_outliers,data.frame)
S3method(f_outliers,formula)
S3method(f_outliers,integer)
S3method(f_outliers,numeric)
S3method(f_scan,data.frame)
S3method(f_scan,formula)
S3method(f_scan,integer)
S3method(f_scan,numeric)
S3method(f_stat_wizard,data.frame)
S3method(f_stat_wizard,formula)
S3method(f_summary,data.frame)
S3method(f_summary,formula)
S3method(f_t_test,default)
S3method(f_t_test,formula)
S3method(f_wilcox_test,default)
S3method(f_wilcox_test,formula)
S3method(plot,f_aov)
S3method(plot,f_bestNormalize)
S3method(plot,f_boxcox)
S3method(plot,f_friedman)
S3method(plot,f_glm)
S3method(plot,f_kruskal_test)
S3method(plot,f_lm)
S3method(plot,f_lmer)
S3method(plot,f_long)
S3method(plot,f_scan)
S3method(plot,f_t_test)
S3method(plot,f_wilcox_test)
S3method(predict,f_boxcox)
S3method(print,f_aov)
S3method(print,f_bestNormalize)
S3method(print,f_boxcox)
S3method(print,f_chisq_test)
S3method(print,f_friedman)
S3method(print,f_glm)
S3method(print,f_kruskal_test)
S3method(print,f_lm)
S3method(print,f_lmer)
S3method(print,f_model_comparison)
S3method(print,f_outliers)
S3method(print,f_scan)
S3method(print,f_stat_wizard)
S3method(print,f_summary)
S3method(print,f_t_test)
S3method(print,f_wilcox_test)
S3method(summary,f_long)
S3method(summary,f_scan)
export(df_to_table)
export(f_aov)
export(f_bestNormalize)
export(f_boxcox)
export(f_boxplot)
export(f_boxplot_worker)
export(f_chisq_test)
export(f_clear)
export(f_conditional_round)
export(f_corplot)
export(f_example_data)
export(f_factors)
export(f_friedman)
export(f_glm)
export(f_hist)
export(f_kruskal_test)
export(f_lm)
export(f_lmer)
export(f_load_packages)
export(f_long)
export(f_model_compare)
export(f_open_file)
export(f_outliers)
export(f_pander)
export(f_qqnorm)
export(f_remove_outliers)
export(f_rename_columns)
export(f_rename_vector)
export(f_scan)
export(f_setwd)
export(f_stat_wizard)
export(f_summary)
export(f_t_test)
export(f_theme)
export(f_wilcox_test)
import(ggplot2)
import(knitr)
import(lifecycle)
import(rmarkdown)
import(stats)
importFrom(DHARMa,plotResiduals)
importFrom(DHARMa,simulateResiduals)
importFrom(DHARMa,testDispersion)
importFrom(DHARMa,testResiduals)
importFrom(MuMIn,r.squaredGLMM)
importFrom(bestNormalize,bestNormalize)
importFrom(car,Anova)
importFrom(car,ncvTest)
importFrom(crayon,red)
importFrom(dplyr,select)
importFrom(emmeans,emmeans)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.off)
importFrom(grDevices,graphics.off)
importFrom(grDevices,hsv)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,recordPlot)
importFrom(grDevices,replayPlot)
importFrom(grDevices,rgb)
importFrom(grDevices,rgb2hsv)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,stripchart)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(grid,grid.raster)
importFrom(grid,unit)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(knitr,knit_child)
importFrom(knitr,knit_hooks)
importFrom(knitr,opts_knit)
importFrom(lme4,VarCorr)
importFrom(lme4,isSingular)
importFrom(lme4,lmerControl)
importFrom(lme4,ngrps)
importFrom(lme4,nobars)
importFrom(lme4,ranef)
importFrom(lmerTest,lmer)
importFrom(magick,image_read)
importFrom(magick,image_scale)
importFrom(nortest,ad.test)
importFrom(pander,pander)
importFrom(pander,panderOptions)
importFrom(rlang,.data)
importFrom(rstatix,dunn_test)
importFrom(rstatix,levene_test)
importFrom(rstudioapi,documentId)
importFrom(rstudioapi,documentPath)
importFrom(rstudioapi,getActiveDocumentContext)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,aov)
importFrom(stats,as.formula)
importFrom(stats,chisq.test)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,dnorm)
importFrom(stats,drop1)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,hatvalues)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,nobs)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,ppoints)
importFrom(stats,predict)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,sigma)
importFrom(stats,spline)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(this.path,this.dir)
importFrom(tidyr,all_of)
importFrom(tidyr,pivot_longer)
importFrom(utils,as.roman)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,tail)
importFrom(writexl,write_xlsx)
importFrom(xfun,dir_exists)
importFrom(xfun,split_lines)

# rfriend

[![CRAN Version](https://www.r-pkg.org/badges/version/rfriend)](https://CRAN.R-project.org/package=rfriend)
[![License: GPL-3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.R-project.org/Licenses/GPL-3)

## Overview

`rfriend` is an R package designed to streamline data analysis and statistical
testing by wrapping complex or repetitive code into convenient, user-friendly
functions prefixed with `f_` (f_riendly). Its primary focus is data
exploration, statistical tests, and creating publication-ready output in
multiple formats including PDF, Microsoft Word, and Microsoft Excel.

The package helps you write shorter code while producing well-formatted
summaries, visualizations, and statistical tests, automatically handling data
transformations, assumption checking, and post hoc testing.

Full documentation is available at
<https://delde001.github.io/rfriend/>.

---

## Features

### Data exploration

- **Summary tables:** `f_summary()` generates comprehensive summary tables
  (with optional skewness, kurtosis, and confidence intervals).
- **Diagnostic dashboards:** `f_scan()` builds a density / boxplot / QQ panel
  for one or more variables, optionally split by grouping factors.
- **Correlation plots:** `f_corplot()` shows Pearson, Spearman, and Kendall
  coefficients together, with support for ordinal variables.
- **Outlier handling:** `f_outliers()` flags outliers using Tukey's fences and
  `f_remove_outliers()` removes them with safe anti-join semantics.
- **Test recommendation (beta):** `f_stat_wizard()` inspects your data from a
  formula and suggests an appropriate test as ready-to-run code.

### Data transformations

- `f_boxcox()` for Box-Cox transformations (wrapping MASS/boxcox and
  rcompanion).
- `f_bestNormalize()` wraps and extends normalization from the `bestNormalize`
  package.

### Visualizations

- Boxplots (`f_boxplot()`), including numeric-vector input and custom palettes
- QQ-plots (`f_qqnorm()`)
- Histograms (`f_hist()`)

### Statistical tests

Run and visualize tests on multiple response variables and predictors, with
automatic assumption checking and post hoc analysis:

- ANOVA: `f_aov()`
- Linear regression (OLS): `f_lm()`
- Linear mixed-effects models: `f_lmer()`
- Kruskal-Wallis: `f_kruskal_test()`
- Friedman rank sum test: `f_friedman()`
- t-test: `f_t_test()`
- Wilcoxon test: `f_wilcox_test()`
- Generalized linear models: `f_glm()`
- Chi-square tests: `f_chisq_test()`

Post hoc results are summarized in a compact letter display (cld) table for
easy interpretation.

### Model comparison

- Compare models easily with `f_model_compare()`.

### Utilities

- `f_clear()` clears the workspace and restarts R.
- `f_setwd()` sets the working directory to the current script's location.
- `f_theme()` switches RStudio themes quickly.
- `f_factors()` converts multiple data frame columns to factors (with optional
  reference-level control).
- `f_long()` reshapes wide data to long format in a single call.
- `f_example_data()` lists and returns the bundled example datasets.
- `f_open_file()`, `f_pander()`, `f_rename_columns()`, `f_rename_vector()`,
  `f_load_packages()`, and more.

---

## Installation

`rfriend` requires R (>= 4.4.0) and several dependencies (see DESCRIPTION for
details). PDF and Word output require Pandoc (>= 3.2), which is bundled with
RStudio.

Install the latest released version from CRAN with:

```r
install.packages("rfriend")
```

To install the latest development version from GitHub:

```r
# install.packages("devtools")
devtools::install_github("delde001/rfriend")
```

---

## Basic Usage

```r
library(rfriend)

# Summary of your dataset
f_summary(your_dataframe)

# Run ANOVA on multiple response variables
f_aov(response_var1 + response_var2 ~ predictor1 * predictor2,
      data = your_dataframe)

# Create a boxplot of one or more variables
data(mtcars)
f_boxplot(hp + disp ~ gear * cyl,
          data = mtcars,
          intro_text = FALSE,
          output_type = "word")

# Fit a linear mixed-effects model
f_lmer(response ~ treatment + (1 | block), data = your_dataframe)

# Perform a Box-Cox transformation
transformed <- f_boxcox(your_dataframe$variable)

# Load a bundled example dataset
f_example_data()                       # list available files
path <- f_example_data("field_trial.csv")

# Clear the workspace and restart R
f_clear()
```

For detailed usage and examples, please refer to the package vignettes, the
help files, and the documentation website at
<https://delde001.github.io/rfriend/>.

---

## Known Issues

- When loading `rfriend`, you may see harmless warnings about S3 method
  overwrites related to `nobs.fitdistr` and `nobs.multinom` due to imported
  packages (`MuMIn`, `rstatix`). These do not affect functionality.

---

## Contributing

Contributions, bug reports, and feature requests are very welcome. Please open
an issue or submit a pull request on GitHub.

Before contributing, please ensure that:

- You have tested your changes locally.
- Code is properly documented using roxygen2.
- You follow the existing style and conventions.

---

## License

This package is licensed under
[GPL-3](https://www.R-project.org/Licenses/GPL-3).

---

## Contact

Author and maintainer: Sander H. van Delden
Email: <plantmind@proton.me>

Feel free to reach out for support, feature requests, or collaborations.

---

*Thank you for using `rfriend`!*

#' Perform multiple Kruskal-Wallis tests with a user-friendly output file, do data inspection and Dunn's test (of 'rstatix') as post hoc.
#'
#' Performs the Kruskal-Wallis rank sum test to assess whether there are statistically significant differences in the distributions (mean ranks) of three or more independent groups. It provides detailed outputs, including plots, assumption checks, and post hoc analyses using Dunn's test. Results can be saved in various formats ('pdf', 'Word', 'Excel', or console only) with customizable output options.
#'
#' @param formula A formula specifying the response and predictor variable (e.g., \code{response ~ predictor)}.
#' more response variables and predictors can be added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor1 + predictor2)}. The function iterates through these combinations or response and predictors, because the Kruskal-Wallis test itself only allows one response and one predictor combination to be tested simultaneously.
#' @param data A \code{data.frame} containing the variables referenced in the formula.
#' @param plot Logical. If \code{TRUE}, generates plots (e.g., density plots and boxplots)
#'   in the output files. Default is \code{TRUE}.
#' @param alpha Numeric. The significance level for the Kruskal-Wallis test and Dunn's
#'   test. Default is  \code{0.05}.
#' @param adjust Character string. Adjustment method for pairwise comparisons in Dunn's test. Options include \code{"holm", "hommel", "bonferroni", "hochberg", "bh", "by", "fdr"} or \code{"none"}. Default is \code{"bonferroni"}, if you don't want to adjust the p value (not recommended), use \code{adjust = "none"}.
#' @param intro_text Logical. If \code{TRUE}, includes a section about Kruskal-Wallis test assumptions in the output document. Default is \code{TRUE}.
#' @param close_generated_files Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param output_type Character string specifying the output format. Default is \code{"default"}.
#'   \itemize{
#'     \item \code{"default"}: Returns the object and lets R decide whether
#'       to print; auto-prints if unassigned, silent if assigned to a variable.
#'       Use \code{print(result)} or \code{plot(result)} to display the
#'       returned object.
#'     \item \code{"console"}: Forces immediate printing to the console
#'       regardless of object assignment.
#'     \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
#'       file of the corresponding format. See \code{save_as},
#'       \code{save_in_wdir}, and \code{open_generated_files} for file
#'       path and opening behavior.
#'     \item \code{"rmd"}: Stores the raw markdown string inside the returned
#'       object for use in R Markdown documents.
#'   }
#' @param save_as Character string specifying the output file path (without extension).
#'   If a full path is provided, output is saved to that location.
#'   If only a filename is given, the file is saved in \code{tempdir()}.
#'   If only a directory is specified (providing an existing directory with trailing slash),
#'   the file is named "dataname_Kruskal_Wallis_output" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
#'   Defaults to \code{file.path(tempdir(), "dataname_summary.pdf")}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.
#' @param ... Additional arguments forwarded to \code{\link[stats]{kruskal.test}}.
#'   The arguments \code{subset} and \code{na.action} are honored: when
#'   supplied, they are applied via \code{\link[stats]{model.frame}} so
#'   that the descriptive summary table, density plot, boxplot, Dunn's
#'   post hoc test and the Kruskal-Wallis test itself all see the exact
#'   same row set.

#' @return An object of class 'f_kruskal_test' (a named list, one entry per
#'   response-predictor combination) containing:
#'   \describe{
#'     \item{kruskal.test}{The \code{htest} object from \code{kruskal.test()}.}
#'     \item{dunn_test}{Data frame of pairwise Dunn's test results from \code{rstatix::dunn_test()}.}
#'     \item{summary_table}{Descriptive statistics with compact letter display (Letters column).}
#'     \item{alpha}{The significance level used.}
#'     \item{DunnTest_adjust}{The p-value adjustment method used.}
#'     \item{distributions}{ggplot density plot (if \code{plot = TRUE}).}
#'     \item{Boxplot}{ggplot boxplot with CLD letters (if \code{plot = TRUE}).}
#'   }
#'
#' Using the option \code{output_type}, it can also generate output in the form of: R Markdown code, 'Word', 'pdf', or 'Excel' files. Includes print and plot methods for 'f_kruskal_test' objects.
#'
#' @details
#' This function offers a comprehensive workflow for non-parametric analysis using the Kruskal-Wallis test:
#' \itemize{
#' \item Assumption Checks: Optionally includes a summary of assumptions in the output.
#' \item Visualization: Generates density plots and boxplots to visualize group distributions.
#' \item Post hoc Analysis: Conducts Dunn's test with specified correction methods if significant differences are found.
#'}
#'-----------\cr
#'
#' Output files are generated in the format specified by \code{output_type =} and saved to the working directory, options are \code{"pdf", "word"} or \code{"excel"}. If \code{output_type = "rmd"} is used it is advised to use it in a chunk with \{r, echo=FALSE, results='asis'\}
#'
#' This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
#'\itemize{
#' \item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
#' \item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
#' \item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
#'
#' \item If Pandoc is not found, this function may not work as intended.
#' }
#'
#' @section Multiple Testing Across Response Variables:
#' When several response variables are analysed in a single call
#' (e.g. \code{y1 + y2 + y3 ~ treatment}), each Kruskal-Wallis test is an
#' independent null-hypothesis test at level \code{alpha}. The post hoc
#' adjustment (e.g. \code{adjust = "bonferroni"}) only controls the
#' family-wise error rate \strong{within} one test (across pairwise Dunn
#' comparisons for that response). It does \strong{not} protect against
#' the inflation of Type I error \strong{across} the set of responses.
#'
#' \strong{Practical implication:} With \eqn{k} independent response
#' variables all tested at \eqn{\alpha = 0.05}, the probability of
#' obtaining at least one false positive is
#' \eqn{1-(1-0.05)^k}, which reaches ~40\% for \eqn{k = 10}.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # Example usage:
#' data(iris)
#'
#' # Perform Kruskal-Wallis test on Sepal.Length and Sepal.Width by Species
#' # with "holm" correction for posthoc dunn_test, without showing the output.
#' output <- f_kruskal_test(
#'                Sepal.Width + Sepal.Length ~ Species,
#'                data = iris,
#'                plot = FALSE,
#'                output_type = "word",
#'                adjust = "holm"
#'                )
#'
#' # Save Kruskal-Wallis test and posthoc to Excel sheets: Sepal.Width and Sepal.Length.
#' f_kruskal_out <- f_kruskal_test(
#'                      Sepal.Width + Sepal.Length ~ Species,
#'                      data = iris,
#'                      plot = FALSE,
#'                      output_type = "excel",
#'                      adjust = "holm"
#'                      )
#'
#' @export
f_kruskal_test <- function(
    formula,              # Kruskall-Wallis function formula
    data = NULL,          # data.frame used for Kruskall-Wallis
    plot = TRUE,          # Show plots in output files
    alpha = 0.05,         # Set significance level alpha for Kruskall and dunn_test
    output_type = "default",
    save_as = NULL,       # Specify the name of the file.
    save_in_wdir = FALSE, # Save file output in the working directory.
    intro_text = TRUE, # Print a short explanation about Kruskall-Wallis assumptions in the pdf or word output file
    adjust = "bonferroni",           # Correction Method for pairwise comparisson in dunn_test.
    close_generated_files = FALSE,   # Closes either open excel or word files depending on the output format.
    open_generated_files = interactive(),     # Open files after creation
    ...
    # Additional arguments forwarded to kruskal.test(). Currently the
    # arguments `subset` and `na.action` are honored: when supplied, they
    # are applied via stats::model.frame() so that the descriptive
    # summary table, density plot, boxplot, Dunn's test, and the KW test
    # itself all see the exact same row set.
)
{



  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state



  ########## Capture ... UNEVALUATED at f_kruskal_test's own frame ##########
  # Must be done HERE (in the top-level body), not inside
  # generate_report(), because match.call() captures the call to
  # whatever function it is called from. Inside generate_report() it
  # would capture generate_report()'s (empty) call and dots_exprs
  # would always be NULL  silently dropping subset / na.action.
  # Lexical scoping makes dots_exprs visible inside generate_report().
  .mc         <- match.call(expand.dots = FALSE)
  # Coerce the `...` pairlist to a real named list so dots_exprs$subset
  # returns the actual expression rather than a `..1` dots-index symbol.
  dots_exprs  <- as.list(.mc[["..."]])

  # Capture the user's calling environment for resolving subset /
  # na.action expressions from inside the generate_report() closure,
  # where parent.frame() would otherwise point at f_kruskal_test itself.
  caller_env  <- parent.frame()



  ####### Save dataframe name and Handle input from vectors (dataframe column) #####

  if(!is.null(data)){
    # Save dataframe name
    data_name <- deparse(substitute(data))

  } else if(is.null(data)){

    if(length(formula_extract_df_names(formula)) == 0){
      data_name <- "data"
    } else if(length(formula_extract_df_names(formula)) == 1){
      data_name <- formula_extract_df_names(formula)
    } else if(length(formula_extract_df_names(formula)) > 1){
      data_name <- paste(formula_extract_df_names(formula), collapse = "_")
    }

    # Make a data.frame based on the formula
    data <- formula_to_dataframe(formula)


    # Rewrite formula without data frame prefixes
    formula <- clean_formula(formula)

  }

  # Generate a temporary file path for "output.Rmd"
  temp_output_dir  <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")

  # Create the output file "output.Rmd" in tempdir()
  file.create(temp_output_file)
  # Create a file_extension switch
  file_extension <- NULL

  # Wrap lines in rmd output document
  f_wrap_lines()

  # match.arg() validates string args and gives clear error messages (CRAN standard)
  output_type <- match.arg(output_type,
                           choices = c("default", "console", "pdf", "word", "excel", "rmd"))


  adjust <- tolower(adjust)   # accept "BH", "bh", "BY", "by" etc.
  adjust <- match.arg(adjust,
                      choices = c("holm", "hommel", "bonferroni",
                                  "hochberg", "bh", "by", "fdr", "none"))

  # Normalize to what base p.adjust() requires
  adjust <- switch(adjust,
                   "bh" = "BH",
                   "by" = "BY",
                   adjust        # everything else passes through unchanged
  )

  # "sidak" and "hs" require rstatix >= 0.7.2
  if (adjust %in% c("sidak", "hs")) {
    if (utils::packageVersion("rstatix") < "0.7.2") {
      stop("adjust = '", adjust, "' requires rstatix >= 0.7.2. ",
           "Please run: install.packages('rstatix')")
    }
  }

  # Parameter validation alpha
  if (!is.numeric(alpha) || length(alpha) != 1 || alpha <= 0 || alpha >= 1) {
    stop("'alpha' must be a single numeric value strictly between 0 and 1 (e.g. 0.05).")
  }

  ##### Handle option "save_as = " #####
  if(save_in_wdir == TRUE){
    save_dir <- getwd()
  }else{
    save_dir <- tempdir()
  }

  #map the output type to extensions
  output_type_map <- c(
    "pdf"  = ".pdf",
    "word" = ".docx",
    "excel"= ".xlsx",
    "rmd"  = ".rmd"
  )

  # If the user specifies a path, filename or save_in_wdir == TRUE an output file should be created
  if (!is.null(save_as) || save_in_wdir == TRUE) {

    if (!is.null(save_as)) {
      #Remove backslash in save_as if needed
      save_as <- gsub(pattern = "\\\\", replacement = "/", x = save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if(file_extension_save_as[1] != FALSE){
        file_extension <- file_extension_save_as
      }
    }

    if(is.null(file_extension) && output_type %in% c("console", "default")){
      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "Kruskal_Wallis_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = ".pdf"
      )
      #set output_type to default
      output_type <- "pdf"

    }
    else if(is.null(file_extension) && output_type %in% c("pdf", "word", "excel", "rmd")){

      #create extension based on input_type
      file.ext <- unname(output_type_map[output_type])

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name,
                                                        "Kruskal_Wallis_output",
                                                        sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file.ext
      )


    }
    else if(!is.null(file_extension)) {

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name,
                                                        "Kruskal_Wallis_output",
                                                        sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file_extension[1]
      )
      # reset the output type to match the user input extention in save_as
      output_type <- file_extension[2]
    }
  } else {

    #create extension based on input_type
    file.ext <- unname(output_type_map[output_type])

    # use helper get_save_path() to create output_path
    output_path <- get_save_path(save_as = save_as,
                                 default_name = paste(data_name,
                                                      "Kruskal_Wallis_output",
                                                      sep = "_"),
                                 default_dir = save_dir,
                                 file.ext = file.ext
    )
  }


  # Prevent output to console and keep files open when output is "rmd" format
  if (output_type == "rmd") close_generated_files <- FALSE

  # Cross-platform close_generated_files (was Windows-only taskkill)
  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    if (output_type == "excel") close_app("EXCEL.EXE",   "Microsoft Excel", "soffice")
  }

  # Create a list to store all outputs in this function
  output_list <- list()


  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(formula) #use helper_check_lhs.R

  # Extract response variables from the left-hand side of the formula
  lhs <- all.vars(formula[[2]])  # Get LHS variables (response)
  response_names <- lhs



  # Extract predictor variables from the right-hand side of the formula
  rhs <- all.vars(formula[[3]])# Get RHS variables (predictors)


  # Ensure response and predictors are in the data
  for (response in lhs) {
    if (!(response %in% names(data))) {
      stop(paste("Response variable", response, "not found in the data."))
    }
  }


  for (predictor in rhs) {
    if (!(predictor %in% names(data))) {
      stop(paste("Predictor variable", predictor, "not found in the data."))
    }

    # Ensure all predictor variables are factors
    data[[predictor]] <- as.factor(data[[predictor]])
  }



  generate_report <- function(output = TRUE) {

    # This text reminds the user of the assumptions of an intro_text it its show by default
    # but can be hidden.
    if(intro_text == TRUE){

      cat("
# Assumptions of the Kruskal-Wallis Test
The Kruskal-Wallis test is a non-parametric test used to assess whether there are statistically significant differences in the distributions (mean ranks) of three or more independent groups. It does not require the data to be normally distributed, making it suitable for ordinal or skewed continuous data. However, the test assumes that data within each group are similarly distributed. Below are its key assumptions:  \n   \n  \n   \n

1.	**Independence of Observations:**
Data points must be independently sampled within and across groups (no repeated measures, no clustering).

2.	**Measurement Scale:**
The dependent variable should be ordinal or continuous (interval or ratio), but not nominal.

3.	**Similar Distribution Shapes:**
The dependent variable should have a similar distribution shape across groups. If this assumption holds, the test can be interpreted as a comparison of medians. If shapes or spreads differ substantially, the result reflects differences in mean ranks (overall distributions) rather than medians specifically.

4.	**Random Sampling:**
Samples should be randomly drawn from their populations to ensure representativeness.

5.	**Adequate Sample Size:**
Each group should ideally include at least five observations for reliable results.   \n   \n


<div style=\"page-break-after: always;\"></div>
\\newpage")
    }

    # Multiple-response/predictor warning: shown when > 1 independent test is run.
    # Fires regardless of intro_text so the user always sees it in this situation.
    k_tests <- length(lhs) * length(rhs)
    if (k_tests > 1) {
      fwer_pct   <- round((1 - (1 - alpha)^k_tests) * 100, 1)
      bonf_alpha <- round(alpha / k_tests, 4)
      cat(paste0(
        "\n\n***\n\n",
        "**[!] NOTE Multiple Testing Across ", k_tests, " Kruskal-Wallis Tests**  \n\n",
        "This report runs **", k_tests, "** independent Kruskal-Wallis tests ",
        "(", length(lhs), " response", if (length(lhs) > 1) "s" else "", " \u00d7 ",
        length(rhs), " predictor", if (length(rhs) > 1) "s" else "", ") on the same dataset. ",
        "The **", adjust,"** correction keeps each individual test honest, it guards against ",
        "false positives among the pairwise group comparisons, but it offers no protection ",
        " against the accumulation of error across all ",k_tests," tests combined. ",

        "\n At \u03b1 = ", alpha, " per test, the probability of obtaining at least one ",
        "spurious significant result across all ", k_tests, " tests is approximately ",
        "**", fwer_pct, "%** ( $1-(1-", alpha, ")^{", k_tests, "}$, assuming independence). ",
        "This risk is highest in exploratory studies; it is less of a concern when ",
        "each response has a clear a priori hypothesis.  \n\n",
        "**Possible remedies:**  \n",
        "\n-  **Bonferroni** (conservative): re-run with `alpha = ", bonf_alpha,
        "` (\u03b1 / ", k_tests, ").  \n",
        "\n-  **False Discovery Rate (FDR)**: apply `p.adjust(p_values, method = \"fdr\")` to the ",
        k_tests, " Kruskal-Wallis p-values after the fact.  \n",
        "\n-  **Pre-registration**: if each response-predictor combination was a pre-specified ",
        "study outcome, correction may not be required; document this decision explicitly.  \n",
        "\n\n***\n\n"
      ))
    }

    #create count to remove last page break
    i <- 0
    # Build the analysis data set ONCE, before the loops, using ALL
    # responses and ALL predictors. Every response x predictor
    # combination is then tested on the identical row set, so results
    # are directly comparable across responses. Rows with NA in ANY
    # response or predictor are dropped (subject to na.action), and any
    # subset / na.action passed via `...` is applied here.
    #
    # Strategy: pre-evaluate subset eagerly against data first, falling
    # back to the user's calling environment. Then apply the filter
    # manually and pass plain values to stats::model.frame via do.call.
    # This avoids the fragile match.call/substitute dance where spliced
    # expressions inside a constructed call can end up containing `..N`
    # dots-index symbols that crash model.frame with "the ... list
    # contains fewer than 3 elements".
    combined_formula <- as.formula(
      paste("~", paste(c(lhs, rhs), collapse = " + "))
    )

    # Resolve subset expression against data columns + caller env
    subset_vec <- NULL
    if (!is.null(dots_exprs$subset)) {
      subset_vec <- eval(dots_exprs$subset, envir = data, enclos = caller_env)
      if (is.logical(subset_vec)) subset_vec[is.na(subset_vec)] <- FALSE
    }
    if (!is.null(subset_vec)) {
      data <- data[subset_vec, , drop = FALSE]
    }

    # Resolve na.action (a function, defaults to na.omit)
    na_action_fn <- stats::na.omit
    if (!is.null(dots_exprs$na.action)) {
      na_action_fn <- eval(dots_exprs$na.action, envir = caller_env)
    }

    mf_args <- list(
      formula            = combined_formula,
      data               = data,
      drop.unused.levels = TRUE,
      na.action          = na_action_fn
    )

    data <- do.call(stats::model.frame, mf_args)

    #Main loop starts here
    for (response_name in lhs) {
      for (predictor_name in rhs) {
        p <- NULL
        d <- NULL
        letter_df <- NULL
        cld_letters <- NULL
        i <- i+1
        # Create a new formula for each response
        current_formula <- as.formula(paste0(response_name, "~", predictor_name))

        cat("   \n#  Analysis of: ", response_name, " by ", predictor_name,"  \n")
        cat("   \n     \n&nbsp;  \n   \n ")

        if(plot == TRUE){
          cat("  \n## Visual check on similarity of distributions  \n")

          d <- ggplot(data, aes(x = !!sym(response_name), fill = factor(!!sym(predictor_name)))) +
            geom_density(alpha = 0.4) +
            labs(title = "Density Plot by Group", x = response_name, fill = predictor_name) +
            theme_bw(base_size = 14)

          # Print d, i.e. distributions plot
          # Create a temporary file path with a .png extension
          temp_file_path_d <- tempfile(fileext = ".png")

          # Save the plot, specifying the device as "pdf"
          suppressMessages(ggsave(filename = temp_file_path_d, plot = d))

          # Include the saved plot in R Markdown
          cat(paste0("![](", temp_file_path_d, ")"), "   \n  \n")
          cat("&nbsp;\n   \n")

        }

        output_list[[paste0(response_name,"_",predictor_name)]][["distributions"]] <- d
        # Store alpha and adjust for print.class function
        output_list[[paste0(response_name,"_",predictor_name)]][["alpha"]]          <- alpha
        output_list[[paste0(response_name,"_",predictor_name)]][["DunnTest_adjust"]]<- adjust

        # cat("   \n     \n&nbsp;  \n   \n ")
        if(output_type != "rmd"){
          # Pagebreak
          cat("
<div style=\"page-break-after: always;\"></div>
\\newpage")
        }

        cat("
\n## Result of Kruskal-Wallis rank sum test  \n  \n")
        cat("&nbsp;\n   \n")

        # Perform the Kruskal-Wallis test
        kruskal.test_result <- kruskal.test(current_formula, data = data)

        # Format p-value
        kw_p_fmt <- if (kruskal.test_result$p.value < 0.001) "< 0.001" else
          round(kruskal.test_result$p.value, 4)

        # Show formatted result in output document
        cat(paste0(
          "**Kruskal-Wallis rank sum test** of ", response_name, " by ", predictor_name, ":  \n",
          "$\\chi^{2}$ = ", round(kruskal.test_result$statistic, 3),
          ", df = ", kruskal.test_result$parameter,
          ", p = **", kw_p_fmt, "**",
          if (kruskal.test_result$p.value < alpha) {
            paste0("  \nThe test is **significant** (\u03b1 = ", alpha,
                   "), indicating that at least one group differs from the others.")
          } else {
            paste0("  \nThe test is **not significant** (\u03b1 = ", alpha,
                   "). There is insufficient evidence to conclude that the groups differ.")
          },
          "\n\n&nbsp;\n   \n"
        ))
        # Store Kruskal-Wallis test in ouput
        output_list[[paste0(response_name,"_",predictor_name)]][["kruskal.test"]] <- kruskal.test_result

        # Create data summary table for output and store in output_list
        summary_table <- f_summary(data,
                                   response_name,
                                   predictor_name,
                                   show_name = FALSE,
                                   show_sd = FALSE,
                                   show_se = FALSE,
                                   digits = NULL
        )$output_df

        # Conduct Dunn's test for post hoc analysis (always computed for CLD letters)
        dunn_test_result <- rstatix::dunn_test(current_formula,
                                               data = data,
                                               p.adjust.method = adjust)

        if(kruskal.test_result$p.value < alpha){
          cat("
\n  \n## Post hoc Analysis of:  `", deparse(current_formula),"`   \n")
              cat("\nBecause the overall Kruskal-Wallis test was significant, a Dunn\u2019s (1964) test was conducted to identify which specific groups differ from one another.\n")
              cat("&nbsp;\n   \n&nbsp;\n   \n")
              cat("
       \n**How to interpret the results:**\n
       \n- **Dunn's Test compares ranks, not medians.** Although Dunn\u2019s test is often discussed in terms of median differences, it actually compares the mean ranks of groups. In other words, it tests whether observations in one group tend to have higher values than those in another (stochastic dominance).
       \n- **Caution about medians.** You can interpret group differences as differences in medians only if the distributions have similar shapes (see boxplots below and distribution graphs above).
       If one distribution is skewed and another is symmetric, Dunn's test may indicate a difference even when their medians are the same.
       \n")
          cat("&nbsp;\n   \n")
          cat("
Dunn (1964) Kruskal-Wallis multiple comparison.  \n")
          # Drop the .y. column (just repeats response name on every row)
          dunn_print <- dunn_test_result[, !names(dunn_test_result) %in% c(".y.", "y"), drop = FALSE]
          f_pander(dunn_print)
          cat("p-values (p) were adjusted with ", adjust, " (p.adj) \n")
          cat("
Group 1 and group 2 indicate the compared groups with respectively n1 and n2 replicates. The **statistic** column represents the z-test statistic for each pairwise comparison in the Dunn test. This statistic is derived by standardizing the difference in mean ranks between two groups using the pooled standard error. It follows the standard normal distribution under the null hypothesis, which assumes no difference in ranks between groups.

- A **positive Z** value indicates that the first group in the comparison has higher ranks (on average) than the second group.
- A **negative Z** value indicates that the second group has higher ranks (on average) than the first group.    \n   \n")

          if(output_type != "rmd"){
            # Pagebreak
            cat("
<div style=\"page-break-after: always;\"></div>
\\newpage")
          }
        } # end if KW significant (Dunn output)

        # Store dunn_test_result in ouput
        output_list[[paste0(response_name,"_",predictor_name)]][["dunn_test"]] <- dunn_test_result

        # Extract p-values and convert to a compact letter display
        dunn_pvalues <- dunn_test_result$p.adj
        names(dunn_pvalues) <- paste0(dunn_test_result$group1,"-",dunn_test_result$group2)

        # Can be that spaces are added to names in group remove these by:
        names(dunn_pvalues) <- lapply(names(dunn_pvalues), function(x) gsub(" ", "", x))
        cld <- multcompLetters(dunn_pvalues, threshold = alpha)

        # Create a data frame with letters for each group
        letter_df <- as.data.frame(cld$Letters)
        letter_df[[predictor_name]] <- row.names(letter_df)
        colnames(letter_df)[colnames(letter_df) == "cld$Letters"] <- "Letters"

        # Build footnote text (used after summary table)
        footnote <- paste0("
\n-  **Note 1 (Assumptions):** Dunn's test does not assume normality, but implies a
      test for difference in medians only if distribution shapes are similar across groups.
      If shapes or spreads differ substantially (check side-by-side boxplots and density plots),
      the result reflects a difference in mean ranks rather than medians.
\n  **Note 2 (Results):** Groups in the \"Letters\" column sharing the same letter are ",
                           "**not** significantly different (\u03B1 = ", alpha, ".",
                           "Groups with different letters are significantly different. Sharing a letter ",
                           "indicates insufficient evidence to claim a difference; it does not ",
                           "prove the groups are identical.\n",
                           if(adjust == "none"){paste0("\n**WARNING**: No p-value correction was applied. This increases the risk of finding \"significant\" differences that generally exist only by chance.")
                           } else {paste0(" P-values were adjusted with ", adjust,".")
                           })

        # Store adjust method in output
        output_list[[paste0(response_name,"_",predictor_name)]][["adjust"]] <- adjust

        if(plot == TRUE){

          cat("  \n## Boxplot of: ", response_name, " by ", predictor_name,"  and Dunn's  post hoc test  \n  \n")
          # Add the compact letter display to the data
          data2 <- merge(data, letter_df, by = predictor_name, all.x = TRUE)

          # Set the location for the letters in the boxplot.
          y_max <- max(data[[response_name]], na.rm = TRUE)
          y_min <- min(data[[response_name]], na.rm = TRUE)

          # Calculate a proportional position above the top of the plot
          y_position <- y_max + 0.08 * (y_max - y_min)  # 5% above ymax

          # Create the boxplot
          p <- ggplot(data2, aes(x = !!sym(predictor_name), y = !!sym(response_name), fill = factor(!!sym(predictor_name)))) +
            geom_boxplot(alpha = 0.4) +
            geom_jitter(width = 0.2,
                        color = "steelblue",
                        height = 0,    # usually 0 for boxplots (don't distort y-axis values)
                        alpha = 0.5) +
            geom_text(
              data = letter_df,
              aes(y = y_position, label = .data[["Letters"]][match(!!sym(predictor_name),  !!sym(predictor_name))])
            ) +
            labs(x = predictor_name, y = response_name) +
            theme_bw(base_size = 14)

          # Create a temporary file path with a .png extension
          temp_file_path_p <- tempfile(fileext = ".png")

          # Save the plot, specifying the device as "pdf"
          suppressMessages(ggsave(filename = temp_file_path_p, plot = p))

          # Include the saved plot in R Markdown
          cat(paste0("![](", temp_file_path_p, ")"), "   \n  \n")
          cat("&nbsp;\n   \n")

          output_list[[paste0(response_name,"_",predictor_name)]][["Boxplot"]] <- p
        }

        # Create data summary with letters table for output and store in output_list
        summary_table <- merge(summary_table, letter_df, by = predictor_name, all.x = TRUE)

        if(kruskal.test_result$p.value > alpha){
          summary_table$Letters <- "ns"
        }

        # Reorder columns: Letters next to group labels (indicates distribution differences),
        # median-centric layout for non-parametric test, mean last (de-emphasized)
        col_order <- c(predictor_name, "Letters", "n", "min", "Q1", "median", "Q3", "max", "mean")
        col_order <- col_order[col_order %in% names(summary_table)]
        # Append any remaining columns not in col_order
        remaining <- setdiff(names(summary_table), col_order)
        summary_table <- summary_table[, c(col_order, remaining), drop = FALSE]

        output_list[[paste0(response_name,"_",predictor_name)]][["summary_table"]] <- summary_table

        # Helper: bold the median column for markdown/pander display only
        bold_median <- function(tbl) {
          if ("median" %in% names(tbl)) {
            tbl$median <- paste0("**", tbl$median, "**")
          }
          tbl
        }


#         if(output_type != "rmd"){
#           # Pagebreak
#           cat("
# <div style=\"page-break-after: always;\"></div>
# \\newpage")
#         }


        if(kruskal.test_result$p.value < alpha){
          cat("&nbsp;\n   \n")
          cat("  \n## Summary Statistics of: ", response_name, " by ", predictor_name,"  and Dunn's test post hoc test  \n  \n")
          cat("&nbsp;\n   \n")
          cat(paste("  \nSummary Statistics with Dunn's post hoc Comparison, `Model:", deparse(current_formula),"`  \n"))
          f_pander(bold_median(f_conditional_round(summary_table, digits = 2)), col_width = 9)

          cat(footnote)

        } else {
          cat("  \n## Data Summary Table of: ", response_name,"
      \n
        Since the Kruskal-Wallis test was **NOT** significant, no post hoc test was performed.\n   \n")

          cat(paste("\n**Summary Statistics** `Model:", deparse(current_formula),"`  \n"))
          f_pander(bold_median(f_conditional_round(summary_table, digits = 2)), col_width = 9)
        }

        if(output_type != "rmd" &&  i < length(lhs)){
          # Pagebreak
          cat("
\n    \n
<div style=\"page-break-after: always;\"></div>
\\newpage
        ")
        }
      }
    } #Main loop end
    if (output == TRUE) {
      return(output_list)
    }
  } # End generate report function.

  # Execute analysis and return results but hide this from console.
  suppressMessages(
    utils::capture.output(
      output_list <- generate_report(),
      file = nullfile()
    )
  )
  class(output_list) <- "f_kruskal_test"



  # Here the documents are constructed.
  if (output_type %in% c("word", "pdf")) {

    # Show save location before knitting else it will not display in console.
    message(paste0("Saving output in: ", output_path))


    # Create a temporary R Markdown file
    word_pdf_preamble <- function(){ paste0("
---
title: \"Kruskal-Wallis Analysis Report\"
date: \"`r Sys.Date()`\"
output:
   word_document:
      reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
        latex_engine: pdflatex
header-includes:
  - \\usepackage[T1]{fontenc}
  - \\usepackage[utf8]{inputenc}
  - \\usepackage{textcomp}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
  - \\DeclareUnicodeCharacter{2264}{\\ensuremath{\\leq}}
  - \\DeclareUnicodeCharacter{2265}{\\ensuremath{\\geq}}
  - \\DeclareUnicodeCharacter{2192}{\\ensuremath{\\rightarrow}}
  - \\DeclareUnicodeCharacter{00D7}{\\ensuremath{\\times}}
  - \\DeclareUnicodeCharacter{2014}{\\textemdash}
  - \\DeclareUnicodeCharacter{03B1}{\\ensuremath{\\alpha}}
  - \\DeclareUnicodeCharacter{2013}{\\textendash}
  - \\DeclareUnicodeCharacter{2019}{\\textquoteright}
  - \\DeclareUnicodeCharacter{0160}{\\v{S}}
  - \\DeclareUnicodeCharacter{00E1}{\\'{a}}
  - \\usepackage{titling}
  - \\setlength{\\droptitle}{-2.5cm} % Adjust vertical spacing
---
")}

    # Prevent ## before printed output
    knitr::opts_chunk$set(comment = "")

    # re-run generate_report, but this time capture its output to a string
    generated_markdown <- capture.output(generate_report(output = FALSE))

    # Combine the preamble, assumptions, and the captured report into one string
    rmd_content <- paste(
      word_pdf_preamble(),
      # The captured output already contains the necessary markdown formatting and image links
      paste(generated_markdown, collapse = "\n"),
      sep = "\n"
    )

    # Write the complete Rmd content to the temp file
    writeLines(rmd_content, temp_output_file)

    # Create the RMarkdown file
    rmarkdown::render(
      temp_output_file,
      output_file = output_path,
      intermediates_dir = temp_output_dir,
      knit_root_dir = temp_output_dir,
      quiet = TRUE,
      output_format = paste0(output_type, "_document")
    )

    if(open_generated_files == TRUE){
      # Open the file with default program
      f_open_file(output_path)
    }

    return(invisible(output_list))

  } else if (output_type == "excel") {

    # show the location were the file is saved
    message(paste0("Saving output in: ", output_path))

    # Extract all post_hoc_summary_table tables and keep their names
    post_hoc_tables <- lapply(output_list, function(obj)
      obj$summary_table)

    # Assign names to the list for Excel sheet names based on response names
    names(post_hoc_tables) <- response_names

    # Write to an Excel file with each table in its own sheet
    write_xlsx(post_hoc_tables, path = output_path)

    # Open files after creation
    if(open_generated_files == TRUE){
      f_open_file(output_path)
    }

    return(invisible(output_list))

  } else if (output_type == "rmd"){

    if (is.null(opts_knit$get("output.dir"))) {
      opts_knit$set(output.dir = tempdir())
    }

    # Re-capture the markdown text for the rmd output
    generated_markdown <- capture.output(generate_report(output = FALSE))

    clean_rmd_output <- paste(generated_markdown, collapse = "\n")

    output_list[["rmd"]] <- clean_rmd_output

    return(invisible(output_list))

  } else if (output_type == "default"){
    #Nothing to show output will be output_list.
    return(output_list)

  } else if (output_type == "console"){
    #Print output list to the console (forced)
    print(output_list)

    return(invisible(output_list))

  } else {
    warning("Invalid output format specified. No file generated.")

  }

  # Remove the temporary R Markdown file
  invisible(suppressWarnings(file.remove(temp_output_file)))

}


#' @export
print.f_kruskal_test <- function(x, ...) {
  # Create a flag to now if Dunn Test expl. should be printed.
  flag_dunnTest_used <- FALSE

  # get the line with from user
    line_width <- 72




# Loop over each category (a, b, etc.)
  for (category in names(x)) {
    # Get the sublist for this category
    sublist <- x[[category]]

    # Skip non-result entries (e.g. the "rmd" character string)
    if (!is.list(sublist)) next

    cat("\n==========================================================\n")
    cat("Results of testing", sublist$kruskal.test$data.name)
    cat("\n==========================================================\n")

    print(sublist$kruskal.test)

    if (sublist$kruskal.test$p.value < sublist$alpha) {
      cat(
        "\nSummary table with Dunn-Test Post Hoc:",
        sublist$kruskal.test$data.name,
        "\n"
      )
      flag_dunnTest_used <- TRUE
    } else {
      cat(
        "\nNo differences found, summary table of:",
        sublist$kruskal.test$data.name,
        "\n"
      )
    }

    f_pander(sublist$summary_table, col_width = 7)
    if (flag_dunnTest_used == TRUE) {


      dunnTest_used <- paste0("
Note 1 (Assumptions): Dunn's test does not assume normality, but implies a test for difference in medians only if distribution shapes are similar across groups. If shapes or spreads differ substantially (check boxplots and density plots using, plot(...)), the result reflects a difference in mean ranks rather than medians.\n
Note 2 (Results): Groups in the \"Letters\" column sharing the same letter are ",
"**not** significantly different (\u03B1 = ", format(sublist$alpha, nsmall = 3), ".",
"Groups with different letters are significantly different. Sharing a letter ",
"indicates insufficient evidence to claim a difference; it does not ",
"prove the groups are identical.\n",
        if (sublist$adjust == "none") {
          paste0(
            "\n**WARNING**: No p-value correction was applied. This increases the risk of finding \"significant\" differences that generally exist only by chance."
          )
        } else {
          paste0("P-values were adjusted with ", sublist$adjust, ".")
        }
        )

      # Wrap the text
      wrapped_text <- strwrap(dunnTest_used, width = line_width)

      # Print the text with newlines collapsing the vector
      cat(paste(wrapped_text, collapse = "\n"), "\n")
    }
  }
}



#' Plot method for f_kruskal_test objects
#'
#' Displays the density plot and/or boxplot stored in an \code{f_kruskal_test}
#' object. Plots are only available when the original call used \code{plot = TRUE}.
#'
#' @param x An object of class \code{f_kruskal_test}.
#' @param which Character vector indicating which plots to show. Options are
#'   \code{"distributions"} (density plot), \code{"Boxplot"}, or both (default).
#' @param ... Additional arguments (currently ignored).
#'
#' @return Returns \code{x} invisibly.
#'
#' @examples
#' result <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
#'                          output_type = "default")
#' plot(result)                          # both plots
#' plot(result, which = "Boxplot")       # boxplot only
#'
#' @export
plot.f_kruskal_test <- function(x, which = c("distributions", "Boxplot"), ...) {

  which <- match.arg(which, choices = c("distributions", "Boxplot"), several.ok = TRUE)

  for (category in names(x)) {
    sublist <- x[[category]]

    # Skip non-result entries (e.g. the "rmd" character string)
    if (!is.list(sublist)) next

    for (plt_name in which) {
      if (!is.null(sublist[[plt_name]])) {
        print(sublist[[plt_name]])
      }
    }
  }

  invisible(x)
}

# =============================================================================
# helper-quiet.R
# =============================================================================
# Output-suppression wrappers used across the rfriend test suite.
#
# Almost every f_*() function in rfriend writes Markdown to the console via
# cat() when called with output_type = "default", and several also emit
# informational messages() or package-startup warnings. Letting all of that
# stream to stdout during testthat runs makes failures unreadable.
#
# Pre-refactor the suite had near-identical quiet_f_aov(), quiet_lmer(),
# quiet_bp(), quiet_bn(), quiet_tt(), quiet_summary(), quiet_boxcox()
# wrappers defined inline in each test file. They all did the same thing:
# suppress messages, capture stdout, return the result.
# =============================================================================


# -----------------------------------------------------------------------------
# Low-level wrappers
# -----------------------------------------------------------------------------
#
# These operate on an already-built call by capturing it unevaluated. They
# are suitable when you are NOT forwarding `...` and are safe to use from
# within a test_that() block.

# Suppress messages and capture stdout. Returns the value of `expr`.
run_quiet <- function(expr) {
  suppressMessages(
    utils::capture.output(
      result <- eval.parent(substitute(expr)),
      file = nullfile()
    )
  )
  result
}

# Same, but also swallow warnings. Use sparingly: a test that needs this
# is probably a test that should be checking for a specific warning with
# expect_warning() instead. The main legitimate use case is tests of
# edge-case model fits where lme4 / MASS emit convergence chatter that
# the rfriend wrapper is specifically designed to tolerate.
run_quiet_warn <- function(expr) {
  suppressMessages(suppressWarnings(
    utils::capture.output(
      result <- eval.parent(substitute(expr)),
      file = nullfile()
    )
  ))
  result
}


# -----------------------------------------------------------------------------
# Per-function convenience wrappers
# -----------------------------------------------------------------------------
#
# Each of these calls its namesake f_*() function with the canonical
# "no interactive side effects" argument set. They exist so that a
# test block reads:
#
#     out <- quiet_f_aov(y ~ grp, data = df)
#
# instead of:
#
#     suppressMessages(capture.output(out <- f_aov(y ~ grp, data = df,
#         output_type = "default", norm_plots = FALSE, intro_text = FALSE,
#         open_generated_files = FALSE), file = nullfile()))
#
# Implementation note: each wrapper captures its own `...` and forwards
# them explicitly rather than calling run_quiet(). This is deliberate:
# run_quiet() uses eval.parent(substitute(expr)), which does not expand
# `...` correctly when the expression itself contains `...`.
# -----------------------------------------------------------------------------

quiet_f_aov <- function(...) {
  suppressMessages(
    utils::capture.output(
      result <- f_aov(..., output_type = "default",
                      norm_plots = FALSE, intro_text = FALSE,
                      open_generated_files = FALSE),
      file = nullfile()
    )
  )
  result
}

quiet_f_lmer <- function(...) {
  suppressMessages(suppressWarnings(
    utils::capture.output(
      result <- f_lmer(..., output_type = "default",
                       norm_plots = FALSE, intro_text = FALSE,
                       open_generated_files = FALSE),
      file = nullfile()
    )
  ))
  result
}

quiet_f_glm <- function(...) {
  suppressMessages(
    utils::capture.output(
      result <- f_glm(..., output_type = "default",
                      norm_plots = FALSE, intro_text = FALSE,
                      open_generated_files = FALSE),
      file = nullfile()
    )
  )
  result
}

quiet_f_t_test <- function(...) {
  suppressMessages(
    utils::capture.output(
      result <- f_t_test(..., output_type = "default",
                         open_generated_files = FALSE),
      file = nullfile()
    )
  )
  result
}

quiet_f_wilcox_test <- function(...) {
  suppressMessages(
    utils::capture.output(
      result <- f_wilcox_test(..., output_type = "default",
                              open_generated_files = FALSE),
      file = nullfile()
    )
  )
  result
}

quiet_f_kruskal_test <- function(...) {
  suppressMessages(
    utils::capture.output(
      result <- f_kruskal_test(..., output_type = "default",
                               open_generated_files = FALSE),
      file = nullfile()
    )
  )
  result
}

quiet_f_chisq_test <- function(...) {
  suppressMessages(
    utils::capture.output(
      result <- f_chisq_test(..., output_type = "default",
                             open_generated_files = FALSE),
      file = nullfile()
    )
  )
  result
}


quiet_f_boxplot <- function(...) {
  suppressMessages(
    utils::capture.output(
      result <- f_boxplot(..., output_type = "default",
                          open_generated_files = FALSE,
                          boxplot_explanation = FALSE,
                          outliers = FALSE),
      file = nullfile()
    )
  )
  result
}

quiet_f_boxcox <- function(x, ...) {
  suppressMessages(
    utils::capture.output(
      result <- f_boxcox(x, ..., output_type = "default",
                         open_generated_files = FALSE),
      file = nullfile()
    )
  )
  result
}

quiet_f_bestNormalize <- function(x, ...) {
  suppressMessages(
    utils::capture.output(
      result <- f_bestNormalize(x, plots = FALSE, output_type = "default", ...),
      file = nullfile()
    )
  )
  result
}

# test-f_kruskal_test.R
# Comprehensive stress tests for f_kruskal_test()
# Uses testthat framework
# ============================================================
# 1. INPUT VALIDATION
# ============================================================

test_that("rejects invalid alpha values", {
  skip_on_cran()
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, alpha = -1,          plot = FALSE), "alpha")
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, alpha = 0,           plot = FALSE), "alpha")
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, alpha = 1,           plot = FALSE), "alpha")
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, alpha = 1.5,         plot = FALSE), "alpha")
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, alpha = "high",      plot = FALSE), "alpha")
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, alpha = c(0.01, 0.05), plot = FALSE), "alpha")
})

test_that("rejects invalid adjust method", {
  skip_on_cran()
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, adjust = "invalid",    plot = FALSE))
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, adjust = "bonfferoni", plot = FALSE))
})

test_that("rejects invalid output_type", {
  skip_on_cran()
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, output_type = "html", plot = FALSE))
  expect_error(f_kruskal_test(Sepal.Width ~ Species, data = iris, output_type = "pptx", plot = FALSE))
})

test_that("errors when variables are missing from data", {
  skip_on_cran()
  expect_error(f_kruskal_test(Nonexistent ~ Species,    data = iris, plot = FALSE), "not found")
  expect_error(f_kruskal_test(Sepal.Width ~ Nonexistent, data = iris, plot = FALSE), "not found")
})


# ============================================================
# 2. OUTPUT STRUCTURE - SINGLE RESPONSE, SINGLE PREDICTOR
# ============================================================

test_that("returns correct class and named structure", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width ~ Species, data = iris, plot = FALSE)

  expect_s3_class(res, "f_kruskal_test")
  expect_true("Sepal.Width_Species" %in% names(res))

  entry <- res[["Sepal.Width_Species"]]

  # All expected components present
  expect_true(all(c("kruskal.test", "dunn_test", "summary_table",
                     "alpha", "DunnTest_adjust", "adjust") %in% names(entry)))

  # kruskal.test is an htest object
  expect_s3_class(entry$kruskal.test, "htest")
  expect_equal(entry$kruskal.test$method, "Kruskal-Wallis rank sum test")

  # dunn_test is a data frame with expected columns
  expect_true(is.data.frame(entry$dunn_test))
  expect_true(all(c("group1", "group2", "statistic", "p", "p.adj") %in%
                     names(entry$dunn_test)))

  # summary_table is a data frame with expected columns
  expect_true(is.data.frame(entry$summary_table))
  expect_true(all(c("Species", "Letters", "n", "median", "mean") %in%
                     names(entry$summary_table)))

  # alpha and adjust stored correctly
  expect_equal(entry$alpha, 0.05)
  expect_equal(entry$DunnTest_adjust, "bonferroni")
  expect_equal(entry$adjust, "bonferroni")
})


# ============================================================
# 3. PLOT TOGGLE
# ============================================================

test_that("plot = FALSE: no ggplot objects stored", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width ~ Species, data = iris, plot = FALSE)
  entry <- res[["Sepal.Width_Species"]]

  expect_null(entry$distributions)
  expect_null(entry$Boxplot)
})

test_that("plot = TRUE: ggplot objects are stored", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width ~ Species, data = iris, plot = TRUE)
  entry <- res[["Sepal.Width_Species"]]

  expect_s3_class(entry$distributions, "ggplot")
  expect_s3_class(entry$Boxplot, "ggplot")
})


# ============================================================
# 4. COLUMN ORDERING OF SUMMARY TABLE
# ============================================================

test_that("summary_table: Letters 2nd, median-centric layout, mean last", {
  skip_on_cran()
  res  <- f_kruskal_test(Sepal.Width ~ Species, data = iris, plot = FALSE)
  cols <- names(res[["Sepal.Width_Species"]]$summary_table)

  # First column is the predictor

  expect_equal(cols[1], "Species")

  # Second column is Letters
  expect_equal(cols[2], "Letters")

  # median comes before Q3
  expect_true(which(cols == "median") < which(cols == "Q3"))

  # mean is the last of the standard columns
  standard_order <- c("Species", "Letters", "n", "min", "Q1",
                       "median", "Q3", "max", "mean")
  present <- standard_order[standard_order %in% cols]
  expect_equal(cols[seq_along(present)], present)
})


# ============================================================
# 5. MULTIPLE RESPONSES AND/OR PREDICTORS
# ============================================================

test_that("multiple responses: each gets its own entry", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width + Sepal.Length ~ Species,
                         data = iris, plot = FALSE)

  expect_true("Sepal.Width_Species"  %in% names(res))
  expect_true("Sepal.Length_Species" %in% names(res))
})

test_that("formula with minus separator on LHS works", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width - Sepal.Length ~ Species,
                         data = iris, plot = FALSE)

  expect_true("Sepal.Width_Species"  %in% names(res))
  expect_true("Sepal.Length_Species" %in% names(res))
})

test_that("multiple predictors: all response x predictor combinations", {
  skip_on_cran()
  df <- data.frame(
    y1 = rnorm(60), y2 = rnorm(60, 5),
    grpA = factor(rep(c("a1", "a2", "a3"), each = 20)),
    grpB = factor(rep(c("b1", "b2"), times = 30))
  )

  res <- f_kruskal_test(y1 + y2 ~ grpA + grpB, data = df, plot = FALSE)

  expected <- c("y1_grpA", "y1_grpB", "y2_grpA", "y2_grpB")
  expect_true(all(expected %in% names(res)))
  # Only these 4 entries (no extras)
  expect_equal(length(res), length(expected))
})


# ============================================================
# 6. STATISTICAL CORRECTNESS
# ============================================================

test_that("KW statistic and p-value match base R kruskal.test()", {
  skip_on_cran()
  base_res <- kruskal.test(Sepal.Width ~ Species, data = iris)
  pkg_res  <- f_kruskal_test(Sepal.Width ~ Species, data = iris, plot = FALSE)
  stored   <- pkg_res[["Sepal.Width_Species"]]$kruskal.test

  expect_equal(stored$statistic, base_res$statistic)
  expect_equal(stored$p.value,   base_res$p.value)
  expect_equal(stored$parameter, base_res$parameter)
})

test_that("significant KW: letters differ across groups", {
  skip_on_cran()
  # iris Sepal.Width ~ Species is highly significant (p << 0.001)
  res     <- f_kruskal_test(Sepal.Width ~ Species, data = iris, plot = FALSE)
  letters <- res[["Sepal.Width_Species"]]$summary_table$Letters

  expect_true(length(unique(letters)) > 1)
  expect_false(any(letters == "ns"))
})

test_that("non-significant KW: all letters are 'ns'", {
  skip_on_cran()
  set.seed(42)
  ns_df <- data.frame(
    y   = rnorm(150, mean = 50, sd = 10),
    grp = factor(rep(c("A", "B", "C"), each = 50))
  )

  res   <- f_kruskal_test(y ~ grp, data = ns_df, plot = FALSE)
  entry <- res[["y_grp"]]

  # Guard: only check letters if KW was indeed non-significant
  # (with this seed and n = 150, it should be, but we guard against rare flukes)
  if (entry$kruskal.test$p.value >= 0.05) {
    expect_true(all(entry$summary_table$Letters == "ns"))
  }
})

test_that("custom alpha is stored and propagated", {
  skip_on_cran()
  res_strict <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                                alpha = 0.001, plot = FALSE)
  res_lax    <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                                alpha = 0.10,  plot = FALSE)

  expect_equal(res_strict[["Sepal.Width_Species"]]$alpha, 0.001)
  expect_equal(res_lax[["Sepal.Width_Species"]]$alpha,    0.10)
})

test_that("Dunn's test has correct number of pairwise comparisons", {
  skip_on_cran()
  # 3 groups -> C(3,2) = 3
  res3 <- f_kruskal_test(Sepal.Width ~ Species, data = iris, plot = FALSE)
  expect_equal(nrow(res3[["Sepal.Width_Species"]]$dunn_test), 3)

  # 6 groups -> C(6,2) = 15
  set.seed(7)
  df6 <- data.frame(
    val   = rnorm(120),
    group = factor(rep(LETTERS[1:6], each = 20))
  )
  res6 <- f_kruskal_test(val ~ group, data = df6, plot = FALSE)
  expect_equal(nrow(res6[["val_group"]]$dunn_test), 15)
})

test_that("Dunn's test: adjust method is passed to rstatix", {
  skip_on_cran()
  res_holm <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                              adjust = "holm", plot = FALSE)
  res_none <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                              adjust = "none", plot = FALSE)

  dunn_holm <- res_holm[["Sepal.Width_Species"]]$dunn_test
  dunn_none <- res_none[["Sepal.Width_Species"]]$dunn_test

  # Raw p-values should be identical (same z-statistics)
  expect_equal(dunn_holm$p, dunn_none$p)

  # "none" means adjusted == raw
  expect_equal(dunn_none$p.adj, dunn_none$p)

  # Holm-adjusted p >= raw p (with floating-point tolerance)
  expect_true(all(dunn_holm$p.adj >= dunn_holm$p - 1e-10))
})


# ============================================================
# 7. ALL VALID ADJUST METHODS
# ============================================================

test_that("all adjust methods run without error", {
  skip_on_cran()
  methods <- c("holm", "hommel", "bonferroni",
               "hochberg", "bh", "by", "fdr", "none")

  for (m in methods) {
    res <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                           adjust = m, plot = FALSE)
    expect_true(inherits(res, "f_kruskal_test"),
                info = paste("adjust =", m))
    expected_stored <- switch(m, "bh" = "BH", "by" = "BY", m)
    expect_equal(res[["Sepal.Width_Species"]]$DunnTest_adjust, expected_stored,
                 info = paste("stored adjust should match for method:", m))
  }
})


# ============================================================
# 8. EDGE CASES
# ============================================================

test_that("two-group data works (KW with k = 2)", {
  skip_on_cran()
  set.seed(11)
  df2 <- data.frame(
    score = c(rnorm(30, 5), rnorm(30, 8)),
    grp   = factor(rep(c("X", "Y"), each = 30))
  )

  res   <- f_kruskal_test(score ~ grp, data = df2, plot = FALSE)
  entry <- res[["score_grp"]]

  expect_s3_class(entry$kruskal.test, "htest")
  expect_equal(unname(entry$kruskal.test$parameter), 1)   # df = k - 1 = 1
  expect_equal(nrow(entry$summary_table), 2)
  # Dunn's test: C(2,2) = 1 pairwise comparison
  expect_equal(nrow(entry$dunn_test), 1)
})

test_that("integer/ordinal data with many ties works", {
  skip_on_cran()
  set.seed(99)
  df_tie <- data.frame(
    rating    = c(sample(1:5, 50, replace = TRUE, prob = c(.40, .30, .20, .08, .02)),
                  sample(1:5, 50, replace = TRUE, prob = c(.05, .10, .25, .30, .30)),
                  sample(1:5, 50, replace = TRUE, prob = c(.10, .15, .25, .25, .25))),
    condition = factor(rep(c("low", "med", "high"), each = 50))
  )

  res <- f_kruskal_test(rating ~ condition, data = df_tie, plot = FALSE)
  expect_s3_class(res, "f_kruskal_test")
  expect_true(is.data.frame(res[["rating_condition"]]$summary_table))
  expect_equal(nrow(res[["rating_condition"]]$summary_table), 3)
})

test_that("data with NAs: runs without error, structure intact", {
  skip_on_cran()
  na_df <- iris[, c("Sepal.Width", "Species")]
  na_df$Sepal.Width[c(1, 25, 75, 130)] <- NA

  res <- f_kruskal_test(Sepal.Width ~ Species, data = na_df, plot = FALSE)

  expect_s3_class(res, "f_kruskal_test")
  expect_true(is.data.frame(res[["Sepal.Width_Species"]]$summary_table))
  expect_s3_class(res[["Sepal.Width_Species"]]$kruskal.test, "htest")
})

test_that("many groups (k = 6) work", {
  skip_on_cran()
  set.seed(7)
  df6 <- data.frame(
    val   = rnorm(120),
    group = factor(rep(LETTERS[1:6], each = 20))
  )

  res   <- f_kruskal_test(val ~ group, data = df6, plot = FALSE)
  entry <- res[["val_group"]]

  expect_equal(nrow(entry$summary_table), 6)
  expect_equal(unname(entry$kruskal.test$parameter), 5)  # df = 6 - 1
})

test_that("numeric predictor is auto-converted to factor", {
  skip_on_cran()
  df_num <- data.frame(
    y = rnorm(60),
    x = rep(c(1, 2, 3), each = 20)
  )

  # x is numeric, should be silently coerced to factor
  res <- f_kruskal_test(y ~ x, data = df_num, plot = FALSE)
  expect_s3_class(res, "f_kruskal_test")
  expect_equal(nrow(res[["y_x"]]$summary_table), 3)
})

test_that("character predictor works (coerced to factor)", {
  skip_on_cran()
  df_chr <- data.frame(
    y = rnorm(60),
    g = rep(c("alpha", "beta", "gamma"), each = 20),
    stringsAsFactors = FALSE
  )

  res <- f_kruskal_test(y ~ g, data = df_chr, plot = FALSE)
  expect_s3_class(res, "f_kruskal_test")
  expect_equal(nrow(res[["y_g"]]$summary_table), 3)
})


# ============================================================
# 9. OUTPUT TYPES
# ============================================================

test_that("output_type = 'default': returns list silently", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                         plot = FALSE, output_type = "default")
  expect_s3_class(res, "f_kruskal_test")
})

test_that("output_type = 'console': prints and returns invisibly", {
  skip_on_cran()
  out <- capture.output(
    res <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                           plot = FALSE, output_type = "console")
  )
  expect_s3_class(res, "f_kruskal_test")
  expect_true(length(out) > 0)
})

test_that("output_type = 'rmd': includes rmd character element", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                         plot = FALSE, output_type = "rmd")

  expect_true("rmd" %in% names(res))
  expect_true(is.character(res$rmd))
  expect_true(nchar(res$rmd) > 100)
})

test_that("intro_text controls assumptions section in rmd output", {
  skip_on_cran()
  res_with <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                              plot = FALSE, output_type = "rmd", intro_text = TRUE)
  res_without <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                                 plot = FALSE, output_type = "rmd", intro_text = FALSE)

  expect_true(grepl("# Assumptions of the Kruskal-Wallis Test",  res_with$rmd))
  expect_false(grepl("# Assumptions of the Kruskal-Wallis Test", res_without$rmd))

})


# ============================================================
# 10. PRINT METHOD
# ============================================================

test_that("print method runs without error", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width ~ Species, data = iris, plot = FALSE)
  out <- capture.output(print(res))

  expect_true(length(out) > 0)
  expect_true(any(grepl("Kruskal", out, ignore.case = TRUE)))
})

test_that("print method uses stored alpha, not hardcoded 0.05", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                         alpha = 0.10, plot = FALSE)
  out <- capture.output(print(res))

  # The stored alpha 0.1 should appear in the printed output
  expect_true(any(grepl("0.1", out)))
})

test_that("print method handles non-significant result gracefully", {
  skip_on_cran()
  set.seed(42)
  ns_df <- data.frame(
    y   = rnorm(150, 50, 10),
    grp = factor(rep(c("A", "B", "C"), each = 50))
  )

  res <- f_kruskal_test(y ~ grp, data = ns_df, plot = FALSE)
  # Should not error regardless of significance
  out <- capture.output(print(res))
  expect_true(length(out) > 0)
})


# ============================================================
# 11. INTRO TEXT AND MULTIPLE-TESTING WARNING
# ============================================================

test_that("intro_text = FALSE does not cause errors", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                         plot = FALSE, intro_text = FALSE)
  expect_s3_class(res, "f_kruskal_test")
})

test_that("multiple-testing warning appears in rmd when k > 1", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width + Sepal.Length ~ Species,
                         data = iris, plot = FALSE, output_type = "rmd")

  expect_true(grepl("Multiple Testing", res$rmd))
})

test_that("multiple-testing warning absent in rmd when k = 1", {
  skip_on_cran()
  res <- f_kruskal_test(Sepal.Width ~ Species,
                         data = iris, plot = FALSE, output_type = "rmd")

  expect_false(grepl("Multiple Testing", res$rmd))
})


# ============================================================
# 12. STRESS: COMBINE MANY OPTIONS AT ONCE
# ============================================================

test_that("stress: multiple responses, custom alpha, holm adjust, no plots, rmd output", {
  skip_on_cran()
  res <- f_kruskal_test(
    Sepal.Width + Sepal.Length + Petal.Length + Petal.Width ~ Species,
    data   = iris,
    alpha  = 0.01,
    adjust = "holm",
    plot   = FALSE,
    intro_text = FALSE,
    output_type = "rmd"
  )

  # 4 responses x 1 predictor = 4 entries + 1 rmd element
  expect_equal(length(res), 5)
  expect_true("rmd" %in% names(res))

  # All 4 response entries present
  for (resp in c("Sepal.Width", "Sepal.Length", "Petal.Length", "Petal.Width")) {
    key <- paste0(resp, "_Species")
    expect_true(key %in% names(res), info = paste("missing:", key))
    expect_equal(res[[key]]$alpha, 0.01, info = key)
    expect_equal(res[[key]]$DunnTest_adjust, "holm", info = key)
  }

  # rmd should contain the multiple-testing warning (k = 4)
  expect_true(grepl("Multiple Testing", res$rmd))
  expect_true(grepl("4.*Kruskal-Wallis Tests", res$rmd))  # "4 Kruskal-Wallis Tests"
})

test_that("stress: 2 responses x 2 predictors, fdr adjust, plots, default output", {
  skip_on_cran()
  df <- data.frame(
    y1 = rnorm(90), y2 = rnorm(90, 10),
    f1 = factor(rep(c("lo", "mid", "hi"), each = 30)),
    f2 = factor(rep(c("ctrl", "trt"), times = 45))
  )

  res <- f_kruskal_test(
    y1 + y2 ~ f1 + f2,
    data   = df,
    alpha  = 0.05,
    adjust = "fdr",
    plot   = TRUE,
    output_type = "default"
  )

  expected <- c("y1_f1", "y1_f2", "y2_f1", "y2_f2")
  expect_true(all(expected %in% names(res)))

  # Each entry should have ggplot objects (plot = TRUE)
  for (key in expected) {
    expect_true(inherits(res[[key]]$distributions, "ggplot"), info = key)
    expect_true(inherits(res[[key]]$Boxplot,       "ggplot"), info = key)
    expect_equal(res[[key]]$DunnTest_adjust,  "fdr",    info = key)
  }
})

test_that("stress: large dataset (1000 obs x 10 groups)", {
  skip_on_cran()
  set.seed(321)
  big_df <- data.frame(
    value = c(rnorm(500, 10, 3), rnorm(500, 12, 3)),
    group = factor(rep(paste0("G", sprintf("%02d", 1:10)), each = 100))
  )

  res <- f_kruskal_test(value ~ group, data = big_df, plot = FALSE)

  expect_s3_class(res, "f_kruskal_test")
  entry <- res[["value_group"]]

  expect_equal(nrow(entry$summary_table), 10)
  expect_equal(unname(entry$kruskal.test$parameter), 9)  # df = 10 - 1
  # C(10,2) = 45 pairwise comparisons
  expect_equal(nrow(entry$dunn_test), 45)
})

# =============================================================================
# f_kruskal_test_new_tests.R
# Additional tests for f_kruskal_test() covering NEW behavior:
#
#   subset / na.action pass-through via stats::model.frame(), so the
#   descriptive summary, density plot, boxplot, Dunn's test and the
#   KW test itself all see the exact same row set.
# =============================================================================
# ---------------------------------------------------------------------------
# A. subset pass-through
# ---------------------------------------------------------------------------

test_that("subset drops rows before kruskal.test is called", {
  skip_on_cran()
  # iris has 50 rows per Species. Subsetting away one Species should
  # leave 100 rows in the model and only two groups in the summary.
  res <- suppressMessages(
    f_kruskal_test(Sepal.Width ~ Species, data = iris,
                   subset = iris$Species != "setosa",
                   plot = FALSE, output_type = "default")
  )

  entry <- res[[1]]
  # Summary table reflects the subset (2 Species only)
  expect_equal(nrow(entry$summary_table), 2L)
  expect_false("setosa" %in% as.character(entry$summary_table$Species))

  # The n column should sum to 100 (50 versicolor + 50 virginica)
  expect_equal(sum(entry$summary_table$n), 100L)
})

test_that("subset flows through to Dunn's test (same row set)", {
  skip_on_cran()
  res <- suppressMessages(
    f_kruskal_test(Sepal.Width ~ Species, data = iris,
                   subset = iris$Species != "virginica",
                   plot = FALSE, output_type = "default")
  )

  dunn <- res[[1]]$dunn_test
  # With only two Species left, Dunn's test has exactly 1 comparison
  expect_equal(nrow(dunn), 1L)
  compared <- c(dunn$group1, dunn$group2)
  expect_false("virginica" %in% compared)
})

# ---------------------------------------------------------------------------
# B. na.action pass-through
# ---------------------------------------------------------------------------

test_that("na.action = na.omit strips NA rows consistently", {
  skip_on_cran()
  df <- iris
  df$Sepal.Width[c(1, 2, 3, 60, 120)] <- NA    # 5 NAs across groups

  res <- suppressMessages(
    f_kruskal_test(Sepal.Width ~ Species, data = df,
                   na.action = na.omit,
                   plot = FALSE, output_type = "default")
  )

  entry <- res[[1]]
  # Total n after NA drop
  expect_equal(sum(entry$summary_table$n), 145L)   # 150 - 5
})

test_that("default behavior (no dots) still works and uses full data", {
  skip_on_cran()
  # Regression guard: default path with no subset / na.action should
  # fit on all 150 iris rows exactly as before.
  res <- suppressMessages(
    f_kruskal_test(Sepal.Width ~ Species, data = iris,
                   plot = FALSE, output_type = "default")
  )
  entry <- res[[1]]
  expect_equal(sum(entry$summary_table$n), 150L)
  expect_equal(nrow(entry$summary_table), 3L)
})

#' Perform multiple \code{aov()} functions with optional data transformation, inspection and Post Hoc test.
#'
#' Performs an Analysis of Variance (ANOVA) on a given dataset with options for (Box-Cox)
#' transformations, normality tests, and post hoc analysis. Several response parameters can be analysed in sequence and the generated output can be in various formats ('Word', 'pdf', 'Excel').
#'
#' @param formula A formula specifying the model to be fitted. More response variables can be added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor}) to do a sequential \code{aov()} for each response parameter.
#' @param data A data frame containing the variables in the model.
#' @param norm_plots Logical. If \code{TRUE}, diagnostic residual plots are included in the output files. Default is \code{TRUE}.
#' @param interaction_plots Logical. If \code{TRUE}, estimated means / interaction plots are included in the output files after the post hoc table. Default is \code{TRUE}.
#' @param ANCOVA Logical. If \code{TRUE}, prevents automatic conversion of predictors to factors, allowing for Analysis of Covariance (ANCOVA). Default is \code{FALSE}.
#' @param transformation Logical or character string. If \code{TRUE}, or if \code{"boxcox"} applies a \code{f_boxcox()} transformation if residuals are not normal. If \code{"bestnormalize"}, applies \code{f_bestNormalize()} transformation. If \code{FALSE} no transformation will be applied. Default is \code{TRUE}.
#' @param force_transformation Character string. A vector containing the names of response variables that should be transformed regardless of the normality test. Default is \code{NULL}
#' @param force_aov Logical. If \code{TRUE}, runs the ANOVA even when at least
#'   one cell has \eqn{n = 1} (saturated model). By default (\code{FALSE}),
#'   such responses are skipped with a warning because F-statistics and
#'   p-values are undefined for saturated models. Set to \code{TRUE} only for
#'   diagnostic purposes -- results should \strong{not} be reported or
#'   interpreted as valid. Default is \code{FALSE}.
#' @param alpha Numeric. Significance level for ANOVA, post hoc tests, and Shapiro-Wilk test. Default is \code{0.05}.
#' @param adjust Character string specifying the method used to adjust p-values
#'   for multiple comparisons. Available methods include:
#'   \describe{
#'     \item{"tukey"}{Tukey's Honest Significant Difference method, appropriate for
#'                   all pairwise comparisons. Controls family-wise error rate.}
#'     \item{"sidak"}{Sidak correction that controls the family-wise error rate.
#'                   Less conservative than Bonferroni.}
#'     \item{"bonferroni"}{Conservative adjustment that multiplies p-values by
#'                        the number of comparisons.}
#'     \item{"none"}{No adjustment. Equivalent to Fisher's LSD method.}
#'     \item{"fdr"}{False Discovery Rate adjustment, controls the expected proportion
#'                 of false positives among significant results.}
#'   } Default is \code{"sidak"}.
#' @param intro_text Logical. If \code{TRUE}, includes a short explanation about ANOVA assumptions in the output file. Default is \code{TRUE}.
#' @param close_generated_files Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param output_type Character string specifying the output format. Default is \code{"default"}.
#'   \itemize{
#'     \item \code{"default"}: Returns the object and lets R decide whether
#'       to print; auto-prints if unassigned, silent if assigned to a variable.
#'       Use \code{print(result)} or \code{plot(result)} to display the
#'       returned object.
#'     \item \code{"console"}: Forces immediate printing to the console
#'       regardless of object assignment.
#'     \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
#'       file of the corresponding format. See \code{save_as},
#'       \code{save_in_wdir}, and \code{open_generated_files} for file
#'       path and opening behavior.
#'     \item \code{"rmd"}: Stores the raw markdown string inside the returned
#'       object for use in R Markdown documents.
#'   }
#'
#' @param save_as Character string specifying the output file path (without extension).
#'   If a full path is provided, output is saved to that location.
#'   If only a filename is given, the file is saved in \code{tempdir()}.
#'   If only a directory is specified (providing an existing directory with trailing slash),
#'   the file is named "dataname_aov_output" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
#'   Defaults to \code{file.path(tempdir(), "dataname_summary.pdf")}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.
#' @param ... Additional arguments forwarded to \code{\link[stats]{aov}}.
#'   The arguments \code{subset}, \code{na.action}, and \code{weights} are
#'   handled specially: when supplied, they are applied via
#'   \code{\link[stats]{model.frame}} so that the n=1 cell check,
#'   Shapiro-Wilk test, Levene test, optional transformations, residual
#'   diagnostics, and \code{emmeans} post hoc tests all see the exact
#'   same row set as \code{aov()} itself. Any other \code{aov()}
#'   arguments (e.g. \code{contrasts}, \code{projections},
#'   \code{qr}, \code{contrasts.arg}) are passed through unchanged.

#' @return An object of class 'f_aov' containing results from \code{aov()}, normality tests, transformations, and post hoc tests. Using the option "output_type", it can also generate output in the form of: R Markdown code, 'Word', 'pdf', or 'Excel' files. Includes print and plot methods for 'f_aov' objects.

#' @details
#' The function performs the following steps:
#' \itemize{
#' \item Check if all specified variables are present in the data.
#' \item Ensure that the response variable is numeric.
#' \item Perform Analysis of Variance (ANOVA) using the specified formula and data.
#' \item If \code{shapiro = TRUE}, check for normality of residuals using the Shapiro-Wilk test.
#' \item If residuals are not normal and \code{transformation = TRUE} apply a data transformation.
#' \item If significant differences are found in ANOVA, proceed with post hoc tests using estimated marginal means from \code{emmeans()} and Sidak adjustment (or another option of \code{adjust =}.
#' }
#'
#' More response variables can be added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor}) to do a sequential \code{aov()} for each response parameter captured in one output file.
#'
#' Outputs can be generated in multiple formats ("pdf", "word", "excel" and "rmd") as specified by \code{output_type}. The function also closes any open 'Word' files to avoid conflicts when generating 'Word' documents. If \code{output_type = "rmd"} is used it is adviced to use it in a chunk with \{r, echo=FALSE, results='asis'\}
#'
#' *Non-significant ANOVA results*: When the overall F-test is not significant, f_aov still reports the estimated marginal means table, but with all pairwise comparison letters replaced by *"ns"*. The numeric estimates (and their confidence intervals) are provided because they are often needed for manuscript tables, especially when the response was back-transformed from a Box-Cox or bestNormalize scale - the raw descriptive means and the emmeans values can differ, and it is the emmeans values that correspond to the actual model. The *"ns"* labels signal that pairwise differences should not be interpreted.
#'
#' This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
#'\itemize{
#' \item \bold{Windows:} Install Pandoc and ensure the installation folder.
#' \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
#' \item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
#' \item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
#'
#' \item If Pandoc is not found, this function may not work as intended.
#' }
#'
#' @section Multiple Testing Across Response Variables:
#' When several response variables are analysed in a single call
#' (e.g. \code{y1 + y2 + y3 ~ treatment}), each ANOVA is an independent
#' null-hypothesis test at level \code{alpha}. The post hoc adjustments
#' (\code{adjust = "sidak"}, \code{"tukey"}, etc.) only control the
#' family-wise error rate \strong{within} one ANOVA (across pairwise group
#' comparisons for that response). They do \strong{not} protect against
#' the inflation of Type I error \strong{across} the set of responses.
#'
#' \strong{Practical implication:} With \eqn{k} independent response
#' variables all tested at \eqn{\alpha = 0.05}, the probability of
#' obtaining at least one false positive is
#' \eqn{1 - (1 - 0.05)^k}, which reaches ~40\% for \eqn{k = 10}.
#'
#' \strong{When this matters:} The risk is highest in exploratory studies
#' where many responses are screened simultaneously without a clear
#' a priori hypothesis for each one. It is less of a concern when
#' each response is a pre-specified primary outcome with its own
#' biological rationale.
#'
#' \strong{Possible remedies:}
#' \itemize{
#'   \item \strong{Bonferroni correction across responses:} use
#'     \code{alpha = 0.05 / k} where \code{k} is the number of
#'     response variables. Conservative but simple.
#'   \item \strong{False Discovery Rate (FDR):} apply
#'     \code{p.adjust(p_values, method = "fdr")} to the vector of
#'     per-response ANOVA p-values after the fact.
#'   \item \strong{MANOVA:} if the responses are correlated and you
#'     want a single omnibus test across all of them, use
#'     \code{manova()} before interpreting individual ANOVAs.
#'   \item \strong{Pre-registration:} declare primary vs. exploratory
#'     responses before data collection to justify differential
#'     correction thresholds.
#' }
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # Make a factor of Species.
#' iris$Species <- factor(iris$Species)
#'
#' # The left hand side contains two response variables,
#' # so two aov's will be conducted, i.e. "Sepal.Width"
#' # and "Sepal.Length" in response to the explanatory variable: "Species".
#' f_aov_out <- f_aov(Sepal.Width + Sepal.Length ~ Species,
#'                    data = iris,
#'                    # Save output in MS Word file (Default is console)
#'                    output_type = "word",
#'                    # Do bestNormalize transformation for non-normal residual (Default is boxcox)
#'                    transformation = "bestnormalize"
#'                    )
#'
#' # Print output to the console.
#' print(f_aov_out)
#'
#' # Plot residual plots.
#' plot(f_aov_out)
#'
#' #To print rmd output set chunck option to results = 'asis' and use cat().
#' f_aov_rmd_out <- f_aov(Sepal.Width ~ Species, data = iris, output_type = "rmd")
#' cat(f_aov_rmd_out$rmd)
#'
#' @export
f_aov <- function(formula,
                  # aov function formula
                  data = NULL,
                  # Data.frame used for aov
                  norm_plots = TRUE,
                  # Show diagnostic residual plots in output files
                  interaction_plots = TRUE,
                  # Show estimated means / interaction plots in output files
                  ANCOVA = FALSE,
                  # Prevent automatic conversion to factors of all predictors for ANCOVA
                  transformation = TRUE,
                  # Preform transformation
                  force_transformation = NULL,
                  # force transformation for response var regardless of normality.
                  force_aov = FALSE,
                  # If TRUE, runs ANOVA even when min cell size = 1 (saturated model).
                  # Results are unreliable -- use only for diagnostics, not for reporting.
                  alpha = 0.05,
                  # Significance level for both aov, posthoc and Shapiro-Wilk Test
                  adjust = "sidak",
                  # Specifying the method used to adjust p-values
                  intro_text = TRUE,
                  # Print short explanation about aov assumptions in output file
                  close_generated_files = FALSE,
                  # Closes either open excel or word files depending on the output format.
                  open_generated_files = interactive(),
                  # Open files after creation
                  output_type = "default",
                  # Output type can be excel, word, pdf, rmd, console, default
                  save_as = NULL,
                  # Specify the name of the output dir and file (name and type).
                  save_in_wdir = FALSE,         # Save file output in the working directory.
                  ...
                  # Additional arguments forwarded to aov(). Currently
                  # honored: subset, na.action, weights. If supplied, they
                  # are applied via stats::model.frame() so that ALL
                  # downstream steps (n=1 cell check, Shapiro, Levene,
                  # transformations, residual plots, emmeans post hoc)
                  # see the exact same row set as aov() itself.
)
{



  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state


  ########## Capture ... UNEVALUATED at f_aov's own frame ##################
  # Must be done HERE (in f_aov's body), not inside generate_report(),
  # because match.call() captures the call to whatever function it is
  # called from. Inside generate_report() it would capture
  # generate_report()'s (empty) call and dots_exprs would always be
  # NULL -- silently dropping subset / weights / na.action. The closure
  # makes dots_exprs visible inside generate_report().
  .mc         <- match.call(expand.dots = FALSE)
  # Coerce the `...` pairlist returned by match.call(expand.dots = FALSE)
  # to a real named list. Without as.list(), dots_exprs$subset partial-
  # matches into the raw pairlist and returns a `..1` dots-index symbol
  # instead of the actual expression, which then crashes model.frame()
  # with "the ... list contains fewer than 3 elements" the moment a
  # user actually passes subset / weights / na.action through dots.
  dots_exprs  <- as.list(.mc[["..."]])

  # Capture the user's calling environment so that subset / weights /
  # na.action expressions passed via `...` can be resolved against it
  # (after `data` columns) from inside the generate_report() closure,
  # where parent.frame() would otherwise point at f_aov itself.
  caller_env  <- parent.frame()


  ####### Save dataframe name and Handle input from vectors (dataframe column) #####

  if(!is.null(data)){
    # Save dataframe name
    data_name <- deparse(substitute(data))

  } else if(is.null(data)){

    if(length(formula_extract_df_names(formula)) == 0){
      data_name <- "data"

    } else if(length(formula_extract_df_names(formula)) == 1){
      data_name <- formula_extract_df_names(formula)

    } else if(length(formula_extract_df_names(formula)) > 1){
      data_name <- paste(formula_extract_df_names(formula), collapse = "_")

    }

    # Make a data.frame based on the formula
    data <- formula_to_dataframe(formula)

    # Rewrite formula without data frame prefixes
    formula <- clean_formula(formula)

  }

  # Generate a temporary file path for "output.Rmd"
  temp_output_dir <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")

  # Create the output file "output.Rmd" in tempdir()
  file.create(temp_output_file)
  file_extension <- NULL

  # Wrap lines in rmd output document
  f_wrap_lines()

  # Create a list to store all outputs in this function
  output_list <- list()

  # Parameter validation
  if( !(output_type %in% c("pdf", "word", "excel", "rmd", "console" , "default")) ){
    stop("Character string specifying the output format (output_type = ) should be either: 'pdf', 'word', 'excel', 'console','rmd', 'default'")
  }


  #### Handle option "save_as = " ###
  if(save_in_wdir == TRUE){
    save_dir <- getwd()
  }else{
    save_dir <- tempdir()
  }

  #map the output type to extensions
  output_type_map <- c(
    "pdf"  = ".pdf",
    "word" = ".docx",
    "excel"= ".xlsx",
    "rmd"  = ".rmd"
  )

  # If the user specifies a path, filename or save_in_wdir == TRUE an output file should be created
  if (!is.null(save_as) || save_in_wdir == TRUE) {

    if (!is.null(save_as)) {
      #Remove backslash in save_as if needed
      save_as <- gsub(pattern = "\\\\", replacement = "/", x = save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if(file_extension_save_as[1] != FALSE){
        file_extension <- file_extension_save_as
      }
    }

    if(is.null(file_extension) && output_type %in% c("console", "default")){
      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "aov_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = ".pdf"
      )
      #set output_type to default
      output_type <- "pdf"

    }
    else if(is.null(file_extension) && output_type %in% c("pdf", "word", "excel", "rmd")){

      #create extension based on input_type
      file.ext <- unname(output_type_map[output_type])

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "aov_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file.ext
      )


    }
    else if(!is.null(file_extension)) {

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "aov_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file_extension[1]
      )
      # reset the output type to match the user input extention in save_as
      output_type <- file_extension[2]
    }
  } else {

    #create extension based on input_type
    file.ext <- unname(output_type_map[output_type])

    # use helper get_save_path() to create output_path
    output_path <- get_save_path(save_as = save_as,
                                 default_name = paste(data_name, "aov_output", sep = "_"),
                                 default_dir = save_dir,
                                 file.ext = file.ext
    )
  }


  # Prevent output to console and keep files open when output is "rmd" format
  if (output_type == "rmd") close_generated_files <- FALSE

  # Cross-platform close_generated_files (was Windows-only taskkill)
  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    if (output_type == "excel") close_app("EXCEL.EXE",   "Microsoft Excel", "soffice")
  }



  # Convert the input to a character string and force lowercase for case-insensitive matching
  if(is.character(transformation)){
    trans_input <- tolower(as.character(transformation))

    # Valid options include the full names and also the string versions of logicals.
    valid_options <- c("bestnormalize", "boxcox", "true", "false")

    # Use pmatch for partial matching; pmatch returns NA for ambiguous or no matches.
    matched_index <- pmatch(trans_input, valid_options)

    if (is.na(matched_index))
      stop("Invalid transformation option!")

    matched_option <- valid_options[matched_index]

    # Convert string "true" or "false" to logical values if applicable
    if (matched_option == "true") {
      transformation <- TRUE
    } else if (matched_option == "false") {
      transformation <- FALSE
    } else {
      transformation <- matched_option  # Either "bestnormalize" or "boxcox"
    }
  }

  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(formula) #use helper_check_lhs.R

  # Extract response variables from the left-hand side of the formula
  lhs <- all.vars(formula[[2]])  # Get LHS variables (response)
  response_names <- lhs

  # Extract predictor variables from the right-hand side of the formula
  predictor_names <- all.vars(formula[[3]]) # Get RHS variables (predictors)

  # Extract the right-hand side (RHS) of the formula as a string
  rhs <- deparse(formula[[3]])  # Preserve the RHS structure

  # Ensure response and predictors are in the data
  for (response in response_names) {

    if (!(response %in% names(data))) {
      stop(paste("Response variable", response, "not found in the data."))
    }
    # Ensure the response variable is numeric
    response_var <- data[[response]]

    if (!is.numeric(response_var)) {
      stop("The response variable must be numeric.")
    }
  }

  for (predictor in predictor_names) {
    if (!(predictor %in% names(data))) {
      stop(paste("Predictor variable", predictor, "not found in the data."))
    }
  }

  # Prevent from automatic conversion to factors to allow for ANCOVA
  if(ANCOVA == FALSE){
    # Ensure all predictor variables are factors
    for (predictor in predictor_names) {
      data[[predictor]] <- as.factor(data[[predictor]])
    }
  }

  # This is the main function that generates the content for all the output types
  # output = FALSE, return output_list FALSE no output_list used for rmd output
  generate_report <- function(output = TRUE) {

    # This text reminds the user of the assumptions of an aov
    # it its show by default but can be hidden.
    if(intro_text == TRUE){

      cat("
# Assumptions of ANOVA
Checking the assumptions of ANOVA (Analysis of Variance) are critical for ensuring the validity of its results:\n

## 1. Independence
- Observations must be independent both within and between groups. This means that the value of one observation should not influence another.
- Independence violations cannot be corrected statistically and invalidate the analysis, making proper experimental design essential.

## 2. Normality
- The residuals (errors) of the model are assumed to be normally distributed. This assumption applies to the residuals, not necessarily the raw data.
- ANOVA is robust to minor deviations from normality, especially with large and balanced sample sizes. For small or unbalanced samples, violations can lead to **unreliable results**, requiring a data transformation or alternative tests (Welch's ANOVA, Kruskal-Wallis Test).
- Normality of the residuals can be tested using a Shapiro-Wilk or Anderson-Darling Test. It can also be graphically assessed using a Box Plot, Q-Q plot or Histogram.

## 3. Homogeneity of Variances (Homoscedasticity)
- The variances within each group should be approximately equal (homogeneity of variances). This ensures that the F-test statistic is reliable.
- Homogeneity of variances assumption in ANOVA should be tested on the residuals, not directly on the raw data.
- Levene's test can be applied to check for homogeneity of variances.It can also be graphically assessed by plotting residuals vs. fitted values and checking for patterns.
- If violated, a data transformation or alternative tests (Welch's ANOVA, Kruskal-Wallis Test) are required.


## 4. Additivity (No Unaccounted Systematic Effects)
- For models without interaction terms, it is assumed that the effects of different factors are additive. That is, the combined effect of factors can be expressed as the sum of their individual effects.
- However, if interaction terms are included in the model, this assumption does not apply because ANOVA then explicitly accounts for potential interactions.
  \n  \n")

      if(output_type != "rmd"){
        # Pagebreak
        cat("
<div style=\"page-break-after: always;\"></div>
\\newpage
        ")
      }
    }

    #create count to remove last page break
    i <- 0

    # Multiple-response warning: only shown when > 1 response variable is analysed.
    # Fires regardless of intro_text so the user always sees it in this situation.
    if (length(lhs) > 1) {
      k          <- length(lhs)
      fwer_pct   <- round((1 - (1 - alpha)^k) * 100, 1)
      bonf_alpha <- round(alpha / k, 4)
      cat(paste0(
        "\n\n***\n\n",
        "**[!] NOTE: Multiple Testing Across ", k, " Response Variables**  \n\n",
        "This report runs ", k, " independent ANOVAs on the same dataset. ",
        "The **", adjust,"** correction keeps each individual test honest, it guards against ",
        "false positives among the pairwise group comparisons, but it offers no protection ",
        " against the accumulation of error across all ",k," tests combined. ",
        "\nAt \u03b1 = ", alpha, " per test, the probability of obtaining at least one ",
        "spurious significant result across all ", k, " responses is approximately ",
        "**", fwer_pct, "%** ($1-(1-", alpha, ")^{", k, "}$, assuming independence). ",
        "This risk is highest in exploratory studies; it is less of a concern when ",
        "each response has a clear a priori hypothesis.  \n\n",
        "**Possible remedies:**  \n",
        "\n-  **Bonferroni** (conservative): re-run with `alpha = ", bonf_alpha,
        "` (\u03b1 / ", k, ").",
        "\n-  **False Discovery Rate (FDR)**: apply `p.adjust(p_values, method = \"fdr\")` ",
        "to the ", k, " ANOVA p-values after the fact.",
        "\n-  **MANOVA**: if responses are correlated, use `manova()` as a single ",
        "omnibus test before interpreting individual ANOVAs.",
        "\n-  **Pre-registration**: if each response was a pre-specified (primary) study ",
        "outcome, correction may not be required; document this decision explicitly.  \n  \n",
        "\n\n***\n\n"
      ))
    }

    # Build the analysis data set ONCE, before the response loop.
    # Use ALL responses + ALL predictors so every response is analysed
    # on the identical row set. Rows with NA in ANY response or
    # predictor are dropped (subject to na.action). Any subset /
    # na.action / weights passed via `...` is applied here.
    #
    # Strategy: pre-evaluate subset/weights eagerly against data first,
    # falling back to the user's calling environment. Then apply the
    # filter manually and pass plain values to stats::model.frame via
    # do.call. This avoids the fragile match.call/substitute dance
    # where spliced expressions inside a constructed call can end up
    # containing `..N` dots-index symbols that crash model.frame with
    # "the ... list contains fewer than 3 elements".
    combined_formula <- as.formula(
      paste("~", paste(c(lhs, predictor_names), collapse = " + "))
    )

    # Resolve subset expression against data columns + caller env
    subset_vec <- NULL
    if (!is.null(dots_exprs$subset)) {
      subset_vec <- eval(dots_exprs$subset, envir = data, enclos = caller_env)
      if (is.logical(subset_vec)) subset_vec[is.na(subset_vec)] <- FALSE
    }

    # Resolve weights against FULL data before subsetting
    weights_vec <- NULL
    if (!is.null(dots_exprs$weights)) {
      weights_vec <- eval(dots_exprs$weights, envir = data, enclos = caller_env)
    }

    # Apply the manual subset filter to data (and weights, if its
    # length matches the pre-subset row count)
    if (!is.null(subset_vec)) {
      if (!is.null(weights_vec) && length(weights_vec) == nrow(data)) {
        weights_vec <- weights_vec[subset_vec]
      }
      data <- data[subset_vec, , drop = FALSE]
    }

    # Resolve na.action (a function, defaults to na.omit)
    na_action_fn <- stats::na.omit
    if (!is.null(dots_exprs$na.action)) {
      na_action_fn <- eval(dots_exprs$na.action, envir = caller_env)
    }

    # Build the model.frame call with plain values
    mf_args <- list(
      formula            = combined_formula,
      data               = data,
      drop.unused.levels = TRUE,
      na.action          = na_action_fn
    )
    if (!is.null(weights_vec)) mf_args$weights <- weights_vec

    data_master     <- do.call(stats::model.frame, mf_args)
    n_before_master <- nrow(data)
    n_after_master  <- nrow(data_master)

    # Remaining ... args (e.g. contrasts, projections, qr) are
    # eager-evaluated in the caller's environment because they do not
    # reference data columns.
    extra_names <- setdiff(names(dots_exprs),
                           c("subset", "weights", "na.action"))
    aov_extra   <- lapply(dots_exprs[extra_names], eval,
                          envir = caller_env)
    # ---------------------------------------------------------------

    # Loop for several response parameters
    for (response_name in lhs) {

      # set these paramters to default for each respone_name
      aov_summary_transformed <- NULL
      Response_Transformed    <- FALSE

      # Store the transformation option and adjust method for output
      output_list[[response_name]][["transformation_option"]] <- transformation
      output_list[[response_name]][["adjust"]]                <- adjust

      # Create a new formula for each response, preserving interactions
      current_formula <- as.formula(paste0(response_name, "~", rhs))

      # Document the data loss (same for every response now -- both come
      # from the master filter applied before the loop).
      n_before <- n_before_master
      n_after  <- n_after_master

      # Fresh per-iteration copy of the master filtered frame. Required
      # because the transformation step (Box-Cox / bestNormalize) mutates
      # data_complete[[response_name]] in place; without a copy, that
      # mutation would persist into subsequent responses.
      data_complete <- data_master

      # Extract weights vector (NULL if none supplied) for the aov() calls.
      # model.frame stores them in a "(weights)" column; model.weights() pulls
      # them out as a plain numeric vector aligned with data_complete rows.
      mf_weights <- stats::model.weights(data_complete)

      n_after <- nrow(data_complete)

      # Snapshot the original (untransformed) response before data_complete is
      # modified in-place by the transformation step. Used for raw-data overlay
      # in plot.f_aov so points are always on the original scale.
      original_response <- data_complete[[response_name]]

      # n=1 cell check: must happen BEFORE aov()
      # In a factorial design, a cell with n=1 saturates the model,
      # residual df = 0, F-statistics are undefined, p-values meaningless.
      # Skip this response entirely and continue with the others.
      #
      # Only categorical predictors (factor / character / logical)
      # define cells. In ANCOVA mode, numeric covariates have as many
      # "levels" as observations, which would falsely trigger the n=1
      # skip. Continuous predictors are therefore excluded from the
      # cell-size count.
      cat_predictors <- predictor_names[vapply(
        data_complete[predictor_names],
        function(v) is.factor(v) || is.character(v) || is.logical(v),
        logical(1)
      )]
      if (length(cat_predictors) == 0L) {
        # Pure regression (no categorical predictors): no factorial
        # design, no cell-size issue possible.
        min_cell_n <- nrow(data_complete)
      } else {
        cell_counts <- table(interaction(data_complete[cat_predictors], drop = TRUE))
        min_cell_n  <- min(cell_counts)
      }

      if (min_cell_n == 1) {
        if (!isTRUE(force_aov)) {
          # Default: skip this response entirely, continue with others
          cat(paste0(
            "   \n  \n# Analysis of: ", response_name, "  \n\n",
            "**WARNING: `", response_name, "` skipped.**  \n\n",
            "At least one cell in the factorial design has $n = 1$. ",
            "This saturates the ANOVA model (residual degrees of freedom = 0), ",
            "making F-statistics and p-values undefined and uninterpretable.  \n\n",
            "**What to do:**  \n",
            "\n- Collect more observations so every factor-level combination has $n \\geq 2$.",
            "\n- Simplify the model by removing interaction terms (`", response_name,
            " ~ ", paste(predictor_names, collapse = " + "), "`).",
            "\n- If cells are structurally empty (impossible combinations), ",
            "consider a different experimental design.",
            "\n- If you want to run the ANOVA anyway for diagnostic purposes only, ",
            "re-run with `force_aov = TRUE`. ",
            "Results will be unreliable and should **not** be reported.  \n   \n"
          ))

          skipped_reason <- paste0("ANOVA undefined, n=1 in at least one cell; model ",
                                  "saturated.\n  Only re-run with ",
                                  "`force_aov = TRUE` for diagnostic purposes."
                                  )
          output_list[[response_name]][["skipped"]]        <- TRUE
          output_list[[response_name]][["skipped_reason"]] <- skipped_reason
          output_list[[response_name]][["min_cell_n"]]     <- min_cell_n
          i <- i + 1
          next

        } else {
          # force_aov = TRUE: warn loudly and proceed
          cat(paste0(
            "   \n  \n# Analysis of: ", response_name, "  \n\n",
            "**[!] WARNING  Saturated model: results are for diagnostic use only.**  \n\n",
            "At least one cell has $n = 1$ (`force_aov = TRUE` overrides the skip). ",
            "The ANOVA model has zero residual degrees of freedom. ",
            "F-statistics and p-values below are **undefined and must not be reported**.  \n\n",
            "***  \n\n"
          ))
          output_list[[response_name]][["force_aov_used"]] <- TRUE
          output_list[[response_name]][["min_cell_n"]]     <- min_cell_n
        }
      }
      # ------------------------------------------------------------------

      cat("   \n  \n# Analysis of: ", response_name, "  \n")
      cat("  \n## Normality and homoscedasticity of residuals of: ", response_name, "  \n")


      # Perform ANOVA. Forward any extra args from `...` to aov().
      # subset / na.action / weights have already been baked into
      # data_complete via model.frame(); aov_extra was assembled once at
      # the master level above and contains only safe pass-through args
      # such as `contrasts`. weights is re-supplied from mf_weights.
      aov_test <- do.call(
        stats::aov,
        c(list(formula = current_formula, data = data_complete,
               weights = mf_weights), aov_extra)
      )
      output_list[[response_name]][["aov_test"]] <- aov_test
      res_aov <- residuals(aov_test)
      aov_summary <- summary(aov_test)
      output_list[[response_name]][["aov_summary"]] <- aov_summary
      output_list[[response_name]][["aov_call"]] <- deparse(current_formula)


      # Perform levene test on residuals
      levene_res <- rstatix::levene_test(as.formula(paste0(
        "res_aov~interaction(",
        paste(predictor_names, collapse = ", "),
        ")"
      )), data = data_complete)

      # Create a safe logical check. If p is NA, we assume it is NOT significant (FALSE)
      is_levene_sig <- !is.na(levene_res$p) & levene_res$p < alpha

      # Build the cell-size warning AFTER the test so we can condition on the
      # actual result rather than making preemptive claims that may contradict it.
      # - n=1 AND result IS NA  -> test truly failed; warn accordingly
      # - n=1 BUT result is valid -> Levene managed to compute; note small cells but
      #                              do not say the result is uninformative
      # - n<=3 AND result is valid -> low power note only
      levene_size_warning <- if (min_cell_n == 1 && is.na(levene_res$p)) {
        paste0("**Warning:** At least one group cell contains only $n = 1$ observation. ",
               "Levene's Test could not estimate within-group variance and returned NA ",
               "(result is uninformative).  \n  \n&nbsp;  \n")
      } else if (min_cell_n == 1 && !is.na(levene_res$p)) {
        paste0("**Note:** At least one group cell contains only $n = 1$ observation. ",
               "Levene's Test produced a result, but interpret it with caution; ",
               "variance estimates from single observations are unreliable.  \n  \n&nbsp;  \n")
      } else if (min_cell_n <= 3) {
        paste0("**Note:** The smallest group cell contains only $n = ", min_cell_n,
               "$ observation(s). Levene's Test has very low statistical power with ",
               "such small cell sizes; a non-significant result does **not** reliably ",
               "confirm homoscedasticity.  \n  \n&nbsp;  \n")
      } else {
        paste0("&nbsp;\n  \n&nbsp;  \n")
      }

      # Generate text for interpretation of levene output
      if (is.na(levene_res$p)) {
        levene_res_intp_text <- "Levene's Test could not be calculated (likely due to saturation or insufficient replication per group).  \n"
      } else if (levene_res$p > alpha) {
        levene_res_intp_text <- paste0(
          "According to 'Levene's Test' (",
          round(levene_res$p, digits = 4),
          " > ",
          alpha,
          ") residuals **have equal variance** (homoscedasticity).  \n"
        )
      } else if (is_levene_sig) {
        levene_res_intp_text <- paste0(
          "According to 'Levene's Test' (",
          round(levene_res$p, digits = 4),
          "\u2264",
          alpha,
          ") residuals do **NOT** have equal variance (Heteroskedasticity).  \n"
        )
      }
      # levene_output_res
      cat(
        "**Levene's test** for homogeneity of residuals: F-Statistic =",
        round(levene_res$statistic, digits = 4),
        "p-value =",
        round(levene_res$p, digits = 4),
        ". ", levene_res_intp_text)
      if (!is.null(levene_size_warning)) cat(levene_size_warning)

      # Shapiro-Wilk Test for normality of the aov residuals
      # shapiro.test() errors above n = 5000; fall back to AD-only when that occurs.
      # safe_shapiro helper
      shapiro_res <- safe_shapiro(res_aov)


      if (is.na(shapiro_res$p.value)) {
        shapiro_res_intp_text <- paste0(
          "Shapiro-Wilk was skipped (", shapiro_res$method,
          "). Normality decision is based on the Anderson-Darling test below.  \n  \n&nbsp;  \n"
        )
      } else if (shapiro_res$p.value > alpha) {
        shapiro_res_intp_text <- paste0(
          "According to 'Shapiro-Wilk Test' (",
          round(shapiro_res$p.value, digits = 4),
          " > ",
          alpha,
          ") residuals **ARE normally distributed**.  \n  \n&nbsp;  \n"
        )
      } else if (shapiro_res$p.value <= alpha) {
        shapiro_res_intp_text <- paste0(
          "According to 'Shapiro-Wilk Test' (",
          round(shapiro_res$p.value, digits = 4),
          "\u2264",
          alpha,
          ") residuals are **NOT** normally distributed.  \n  \n&nbsp;  \n"
        )
      }
      # shapiro_output_res
      cat(
        "**Shapiro-Wilk Test** for Normality of residuals: W =",
        round(shapiro_res$statistic, digits = 4),
        "p-value =",
        round(shapiro_res$p.value, digits = 4),
        ". ", shapiro_res_intp_text
      )

      # Perform Anderson-Darling normality test on residuals
      # ad.test() requires n >= 8; skip gracefully for very small samples
      if (length(res_aov) >= 8) {
        adt_res <- nortest::ad.test(res_aov)
      } else {
        adt_res <- list(statistic = NA_real_, p.value = NA_real_,
                        method = "Anderson-Darling test (skipped: n < 8)")
      }

      if (is.na(adt_res$p.value)) {
        adt_res_intp_text <- paste0(
          "Anderson-Darling test skipped (n < 8, minimum sample size not met).  \n  \n"
        )
      } else if (as.numeric(adt_res$p.value) > alpha) {
        adt_res_intp_text <- paste0(
          "According to 'Anderson-Darling test' (",
          round(adt_res$p.value, digits = 4),
          " > ",
          alpha,
          ") residuals **ARE normally distributed**.  \n  \n"
        )
      } else if (as.numeric(adt_res$p.value) <= alpha) {
        adt_res_intp_text <- paste0(
          "According to 'Anderson-Darling test' (",
          round(adt_res$p.value, digits = 4),
          "\u2264",
          alpha,
          ") residuals are **NOT** normally distributed.  \n  \n"
        )
      }
      # adt_output_res
      cat(
        adt_res$method,
        ": A =",
        round(adt_res$statistic, digits = 4),
        " p =",
        round(adt_res$p.value, digits = 4),
        "   \n", adt_res_intp_text
      )

      output_list[[response_name]][["Levene_test_on_res"]] <- levene_res
      output_list[[response_name]][["min_cell_n"]]         <- min_cell_n
      output_list[[response_name]][["shapiro_test_residuals"]] <- shapiro_res
      output_list[[response_name]][["adt_test_residuals"]] <- adt_res

      temp_file <- tempfile(fileext = ".png")
      png(temp_file, width = 7.8, height = 7.8, units = "in", res = 300)
      # Set up a 2x2 plotting layout with adjusted margins mar = c(bottom, left, top, right)
      par(mfrow = c(2, 2), mar = c(4, 4, 4, 1))  # Adjust left margin

      # Residual plot aov
      plot(aov_test, 1)

      # Boxplot
      boxplot(residuals(aov_test), main = "Boxplot", xlab = "Residuals")

      # Histogram
      f_hist(residuals(aov_test),  xlab = "Residuals")

      # QQPlot
      f_qqnorm(residuals(aov_test))

      # Close the png file
      dev.off()

      if(norm_plots == TRUE){
        # Include the saved plots in R Markdown
        cat("Check the plots in the figure below to assess normality.  \n")
        cat(paste0("![](", temp_file, ")"), "   \n  \n")
        # cat("&nbsp;\n   \n")
      }

      output_list[[response_name]][["normality_plots"]] <- temp_file
      output_list[[response_name]][["alpha"]] <- alpha


      if(output_type != "rmd"){
        # Pagebreak
        cat("
<div style=\"page-break-after: always;\"></div>
\\newpage
        ")
      }

      # Decide whether residuals fail the normality check.
      # Shapiro-Wilk is the primary test because it is the most powerful for
      # n <= 5000. For n > 5000 it is skipped (safe_shapiro returns NA), and
      # we fall back to Anderson-Darling, which has no upper sample-size
      # limit and is also reasonably powerful. If both are unavailable
      # (extreme edge case: n < 3 and n < 8), we leave trigger_normality
      # FALSE and defer the decision to the user inspecting the qq-plot.
      if (!is.na(shapiro_res$p.value)) {
        trigger_normality <- shapiro_res$p.value < alpha
      } else if (!is.na(adt_res$p.value)) {
        trigger_normality <- adt_res$p.value < alpha
      } else {
        trigger_normality <- FALSE
      }
      trigger_levene    <- is_levene_sig
      trigger_forced    <- response_name %in% force_transformation

      # If not normal or force_transformation directs to: apply transformation
      if (trigger_normality || trigger_levene || trigger_forced) {
        if (transformation == FALSE) {
          if (trigger_normality) {
            test_label <- if (!is.na(shapiro_res$p.value)) "Shapiro-Wilk" else "Anderson-Darling"
            cat(paste0(
              "   \n  \n**WARNING !!!**   \nBased on the ", test_label,
              " test the residuals are **NOT** normally distributed.   \n  \n",
              "Please enable the transformation function (transformation = TRUE) ",
              "in the f_aov function.   \n  \n"
            ))
          }
          if (trigger_levene){
            cat("   \n  \n**WARNING !!!**   \nBased on the Levene's Test the residuals do **NOT** have equal variance (Heteroskedasticity).   \n  \nPlease enable the transformation function function (transformation  == TRUE) in the f_aov function.   \n  \n")
          }
        }

        # Only apply Box-Cox / bestNormalize when normality is violated or forced.
        # Levene-only failure is flagged but transformation is not automatically applied,
        # because Box-Cox targets non-normality; a variance-stabilising transformation
        # (e.g. log/sqrt) may be more appropriate for pure heteroscedasticity.
        if (!trigger_normality && !trigger_forced && trigger_levene && transformation != FALSE) {
          cat("   \n  \n**NOTE**   \nHeteroscedasticity was detected (Levene's Test), but residuals are normally distributed (Shapiro-Wilk). Welch's ANOVA (`oneway.test(..., var.equal = FALSE)`) is recommended. \nNevertheless, Box-Cox is applied since it can also stabilise variance; check the transformed Levene result below.   \nIf heteroscedasticity persists after transformation, resort to other statistical tests.   \n  \n")
        }

        if (transformation == "boxcox" || (transformation == TRUE)) {
          cat("\n   \n## Box-Cox transformation of: ", response_name, "  \n  \n")

          capture.output(transformed_var <- f_boxcox(data_complete[[response_name]],
                                                     output_type = "rmd",
                                                     alpha = alpha
          )
          )
          cat("Note: the Shapiro-Wilk test below is performed on the response data, not the residuals.")
          cat(transformed_var$rmd)

          output_list[[response_name]][["boxcox"]] <-  transformed_var$transformed_data
          data_complete[[response_name]] <- transformed_var$transformed_data# Update the response variable in the data

        }

        if (transformation == "bestnormalize" ) {

          # Apply the f_bestNormalize function
          transformed_var <- f_bestNormalize(data_complete[[response_name]],
                                             data_name = response_name,
                                             output_type = "rmd",
                                             alpha = alpha
          )

          # Save the transformed data object
          output_list[[response_name]][["bestNormalize"]] <-  transformed_var

          # Save the transformed data
          data_complete[[response_name]] <- transformed_var$transformed_data  # Update the response variable in the data

          # Print the ouput of f_bestNormalize
          cat("Note: the Shapiro-Wilk test below is performed on the response data, not the residuals.")
          cat(transformed_var$rmd)
          cat("    \n    \n")
        }

        if (transformation != FALSE) {
          # Perform ANOVA on transformed data
          aov_test_transformed <- do.call(
            stats::aov,
            c(list(formula = current_formula, data = data_complete,
                   weights = mf_weights), aov_extra)
          )
          transformed_aov_res  <- residuals(aov_test_transformed)
          aov_summary_transformed <- summary(aov_test_transformed)

          # Perform Shapiro-Wilk test on transformed aov residuals
          shapiro_res_transformed <- safe_shapiro(transformed_aov_res)

          # Perform Anderson-Darling normality test on transformed aov residuals
          # ad.test() requires n >= 8; skip gracefully for very small samples
          if (length(transformed_aov_res) >= 8) {
            adt_res_transformed <- nortest::ad.test(transformed_aov_res)
          } else {
            adt_res_transformed <- list(statistic = NA_real_, p.value = NA_real_,
                                        method = "Anderson-Darling test (skipped: n < 8)")
          }

          # Perform Levene's test on transformed aov residuals
          levene_res_transformed <- rstatix::levene_test(
            as.formula(
              paste0("transformed_aov_res~interaction(",
                     paste(predictor_names, collapse = ", "),
                     ")")), data = data_complete) #residuals(aov_test)~predictor_names

          is_levene_trans_sig <- !is.na(levene_res_transformed$p) & levene_res_transformed$p < alpha

          cat("
   \n## Normality and homoscedasticity of **TRANSFORMED residuals** of : ",
              response_name,
              "   \n  \n"
          )

          # Generate text for interpretation of Levene's output
          if (is.na(levene_res_transformed$p)) {
            levene_res_intp_text <- "Levene's Test could not be calculated on transformed residuals (likely due to saturation or insufficient replication per group).  \n"
          } else if (levene_res_transformed$p > alpha) {
            levene_res_intp_text <- paste("According to 'Levene's Test' (",
                                          round(levene_res_transformed$p, digits = 4),
                                          " > ",
                                          alpha,
                                          ") transformed residuals **have equal variance** (homoscedasticity)."
            )
          } else if (is_levene_trans_sig) {
            levene_res_intp_text <- paste("According to 'Levene's Test' (",
                                          round(levene_res_transformed$p, digits = 4),
                                          "\u2264",
                                          alpha,
                                          ") transformed residuals do **NOT** have equal variance (Heteroskedasticity)."
            )
          }
          # levene_output_transformed
          cat(
            "**Levene's test** for homogeneity of transformed residuals: F-Statistic =",
            round(levene_res_transformed$statistic, digits = 4),
            "p-value =",
            round(levene_res_transformed$p, digits = 4),
            ". ", levene_res_intp_text
          )
          if (!is.null(levene_size_warning)) cat(levene_size_warning)


          # Generate text for interpretation of shapiro output shapiro_res_intp_text
          if (is.na(shapiro_res_transformed$p.value)) {
            shapiro_res_intp_text <- paste0(
              "Shapiro-Wilk was skipped (", shapiro_res_transformed$method,
              "). Normality decision is based on the Anderson-Darling test below.  \n  \n&nbsp;  \n"
            )
          } else if (shapiro_res_transformed$p.value > alpha) {
            shapiro_res_intp_text <- paste(
              "According to the 'Shapiro-Wilk' test (",
              round(shapiro_res_transformed$p.value, digits = 4),
              " > ",
              alpha,
              ") transformed residuals **ARE normally distributed**.  \n  \n&nbsp;  \n"
            )
          } else if (shapiro_res_transformed$p.value <= alpha) {
            shapiro_res_intp_text <- paste(
              "According to the 'Shapiro-Wilk' test (",
              round(shapiro_res_transformed$p.value, digits = 4),
              "\u2264",
              alpha,
              ") transformed residuals are **NOT** normally distributed.  \n  \n&nbsp;  \n"
            )
          }


          # shapiro_output_transformed
          cat(
            "**Shapiro-Wilk Test** for Normality of transformed residuals: W =",
            round(shapiro_res_transformed$statistic, digits = 4),
            "p-value =",
            round(shapiro_res_transformed$p.value, digits = 4),
            ". ", shapiro_res_intp_text
          )

          # Generate text for interpretation of Anderson-Darling test output
          if (is.na(adt_res_transformed$p.value)) {
            adt_res_intp_text <- paste0(
              "      Anderson-Darling test skipped (n < 8, minimum sample size not met).  \n  \n"
            )
          } else if (as.numeric(adt_res_transformed$p.value) > alpha) {
            adt_res_intp_text <- paste(
              "According to 'Anderson-Darling test'  (",
              round(adt_res_transformed$p.value, digits = 4),
              " > ",
              alpha,
              ") transformed residuals **ARE normally distributed**.  \n  \n"
            )
          } else if (as.numeric(adt_res_transformed$p.value) <= alpha) {
            adt_res_intp_text <- paste(
              "According to 'Anderson-Darling test' (",
              round(adt_res_transformed$p.value, digits = 4),
              "\u2264",
              alpha,
              ") transformed residuals are **NOT** normally distributed.  \n  \n"
            )
          }

          # adt_output_transformed
          cat(
            adt_res_transformed$method,
            ": A =",
            round(adt_res_transformed$statistic, digits = 4),
            " p =",
            round(adt_res_transformed$p.value, digits = 4),
            ". ", adt_res_intp_text
          )

          # Store output in output_list
          output_list[[response_name]][["transformed_aov_test"]]     <- aov_test_transformed
          output_list[[response_name]][["transformed_shapiro_test"]] <- shapiro_res_transformed
          output_list[[response_name]][["transformed_adt_test"]]     <- adt_res_transformed
          output_list[[response_name]][["transformed_levene_test"]]  <- levene_res_transformed

          temp_file <- tempfile(fileext = ".png")
          png(temp_file, width = 7.8, height = 7.8, units = "in", res = 300)
          # Set up a 2x2 plotting layout with adjusted margins mar = c(bottom, left, top, right)
          par(mfrow = c(2, 2), mar = c(4, 4, 4, 1))  # Adjust left margin

          # Residual plot aov
          plot(aov_test_transformed, 1, main = "Residuals transformed data")

          # Boxplot
          boxplot(residuals(aov_test_transformed), main = "Boxplot", xlab = "Residuals transformed data")

          # Histogram
          f_hist(residuals(aov_test_transformed),  xlab = "Residuals transformed data")

          # QQPlot
          f_qqnorm(residuals(aov_test_transformed))

          # Close the png file
          dev.off()

          output_list[[response_name]][["transformed_normality_plots"]] <- temp_file

          if(norm_plots == TRUE){
            cat("  \nCheck the **residual** plots of the **transformed data** in the figure below to assess normality.  \n")
            # Include the saved plots in R Markdown
            cat(paste0("![](", temp_file, ")"), "   \n  \n")
          }



          if(output_type != "rmd"){
            # Pagebreak
            cat("
<div style=\"page-break-after: always;\"></div>
\\newpage
        ")
          }
        }
      }

      if(!is.null(aov_summary_transformed)){
        #Overwrite earlier object with transformed output
        aov_test_out <- aov_test_transformed
        aov_summary  <- aov_summary_transformed
        Response_Transformed <- TRUE
        output_list[[response_name]][["Response_Transformed"]] <- Response_Transformed
      } else {
        aov_test_out <- aov_test
        Response_Transformed <- FALSE
        output_list[[response_name]][["Response_Transformed"]] <- Response_Transformed
      }

      # Store a back-transform closure so plot.f_aov can convert emmip output
      # (which is always on the transformed scale) back to the original scale.
      # If no transformation was applied the closure is the identity function.
      if (Response_Transformed) {
        if (transformation == "boxcox" || isTRUE(transformation)) {
          lambda_val <- transformed_var$lambda
          output_list[[response_name]][["back_transform_fn"]] <- local({
            lam <- lambda_val
            function(x) {
              if (abs(lam) < 1e-6) exp(x)
              else (lam * x + 1)^(1 / lam)
            }
          })
        } else if (transformation == "bestnormalize") {
          bn_obj <- transformed_var$bestNormalize
          output_list[[response_name]][["back_transform_fn"]] <- local({
            bn <- bn_obj
            function(x) predict(bn, newdata = x, inverse = TRUE)
          })
        }
      } else {
        output_list[[response_name]][["back_transform_fn"]] <- identity
      }


      # Create data summary table for output and store in output_list
      data_summary_table <- f_summary(data,
                                      response_name,
                                      predictor_names,
                                      show_name = FALSE,
                                      digits = NULL
      )$output_df


      # Extract p-values for the model terms (drop the NA residual row)
      p_values     <- aov_summary[[1]][["Pr(>F)"]]
      term_names   <- rownames(aov_summary[[1]])
      # Remove the residuals row (NA p-value) for the significance check
      non_resid    <- !is.na(p_values)
      p_values_sig <- p_values[non_resid]
      term_names_sig <- term_names[non_resid]
      # Overall significance: any term significant (used to suppress "ns" letters)
      overall_p_value <- min(p_values_sig, na.rm = TRUE)

      # Classify model terms into interactions and main effects
      interaction_terms <- term_names_sig[grepl(":", term_names_sig)]
      main_effect_terms <- term_names_sig[!grepl(":", term_names_sig)]

      sig_interactions  <- if (length(interaction_terms) > 0)
        interaction_terms[p_values_sig[match(interaction_terms, term_names_sig)] < alpha]
      else character(0)

      sig_main_effects  <- main_effect_terms[
        p_values_sig[match(main_effect_terms, term_names_sig)] < alpha]

      ns_main_effects   <- main_effect_terms[
        p_values_sig[match(main_effect_terms, term_names_sig)] >= alpha]

      # Calculate the confidence level that matches alpha
      conf_level <- 1 - alpha
      conf_perc  <- conf_level * 100

      # ----------------------------------------------------------------
      # Determine emmeans specs and display labelling.
      # emmeans ALWAYS uses predictor_names (the full set of predictors
      # from the model formula). This guarantees the reference grid
      # matches the fitted model regardless of significance.
      #
      # emm_is_cells  : TRUE  -> significant interaction present;
      #                         emmeans returns cell means (all combinations).
      #                 FALSE -> no significant interaction;
      #                         emmeans returns marginal means.
      # emm_type_label: used only in headings and description text.
      # ----------------------------------------------------------------
      # Initialize a deferred-display note for the post-hoc table.
      # Both branches below set this; it is cat()-ed later, directly
      # above the post-hoc header, so the NOTE sits next to the table
      # it describes rather than being visually attached to the
      # Observed Descriptives Table that comes earlier in the report.
      post_hoc_intro_note <- ""

      if (length(sig_interactions) > 0) {
        emm_is_cells   <- TRUE
        emm_type_label <- "Cell Means"

        sig_int_preds  <- unique(unlist(strsplit(sig_interactions, ":")))

        post_hoc_intro_note <- paste0(
          "\n   \n**NOTE: Significant interaction(s) detected: ",
          paste(sig_interactions, collapse = ", "), "**  \n",
          "The post hoc table below shows **cell means**, i.e. the estimated mean for ",
          "every combination of ", paste(sig_int_preds, collapse = " \u00d7 "), ".  \n",
          "Letters compare all cells simultaneously. ",
          "Interpretation should focus on the full interaction pattern, ",
          "not on individual factor effects in isolation.  \n\n"
        )

      } else {
        emm_is_cells   <- FALSE
        emm_type_label <- "Marginal Means"

        # Report non-significant main effects so the reader knows they
        # were tested. They are still included in the emmeans call
        # (correct: the model controls for them), but the reader is
        # informed they were not significant.
        #
        # The NOTE is *stored* here rather than printed immediately,
        # because it describes letter groups in the emmeans post-hoc
        # table -- which appears much later in the report, after the
        # Observed Descriptives Table. Printing it here would place
        # the warning directly above the descriptive table and create
        # the false impression that the letters being warned about
        # are in the descriptive table. See the cat(post_hoc_intro_note)
        # call below, just before the post-hoc header.
        if (length(ns_main_effects) > 0) {
          post_hoc_intro_note <- paste0(
            "\n   \n**NOTE:** The following term(s) were **not significant** ",
            "(p \u2265 ", alpha, "): ",
            paste(ns_main_effects, collapse = ", "), ".  \n",
            "Their marginal means are shown in the emmeans table below for ",
            "completeness (the model controls for them), but their letter ",
            "groups are not meaningful given the non-significant ANOVA ",
            "result.  \n\n"
          )
        }
      }

      # Estimated Marginal Means -- always uses all predictor_names so that
      # the specs exactly match the model's reference grid.
      emm <- emmeans::emmeans(aov_test_out,
                              specs = predictor_names,
                              level = conf_level)

      # Store objects needed for plot.f_aov interaction/means plot
      output_list[[response_name]][["emm"]]               <- emm
      output_list[[response_name]][["predictor_names"]]   <- predictor_names
      output_list[[response_name]][["data_complete"]]     <- data_complete
      output_list[[response_name]][["original_response"]] <- original_response
      output_list[[response_name]][["emm_is_cells"]]      <- emm_is_cells

      # Perform post hoc test with adjustment
      mult_cld <- multcomp::cld(emm,
                                alpha = alpha,
                                Letters = letters,
                                adjust = adjust)

      # Convert the result to a data frame
      summary_table <- as.data.frame(mult_cld)

      # ============================================================
      # START: Emmeans table Back-transformation
      # ============================================================
      if (Response_Transformed == TRUE) {
        # Columns to back-transform
        bt_cols <- intersect(c("emmean", "lower.CL", "upper.CL"), names(summary_table))


        # Handle Box-Cox Back-transformation
        if (transformation == "boxcox" || transformation == TRUE) {
          # f_boxcox function returns the lambda value
          lambda <- transformed_var$lambda

          if (!is.null(lambda)) {
            # Define Inverse BoxCox function
            inv_boxcox <- function(x, lambda) {
              if (abs(lambda) < 1e-6)
                exp(x)
              else
                (lambda * x + 1)^(1 / lambda)
            }

            summary_table[bt_cols] <- lapply(summary_table[bt_cols], function(x) {
              inv_boxcox(x, lambda)
            })
          }
        }


        # Handle BestNormalize Back-transformation
        else if (transformation == "bestnormalize") {
          # f_bestNormalize function returns the BN object
          bn_object <- transformed_var$bestNormalize

          if (!is.null(bn_object)) {
            # Use the predict method with inverse = TRUE
            summary_table[bt_cols] <- lapply(summary_table[bt_cols], function(x) {
              predict(bn_object, newdata = x, inverse = TRUE)
            })

            # Flag column to indicate these are back-transformed
            summary_table$scale <- "response (back-transformed)"
          }
        }
      }
      output_list[[response_name]][["summary_table"]] <- summary_table
      # ============================================================
      # END: Emmeans table Back-transformation
      # ============================================================

      # Remove row names
      rownames(summary_table) <- NULL

      # Rename columns for clarity
      names(summary_table)[names(summary_table) == ".group"] <- "Letter"

      if (overall_p_value >= alpha) {
        summary_table$Letter <- "ns"
      }

      # Identify the grouping key (e.g., "treatment")
      common_cols <- intersect(names(summary_table), names(data_summary_table))

      # exclude 'df' here by simply not listing it
      emm_cols <- c(common_cols, "emmean", "SE", "lower.CL", "upper.CL", "Letter")

      # Subset the Emmeans table to just those columns
      #    (This is your 'x' - the left side of the join)
      emm_subset <- summary_table[, emm_cols]

      # Getting n from data_summary_table
      n_df <- as.data.frame(data_summary_table[common_cols])
      n_df$n <- data_summary_table$n

      # Perform the Merge
      #    sort = FALSE -> CRITICAL! Prevents R from re-shuffling rows alphabetically
      post_hoc_summary_table <- merge(x = emm_subset,
                                      y = n_df,
                                      by = common_cols,
                                      all.x = TRUE,
                                      sort = FALSE)

      # Rename emmean column: back-transformed values estimate the MEDIAN on the
      # original scale (back-transforming a mean gives the median for any monotone
      # transformation). Label clearly to avoid misreporting as an arithmetic mean.
      # Also drop SE when back-transformed: the SE was computed on the transformed
      # scale and is invalid on the original scale (asymmetric error). The footnote
      # directs the reader to the CIs instead.
      if (Response_Transformed == TRUE) {
        names(post_hoc_summary_table)[names(post_hoc_summary_table) == "emmean"] <- "median (BT)"
        post_hoc_summary_table$SE <- NULL
      } else {
        # Prevent breaking of emmean col title in wide tables
        names(post_hoc_summary_table)[names(post_hoc_summary_table) == "emmean"] <- "emmean.."
      }



      # Store output in output_list (after rename and SE removal so print.f_aov
      # receives the same clean table as the document output)
      output_list[[response_name]][["post_hoc_summary_table"]] <- post_hoc_summary_table

      # Prepare table footnote CLD text string
      if (exists("mult_cld")) {

        raw_msg <- attr(mult_cld, "mesg")

        # Identify the indices of the old disclaimer (lines starting with NOTE or indentation)
        bad_lines <- grep("NOTE:|share the same|show them to be", raw_msg)

        # Remove the old lines
        if (length(bad_lines) > 0) {
          raw_msg <- raw_msg[-bad_lines]
        }

        # Add your new note
        raw_msg <- c(raw_msg, paste0(
          "\n**Note:** Groups in the \"Letters\" column sharing the same letter are ",
          "**not** significantly different (\u03B1 = ", alpha, "). Groups with ",
          "different letters are significantly different. Sharing a letter ",
          "indicates insufficient evidence to claim a difference; it does not ",
          "prove the groups are identical.
        \n")
        )

        # Collapse
        cld_text <- paste(raw_msg, collapse = "  \n")

        # Substitute 'alpha' with '$\alpha$'
        # We use "\\alpha" because in R strings, you need \\ to produce a single \
        cld_text <- gsub("alpha", "\u03B1", cld_text)

        # Extract min and max once to keep code clean and efficient
        min_df <- min(summary_table$df, na.rm = TRUE)
        max_df <- max(summary_table$df, na.rm = TRUE)

        # add the degrees of freedom
        cld_text <- paste0("Degrees of freedom: ",
                           if (min_df != max_df) {
                             paste(min_df, "-", max_df)
                           } else {
                             max_df
                           }, "  \n", cld_text
        )
      } else {
        cld_text <- ""
      }

      cat(paste("\n## Observed Descriptives Table of: ", deparse(current_formula), "  \n"))
      f_pander(f_conditional_round(data_summary_table, digits = 3, replace_na = FALSE))
      # Add footnote
      cat(
        "**TIP:** These values represent your actual observed sample characteristics.
Use this table for *Methods* sections or Supplementary materials (to describe the sample).\n  \n
**CAUTION:** For statistical inference (significance letters and *p-values*) and reporting
main findings in the *Results* section, you **must** use the Emmeans table below.\n"
      )

      # Show WARNING when assumptions of aov() are violated
      if (Response_Transformed == TRUE) {

        # Build diagnosis of what is still wrong after transformation.
        # For the normality check, prefer Shapiro-Wilk where available and
        # fall back to Anderson-Darling when Shapiro was skipped (n > 5000),
        # so the diagnosis does not silently pass for very large samples.
        normality_still_bad <- if (!is.na(shapiro_res_transformed$p.value)) {
          shapiro_res_transformed$p.value <= alpha
        } else if (!is.na(adt_res_transformed$p.value)) {
          adt_res_transformed$p.value <= alpha
        } else {
          FALSE
        }

        still_wrong <- c(
          if (is_levene_trans_sig) "heteroscedasticity (Levene's p < \u03b1)" else NULL,
          if (isTRUE(normality_still_bad)) {
            if (!is.na(shapiro_res_transformed$p.value))
              "non-normality (Shapiro-Wilk p \u2264 \u03b1)"
            else
              "non-normality (Anderson-Darling p \u2264 \u03b1)"
          } else NULL
        )

        if (length(still_wrong) > 0) {
          still_wrong_str <- paste(still_wrong, collapse = " and ")

          cat(paste0(
            "\n\n***\n\n",
            "**WARNING: \nTransformation did not resolve ", still_wrong_str, "**  \n\n",
            "A ", if(transformation == TRUE) "boxcox", " transformation was applied but **",
            still_wrong_str, "** persists in the transformed residuals for `",
            response_name, "`.  \n",
            "The ANOVA results below are **unreliable**. Heteroscedasticity inflates the ",
            "Type I error rate, meaning significant results may be false positives and ",
            "F-statistics are biased. Consider one of these **alternative tests** before ",
            "drawing conclusions:  \n",
            "\n-  **Kruskal-Wallis test:** non-parametric, no normality or equal-variance ",
            "assumption. Tests stochastic dominance between groups. Follow up with Dunn's test ",
            "(`rfriend::f_kruskal_test()`).",
            "\n-  **Generalised Linear Model (GLM):** if the data follow a specific non-normal ",
            "distribution (e.g. counts \u2192 Poisson, positive skewed \u2192 Gamma) modelling ",
            "the correct error distribution via `rfriend::f_glm()` is more principled than ",
            "transforming.",
            "\n-  **Aligned Rank Transform (ART) ANOVA:** extends rank-based testing to ",
            "factorial designs including interactions. Available via `ARTool::art()`. ",
            "Recommended for multi-factor designs. \n",
            "\n***\n\n"
          ))

          # Store flag for print.f_aov
          output_list[[response_name]][["last_resort_triggered"]] <- TRUE
          output_list[[response_name]][["last_resort_reason"]]    <- still_wrong_str
          output_list[[response_name]][["cld_text"]]              <- cld_text
        }

      } else {
        # Untransformed: simpler warnings pointing to transformation
        if (is_levene_sig) {
          cat(paste0(
            "\n   \n**WARNING**  \n",
            "*Levene's Test (", round(levene_res$p, digits = 4), " < ", alpha, "): ",
            "residuals do NOT have equal variance (heteroscedasticity).  \n",
            "ANOVA results can be misleading. Enable the transformation option ",
            "(`transformation = TRUE`) or resort to other statistical tests.*"
          ))
        }
        if (isTRUE(shapiro_res$p.value <= alpha)) {
          cat(paste0(
            "\n   \n**WARNING**  \n",
            "*Shapiro-Wilk Test (", round(shapiro_res$p.value, digits = 4),
            " \u2264 ", alpha, "): residuals are NOT normally distributed.  \n",
            "ANOVA results can be misleading. Enable the transformation option ",
            "(`transformation = TRUE`) or resort to other statistical tests.*"
          ))
        }
      }

      if(n_before != n_after) {
        cat("\n   \n**WARNING**   \n *Removed ", n_before - n_after, " rows (",
            round((n_before - n_after)/n_before * 100, 1),
            "%) with missing values in ", response_name,".*", sep = "")
      }


      if(Response_Transformed == TRUE){
        if(transformation == "bestnormalize"){
          cat("
   \n## ANOVA Summary of '",transformed_var$transformation_name,"TRANSFORMED' response variable:", response_name, "  \n")
        }

        if(transformation == "boxcox" ||
           transformation == TRUE){
          cat("
   \n## ANOVA Summary of 'Box-Cox TRANSFORMED' response variable:", response_name, "  \n")
        }
      } else {
        cat("
   \n## ANOVA Summary of ", response_name, "  \n")
      }

      cat("&nbsp;\n  \n")
      cat(paste("\n**Table** of aov call: ", deparse(current_formula), "  \n"))
      cat(rmd_anova_summary(aov_test_out))
      cat("&nbsp;\n  \n")

      # Repeat banner when force_aov overrode the n=1 skip
      if (isTRUE(output_list[[response_name]][["force_aov_used"]])) {
        cat(paste0(
          "\n\n***\n\n",
          "**[!] Diagnostic output only, do not report these results.**  \n",
          "The model above is saturated (at least one cell has $n = 1$). ",
          "F-statistics and p-values are undefined.  \n",
          "\n\n***\n\n"
        ))
      }


      if (!is.null(post_hoc_summary_table)) {
        # Emit the deferred post-hoc intro note right here, so it
        # sits directly above the post-hoc table it describes rather
        # than being visually attached to the Observed Descriptives
        # Table that comes earlier in the report. An empty string is
        # a no-op for cat(), so we do not need a length check.
        cat(post_hoc_intro_note)

        cat(paste0("  \n## Post Hoc Test on Estimated ", emm_type_label, " of ",
                   response_name, "  \n"))

        if (emm_is_cells) {
          cat(paste0("
        \nEstimated Cell Means (emmeans) are the model-based mean for each unique
          combination of all factors in the model for response variable ", response_name, ".
          Unlike raw data averages (see: Observed Descriptives Table), these model-based
          cell means correct for unbalanced designs.Because a significant interaction was
          detected, pairwise comparisons are performed across **all** factor-level combinations
          simultaneously to pinpoint specific differences.
          The $n$ column corresponds to the raw observed data.
          If $n$ is blank, there is no observed data and emmeans tried to estimate the missing
          point.
        \n  \n&nbsp;\n  \n
        \n*Reporting Tips:* When an interaction is significant, report and plot the full
          cell means pattern. To best show the shape of the interaction, it is strongly
          recommended to use a figure with individual data points overlaid on the cell means
          (with ", conf_perc, "% CIs).
        \n   \n&nbsp;   \n  \n"))
        } else {
          cat(paste0("
        \nEstimated Marginal Means (emmeans) are model-based mean values of ",
                     response_name,
                     " for each level of the significant predictor(s), averaged over all other
          factors in the model. Unlike raw data averages (see: Observed Descriptives Table),
          emmeans correct for unbalanced designs and reflect the statistical
          model used for pairwise comparisons (significance testing **letters**). ",
                     if(Response_Transformed == FALSE){"SE values are identical for groups with equal
          sample sizes and differ only to reflect variation in group size ($n$). "}else{""},
                     "The $n$ column corresponds to the raw observed data of ",
                     response_name, ". If $n$ is blank, there is no observed data and
          emmeans estimated the missing data point.
        \n  \n&nbsp;\n  \n
        \n*Reporting Tips:* For main results showing significant differences, prioritize
          the Emmeans table (preferably with ", conf_perc, "% CIs).
          Figures should include all individual raw data points to show the
          Model Fit (Emmeans) relative to the Observed Spread (Raw Data).
        \n   \n&nbsp;   \n  \n"))
        } # end if/else emm_is_cells

        if(Response_Transformed == TRUE){
          cat(paste0("\n**Back-transformed Post Hoc Table** of aov call: ",
                     deparse(current_formula), "\n"))
        } else {
          cat(paste0("\n**Post Hoc ", emm_type_label, " Table** of aov call: ",
                     deparse(current_formula), "  \n"))
        }

        f_pander(f_conditional_round(post_hoc_summary_table, digits = 3, replace_na = FALSE))
        # Add footnote
        cat(cld_text)
        # Add footnote specifically explaining back-transformed values and median interpretation
        if(Response_Transformed == TRUE){
          cat(paste0("
\n**Note on back-transformation:** The column **'median (BT)'** contains back-transformed
estimated marginal means. Back-transforming a mean from a transformed scale returns the
**median** of the original-scale distribution, not the arithmetic mean. This follows from
the property that medians, unlike means, are equivariant under monotone transformations
($g(\\text{median}(X)) = \\text{median}(g(X))$, but $g(\\bar{X}) \\neq \\overline{g(X)}$ in general).
For reporting purposes, describe these values as **back-transformed medians** (or geometric
means in the specific case of a log transformation), not as means.
\n**Note:** Standard Errors (SE) are omitted for back-transformed data. Transformations are
non-linear, which results in asymmetric uncertainty margins on the original scale (the
distance from the median to the lower bound differs from that to the upper bound).
A single SE value implies symmetry ($\\pm$) and is therefore misleading.
Please rely on the **",conf_perc,"% Confidence Intervals (lower.CL and upper.CL)** to
quantify uncertainty.
"))
        }
      }

      # -- Estimated means / interaction plot in document output ----------
      if (isTRUE(interaction_plots)) {
        bt_fn   <- output_list[[response_name]][["back_transform_fn"]]
        if (is.null(bt_fn)) bt_fn <- identity
        emm_obj <- output_list[[response_name]][["emm"]]
        raw_y_orig_doc <- original_response

        # Helper to build one plot and save to PNG
        make_means_plot <- function(x_var, trace_var, file_suffix) {
          ip <- tryCatch(
            emmeans::emmip(emm_obj,
                           as.formula(paste(trace_var, "~", x_var)),
                           CIs = TRUE, plotit = FALSE),
            error = function(e) NULL
          )
          if (is.null(ip)) return(NULL)

          ip$yvar <- bt_fn(ip$yvar)
          ip$LCL  <- bt_fn(ip$LCL)
          ip$UCL  <- bt_fn(ip$UCL)

          traces   <- as.character(unique(ip[[trace_var]]))
          x_levels <- as.character(unique(ip[[x_var]]))
          if (is.factor(data_complete[[trace_var]]))
            traces   <- levels(data_complete[[trace_var]])
          if (is.factor(data_complete[[x_var]]))
            x_levels <- levels(data_complete[[x_var]])

          n_x  <- length(x_levels)
          n_tr <- length(traces)
          cols <- if (n_tr <= 8)
            c("steelblue","tomato","seagreen3","darkorchid",
              "darkorange","deeppink","cyan4","sienna")[seq_len(n_tr)]
          else rainbow(n_tr)

          y_label_doc <- if (isTRUE(Response_Transformed))
            paste0(response_name, " (back-transformed median)") else response_name

          raw_y_doc <- raw_y_orig_doc
          y_all <- c(ip$LCL, ip$UCL, raw_y_doc)
          y_min <- min(y_all, na.rm = TRUE)
          y_max <- max(y_all, na.rm = TRUE)
          y_pad <- (y_max - y_min) * 0.10

          tmp <- tempfile(fileext = ".png")
          png(tmp, width = 7, height = 5.5, units = "in", res = 200)
          par(mar = c(5.5, 4.5, 4, 1), mgp = c(2.8, 0.7, 0))

          plot(NULL,
               xlim = c(0.7, n_x + 0.3),
               ylim = c(y_min - y_pad, y_max + y_pad),
               xaxt = "n", xlab = x_var, ylab = y_label_doc,
               main = paste0("Estimated Means: ", response_name,
                             "  (", x_var, " \u00d7 ", trace_var, ")"))
          axis(1, at = seq_len(n_x), labels = x_levels)

          for (ti in seq_along(traces)) {
            sub_ip <- ip[as.character(ip[[trace_var]]) == traces[ti], ]
            x_idx  <- match(as.character(sub_ip[[x_var]]), x_levels)
            ord    <- order(x_idx)
            lines(x_idx[ord], sub_ip$yvar[ord], col = cols[ti], lwd = 2)
            points(x_idx[ord], sub_ip$yvar[ord], col = cols[ti], pch = 19, cex = 1.3)
            arrows(x_idx, sub_ip$LCL, x_idx, sub_ip$UCL,
                   angle = 90, code = 3, length = 0.07,
                   col = cols[ti], lwd = 1.3)
            for (xi in seq_along(x_levels)) {
              mask <- as.character(data_complete[[x_var]])     == x_levels[xi] &
                as.character(data_complete[[trace_var]]) == traces[ti]
              pts  <- raw_y_doc[mask]
              if (length(pts) > 0) {
                jit <- xi + runif(length(pts), -0.13, 0.13)
                points(jit, pts, pch = 1,
                       col = adjustcolor(cols[ti], alpha.f = 0.45), cex = 0.7)
              }
            }
          }

          legend("topright", legend = traces, col = cols, lwd = 2,
                 pch = 19, title = trace_var, bty = "n", cex = 0.85)

          int_note_doc <- if (isTRUE(emm_is_cells))
            paste0("Significant interaction, the lines representing ",
                   trace_var, " factor levels are not parallel.")
          else
            paste0("No significant interaction. The lines representing ",
                   trace_var, " factor levels should be roughly parallel.")
          mtext(int_note_doc, side = 1, line = 4.2, cex = 0.72,
                col = if (isTRUE(emm_is_cells)) "tomato" else "grey40")

          dev.off()
          tmp
        }

        if (length(predictor_names) == 1) {
          # Single predictor: simple dot+CI plot
          ip_s <- tryCatch(
            emmeans::emmip(emm_obj,
                           as.formula(paste("~", predictor_names[1])),
                           CIs = TRUE, plotit = FALSE),
            error = function(e) NULL
          )
          if (!is.null(ip_s)) {
            ip_s$yvar <- bt_fn(ip_s$yvar)
            ip_s$LCL  <- bt_fn(ip_s$LCL)
            ip_s$UCL  <- bt_fn(ip_s$UCL)
            pred      <- predictor_names[1]
            grp_vals  <- as.character(ip_s[[pred]])
            n_grp     <- length(grp_vals)
            x_pos     <- seq_len(n_grp)
            raw_y_doc <- raw_y_orig_doc
            y_label_doc <- if (isTRUE(Response_Transformed))
              paste0(response_name, " (back-transformed median)") else response_name
            y_min <- min(c(raw_y_doc, ip_s$LCL), na.rm = TRUE)
            y_max <- max(c(raw_y_doc, ip_s$UCL), na.rm = TRUE)
            y_pad <- (y_max - y_min) * 0.08

            tmp_single <- tempfile(fileext = ".png")
            png(tmp_single, width = 6, height = 5, units = "in", res = 200)
            par(mar = c(4.5, 4.5, 4, 1), mgp = c(2.8, 0.7, 0))
            plot(x_pos, ip_s$yvar,
                 xlim = c(0.5, n_grp + 0.5),
                 ylim = c(y_min - y_pad, y_max + y_pad),
                 xaxt = "n", pch = 19, cex = 1.4,
                 xlab = pred, ylab = y_label_doc,
                 main = paste0("Estimated Means \u00b1 95% CI: ", response_name),
                 col = "steelblue")
            axis(1, at = x_pos, labels = grp_vals)
            arrows(x_pos, ip_s$LCL, x_pos, ip_s$UCL,
                   angle = 90, code = 3, length = 0.07,
                   col = "steelblue", lwd = 1.8)
            raw_grp <- as.character(data_complete[[pred]])
            for (g in seq_len(n_grp)) {
              pts <- raw_y_doc[raw_grp == grp_vals[g]]
              jit <- x_pos[g] + runif(length(pts), -0.15, 0.15)
              points(jit, pts, pch = 1,
                     col = adjustcolor("grey30", alpha.f = 0.5), cex = 0.7)
            }
            dev.off()
            output_list[[response_name]][["interaction_plot_1"]] <- tmp_single
            cat(paste0("\n## Estimated Means Plot of: ", response_name, "  \n"))
            cat(paste0("![](", tmp_single, ")"), "   \n  \n")
          }
        } else {
          # Multiple predictors: both orientations
          pred_pairs <- list(
            list(x = predictor_names[1], tr = predictor_names[2]),
            list(x = predictor_names[2], tr = predictor_names[1])
          )
          cat(paste0("\n## Interaction Plots of: ", response_name, "  \n"))
          for (pi in seq_along(pred_pairs)) {
            tmp_ip <- make_means_plot(pred_pairs[[pi]]$x, pred_pairs[[pi]]$tr,
                                      file_suffix = pi)
            if (!is.null(tmp_ip)) {
              key <- paste0("interaction_plot_", pi)
              output_list[[response_name]][[key]] <- tmp_ip
              cat(paste0("![](", tmp_ip, ")"), "   \n  \n")
            }
          }
        }
      }
      # --------------------------------------------------------------------

      i <- i + 1

      if(output_type != "rmd" &&  i < length(lhs)){
        # Pagebreak
        cat("
<div style=\"page-break-after: always;\"></div>
\\newpage
        ")
      }
    } #Main loop end
    if (output == TRUE) {
      return(output_list)
    }
  } # End generate report function.


  # Execute analysis ONCE: capture both the output_list and the markdown text.
  # Caching avoids running every ANOVA/transformation/emmeans calculation a second
  # time (which could give different results if bestNormalize uses random search).
  cached_markdown <- NULL
  suppressMessages(
    cached_markdown <- utils::capture.output(
      output_list <- generate_report()
    )
  )
  class(output_list) <- "f_aov"

  # Helper so word/pdf/rmd paths all use the same already-generated markdown.
  get_cached_markdown <- function() cached_markdown

  # Here the documents are constructed.
  if (output_type %in% c("word", "pdf")) {

    # Show save location before knitting else it will not display in console.
    message(paste0("Saving output in: ", output_path))

    # Create a temporary R Markdown file
    word_pdf_preamble <- function(){ paste0("
---
title: \"f_aov Analysis Report\"
date: \"`r Sys.Date()`\"
output:
   word_document:
      reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
        latex_engine: pdflatex
header-includes:
  - \\usepackage[T1]{fontenc}
  - \\usepackage[utf8]{inputenc}
  - \\usepackage{textcomp}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
  - \\DeclareUnicodeCharacter{2264}{\\ensuremath{\\leq}}
  - \\DeclareUnicodeCharacter{2265}{\\ensuremath{\\geq}}
  - \\DeclareUnicodeCharacter{2192}{\\ensuremath{\\rightarrow}}
  - \\DeclareUnicodeCharacter{00D7}{\\ensuremath{\\times}}
  - \\DeclareUnicodeCharacter{2014}{\\textemdash}
  - \\DeclareUnicodeCharacter{03B1}{\\ensuremath{\\alpha}}
  - \\DeclareUnicodeCharacter{2019}{\\textquoteright}
  - \\DeclareUnicodeCharacter{0160}{\\v{S}}
  - \\DeclareUnicodeCharacter{00E1}{\\'{a}}
  - \\usepackage{titling}
  - \\setlength{\\droptitle}{-2.5cm} % Adjust vertical spacing
---
")}

    # Prevent ## before printed output
    knitr::opts_chunk$set(comment = "")

    # Use the already-generated markdown (no second run of generate_report)
    generated_markdown <- get_cached_markdown()

    # Combine the preamble, assumptions, and the captured report into one string
    rmd_content <- paste(
      word_pdf_preamble(),
      # The captured output already contains the necessary markdown formatting and image links
      paste(generated_markdown, collapse = "\n"),
      sep = "\n"
    )

    # Write the complete Rmd content to the temp file
    writeLines(rmd_content, temp_output_file)

    # Create the RMarkdown file
    tryCatch(
      rmarkdown::render(
        temp_output_file,
        output_file = output_path,
        intermediates_dir = temp_output_dir,
        knit_root_dir = temp_output_dir,
        quiet = TRUE,
        output_format = paste0(output_type, "_document")
      ),
      error = function(e) {
        # Surface the actual LaTeX log so the user can diagnose the real error
        log_file <- file.path(temp_output_dir,
                              sub("\\.(Rmd|rmd)$", ".log",
                                  basename(temp_output_file)))
        if (file.exists(log_file)) {
          log_lines <- readLines(log_file, warn = FALSE)
          # Extract only the error lines -- look for lines starting with "!"
          error_lines <- grep("^!", log_lines, value = TRUE)
          context_idx <- grep("^!", log_lines)
          # Also grab the 2 lines after each "!" for context
          context_lines <- unique(unlist(lapply(context_idx, function(i)
            log_lines[i:min(i + 2, length(log_lines))])))
          message("\n--- LaTeX log errors ---\n",
                  paste(context_lines, collapse = "\n"),
                  "\n--- Full log: ", log_file, " ---")
        }
        stop(e)
      }
    )

    # Open files after creation
    if(open_generated_files == TRUE){
      # Open the file with default program
      f_open_file(output_path)
    }

    return(invisible(output_list))

  }
  else if (output_type == "excel") {

    # show the location were the file is saved
    message(paste0("Saving output in: ", output_path))

    # Extract all post_hoc_summary_table tables and keep their names
    post_hoc_tables <- lapply(output_list, function(obj) {
      tab <- obj$post_hoc_summary_table
      if (is.data.frame(tab)) tab else data.frame(note = "No post-hoc test performed")
    })
    names(post_hoc_tables) <- response_names

    writexl::write_xlsx(post_hoc_tables, path = output_path)

    # Open files after creation
    if(open_generated_files == TRUE){
      f_open_file(output_path)
    }

    return(invisible(output_list))

  }
  else if (output_type == "rmd"){

    if (is.null(knitr::opts_knit$get("output.dir"))) {
      knitr::opts_knit$set(output.dir = tempdir())
    }

    # Use the already-generated markdown (no second run of generate_report)
    clean_rmd_output <- paste(get_cached_markdown(), collapse = "\n")

    output_list[["rmd"]] <- clean_rmd_output

    return(invisible(output_list))

  } else if (output_type == "default"){
    #Default R behavior only show when not stored in an new object
    return(output_list)

  } else if (output_type == "console"){
    #Print output list to the console (forced)
    print(output_list)

    return(invisible(output_list))

  } else {
    warning("Invalid output format specified. No file generated.")

  }

  # Remove the temporary R Markdown file
  invisible(suppressWarnings(file.remove(temp_output_file)))

}


#' @export
print.f_aov <- function(x, ...) {

  # Loop over each category (a, b, etc.)
  for (category in names(x)) {
    # Skip non-response entries (e.g. "rmd" character string)
    sublist <- x[[category]]
    if (!is.list(sublist)) next

    # Skip responses that were skipped due to n=1 saturation
    if (isTRUE(sublist$skipped)) {
      message(paste0(
        "\n", strrep("=", 65), "\n",
        "  SKIPPED: ", category, "\n",
        "  ", sublist$skipped_reason, "\n",
        strrep("=", 65)
      ))
      next
    }

    # Warn if force_aov overrode the n=1 skip
    if (isTRUE(sublist$force_aov_used)) {
      message(paste0(
        "\n", strrep("!", 60), "\n",
        "  DIAGNOSTIC ONLY: ", category, " \u2014 saturated model (n=1 cell).\n",
        "  F-statistics and p-values are undefined. Do not report.\n",
        strrep("!", 60)
      ))
    }

    if(sublist$Response_Transformed == TRUE){
      if(sublist$transformation_option == "bestnormalize"){
        cat("\n   \n==========================================================\n")
        cat("   ANOVA of", sublist$bestNormalize$transformation_name,"TRANSFORMED response variable:", category, "\n")
        cat("==========================================================\n")



      }

      if(sublist$transformation_option == "boxcox"||
         sublist$transformation_option == TRUE){
        cat("\n   \n===========================================================\n")
        cat("   ANOVA of Box-Cox TRANSFORMED response variable:", category, "\n")
        cat("===========================================================\n")
      }

      cat(paste("\n aov call: ", sublist$aov_call, "\n"))

      cat("\nTRANSFORMED Summary Table:\n")
      print(sublist$aov_summary)


      cat("\n--- BACK TRANSFORMED Post hoc Comparisons of:", category, "---\n")
      print(sublist$post_hoc_summary_table, row.names = FALSE, quote=FALSE)
      cat("___________________________\n")
      cat(sublist$cld_text)
      cat("\n")
      message("Note: 'median (BT)' = back-transformed estimated marginal mean. ",
              "Back-transforming a mean from a transformed scale returns the MEDIAN ",
              "on the original scale, not the arithmetic mean. Report these as ",
              "back-transformed medians. CIs are valid; SE is omitted (asymmetric on original scale).")

      if (!is.null(sublist$min_cell_n) && sublist$min_cell_n <= 3) {
        warning(call. = FALSE, immediate. = TRUE, "\nMinimum cell size = ", sublist$min_cell_n,
                ". Levene's Test has very low power with such small groups; ",
                "interpret its result with caution.  \n")
      }
      if (isTRUE(sublist$transformed_levene_test$p <= sublist$alpha)) {
        warning(call. = FALSE, immediate. = TRUE, "   \nBased on the 'Levene's Test' (",
                round(sublist$transformed_levene_test$p, digits = 4), " \u2264 ", sublist$alpha, ") the transformed\n residuals do NOT have equal variance (heteroskedasticity). \nANOVA results can be misleading, resort to other statistical tests.  \n")
      }

      if (isTRUE(sublist$transformed_shapiro_test$p.value <= sublist$alpha)) {
        warning(call. = FALSE, immediate. = TRUE, "   \nBased on the Shapiro-Wilk Test (",
                round(sublist$transformed_shapiro_test$p.value, digits = 4), " \u2264 ", sublist$alpha,") the transformed\n residuals are **NOT** normally distributed. \nANOVA results can be misleading, resort to other statistical tests.  \n"
        )
      }

      if (isTRUE(sublist$last_resort_triggered)) {
        warning(call. = FALSE, immediate. = TRUE, paste0(
          "\n", strrep("=", 65), "\n",
          "  WARNING: Transformation did not resolve\n  ",
          sublist$last_resort_reason, "\n",
          strrep("=", 65), "\n",
          "  ANOVA results should be interpreted with caution.\n\n",
          "  Alternative tests to consider:\n",
          "  - GLM            : f_glm(", category, " ~ <predictors>, data = <data>, family = <dist>)\n",
          "  - Kruskal-Wallis : f_kruskal_test(", category, " ~ <predictor>, data = <data>)\n",
          "  - ART ANOVA      : ARTool::art(", category, " ~ <predictors>, data = <data>)\n",
          strrep("=", 65)
        ))
      }

    } else {

      cat("\n   \n===========================================================\n")
      cat("   ANOVA of response variable: ", category, "\n")
      cat("===========================================================\n")

      cat(paste("\n aov call: ", sublist$aov_call, "\n"))

      cat("\nSummary Table:\n")
      print(sublist$aov_summary)


      cat("\n--- Post hoc Comparisons of:", category, "---\n")
      cat("_________________________________________\n")
      print(sublist$post_hoc_summary_table, row.names = FALSE, quote=FALSE)

      if (!is.null(sublist$min_cell_n) && sublist$min_cell_n <= 3) {
        warning(call. = FALSE, immediate. = TRUE, "\nNote: Minimum cell size = ", sublist$min_cell_n,
                ". Levene's Test has very low power with such small groups; ",
                "interpret its result with caution.  \n")
      }
      if (isTRUE(sublist$Levene_test_on_res$p <= sublist$alpha)) {
        warning(call. = FALSE, immediate. = TRUE, "   \nBased on the 'Levene's Test' (",
                round(sublist$Levene_test_on_res$p, digits = 4), " \u2264 ", sublist$alpha, ")\nthe residuals do NOT have equal variance (heteroskedasticity).\nANOVA results can be misleading. \nENABLE the transformation option or resort to other statistical tests.  \n"
        )
      }

      if (isTRUE(sublist$shapiro_test_residuals$p.value <= sublist$alpha)) {
        warning(call. = FALSE, immediate. = TRUE, "  \nBased on the Shapiro-Wilk Test (",
                round(sublist$shapiro_test_residuals$p.value, digits = 4), " \u2264 ", sublist$alpha,")\n the residuals are **NOT** normally distributed. \nANOVA results can be misleading. \nENABLE the transformation option or resort to other statistical tests.  \n"
        )
      }
    }
  }# End of loop

  #Some space between output and next input line in console
  cat("\n   \n")
} # End of function



#' @export
plot.f_aov <- function(x, ...) {

  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state


  # Loop over each category (a, b, etc.)
  for (category in names(x)) {
    # Skip non-response entries (e.g. "rmd" character string)
    sublist <- x[[category]]
    if (!is.list(sublist)) next

    # Skip responses that were skipped due to n=1 saturation
    if (isTRUE(sublist$skipped)) next

    if(sublist$Response_Transformed == FALSE){
      par(mfrow = c(1, 2),
          mar = c(3, 2.8, 4, 0.6),
          oma = c(0, 0, 0, 0),
          mgp = c(1.7, .5, 0)
      )
      # Boxplot
      boxplot(residuals(sublist$aov_test),
              main = paste("Variable:", category),
              xlab = "Residuals",
              cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)
      # Histogram
      f_hist(residuals(sublist$aov_test),  xlab = "Residuals",
             cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)

      # Residual plot aov
      plot(sublist$aov_test, 1,
           main = paste("Variable:", category),
           cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)

      # QQPlot
      f_qqnorm(residuals(sublist$aov_test),
               cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)

      par(mfrow = c(1,1))
      layout(1)  # Clear layout matrix

    } else if(sublist$Response_Transformed == TRUE){
      # Set up a 2x2 plotting layout with adjusted margins mar = c(bottom, left, top, right)
      par(mfrow = c(1, 2),
          mar = c(3, 2.8, 4, 0.6), # Default is mar = c(5.1, 4.1, 4.1, 2.1)
          oma = c(0, 0, 2, 0),
          mgp = c(1.7, .5, 0)  # Default is par(mgp = c(3, 1, 0))
      )

      # Histogram
      f_hist(residuals(sublist$aov_test),
             main = paste("Variable:", category),
             xlab = "Residuals",
             cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)
      f_hist(residuals(sublist$transformed_aov_test),
             main = paste("Transformed variable:", category),
             xlab = "Residuals transformed data",
             cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)
      mtext("Histograms", outer = TRUE, cex = 1.3)

      # Residual plot aov
      plot(sublist$aov_test, 1,  main = paste("Variable:", category),
           cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)
      plot(sublist$transformed_aov_test, 1,
           main = paste("Transformed variable:", category),
           cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)
      title("Residual plots", outer = TRUE, cex = 1.3, line = -1)


      # QQPlot
      f_qqnorm(residuals(sublist$aov_test),
               main = paste("Variable:", category),
               cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)
      f_qqnorm(residuals(sublist$transformed_aov_test),
               main = paste("Transformed variable:", category),
               cex.main = 0.8, cex.lab = 0.8, cex.axis = 0.8)
      mtext("Normal Q-Q Plot with 95% Confidence Bands", outer = TRUE, cex = 1.3)

      par(mfrow = c(1,1))
      layout(1)  # Clear layout matrix
    }

    # ------------------------------------------------------------------
    # Means / Interaction plot
    # Uses the stored emmeans object + raw data for overlay.
    # - Single predictor  : dot plot with 95% CIs + jittered raw points
    # - Multiple predictors: emmip() interaction trace plot + raw points
    # ------------------------------------------------------------------
    emm             <- sublist[["emm"]]
    pred_names      <- sublist[["predictor_names"]]
    data_complete   <- sublist[["data_complete"]]
    raw_y_orig      <- sublist[["original_response"]]  # always original scale
    emm_is_cells    <- sublist[["emm_is_cells"]]
    is_transformed  <- isTRUE(sublist[["Response_Transformed"]])

    if (!is.null(emm) && !is.null(pred_names) && !is.null(data_complete)) {

      par(mar = c(4.5, 4.5, 4, 1),
          mgp = c(2.8, 0.7, 0),
          oma = c(0, 0, 0, 0))

      emm_df <- as.data.frame(emm)

      y_label <- if (is_transformed)
        paste0(category, " (back-transformed median)") else category

      if (length(pred_names) == 1) {
        # ---- Single predictor: dot + CI plot with raw data overlay ----
        pred      <- pred_names[1]
        grp_vals  <- emm_df[[pred]]
        n_grp     <- length(grp_vals)
        x_pos     <- seq_len(n_grp)

        # If back-transformed, use post_hoc_summary_table which has BT values
        if (is_transformed) {
          ph <- sublist[["post_hoc_summary_table"]]
          # align by predictor column
          emm_df <- merge(emm_df[, pred, drop = FALSE],
                          ph[, c(pred, "lower.CL", "upper.CL")],
                          by = pred, all.x = TRUE, sort = FALSE)
          centre_col <- intersect(c("median (BT)", "emmean"), names(ph))[1]
          emm_df$centre <- ph[[centre_col]][
            match(emm_df[[pred]], ph[[pred]])]
        } else {
          emm_df$centre <- emm_df$emmean
        }

        # y axis range: span raw data + CI
        raw_y  <- if (!is.null(raw_y_orig)) raw_y_orig else data_complete[[category]]
        y_min  <- min(c(raw_y, emm_df$lower.CL), na.rm = TRUE)
        y_max  <- max(c(raw_y, emm_df$upper.CL), na.rm = TRUE)
        y_pad  <- (y_max - y_min) * 0.08
        grp_labels <- as.character(grp_vals)

        plot(x_pos, emm_df$centre,
             xlim = c(0.5, n_grp + 0.5),
             ylim = c(y_min - y_pad, y_max + y_pad),
             xaxt = "n", pch = 19, cex = 1.4,
             xlab = pred, ylab = y_label,
             main = paste0("Estimated Means \u00b1 95% CI: ", category),
             col  = "steelblue")
        axis(1, at = x_pos, labels = grp_labels)

        # 95% CI bars
        arrows(x_pos, emm_df$lower.CL, x_pos, emm_df$upper.CL,
               angle = 90, code = 3, length = 0.07,
               col = "steelblue", lwd = 1.8)

        # Raw data jittered
        raw_grp <- data_complete[[pred]]
        for (g in seq_len(n_grp)) {
          pts <- raw_y[raw_grp == grp_vals[g]]
          jit <- x_pos[g] + runif(length(pts), -0.15, 0.15)
          points(jit, pts, pch = 1, col = adjustcolor("grey30", alpha.f = 0.5),
                 cex = 0.7)
        }

        bt_note <- if (is_transformed)
          "\nPoints = raw data. Means are back-transformed medians." else
            "\nPoints = raw data. Means are model-estimated marginal means."
        mtext(bt_note, side = 1, line = 3.5, cex = 0.7, col = "grey40")

      } else {
        # ---- Multiple predictors: one interaction plot per predictor pair ----
        # Loop: for each pair (x_var, trace_var), produce one plot.
        # For 2 predictors this gives 2 plots (pred1 on x / pred2 on x).
        bt_fn <- sublist[["back_transform_fn"]]
        if (is.null(bt_fn)) bt_fn <- identity

        pred_pairs <- list(
          list(x = pred_names[1], tr = pred_names[2]),
          list(x = pred_names[2], tr = pred_names[1])
        )

        for (pair in pred_pairs) {
          x_var     <- pair$x
          trace_var <- pair$tr

          ip <- tryCatch(
            emmeans::emmip(emm,
                           as.formula(paste(trace_var, "~", x_var)),
                           CIs = TRUE, plotit = FALSE),
            error = function(e) NULL
          )
          if (is.null(ip)) next

          # Back-transform emmip values to original scale
          ip$yvar <- bt_fn(ip$yvar)
          ip$LCL  <- bt_fn(ip$LCL)
          ip$UCL  <- bt_fn(ip$UCL)

          traces   <- as.character(unique(ip[[trace_var]]))
          x_levels <- as.character(unique(ip[[x_var]]))
          # Preserve factor ordering if available
          if (is.factor(data_complete[[trace_var]]))
            traces   <- levels(data_complete[[trace_var]])
          if (is.factor(data_complete[[x_var]]))
            x_levels <- levels(data_complete[[x_var]])

          n_x  <- length(x_levels)
          n_tr <- length(traces)
          cols <- if (n_tr <= 8)
            c("steelblue","tomato","seagreen3","darkorchid",
              "darkorange","deeppink","cyan4","sienna")[seq_len(n_tr)]
          else rainbow(n_tr)

          raw_y  <- if (!is.null(raw_y_orig)) raw_y_orig else data_complete[[category]]
          y_all  <- c(ip$LCL, ip$UCL, raw_y)
          y_min  <- min(y_all, na.rm = TRUE)
          y_max  <- max(y_all, na.rm = TRUE)
          y_pad  <- (y_max - y_min) * 0.10

          par(mar = c(5.5, 4.5, 4, 1),
              mgp = c(2.8, 0.7, 0),
              oma = c(0, 0, 0, 0))

          plot(NULL,
               xlim = c(0.7, n_x + 0.3),
               ylim = c(y_min - y_pad, y_max + y_pad),
               xaxt = "n",
               xlab = x_var, ylab = y_label,
               main = paste0("Interaction Plot: ", category,
                             "  (", x_var, " \u00d7 ", trace_var, ")"))
          axis(1, at = seq_len(n_x), labels = x_levels)

          for (ti in seq_along(traces)) {
            sub_ip <- ip[as.character(ip[[trace_var]]) == traces[ti], ]
            x_idx  <- match(as.character(sub_ip[[x_var]]), x_levels)
            ord    <- order(x_idx)

            # Trace line + mean points
            lines(x_idx[ord], sub_ip$yvar[ord], col = cols[ti], lwd = 2)
            points(x_idx[ord], sub_ip$yvar[ord], col = cols[ti], pch = 19, cex = 1.3)
            # CI bars
            arrows(x_idx, sub_ip$LCL, x_idx, sub_ip$UCL,
                   angle = 90, code = 3, length = 0.07,
                   col = cols[ti], lwd = 1.3)
            # Raw data overlay (jittered, original scale)
            for (xi in seq_along(x_levels)) {
              mask <- as.character(data_complete[[x_var]])     == x_levels[xi] &
                as.character(data_complete[[trace_var]]) == traces[ti]
              pts  <- raw_y[mask]
              if (length(pts) > 0) {
                jit <- xi + runif(length(pts), -0.13, 0.13)
                points(jit, pts, pch = 1,
                       col = adjustcolor(cols[ti], alpha.f = 0.45), cex = 0.7)
              }
            }
          }

          legend("topright", legend = traces, col = cols, lwd = 2,
                 pch = 19, title = trace_var, bty = "n", cex = 0.85)

          int_note <- if (isTRUE(emm_is_cells))
            paste0("\nSignificant interaction; the lines representing ",
                   trace_var, " factor levels are not parallel, indicating that the effect of ",
                   x_var, " depends on ", trace_var, ".")
          else
            paste0("\nNo significant interaction. The lines representing ",
                   trace_var, " factor levels should be roughly parallel.")
          mtext(int_note, side = 1, line = 4.2, cex = 0.72,
                col = if (isTRUE(emm_is_cells)) "tomato" else "grey40")

          par(mfrow = c(1,1))
          layout(1)
        }
      }

      par(mfrow = c(1,1))
      layout(1)
    }

  }
}

# =============================================================================
# test-f_stat_wizard.R
# =============================================================================
# Tests for f_stat_wizard().
#
# Covers:
#   A. S3 dispatch (formula and data.frame methods)
#   B. formula / data argument order
#   C. Structured list return with metadata slots
#   D. paired / id_col handling
#   E. run = TRUE executes the recommended rfriend function
#   F. Intercept-only formula (y ~ 1)
#   G. Large-n safe_shapiro skip branch does not crash the wizard
# =============================================================================


# ---------------------------------------------------------------------------
# A. S3 dispatch
# ---------------------------------------------------------------------------

test_that("formula interface returns an f_stat_wizard object", {
  res <- run_quiet(f_stat_wizard(Sepal.Length ~ Species, data = iris))
  expect_s3_class(res, "f_stat_wizard")
})

test_that("data.frame interface with explicit formula arg works", {
  res <- run_quiet(f_stat_wizard(iris, formula = Sepal.Length ~ Species))
  expect_s3_class(res, "f_stat_wizard")
})

test_that("data.frame interface errors when formula is missing", {
  expect_error(
    suppressMessages(f_stat_wizard(iris)),
    "formula"
  )
})

test_that("named-only call f_stat_wizard(data = ..., formula = ...) works", {
  # Regression guard for the match.call() shim that strips data/formula
  # from dots to avoid 'matched by multiple actual arguments'.
  res <- run_quiet(
    f_stat_wizard(data = iris, formula = Sepal.Length ~ Species)
  )
  expect_s3_class(res, "f_stat_wizard")
})


# ---------------------------------------------------------------------------
# B, C. Structured return with metadata slots
# ---------------------------------------------------------------------------

test_that("return object exposes the documented metadata slots", {
  res <- run_quiet(f_stat_wizard(Sepal.Length ~ Species, data = iris))

  expected_slots <- c("y_type", "x_types", "n_groups", "group_sizes",
                      "recommended_call", "report")
  for (s in expected_slots) {
    expect_true(s %in% names(res))
  }

  expect_equal(res$n_groups, 3L)
  expect_true(is.call(res$recommended_call) ||
              is.character(res$recommended_call) ||
              is.null(res$recommended_call))
  expect_true(is.character(res$report) || is.list(res$report))
})

test_that("group_sizes reports correct counts per level", {
  res <- run_quiet(f_stat_wizard(Sepal.Length ~ Species, data = iris))
  gs  <- res$group_sizes
  expect_true(is.numeric(gs) || is.integer(gs))
  expect_true(all(gs == 50))  # iris has 50 per Species
})

test_that("recommended_call references an rfriend function when appropriate", {
  # 3 groups, normal-ish data should recommend f_aov()
  res <- run_quiet(f_stat_wizard(Sepal.Length ~ Species, data = iris))
  if (!is.null(res$recommended_call)) {
    rc <- if (is.call(res$recommended_call)) {
      deparse(res$recommended_call)
    } else {
      res$recommended_call
    }
    expect_match(rc, "f_aov|aov", ignore.case = TRUE)
  }
})


# ---------------------------------------------------------------------------
# D. paired (via id_col)
# ---------------------------------------------------------------------------

test_that("id_col triggers paired-design handling", {
  set.seed(1)
  df <- data.frame(
    subject = factor(rep(1:20, each = 2)),
    time    = factor(rep(c("pre", "post"), times = 20)),
    score   = c(rnorm(20, 10, 2), rnorm(20, 12, 2))
  )

  res <- run_quiet(
    f_stat_wizard(score ~ time, data = df, id_col = "subject")
  )
  expect_s3_class(res, "f_stat_wizard")

  # A paired 2-group numeric recommendation should mention paired t-test
  # or Wilcoxon signed rank.
  if (!is.null(res$recommended_call)) {
    rc <- if (is.call(res$recommended_call)) {
      deparse(res$recommended_call)
    } else {
      res$recommended_call
    }
    expect_match(rc, "paired|signed|f_t_test|f_wilcox", ignore.case = TRUE)
  }
})

test_that("id_col not in data is rejected", {
  expect_error(
    suppressMessages(
      f_stat_wizard(Sepal.Length ~ Species, data = iris,
                    id_col = "no_such_column")
    ),
    "id_col"
  )
})


# ---------------------------------------------------------------------------
# E. run = TRUE
# ---------------------------------------------------------------------------

test_that("run = TRUE executes the recommended function and stores result", {
  # Skipped on CRAN: run = TRUE fires the full recommended pipeline
  # (e.g. f_aov with Pandoc) which is far too heavy for CRAN's budget.
  skip_on_cran()

  res <- run_quiet_warn(
    f_stat_wizard(Sepal.Length ~ Species, data = iris, run = TRUE, output_type = "default")
  )
  expect_true("run_result" %in% names(res))
})


# ---------------------------------------------------------------------------
# F. Intercept-only formula
# ---------------------------------------------------------------------------

test_that("y ~ 1 is classified before advice is generated", {
  # Regression guard: before the fix, non-numeric Y blindly triggered
  # a one-sample t-test recommendation.
  res <- run_quiet(f_stat_wizard(Sepal.Length ~ 1, data = iris))
  expect_s3_class(res, "f_stat_wizard")
  expect_true(!is.null(res$y_type))
})


# ---------------------------------------------------------------------------
# G. Large-n safe_shapiro skip branch
# ---------------------------------------------------------------------------

test_that("wizard does not crash with n > 5000 (safe_shapiro skip)", {
  skip_on_cran()  # n = 6000 exceeds CRAN's fast-test budget

  set.seed(7)
  big <- data.frame(
    y = rnorm(6000),
    g = factor(sample(c("A", "B"), 6000, replace = TRUE))
  )
  expect_no_error(
    run_quiet(f_stat_wizard(y ~ g, data = big))
  )
})

# =============================================================================
# test-f_boxplot.R
# =============================================================================
# Tests for f_boxplot().
#
# Fixtures: local make_test_df() and make_outlier_bp_df(). The local
# make_outlier_bp_df() is renamed from make_outlier_bp_df() to avoid colliding
# with the shared helper of the same name in helper-data.R (which has a
# different column layout).
# =============================================================================

# ---------------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------------

# Small deterministic test data (avoids dependency on external datasets)
make_test_df <- function() {
  set.seed(42)
  data.frame(
    value1  = c(rnorm(10, 5), rnorm(10, 8), rnorm(10, 3)),
    value2  = c(rnorm(10, 1), rnorm(10, 4), rnorm(10, 7)),
    group   = factor(rep(c("A", "B", "C"), each = 10)),
    subgroup = factor(rep(c("X", "Y"), times = 15))
  )
}

# Data with an obvious outlier for outlier-detection tests
make_outlier_bp_df <- function() {
  set.seed(7)
  df <- make_test_df()
  df$value1[1] <- 999   # extreme outlier
  df
}

# A fancy_names map that covers all columns
test_fancy <- c(
  value1   = "Response 1 (units)",
  value2   = "Response 2 (units)",
  group    = "Treatment Group",
  subgroup = "Sub Group"
)

# =============================================================================
# 1.  INPUT VALIDATION
# =============================================================================
test_that("invalid output_type raises an error", {
  df <- make_test_df()
  expect_error(
    suppressMessages(f_boxplot(df, output_type = "html", open_generated_files = FALSE)),
    regexp = "output_type"
  )
})

test_that("data with no factor variables raises an error", {
  skip_on_cran()
  df_no_factor <- data.frame(a = 1:10, b = rnorm(10))   # no factor columns
  expect_error(
    suppressMessages(f_boxplot(df_no_factor, output_type = "png", open_generated_files = FALSE)),
    regexp = "factor"
  )
})

test_that("NULL data with no formula raises an error", {
  expect_error(f_boxplot(data = NULL, formula = NULL))
})

# =============================================================================
# 2.  BASIC DATA-ONLY USAGE  (no formula)
# =============================================================================
test_that("data-only call returns NULL invisibly for pdf output", {
  skip_on_cran()
  df <- make_test_df()
  suppressMessages(result <- f_boxplot(df,
                      output_type = "pdf",
                      open_generated_files = FALSE,
                      boxplot_explanation  = FALSE,
                      outliers             = FALSE))
  expect_null(result)
})

test_that("data-only call returns NULL invisibly for word output", {
  skip_on_cran()
  skip_if_no_pandoc()
  df <- make_test_df()
  suppressMessages(result <- f_boxplot(df,
                      output_type = "word",
                      open_generated_files = FALSE,
                      boxplot_explanation  = FALSE,
                      outliers             = FALSE))
  expect_null(result)
})

test_that("data-only PNG call creates files in tempdir", {
  skip_on_cran()
  df      <- make_test_df()
  out_dir <- tempdir()
  suppressMessages(result  <- f_boxplot(df,
                       output_type          = "png",
                       save_as              = out_dir,
                       open_generated_files = FALSE,
                       outliers             = FALSE))
  expect_null(result)
  # At least one PNG should have been created
  pngs <- list.files(out_dir, pattern = "\\.png$", full.names = FALSE)
  expect_gt(length(pngs), 0)
})

# =============================================================================
# 3.  FORMULA USAGE
# =============================================================================
test_that("formula with single response and single predictor works (pdf)", {
  skip_on_cran()
  df <- make_test_df()
  expect_null(
    suppressMessages(f_boxplot(value1 ~ group,
              data                 = df,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = FALSE))
  )
})

test_that("formula with multiple responses (LHS +) works (pdf)", {
  skip_on_cran()
  df <- make_test_df()
  expect_null(
    suppressMessages(f_boxplot(value1 + value2 ~ group,
              data                 = df,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = FALSE)
  ))
})

test_that("formula with interaction on RHS works (pdf)", {
  skip_on_cran()
  df <- make_test_df()
  expect_null(
    suppressMessages( f_boxplot(value1 ~ group * subgroup,
              data                 = df,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = FALSE))
  )
})

test_that("formula with + on RHS works (pdf)", {
  skip_on_cran()
  df <- make_test_df()
  expect_null(
    suppressMessages(f_boxplot(value1 ~ group + subgroup,
              data                 = df,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = FALSE))
  )
})

# =============================================================================
# 4.  OUTPUT TYPES COMBINED WITH KEY OPTIONS
# =============================================================================
test_that("rmd output returns (cat output) without error", {
  skip_on_cran()
  df <- make_test_df()
  # rmd should not error; we just check it runs
  expect_no_error(
    suppressMessages(f_boxplot(df,
                output_type         = "rmd",
                boxplot_explanation = FALSE,
                outliers            = FALSE)
    )
  )
})


test_that("PNG output produces one file per response x factor combination", {
  skip_on_cran()
  df      <- make_test_df()
  out_dir <- file.path(tempdir(), paste0("png_test_", Sys.getpid()))
  dir.create(out_dir, showWarnings = FALSE)
  suppressMessages(f_boxplot(df,
            output_type          = "png",
            save_as              = paste0(out_dir, "/"),
            open_generated_files = FALSE,
            outliers             = FALSE))
  pngs <- list.files(out_dir, pattern = "\\.png$")
  # 2 numeric vars x 2 factor vars = 4 PNGs expected
  expect_equal(length(pngs), 4)
})

# =============================================================================
# 5.  fancy_names
# =============================================================================
test_that("fancy_names renames columns without error (pdf)", {
  skip_on_cran()
  df <- make_test_df()
  expect_null(
    suppressMessages(f_boxplot(df,
              fancy_names          = test_fancy,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = FALSE)
  ))
})

test_that("fancy_names + formula combination works (pdf)", {
  skip_on_cran()
  df <- make_test_df()
  expect_null(
    suppressMessages(f_boxplot(value1 ~ group,
              data                 = df,
              fancy_names          = test_fancy,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = FALSE))
  )
})

# =============================================================================
# 6.  JITTER OPTION
# =============================================================================
test_that("jitter = TRUE works without error (pdf)", {
  skip_on_cran()
  df <- make_test_df()
  expect_null(
    suppressMessages(f_boxplot(df,
              jitter               = TRUE,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = FALSE)
  ))
})

test_that("jitter = TRUE + formula works without error (pdf)", {
  skip_on_cran()
  df <- make_test_df()
  expect_null(
    suppressMessages( f_boxplot(value1 + value2 ~ group,
              data                 = df,
              jitter               = TRUE,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = FALSE)
  ))
})

# =============================================================================
# 7.  OUTLIER DETECTION
# =============================================================================


test_that("outliers = TRUE includes outlier section in rmd output", {
  skip_on_cran()
  df  <- make_outlier_bp_df()
  out <-
    suppressMessages(f_boxplot(df,
              output_type         = "rmd",
              outliers            = TRUE,
              boxplot_explanation = FALSE)
  )
  combined <- paste(out, collapse = "\n")
  # Should contain an outlier heading or "No outliers" message
  has_outlier_section <- grepl("Outlier Table|No outliers", combined)
  expect_true(has_outlier_section)
})

test_that("outliers = FALSE skips outlier detection silently", {
  skip_on_cran()
  df  <- make_outlier_bp_df()
  out <-
    suppressMessages( f_boxplot(df,
              output_type         = "rmd",
              outliers            = FALSE,
              boxplot_explanation = FALSE)
  )
  combined <- paste(out, collapse = "\n")
  expect_false(grepl("Outlier Table", combined))
})

test_that("custom coef value is accepted without error", {
  skip_on_cran()
  df <- make_outlier_bp_df()
  expect_null(
    suppressMessages(f_boxplot(df,
              outliers             = TRUE,
              coef                 = 3.0,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE)
  ))
})

# =============================================================================
# 8.  boxplot_explanation FLAG
# =============================================================================
test_that("boxplot_explanation = TRUE adds explanation text to rmd", {
  skip_on_cran()
  df  <- make_test_df()
  out <- suppressMessages(f_boxplot(df,
              output_type         = "rmd",
              boxplot_explanation = TRUE,
              outliers            = FALSE)
  )
  combined <- paste(out, collapse = "\n")
  expect_match(combined, "Understanding Boxplots", fixed = TRUE)
})

test_that("boxplot_explanation = FALSE omits explanation text from rmd", {
  skip_on_cran()
  df  <- make_test_df()
  out <-
    suppressMessages( f_boxplot(df,
              output_type         = "rmd",
              boxplot_explanation = FALSE,
              outliers            = FALSE)
  )
  combined <- paste(out, collapse = "\n")
  expect_false(grepl("Understanding Boxplots", combined, fixed = TRUE))
})

test_that("boxplot_explanation is forced FALSE for png output", {
  skip_on_cran()
  # If boxplot_explanation were TRUE for png it would cat() markdown mid-plot
  df      <- make_test_df()
  out_dir <- file.path(tempdir(), paste0("explain_png_", Sys.getpid()))
  dir.create(out_dir, showWarnings = FALSE)
  # Should not error even if user passes boxplot_explanation = TRUE with png
  expect_no_error(
    suppressMessages(f_boxplot(df,
              output_type          = "png",
              save_as              = paste0(out_dir, "/"),
              open_generated_files = FALSE,
              outliers             = FALSE,
              boxplot_explanation  = TRUE)   # function should override to FALSE
  ))
})

# =============================================================================
# 9.  save_as & FILE PATH HANDLING
# =============================================================================
test_that("save_as with full path creates file at that location (pdf)", {
  skip_on_cran()
  df       <- make_test_df()
  out_path <- file.path(tempdir(), paste0("custom_name_", Sys.getpid(), ".pdf"))
  suppressMessages(f_boxplot(df,
            save_as              = out_path,
            output_type          = "pdf",
            open_generated_files = FALSE,
            boxplot_explanation  = FALSE,
            outliers             = FALSE))
  expect_true(file.exists(out_path))
})

test_that("save_as with .docx extension overrides output_type to word", {
  skip_on_cran()
  df       <- make_test_df()
  out_path <- file.path(tempdir(), paste0("override_ext_", Sys.getpid(), ".docx"))
  # Pass output_type = "pdf" but extension says .docx - should produce docx
  suppressMessages(f_boxplot(df,
            save_as              = out_path,
            output_type          = "pdf",   # should be overridden
            open_generated_files = FALSE,
            boxplot_explanation  = FALSE,
            outliers             = FALSE))
  expect_true(file.exists(out_path))
})

test_that("save_as directory-only path saves file with default name", {
  skip_on_cran()
  df      <- make_test_df()
  out_dir <- file.path(tempdir(), paste0("dir_only_", Sys.getpid()))
  dir.create(out_dir, showWarnings = FALSE)
  suppressMessages(f_boxplot(df,
            save_as              = paste0(out_dir, "/"),
            output_type          = "pdf",
            open_generated_files = FALSE,
            boxplot_explanation  = FALSE,
            outliers             = FALSE))
  pdfs <- list.files(out_dir, pattern = "\\.pdf$")
  expect_equal(length(pdfs), 1)
  expect_match(pdfs[1], "BoxPlot")
})

# =============================================================================
# 10. COMBINED OPTIONS  (the combos most likely to interact badly)
# =============================================================================
test_that("formula + fancy_names + jitter + outliers + word output", {
  skip_on_cran()
  skip_if_no_pandoc()
  df <- make_outlier_bp_df()
  expect_null(
    suppressMessages(f_boxplot(value1 + value2 ~ group,
              data                 = df,
              fancy_names          = test_fancy,
              jitter               = TRUE,
              outliers             = TRUE,
              coef                 = 1.5,
              output_type          = "word",
              open_generated_files = FALSE,
              boxplot_explanation  = TRUE)
  ))
})

test_that("formula + fancy_names + jitter + outliers + pdf output", {
  skip_on_cran()
  df <- make_outlier_bp_df()
  expect_null(
    suppressMessages(f_boxplot(value1 + value2 ~ group * subgroup,
              data                 = df,
              fancy_names          = test_fancy,
              jitter               = TRUE,
              outliers             = TRUE,
              coef                 = 2.0,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE)
  ))
})

test_that("detect_factors = FALSE with pre-factored data works (pdf)", {
  skip_on_cran()
  df <- make_test_df()   # already has factor columns
  expect_null(
    suppressMessages(f_boxplot(df,
              detect_factors       = FALSE,
              output_type          = "pdf",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = FALSE)
  ))
})

test_that("iris dataset end-to-end with fancy_names (word)", {
  skip_on_cran()
  skip_if_no_pandoc()
  data(iris)
  new_names <- c(
    "Sepal.Length" = "Sepal length (cm)",
    "Sepal.Width"  = "Sepal width (cm)",
    "Petal.Length" = "Petal length (cm)",
    "Petal.Width"  = "Petal width (cm)",
    "Species"      = "Cultivar"
  )
  expect_null(
    suppressMessages( f_boxplot(iris,
              fancy_names          = new_names,
              output_type          = "word",
              open_generated_files = FALSE,
              boxplot_explanation  = FALSE,
              outliers             = TRUE)
  ))
})

test_that("mtcars formula example from documentation works (word)", {
  skip_on_cran()
  skip_if_no_pandoc()
  data(mtcars)
  expect_null(
    suppressMessages(f_boxplot(hp + disp ~ gear * cyl,
              data                 = mtcars,
              boxplot_explanation  = FALSE,
              output_type          = "word",
              open_generated_files = FALSE)
  ))
})

# =============================================================================
# 11. PNG SPECIFIC OPTIONS  (width / height / units / res)
# =============================================================================
test_that("custom PNG dimensions are accepted without error", {
  skip_on_cran()
  df      <- make_test_df()
  out_dir <- file.path(tempdir(), paste0("png_dims_", Sys.getpid()))
  dir.create(out_dir, showWarnings = FALSE)
  expect_no_error(
    suppressMessages(f_boxplot(df,
              output_type          = "png",
              save_as              = paste0(out_dir, "/"),
              open_generated_files = FALSE,
              outliers             = FALSE,
              width  = 12,
              height = 9,
              units  = "in",
              res    = 150)
  ))
})

test_that("las parameter variants (0-3) are accepted without error", {
  skip_on_cran()
  df <- make_test_df()
  for (las_val in 0:3) {
    # Use expect_error with regexp=NA as alternative - it does accept label
    expect_error(
      suppressMessages(f_boxplot(df,
                las                  = las_val,
                output_type          = "pdf",
                open_generated_files = FALSE,
                boxplot_explanation  = FALSE,
                outliers             = FALSE),
      regexp = NA,   # NA means "expect NO error"
      label  = paste("las =", las_val)
    ))
  }
})

# =============================================================================
# 12. GRAPHICAL PARAMETER RESTORATION  (tests bug #2 - par inside boxplot)
# =============================================================================
test_that("par() settings are restored after function call", {
  skip_on_cran()
  df       <- make_test_df()
  mar_before <- par("mar")
  suppressMessages( f_boxplot(df,
            output_type          = "pdf",
            open_generated_files = FALSE,
            boxplot_explanation  = FALSE,
            outliers             = FALSE))
  mar_after <- par("mar")
  expect_equal(mar_before, mar_after)
})


# =============================================================================
# S3 dispatch cleanup
# =============================================================================
#   A. f_boxplot.formula uses formula, data arguments (exempted name).
#   B. f_boxplot.data.frame uses x as the first argument.
#   C. Both dispatch paths produce a plot object without erroring.

# Local wrapper (differs from shared quiet_f_boxplot in helper-quiet.R:
# uses output_type = "rmd" and tryCatch-to-error pattern specific to
# these dispatch tests).
quiet_bp <- function(...) {
  suppressMessages(
    suppressWarnings(
      tryCatch(
        f_boxplot(..., open_generated_files = FALSE,
                  output_type = "rmd"),
        error = function(e) e
      )
    )
  )
}

# ---------------------------------------------------------------------------
# A. formula method
# ---------------------------------------------------------------------------

test_that("formula interface dispatches to f_boxplot.formula", {
  res <- quiet_bp(Sepal.Width ~ Species, data = iris)
  expect_false(inherits(res, "error"),
               info = if (inherits(res, "error")) conditionMessage(res)
                      else "")
})

test_that("formula interface with explicit data arg name works", {
  res <- quiet_bp(formula = Sepal.Width ~ Species, data = iris)
  expect_false(inherits(res, "error"))
})

# ---------------------------------------------------------------------------
# B. data.frame method
# ---------------------------------------------------------------------------

test_that("data.frame interface dispatches to f_boxplot.data.frame", {
  df <- iris[, c("Sepal.Width", "Species")]
  res <- quiet_bp(df)
  expect_false(inherits(res, "error"))
})

test_that("named call f_boxplot(x = df) works", {
  df <- iris[, c("Sepal.Width", "Species")]
  res <- quiet_bp(x = df)
  expect_false(inherits(res, "error"))
})

# =============================================================================
# teardown.R
# =============================================================================
# Runs ONCE at the very end of the test suite, after every test-*.R file has
# finished. Use for cleanup that cannot be attached to an individual test.
#
# Anything that should be scoped to a single test belongs in the test block
# itself via withr::local_*() / defer(), not here.
# =============================================================================


# -----------------------------------------------------------------------------
# Close the throwaway graphics device opened in setup.R
# -----------------------------------------------------------------------------
# Wrapped in tryCatch because a test that mishandled dev.off() might have
# closed it already. A secondary dev.off() on a closed device raises an
# error we do not want bubbling up as a suite failure.
tryCatch(
  while (!is.null(grDevices::dev.list())) grDevices::dev.off(),
  error = function(e) invisible(NULL)
)


# -----------------------------------------------------------------------------
# Restore options() to whatever they were before setup.R ran
# -----------------------------------------------------------------------------
if (exists("old_opts", inherits = TRUE)) {
  options(old_opts)
}


# -----------------------------------------------------------------------------
# Wipe any stray rfriend-generated output files from tempdir()
# -----------------------------------------------------------------------------
# Several f_*() functions write PDFs / .docx / .xlsx to tempdir() by
# default. A well-behaved test unlinks its own output via
# withr::local_tempfile(), but this is a safety net for tests that forgot.
stray <- list.files(
  tempdir(),
  pattern    = "\\.(pdf|docx|xlsx|png|html|md|rmd)$",
  full.names = TRUE,
  ignore.case = TRUE
)
if (length(stray)) {
  unlink(stray, force = TRUE)
}

#' Wrap lines in rmd output documents
#' @details Does not have any options or arguments. It modifies 'knitr' settings and creates a function that is able to wrap lines.
#'
#' @return A list with 'knitr' setting and functions to wrap lines. Used for its side effects on 'knitr'
#'
#'
#' @noRd
f_wrap_lines <- function(){
  hook_output <- knit_hooks$get('output')
  knit_hooks$set(output = function(x, options) {
    # this hook is used only when the linewidth option is not NULL
    if (!is.null(n <- options$linewidth)) {
      x <- xfun::split_lines(x)
      # any lines with more character than n should be wrapped
      if (any(nchar(x) > n)) x = strwrap(x, width = n)
      x <- paste(x, collapse = '\n')
    }
    hook_output(x, options)
  })
}

# =============================================================================
# Save and restore session state (par, options, panderOptions)
# =============================================================================
#
# Internal helpers used by every f_* function in the package that
# temporarily mutates graphical parameters or global options. The pair
# is meant to be used together at the top of a function, like this:
#
#   my_function <- function(...) {
#     .session_state <- save_session_state()
#     on.exit(restore_session_state(.session_state), add = TRUE)
#     # ... function body that freely mutates par() / options() ...
#   }
#
# save_session_state() captures a snapshot of everything worth
# restoring. restore_session_state() puts it all back, tolerating any
# individual restore error so that one bad value can't abort the entire
# on.exit() cleanup.
#
# NOTE: on.exit() must be called from the function whose state we
# want to restore, NOT from inside a helper. That is why this module
# exposes two separate functions instead of a single
# "setup_exit_handler()" wrapper: on.exit() inside a helper would
# register its handler on the helper itself, which exits immediately,
# and the real caller would never get the cleanup.
# =============================================================================


# -----------------------------------------------------------------------------
# save_session_state
# -----------------------------------------------------------------------------
# Captures the current graphical parameters, global options, and
# panderOptions into a single list suitable for later restoration.
#
# Returns a list with three elements:
#   $par     - named list from par(no.readonly = TRUE), with unsafe keys
#              stripped (see below)
#   $options - named list from options()
#   $pander  - named list from pander::panderOptions(), or NULL if the
#              pander package is not available
#
# Why some par() keys are stripped:
#
# par(no.readonly = TRUE) returns every readable graphical parameter,
# but several of them describe the CURRENT plot region on the CURRENT
# device and become meaningless (or invalid) once the device state has
# changed. Restoring them after the function has drawn into a new
# device, changed the layout, or closed and reopened a graphics device
# raises errors like
#
#     invalid value specified for graphical parameter "fig"
#
# which is a perennial CRAN example-check failure. The stripped keys:
#
#   $new  - write-only trigger; saving / restoring it is meaningless
#   $fig  - figure region in normalised device coordinates; tied to
#           the device that was open at snapshot time
#   $plt  - plot region in normalised figure coordinates; same issue
#   $fin  - figure region dimensions in inches
#   $pin  - plot region dimensions in inches
#   $usr  - user coordinate ranges for the current plot; undefined
#           before any high-level plot has been drawn
#
# Everything else ("mar", "mfrow", "mfcol", "bg", "col", "cex", ...)
# is safe to save and restore across arbitrary device activity.
# -----------------------------------------------------------------------------

save_session_state <- function() {

  old_par <- graphics::par(no.readonly = TRUE)

  # Strip parameters that cannot be safely restored. See the block
  # comment above for the rationale.
  old_par$new <- NULL
  old_par$fig <- NULL
  old_par$plt <- NULL
  old_par$fin <- NULL
  old_par$pin <- NULL
  old_par$usr <- NULL

  old_options <- options()

  old_options$warn <- NULL
  old_options$warning.length <- NULL
  old_options$nwarnings <- NULL

  # pander is a soft dependency. Only snapshot its options if the
  # package is actually available and its API is what we expect.
  old_pander <- if (requireNamespace("pander", quietly = TRUE) &&
                    is.function(pander::panderOptions)) {
    pander::panderOptions()
  } else {
    NULL
  }

  list(
    par     = old_par,
    options = old_options,
    pander  = old_pander
  )
}


# -----------------------------------------------------------------------------
# restore_session_state
# -----------------------------------------------------------------------------
# Restores a snapshot produced by save_session_state(). Each of the
# three sub-restores is wrapped in tryCatch() so that a failure in one
# cannot prevent the others from running. After stripping the known-
# bad par() keys in save_session_state() the par() restore should
# always succeed in practice, but the guard is kept as
# belt-and-suspenders: on.exit() handlers run during function exit
# (including error exits), and raising a new error there would mask
# the original problem.
#
# Arguments:
#   state - a list produced by save_session_state(). Must contain
#           elements named "par", "options", and "pander" (the last
#           may be NULL).
#
# Returns NULL invisibly.
# -----------------------------------------------------------------------------

restore_session_state <- function(state) {

  # Guard against being called with a malformed or NULL snapshot
  # (e.g. if a caller forgot to assign the return value of
  # save_session_state() before registering on.exit()).
  if (is.null(state) || !is.list(state)) {
    return(invisible(NULL))
  }

  # Reset the layout matrix first. layout() is not captured by
  # par(no.readonly = TRUE), so a layout set during the function
  # body would otherwise persist and affect subsequent plots.
  # layout(1) is cheap and idempotent, so we always run it.
  tryCatch(
    graphics::layout(1),
    error = function(e) invisible(NULL)
  )


  # ---- Restore graphical parameters ---------------------------------------
  if (!is.null(state$par) && length(state$par) > 0) {
    tryCatch(
      graphics::par(state$par),
      error   = function(e) invisible(NULL)
      # ,warning = function(w) invisible(NULL)
    )
  }

  # ---- Restore global options ---------------------------------------------
  if (!is.null(state$options)) {
    tryCatch(
      options(state$options),
      error   = function(e) invisible(NULL)
      # ,warning = function(w) invisible(NULL)
    )
  }

  # ---- Restore panderOptions ----------------------------------------------
  # panderOptions() does not accept a whole-list form; each option
  # must be restored individually. If pander was not available at
  # snapshot time, state$pander is NULL and this block is skipped.
  if (!is.null(state$pander) &&
      requireNamespace("pander", quietly = TRUE) &&
      is.function(pander::panderOptions)) {

    for (opt in names(state$pander)) {
      tryCatch(
        pander::panderOptions(opt, state$pander[[opt]]),
        error   = function(e) invisible(NULL)
        # ,warning = function(w) invisible(NULL)
      )
    }
  }

  invisible(NULL)
}

# Helper function (internal) to format Analysis of Variance (ANOVA) table with bold p-values
rmd_anova_summary <- function(aov_object) {

  #Set Pander options for a fancy output tabel
  panderOptions('table.alignment.default', 'left')
  panderOptions('table.alignment.rownames', 'left')
  panderOptions('keep.trailing.zeros', TRUE)
  panderOptions('knitr.auto.asis', FALSE)
  panderOptions('table.split.table', 300)
  panderOptions('table.caption.prefix', '')  # Remove prefix Table in caption
  # Extract summary as a list
  summary_list <- summary(aov_object)

  # Extract the ANOVA table
  anova_table <- as.data.frame(summary_list[[1]])

  # Check if "Pr(>F)" column exists (to handle cases without p-values)
  if ("Pr(>F)" %in% colnames(anova_table)) {
    # Format p-values: Bold those less than 0.05
    anova_table[["Pr(>F)"]] <- ifelse(
      anova_table[["Pr(>F)"]] < 0.05,
      paste0("**", formatC(anova_table[["Pr(>F)"]]), "**"),
      formatC(anova_table[["Pr(>F)"]])
    )
  }

  # Use pander to render the table
  pander(anova_table)
}

# =============================================================================
# helper-data.R
# =============================================================================
# Shared test fixtures used across the rfriend test suite.
#
# Files starting with "helper-" are sourced automatically by testthat before
# any test is run. Keep this file free of expect_*() calls: it exists only to
# define reusable fixture builders.
#
# Naming convention:
#   make_<shape>_<variant>()   returns a data.frame / vector, seeded for
#                              reproducibility. Every builder accepts a `seed`
#                              argument so a test can override it if needed.
#
# Rationale for consolidation:
#   Pre-refactor the suite contained ~12 near-duplicate builders scattered
#   across files (make_df, make_test_df, make_two_factor_data vs
#   make_two_factor_df, etc.). They diverged only in cosmetic ways. Anything
#   test-specific (edge cases a single test needs) should stay inline in that
#   test file, not bloat this module.
# =============================================================================


# -----------------------------------------------------------------------------
# One-way designs: y ~ group
# -----------------------------------------------------------------------------

# Normal-ish, 3 groups, well-separated means. Good default for happy-path tests
# of f_aov(), f_t_test() (pairwise), f_kruskal_test(), f_boxplot().
make_normal_data <- function(seed = 42, n_per_group = 15) {
  set.seed(seed)
  data.frame(
    y   = c(rnorm(n_per_group, 10, 1),
            rnorm(n_per_group, 13, 1),
            rnorm(n_per_group, 16, 1)),
    grp = factor(rep(c("A", "B", "C"), each = n_per_group))
  )
}

# Right-skewed data where Box-Cox / bestNormalize transformations should fire.
make_skewed_data <- function(seed = 7, n_per_group = 20) {
  set.seed(seed)
  data.frame(
    y   = c(rexp(n_per_group, 0.10),
            rexp(n_per_group, 0.05),
            rexp(n_per_group, 0.02)),
    grp = factor(rep(c("low", "med", "high"), each = n_per_group))
  )
}

# Plain numeric vector of skewed values. Used by transform-only tests
# (f_boxcox, f_bestNormalize) that do not need a grouping factor.
make_skewed_vec <- function(seed = 3, n = 100) {
  set.seed(seed)
  rexp(n, rate = 0.1)
}


# -----------------------------------------------------------------------------
# Factorial / mixed designs
# -----------------------------------------------------------------------------

# Two crossed factors, balanced cells. Used by interaction-model tests.
make_two_factor_data <- function(seed = 99, n_per_cell = 8) {
  set.seed(seed)
  d <- expand.grid(
    A   = factor(c("a1", "a2")),
    B   = factor(c("b1", "b2")),
    rep = seq_len(n_per_cell)
  )
  d$y <- rnorm(nrow(d), mean = as.integer(d$A) + as.integer(d$B), sd = 0.8)
  d
}

# Repeated-measures: n_subj subjects, n_time time points, random intercept
# per subject plus a linear time effect. Used by f_lmer() tests.
make_repeated_measures_data <- function(seed = 2026, n_subj = 20, n_time = 4) {
  set.seed(seed)
  subj     <- factor(rep(seq_len(n_subj), each = n_time))
  tm       <- factor(rep(seq_len(n_time), times = n_subj))
  subj_int <- rnorm(n_subj, 0, 2)[as.integer(subj)]
  y        <- 10 + as.integer(tm) * 1.5 + subj_int +
              rnorm(n_subj * n_time, 0, 1)
  data.frame(y = y, time = tm, subj = subj)
}

# Same shape but with a treatment factor replacing continuous time, for
# factor-by-subject tests.
make_factor_repeated_data <- function(seed = 31, n_subj = 24, reps = 5) {
  set.seed(seed)
  trt  <- factor(rep(c("ctrl", "low", "high"), length.out = n_subj))
  subj <- factor(seq_len(n_subj))
  d <- data.frame(
    subj = rep(subj, each = reps),
    trt  = rep(trt,  each = reps),
    rep  = rep(seq_len(reps), times = n_subj)
  )
  subj_int <- rnorm(n_subj, 0, 1.5)[as.integer(d$subj)]
  trt_eff  <- c(ctrl = 0, low = 2, high = 5)[as.character(d$trt)]
  d$y      <- 10 + trt_eff + subj_int + rnorm(nrow(d), 0, 1)
  d
}


# -----------------------------------------------------------------------------
# GLM / binary response
# -----------------------------------------------------------------------------

# Binary outcome with a group effect. Used by f_glm() logistic tests.
make_binary_data <- function(seed = 2026, n_per_group = 40) {
  set.seed(seed)
  grp <- factor(rep(c("ctrl", "trt"), each = n_per_group))
  p   <- ifelse(grp == "ctrl", 0.3, 0.7)
  data.frame(
    y   = rbinom(2 * n_per_group, 1, p),
    grp = grp
  )
}


# -----------------------------------------------------------------------------
# Outlier / cleaning fixtures
# -----------------------------------------------------------------------------

# Mixed data frame with one deliberately injected numeric outlier per relevant
# column. Used by f_outliers(), f_remove_outliers(), f_scan(), f_boxplot()
# (outlier-labeling path).
make_outlier_df <- function(seed = 42) {
  set.seed(seed)
  data.frame(
    Team       = rep(c("A", "B"), each = 20),
    Department = rep(c("Sales", "IT"), each = 10, times = 2),
    Salary     = c(rnorm(19, 50000, 500), 100000,
                   rnorm(19, 50000, 500),   1000),
    Age        = c(rnorm(38, 35, 2), 90, 35),
    EmployeeID = paste0("E", sprintf("%03d", seq_len(40))),
    stringsAsFactors = FALSE
  )
}

# Plain numeric vector with a single extreme value. Used for the
# vector-method tests of f_outliers().
make_outlier_vec <- function(seed = 42, n = 30) {
  set.seed(seed)
  x <- rnorm(n, 10, 1)
  x[1] <- 999
  x
}


# -----------------------------------------------------------------------------
# Correlation / multi-column fixtures
# -----------------------------------------------------------------------------

# All-numeric frame. Used by f_corplot() numeric-only tests.
make_numeric_df <- function(seed = 42, n = 50) {
  set.seed(seed)
  data.frame(
    a = rnorm(n), b = rnorm(n), c = rnorm(n), d = rnorm(n)
  )
}

# Numeric frame plus k factor columns. The factor columns are named
# f1, f2, ..., fk so tests can reference them positionally.
make_mixed_df <- function(seed = 42, n = 60, n_factors = 1L,
                          factor_levels = c("A", "B", "C")) {
  set.seed(seed)
  nums <- as.data.frame(
    replicate(4, rnorm(n), simplify = FALSE),
    col.names = paste0("num", seq_len(4))
  )
  facs <- as.data.frame(
    replicate(
      n_factors,
      factor(sample(factor_levels, n, replace = TRUE)),
      simplify = FALSE
    )
  )
  names(facs) <- paste0("f", seq_len(n_factors))
  cbind(nums, facs)
}


# -----------------------------------------------------------------------------
# Summary-statistics fixtures
# -----------------------------------------------------------------------------

# Small, fully-specified frame used by f_summary() tests. Kept non-random so
# exact-value assertions (sum, mean, sd) stay stable without a seed.
make_summary_df <- function() {
  data.frame(
    value   = c(10, 20, 30, 40, 50, 60, 70, 80, 90, 100),
    weight  = c(1.1, 2.2, 3.3, 4.4, 5.5, 6.6, 7.7, 8.8, 9.9, 11.0),
    group_a = rep(c("A", "B"), each = 5),
    group_b = rep(c("X", "Y"), times = 5),
    int_col = as.integer(seq_len(10)),
    stringsAsFactors = FALSE
  )
}

# =============================================================================
# f_lmer_stress_test.R
# Comprehensive stress tests for f_lmer().
#
# Run with: testthat::test_file("f_lmer_stress_test.R")
#
# API note: results are ALWAYS nested under response name, consistent with
# f_aov / f_kruskal_test / f_glm / f_t_test / f_wilcox_test. For a single
# response use `res[["y"]]$model`; for multiple responses use
# `res[["y1"]]$model`, `res[["y2"]]$model`, etc.
#
# Test organisation:
#   1.  Return object class and structure
#   2.  Random-intercept model (sleepstudy)
#   3.  Random intercept + slope
#   4.  Factor fixed effect triggers post-hoc
#   5.  Multiple response variables
#   6.  ddf options (Satterthwaite, Kenward-Roger, lme4)
#   7.  Pass-through of subset / na.action / weights
#   8.  Contrasts forwarded via dots
#   9.  Input validation
#  10.  safe_shapiro large-n guard on residuals
#  11.  Singular fit handling
#  12.  intro_text toggle
#  13.  rmd output type stores markdown
# =============================================================================
# ---------------------------------------------------------------------------
# Skip-if-no-lme4 helper
# ---------------------------------------------------------------------------
skip_if_no_lme4 <- function() {
  testthat::skip_if_not_installed("lme4")
  testthat::skip_if_not_installed("lmerTest")
}

# ---------------------------------------------------------------------------
# Wrappers (SAFE only when not forwarding subset/weights/na.action through
# dots - wrappers break match.call capture for those args; direct f_lmer
# calls are used in section 7 and 8 below).
# ---------------------------------------------------------------------------

# quiet_lmer removed - use shared quiet_f_lmer() from helper-quiet.R

# ---------------------------------------------------------------------------
# Test data generators
# ---------------------------------------------------------------------------

# Repeated-measures: 20 subjects x 4 time points
# make_rm_data removed - use shared make_repeated_measures_data() from helper-data.R

# Factor fixed effect for post-hoc testing
make_factor_rm_data <- function(seed = 31, n_subj = 24) {
  set.seed(seed)
  trt  <- factor(rep(c("ctrl", "low", "high"), length.out = n_subj))
  subj <- factor(seq_len(n_subj))
  reps <- 5
  expand_df <- data.frame(
    subj = rep(subj, each = reps),
    trt  = rep(trt,  each = reps),
    rep  = rep(seq_len(reps), times = n_subj)
  )
  subj_int <- rnorm(n_subj, 0, 1.5)[as.integer(expand_df$subj)]
  trt_eff  <- c(ctrl = 0, low = 2, high = 5)[as.character(expand_df$trt)]
  expand_df$y <- 10 + trt_eff + subj_int + rnorm(nrow(expand_df), 0, 1)
  expand_df
}

# ===========================================================================
# 1. Return object class and structure
# ===========================================================================

test_that("f_lmer returns an object of class f_lmer", {
  skip_on_cran()
  skip_if_no_lme4()
  df  <- make_repeated_measures_data()
  res <- quiet_f_lmer(y ~ time + (1 | subj), data = df)
  expect_s3_class(res, "f_lmer")
})

test_that("result contains one element per response variable", {
  skip_on_cran()
  skip_if_no_lme4()
  df  <- make_repeated_measures_data()
  res <- quiet_f_lmer(y ~ time + (1 | subj), data = df)
  expect_true("y" %in% names(res))
})

test_that("per-response slot exposes documented components", {
  skip_on_cran()
  skip_if_no_lme4()
  df  <- make_repeated_measures_data()
  res <- quiet_f_lmer(y ~ time + (1 | subj), data = df)

  entry <- res[["y"]]

  expected_slots <- c("model", "fixed_effects", "var_components",
                      "r_squared", "fit_indices")
  for (s in expected_slots) {
    expect_true(s %in% names(entry),
                info = paste("Missing slot:", s))
  }
  expect_true(inherits(entry$model, "lmerMod") ||
              inherits(entry$model, "lmerModLmerTest"))
})

# ===========================================================================
# 2. Random-intercept model
# ===========================================================================

test_that("sleepstudy random-intercept model runs cleanly", {
  skip_on_cran()
  skip_if_no_lme4()
  data(sleepstudy, package = "lme4")
  expect_no_error(
    res <- quiet_f_lmer(Reaction ~ Days + (1 | Subject), data = sleepstudy)
  )
  expect_s3_class(res, "f_lmer")

  fe <- res[["Reaction"]]$fixed_effects
  expect_true(is.data.frame(fe) || is.matrix(fe))
})

# ===========================================================================
# 3. Random intercept + slope
# ===========================================================================

test_that("random intercept + slope model runs", {
  skip_on_cran()
  skip_if_no_lme4()
  data(sleepstudy, package = "lme4")
  expect_no_error(
    res <- quiet_f_lmer(Reaction ~ Days + (1 + Days | Subject),
                      data = sleepstudy)
  )
  vc <- res[["Reaction"]]$var_components
  expect_false(is.null(vc))
})

# ===========================================================================
# 4. Factor fixed effect triggers post hoc
# ===========================================================================

test_that("factor fixed effect produces a post-hoc table", {
  skip_on_cran()
  skip_if_no_lme4()
  testthat::skip_if_not_installed("emmeans")

  df  <- make_factor_rm_data()
  res <- quiet_f_lmer(y ~ trt + (1 | subj), data = df)

  ph <- res[["y"]]$post_hoc
  expect_false(is.null(ph))
  if (!is.null(ph) && length(ph) > 0) {
    expect_true("trt" %in% names(ph))
  }
})

test_that("post_hoc = FALSE suppresses the pairwise comparisons", {
  skip_on_cran()
  skip_if_no_lme4()
  df  <- make_factor_rm_data()
  res <- quiet_f_lmer(y ~ trt + (1 | subj), data = df, post_hoc = FALSE)
  ph  <- res[["y"]]$post_hoc
  expect_true(is.null(ph) || length(ph) == 0L)
})

# ===========================================================================
# 5. Multiple responses
# ===========================================================================

test_that("multiple responses produce one slot per response", {
  skip_on_cran()
  skip_if_no_lme4()
  data(sleepstudy, package = "lme4")
  ss <- sleepstudy
  ss$Reaction2 <- ss$Reaction + rnorm(nrow(ss), 0, 3)

  res <- quiet_f_lmer(Reaction + Reaction2 ~ Days + (1 | Subject),
                    data = ss)
  expect_true(all(c("Reaction", "Reaction2") %in% names(res)))
  expect_true(!is.null(res[["Reaction"]]$model))
  expect_true(!is.null(res[["Reaction2"]]$model))
})

# ===========================================================================
# 6. ddf options
# ===========================================================================

test_that("ddf = 'Satterthwaite' (default) produces p-values", {
  skip_on_cran()
  skip_if_not_installed("pbkrtest")
  skip_if_no_lme4()
  df  <- make_repeated_measures_data()
  res <- quiet_f_lmer(y ~ time + (1 | subj), data = df)
  fe  <- res[["y"]]$fixed_effects
  has_p <- any(grepl("Pr\\(|p[\\._]value|p$",
                     names(fe), ignore.case = TRUE))
  expect_true(has_p)
})

test_that("ddf = 'lme4' suppresses p-values (t-stats only)", {
  skip_on_cran()
  skip_if_no_lme4()
  df  <- make_repeated_measures_data()
  res <- quiet_f_lmer(y ~ time + (1 | subj), data = df, ddf = "lme4")
  expect_false(is.null(res[["y"]]$fixed_effects))
})

test_that("ddf = 'Kenward-Roger' runs when pbkrtest is available", {
  skip_on_cran()
  skip_if_no_lme4()
  testthat::skip_if_not_installed("pbkrtest")
  df <- make_repeated_measures_data()
  expect_no_error(
    res <- quiet_f_lmer(y ~ time + (1 | subj), data = df,
                      ddf = "Kenward-Roger")
  )
})

test_that("bad ddf is rejected", {
  skip_on_cran()
  skip_if_no_lme4()
  df <- make_repeated_measures_data()
  expect_error(
    quiet_f_lmer(y ~ time + (1 | subj), data = df, ddf = "invalid")
  )
})

# ===========================================================================
# 7. Pass-through of subset / na.action / weights
#    Direct f_lmer() calls only.
# ===========================================================================

test_that("subset drops rows before the lmer is fit", {
  skip_on_cran()
  skip_if_no_lme4()
  df <- make_repeated_measures_data(n_subj = 20, n_time = 4)     # 80 rows
  suppressMessages(suppressWarnings(
    utils::capture.output(
      res <- f_lmer(y ~ time + (1 | subj), data = df,
                    subset = df$time != "1",
                    output_type = "default",
                    norm_plots = FALSE, intro_text = FALSE,
                    open_generated_files = FALSE),
      file = nullfile()
    )
  ))
  m <- res[["y"]]$model
  # time had 4 levels, 20 obs each; subset drops level 1 -> 60 rows
  expect_equal(nobs(m), 60L)
})

test_that("na.action = na.omit strips NA rows before fitting", {
  skip_on_cran()
  skip_if_no_lme4()
  df <- make_repeated_measures_data()
  df$y[c(1, 5, 10, 15)] <- NA
  suppressMessages(suppressWarnings(
    utils::capture.output(
      res <- f_lmer(y ~ time + (1 | subj), data = df,
                    na.action = na.omit,
                    output_type = "default",
                    norm_plots = FALSE, intro_text = FALSE,
                    open_generated_files = FALSE),
      file = nullfile()
    )
  ))
  expect_equal(nobs(res[["y"]]$model), 76L)        # 80 - 4
})

test_that("weights pass through to lmer without error", {
  skip_on_cran()
  skip_if_no_lme4()
  df <- make_repeated_measures_data()
  w  <- runif(nrow(df), 0.5, 2)
  expect_no_error(
    suppressMessages(suppressWarnings(
      utils::capture.output(
        res <- f_lmer(y ~ time + (1 | subj), data = df, weights = w,
                      output_type = "default",
                      norm_plots = FALSE, intro_text = FALSE,
                      open_generated_files = FALSE),
        file = nullfile()
      )
    ))
  )
  expect_equal(nobs(res[["y"]]$model), nrow(df))
})

test_that("multi-response share identical row set after subset", {
  skip_on_cran()
  skip_if_no_lme4()
  df <- make_repeated_measures_data()
  df$y2 <- df$y + rnorm(nrow(df), 0, 0.5)

  suppressMessages(suppressWarnings(
    utils::capture.output(
      res <- f_lmer(y + y2 ~ time + (1 | subj), data = df,
                    subset = df$time != "1",
                    output_type = "default",
                    norm_plots = FALSE, intro_text = FALSE,
                    open_generated_files = FALSE),
      file = nullfile()
    )
  ))
  expect_equal(nobs(res[["y"]]$model),  60L)
  expect_equal(nobs(res[["y2"]]$model), 60L)
})

# ===========================================================================
# 8. Contrasts forwarded
# ===========================================================================

test_that("contrasts argument flows through to the fitted model", {
  skip_on_cran()
  skip_if_no_lme4()
  df  <- make_factor_rm_data()
  suppressMessages(suppressWarnings(
    utils::capture.output(
      res <- f_lmer(y ~ trt + (1 | subj), data = df,
                    contrasts = list(trt = "contr.sum"),
                    output_type = "default",
                    norm_plots = FALSE, intro_text = FALSE,
                    open_generated_files = FALSE),
      file = nullfile()
    )
  ))
  m <- res[["y"]]$model
  fe_names <- rownames(summary(m)$coefficients)
  expect_true(any(grepl("^trt[0-9]", fe_names)))
})

# ===========================================================================
# 9. Input validation
# ===========================================================================

test_that("formula without random effect is rejected", {
  skip_on_cran()
  skip_if_no_lme4()
  df <- make_repeated_measures_data()
  expect_error(
    quiet_f_lmer(y ~ time, data = df),
    "random"
  )
})

test_that("invalid alpha is rejected", {
  skip_on_cran()
  skip_if_no_lme4()
  df <- make_repeated_measures_data()
  expect_error(quiet_f_lmer(y ~ time + (1 | subj), data = df, alpha = -0.1))
  expect_error(quiet_f_lmer(y ~ time + (1 | subj), data = df, alpha = 1.5))
})

test_that("invalid output_type is rejected", {
  skip_on_cran()
  skip_if_no_lme4()
  df <- make_repeated_measures_data()
  expect_error(quiet_f_lmer(y ~ time + (1 | subj), data = df,
                          output_type = "html"))
})

# ===========================================================================
# 10. safe_shapiro large-n guard
# ===========================================================================

test_that("f_lmer does not crash with n > 5000 residuals", {
  skip_on_cran()
  skip_if_no_lme4()
  set.seed(91)
  n_subj <- 800
  n_time <- 8                                 # 6400 total rows
  big <- data.frame(
    subj = factor(rep(seq_len(n_subj), each = n_time)),
    time = factor(rep(seq_len(n_time), times = n_subj)),
    y    = NA_real_
  )
  si <- rnorm(n_subj, 0, 1)[as.integer(big$subj)]
  big$y <- 10 + as.integer(big$time) * 0.2 + si +
           rnorm(nrow(big), 0, 1)

  expect_no_error(
    res <- quiet_f_lmer(y ~ time + (1 | subj), data = big)
  )
  expect_s3_class(res, "f_lmer")
})

# ===========================================================================
# 11. Singular fit handling
# ===========================================================================

test_that("singular fit is captured in the result, not thrown", {
  skip_on_cran()
  skip_if_no_lme4()
  set.seed(3)
  df <- data.frame(
    subj = factor(rep(1:4, each = 3)),
    x    = rnorm(12),
    y    = rnorm(12)
  )
  res <- tryCatch(
    quiet_f_lmer(y ~ x + (1 | subj), data = df),
    error = function(e) e
  )
  if (!inherits(res, "error")) {
    expect_true("is_singular" %in% names(res[["y"]]))
  }
})

# ===========================================================================
# 12. intro_text toggle
# ===========================================================================

test_that("intro_text = FALSE omits the intro section in rmd output", {
  skip_on_cran()
  skip_if_no_lme4()
  df  <- make_repeated_measures_data()
  res <- suppressMessages(suppressWarnings(
    f_lmer(y ~ time + (1 | subj), data = df,
           output_type = "rmd",
           intro_text = FALSE,
           norm_plots = FALSE,
           open_generated_files = FALSE)
  ))
  expect_true("rmd" %in% names(res))
  expect_false(grepl("assumption", res$rmd, ignore.case = TRUE))
})

test_that("intro_text = TRUE includes assumption text in rmd output", {
  skip_on_cran()
  skip_if_no_lme4()
  df  <- make_repeated_measures_data()
  res <- suppressMessages(suppressWarnings(
    f_lmer(y ~ time + (1 | subj), data = df,
           output_type = "rmd",
           intro_text = TRUE,
           norm_plots = FALSE,
           open_generated_files = FALSE)
  ))
  expect_true(grepl("assumption|random", res$rmd, ignore.case = TRUE))
})

# ===========================================================================
# 13. rmd output type
# ===========================================================================

test_that("rmd output_type stores the markdown string on result$rmd", {
  skip_on_cran()
  skip_if_no_lme4()
  df  <- make_repeated_measures_data()
  res <- suppressMessages(suppressWarnings(
    f_lmer(y ~ time + (1 | subj), data = df,
           output_type = "rmd",
           intro_text = FALSE,
           norm_plots = FALSE,
           open_generated_files = FALSE)
  ))
  expect_true(is.character(res$rmd))
  expect_true(nchar(res$rmd) > 0)
})

# =============================================================================
# test-f_remove_outliers.R
# =============================================================================
# Tests for f_remove_outliers().
#
# Fixtures: make_outlier_df() from helper-data.R
#           (replaces the inline make_df() from the pre-refactor file)
# =============================================================================


# ---------------------------------------------------------------------------
# 1. Core: removes correct rows via f_outliers output
# ---------------------------------------------------------------------------

test_that("removes correct rows when passed f_outliers output", {
  df       <- make_outlier_df()
  outliers <- f_outliers(df, columns = "Salary")
  clean    <- f_remove_outliers(df, outliers)

  # Injected outlier rows were row 20 (Salary = 100000, EmployeeID = E020)
  # and row 40 (Salary = 1000, EmployeeID = E040). Use EmployeeID to verify
  # removal: rownames are reset to 1..nrow(clean) by f_remove_outliers, so
  # checking rownames would always find "20" whenever nrow(clean) >= 20.
  expect_false("E020" %in% clean$EmployeeID)
  expect_false("E040" %in% clean$EmployeeID)
  expect_false(100000 %in% clean$Salary)
  expect_false(1000   %in% clean$Salary)
  expect_equal(nrow(clean), nrow(df) - nrow(outliers$output_df))
})

test_that("returned row names are sequential after removal", {
  df       <- make_outlier_df()
  outliers <- f_outliers(df, columns = "Salary")
  clean    <- f_remove_outliers(df, outliers)

  expect_equal(rownames(clean), as.character(seq_len(nrow(clean))))
})


# ---------------------------------------------------------------------------
# 2. f_outliers multi-column list output
# ---------------------------------------------------------------------------

test_that("handles multi-column f_outliers list: all flagged rows removed", {
  df       <- make_outlier_df()
  # Salary flags rows 20, 40; Age flags row 39
  outliers <- f_outliers(df, columns = c("Salary", "Age"))
  clean    <- f_remove_outliers(df, outliers)

  removed_ids <- unique(c(
    outliers$Salary$row_id,
    outliers$Age$row_id
  ))
  expect_equal(nrow(clean), nrow(df) - length(removed_ids))
})


# ---------------------------------------------------------------------------
# 3. Manual data.frame input (subset of outliers)
# ---------------------------------------------------------------------------

test_that("accepts a plain data.frame with row_id for selective removal", {
  df       <- make_outlier_df()
  outliers <- f_outliers(df, columns = "Salary")
  # Remove only the high outlier (row 20); keep the low one (row 40)
  just_high <- outliers$output_df[outliers$output_df$Salary > 90000, ]
  clean     <- f_remove_outliers(df, just_high)

  expect_equal(nrow(clean), nrow(df) - 1)
  expect_false(100000 %in% clean$Salary)  # high outlier gone
  expect_true(1000    %in% clean$Salary)  # low outlier still present
})


# ---------------------------------------------------------------------------
# 4. Manual vector of row numbers
# ---------------------------------------------------------------------------

test_that("accepts a plain numeric vector of row numbers", {
  df    <- make_outlier_df()
  clean <- f_remove_outliers(df, outliers = c(20, 40))

  expect_equal(nrow(clean), 38)
  expect_false(100000 %in% clean$Salary)
  expect_false(1000   %in% clean$Salary)
})


# ---------------------------------------------------------------------------
# 5. Custom 'by' column (e.g. EmployeeID)
# ---------------------------------------------------------------------------

test_that("removes rows by a custom id column", {
  df       <- make_outlier_df()
  outliers <- f_outliers(df, columns = "Salary", id_var = "EmployeeID")
  ids      <- outliers$output_df$EmployeeID
  clean    <- f_remove_outliers(df, outliers$output_df, by = "EmployeeID")

  expect_false(any(clean$EmployeeID %in% ids))
  expect_equal(nrow(clean), nrow(df) - length(ids))
})


# ---------------------------------------------------------------------------
# 6. No matches: returns original data unchanged
# ---------------------------------------------------------------------------

test_that("returns original data unchanged when no IDs match", {
  df    <- make_outlier_df()
  expect_message(
    clean <- f_remove_outliers(df, outliers = c(999, 1000)),
    regexp = "No outliers were removed"
  )
  expect_equal(nrow(clean), nrow(df))
})


# ---------------------------------------------------------------------------
# 7. verbose = FALSE suppresses messages
# ---------------------------------------------------------------------------

test_that("verbose = FALSE produces no messages", {
  df       <- make_outlier_df()
  outliers <- f_outliers(df, columns = "Salary")
  expect_no_message(
    f_remove_outliers(df, outliers, verbose = FALSE)
  )
})


# ---------------------------------------------------------------------------
# 8. Output class matches input class
# ---------------------------------------------------------------------------

test_that("output class matches data.frame input", {
  df       <- make_outlier_df()
  outliers <- f_outliers(df, columns = "Salary")
  clean    <- f_remove_outliers(df, outliers, verbose = FALSE)
  expect_s3_class(clean, "data.frame")
})

test_that("output class matches tibble input", {
  skip_if_not_installed("tibble")
  df       <- tibble::as_tibble(make_outlier_df())
  outliers <- f_outliers(df, columns = "Salary")
  clean    <- f_remove_outliers(df, outliers, verbose = FALSE)
  expect_s3_class(clean, "tbl_df")
})


# ---------------------------------------------------------------------------
# 9. Input validation errors
# ---------------------------------------------------------------------------

test_that("stops if data is not a data.frame", {
  expect_error(
    f_remove_outliers(list(a = 1), outliers = 1L),
    regexp = "data.frame"
  )
})

test_that("stops if 'by' column missing from outliers data.frame", {
  df        <- make_outlier_df()
  bad_input <- data.frame(NotRowId = 1:2)
  expect_error(
    f_remove_outliers(df, bad_input, by = "row_id"),
    regexp = "row_id"
  )
})

test_that("stops if custom 'by' column missing from source data", {
  df <- make_outlier_df()
  expect_error(
    f_remove_outliers(df, outliers = c(1, 2), by = "ghost_col"),
    regexp = "ghost_col"
  )
})

test_that("stops if outliers is an unsupported type", {
  df <- make_outlier_df()
  expect_error(
    f_remove_outliers(df, outliers = TRUE),
    regexp = "dataframe or a vector"
  )
})


# ---------------------------------------------------------------------------
# 10. Round-trip: f_outliers |> f_remove_outliers
# ---------------------------------------------------------------------------

test_that("round-trip: removed rows match exactly what f_outliers flagged", {
  df       <- make_outlier_df()
  outliers <- f_outliers(df, columns = "Salary")
  clean    <- f_remove_outliers(df, outliers, verbose = FALSE)

  flagged_salaries <- outliers$output_df$Salary
  expect_false(any(clean$Salary %in% flagged_salaries))
  expect_equal(nrow(clean) + nrow(outliers$output_df), nrow(df))
})

# =============================================================================
# Helper factories
# =============================================================================

# Basic numeric-only data frame
make_numeric_df <- function(n = 50) {
  set.seed(42)
  data.frame(x = rnorm(n), y = rnorm(n), z = rnorm(n))
}

# One unordered factor
make_one_factor_df <- function(n = 60) {
  df      <- make_numeric_df(n)
  df$grp  <- factor(rep(c("A", "B", "C"), length.out = n))
  df
}

# Two unordered factors
make_two_factor_df <- function(n = 60) {
  df       <- make_numeric_df(n)
  df$grp1  <- factor(rep(c("Low", "High"),          length.out = n))
  df$grp2  <- factor(rep(c("T1",  "T2",  "T3"),     length.out = n))
  df
}

# Three unordered factors (should trigger > 2 factors warning)
make_three_factor_df <- function(n = 60) {
  df       <- make_two_factor_df(n)
  df$grp3  <- factor(rep(c("P", "Q"), length.out = n))
  df
}

# Data frame with an ordered factor (should be treated as ordinal, not grouped)
make_ordered_factor_df <- function(n = 60) {
  df      <- make_numeric_df(n)
  df$severity <- ordered(rep(c("low", "med", "high"), length.out = n),
                         levels = c("low", "med", "high"))
  df
}

# Data frame with a constant (zero-variance) column
make_constant_col_df <- function(n = 40) {
  data.frame(x = rnorm(n), y = rnorm(n), constant = rep(5, n))
}

# Convenience: temp path for PNG output
png_out <- function() tempfile(fileext = "_CorPlot.png")

# =============================================================================
# 1. Input validation errors
# =============================================================================

test_that("stops on invalid output_type", {
  expect_error(
    f_corplot(make_numeric_df(), output_type = "xlsx",
              open_generated_files = FALSE),
    regexp = "output_type"
  )
})

test_that("stops when fewer than 2 numeric variables", {
  skip_on_cran()
  df <- data.frame(x   = rnorm(30),
                   grp = factor(rep("A", 30)))
  expect_error(
    f_corplot(df, output_type = "png",
              open_generated_files = FALSE),
    regexp = "two numeric"
  )
})

test_that("stops when color_factor not found in data", {
  skip_on_cran()
  expect_error(
    f_corplot(make_numeric_df(),
              color_factor = "nonexistent",
              output_type  = "png",
              open_generated_files = FALSE),
    regexp = "color factor"
  )
})

test_that("stops when shape_factor not found in data", {
  skip_on_cran()
  expect_error(
    suppressMessages(f_corplot(make_numeric_df(),
              shape_factor = "nonexistent",
              output_type  = "png",
              open_generated_files = FALSE)),
    regexp = "shape factor"
  )
})

# =============================================================================
# 2. Basic output creation
# =============================================================================

test_that("creates PNG file at specified path - numeric only", {
  skip_on_cran()
  out <- png_out()
  suppressMessages(f_corplot(make_numeric_df(),
            output_type = "png", save_as = out,
            open_generated_files = FALSE))
  expect_true(file.exists(out))
  expect_gt(file.size(out), 0L)
})

test_that("creates PNG for minimal 2-variable data frame", {
  skip_on_cran()
  out <- png_out()
  suppressMessages(f_corplot(data.frame(a = rnorm(20), b = rnorm(20)),
            output_type = "png", save_as = out,
            open_generated_files = FALSE))
  expect_true(file.exists(out))
  expect_gt(file.size(out), 0L)
})

# =============================================================================
# 3. Factor detection (auto path)
# =============================================================================

test_that("auto-detects one factor without error", {
  skip_on_cran()
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(make_one_factor_df(),
              detect_factors = TRUE,
              output_type    = "png", save_as = out,
              open_generated_files = FALSE))
  )
})

test_that("auto-detects two factors without error", {
  skip_on_cran()
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(make_two_factor_df(),
              detect_factors = TRUE,
              output_type    = "png", save_as = out,
              open_generated_files = FALSE))
  )
})

test_that("three factor data warns and still runs", {
  skip_on_cran()
  out <- png_out()

  expect_warning(
    suppressMessages(f_corplot(make_three_factor_df(),
              output_type = "png",
              save_as = out,
              open_generated_files = FALSE),
    "more than 2 factors"
  ))

  expect_true(file.exists(out))
})

test_that("warns when more than 2 factors are present", {
  skip_on_cran()
  out <- png_out()
    expect_warning(
      suppressMessages(f_corplot(make_three_factor_df(),
                output_type = "png",
                save_as = out,
                open_generated_files = FALSE),
      "more than 2 factors"
    ))
})

test_that("detect_factors = FALSE suppresses auto-detection", {
  skip_on_cran()
  out <- png_out()
  # grp is a factor but should not be picked up
  expect_no_error(
    suppressMessages(f_corplot(make_one_factor_df(),
              detect_factors = FALSE,
              output_type    = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("ordered factor is NOT used as grouping aesthetic", {
  skip_on_cran()
  out <- png_out()
  # No grouping expected - ordered factor is ordinal data
  expect_message(
    f_corplot(make_ordered_factor_df(),
              detect_factors = TRUE,
              output_type    = "png", save_as = out,
              open_generated_files = FALSE),
    "No factors"
  )
})

# =============================================================================
# 4. factor_select and factor_exclude passthrough
# =============================================================================

test_that("factor_select restricts which columns are tested for factors", {
  skip_on_cran()
  out <- png_out()
  df  <- make_two_factor_df()
  # grp2 should not become a factor; only grp1 is in select
  expect_no_error(
    suppressMessages( f_corplot(df,
              detect_factors = TRUE,
              factor_select  = "grp1",
              output_type    = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("factor_exclude prevents a column from becoming a factor", {
  skip_on_cran()
  out <- png_out()
  df  <- make_two_factor_df()
  expect_no_error(
    suppressMessages(f_corplot(df,
              detect_factors = TRUE,
              factor_exclude = "grp1",
              output_type    = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("unique_num_treshold and repeats_threshold are passed without error", {
  skip_on_cran()
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(make_numeric_df(),
              detect_factors      = TRUE,
              unique_num_treshold = 5,
              repeats_threshold   = 3,
              output_type         = "png", save_as = out,
              open_generated_files = FALSE))
  )
})

# =============================================================================
# 5. Manual factor specification
# =============================================================================

test_that("manual color_factor only (shape auto)", {
  skip_on_cran()
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(make_one_factor_df(),
              color_factor = "grp", shape_factor = "auto",
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("manual shape_factor only (color auto)", {
  skip_on_cran()
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(make_one_factor_df(),
              color_factor = "auto", shape_factor = "grp",
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("manual color + shape factor both set", {
  skip_on_cran()
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(make_two_factor_df(),
              color_factor = "grp1", shape_factor = "grp2",
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("non-factor variable is auto-converted with message", {
  skip_on_cran()
  df        <- make_numeric_df()
  df$num_grp <- rep(c(1L, 2L), length.out = 50L)
  out       <- png_out()
  expect_message(
    f_corplot(df,
              color_factor = "num_grp",
              output_type  = "png", save_as = out,
              open_generated_files = FALSE),
    "converted to a factor"
  )
})

# =============================================================================
# 6. factor_count correctly incremented for manually specified factors
#    (regression test for Bug #2 from earlier review)
# =============================================================================

test_that("legend is created when factors set manually - not silently suppressed", {
  skip_on_cran()
  out <- png_out()
  suppressMessages(f_corplot(make_two_factor_df(),
            color_factor = "grp1", shape_factor = "grp2",
            print_legend = TRUE,
            output_type  = "png", save_as = out,
            open_generated_files = FALSE))
  legend_path <- sub("_CorPlot\\.png$", "_Legend.png", out)
  expect_true(file.exists(legend_path))
  expect_gt(file.size(legend_path), 0L)
})

# =============================================================================
# 7. Legend behaviour
# =============================================================================

test_that("legend PNG created alongside corplot when two factors present", {
  skip_on_cran()
  out <- png_out()
  suppressMessages(f_corplot(make_two_factor_df(),
            color_factor = "grp1", shape_factor = "grp2",
            print_legend = TRUE,
            output_type  = "png", save_as = out,
            open_generated_files = FALSE))
  legend_path <- sub("_CorPlot\\.png$", "_Legend.png", out)
  expect_true(file.exists(legend_path))
})

test_that("no legend file when print_legend = FALSE and no ordinal_vars", {
  skip_on_cran()
  out <- png_out()
  suppressMessages(f_corplot(make_two_factor_df(),
            print_legend = FALSE,
            output_type  = "png", save_as = out,
            open_generated_files = FALSE))
  legend_path <- sub("_CorPlot\\.png$", "_Legend.png", out)
  expect_false(file.exists(legend_path))
})

test_that("no legend when no factors and no ordinal_vars", {
  skip_on_cran()
  out <- png_out()
  suppressMessages(f_corplot(make_numeric_df(),
            output_type = "png", save_as = out,
            open_generated_files = FALSE))
  legend_path <- sub("_CorPlot\\.png$", "_Legend.png", out)
  expect_false(file.exists(legend_path))
})

test_that("legend created with no factors when ordinal_vars specified", {
  skip_on_cran()
  out <- png_out()
  suppressMessages(f_corplot(make_numeric_df(),
            ordinal_vars = "x",
            output_type  = "png", save_as = out,
            open_generated_files = FALSE))
  legend_path <- sub("_CorPlot\\.png$", "_Legend.png", out)
  expect_true(file.exists(legend_path))
})

# =============================================================================
# 8. ordinal_vars
# =============================================================================

test_that("ordinal_vars: numeric column - annotation only, no crash", {
  skip_on_cran()
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(make_numeric_df(),
              ordinal_vars = "x",
              output_type  = "png", save_as = out,
              open_generated_files = FALSE))
  )
})


test_that("ordinal_vars: ordered factor coerced to integer", {
  skip_on_cran()
  out <- png_out()
  expect_message(
    f_corplot(
      make_ordered_factor_df(),
      ordinal_vars = "severity",
      output_type  = "png",
      save_as = out,
      open_generated_files = FALSE
    ),
    "coerced to integer"
  )
  expect_true(file.exists(out))
})

test_that("ordinal_vars: non-existent column warns and is ignored", {
  skip_on_cran()
  out <- png_out()
  expect_warning(
    suppressMessages(f_corplot(make_numeric_df(),
              ordinal_vars = "does_not_exist",
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)),
   "not found"
  )
})

test_that("ordinal_vars: multiple ordinal columns", {
  skip_on_cran()
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(make_numeric_df(),
              ordinal_vars = c("x", "y"),
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

# =============================================================================
# 9. fancy_names
# =============================================================================

test_that("fancy_names renames columns without error", {
  skip_on_cran()
  out   <- png_out()
  fancy <- c(x = "Variable X", y = "Variable Y")
  expect_no_error(
    suppressMessages( f_corplot(make_numeric_df(),
              fancy_names  = fancy,
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("fancy_names + manual factor: renamed factor resolves correctly", {
  skip_on_cran()
  out   <- png_out()
  fancy <- c(grp = "Group Label")
  expect_no_error(
    suppressMessages(f_corplot(make_one_factor_df(),
              fancy_names  = fancy,
              color_factor = "Group Label",
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("fancy_names + ordinal_vars: ordinal name renamed correctly", {
  skip_on_cran()
  out   <- png_out()
  fancy <- c(x = "Trait X")
  expect_no_error(
    suppressMessages(f_corplot(make_numeric_df(),
              fancy_names  = fancy,
              ordinal_vars = "Trait X",
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

# =============================================================================
# 10. Missing data and edge cases
# =============================================================================

test_that("handles NAs in numeric columns", {
  skip_on_cran()
  df      <- make_numeric_df(50)
  df$x[c(3, 17, 33)] <- NA_real_
  out     <- png_out()
  expect_no_error(
    suppressMessages( f_corplot(df, output_type = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("handles constant (zero-variance) column - shows NA, no crash", {
  skip_on_cran()
  out <- png_out()
  expect_no_error(
    suppressMessages( f_corplot(make_constant_col_df(),
              output_type = "png", save_as = out,
              open_generated_files = FALSE)
  ))
  expect_true(file.exists(out))
})

test_that("handles NAs in factor column", {
  skip_on_cran()
  df      <- make_one_factor_df()
  df$grp[c(1, 5)] <- NA
  out     <- png_out()
  expect_no_error(
    suppressMessages( f_corplot(df,
              color_factor = "grp",
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

# =============================================================================
# 11. Large numbers of factor levels (extended shape sequence)
# =============================================================================

test_that("shape factor with exactly 5 levels", {
  skip_on_cran()
  df   <- make_numeric_df(50)
  df$g <- factor(rep(letters[1:5], 10))
  out  <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(df, shape_factor = "g",
              output_type = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("shape factor with 6 levels triggers extended shape list", {
  skip_on_cran()
  df   <- make_numeric_df(60)
  df$g <- factor(rep(letters[1:6], 10))
  out  <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(df, shape_factor = "g",
              output_type = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("shape factor with 8 levels - well beyond default pch range", {
  skip_on_cran()
  df   <- make_numeric_df(80)
  df$g <- factor(rep(letters[1:8], 10))
  out  <- png_out()
  expect_no_error(
    suppressMessages( f_corplot(df, shape_factor = "g",
              output_type = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

# =============================================================================
# 12. Output file integrity
# =============================================================================

test_that("output file is non-empty PNG with correct path", {
  skip_on_cran()
  out <- png_out()
  suppressMessages(f_corplot(make_two_factor_df(),
            color_factor = "grp1",
            output_type  = "png", save_as = out,
            open_generated_files = FALSE))
  expect_true(file.exists(out))
  expect_gt(file.size(out), 1000L)   # at minimum a few KB
})

test_that("legend file is non-empty PNG", {
  skip_on_cran()
  out <- png_out()
  suppressMessages(f_corplot(make_two_factor_df(),
            color_factor = "grp1", shape_factor = "grp2",
            print_legend = TRUE,
            output_type  = "png", save_as = out,
            open_generated_files = FALSE))
  legend_path <- sub("_CorPlot\\.png$", "_Legend.png", out)
  expect_gt(file.size(legend_path), 1000L)
})

# =============================================================================
# 13. Stress tests - combinations
# =============================================================================

test_that("STRESS: two factors + fancy_names + ordinal_vars + custom dims", {
  skip_on_cran()
  df    <- make_two_factor_df()
  fancy <- c(x = "Trait X", y = "Trait Y", z = "Trait Z")
  out   <- png_out()
  expect_no_error(
    suppressMessages( f_corplot(df,
              color_factor = "grp1",
              shape_factor = "grp2",
              fancy_names  = fancy,
              ordinal_vars = "Trait X",
              print_legend = TRUE,
              width        = 20,
              height       = 20,
              res          = 150,
              pointsize    = 10,
              output_type  = "png", save_as = out,
              open_generated_files = FALSE))
  )
  expect_true(file.exists(out))
  legend_path <- sub("_CorPlot\\.png$", "_Legend.png", out)
  expect_true(file.exists(legend_path))
})

test_that("STRESS: detect_factors=FALSE + manual factors + NAs + fancy_names", {
  skip_on_cran()
  df        <- make_two_factor_df()
  df$x[1:5] <- NA_real_
  fancy     <- c(x = "X Axis", grp1 = "Primary Group")
  out       <- png_out()
  expect_no_error(
    suppressMessages( f_corplot(df,
              detect_factors = FALSE,
              color_factor   = "Primary Group",
              shape_factor   = "grp2",
              fancy_names    = fancy,
              print_legend   = TRUE,
              output_type    = "png", save_as = out,
              open_generated_files = FALSE)
  ))
})

test_that("STRESS: >5 shape levels + color factor + ordinal_vars + NAs", {
  skip_on_cran()
  set.seed(1)
  n   <- 70
  df  <- data.frame(
    x   = c(rnorm(35), NA_real_, rnorm(34)),
    y   = rnorm(n),
    z   = rnorm(n),
    col = factor(rep(c("Red", "Blue"), length.out = n)),
    shp = factor(rep(letters[1:7], length.out = n))
  )
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(df,
              color_factor = "col",
              shape_factor = "shp",
              ordinal_vars = "z",
              print_legend = TRUE,
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
  legend_path <- sub("_CorPlot\\.png$", "_Legend.png", out)
  expect_true(file.exists(legend_path))
})

test_that("STRESS: iris - auto factor, fancy names, ordinal_vars, legend", {
  skip_on_cran()
  data(iris)
  fancy <- c(Sepal.Length = "Sepal Length (cm)",
             Petal.Length = "Petal Length (cm)")
  out   <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(iris,
              fancy_names  = fancy,
              ordinal_vars = "Petal.Width",
              print_legend = TRUE,
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
  expect_true(file.exists(out))
})

test_that("STRESS: mtcars subset - manual factors, factor_exclude, no legend", {
  skip_on_cran()
  data(mtcars)
  sub <- subset(mtcars, select = -c(am, qsec, vs))
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(sub,
              color_factor   = "gear",
              shape_factor   = "cyl",
              factor_exclude = "carb",
              print_legend   = FALSE,
              res            = 150,
              output_type    = "png", save_as = out,
              open_generated_files = FALSE)
  ))
  expect_true(file.exists(out))
})

test_that("STRESS: all factor_select / exclude / threshold params combined", {
  skip_on_cran()
  set.seed(7)
  n  <- 80
  df <- data.frame(
    a    = rnorm(n),
    b    = rnorm(n),
    c    = rnorm(n),
    cat1 = factor(rep(c("X", "Y"), length.out = n)),
    cat2 = factor(rep(c("P", "Q", "R"), length.out = n)),
    num  = rep(1:4, length.out = n)   # low unique count - could become factor
  )
  out <- png_out()
  expect_no_error(
    suppressMessages(f_corplot(df,
              detect_factors      = TRUE,
              factor_select       = c("cat1", "cat2"),
              factor_exclude      = NULL,
              unique_num_treshold = 6,
              repeats_threshold   = 2,
              ordinal_vars        = "num",
              print_legend        = TRUE,
              output_type         = "png", save_as = out,
              open_generated_files = FALSE)
  ))
  expect_true(file.exists(out))
})

test_that("STRESS: constant column + ordinal + factor + fancy_names all together", {
  skip_on_cran()
  set.seed(99)
  n  <- 50
  df <- data.frame(
    x    = rnorm(n),
    y    = rnorm(n),
    flat = rep(3.14, n),                              # zero variance
    grp  = factor(rep(c("A", "B"), length.out = n))
  )
  fancy <- c(x = "Variable X", flat = "Constant")
  out   <- png_out()
  expect_no_error(
    suppressMessages( f_corplot(df,
              color_factor = "grp",
              fancy_names  = fancy,
              ordinal_vars = "Variable X",
              print_legend = TRUE,
              output_type  = "png", save_as = out,
              open_generated_files = FALSE)
  ))
  expect_true(file.exists(out))
})

test_that("legend colour order matches label order for two factors", {
  skip_on_cran()
  # 2 colors x 3 shapes = 6 entries
  # outer() produces: col1-s1, col2-s1, col1-s2, col2-s2, col1-s3, col2-s3
  # legend_col must cycle colors fast, not slow
  df  <- make_two_factor_df()
  out <- png_out()
  suppressMessages(f_corplot(df, color_factor = "grp1", shape_factor = "grp2",
            print_legend = TRUE, output_type = "png",
            save_as = out, open_generated_files = FALSE))
  legend_path <- sub("_CorPlot\\.png$", "_Legend.png", out)
  # If the bug is present the file still renders - we can only verify it exists;
  # visual inspection confirms colour/label alignment
  expect_true(file.exists(legend_path))
})

# =============================================================================
# test-f_outliers.R
# =============================================================================
# Tests for f_outliers() including S3 dispatch (numeric, integer, formula,
# data.frame methods).
#
# Fixtures:
#   make_outlier_df()  from helper-data.R
#   make_outlier_vec() defined locally (this file uses a different seed and
#                      length than the shared helper, so it stays local)
# =============================================================================

df <- make_outlier_df()

# ---------------------------------------------------------------------------
# 1. Basic: single column, no groups
# ---------------------------------------------------------------------------
test_that("detects outlier in single column without groups", {
  result <- f_outliers(df, columns = "Salary")
  expect_s3_class(result, "f_outliers")
  expect_true("output_df" %in% names(result))
  out_df <- result$output_df
  expect_true(nrow(out_df) >= 1)
  expect_true(all(c("row_id", "Salary") %in% names(out_df)))
  # The injected outliers should appear
  expect_true(20 %in% out_df$row_id || 40 %in% out_df$row_id)
})

# ---------------------------------------------------------------------------
# 2. Grouped detection
# ---------------------------------------------------------------------------
test_that("grouped detection finds within-group outliers", {
  result <- f_outliers(df, columns = "Salary", group_vars = "Team")
  out_df <- result$output_df
  # Row 20 is outlier in group A, row 40 in group B
  expect_true(20 %in% out_df$row_id)
  expect_true(40 %in% out_df$row_id)
})

test_that("multi-group interaction works", {
  result <- f_outliers(df, columns = "Salary", group_vars = c("Team", "Department"))
  expect_s3_class(result, "f_outliers")
  expect_true(nrow(result$output_df) >= 1)
})

# ---------------------------------------------------------------------------
# 3. Formula interface
# ---------------------------------------------------------------------------
test_that("formula interface matches direct call", {
  res_formula <- f_outliers(Salary ~ Team, data = df)
  res_direct  <- f_outliers(df, columns = "Salary", group_vars = "Team")
  expect_equal(res_formula$output_df$row_id,
               res_direct$output_df$row_id)
})

test_that("formula with multiple LHS columns works", {
  result <- f_outliers(Salary + Age ~ Team, data = df)
  expect_true("Salary" %in% names(result))
  expect_true("Age"    %in% names(result))
})

test_that("one-sided formula throws informative error", {
  expect_error(f_outliers(~ Team, data = df),
               regexp = "two-sided")
})

# ---------------------------------------------------------------------------
# 4. Multiple columns: no-outlier in first column does not suppress second
# ---------------------------------------------------------------------------
test_that("no-outlier in first column does not suppress second column results", {
  df_test <- df
  df_test$CleanCol <- rep(1:2, 20)
  result <- f_outliers(df_test, columns = c("CleanCol", "Age"))
  expect_false(is.null(result))
  expect_true("Age" %in% names(result))
  expect_true(39 %in% result$Age$row_id)
})

# ---------------------------------------------------------------------------
# 5. No outliers at all: returns NULL
# ---------------------------------------------------------------------------
test_that("returns NULL with message when no outliers exist", {
  clean_df <- data.frame(x = 1:20)
  expect_message(
    result <- f_outliers(clean_df, columns = "x"),
    "No outliers"
  )
  expect_null(result)
})

# ---------------------------------------------------------------------------
# 6. coef parameter changes sensitivity
# ---------------------------------------------------------------------------
test_that("higher coef finds fewer outliers than lower coef", {
  res_loose  <- f_outliers(df, columns = "Salary", coef = 1.5)
  res_strict <- f_outliers(df, columns = "Salary", coef = 3.0)
  n_loose    <- if (is.null(res_loose))  0L else nrow(res_loose$output_df)
  n_strict   <- if (is.null(res_strict)) 0L else nrow(res_strict$output_df)
  expect_gte(n_loose, n_strict)
})

# ---------------------------------------------------------------------------
# 7. id_var: placed first, output sorted by it
# ---------------------------------------------------------------------------
test_that("id_var appears as first column and output is sorted by it", {
  result <- f_outliers(df, columns = "Salary", id_var = "EmployeeID")
  out_df <- result$output_df
  expect_equal(names(out_df)[1], "EmployeeID")
  expect_equal(out_df$EmployeeID, sort(out_df$EmployeeID))
})

# ---------------------------------------------------------------------------
# 8. Input validation
# ---------------------------------------------------------------------------
test_that("non-existent column throws error", {
  expect_error(f_outliers(df, columns = "NonExistent"),
               regexp = "not found")
})

test_that("non-existent group_var throws error", {
  expect_error(f_outliers(df, columns = "Salary", group_vars = "Ghost"),
               regexp = "not found")
})

test_that("non-numeric column throws error", {
  expect_error(f_outliers(df, columns = "Team"),
               regexp = "[Nn]umeric|[Nn]on-numeric")
})

test_that("missing x with no data arg throws error", {
  expect_error(f_outliers(), regexp = "missing")
})

# ---------------------------------------------------------------------------
# 9. row_id maps back to original data
# ---------------------------------------------------------------------------
test_that("row_id in output maps exactly to original rows", {
  result <- f_outliers(df, columns = "Salary")
  out_df <- result$output_df
  for (i in seq_len(nrow(out_df))) {
    rid <- out_df$row_id[i]
    expect_equal(out_df$Salary[i], df$Salary[rid])
  }
})

# ---------------------------------------------------------------------------
# 10. Edge cases
# ---------------------------------------------------------------------------
test_that("handles NA values without error", {
  df_na <- df
  df_na$Salary[5] <- NA
  expect_no_error(f_outliers(df_na, columns = "Salary"))
})

test_that("single-row group does not crash", {
  df_small <- data.frame(
    G   = c(rep("A", 10), "B"),
    Val = c(rnorm(9, 50, 1), 200, 50)
  )
  expect_no_error(f_outliers(df_small, columns = "Val", group_vars = "G"))
})

test_that("all-NA column returns NULL gracefully", {
  df_allna <- data.frame(x = rep(NA_real_, 10))
  expect_no_error(
    res <- f_outliers(df_allna, columns = "x")
  )
  expect_null(res)
})


# =============================================================================
# S3 dispatch (numeric, integer, formula, data.frame methods)
# =============================================================================

# Local fixture: uses seed 5 and length 32, distinct from the shared
# helper-data.R version (seed 42, length 30).
make_outlier_vec_local <- function() {
  set.seed(5)
  c(rnorm(30, 10, 1), 25, 26)    # two clear high outliers
}

# ---------------------------------------------------------------------------
# A, B. numeric and integer dispatch
# ---------------------------------------------------------------------------
test_that("numeric vector dispatches to f_outliers.numeric", {
  v <- make_outlier_vec_local()
  expect_no_error(res <- suppressMessages(f_outliers(v)))
  expect_true(is.list(res) || is.data.frame(res) ||
              inherits(res, "f_outliers"))
})

test_that("integer vector dispatches to f_outliers.integer", {
  v <- as.integer(c(1, 2, 3, 4, 5, 100))
  expect_no_error(res <- suppressMessages(f_outliers(v)))
})

# ---------------------------------------------------------------------------
# C. formula method
# ---------------------------------------------------------------------------
test_that("formula interface works with a data frame", {
  df_loc <- data.frame(
    y = make_outlier_vec_local(),
    g = factor(rep(c("A", "B"), length.out = 32))
  )
  expect_no_error(
    res <- suppressMessages(f_outliers(y ~ g, data = df_loc))
  )
})

# ---------------------------------------------------------------------------
# D. data.frame method with x as the dispatch argument
# ---------------------------------------------------------------------------
test_that("data.frame interface uses x as first argument", {
  df_loc <- data.frame(
    v1 = make_outlier_vec_local(),
    v2 = rnorm(32)
  )
  expect_no_error(
    res <- suppressMessages(f_outliers(df_loc, columns = c("v1", "v2")))
  )
})

test_that("data.frame interface via named x also works", {
  df_loc <- data.frame(v1 = make_outlier_vec_local())
  expect_no_error(
    res <- suppressMessages(f_outliers(x = df_loc, columns = "v1"))
  )
})

# =============================================================================
# test-f_aov.R
# Comprehensive testthat tests for f_aov()
#
# Run with: testthat::test_file("tests/testthat/test-f_aov.R")
#           or devtools::test() from the package root
#
# CRAN strategy
# -------------
# Most tests run on CRAN because the file completes in ~12s on a recent
# laptop (well within CRAN's per-package budget even at 3x slower hardware).
# The tests marked with skip_on_cran() are ONLY those that are genuinely
# expensive or dependency-sensitive:
#
#   * bestNormalize transformation (section 4): iterates over many
#     candidate transformations with cross-validation; unpredictable
#     runtime and depends on the bestNormalize package being installed.
#   * Shapiro-Wilk n > 5000 guard (section 14): generates 6000 rows and
#     runs the full pipeline on them.
#
# Everything else (structure, input validation, ANCOVA, NAs, interactions,
# rmd output, restoration, etc.) is fast and gets CRAN coverage.
#
# Test organisation (each section is independent):
#   1.  Return object structure
#   2.  Single response, no transformation
#   3.  Multiple response variables
#   4.  Transformation options (boxcox / bestnormalize / FALSE)
#   5.  force_transformation
#   6.  Alpha and adjust parameters
#   7.  ANCOVA mode
#   8.  Missing values (NAs)
#   9.  Interaction models
#  10.  rmd output type
#  11.  norm_plots = FALSE
#  12.  Input validation / error handling
#  13.  intro_text = FALSE
#  14.  Large n (Shapiro-Wilk guard)
#  15.  Small cell sizes (Levene power warning)
#  16.  Non-significant ANOVA (letters should be 'ns')
#  17.  Post-hoc table content and column names
#  18.  par() and options() are restored on exit
#  19.  Last-resort message (seed 335, empirically robust trigger)
#  20.  Multiple-response NOTE
#  21.  Output text correctness
#  22.  subset / na.action / weights pass-through
#  23.  safe_shapiro() integration
# =============================================================================

# ---------------------------------------------------------------------------
# Helpers used across multiple tests
# ---------------------------------------------------------------------------
# make_normal_data, make_skewed_data, make_two_factor_data come from
# helper-data.R. quiet_f_aov comes from helper-quiet.R.
#
# quiet_f_aov_typed is file-local: it intentionally does NOT use
# capture.output because nested capture.output calls redirect stdout
# before f_aov's internal markdown capture runs, leaving cached_markdown
# empty. For non-default output_type, f_aov does not print to console
# anyway, so suppressMessages alone is sufficient.

quiet_f_aov_typed <- function(output_type, ...) {
  suppressMessages(
    f_aov(..., output_type = output_type,
          norm_plots = FALSE, intro_text = FALSE,
          open_generated_files = FALSE)
  )
}

# =============================================================================
# 1. Return object structure
# =============================================================================
df  <- make_normal_data()
out <- quiet_f_aov(y ~ grp, data = df)

test_that("f_aov return object structure: returns an object with class 'f_aov'", {
  expect_s3_class(out, "f_aov")
})

test_that("f_aov return object structure: is a named list with one element per response variable", {
  expect_true(is.list(out))
  expect_true("y" %in% names(out))
})

test_that("f_aov return object structure: each response element contains mandatory keys", {
  keys <- names(out[["y"]])
  mandatory <- c("aov_test", "aov_summary", "aov_call",
                 "Levene_test_on_res", "shapiro_test_residuals",
                 "adt_test_residuals", "normality_plots",
                 "alpha", "Response_Transformed",
                 "post_hoc_summary_table", "summary_table")
  for (k in mandatory) {
    expect_true(k %in% keys,
                label = paste("mandatory key present:", k))
  }
})

test_that("f_aov return object structure: aov_test is an aov object", {
  expect_s3_class(out[["y"]][["aov_test"]], "aov")
})

test_that("f_aov return object structure: aov_call is a character string matching the formula", {
  expect_type(out[["y"]][["aov_call"]], "character")
  expect_match(out[["y"]][["aov_call"]], "y")
  expect_match(out[["y"]][["aov_call"]], "grp")
})

test_that("f_aov return object structure: shapiro_test_residuals is an htest object with p.value", {
  sw <- out[["y"]][["shapiro_test_residuals"]]
  expect_true(!is.null(sw$p.value))
  expect_true(sw$p.value >= 0 && sw$p.value <= 1)
})

test_that("f_aov return object structure: Levene_test_on_res has a p column", {
  lev <- out[["y"]][["Levene_test_on_res"]]
  expect_true("p" %in% names(lev))
})

test_that("f_aov return object structure: alpha stored in output_list matches the argument passed", {
  expect_equal(out[["y"]][["alpha"]], 0.05)
  out2 <- quiet_f_aov(y ~ grp, data = df, alpha = 0.01)
  expect_equal(out2[["y"]][["alpha"]], 0.01)
})

test_that("f_aov return object structure: post_hoc_summary_table is a data frame", {
  expect_s3_class(out[["y"]][["post_hoc_summary_table"]], "data.frame")
})

test_that("f_aov return object structure: post_hoc_summary_table contains 'Letter' column", {
  expect_true("Letter" %in% names(out[["y"]][["post_hoc_summary_table"]]))
})

test_that("f_aov return object structure: normality_plots path points to a real file", {
  expect_true(file.exists(out[["y"]][["normality_plots"]]))
})

# =============================================================================
# 2. Single response, no transformation  (normal data)
# =============================================================================
df  <- make_normal_data()
out <- quiet_f_aov(y ~ grp, data = df, transformation = FALSE)

test_that("f_aov single response no transformation: Response_Transformed is FALSE when residuals are normal", {
  expect_false(out[["y"]][["Response_Transformed"]])
})

test_that("f_aov single response no transformation: transformed keys are absent when no transformation occurred", {
  keys <- names(out[["y"]])
  expect_false("transformed_aov_test" %in% keys)
  expect_false("boxcox" %in% keys)
  expect_false("bestNormalize" %in% keys)
})

test_that("f_aov single response no transformation: ANOVA F-statistic is positive", {
  f_val <- out[["y"]][["aov_summary"]][[1]][["F value"]]
  expect_true(all(f_val[!is.na(f_val)] > 0))
})

test_that("f_aov single response no transformation: post_hoc_summary_table has one row per group level", {
  ph <- out[["y"]][["post_hoc_summary_table"]]
  expect_equal(nrow(ph), 3L)  # 3 groups: A, B, C
})

# =============================================================================
# 3. Multiple response variables
# =============================================================================
df  <- make_normal_data()
df$y2 <- df$y * 1.5 + rnorm(nrow(df), 0, 0.5)

out <- quiet_f_aov(y + y2 ~ grp, data = df, transformation = FALSE)

test_that("f_aov multiple response variables: output list contains both response names", {
  expect_true("y"  %in% names(out))
  expect_true("y2" %in% names(out))
})

test_that("f_aov multiple response variables: each response has its own aov_test", {
  expect_s3_class(out[["y"]][["aov_test"]],  "aov")
  expect_s3_class(out[["y2"]][["aov_test"]], "aov")
})

test_that("f_aov multiple response variables: aov_call for y2 references y2 not y", {
  expect_match(out[["y2"]][["aov_call"]], "y2")
  expect_false(grepl("^y ~", out[["y2"]][["aov_call"]]))
})

test_that("f_aov multiple response variables: alpha is stored consistently across all responses", {
  expect_equal(out[["y"]][["alpha"]],  0.05)
  expect_equal(out[["y2"]][["alpha"]], 0.05)
})

test_that("f_aov multiple response variables: three response variables all stored independently", {
  df$y3 <- rnorm(nrow(df), 5, 1)
  out3  <- quiet_f_aov(y + y2 + y3 ~ grp, data = df, transformation = FALSE)
  expect_setequal(setdiff(names(out3), "rmd"), c("y", "y2", "y3"))
})

# =============================================================================
# 4. Transformation options
# =============================================================================
df_skew <- make_skewed_data()

test_that("f_aov transformation options: transformation = TRUE triggers boxcox when residuals are non-normal", {
  out <- quiet_f_aov(y ~ grp, data = df_skew, transformation = TRUE)
  transformed <- out[["y"]][["Response_Transformed"]]
  if (transformed) {
    expect_true("boxcox" %in% names(out[["y"]]) ||
                  "bestNormalize" %in% names(out[["y"]]))
  }
})

test_that("f_aov transformation options: transformation = 'boxcox' stores boxcox key", {
  out <- quiet_f_aov(y ~ grp, data = df_skew, transformation = "boxcox")
  if (out[["y"]][["Response_Transformed"]]) {
    expect_true("boxcox" %in% names(out[["y"]]))
  }
})

test_that("f_aov transformation options: transformation = FALSE never transforms, no transformed keys", {
  out <- quiet_f_aov(y ~ grp, data = df_skew, transformation = FALSE)
  expect_false(out[["y"]][["Response_Transformed"]])
  expect_false("transformed_aov_test" %in% names(out[["y"]]))
})

test_that("f_aov transformation options: transformation = 'bestnormalize' stores bestNormalize key when triggered", {
  # bestNormalize iterates over many candidate transformations with CV;
  # it is the slowest transformation option and has the heaviest deps.
  skip_on_cran()
  set.seed(1)
  out <- quiet_f_aov(y ~ grp, data = df_skew, transformation = "bestnormalize")
  if (out[["y"]][["Response_Transformed"]]) {
    expect_true("bestNormalize" %in% names(out[["y"]]))
  }
})

test_that("f_aov transformation options: when transformed, transformed_shapiro_test is present", {
  out <- quiet_f_aov(y ~ grp, data = df_skew, transformation = TRUE)
  if (out[["y"]][["Response_Transformed"]]) {
    expect_true("transformed_shapiro_test" %in% names(out[["y"]]))
    expect_true("transformed_levene_test"  %in% names(out[["y"]]))
    expect_true("transformed_adt_test"     %in% names(out[["y"]]))
  }
})

test_that("f_aov transformation options: partial string 'box' is accepted via pmatch", {
  expect_no_error(
    quiet_f_aov(y ~ grp, data = df_skew, transformation = "box")
  )
})

test_that("f_aov transformation options: invalid transformation string throws an error", {
  expect_error(
    quiet_f_aov(y ~ grp, data = df_skew, transformation = "logtransform"),
    regexp = "Invalid transformation"
  )
})

# =============================================================================
# 5. force_transformation
# =============================================================================
df <- make_normal_data()  # normal data: would not ordinarily be transformed

test_that("f_aov force_transformation: forces transformation on named response even when residuals are normal", {
  out <- quiet_f_aov(y ~ grp, data = df,
                     transformation       = TRUE,
                     force_transformation = "y")
  expect_true(out[["y"]][["Response_Transformed"]])
})

test_that("f_aov force_transformation: does not affect other responses not listed in force_transformation", {
  # Use n=200 per group so Shapiro-Wilk reliably passes for a truly normal y2.
  # Small samples can reject normality by chance; this test is about the
  # force_transformation logic, not about Shapiro-Wilk sensitivity.
  set.seed(77)
  n_big <- 200
  df_big <- data.frame(
    y   = c(rnorm(n_big, 10, 1), rnorm(n_big, 13, 1), rnorm(n_big, 16, 1)),
    y2  = rnorm(n_big * 3, mean = 5, sd = 0.5),   # clearly normal, large n
    grp = factor(rep(c("A", "B", "C"), each = n_big))
  )
  out <- quiet_f_aov(y + y2 ~ grp, data = df_big,
                     transformation       = TRUE,
                     force_transformation = "y")
  # y is in force_transformation: must be transformed regardless
  expect_true(out[["y"]][["Response_Transformed"]])
  # y2 is NOT in force_transformation and is robustly normal: not transformed
  expect_false(out[["y2"]][["Response_Transformed"]])
})

# =============================================================================
# 6. Alpha and adjust parameters
# =============================================================================
df <- make_normal_data()

test_that("f_aov alpha and adjust parameters: alpha = 0.01 is stored correctly in each response", {
  out <- quiet_f_aov(y ~ grp, data = df, alpha = 0.01,
                     transformation = FALSE)
  expect_equal(out[["y"]][["alpha"]], 0.01)
})

test_that("f_aov alpha and adjust parameters: alpha = 0.10 affects significance evaluation", {
  out_05 <- quiet_f_aov(y ~ grp, data = df, alpha = 0.05, transformation = FALSE)
  out_10 <- quiet_f_aov(y ~ grp, data = df, alpha = 0.10, transformation = FALSE)
  expect_s3_class(out_05[["y"]][["post_hoc_summary_table"]], "data.frame")
  expect_s3_class(out_10[["y"]][["post_hoc_summary_table"]], "data.frame")
})

# =============================================================================
# 7. ANCOVA mode (ANCOVA = TRUE prevents factor coercion)
# =============================================================================
set.seed(11)
df_ancova <- data.frame(
  y         = rnorm(60, 10, 2),
  group     = factor(rep(c("ctrl", "trt"), each = 30)),
  covariate = runif(60, 0, 10)
)

test_that("f_aov ANCOVA mode: ANCOVA = TRUE runs without error for a model with covariate", {
  expect_no_error(
    quiet_f_aov(y ~ group + covariate,
                data   = df_ancova,
                ANCOVA = TRUE,
                transformation = FALSE)
  )
})

test_that("f_aov ANCOVA mode: ANCOVA = TRUE keeps covariate numeric (not coerced to factor)", {
  out <- quiet_f_aov(y ~ group + covariate,
                     data   = df_ancova,
                     ANCOVA = TRUE,
                     transformation = FALSE)
  expect_s3_class(out[["y"]][["aov_test"]], "aov")
})

# =============================================================================
# 8. Missing values (NAs)
# =============================================================================
df      <- make_normal_data()
df$y[c(3, 7, 22)] <- NA   # introduce 3 NAs

test_that("f_aov missing value handling: runs successfully with NAs present", {
  expect_no_error(
    quiet_f_aov(y ~ grp, data = df, transformation = FALSE)
  )
})

test_that("f_aov missing value handling: returns a valid aov object with fewer rows than original data", {
  out      <- quiet_f_aov(y ~ grp, data = df, transformation = FALSE)
  n_resid  <- length(residuals(out[["y"]][["aov_test"]]))
  expect_equal(n_resid, sum(!is.na(df$y)))
})

test_that("f_aov missing value handling: post_hoc_summary_table is still a data frame when NAs are present", {
  out <- quiet_f_aov(y ~ grp, data = df, transformation = FALSE)
  expect_s3_class(out[["y"]][["post_hoc_summary_table"]], "data.frame")
})

# =============================================================================
# 9. Interaction models
# =============================================================================
df2 <- make_two_factor_data()

test_that("f_aov interaction models: two-factor additive model runs without error", {
  expect_no_error(
    quiet_f_aov(y ~ A + B, data = df2, transformation = FALSE)
  )
})

test_that("f_aov interaction models: two-factor interaction model runs without error", {
  expect_no_error(
    quiet_f_aov(y ~ A * B, data = df2, transformation = FALSE)
  )
})

test_that("f_aov interaction models: aov_summary has three terms for A * B model (A, B, A:B, Residuals)", {
  out    <- quiet_f_aov(y ~ A * B, data = df2, transformation = FALSE)
  n_rows <- nrow(out[["y"]][["aov_summary"]][[1]])
  expect_equal(n_rows, 4L)   # A, B, A:B, Residuals
})

test_that("f_aov interaction models: emmeans runs without 'no variable in reference grid' error for A*B", {
  # Regression guard for a previously fixed bug
  expect_no_error(
    quiet_f_aov_typed("rmd", y ~ A * B, data = df2, transformation = FALSE)
  )
  out <- quiet_f_aov(y ~ A * B, data = df2, transformation = FALSE)
  expect_s3_class(out[["y"]][["post_hoc_summary_table"]], "data.frame")
})

# =============================================================================
# 10. output_type = "rmd"
# =============================================================================
df <- make_normal_data()

test_that("f_aov output_type rmd: rmd output stores a character string in output_list$rmd", {
  out <- quiet_f_aov_typed("rmd", y ~ grp, data = df,
                           transformation = FALSE)
  expect_true("rmd" %in% names(out))        # rmd is top-level key
  expect_type(out[["rmd"]], "character")
  expect_gt(nchar(out[["rmd"]]), 100L)
})

test_that("f_aov output_type rmd: rmd string contains expected section markers", {
  out <- quiet_f_aov_typed("rmd", y ~ grp, data = df,
                           transformation = FALSE)
  expect_match(out[["rmd"]], "Analysis of:")
  expect_match(out[["rmd"]], "ANOVA Summary")
})

# =============================================================================
# 11. norm_plots = FALSE
# =============================================================================
df <- make_normal_data()

test_that("f_aov norm_plots = FALSE: runs without error when norm_plots is FALSE", {
  expect_no_error(
    suppressMessages(
      f_aov(y ~ grp, data = df,
            norm_plots = FALSE, transformation = FALSE,
            output_type = "default", open_generated_files = FALSE)
    )
  )
})

test_that("f_aov norm_plots = FALSE: normality_plots path is still stored (file written even if not cat'd)", {
  out <- suppressMessages(
    f_aov(y ~ grp, data = df,
          norm_plots = FALSE, transformation = FALSE,
          output_type = "default", open_generated_files = FALSE)
  )
  expect_true("normality_plots" %in% names(out[["y"]]))
})

# =============================================================================
# 12. Input validation / error handling
# =============================================================================
df <- make_normal_data()

test_that("f_aov input validation: errors when response variable is not in data", {
  expect_error(
    quiet_f_aov(z ~ grp, data = df),
    regexp = "not found in the data"
  )
})

test_that("f_aov input validation: errors when predictor variable is not in data", {
  expect_error(
    quiet_f_aov(y ~ treatment, data = df),
    regexp = "not found in the data"
  )
})

test_that("f_aov input validation: errors when response variable is not numeric", {
  df$y_chr <- as.character(df$y)
  expect_error(
    quiet_f_aov(y_chr ~ grp, data = df),
    regexp = "numeric"
  )
})

test_that("f_aov input validation: errors for invalid output_type string", {
  expect_error(
    quiet_f_aov(y ~ grp, data = df, output_type = "csv"),
    regexp = "output_type"
  )
})

test_that("f_aov input validation: errors for invalid transformation string (unambiguous non-match)", {
  expect_error(
    quiet_f_aov(y ~ grp, data = df, transformation = "logtransform"),
    regexp = "Invalid transformation"
  )
})

# =============================================================================
# 13. intro_text = FALSE
# =============================================================================
df  <- make_normal_data()
faov_direct <- function(intro, ...) {
  suppressMessages(
    f_aov(..., intro_text = intro,
          output_type = "rmd", norm_plots = FALSE,
          transformation = FALSE, open_generated_files = FALSE)
  )
}

test_that("f_aov intro_text = FALSE: runs without error when intro_text is FALSE", {
  expect_no_error(faov_direct(FALSE, y ~ grp, data = df))
})

test_that("f_aov intro_text = FALSE: rmd output does NOT contain assumption section when intro_text = FALSE", {
  out <- faov_direct(FALSE, y ~ grp, data = df)
  expect_false(grepl("Assumptions of ANOVA", out[["rmd"]]))
})

test_that("f_aov intro_text = FALSE: rmd output DOES contain assumption section when intro_text = TRUE", {
  out <- faov_direct(TRUE, y ~ grp, data = df)
  expect_match(out[["rmd"]], "Assumptions of ANOVA")
})

# =============================================================================
# 14. Shapiro-Wilk guard for n > 5000
# =============================================================================
test_that("f_aov Shapiro-Wilk large n guard: does not throw an error when n > 5000", {
  # n = 6000 adds meaningful runtime and is about robustness, not
  # typical user-facing behavior; skipping on CRAN is reasonable.
  skip_on_cran()
  set.seed(3)
  df_big <- data.frame(
    y   = rnorm(6000),
    grp = factor(rep(c("X", "Y", "Z"), length.out = 6000))
  )
  expect_no_error(
    quiet_f_aov(y ~ grp, data = df_big, transformation = FALSE)
  )
})

test_that("f_aov Shapiro-Wilk large n guard: Shapiro result is a shaped htest with NA p.value when n > 5000", {
  skip_on_cran()
  set.seed(3)
  df_big <- data.frame(
    y   = rnorm(6000),
    grp = factor(rep(c("X", "Y", "Z"), length.out = 6000))
  )
  out <- quiet_f_aov(y ~ grp, data = df_big, transformation = FALSE)
  sh  <- out[["y"]][["shapiro_test_residuals"]]

  # safe_shapiro() returns a real htest object even in the skip branch
  expect_s3_class(sh, "htest")
  expect_true(all(c("statistic", "p.value", "method", "data.name") %in% names(sh)))

  # p.value is NA (not the old placeholder 1), and the method label
  # explains why
  expect_true(is.na(sh$p.value))
  expect_match(sh$method, "skipped", fixed = TRUE)
  expect_match(sh$method, "n > 5000")
})

# =============================================================================
# 15. Small cell sizes (Levene power warning stored in output)
# =============================================================================
test_that("f_aov small cell size handling: runs without error when minimum cell size is 2", {
  set.seed(9)
  # n=2 per group is valid for aov() but triggers AD test guard (n<8).
  # The function should handle this gracefully with a warning, not an error.
  df_small <- data.frame(
    y   = c(rnorm(2, 5), rnorm(2, 8), rnorm(2, 11)),
    grp = factor(rep(c("A", "B", "C"), each = 2))
  )
  expect_no_error(
    suppressWarnings(quiet_f_aov(y ~ grp, data = df_small, transformation = FALSE))
  )
})

test_that("f_aov small cell size handling: min_cell_n is stored in output_list", {
  set.seed(9)
  df_small <- data.frame(
    y   = c(rnorm(2, 5), rnorm(2, 8), rnorm(2, 11)),
    grp = factor(rep(c("A", "B", "C"), each = 2))
  )
  out <- suppressWarnings(quiet_f_aov(y ~ grp, data = df_small, transformation = FALSE))
  expect_true("min_cell_n" %in% names(out[["y"]]))
  expect_lte(out[["y"]][["min_cell_n"]], 3L)
})

# =============================================================================
# 16. Non-significant ANOVA: letters should be 'ns'
# =============================================================================
test_that("f_aov non-significant ANOVA: Letter column is 'ns' when ANOVA is not significant", {
  set.seed(1234)
  df_null <- data.frame(
    y   = rnorm(60),           # no group effect
    grp = factor(rep(c("A", "B", "C"), each = 20))
  )
  out    <- quiet_f_aov(y ~ grp, data = df_null, transformation = FALSE)
  ph     <- out[["y"]][["post_hoc_summary_table"]]
  expect_true(all(ph$Letter == "ns"))
})

# =============================================================================
# 17. Post-hoc table content and column integrity
# =============================================================================
df  <- make_normal_data()
out <- quiet_f_aov(y ~ grp, data = df, transformation = FALSE)
ph  <- out[["y"]][["post_hoc_summary_table"]]

test_that("f_aov post-hoc table content: post_hoc_summary_table has expected columns (untransformed)", {
  expect_true("grp"      %in% names(ph))
  expect_true("SE"       %in% names(ph))
  expect_true("lower.CL" %in% names(ph))
  expect_true("upper.CL" %in% names(ph))
  expect_true("Letter"   %in% names(ph))
  expect_true("n"        %in% names(ph))
})

test_that("f_aov post-hoc table content: SE column absent when back-transformed (invalid on original scale)", {
  df_skew <- make_skewed_data()
  out_t   <- quiet_f_aov(y ~ grp, data = df_skew, transformation = TRUE)
  if (out_t[["y"]][["Response_Transformed"]]) {
    ph_t <- out_t[["y"]][["post_hoc_summary_table"]]
    expect_false("SE" %in% names(ph_t))
    expect_true("lower.CL" %in% names(ph_t))
    expect_true("upper.CL" %in% names(ph_t))
  }
})

test_that("f_aov post-hoc table content: lower.CL < emmean (or median BT) < upper.CL for each row", {
  centre_col <- intersect(c("emmean..", "median (BT)"), names(ph))
  if (length(centre_col) == 1) {
    expect_true(all(ph$lower.CL < ph[[centre_col]]))
    expect_true(all(ph[[centre_col]] < ph$upper.CL))
  }
})

test_that("f_aov post-hoc table content: n column sums to total observations", {
  expect_equal(sum(ph$n, na.rm = TRUE), nrow(df))
})

test_that("f_aov post-hoc table content: SE values are positive for untransformed table", {
  expect_true(all(ph$SE > 0))
})

test_that("f_aov post-hoc table content: back-transformed emmean column is named 'median (BT)' when transformed", {
  df_skew <- make_skewed_data()
  out_t   <- quiet_f_aov(y ~ grp, data = df_skew, transformation = TRUE)
  if (out_t[["y"]][["Response_Transformed"]]) {
    ph_t <- out_t[["y"]][["post_hoc_summary_table"]]
    expect_true("median (BT)" %in% names(ph_t))
    expect_false("emmean.."   %in% names(ph_t))
  }
})

# =============================================================================
# 18. par() and options() are restored on exit
# =============================================================================
test_that("f_aov restores global state on exit: par() settings are restored after call", {
  df          <- make_normal_data()
  par_before  <- par(no.readonly = TRUE)
  par_before$new <- NULL
  quiet_f_aov(y ~ grp, data = df, transformation = FALSE)
  par_after   <- par(no.readonly = TRUE)
  par_after$new <- NULL
  # mfrow is the most commonly altered setting
  expect_equal(par_before$mfrow, par_after$mfrow)
})

test_that("f_aov restores global state on exit: options() are restored after call", {
  df             <- make_normal_data()
  options_before <- options()
  quiet_f_aov(y ~ grp, data = df, transformation = FALSE)
  options_after  <- options()
  # Check a selection of commonly-changed options
  for (opt in c("warn", "scipen", "digits")) {
    expect_equal(options_before[[opt]], options_after[[opt]],
                 label = paste("option restored:", opt))
  }
})

# =============================================================================
# 19. Last-resort message
# =============================================================================
# Data deliberately designed so Box-Cox CANNOT fix both normality AND
# homoscedasticity simultaneously: three exponential groups with rates
# 2, 0.2, 0.05. At seed 335, the Box-Cox-transformed residuals still
# give Shapiro p = 0.00115 and Levene p = 0.00213 (both reject even at
# alpha = 0.01). Seed 335 was selected as the most robust trigger from
# seeds 1..500 (worst-case p-value 0.00213). The earlier default seed
# 55 was a lucky case where Box-Cox happened to succeed, which caused
# these tests to skip unconditionally.
make_hetero_data <- function(seed = 335) {
  set.seed(seed)
  data.frame(
    y   = c(rexp(30, rate = 2),      # tight group
            rexp(30, rate = 0.2),    # spread group
            rexp(30, rate = 0.05)),  # very spread group
    grp = factor(rep(c("A", "B", "C"), each = 30))
  )
}

test_that("f_aov last-resort message: last_resort_triggered is FALSE when transformation resolves assumptions", {
  df  <- make_normal_data()
  out <- quiet_f_aov(y ~ grp, data = df, transformation = TRUE)
  # For well-behaved normal data, transformation should not be needed,
  # so last_resort_triggered should be absent or FALSE
  flag <- out[["y"]][["last_resort_triggered"]]
  expect_true(is.null(flag) || isFALSE(flag))
})

test_that("f_aov last-resort message: last_resort_triggered and last_resort_reason are stored when Box-Cox cannot fix assumptions", {
  df  <- make_hetero_data()
  out <- quiet_f_aov_typed("rmd", y ~ grp, data = df, transformation = TRUE)
  expect_true(isTRUE(out[["y"]][["last_resort_triggered"]]))
  expect_false(is.null(out[["y"]][["last_resort_reason"]]))
  expect_type(out[["y"]][["last_resort_reason"]], "character")
  # The reason must mention at least one of the two failed assumptions
  expect_match(out[["y"]][["last_resort_reason"]],
               "heteroscedasticity|non-normality")
})

test_that("f_aov last-resort message: rmd output contains the last-resort warning text when triggered", {
  df  <- make_hetero_data()
  # Must use output_type = 'rmd' so out[['rmd']] is populated;
  # the previous version of this test used plain quiet_f_aov() which
  # left out[['rmd']] NULL and would have silently passed.
  out <- quiet_f_aov_typed("rmd", y ~ grp, data = df,
                           transformation = TRUE)
  expect_true(isTRUE(out[["y"]][["last_resort_triggered"]]))
  expect_match(out[["rmd"]], "Transformation did not resolve")
  # The alternative tests (Kruskal, GLM, ART) should be listed
  expect_match(out[["rmd"]], "Kruskal|ART|GLM")
})

# =============================================================================
# 20. Multiple-response NOTE
# =============================================================================
test_that("f_aov multiple-response NOTE in rmd output: rmd output contains multiple-testing NOTE for 2+ responses", {
  df      <- make_normal_data()
  df$y2   <- df$y + rnorm(nrow(df), 0, 0.3)
  out <- quiet_f_aov_typed("rmd", y + y2 ~ grp, data = df,
                           transformation = FALSE)
  expect_match(out[["rmd"]], "Multiple Testing Across")
  expect_match(out[["rmd"]], "9.8|9.75|FWER|1.*1.*0.05.*2")
})

test_that("f_aov multiple-response NOTE in rmd output: rmd output does NOT contain multiple-testing NOTE for single response", {
  df  <- make_normal_data()
  out <- quiet_f_aov_typed("rmd", y ~ grp, data = df,
                           transformation = FALSE)
  expect_false(grepl("Multiple Testing Across", out[["rmd"]]))
})

test_that("f_aov multiple-response NOTE in rmd output: NOTE contains personalised Bonferroni alpha for k=2", {
  df    <- make_normal_data()
  df$y2 <- df$y + rnorm(nrow(df))
  out   <- quiet_f_aov_typed("rmd", y + y2 ~ grp, data = df,
                             alpha          = 0.05,
                             transformation = FALSE)
  # Bonferroni for k=2, alpha=0.05 should be 0.025
  expect_match(out[["rmd"]], "0.025")
})

# =============================================================================
# 21. Output text correctness
# =============================================================================
df  <- make_normal_data()

test_that("f_aov output text correctness: CLD footnote says 'Groups' not 'Means'", {
  out <- quiet_f_aov_typed("rmd", y ~ grp, data = df,
                           transformation = FALSE)
  expect_match(out[["rmd"]], "Groups in the \"Letters\" column sharing")
  expect_false(grepl("Means sharing the same letter", out[["rmd"]]))
})

test_that("f_aov output text correctness: F-Statistic is spelled correctly in output", {
  out <- quiet_f_aov_typed("rmd", y ~ grp, data = df,
                           transformation = FALSE)
  expect_match(out[["rmd"]], "F-Statistic")
  expect_false(grepl("F-Statisic", out[["rmd"]]))
})

test_that("f_aov output text correctness: Shapiro-Wilk is spelled correctly (not Shapiro-Wilkinson)", {
  out <- quiet_f_aov_typed("rmd", y ~ grp, data = make_skewed_data(),
                           transformation = TRUE)
  expect_false(grepl("Shapiro-Wilkinson", out[["rmd"]]))
})

test_that("f_aov output text correctness: back-transformed description text does not mention SE as present", {
  out <- quiet_f_aov_typed("rmd", y ~ grp, data = make_skewed_data(),
                           transformation = TRUE)
  if (out[["y"]][["Response_Transformed"]]) {
    rmd <- out[["rmd"]]
    bt_section_start <- regexpr("Back-transformed Post Hoc Table", rmd)
    if (bt_section_start > 0) {
      bt_section <- substr(rmd, bt_section_start, bt_section_start + 500)
      # SE should only appear in the "SE are omitted" note, not as a column claim
      expect_false(grepl("SE values are identical", bt_section))
    }
  }
})

# =============================================================================
# 22. subset / na.action / weights pass-through (direct f_aov calls)
# =============================================================================
# IMPORTANT: tests that pass subset / weights / na.action MUST call
# f_aov directly, NOT through a wrapper. Wrappers that forward `...`
# break match.call()'s expression capture and turn the subset
# expression into a `..1` dots-index symbol, which then crashes
# model.frame() with "the ... list contains fewer than 3 elements".

test_that("subset argument drops rows before the ANOVA is fit", {
  df <- make_normal_data()                      # 45 rows, 3 groups of 15
  suppressMessages(
    utils::capture.output(
      res <- f_aov(y ~ grp, data = df,
                   subset = grp != "A",
                   output_type = "default",
                   norm_plots = FALSE, intro_text = FALSE,
                   open_generated_files = FALSE),
      file = nullfile()
    )
  )
  aov_model <- res[["y"]][["aov_test"]]
  expect_s3_class(aov_model, "aov")
  expect_equal(nobs(aov_model), 30)
  expect_equal(length(unique(aov_model$model$grp)), 2L)
})

test_that("na.action = na.omit drops rows with NA in any variable", {
  df <- make_normal_data()
  df$y[c(1, 5, 10)] <- NA                       # 3 NAs in response
  suppressMessages(
    utils::capture.output(
      res <- f_aov(y ~ grp, data = df,
                   na.action = na.omit,
                   output_type = "default",
                   norm_plots = FALSE, intro_text = FALSE,
                   open_generated_files = FALSE),
      file = nullfile()
    )
  )
  aov_model <- res[["y"]][["aov_test"]]
  expect_s3_class(aov_model, "aov")
  expect_equal(nobs(aov_model), 42)             # 45 - 3
})

test_that("weights argument is applied by aov() when passed via dots", {
  df  <- make_normal_data()
  wts <- runif(nrow(df), 0.5, 2)

  suppressMessages(
    utils::capture.output(
      res_w <- f_aov(y ~ grp, data = df, weights = wts,
                     output_type = "default",
                     norm_plots = FALSE, intro_text = FALSE,
                     open_generated_files = FALSE),
      file = nullfile()
    )
  )
  suppressMessages(
    utils::capture.output(
      res_u <- f_aov(y ~ grp, data = df,
                     output_type = "default",
                     norm_plots = FALSE, intro_text = FALSE,
                     open_generated_files = FALSE),
      file = nullfile()
    )
  )

  expect_s3_class(res_w[["y"]][["aov_test"]], "aov")
  expect_equal(nobs(res_w[["y"]][["aov_test"]]), nrow(df))

  # Weighted and unweighted F-values differ (sanity check that
  # weights actually reached aov() and were not silently dropped).
  f_w <- summary(res_w[["y"]][["aov_test"]])[[1]][["F value"]][1]
  f_u <- summary(res_u[["y"]][["aov_test"]])[[1]][["F value"]][1]
  expect_false(isTRUE(all.equal(f_w, f_u)))
})

test_that("subset is honored by downstream steps, not just aov()", {
  df <- make_normal_data()
  suppressMessages(
    utils::capture.output(
      res <- f_aov(y ~ grp, data = df,
                   subset = grp != "A",
                   output_type = "default",
                   norm_plots = FALSE, intro_text = FALSE,
                   open_generated_files = FALSE),
      file = nullfile()
    )
  )
  aov_model <- res[["y"]][["aov_test"]]
  groups_seen <- as.character(unique(aov_model$model$grp))
  expect_false("A" %in% groups_seen)
  expect_true(all(c("B", "C") %in% groups_seen))
})

test_that("contrasts argument is forwarded to aov()", {
  df <- make_normal_data()
  suppressMessages(
    utils::capture.output(
      res <- f_aov(y ~ grp, data = df,
                   contrasts = list(grp = "contr.sum"),
                   output_type = "default",
                   norm_plots = FALSE, intro_text = FALSE,
                   open_generated_files = FALSE),
      file = nullfile()
    )
  )
  aov_model <- res[["y"]][["aov_test"]]
  expect_s3_class(aov_model, "aov")
  coef_names <- names(coef(aov_model))
  expect_true(any(grepl("grp1|grp2", coef_names)))
})

# =============================================================================
# 23. safe_shapiro() integration (wrapper is fine here)
# =============================================================================
test_that("Shapiro residuals slot is an htest object", {
  df  <- make_normal_data()
  res <- quiet_f_aov(y ~ grp, data = df, transformation = FALSE)

  sh <- res[["y"]][["shapiro_test_residuals"]]
  if (!is.null(sh)) {
    expect_s3_class(sh, "htest")
    expect_true(all(c("statistic", "p.value", "method") %in% names(sh)))
  }
})

test_that("f_aov does not crash with n > 5000 (safe_shapiro skip branch)", {
  # Same rationale as section 14: n = 6000 is the slow part.
  skip_on_cran()
  set.seed(1)
  n   <- 6000
  big <- data.frame(
    y   = c(rnorm(n / 2, 10), rnorm(n / 2, 11)),
    grp = factor(rep(c("A", "B"), each = n / 2))
  )
  expect_no_error(
    res <- quiet_f_aov(y ~ grp, data = big, transformation = FALSE)
  )
  sh <- res[["y"]][["shapiro_test_residuals"]]
  if (!is.null(sh)) {
    expect_s3_class(sh, "htest")
    if (is.na(sh$p.value)) {
      expect_match(sh$method, "skipped", fixed = TRUE)
    }
  }
})

.onAttach <- function(libname, pkgname) {
  # # Get the package version
  # pkg_version <- utils::packageVersion(pkgname)
  #
  # # Print the library path, package name, and version
  # packageStartupMessage("Package '", pkgname, "' version ", pkg_version, " loaded from ", libname)

  # Check R version compatibility (assuming the package requires R >= 4.0.0)
  required_R_version <- "4.0.0"
  current_R_version <- getRversion()

  if (current_R_version < as.package_version(required_R_version)) {
    warning("This package was built for R version ", required_R_version,
            ", but you are running R version ", current_R_version, ".")
  }

  # # Check dependent packages (example: package "rstudioapi" and "ggplot2" are required)
  # required_deps <- c("rmarkdown",
  #                    "rstudioapi",
  #                    "knitr",
  #                    "pander",
  #                    "writexl",
  #                    "nortest",
  #                    "multcomp",
  #                    "emmeans",
  #                    "ggplot2",
  #                    "rstatix",
  #                    "grDevices"
  #                    )  # List of required packages
  # for (dep in required_deps) {
  #   if (!requireNamespace(dep, quietly = TRUE)) {
  #     warning("The required package '", dep, "' is not installed.")
  #
  #     # Offer to install the missing package
  #     install_choice <- readline(prompt = paste("Would you like to install the missing package '", dep, "'? (yes/no): ", sep = ""))
  #     if (tolower(install_choice) == "yes") {
  #       install.packages(dep)
  #       message("Package '", dep, "' has been installed.")
  #
  #       # Load the installed package
  #       library(dep, character.only = TRUE)
  #       message("Package '", dep, "' has been loaded.")
  #     } else {
  #       message("Package '", dep, "' was not installed. You may experience issues if the package is required.")
  #     }
  #   } else {
  #     # Package is installed, so load it
  #     library(dep, character.only = TRUE)
  #     #message("Package '", dep, "' is already installed and has been loaded.")
  #
  #     # Check if the version is compatible
  #     dep_version <- packageVersion(dep)
  #     required_version <- "0.1.0"
  #
  #     if (dep_version < as.package_version(required_version)) {
  #       warning("Package '", dep, "' is installed but its version (", dep_version,
  #               ") is older than the required version (", required_version, ").")
  #
  #       # Offer to update the package
  #       update_choice <- readline(prompt = paste("Would you like to update '", dep, "' to the latest version? (yes/no): ", sep = ""))
  #       if (tolower(update_choice) == "yes") {
  #         install.packages(dep)
  #         message("Package '", dep, "' has been updated.")
  #
  #         # Load the updated package
  #         library(dep, character.only = TRUE)
  #         message("Package '", dep, "' has been loaded.")
  #       } else {
  #         message("Package '", dep, "' was not updated. You may experience issues if the version is outdated.")
  #       }
  #     }
  #   }
  # }
}



.onUnload <- function(libpath) {
  # Print the library path from where the package is being unloaded
  message("Package unloaded from: ", libpath)
}

#' Perform a visual check on your data
#'
#' @description
#' Creates a 3-panel diagnostic dashboard to check data distribution and assumptions. It can also output a data summary table and identify outliers.
#'
#'
#' @param x A data.frame or formula (dispatches to the right method).
#' @param formula A formula specifying the columns (right hand side) to be summarized by maximal 3 groups (left hand side). More columns or groups can be added using \code{-} or \code{+} (e.g., \code{col1 + col2 ~ group1 + group2}) to do a sequential summary for each column parameter.
#' @param data A 'data.frame', 'data.table', or 'tibble'.
#' @param columns The numerical column(s) to summarize if no formula is used. Can be entered as a single character string (e.g., \code{"weight"}) or as a character vector \code{c("weight", "length"}).
#' @param group_vars Character vector of up to 3 grouping variables (e.g., \code{c("species", "fertilizer")}).
#' @param summary Logical. Show a summary table of the data. Default is \code{TRUE}.
#' @param outliers Logical. If \code{TRUE}, scans for outliers using Tukey's fences and if they exist, adds them to the result object. Default \code{TRUE}.
#' @param coef Numeric. The multiplier for the Interquartile Range (IQR) used for outlier detection. Default \code{1.5}.
#' @param limit_columns Integer or \code{NULL}. Defines the number of columns shown in the outlier table. Default = \code{7}. \code{NULL} = all columns are shown.
#' @param fancy_names Named character vector or \code{NULL}. Optional mapping of column names to more readable names for display in plots and legends.
#' @param advice Logical. If \code{TRUE}, runs \code{f_stat_wizard()} on each response
#'   column and appends the recommendation to the result. The advice is accessible
#'   via \code{result[["column_name"]]$advice} and is printed automatically.
#'   Default \code{FALSE}.
#' @param close_generated_files Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param output_type Character string specifying the output format. Default is \code{"default"}.
#'   \itemize{
#'     \item \code{"default"}: Returns the object and lets R decide whether
#'       to print; auto-prints if unassigned, silent if assigned to a variable.
#'       Use \code{print(result)} or \code{plot(result)} to display the
#'       returned object.
#'     \item \code{"console"}: Forces immediate printing to the console
#'       regardless of object assignment.
#'     \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
#'       file of the corresponding format. See \code{save_as},
#'       \code{save_in_wdir}, and \code{open_generated_files} for file
#'       path and opening behavior.
#'     \item \code{"rmd"}: Stores the raw markdown string inside the returned
#'       object for use in R Markdown documents.
#'   }
#' @param save_as Character string specifying the output file path (without extension).
#'   If a full path is provided, output is saved to that location.
#'   If only a filename is given, the file is saved in \code{tempdir()}.
#'   If only a directory is specified (providing an existing directory with trailing slash),
#'   the file is named "dataname_fscan" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
#'   Defaults to \code{file.path(tempdir(), "dataname_fscan.pdf")}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.
#' @param digits Integer. Decimal places for printed tables in 'pdf' and 'Word' output files. Default \code{3}.
#' @param ... Further arguments forwarded to \code{f_scan.data.frame}.
#'
#' @return A list of class \code{f_scan} containing plots, the summary table, and the outlier table. Using the option "output_type", it can also generate output in the form of: R Markdown code, 'Word', 'pdf', or 'Excel' files. Includes print, summary and plot methods for 'f_scan' objects.
#'
#' @details
#' \code{f_scan} automatically adapts the visualization based on the number of grouping variables provided:
#' \itemize{
#'   \item \strong{0 groups:} Univariate analysis (Single density/boxplot).
#'   \item \strong{1 group :} Main grouping variable (X-axis and Color).
#'   \item \strong{2 groups:} Adds Facet Wrapping.
#'   \item \strong{3 groups:} Adds Facet Grid (Row vs Column).
#' }
#'
#'
#' This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
#'\itemize{
#' \item \bold{Windows:} Install Pandoc and ensure the installation folder.
#' \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
#' \item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
#' \item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
#'
#' \item If Pandoc is not found, this function may not work as intended.
#' }
#'
#' @examples
#' # 1. Non-formula | No groups | Default output (default)
#' result <- f_scan(iris, columns = "Sepal.Length")
#' print(result)
#'
#' # 2. Non-formula | 1 group | Console output
#' result <- f_scan(
#'   mtcars,
#'   columns   = "mpg",
#'   group_vars = "cyl",
#'   output_type = "console"
#' )
#'
#' \donttest{
#' # 3. Non-formula | 2 groups | Multiple columns | Excel output
#'  result <- f_scan(
#'    mtcars,
#'    columns    = c("mpg", "hp"),
#'    group_vars = c("cyl", "am"),
#'    outliers   = TRUE,
#'    coef       = 1.5,
#'    output_type = "excel",
#'    save_as    = "mtcars_scan"
#'  )
#'
#' # 4. Formula | 1 group | Strict outlier detection | Word output
#' result <- f_scan(
#'   Sepal.Width ~ Species,
#'   data        = iris,
#'   outliers    = TRUE,
#'   coef        = 3.0,
#'   output_type = "word",
#'   save_as     = "iris_scan"
#'  )
#'
#' # 5. Formula | 2 groups | Multiple columns | Fancy names
#' result <- f_scan(
#'  mpg + hp + wt ~ vs + am,
#'  data        = mtcars,
#'  fancy_names = c(mpg = "Fuel Efficiency", hp = "Horsepower",
#'                  wt  = "Weight",          vs = "Engine Type",
#'                  am  = "Transmission"),
#'  summary     = TRUE
#' )
#' print(result)
#'
#'
#' #Create a small reproducible dataset with 3 grouping variables
#' set.seed(42)
#' plant_data <- data.frame(
#'   weight    = c(rnorm(60, 10, 2), rnorm(60, 14, 2)),
#'   species   = rep(c("A", "B"), each = 60),
#'   treatment = rep(rep(c("control", "treated"), each = 30), 2),
#'   batch     = factor(rep(c("1", "2", "3"), 40))
#'  )
#'
#' # 6. Formula | 3 groups | Facet Grid
#' result <- f_scan(
#'   weight ~ species + treatment + batch,
#'   data         = plant_data,
#'   coef         = 2.0,
#'   digits       = 2,
#'   output_type  = "word"
#' )
#' print(result)
#'
#' # 7. With statistical advice
#' result <- f_scan(
#'   Sepal.Length ~ Species,
#'   data    = iris,
#'   advice  = TRUE
#' )
#' #' print(result)
#' result[["Sepal.Length"]]$advice$y_type
#' }
#'
#' @export
f_scan <- function(x, ...) {
  # Use match.call() to check for 'data' WITHOUT evaluating the arguments.
  # This prevents the "object x not found" error.
  mc <- match.call()

  if (missing(x)) {
    # Check if 'data' was provided in the call (e.g. f_summary(data = mtcars))
    if (!is.null(mc$data)) {
      # Manually retrieve the data object
      x_val <- eval(mc$data, envir = parent.frame())
      # Dispatch manually to the data.frame method
      return(f_scan.data.frame(x_val, ...))
    } else {
      stop("Argument 'x' (or 'data') is missing.")
    }
  }

  # Standard S3 Dispatch
  UseMethod("f_scan")
}

#' @export
#' @rdname f_scan
f_scan.formula <- function(formula, data, ...) {
  # x is the formula (e.g., y ~ B0 + B1)
  x <- formula

  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(x) #use helper_check_lhs.R

  # Parse LHS (Response Variable)
  lhs_vars <- all.vars(x[[2]])


  # Parse RHS (Grouping Variables)
  rhs_vars  <- all.vars(x[[3]])
  if (length(rhs_vars) == 0)
    rhs_vars <- NULL

  # Capture the name of the data object passed to the formula method
  data_name_str <- deparse(substitute(data))
  column_name_str <- deparse(substitute(lhs_vars))

  if (length(data_name_str) > 1)
    data_name_str <- "data"

  # Pass it to the data.frame method
  f_scan.data.frame(
    x = data,
    columns = lhs_vars,
    group_vars = rhs_vars,
    internal_data_name = data_name_str,
    internal_column_name = column_name_str,
    ...
  )
}
#'
#' @export
#' @rdname f_scan
f_scan.data.frame <- function(x,
                              columns,
                              group_vars = NULL,
                              summary = TRUE,
                              outliers = TRUE,
                              coef = 1.5,
                              limit_columns = 7,
                              fancy_names = NULL,
                              advice = FALSE,
                              close_generated_files = FALSE,
                              open_generated_files = interactive(),
                              output_type = "default",
                              save_as = NULL,
                              save_in_wdir = FALSE,
                              digits = NULL,
                              ...) {
  # Map 'x' (S3 standard) back to 'data' (Internal logic)
  data <- x

  # Input Validation & Setup
  if (!is.data.frame(data)) {
    stop("Input 'data' must be a data frame.")
  }

  # Parameter validation
  if (!(output_type %in% c("pdf", "word", "excel", "rmd", "console" , "default"))) {
    stop(
      "Character string specifying the output format (output_type = ) should be either: 'pdf', 'word', 'excel', 'console','rmd', 'default'"
    )
  }

  # Safe filename logic
  # Default safety fallback
  data_name <- "data"

  # Check for the hidden internal name (passed from formula method)
  dots <- list(...)
  if ("internal_data_name" %in% names(dots)) {
    data_name <- dots[["internal_data_name"]]
  } else {
    # If no internal name, try to grab it from 'x' using:
    try_name <- try(deparse(substitute(x)), silent = TRUE)
    if (!inherits(try_name, "try-error") &&
        length(try_name) == 1 && nchar(try_name) < 50) {
      data_name <- try_name
    }
  }
  if ("internal_column_name" %in% names(dots)) {
    target_col_name <- dots[["internal_column_name"]]
  } else {
    target_col_name <- deparse(substitute(column))
  }

  # Add a safety check. If x was a complex expression (like filtered data),
  # deparse might still return multiple lines. Force it to length 1.
  if (length(data_name) > 1) {
    data_name <- "data"
  }


  if (target_col_name %in% names(data)) {
    target_cols <- target_col_name
  } else {
    target_cols <- columns
  }


  # Ensure response and predictors are in the data
  for (target_col in target_cols) {

    if (!(target_col %in% names(data))) {
      stop(paste("Column '", target_col_var, "' not found in the data."))
    }
    # Ensure the response variable is numeric
    target_col_var <- data[[target_col]]

    if (!is.numeric(target_col_var)) {
      stop(paste0("Column '", target_col_var, "' must be numeric."))
    }
  }

  for (group_var in group_vars) {
    if (!(group_var %in% names(data))) {
      stop(paste("Grouping variable", group_var, "not found in the data."))
    }
  }

  # Rename the data.frame with fancy names if fancy_names are provided.
  if (!is.null(fancy_names)) {
    data        <- f_rename_columns(data, fancy_names)
    target_cols <- f_rename_vector(target_cols, fancy_names)
    group_vars  <- f_rename_vector(group_vars, fancy_names)
  }

  # Handle Grouping Variables
  # Define roles based on how many groups are provided
  main_cat <- NULL # The X-axis / Color variable
  facet_1  <- NULL # The Wrap variable
  facet_2  <- NULL # The Grid variable (Row)

  # Check existence of groups
  if (!is.null(group_vars)) {
    if (!all(group_vars %in% names(data))) {
      missing <- group_vars[!group_vars %in% names(data)]
      stop(paste("Group columns not found:", paste(missing, collapse = ", ")))
    }

    # Convert groups to factors
    for (col in group_vars) {
      if (!is.factor(data[[col]])) {
        data[[col]] <- as.factor(data[[col]])
      }
    }

    # Assign roles dynamically
    n_groups <- length(group_vars)
    main_cat <- group_vars[1]
    if (n_groups > 1)
      facet_1 <- group_vars[2]
    if (n_groups > 2)
      facet_2 <- group_vars[3]
    if (n_groups > 3)
      warning("Only the first 3 grouping variables are used for plotting.")
  }

  #### Handle option "save_as = " ###
  if (save_in_wdir == TRUE) {
    save_dir <- getwd()
  } else{
    save_dir <- tempdir()
  }

  ### Prepare doc output
  # Generate a temporary file path for "output.Rmd"
  temp_output_dir <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")
  output_list <- list()

  # Create the output file "output.Rmd" in tempdir()
  file.create(temp_output_file)
  # Create a file_extension switch
  file_extension <- NULL

  # Wrap lines in rmd output document
  f_wrap_lines()

  #map the output type to extensions
  output_type_map <- c(
    "pdf"  = ".pdf",
    "word" = ".docx",
    "excel" = ".xlsx",
    "rmd"  = ".rmd"
  )

  # If the user specifies a path, filename or save_in_wdir == TRUE an output file should be created
  if (!is.null(save_as) || save_in_wdir == TRUE) {
    if (!is.null(save_as)) {
      #Remove backslash in save_as if needed
      save_as <- gsub(pattern = "\\\\",
                      replacement = "/",
                      x = save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if (file_extension_save_as[1] != FALSE) {
        file_extension <- file_extension_save_as
      }
    }

    if (is.null(file_extension) &&
        output_type %in% c("console", "default")) {
      # use helper get_save_path() to create output_path
      output_path <- get_save_path(
        save_as = save_as,
        default_name = paste(data_name, "fscan_output", sep = "_"),
        default_dir = save_dir,
        file.ext = ".pdf"
      )
      #set output_type to default
      output_type <- "pdf"

    }
    else if (is.null(file_extension) &&
             output_type %in% c("pdf", "word", "excel", "rmd")) {
      #create extension based on input_type
      file.ext <- unname(output_type_map[output_type])

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(
        save_as = save_as,
        default_name = paste(data_name, "fscan_output", sep = "_"),
        default_dir = save_dir,
        file.ext = file.ext
      )


    }
    else if (!is.null(file_extension)) {
      # use helper get_save_path() to create output_path
      output_path <- get_save_path(
        save_as = save_as,
        default_name = paste(data_name, "fscan_output", sep = "_"),
        default_dir = save_dir,
        file.ext = file_extension[1]
      )
      # reset the output type to match the user input extention in save_as
      output_type <- file_extension[2]
    }
  } else {
    #create extension based on input_type
    file.ext <- unname(output_type_map[output_type])

    # use helper get_save_path() to create output_path
    output_path <- get_save_path(
      save_as = save_as,
      default_name = paste(data_name, "fscan_output", sep = "_"),
      default_dir = save_dir,
      file.ext = file.ext
    )
  }


  # Prevent output to console and keep files open when output is "rmd" format
  if (output_type == "rmd") close_generated_files <- FALSE

  # Cross-platform close_generated_files (was Windows-only taskkill)
  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    if (output_type == "excel") close_app("EXCEL.EXE",   "Microsoft Excel", "soffice")
  }

  # This is the main function that generates the content for all the output types
  # render = TRUE: also produces ggsave PNGs + markdown (needed for word/pdf/rmd)
  # render = FALSE: only builds the R objects (plots, tables) -- much faster
  generate_report <- function(render = FALSE) {
    for (target_col in target_cols) {
      # Prepare Plot Data
      # If no groups provided, create a dummy "All Data" column for plotting consistency
      plot_data <- data
      if (is.null(main_cat)) {
        main_cat <- "All Data"
        plot_data[[main_cat]] <- factor("All Data")
      }

      # Faceting Logic
      my_facet <- NULL
      if (!is.null(facet_1) && is.null(facet_2)) {
        # 2 Groups: Facet Wrap
        my_facet <- facet_wrap(as.formula(paste("~", facet_1)), labeller = label_both)
      } else if (!is.null(facet_1) && !is.null(facet_2)) {
        # 3 Groups: Facet Grid
        my_facet <- facet_grid(as.formula(paste(facet_1, "~", facet_2)), labeller = label_both)
      }

      # Common Theme
      my_theme <- theme_bw(base_size = 11) +
        theme(
          panel.grid.minor = element_blank(),
          plot.title = element_text(
            face = "bold",
            size = 10,
            hjust = 0
          ),
          axis.title = element_text(size = 9),
          plot.margin = margin(5, 5, 5, 5),
          strip.background = element_rect(fill = "grey95")
        )

      # Helper: Legend Extractor
      get_legend <- function(myggplot) {
        tmp <- ggplot_gtable(ggplot_build(myggplot))
        leg <- which(sapply(tmp$grobs, function(x)
          x$name) == "guide-box")
        if (length(leg) > 0)
          legend <- tmp$grobs[[leg]]
        else
          legend <- NULL
        return(legend)
      }

      # Define the trendline layer conditionally -- NULL skips it in ggplot2
      trendline_layer <- if (main_cat != "All Data") {
        stat_summary(
          fun      = mean,
          geom     = "line",
          aes(group = 1),
          color    = "darkblue",
          linewidth = 0.8,
          linetype = "dashed",
          alpha    = 0.8,
          na.rm    = TRUE
        )
      } else {
        NULL  # ggplot2 silently ignores NULL layers
      }

      # Combined Boxplot + Means
      p1_base <- ggplot(plot_data, aes(
        x = .data[[main_cat]],
        y = .data[[target_col]],
        fill = .data[[main_cat]]
      )) +
        geom_boxplot(
          alpha = 0.4,
          outlier.shape = NA,
          width = 0.5,
          na.rm = TRUE
        ) +
        geom_jitter(
          width = 0.1,
          size = 1.2,
          alpha = 0.3,
          color = "grey30",
          na.rm = TRUE
        ) +
        stat_summary(
          fun.data = mean_se,
          geom = "errorbar",
          width = 0.1,
          color = "darkblue",
          linewidth = 0.8,
          alpha = 0.8,
          na.rm = TRUE
        ) +
        trendline_layer +
        stat_summary(
          fun = mean,
          geom = "point",
          size = 3,
          color = "darkblue",
          shape = 18,
          na.rm = TRUE
        ) +
        labs(title = "A. Overview: Data Spread, Mean (SE) & Trendline", y = target_col, x = "") +
        my_theme +
        theme(legend.position = "top", legend.title = element_blank()) +
        #use this to get the legend in one row
        guides(fill = guide_legend(nrow = 1)) +
        my_facet

      # Extract the legend
      shared_legend <- get_legend(p1_base)

      # Remove legend from p1 for the final plot
      p1_combined <- p1_base + theme(legend.position = "none")

      # Histogram + Density
      p2_hist <- ggplot(plot_data, aes(x = .data[[target_col]], fill = .data[[main_cat]])) +
        geom_histogram(
          aes(y = after_stat(density)),
          color = "white",
          alpha = 0.4,
          bins = 20,
          position = "identity",
          na.rm = TRUE
        ) +
        geom_density(alpha = 0.2,
                     linewidth = 0.8,
                     na.rm = TRUE) +
        labs(title = "B. Distribution Shape", x = target_col, y = "Density") +
        my_theme +
        theme(legend.position = "none") +
        my_facet

      # QQ-Plot
      p3_qq <- ggplot(plot_data, aes(sample = .data[[target_col]], color = .data[[main_cat]])) +
        stat_qq(size = 1.5,
                alpha = 0.6,
                na.rm = TRUE) +
        stat_qq_line(linetype = "dashed",
                     linewidth = 0.8,
                     na.rm = TRUE) +
        labs(title = "C. Normality Check (QQ)", x = "Theoretical", y = "Sample") +
        my_theme +
        theme(legend.position = "none") +
        my_facet

      # Layout Strategy
      bottom_row <- gridExtra::arrangeGrob(p2_hist, p3_qq, ncol = 2)

      # Construct Title
      title_str <- paste("Data scan:", target_col)
      if (!is.null(group_vars))
        title_str <- paste(title_str, "by", paste(group_vars, collapse = " | "))

      # Combine
      if (is.null(shared_legend)) {
        final_plot <- gridExtra::arrangeGrob(
          p1_combined,
          bottom_row,
          ncol = 1,
          heights = c(10, 8),
          top = title_str
        )
      } else {
        final_plot <- gridExtra::arrangeGrob(
          shared_legend,
          p1_combined,
          bottom_row,
          ncol = 1,
          heights = c(1, 10, 8),
          top = title_str
        )
      }

      output_list[[target_col]][["boxplot"]]   <- p1_base
      output_list[[target_col]][["histogram"]] <- p2_hist
      output_list[[target_col]][["qqplot"]]    <- p3_qq
      output_list[[target_col]][["main_plot"]] <- final_plot


      if (summary == TRUE) {
        sum_table <- f_summary(
          data = data,
          columns = target_col,
          group_vars = group_vars,
          show_name = FALSE,
          digits = NULL
        )$output_df
        output_list[[target_col]][["f_summary"]] <- sum_table
      }

      if (outliers == TRUE) {
        # Outlier Table Option
        # Suppress message "No outliers found" inside the scan to keep console clean
        suppressMessages({
          out_table <- f_outliers(
            data = data,
            column = target_col,
            group_vars = group_vars,
            coef = coef
          )$output_df
        })
        # Safely add Outlier Table
        # Check if 'out_table' is not NULL
        if (!is.null(out_table)) {
          output_list[[target_col]][["f_outliers"]] <- out_table
        } else {
          # If NULL (no outliers), add a placeholder sheet so the user knows
          output_list[[target_col]][["f_outliers"]] <- data.frame(Message = "No outliers detected")
        }
       }


      # --- Markdown rendering (ggsave + cat) only when needed for word/pdf/rmd ---
      if (render) {

      cat("
# Diagnostic dashboard of:", target_col , "from", data_name, "  \n")

      temp_file_final_plot <- tempfile(fileext = ".png")
      ggsave(
        temp_file_final_plot,
        plot = final_plot,
        width = 7.8,
        height = 7.8,
        units = "in",
        dpi = 300
      )

      cat(paste0("![](", temp_file_final_plot, ")"), "   \n  \n")
      cat("&nbsp;\n   \n")

      if (output_type != "rmd") {
        # Pagebreak
        cat("
<div style=\"page-break-after: always;\"></div>
\\newpage")
      }


      if (summary == TRUE) {
        cat("
## Summary table of variable: ",
            target_col,
            "from",
            data_name,
            "  \n")
        f_pander(f_conditional_round(sum_table, digits = digits))
      }


      if (outliers == TRUE) {
        cat("
## Outliers table of variable: ",
            target_col,
            "from",
            data_name,
            "  \n")
        # Check if out_table is NULL before passing to f_pander
        if (!is.null(out_table)) {
          f_pander(out_table, digits = digits)
        } else {
          cat("No outliers detected.\n")
        }
      }

      if (output_type != "rmd") {
        # Pagebreak
        cat("
<div style=\"page-break-after: always;\"></div>
\\newpage")
      }


      cat("
## Boxplots of:", target_col ,"from", data_name,"  \n")
      temp_file_p1_base <- tempfile(fileext = ".png")
      ggsave(
        temp_file_p1_base,
        plot = p1_base + theme(legend.position = "top"),
        width = 7.8,
        height = 5.8,
        units = "in",
        dpi = 300
      )

      cat(paste0("![](", temp_file_p1_base, ")"), "   \n  \n")

      cat("
## Histograms of:", target_col , "from", data_name,"  \n")
      temp_file_p2_hist <- tempfile(fileext = ".png")
      ggsave(
        temp_file_p2_hist,
        plot = p2_hist + theme(legend.position = "top"),
        width = 7.8,
        height = 5.8,
        units = "in",
        dpi = 300
      )

      cat(paste0("![](", temp_file_p2_hist, ")"), "   \n  \n")

      cat("
## QQ-plots of:", target_col , "from", data_name,"  \n")
      temp_file_p3_qq <- tempfile(fileext = ".png")
      ggsave(
        temp_file_p3_qq,
        plot = p3_qq + theme(legend.position = "top"),
        width = 7.8,
        height = 5.8,
        units = "in",
        dpi = 300
      )

      cat(paste0("![](", temp_file_p3_qq, ")"), "   \n  \n")

      } # End render block

    } #Main loop end
    return(output_list)
  } # End generate report function.



  # --- Execute generate_report() exactly ONCE ---
  # For word/pdf/rmd: render = TRUE  (builds R objects + ggsave/markdown)
  # For everything else: render = FALSE (builds R objects only -- much faster)
  needs_render <- output_type %in% c("word", "pdf", "rmd")

  if (needs_render) {
    # Single call: capture markdown AND collect output_list simultaneously
    suppressMessages(
      generated_markdown <- utils::capture.output(
        output_list <- generate_report(render = TRUE)
      )
    )
  } else {
    # Single call: only build R objects (no ggsave, no cat)
    suppressMessages(
      utils::capture.output(
        output_list <- generate_report(render = FALSE),
        file = nullfile()
      )
    )
  }
  class(output_list) <- "f_scan"

  # --- STATISTICAL ADVICE (optional) ---
  if (advice) {
    for (col in target_cols) {
      tryCatch({
        # Build formula: column ~ group_vars (or column ~ 1 if no groups)
        if (!is.null(group_vars) && length(group_vars) > 0) {
          wizard_formula <- as.formula(paste(col, "~", paste(group_vars, collapse = " + ")))
        } else {
          wizard_formula <- as.formula(paste(col, "~ 1"))
        }
        wizard_result <- f_stat_wizard(
          formula     = wizard_formula,
          data        = data,
          data_name   = data_name,      # <- pass user's name (was being overridden post-hoc)
          output_type = output_type,    # <- forward f_scan's output_type
          interactive = FALSE
        )
        output_list[[col]][["advice"]] <- wizard_result
      }, error = function(e) {
        output_list[[col]][["advice"]] <<- paste("Wizard could not analyse", col, ":", conditionMessage(e))
      })
    }
    class(output_list) <- "f_scan"
  }



  # Here the documents are constructed.
  if (output_type %in% c("word", "pdf")) {
    # Show save location before knitting else it will not display in console.
    message(paste0("Saving output in: ", output_path))

    # Create a temporary R Markdown file
    word_pdf_preamble <- function() {
      paste0(
        "
---
title: \"f_Scan Data Analysis Report\"
date: \"`r Sys.Date()`\"
output:
   word_document:
      reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
        latex_engine: pdflatex
header-includes:
  - \\usepackage[utf8]{inputenc}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
  - \\usepackage{titling}
  - \\setlength{\\droptitle}{-2.5cm} % Adjust vertical spacing
---
"
      )
    }

    # Prevent ## before printed output
    knitr::opts_chunk$set(comment = "")

    # Reuse the markdown already captured above -- no second generate_report() call
    rmd_content <- paste(
      word_pdf_preamble(),
      paste(generated_markdown, collapse = "\n"),
      sep = "\n"
    )

    # Write the complete Rmd content to the temp file
    writeLines(rmd_content, temp_output_file)

    # Create the RMarkdown file
    rmarkdown::render(
      temp_output_file,
      output_file = output_path,
      intermediates_dir = temp_output_dir,
      knit_root_dir = temp_output_dir,
      quiet = TRUE,
      output_format = paste0(output_type, "_document")
    )

    # Open files after creation
    if (open_generated_files == TRUE) {
      # Open the file with default program
      f_open_file(output_path)
    }

    return(invisible(output_list))


  } else if (output_type == "excel") {


    # Extract all f_summary_tables and keep their names
    f_summary_tables <- lapply(output_list, function(obj)
      obj$f_summary)

    # Assign names to the list for Excel sheet names based on response names
    names(f_summary_tables) <- paste0("sum_", target_cols)

    if (outliers == TRUE) {
    # Extract all f_outlier_tables and keep their names
    f_outliers_tables <- lapply(output_list, function(obj)
      obj$f_outliers)

    # Assign names to the list for Excel sheet names based on response names
    names(f_outliers_tables) <- paste0("out_", target_cols)

      output_excel <- c(f_summary_tables, f_outliers_tables)
    } else {
      output_excel <- f_summary_tables
    }

    # show the location were the file is saved
    message(paste0("Saving output in: ", output_path))

    # Write the safe list
    writexl::write_xlsx(output_excel, path = output_path)

    if (open_generated_files == TRUE) {
      f_open_file(output_path)
    }

    return(invisible(output_list))

  } else if (output_type == "rmd") {
    if (is.null(opts_knit$get("output.dir"))) {
      opts_knit$set(output.dir = tempdir())
    }

    # Reuse the markdown already captured above -- no extra generate_report() call
    clean_rmd_output <- paste(generated_markdown, collapse = "\n")

    output_list[["rmd"]] <- clean_rmd_output

    return(invisible(output_list))

  } else if (output_type == "default") {
    #Default R behavior only show when not stored in an new object
    return(output_list)

  } else if (output_type == "console") {
    #Print output list to the console (forced)
    print(output_list)

    return(invisible(output_list))

  } else {
    warning("Invalid output format specified. No file generated.")

  }

  # Remove the temporary R Markdown file
  invisible(suppressWarnings(file.remove(temp_output_file)))

} # end of f_scan function


# -------------------------------------------------------------------------
# S3 METHODS (Print, Summary, Plot)
# -------------------------------------------------------------------------

#' Print method for f_scan objects
#'
#' @param x An \code{f_scan} object.
#' @param summary Logical. Print summary statistics table? Default \code{TRUE}.
#' @param outliers Logical. Print outlier table? Default \code{TRUE}.
#' @param boxplot,histogram,qqplot,main_plot Logical. Which plots to print? All default \code{TRUE}.
#' @param advice Logical. Print statistical test recommendations? Default \code{TRUE} (shown only if \code{advice=TRUE} was used during \code{f_scan}).
#' @param digits Integer. Decimal places for printed tables. Default \code{3}.
#' @param ... Further arguments passed to or from other methods. Currently
#'   unused by the \code{f_scan} methods themselves, but accepted so the
#'   methods remain consistent with the base generics \code{print},
#'   \code{summary}, and \code{plot}.

#' @export
#' @method print f_scan
print.f_scan <- function(x,
                         summary = TRUE,
                         outliers = TRUE,
                         boxplot = TRUE,
                         histogram = TRUE,
                         qqplot = TRUE,
                         main_plot = TRUE,
                         advice = TRUE,
                         digits = 3,
                         ...
                         ) {
  # Loop over each category (a, b, etc.)
  for (category in names(x)) {
    # Get the sublist for this category
    sublist <- x[[category]]
    if (!is.list(sublist)) next

    # Print Summary Table
    if (summary && !is.null(sublist[["f_summary"]])) {
      cat("\n--- Summary Statistics ---\n")
      f_pander(sublist[["f_summary"]], digits = digits)
      cat("\n")
    }

    # Print Outlier Table
    if (outliers && "f_outliers" %in% names(sublist)) {
      cat("\n--- Outlier Detection ---\n")
      out_data <- sublist[["f_outliers"]]

      if (is.null(out_data) || ncol(out_data) == 1) {
        cat("No outliers detected.\n")
      } else {
        # If there are many outliers, print the top 10 to avoid spamming the console
        n_out <- nrow(out_data)
        cat(paste0("\nFound ", n_out, " outliers:\n"))
        f_pander(
          head(out_data, 10),
          col_width = 6,
          table_width = 90,
          limit_columns = 8,
          digits = digits
        )
        if (n_out > 15)
          cat(paste0("... and ", n_out - 15, " more rows.\n"))
      }
      cat("\n")
    }

    # Print Plots
    if (boxplot && !is.null(sublist[["boxplot"]]))
      print(sublist[["boxplot"]])
    if (histogram &&
        !is.null(sublist[["histogram"]]))
      print(sublist[["histogram"]])
    if (qqplot && !is.null(sublist[["qqplot"]]))
      print(sublist[["qqplot"]])

    if (main_plot && !is.null(sublist[["main_plot"]])) {
      if (inherits(sublist[["main_plot"]], "grob")) {
        if (!isTRUE(getOption("knitr.in.progress"))) {
          grid::grid.newpage()
        }
        grid::grid.draw(sublist[["main_plot"]])
      } else {
        print(sublist[["main_plot"]])
      }
    }

    # Print Statistical Advice (from f_stat_wizard)
    if (advice && !is.null(sublist[["advice"]])) {
      advice_obj <- sublist[["advice"]]
      if (inherits(advice_obj, "f_stat_wizard")) {
        cat("\n")
        print(advice_obj)
        cat("\n")
      } else if (is.character(advice_obj)) {
        cat("\n--- Statistical Advice ---\n")
        cat(advice_obj, "\n")
      }
    }
  }
  invisible(x)
}

#' Summary method for f_scan objects
#' @rdname print.f_scan
#' @param object f_scan object to make a summary table from.
#'
#' @export
#' @method summary f_scan
summary.f_scan <- function(object,
                           digits = 3,
                           ...) {
  # Loop over each category (a, b, etc.)
  for (category in names(object)) {
    # Get the sublist for this category
    sublist <- object[[category]]
    if (!is.list(sublist)) next

    if ("f_summary" %in% names(sublist)) {
      cat("\n--- Summary Statistics ---\n")
      f_pander(sublist[["f_summary"]],
               digits = digits
               )
      cat("\n")
    }
  }
  invisible(object)
}

#' Plot method for f_scan objects
#' @rdname print.f_scan
#' @param boxplot,histogram,qqplot,main_plot Logical. Which plots to render?
#' @export
#' @method plot f_scan
plot.f_scan <- function(x,
                        boxplot = TRUE,
                        histogram = TRUE,
                        qqplot = TRUE,
                        main_plot = TRUE,
                        ...
                        ) {
  # Loop over each category (a, b, etc.)
  for (category in names(x)) {
    # Get the sublist for this category
    sublist <- x[[category]]
    if (!is.list(sublist)) next

    # Logic matches print method but focuses on visuals
    if (boxplot && !is.null(sublist[["boxplot"]]))
      print(sublist[["boxplot"]])
    if (histogram &&
        !is.null(sublist[["histogram"]]))
      print(sublist[["histogram"]])
    if (qqplot && !is.null(sublist[["qqplot"]]))
      print(sublist[["qqplot"]])

    if (main_plot && !is.null(sublist[["main_plot"]])) {
      if (inherits(sublist[["main_plot"]], "grob")) {
        grid::grid.newpage()
        grid::grid.draw(sublist[["main_plot"]])
      } else {
        print(sublist[["main_plot"]])
      }
    }
  }
  invisible(x)
}

# =============================================================================
# Stress Tests for f_scan()
# =============================================================================
# These tests combine multiple parameter options to exercise as many code paths
# as possible. Run with: testthat::test_file("test-fscan-stress.R")
#
# Prerequisites: the rfriend package (or at minimum f_scan, f_summary,
# f_outliers, f_pander, f_conditional_round) must be loaded.
# =============================================================================
# ---------------------------------------------------------------------------
# Test Data
# ---------------------------------------------------------------------------

# Standard data: 2 groups, 2 numeric columns, known outliers
set.seed(42)
df_standard <- data.frame(
  Team       = rep(c("A", "B"), each = 20),
  Department = rep(c("Sales", "IT"), each = 10, times = 2),
  Salary     = rnorm(40, mean = 50000, sd = 2000),
  Age        = rnorm(40, mean = 35, sd = 3),
  EmployeeID = paste0("E", sprintf("%03d", 1:40))
)
# Inject clear outliers
df_standard[1, "Salary"] <- 100000
df_standard[2, "Salary"] <- 1000
df_standard[3, "Age"]    <- 100

# Data with no outliers (tight distribution)
set.seed(99)
df_no_outliers <- data.frame(
  Group = rep(c("X", "Y"), each = 50),
  Value = rep(10, 100) + runif(100, -0.01, 0.01)
)

# Data with NAs
set.seed(7)
df_with_na <- data.frame(
  Species = rep(c("Cat", "Dog"), each = 30),
  Weight  = c(rnorm(30, 5, 1), rnorm(30, 25, 5)),
  Length  = c(rnorm(30, 40, 5), rnorm(30, 60, 10))
)
df_with_na$Weight[c(1, 15, 31, 45)] <- NA
df_with_na$Length[c(5, 50)]          <- NA
# Inject outlier
df_with_na$Weight[10] <- 50

# Data with 3 grouping variables
set.seed(11)
df_3groups <- data.frame(
  weight    = c(rnorm(60, 10, 2), rnorm(60, 14, 2)),
  species   = rep(c("A", "B"), each = 60),
  treatment = rep(rep(c("control", "treated"), each = 30), 2),
  batch     = factor(rep(c("1", "2", "3"), 40))
)
# Inject outlier
df_3groups$weight[1] <- 50

# Small data (edge case: very few rows per group)
df_small <- data.frame(
  Group = c("A", "A", "A", "B", "B", "B"),
  Value = c(1, 2, 100, 10, 20, 30)
)

# Single numeric column, no groups
df_minimal <- data.frame(
  Score = c(rnorm(50, 100, 10), 200, 5)
)


# ---------------------------------------------------------------------------
# Helper: validate output structure
# ---------------------------------------------------------------------------

expect_fscan_structure <- function(result, expected_cols, summary, outliers) {
  expect_s3_class(result, "f_scan")
  expect_type(result, "list")

  for (col in expected_cols) {
    expect_true(col %in% names(result))
    sub <- result[[col]]

    # Plots should always be present
    expect_true("boxplot"   %in% names(sub))
    expect_true("histogram" %in% names(sub))
    expect_true("qqplot"    %in% names(sub))
    expect_true("main_plot" %in% names(sub))

    # Summary table
    if (summary) {
      expect_true("f_summary" %in% names(sub))
      expect_s3_class(sub[["f_summary"]], "data.frame")
    } else {
      expect_false("f_summary" %in% names(sub))
    }

    # Outlier table (always present when outliers = TRUE, even if "No outliers detected")
    if (outliers) {
      expect_true("f_outliers" %in% names(sub))
      expect_s3_class(sub[["f_outliers"]], "data.frame")
    } else {
      expect_false("f_outliers" %in% names(sub))
    }
  }
}


# ===========================================================================
# 1. DATA.FRAME NOTATION - PARAMETER COMBINATIONS
# ===========================================================================

test_that("single column | no groups | defaults", {
  result <- f_scan(df_standard, columns = "Salary", output_type = "default",
                   open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("single column | 1 group | defaults", {
  result <- f_scan(df_standard, columns = "Salary", group_vars = "Team",
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("single column | 2 groups | defaults", {
  result <- f_scan(df_standard, columns = "Salary",
                   group_vars = c("Team", "Department"),
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("multiple columns | 2 groups | defaults", {
  result <- f_scan(df_standard, columns = c("Salary", "Age"),
                   group_vars = c("Team", "Department"),
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, c("Salary", "Age"),
                         summary = TRUE, outliers = TRUE)
})

test_that("summary = FALSE, outliers = TRUE", {
  result <- f_scan(df_standard, columns = "Salary", group_vars = "Team",
                   summary = FALSE, outliers = TRUE,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = FALSE, outliers = TRUE)
})

test_that("summary = TRUE, outliers = FALSE", {
  result <- f_scan(df_standard, columns = "Salary", group_vars = "Team",
                   summary = TRUE, outliers = FALSE,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = FALSE)
})

test_that("summary = FALSE, outliers = FALSE", {
  result <- f_scan(df_standard, columns = "Salary",
                   summary = FALSE, outliers = FALSE,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = FALSE, outliers = FALSE)
})

test_that("strict coef = 3.0 catches fewer outliers", {
  r_loose  <- f_scan(df_standard, columns = "Salary", coef = 1.5,
                     output_type = "default", open_generated_files = FALSE)
  r_strict <- f_scan(df_standard, columns = "Salary", coef = 3.0,
                     output_type = "default", open_generated_files = FALSE)
  n_loose  <- nrow(r_loose[["Salary"]][["f_outliers"]])
  n_strict <- nrow(r_strict[["Salary"]][["f_outliers"]])
  expect_true(n_loose >= n_strict)
})


# ===========================================================================
# 2. FORMULA NOTATION - PARAMETER COMBINATIONS
# ===========================================================================

test_that("formula | single column | no groups", {
  result <- f_scan(Salary ~ 1, data = df_standard,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("formula | single column | 1 group", {
  result <- f_scan(Salary ~ Team, data = df_standard,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("formula | single column | 2 groups", {
  result <- f_scan(Salary ~ Team + Department, data = df_standard,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("formula | multiple columns | 2 groups", {
  result <- f_scan(Salary + Age ~ Team + Department, data = df_standard,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, c("Salary", "Age"),
                         summary = TRUE, outliers = TRUE)
})

test_that("formula | 3 groups (facet grid)", {
  result <- f_scan(weight ~ species + treatment + batch, data = df_3groups,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "weight", summary = TRUE, outliers = TRUE)
})

test_that("formula | summary = FALSE, outliers = FALSE, coef = 3", {
  result <- f_scan(Salary ~ Team, data = df_standard,
                   summary = FALSE, outliers = FALSE, coef = 3.0,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = FALSE, outliers = FALSE)
})


# ===========================================================================
# 3. GENERIC DISPATCH (data = instead of x =)
# ===========================================================================

test_that("dispatch via data = (x missing)", {
  result <- f_scan(data = df_standard, columns = "Salary",
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})


# ===========================================================================
# 4. DIGITS PARAMETER
# ===========================================================================

test_that("digits = NULL (no rounding)", {
  result <- f_scan(df_standard, columns = "Salary", digits = NULL,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("digits = 2", {
  result <- f_scan(df_standard, columns = "Salary", digits = 2,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("digits = 5", {
  result <- f_scan(df_standard, columns = "Salary", digits = 5,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("digits = 0", {
  result <- f_scan(df_standard, columns = "Salary", digits = 0,
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})


# ===========================================================================
# 5. FANCY NAMES
# ===========================================================================

test_that("fancy_names | single column + 1 group", {
  result <- f_scan(df_standard, columns = "Salary", group_vars = "Team",
                   fancy_names = c(Salary = "Annual Pay", Team = "Division"),
                   output_type = "default", open_generated_files = FALSE)
  # After renaming, keys should use fancy names
  expect_true("Annual Pay" %in% names(result))
})

test_that("fancy_names | formula | multi-column + multi-group", {
  result <- f_scan(Salary + Age ~ Team + Department, data = df_standard,
                   fancy_names = c(Salary     = "Pay",
                                   Age        = "Years",
                                   Team       = "Division",
                                   Department = "Dept"),
                   output_type = "default", open_generated_files = FALSE)
  expect_true("Pay"   %in% names(result))
  expect_true("Years" %in% names(result))
})


# ===========================================================================
# 6. EDGE CASES
# ===========================================================================

test_that("data with NAs does not error", {
  result <- f_scan(df_with_na, columns = "Weight", group_vars = "Species",
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Weight", summary = TRUE, outliers = TRUE)
})

test_that("data with NAs | multiple columns", {
  result <- f_scan(df_with_na, columns = c("Weight", "Length"),
                   group_vars = "Species",
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, c("Weight", "Length"),
                         summary = TRUE, outliers = TRUE)
})

test_that("data with no outliers produces placeholder", {
  result <- f_scan(df_no_outliers, columns = "Value", group_vars = "Group",
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Value", summary = TRUE, outliers = TRUE)
  out_df <- result[["Value"]][["f_outliers"]]
  # Should either have 0 data rows or the placeholder "No outliers detected"
  expect_true(nrow(out_df) >= 0)
})

test_that("very small groups (3 rows each)", {
  result <- f_scan(df_small, columns = "Value", group_vars = "Group",
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Value", summary = TRUE, outliers = TRUE)
})

test_that("no groups, no grouping at all (minimal data)", {
  result <- f_scan(df_minimal, columns = "Score",
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Score", summary = TRUE, outliers = TRUE)
})


# ===========================================================================
# 7. OUTPUT_TYPE = "console" (forced print, returns invisible)
# ===========================================================================

test_that("console output | single column | no groups", {
  expect_output(
    result <- f_scan(df_standard, columns = "Salary",
                     output_type = "console", open_generated_files = FALSE),
    "."
  )
  expect_fscan_structure(result, "Salary", summary = TRUE, outliers = TRUE)
})

test_that("console output | multi-column | 2 groups | digits = 2", {
  expect_output(
    result <- f_scan(df_standard, columns = c("Salary", "Age"),
                     group_vars = c("Team", "Department"),
                     digits = 2,
                     output_type = "console", open_generated_files = FALSE),
    "."
  )
  expect_fscan_structure(result, c("Salary", "Age"),
                         summary = TRUE, outliers = TRUE)
})


# ===========================================================================
# 8. INPUT VALIDATION (should error)
# ===========================================================================

test_that("non-data.frame input errors", {
  expect_error(f_scan("not a data frame", columns = "x"))
})

test_that("non-existent column errors", {
  expect_error(
    f_scan(df_standard, columns = "NonExistent",
           output_type = "default", open_generated_files = FALSE)
  )
})

test_that("non-existent group_var errors", {
  expect_error(
    f_scan(df_standard, columns = "Salary", group_vars = "Ghost",
           output_type = "default", open_generated_files = FALSE)
  )
})

test_that("non-numeric column errors", {
  expect_error(
    f_scan(df_standard, columns = "EmployeeID",
           output_type = "default", open_generated_files = FALSE)
  )
})

test_that("invalid output_type errors", {
  expect_error(
    f_scan(df_standard, columns = "Salary", output_type = "banana",
           open_generated_files = FALSE)
  )
})

test_that("one-sided formula errors", {
  expect_error(
    f_scan(~ Team, data = df_standard,
           output_type = "default", open_generated_files = FALSE)
  )
})


# ===========================================================================
# 9. PRINT / SUMMARY / PLOT METHODS
# ===========================================================================

test_that("print method works with all flags", {
  result <- f_scan(df_standard, columns = "Salary", group_vars = "Team",
                   output_type = "default", open_generated_files = FALSE)

  # Cases that produce text output (summary and/or outlier tables)
  expect_output(print(result), ".")
  expect_output(print(result, summary = TRUE, outliers = TRUE), ".")
  expect_output(print(result, summary = FALSE, outliers = TRUE), ".")
  expect_output(print(result, summary = TRUE, outliers = FALSE), ".")
  expect_output(print(result, digits = 5), ".")

  # When both tables are off, only plots are produced (no text to stdout)
  expect_error(
    suppressWarnings(print(result, summary = FALSE, outliers = FALSE)),
    NA
  )
})

test_that("summary method works", {
  result <- f_scan(df_standard, columns = "Salary", group_vars = "Team",
                   output_type = "default", open_generated_files = FALSE)
  expect_output(summary(result), ".")
  expect_output(summary(result, digits = 1), ".")
})

test_that("plot method runs without error", {
  result <- f_scan(df_standard, columns = "Salary", group_vars = "Team",
                   output_type = "default", open_generated_files = FALSE)
  # plot() should produce ggplot output without error
  expect_error(
    suppressWarnings(
      invisible(capture.output(plot(result), type = "message"))
    ),
    NA  # expect_error(..., NA) means "expect NO error"
  )
})

test_that("print method works for multi-column results", {
  result <- f_scan(df_standard, columns = c("Salary", "Age"),
                   group_vars = "Team",
                   output_type = "default", open_generated_files = FALSE)
  expect_output(print(result), ".")
})


# ===========================================================================
# 10. COMBINATORIAL STRESS: all toggle pairs
# ===========================================================================

test_that("combinatorial: summary x outliers x coef x digits x groups", {
  combos <- expand.grid(
    summary  = c(TRUE, FALSE),
    outliers = c(TRUE, FALSE),
    coef     = c(1.5, 3.0),
    digits   = list(NULL, 2),
    n_groups = c(0, 1, 2),
    stringsAsFactors = FALSE
  )

  group_options <- list(
    `0` = NULL,
    `1` = "Team",
    `2` = c("Team", "Department")
  )

  for (i in seq_len(nrow(combos))) {
    row <- combos[i, ]
    gv  <- group_options[[as.character(row$n_groups)]]
    dg  <- row$digits[[1]]  # unlist from the list column

    combo_label <- paste0(
      "summary=", row$summary,
      " outliers=", row$outliers,
      " coef=", row$coef,
      " digits=", if (is.null(dg)) "NULL" else dg,
      " groups=", row$n_groups
    )

    result <- tryCatch(
      f_scan(df_standard,
             columns    = "Salary",
             group_vars = gv,
             summary    = row$summary,
             outliers   = row$outliers,
             coef       = row$coef,
             digits     = dg,
             output_type = "default",
             open_generated_files = FALSE),
      error = function(e) e
    )

    expect_false(inherits(result, "error"))
    if (!inherits(result, "error")) {
      expect_s3_class(result, "f_scan")
    } else {
      message("FAILED combo: ", combo_label, " -> ", conditionMessage(result))
    }
  }
})


# ===========================================================================
# 11. FORMULA vs DATA.FRAME EQUIVALENCE
# ===========================================================================

test_that("formula and data.frame notation produce same structure", {
  r_df <- f_scan(df_standard, columns = "Salary", group_vars = "Team",
                 output_type = "default", open_generated_files = FALSE)
  r_fm <- f_scan(Salary ~ Team, data = df_standard,
                 output_type = "default", open_generated_files = FALSE)

  expect_identical(names(r_df), names(r_fm))
  for (col in names(r_df)) {
    expect_identical(sort(names(r_df[[col]])), sort(names(r_fm[[col]])))
    # Summary tables should be identical
    if ("f_summary" %in% names(r_df[[col]])) {
      expect_equal(r_df[[col]][["f_summary"]], r_fm[[col]][["f_summary"]])
    }
    # Outlier tables should be identical
    if ("f_outliers" %in% names(r_df[[col]])) {
      expect_equal(r_df[[col]][["f_outliers"]], r_fm[[col]][["f_outliers"]])
    }
  }
})

test_that("multi-column formula and data.frame produce same keys", {
  r_df <- f_scan(df_standard, columns = c("Salary", "Age"),
                 group_vars = c("Team", "Department"),
                 output_type = "default", open_generated_files = FALSE)
  r_fm <- f_scan(Salary + Age ~ Team + Department, data = df_standard,
                 output_type = "default", open_generated_files = FALSE)

  expect_identical(sort(names(r_df)), sort(names(r_fm)))
})


# ===========================================================================
# 12. LARGE DATA STRESS
# ===========================================================================

test_that("large data (10k rows) completes without error", {
  skip_on_cran()
  set.seed(321)
  df_large <- data.frame(
    Group1 = sample(LETTERS[1:5], 10000, replace = TRUE),
    Group2 = sample(c("X", "Y"), 10000, replace = TRUE),
    Value  = rnorm(10000, 100, 15)
  )
  # Inject some outliers
  df_large$Value[1:10] <- 300

  result <- f_scan(df_large, columns = "Value",
                   group_vars = c("Group1", "Group2"),
                   output_type = "default", open_generated_files = FALSE)
  expect_fscan_structure(result, "Value", summary = TRUE, outliers = TRUE)
})



# =============================================================================
# f_scan_new_tests.R
# Tests for the new f_scan() features:
#
#   A. advice = TRUE runs f_stat_wizard() on each response column and
#      attaches the result under result[["col"]]$advice.
#   B. S3 dispatch: formula interface and data.frame interface (via x).
#   C. print.f_scan() respects an advice argument.
# =============================================================================
# ---------------------------------------------------------------------------
# A. advice parameter
# ---------------------------------------------------------------------------

test_that("advice = TRUE attaches an f_stat_wizard object per column", {
  res <- suppressMessages(
    f_scan(Sepal.Length ~ Species, data = iris,
           advice = TRUE, output_type = "default",
           open_generated_files = FALSE)
  )
  expect_s3_class(res, "f_scan")
  # Response column should have an 'advice' entry
  expect_true(!is.null(res[["Sepal.Length"]][["advice"]]))

  advice <- res[["Sepal.Length"]][["advice"]]
  # Either a real f_stat_wizard object or a fall-back error string
  expect_true(inherits(advice, "f_stat_wizard") || is.character(advice))

  if (inherits(advice, "f_stat_wizard")) {
    expect_true(!is.null(advice$y_type))
    expect_true("recommended_call" %in% names(advice))
  }
})

test_that("advice = FALSE (default) does not populate the advice slot", {
  res <- suppressMessages(
    f_scan(Sepal.Length ~ Species, data = iris,
           output_type = "default",
           open_generated_files = FALSE)
  )
  expect_null(res[["Sepal.Length"]][["advice"]])
})

test_that("advice survives a non-numeric response without crashing", {
  # Species is a factor; the wizard cannot analyse it as Y. Scan
  # should still run and store an error message or an f_stat_wizard
  # object instead of blowing up.
  df <- iris
  df$Species2 <- iris$Species
  expect_no_error(
    res <- suppressMessages(
      f_scan(iris, columns = "Sepal.Length", group_vars = "Species",
             advice = TRUE, output_type = "default",
             open_generated_files = FALSE)
    )
  )
  expect_s3_class(res, "f_scan")
})

test_that("advice works with multiple response columns", {
  res <- suppressMessages(
    f_scan(iris, columns = c("Sepal.Length", "Sepal.Width"),
           group_vars = "Species",
           advice = TRUE, output_type = "default",
           open_generated_files = FALSE)
  )
  expect_true("advice" %in% names(res[["Sepal.Length"]]))
  expect_true("advice" %in% names(res[["Sepal.Width"]]))
})

# ---------------------------------------------------------------------------
# B. S3 dispatch
# ---------------------------------------------------------------------------

test_that("formula interface dispatches via f_scan.formula", {
  res <- suppressMessages(
    f_scan(Sepal.Length ~ Species, data = iris,
           output_type = "default",
           open_generated_files = FALSE)
  )
  expect_s3_class(res, "f_scan")
})

test_that("data.frame interface dispatches via f_scan.data.frame", {
  res <- suppressMessages(
    f_scan(iris, columns = "Sepal.Length", group_vars = "Species",
           output_type = "default",
           open_generated_files = FALSE)
  )
  expect_s3_class(res, "f_scan")
})

test_that("named call f_scan(data = iris, columns = ...) still works", {
  # Regression guard for the match.call() shim that looks up `data`
  # when `x` is missing.
  res <- suppressMessages(
    f_scan(data = iris, columns = "Sepal.Length", group_vars = "Species",
           output_type = "default",
           open_generated_files = FALSE)
  )
  expect_s3_class(res, "f_scan")
})

# ---------------------------------------------------------------------------
# C. print.f_scan with advice argument
# ---------------------------------------------------------------------------

test_that("print.f_scan accepts an advice argument without erroring", {
  res <- suppressMessages(
    f_scan(Sepal.Length ~ Species, data = iris,
           advice = TRUE, output_type = "default",
           open_generated_files = FALSE)
  )
  expect_no_error(
    utils::capture.output(print(res, advice = TRUE))
  )
  expect_no_error(
    utils::capture.output(print(res, advice = FALSE))
  )
})

#' Compare Two Statistical Models
#'
#' Compares two statistical models by calculating key metrics such as AIC, BIC, log-likelihood, \eqn{R^2},
#' and others. Supports comparison of nested models using ANOVA tests.
#'
#' @param model1 The first model object. Supported classes include: \code{"lm", "glm", "aov", "lmerMod", "glmerMod", and "nls"}.
#' @param model2 The second model object. Supported classes include: \code{"lm", "glm", "aov", "lmerMod", "glmerMod", and "nls"}.
#' @param nested Logical. If \code{TRUE}, assumes the models are nested and performs an ANOVA comparison.
#'   If \code{NULL} (default), the function attempts to automatically determine if the models are nested.
#' @param model1_name Optional character string. A custom name for model1 in the output. If \code{NULL} (default),
#'   the function uses \code{deparse(substitute(model1))}.
#' @param model2_name Optional character string. A custom name for model2 in the output. If \code{NULL} (default),
#'   the function uses \code{deparse(substitute(model2))}.
#' @param digits Integer. The number of decimal places to round the output metrics. Defaults to \code{3}.
#'
#' @return A list of class "f_model_comparison" containing:
#'   \item{model1_name}{The name of the first model (always the simpler model when nested).}
#'   \item{model2_name}{The name of the second model (always the more complex model when nested).}
#'   \item{model1_class}{The class of the first model.}
#'   \item{model2_class}{The class of the second model.}
#'   \item{metrics_table}{A data frame summarizing metrics for both models, their differences, and (if applicable) the ANOVA p-value.}
#'   \item{formatted_metrics_table}{A formatted version of the metrics table for printing.}
#'   \item{anova_comparison}{The ANOVA comparison results if the models are nested and an ANOVA test was performed.}
#'   \item{nested}{Logical indicating whether the models were treated as nested.}
#'   \item{swapped}{Logical indicating whether the model order was swapped to ensure model1 is the simpler model.}
#'
#' @section Supported Model Classes:
#' The function supports the following model classes:
#'   \itemize{
#'     \item Linear models ("lm")
#'     \item Generalized linear models ("glm")
#'     \item Analysis of variance models ("aov")
#'     \item Linear mixed models ("lmerMod")
#'     \item Generalized linear mixed models ("glmerMod")
#'     \item Nonlinear least squares models ("nls")
#'   }
#'
#' Note: Multi-stratum AOV models (fitted with \code{Error()}) are not supported
#' and will produce a warning.
#'
#' @details
#' Calculate various metrics to assess model fit:
#'   \itemize{
#'     \item \bold{AIC/BIC:} Lower values indicate better fit.
#'     \item \bold{Log-Likelihood:} Higher values (less negative) indicate better fit.
#'     \item \bold{\eqn{R^2}:} Proportion of variance explained by the model.
#'     \item \bold{Adjusted \eqn{R^2}:} \eqn{R^2} penalized for the number of parameters (for linear models).
#'     \item \bold{Nagelkerke \eqn{R^2}:} A pseudo-\eqn{R^2} for generalized linear models (GLMs).
#'     \item \bold{Marginal/Conditional \eqn{R^2}:} For mixed models, marginal \eqn{R^2} reflects fixed effects, while conditional \eqn{R^2} includes random effects.
#'     \item \bold{Sigma:} Residual standard error.
#'     \item \bold{Deviance:} Model deviance.
#'     \item \bold{SSE:} Sum of squared errors.
#'     \item \bold{Parameters (df):} Number of model parameters.
#'     \item \bold{Residual df:} Residual degrees of freedom.
#'   }
#'
#' When nested models are detected or specified, model1 is always treated as the simpler
#' model (fewer parameters). If the user passes the complex model first, the function
#' automatically swaps them and issues a message.
#'
#' If the models are nested, an ANOVA test is performed to compare them, and a p-value
#' is provided to assess whether the more complex model significantly improves fit.
#'
#' @note
#'   \itemize{
#'     \item The function supports a variety of model types but may issue warnings if unsupported or partially supported classes are used.
#'     \item For GLMs, Nagelkerke's \eqn{R^2} is used as a pseudo-\eqn{R^2} approximation, computed
#'       from the model's null deviance to avoid refitting a null model.
#'     \item For mixed models, the function relies on the 'r.squaredGLMM' function from the 'MuMIn' package for \eqn{R^2} calculation.
#'     \item For NLS models, \eqn{R^2} is provided for convenience but should be interpreted with caution
#'       as it does not have the same statistical properties as in linear models.
#'     \item The idea of this function (not the code), I got from Dustin Fife's function \href{https://github.com/dustinfife/flexplot/blob/master/R/model.comparison.R}{'model.comparison'} in the super cool \href{https://github.com/dustinfife/flexplot/}{'flexplot package'}.
#'   }
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#'
#' @examples
#' # Example with linear models.
#' model1 <- lm(mpg ~ wt, data = mtcars)
#' model2 <- lm(mpg ~ wt + hp, data = mtcars)
#' comparison <- f_model_compare(model1, model2)
#' print(comparison)
#'
#' # Example with GLMs.
#' \donttest{
#' model1 <- glm(am ~ wt, data = mtcars, family = binomial)
#' model2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
#' comparison <- f_model_compare(model1, model2)
#' print(comparison)
#' }
#'
#' # Models can be passed in any order - the function auto-swaps if needed.
#' complex <- lm(mpg ~ wt + hp + qsec, data = mtcars)
#' simple  <- lm(mpg ~ wt, data = mtcars)
#' comparison <- f_model_compare(complex, simple)
#' # model1 will be "simple", model2 will be "complex" in the output
#'
#' # Example with custom model names (useful when calling from wrapper functions).
#' comparison <- f_model_compare(model1, model2,
#'                               model1_name = "Weight only",
#'                               model2_name = "Weight + Horsepower")
#' print(comparison)
#'
#' @seealso
#' \code{\link{AIC}}, \code{\link{BIC}}, \code{\link{anova}}, \code{\link{logLik}}, \code{\link[MuMIn]{r.squaredGLMM}}
#'
#' @export


f_model_compare <- function(model1, model2, nested = NULL,
                            model1_name = NULL, model2_name = NULL,
                            digits = 3) {

  # ---- Identify model classes using inherits() ----
  # Order matters: check more specific classes first (glm before lm, aov before lm)
  supported_classes <- c("glmerMod", "lmerMod", "glm", "aov", "nls", "lm")

  get_model_class <- function(model) {
    for (cls in supported_classes) {
      if (inherits(model, cls)) return(cls)
    }
    return(class(model)[1])  # fallback for unsupported classes
  }

  # ---- Guard against aovlist (multi-stratum Error() models) ----
  if (inherits(model1, "aovlist") || inherits(model2, "aovlist")) {
    warning(
      "Multi-stratum aov models (fitted with Error()) are not supported. ",
      "The summary structure differs from standard aov. Results may be unreliable."
    )
  }

  m1_class <- get_model_class(model1)
  m2_class <- get_model_class(model2)

  if (!m1_class %in% supported_classes || !m2_class %in% supported_classes) {
    warning("One or both models are not of a supported class. Results may be unreliable.")
  }

  # ---- Get model names (before potential swap) ----
  # deparse(substitute()) must be called before any reassignment
  if (is.null(model1_name)) {
    m1_name <- deparse(substitute(model1))
  } else {
    m1_name <- model1_name
  }
  if (is.null(model2_name)) {
    m2_name <- deparse(substitute(model2))
  } else {
    m2_name <- model2_name
  }

  # ---- Validate that models were fit on the same number of observations ----
  n1 <- tryCatch(nobs(model1), error = function(e) NA)
  n2 <- tryCatch(nobs(model2), error = function(e) NA)
  if (!is.na(n1) && !is.na(n2) && n1 != n2) {
    warning(
      "Models were fit on different numbers of observations (", n1, " vs ", n2, "). ",
      "Comparisons of AIC, BIC, and likelihood-based metrics are not valid. ",
      "This can happen due to different NA handling."
    )
  }

  # ---- Determine if models are nested ----
  # Use isTRUE() to safely handle NULL default for nested
  if (m1_class != m2_class && isTRUE(nested)) {
    nested <- FALSE
    warning("Models are not of the same class and thus can't be nested. Assuming non-nested models.")
  }

  # Auto-detect if models are nested if not specified by the user
  if (is.null(nested)) {
    if (m1_class != m2_class) {
      nested <- FALSE
    } else {
      tryCatch({
        check_nested <- is_nested(model1, model2)
        nested       <- check_nested$nested
        if (!nested) {
          reason_message <- "Automatically determined if models were nested and found:\n"
          message(paste0(reason_message, check_nested$reason))
        }
      }, error = function(e) {
        warning(
          "Could not automatically determine if models are nested. ",
          "Assuming non-nested models. If the models ARE nested use the option: nested = TRUE"
        )
        nested <<- FALSE
      })
    }
  }

  # Safety net: if nested is still NULL at this point, default to FALSE
  if (is.null(nested)) nested <- FALSE


  # ---- Ensure model1 is the simpler model (fewer parameters) ----
  # This guarantees correct sign on differences and valid ANOVA ordering.
  swapped <- FALSE

  get_df <- function(model) {
    mc <- get_model_class(model)
    if (mc %in% c("lmerMod", "glmerMod")) {
      tryCatch(attr(stats::logLik(model), "df"), error = function(e) NA)
    } else {
      tryCatch(sum(!is.na(stats::coef(model))), error = function(e) NA)
    }
  }

  df1 <- get_df(model1)
  df2 <- get_df(model2)

  if (!is.na(df1) && !is.na(df2) && df1 > df2) {
    # Swap models so model1 is always simpler
    tmp_model <- model1;  model1 <- model2;  model2 <- tmp_model
    tmp_name  <- m1_name; m1_name <- m2_name; m2_name <- tmp_name
    tmp_class <- m1_class; m1_class <- m2_class; m2_class <- tmp_class
    swapped <- TRUE
    message(
      "Note: Model input order was swapped, applied automated FIX:\n
      - model1 ('", m1_name,"', df=", df2, ") is the simpler model\n
      - model2 ('", m2_name,"', df=", df1, ") is the more complex model."
    )
  }


  # ---- Function to safely extract metrics ----
  get_metrics <- function(model) {
    model_class <- get_model_class(model)

    metrics <- list(
      AIC = NA,
      BIC = NA,
      logLik = NA,
      r.squared = NA,
      adj.r.squared = NA,
      sigma = NA,
      deviance = NA,
      SSE = NA,
      df = NA,
      df.residual = NA
    )

    # AIC and BIC
    metrics$AIC <- tryCatch(stats::AIC(model), error = function(e) NA)
    metrics$BIC <- tryCatch(stats::BIC(model), error = function(e) NA)

    # Log-likelihood
    metrics$logLik <- tryCatch(as.numeric(stats::logLik(model)), error = function(e) NA)

    # ---- R-squared and adjusted R-squared ----
    if (model_class == "aov") {
      anova_table <- summary(model)
      SSR <- utils::tail(anova_table[[1]]$"Sum Sq", 1)  # Residual sum of squares
      SST <- sum(anova_table[[1]]$"Sum Sq")              # Total sum of squares
      n   <- nobs(model)
      # Model df = sum of all factor df (everything except the residual row)
      df_model <- sum(utils::head(anova_table[[1]]$"Df", -1))

      metrics$r.squared     <- 1 - (SSR / SST)
      metrics$adj.r.squared <- 1 - ((1 - metrics$r.squared) * (n - 1) / (n - df_model - 1))

    } else if (model_class == "nls") {
      n              <- nobs(model)
      df_reg         <- length(stats::coef(model))
      fitted_vals    <- stats::fitted(model)
      residuals_vals <- stats::residuals(model)
      y_reconstructed      <- fitted_vals + residuals_vals
      mean_y_reconstructed <- mean(y_reconstructed, na.rm = TRUE)
      SST <- sum((y_reconstructed - mean_y_reconstructed)^2)
      SSR <- sum(stats::residuals(model)^2)

      metrics$r.squared     <- 1 - (SSR / SST)
      metrics$adj.r.squared <- 1 - ((1 - metrics$r.squared) * (n - 1) / (n - df_reg - 1))

      # Warn about NLS R-squared limitations
      message(
        "Note: R\u00b2 for nonlinear models (nls) does not have the same ",
        "statistical properties as for linear models. Interpret with caution."
      )

    } else if (model_class == "lm") {
      # Cache summary() to avoid computing it twice
      s <- summary(model)
      metrics$r.squared     <- s$r.squared
      metrics$adj.r.squared <- s$adj.r.squared

    } else if (model_class == "glm") {
      # Nagelkerke's R^2 using null.deviance from the model object directly.
      # This is safer than update(model, . ~ 1) which can fail with offsets or subsets.
      tryCatch({
        ll_full <- as.numeric(stats::logLik(model))
        n       <- nobs(model)

        # Reconstruct null log-likelihood from null.deviance
        # For GLMs: null.deviance = -2 * (ll_null - ll_saturated)
        #           deviance      = -2 * (ll_full - ll_saturated)
        # So: ll_null = ll_full - (null.deviance - deviance) / 2
        ll_null <- ll_full - (model$null.deviance - model$deviance) / 2

        cox_snell     <- 1 - exp((2 / n) * (ll_null - ll_full))
        max_cox_snell <- 1 - exp((2 / n) * ll_null)
        metrics$r.squared     <- cox_snell / max_cox_snell
        metrics$adj.r.squared <- NA
      }, error = function(e) {
        # Defaults are already NA
        NULL
      })

    } else if (model_class %in% c("lmerMod", "glmerMod")) {
      # For mixed models, calculate marginal and conditional R^2
      tryCatch({
        if (!requireNamespace("MuMIn", quietly = TRUE)) {
          warning("Package 'MuMIn' is required for R\u00b2 of mixed models. Install it with install.packages('MuMIn').")
        } else {
          r2 <- MuMIn::r.squaredGLMM(model)
          metrics$r.squared     <- r2[1]  # Marginal R^2 (fixed effects)
          metrics$adj.r.squared <- r2[2]  # Conditional R^2 (fixed + random)
        }
      }, error = function(e) {
        # Defaults are already NA
        NULL
      })

    } else {
      metrics$r.squared     <- NA
      metrics$adj.r.squared <- NA
    }

    # ---- Sigma (residual standard error) ----
    if (model_class %in% c("lm", "aov", "glm", "nls")) {
      metrics$sigma <- tryCatch(stats::sigma(model), error = function(e) NA)
    } else {
      metrics$sigma <- tryCatch(
        sqrt(sum(stats::residuals(model, type = "pearson")^2) / stats::df.residual(model)),
        error = function(e) NA
      )
    }

    # Deviance
    #
    # For REML-fit mixed models, stats::deviance() is deprecated (lme4 >= 1.1-28)
    # and emits a warning. The historical behaviour of deviance() on a REML fit
    # was to return the REML criterion, so we call lme4::REMLcrit() explicitly.
    # For ML-fit mixed models and all other model classes, stats::deviance()
    # is the correct call.
    metrics$deviance <- tryCatch({
      if (inherits(model, c("lmerMod", "glmerMod")) &&
          isTRUE(lme4::isREML(model))) {
        lme4::REMLcrit(model)
      } else {
        stats::deviance(model)
      }
    }, error = function(e) NA)

    # ---- Degrees of freedom ----
    if (model_class %in% c("lmerMod", "glmerMod")) {
      metrics$df <- tryCatch(attr(stats::logLik(model), "df"), error = function(e) NA)
    } else {
      # Handle NA coefficients in models like lm with singular fits
      metrics$df <- tryCatch(sum(!is.na(stats::coef(model))), error = function(e) NA)
    }

    # Residual degrees of freedom
    metrics$df.residual <- tryCatch(stats::df.residual(model), error = function(e) NA)

    # ---- SSE (Sum of Squared Errors) ----
    if (model_class %in% c("lm", "aov", "lmerMod", "nls")) {
      metrics$SSE <- tryCatch(
        sum(stats::residuals(model, type = "response")^2),
        error = function(e) NA
      )
    } else if (model_class %in% c("glm", "glmerMod")) {
      if (stats::family(model)$family == "gaussian") {
        metrics$SSE <- tryCatch(stats::deviance(model), error = function(e) NA)
      } else {
        metrics$SSE <- NA
      }
    } else {
      metrics$SSE <- tryCatch(sum(stats::residuals(model)^2), error = function(e) NA)
    }

    return(metrics)
  }

  # ---- Get metrics for both models ----
  m1_metrics <- get_metrics(model1)
  m2_metrics <- get_metrics(model2)

  # ---- Calculate differences ----
  diff_metrics <- list(
    AIC           = m2_metrics$AIC - m1_metrics$AIC,
    BIC           = m2_metrics$BIC - m1_metrics$BIC,
    logLik        = m2_metrics$logLik - m1_metrics$logLik,
    r.squared     = m2_metrics$r.squared - m1_metrics$r.squared,
    adj.r.squared = m2_metrics$adj.r.squared - m1_metrics$adj.r.squared,
    sigma         = m2_metrics$sigma - m1_metrics$sigma,
    deviance      = m2_metrics$deviance - m1_metrics$deviance,
    SSE           = m2_metrics$SSE - m1_metrics$SSE,
    df            = m2_metrics$df - m1_metrics$df,
    df.residual   = m2_metrics$df.residual - m1_metrics$df.residual
  )

  # ---- Perform nested model comparison if applicable ----
  anova_result <- NULL
  p_value <- NA

  if (isTRUE(nested)) {
    # Determine test type
    test_type <- if (m1_class == "glm") {
      ifelse(stats::family(model1)$family == "binomial", "Chisq", "F")
    } else if (m1_class %in% c("lmerMod", "glmerMod")) {
      "Chisq"
    } else if (m1_class == "nls") {
      "F"
    } else {
      "none"
    }

    # Perform ANOVA comparison with error handling
    anova_result <- tryCatch({
      if (test_type %in% c("Chisq", "F")) {
        stats::anova(model1, model2, test = test_type)
      } else {
        stats::anova(model1, model2)
      }
    }, error = function(e) {
      warning("Model comparison failed: ", e$message)
      return(NULL)
    })

    # Extract p-value based on model type
    p_value <- tryCatch({
      if (m1_class %in% c("lmerMod", "glmerMod")) {
        stats::pchisq(anova_result$Chisq[2], df = anova_result$Df[2], lower.tail = FALSE)
      } else if (!is.null(anova_result$`Pr(>F)`)) {
        anova_result[nrow(anova_result), "Pr(>F)"]
      } else if (!is.null(anova_result$`Pr(>Chi)`)) {
        anova_result[nrow(anova_result), "Pr(>Chi)"]
      } else {
        NA_real_
      }
    }, error = function(e) NA_real_)
  }


  # ---- Create metric labels depending on model class ----
  # Use model1's class for label selection (model2 should match for nested,
  # and for non-nested comparisons the labels are a best-effort representation)
  metric_labels <- if (m1_class %in% c("glm")) {
    c("AIC", "BIC", "Log-Likelihood", "Nagelkerke R\u00b2", "",
      "Sigma", "Deviance", "SSE", "Parameters (df)", "Residual df", "ANOVA p-value")
  } else if (m1_class %in% c("lmerMod", "glmerMod")) {
    c("AIC", "BIC", "Log-Likelihood", "Marginal R\u00b2", "Conditional R\u00b2",
      "Sigma", "Deviance", "SSE", "Parameters (df)", "Residual df", "ANOVA p-value")
  } else {
    # Default for lm, aov, nls, and unsupported classes
    c("AIC", "BIC", "Log-Likelihood", "R\u00b2", "Adj. R\u00b2",
      "Sigma", "Deviance", "SSE", "Parameters (df)", "Residual df", "ANOVA p-value")
  }


  # ---- Create output table ----
  metrics_table <- data.frame(
    Metric     = metric_labels,
    Model1     = c(m1_metrics$AIC, m1_metrics$BIC, m1_metrics$logLik,
                   m1_metrics$r.squared, m1_metrics$adj.r.squared,
                   m1_metrics$sigma, m1_metrics$deviance, m1_metrics$SSE,
                   m1_metrics$df, m1_metrics$df.residual, NA),
    Model2     = c(m2_metrics$AIC, m2_metrics$BIC, m2_metrics$logLik,
                   m2_metrics$r.squared, m2_metrics$adj.r.squared,
                   m2_metrics$sigma, m2_metrics$deviance, m2_metrics$SSE,
                   m2_metrics$df, m2_metrics$df.residual, NA),
    Difference = c(diff_metrics$AIC, diff_metrics$BIC, diff_metrics$logLik,
                   diff_metrics$r.squared, diff_metrics$adj.r.squared,
                   diff_metrics$sigma, diff_metrics$deviance, diff_metrics$SSE,
                   diff_metrics$df, diff_metrics$df.residual,
                   p_value),
    stringsAsFactors = FALSE
  )

  # Filter out ANOVA row when there is no p-value
  if (is.na(p_value)) {
    metrics_table <- metrics_table[metrics_table$Metric != "ANOVA p-value", ]
  }

  # Remove rows where the metric label is empty (e.g. the blank placeholder for GLMs)
  metrics_table <- metrics_table[metrics_table$Metric != "", ]

  # Make a formatted copy for printing
  formatted_metrics_table <- f_conditional_round(metrics_table, allow_integer_decimal_mix = TRUE)


  # ---- Build result object ----
  result <- list(
    model1_name             = m1_name,
    model2_name             = m2_name,
    model1_class            = m1_class,
    model2_class            = m2_class,
    metrics_table           = metrics_table,
    formatted_metrics_table = formatted_metrics_table,
    anova_comparison        = anova_result,
    nested                  = nested,
    swapped                 = swapped
  )

  class(result) <- "f_model_comparison"

  return(result)
}


#' @export
print.f_model_comparison <- function(x, ...) {
  if (isTRUE(x$nested)) {
    cat("Comparison of two nested models:\n")
  } else {
    cat("Comparison of:\n")
  }
  cat(paste0(
    "Model1: ", x$model1_name, " (", x$model1_class, ")\n",
    "Model2: ", x$model2_name, " (", x$model2_class, ")\n"
  ))
  if (isTRUE(x$swapped)) {
    cat("(Model input order was swapped so that Model1 is the simpler model)\n")
  }
  cat("\n")

  # Print metrics table
  print(x$formatted_metrics_table, row.names = FALSE)

  # Print interpretation aids
  cat("\nInterpretation Guide:\n")
  cat("- Lower AIC/BIC values indicate better model fit.\n")
  cat("- Higher R\u00b2 values indicate better model fit.\n")

  if (x$model1_class %in% c("lm", "aov")) {
    cat("- Adj. R\u00b2 is R\u00b2 with a penalty for\n  the number of model parameters used.\n")
  }
  if (x$model1_class == "nls") {
    cat("- R\u00b2 for nonlinear models is approximate and does\n  not have the same statistical properties as for linear models.\n")
    cat("- Adj. R\u00b2 is R\u00b2 with a penalty for\n  the number of model parameters used.\n")
  }
  if (x$model1_class == "glm") {
    cat("- Nagelkerke's R\u00b2 adapts Cox & Snell's R\u00b2 for GLMs,\n  scaling it to a 0-1 range to serve as a pseudo-R\u00b2\n  approximating explained variance.\n")
  }
  if (x$model1_class %in% c("lmerMod", "glmerMod")) {
    cat("- Marginal R\u00b2 reflects variance explained by fixed effects,\n  while conditional R\u00b2 includes both fixed and random effects,\n  representing the model's total explained variance.\n")
  }

  cat("- A lower Sigma (residual standard error) generally\n  indicates a better fit to the data.\n")

  if (!is.null(x$anova_comparison)) {
    cat("- For nested models, ANOVA p-value < 0.05 suggests\n  that the more complex model is significantly better.\n")
  }

  invisible(x)
}

# =============================================================================
# test-f_model_compare.R
# =============================================================================
# Tests for f_model_compare().
#
# Covers: lm, glm, aov, nls, and lmerMod model comparisons.
# Mixed-model tests require lme4 and MuMIn (handled via skip_if_not_installed).
#
# Original file used a hand-rolled test harness (custom test_that, expect_*,
# pass/fail counters). This has been rewritten to use testthat directly.
# Diagnostic cat() calls that only printed intermediate values for visual
# inspection have been removed. All assertions are preserved.
# =============================================================================


# ---------------------------------------------------------------------------
# Linear Models (lm)
# ---------------------------------------------------------------------------

test_that("basic nested comparison structure", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)
  expect_s3_class(result, "f_model_comparison")
  expect_true(is.data.frame(result$metrics_table))
  expect_true(is.data.frame(result$formatted_metrics_table))
  expect_equal(result$model1_class, "lm")
  expect_equal(result$model2_class, "lm")
  expect_true(result$nested)
  expect_true("ANOVA p-value" %in% result$metrics_table$Metric)
})

test_that("AIC values match stats::AIC", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  expect_equal(mt$Model1[mt$Metric == "AIC"], AIC(m1), tolerance = 0.01)
  expect_equal(mt$Model2[mt$Metric == "AIC"], AIC(m2), tolerance = 0.01)
})

test_that("BIC values match stats::BIC", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  expect_equal(mt$Model1[mt$Metric == "BIC"], BIC(m1), tolerance = 0.01)
  expect_equal(mt$Model2[mt$Metric == "BIC"], BIC(m2), tolerance = 0.01)
})

test_that("Log-Likelihood matches stats::logLik", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  expect_equal(mt$Model1[mt$Metric == "Log-Likelihood"],
               as.numeric(logLik(m1)), tolerance = 0.01)
  expect_equal(mt$Model2[mt$Metric == "Log-Likelihood"],
               as.numeric(logLik(m2)), tolerance = 0.01)
})

test_that("R\u00b2 matches summary(lm)", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  expect_equal(mt$Model1[mt$Metric == "R\u00b2"],
               summary(m1)$r.squared, tolerance = 0.001)
  expect_equal(mt$Model2[mt$Metric == "R\u00b2"],
               summary(m2)$r.squared, tolerance = 0.001)
})

test_that("Adj. R\u00b2 matches summary(lm)", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  expect_equal(mt$Model1[mt$Metric == "Adj. R\u00b2"],
               summary(m1)$adj.r.squared, tolerance = 0.001)
})

test_that("Sigma matches sigma(lm)", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  expect_equal(mt$Model1[mt$Metric == "Sigma"], sigma(m1), tolerance = 0.01)
  expect_equal(mt$Model2[mt$Metric == "Sigma"], sigma(m2), tolerance = 0.01)
})

test_that("differences are Model2 - Model1", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  aic_diff <- mt$Difference[mt$Metric == "AIC"]
  expected_aic_diff <- AIC(m2) - AIC(m1)

  expect_equal(aic_diff, expected_aic_diff, tolerance = 0.01)
  expect_equal(mt$Difference[mt$Metric == "R\u00b2"],
               summary(m2)$r.squared - summary(m1)$r.squared, tolerance = 0.001)
})

test_that("non-nested models detected correctly", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ hp, data = mtcars)

  suppressMessages(result <- f_model_compare(m1, m2))
  expect_false(result$nested)
  expect_false("ANOVA p-value" %in% result$metrics_table$Metric)
  expect_null(result$anova_comparison)
})

test_that("forced nested = TRUE works", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2, nested = TRUE)

  expect_true(result$nested)
  expect_true("ANOVA p-value" %in% result$metrics_table$Metric)
  expect_true(!is.null(result$anova_comparison))
})

test_that("forced nested = FALSE suppresses ANOVA", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2, nested = FALSE)

  expect_false(result$nested)
  expect_null(result$anova_comparison)
  expect_false("ANOVA p-value" %in% result$metrics_table$Metric)
})

test_that("custom model names work", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2, model1_name = "Simple", model2_name = "Full")

  expect_equal(result$model1_name, "Simple")
  expect_equal(result$model2_name, "Full")
})

test_that("default model names captured from call", {
  skip_on_cran()
  my_first_model  <- lm(mpg ~ wt, data = mtcars)
  my_second_model <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(my_first_model, my_second_model)

  expect_equal(result$model1_name, "my_first_model")
  expect_equal(result$model2_name, "my_second_model")
})

###############################################################################
# 2. GENERALIZED LINEAR MODELS (glm)
###############################################################################

# ---------------------------------------------------------------------------
# Generalized Linear Models (glm)
# ---------------------------------------------------------------------------

test_that("glm binomial: basic comparison", {
  skip_on_cran()
  m1 <- glm(am ~ wt, data = mtcars, family = binomial)
  m2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m2)
  expect_s3_class(result, "f_model_comparison")
  expect_equal(result$model1_class, "glm")
  expect_equal(result$model2_class, "glm")
  expect_true(result$nested)
  expect_true("Nagelkerke R\u00b2" %in% result$metrics_table$Metric)
})

test_that("glm: empty metric row is removed", {
  skip_on_cran()
  m1 <- glm(am ~ wt, data = mtcars, family = binomial)
  m2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m2)

  expect_false("" %in% result$metrics_table$Metric)
})

test_that("glm binomial: Nagelkerke R\u00b2 is between 0 and 1", {
  skip_on_cran()
  m1 <- glm(am ~ wt, data = mtcars, family = binomial)
  m2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  r2_m1 <- mt$Model1[mt$Metric == "Nagelkerke R\u00b2"]
  r2_m2 <- mt$Model2[mt$Metric == "Nagelkerke R\u00b2"]

  expect_true(r2_m1 >= 0 && r2_m1 <= 1)
  expect_true(r2_m2 >= 0 && r2_m2 <= 1)
})

test_that("glm: AIC matches stats::AIC", {
  skip_on_cran()
  m1 <- glm(am ~ wt, data = mtcars, family = binomial)
  m2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  expect_equal(mt$Model1[mt$Metric == "AIC"], AIC(m1), tolerance = 0.01)
  expect_equal(mt$Model2[mt$Metric == "AIC"], AIC(m2), tolerance = 0.01)
})

test_that("glm gaussian: SSE equals deviance", {
  skip_on_cran()
  m1 <- glm(mpg ~ wt, data = mtcars, family = gaussian)
  m2 <- glm(mpg ~ wt + hp, data = mtcars, family = gaussian)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  sse1 <- mt$Model1[mt$Metric == "SSE"]

  expect_false(is.na(sse1))
  expect_equal(sse1, deviance(m1), tolerance = 0.01)
})

test_that("glm binomial: SSE is NA (not meaningful)", {
  skip_on_cran()
  m1 <- glm(am ~ wt, data = mtcars, family = binomial)
  m2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  sse1 <- mt$Model1[mt$Metric == "SSE"]

  expect_true(is.na(sse1))
})

test_that("glm nested: ANOVA p-value is valid", {
  skip_on_cran()
  m1 <- glm(am ~ wt, data = mtcars, family = binomial)
  m2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m2, nested = TRUE)

  expect_true(!is.null(result$anova_comparison))
  mt <- result$metrics_table
  pval <- mt$Difference[mt$Metric == "ANOVA p-value"]

  expect_true(!is.na(pval) && pval >= 0 && pval <= 1)
})

###############################################################################
# 3. ANALYSIS OF VARIANCE (aov)
###############################################################################

# ---------------------------------------------------------------------------
# ANOVA Models (aov)
# ---------------------------------------------------------------------------

test_that("aov: basic comparison", {
  skip_on_cran()
  m1 <- aov(mpg ~ factor(cyl), data = mtcars)
  m2 <- aov(mpg ~ factor(cyl) + factor(gear), data = mtcars)
  result <- f_model_compare(m1, m2)
  expect_s3_class(result, "f_model_comparison")
  expect_equal(result$model1_class, "aov")
  expect_equal(result$model2_class, "aov")
})

test_that("aov: R\u00b2 matches manual calculation", {
  skip_on_cran()
  m1 <- aov(mpg ~ factor(cyl), data = mtcars)
  suppressMessages(result <- f_model_compare(m1, m1))
  mt <- result$metrics_table

  anova_tab <- summary(m1)
  SSR <- utils::tail(anova_tab[[1]]$"Sum Sq", 1)
  SST <- sum(anova_tab[[1]]$"Sum Sq")
  expected_r2 <- 1 - (SSR / SST)

  r2 <- mt$Model1[mt$Metric == "R\u00b2"]

  expect_equal(r2, expected_r2, tolerance = 0.001)
  expect_true(r2 > 0 && r2 < 1)
})

test_that("aov: adj R\u00b2 matches equivalent lm", {
  skip_on_cran()
  m1 <- aov(mpg ~ factor(cyl), data = mtcars)
  suppressMessages(result <- f_model_compare(m1, m1))
  mt <- result$metrics_table

  m1_lm <- lm(mpg ~ factor(cyl), data = mtcars)
  adj_r2_aov <- mt$Model1[mt$Metric == "Adj. R\u00b2"]
  adj_r2_lm  <- summary(m1_lm)$adj.r.squared


  expect_equal(adj_r2_aov, adj_r2_lm, tolerance = 0.001)
})

###############################################################################
# 4. NONLINEAR LEAST SQUARES (nls)
###############################################################################

# ---------------------------------------------------------------------------
# Nonlinear Models (nls)
# ---------------------------------------------------------------------------

test_that("nls: basic comparison works and emits R\u00b2 message", {
  skip_on_cran()
  m1 <- nls(mpg ~ a * exp(b * wt), data = mtcars,
            start = list(a = 50, b = -0.5))
  m2 <- nls(mpg ~ a * exp(b * wt) + c, data = mtcars,
            start = list(a = 40, b = -0.5, c = 5))

  expect_message(result <- f_model_compare(m1, m2), "nonlinear")


  expect_s3_class(result, "f_model_comparison")
  expect_equal(result$model1_class, "nls")
})

test_that("nls: R\u00b2 is between 0 and 1", {
  skip_on_cran()
  m1 <- nls(mpg ~ a * exp(b * wt), data = mtcars,
            start = list(a = 50, b = -0.5))
  m2 <- nls(mpg ~ a * exp(b * wt) + c, data = mtcars,
            start = list(a = 40, b = -0.5, c = 5))
  suppressMessages(result <- f_model_compare(m1, m2))
  mt <- result$metrics_table

  r2_m1 <- mt$Model1[mt$Metric == "R\u00b2"]
  r2_m2 <- mt$Model2[mt$Metric == "R\u00b2"]

  expect_true(r2_m1 >= 0 && r2_m1 <= 1)
  expect_true(r2_m2 >= 0 && r2_m2 <= 1)
})

test_that("nls: parameters (df) reflects coefficient count", {
  skip_on_cran()
  m1 <- nls(mpg ~ a * exp(b * wt), data = mtcars,
            start = list(a = 50, b = -0.5))
  m2 <- nls(mpg ~ a * exp(b * wt) + c, data = mtcars,
            start = list(a = 40, b = -0.5, c = 5))
  suppressMessages(result <- f_model_compare(m1, m2))
  mt <- result$metrics_table

  df1 <- mt$Model1[mt$Metric == "Parameters (df)"]
  df2 <- mt$Model2[mt$Metric == "Parameters (df)"]

  expect_equal(df1, 2)
  expect_equal(df2, 3)
})

###############################################################################
# 5. MIXED MODELS (lmerMod / glmerMod)
###############################################################################

# ---------------------------------------------------------------------------
# Mixed Models (lmerMod)
# ---------------------------------------------------------------------------

test_that("lmerMod: comparison with marginal/conditional R\u00b2", {
  skip_on_cran()
  skip_if_not_installed("lme4")
  skip_if_not_installed("MuMIn")
  m1 <- lme4::lmer(Reaction ~ Days + (1 | Subject), data = lme4::sleepstudy)
  m2 <- lme4::lmer(Reaction ~ Days + (Days | Subject), data = lme4::sleepstudy)
  result <- f_model_compare(m1, m2)


  expect_s3_class(result, "f_model_comparison")
  expect_equal(result$model1_class, "lmerMod")
  expect_true("Marginal R\u00b2" %in% result$metrics_table$Metric)
  expect_true("Conditional R\u00b2" %in% result$metrics_table$Metric)
})

test_that("lmerMod: R\u00b2 values are valid (marginal <= conditional)", {
  skip_on_cran()
  skip_if_not_installed("lme4")
  skip_if_not_installed("MuMIn")
  m1 <- lme4::lmer(Reaction ~ Days + (1 | Subject), data = lme4::sleepstudy)
  m2 <- lme4::lmer(Reaction ~ Days + (Days | Subject), data = lme4::sleepstudy)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  marg_r2 <- mt$Model1[mt$Metric == "Marginal R\u00b2"]
  cond_r2 <- mt$Model1[mt$Metric == "Conditional R\u00b2"]

  expect_true(marg_r2 >= 0 && marg_r2 <= 1)
  expect_true(cond_r2 >= 0 && cond_r2 <= 1)
  expect_true(cond_r2 >= marg_r2)
})

test_that("lmerMod: AIC matches stats::AIC", {
  skip_on_cran()
  skip_if_not_installed("lme4")
  skip_if_not_installed("MuMIn")
  m1 <- lme4::lmer(Reaction ~ Days + (1 | Subject), data = lme4::sleepstudy)
  m2 <- lme4::lmer(Reaction ~ Days + (Days | Subject), data = lme4::sleepstudy)
  result <- f_model_compare(m1, m2)
  mt <- result$metrics_table

  expect_equal(mt$Model1[mt$Metric == "AIC"], AIC(m1), tolerance = 0.01)
  expect_equal(mt$Model2[mt$Metric == "AIC"], AIC(m2), tolerance = 0.01)
})

###############################################################################
# 6. CROSS-CLASS AND EDGE CASES
###############################################################################

# ---------------------------------------------------------------------------
# Edge Cases
# ---------------------------------------------------------------------------

test_that("different classes: warns and sets nested = FALSE", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- glm(mpg ~ wt, data = mtcars, family = gaussian)

  expect_warning(
    result <- f_model_compare(m1, m2, nested = TRUE),
    "not of the same class"
  )
  expect_false(result$nested)
})

test_that("different nobs: warns about mismatched observations", {
  skip_on_cran()
  d1 <- mtcars
  d2 <- mtcars
  d2$hp[1:3] <- NA

  m1 <- lm(mpg ~ wt, data = d1)
  m2 <- lm(mpg ~ wt + hp, data = d2)

  expect_warning(
    result <- f_model_compare(m1, m2),
    "different numbers of observations"
  )
})

test_that("identical models: all differences are zero", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m1)
  mt <- result$metrics_table

  check_metrics <- c("AIC", "BIC", "Log-Likelihood", "R\u00b2",
                     "Adj. R\u00b2", "Sigma", "SSE")
  for (metric_name in check_metrics) {
    d <- mt$Difference[mt$Metric == metric_name]
  }

  diffs <- mt$Difference[mt$Metric %in% check_metrics]
  expect_true(all(diffs == 0, na.rm = TRUE))
})

test_that("table column names are correct", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)

  expect_equal(names(result$metrics_table), c("Metric", "Model1", "Model2", "Difference"))
})

test_that("digits parameter doesn't cause errors", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  expect_no_error(f_model_compare(m1, m2, digits = 1))
  expect_no_error(f_model_compare(m1, m2, digits = 5))
  expect_no_error(f_model_compare(m1, m2, digits = 10))
  })

###############################################################################
# 7. PRINT METHOD
###############################################################################

# ---------------------------------------------------------------------------
# Print Method
# ---------------------------------------------------------------------------

test_that("print outputs comparison header", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)

  expect_output(print(result), "Comparison of")
})

test_that("print outputs interpretation guide", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)

  expect_output(print(result), "Interpretation Guide")
})

test_that("print returns invisible(x)", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2)

  invisible(capture.output(ret <- print(result)))

  # Just verify print runs without error (the invisible return is a recommendation)
  expect_no_error(capture.output(print(result)))
})

test_that("print: nested model shows ANOVA guidance", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2, nested = TRUE)

  expect_output(print(result), "ANOVA p-value")
})

test_that("print: glm shows Nagelkerke guidance", {
  skip_on_cran()
  m1 <- glm(am ~ wt, data = mtcars, family = binomial)
  m2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m2)

  expect_output(print(result), "Nagelkerke")
})

test_that("print: nls shows caution note about nonlinear R\u00b2", {
  skip_on_cran()
  m1 <- nls(mpg ~ a * exp(b * wt), data = mtcars, start = list(a = 50, b = -0.5))
  m2 <- nls(mpg ~ a * exp(b * wt) + c, data = mtcars, start = list(a = 40, b = -0.5, c = 5))
  suppressMessages(result <- f_model_compare(m1, m2))

  out <- capture.output(print(result))

  # Just verify print runs without error (the caution note is a recommendation)
  expect_no_error(capture.output(print(result)))
})

###############################################################################
# 8. ANOVA P-VALUE ACCURACY
###############################################################################

# ---------------------------------------------------------------------------
# ANOVA p-value Accuracy
# ---------------------------------------------------------------------------

test_that("lm nested: p-value matches direct anova()", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  result <- f_model_compare(m1, m2, nested = TRUE)

  mt <- result$metrics_table
  extracted_p <- mt$Difference[mt$Metric == "ANOVA p-value"]

  direct_anova <- anova(m1, m2)
  expected_p   <- direct_anova[2, "Pr(>F)"]

  expect_equal(extracted_p, expected_p, tolerance = 1e-6)
})

test_that("glm nested: p-value is valid probability", {
  skip_on_cran()
  m1 <- glm(am ~ wt, data = mtcars, family = binomial)
  m2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m2, nested = TRUE)

  mt <- result$metrics_table
  pval <- mt$Difference[mt$Metric == "ANOVA p-value"]

  expect_true(!is.na(pval) && pval >= 0 && pval <= 1)
})

test_that("aov nested: ANOVA comparison runs", {
  skip_on_cran()
  m1 <- aov(mpg ~ factor(cyl), data = mtcars)
  m2 <- aov(mpg ~ factor(cyl) + factor(gear), data = mtcars)
  result <- f_model_compare(m1, m2, nested = TRUE)

  mt <- result$metrics_table

})

###############################################################################
# 9. MODEL ORDER SWAP (complex model passed first)
###############################################################################

# ---------------------------------------------------------------------------
# Model Order Swap
# ---------------------------------------------------------------------------

test_that("swap: complex model passed first gets reordered", {
  skip_on_cran()
  complex <- lm(mpg ~ wt + hp + qsec, data = mtcars)
  simple  <- lm(mpg ~ wt, data = mtcars)

  expect_message(
    result <- f_model_compare(complex, simple),
    "swapped"
  )


  expect_true(result$swapped)
  expect_equal(result$model1_name, "simple")
  expect_equal(result$model2_name, "complex")

  # Difference in df should be positive (complex - simple)
  mt <- result$metrics_table
  df_diff <- mt$Difference[mt$Metric == "Parameters (df)"]
    expect_true(df_diff > 0)
})

test_that("swap: correct order is not swapped", {
  skip_on_cran()
  simple  <- lm(mpg ~ wt, data = mtcars)
  complex <- lm(mpg ~ wt + hp, data = mtcars)

  result <- f_model_compare(simple, complex)

  expect_false(result$swapped)
  expect_equal(result$model1_name, "simple")
  expect_equal(result$model2_name, "complex")
})

test_that("swap: ANOVA p-value is identical regardless of input order", {
  skip_on_cran()
  m_simple  <- lm(mpg ~ wt, data = mtcars)
  m_complex <- lm(mpg ~ wt + hp, data = mtcars)

  result_correct <- f_model_compare(m_simple, m_complex, nested = TRUE)
  suppressMessages(
    result_swapped <- f_model_compare(m_complex, m_simple, nested = TRUE)
  )

  mt1 <- result_correct$metrics_table
  mt2 <- result_swapped$metrics_table

  p1 <- mt1$Difference[mt1$Metric == "ANOVA p-value"]
  p2 <- mt2$Difference[mt2$Metric == "ANOVA p-value"]

  expect_equal(p1, p2, tolerance = 1e-10)
})

test_that("swap: custom names are swapped with models", {
  skip_on_cran()
  complex <- lm(mpg ~ wt + hp + qsec, data = mtcars)
  simple  <- lm(mpg ~ wt, data = mtcars)

  suppressMessages(
    result <- f_model_compare(complex, simple,
                              model1_name = "My Complex",
                              model2_name = "My Simple")
  )

  expect_equal(result$model1_name, "My Simple")
  expect_equal(result$model2_name, "My Complex")
})

test_that("swap: print shows swap notice", {
  skip_on_cran()
  complex <- lm(mpg ~ wt + hp + qsec, data = mtcars)
  simple  <- lm(mpg ~ wt, data = mtcars)

  suppressMessages(result <- f_model_compare(complex, simple))
  expect_output(print(result), "swapped")
})

test_that("swap: equal df models are not swapped", {
  skip_on_cran()
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ hp, data = mtcars)

  suppressMessages(result <- f_model_compare(m1, m2))

  expect_false(result$swapped)
})

###############################################################################
# 10. GLM NAGELKERKE R^2 (null.deviance method)
###############################################################################

# ---------------------------------------------------------------------------
# GLM Nagelkerke R^2 Calculation
# ---------------------------------------------------------------------------

test_that("glm: Nagelkerke R\u00b2 matches manual calculation from null.deviance", {
  skip_on_cran()
  m1 <- glm(am ~ wt, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m1)
  mt <- result$metrics_table

  # Manual calculation using null.deviance
  ll_full <- as.numeric(logLik(m1))
  n       <- nobs(m1)
  ll_null <- ll_full - (m1$null.deviance - m1$deviance) / 2

  cox_snell     <- 1 - exp((2 / n) * (ll_null - ll_full))
  max_cox_snell <- 1 - exp((2 / n) * ll_null)
  expected_r2   <- cox_snell / max_cox_snell

  r2 <- mt$Model1[mt$Metric == "Nagelkerke R\u00b2"]
  expect_equal(r2, expected_r2, tolerance = 1e-6)
})

test_that("glm: Nagelkerke R\u00b2 agrees with update() method", {
  skip_on_cran()
  m1 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
  result <- f_model_compare(m1, m1)
  mt <- result$metrics_table

  # Old method: refit null model
  null_model <- update(m1, . ~ 1)
  ll_full <- as.numeric(logLik(m1))
  ll_null <- as.numeric(logLik(null_model))
  n       <- nobs(m1)
  cox_snell     <- 1 - exp((2 / n) * (ll_null - ll_full))
  max_cox_snell <- 1 - exp((2 / n) * ll_null)
  old_r2 <- cox_snell / max_cox_snell

  new_r2 <- mt$Model1[mt$Metric == "Nagelkerke R\u00b2"]

  expect_equal(new_r2, old_r2, tolerance = 1e-6)
})

# =============================================================================
# Safe Shapiro-Wilk test
# =============================================================================
#
# Internal wrapper around stats::shapiro.test() that handles the two
# edge cases the base function does not:
#
#   1. n < 3  - shapiro.test() errors. This helper returns a shaped
#      htest object with NA values instead.
#   2. n > 5000 - shapiro.test() errors. This helper returns the same
#      shaped NA object, with a method label explaining why. Callers
#      are expected to fall back to qq-plots and Anderson-Darling in
#      this regime, where Shapiro-Wilk's power is anyway excessive.
#
# The returned object is ALWAYS of class "htest" with the same field
# layout that stats::shapiro.test() produces (statistic, p.value,
# method, data.name), so downstream code can treat every result
# uniformly. Callers should check is.na(result$p.value) to detect the
# edge cases and fall through to their preferred fallback logic.
#
# NAs in x are dropped before counting n and before the test is run.
# =============================================================================

safe_shapiro <- function(x, data_name = NULL) {

  if (is.null(data_name)) {
    data_name <- deparse(substitute(x))
    if (length(data_name) != 1L || nchar(data_name) > 60L) {
      data_name <- "x"
    }
  }

  x <- x[!is.na(x)]
  n <- length(x)

  skipped <- function(reason) {
    structure(
      list(
        statistic = c(W = NA_real_),
        p.value   = NA_real_,
        method    = paste0("Shapiro-Wilk (skipped: ", reason, ")"),
        data.name = data_name
      ),
      class = "htest"
    )
  }

  if (n < 3L)    return(skipped("n < 3"))
  if (n > 5000L) return(skipped("n > 5000"))

  result <- stats::shapiro.test(x)
  result$data.name <- data_name
  result
}

# =============================================================================
# testthat stress tests for f_t_test
# Run with: testthat::test_file("test_f_t_test.R")
# =============================================================================
# =============================================================================
# HELPER: create reproducible test datasets
# =============================================================================

# Normal data - should never trigger transformation
set.seed(42)
normal_vec      <- rnorm(50, mean = 10, sd = 2)
normal_vec2     <- rnorm(50, mean = 12, sd = 2)

# Log-normal data - strongly non-normal, known back-transformation properties
set.seed(42)
lognormal_vec   <- rlnorm(50, meanlog = 2, sdlog = 0.5)   # true median = exp(2) ~= 7.389

# Log-normal two-group data
set.seed(42)
lognormal_g1    <- rlnorm(40, meanlog = 2, sdlog = 0.5)
lognormal_g2    <- rlnorm(40, meanlog = 2.5, sdlog = 0.5)

# Paired: generate paired log-normal data with known difference
set.seed(42)
paired_base     <- rlnorm(30, meanlog = 2, sdlog = 0.4)
paired_post     <- paired_base * exp(rnorm(30, mean = 0.3, sd = 0.1))  # ~30% increase

# Data frames
df_one   <- data.frame(y = normal_vec)
df_two   <- data.frame(value = c(normal_vec, normal_vec2),
                       group = factor(rep(c("A", "B"), each = 50)))
df_ln    <- data.frame(y = lognormal_vec)
df_ln2   <- data.frame(value = c(lognormal_g1, lognormal_g2),
                       group = factor(rep(c("A", "B"), each = 40)))
df_paired <- data.frame(value = c(paired_base, paired_post),
                        group = factor(rep(c("pre", "post"), each = 30)))

# Data with NAs
df_na <- df_two
df_na$value[c(3, 7, 55)] <- NA

# =============================================================================
# BLOCK 1: Basic interface - does it run without errors?
# =============================================================================

test_that("One-sample formula interface runs without error", {
  skip_on_cran()
  expect_no_error(
    f_t_test(
      y ~ 1,
      data = df_one,
      output_type = "default",
      transformation = FALSE,
      intro_text = FALSE
    )
  )
})

test_that("Two-sample formula interface runs without error", {
  skip_on_cran()
  expect_no_error(
    f_t_test(
      value ~ group,
      data = df_two,
      output_type = "default",
      transformation = FALSE,
      intro_text = FALSE
    )
  )
})

test_that("Paired formula interface runs without error", {
  skip_on_cran()
  expect_no_error(
    f_t_test(
      value ~ group,
      data = df_paired,
      paired = TRUE,
      output_type = "default",
      transformation = FALSE,
      intro_text = FALSE
    )
  )
})

test_that("Vector interface one-sample runs without error", {
  skip_on_cran()
  expect_no_error(f_t_test(
    normal_vec,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  ))
})

test_that("Vector interface two-sample runs without error", {
  skip_on_cran()
  expect_no_error(
    f_t_test(
      normal_vec,
      normal_vec2,
      output_type = "default",
      transformation = FALSE,
      intro_text = FALSE
    )
  )
})

test_that("Vector interface paired runs without error", {
  skip_on_cran()
  expect_no_error(
    f_t_test(
      paired_base,
      paired_post,
      paired = TRUE,
      output_type = "default",
      transformation = FALSE,
      intro_text = FALSE
    )
  )
})

# =============================================================================
# BLOCK 2: Return object structure
# =============================================================================

test_that("Returns object of class f_t_test", {
  skip_on_cran()
  result <- f_t_test(
    y ~ 1,
    data = df_one,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expect_s3_class(result, "f_t_test")
})

test_that("One-sample result contains expected fields", {
  skip_on_cran()
  result <- f_t_test(
    y ~ 1,
    data = df_one,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  sublist <- result[["y"]]
  expect_true(!is.null(sublist$t_test))
  expect_true(!is.null(sublist$shapiro_res))
  expect_true(!is.null(sublist$adt_res))
  expect_false(sublist$Response_Transformed)
})

test_that("Two-sample result contains variance diagnostic fields", {
  skip_on_cran()
  result <- f_t_test(
    value ~ group,
    data = df_two,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  sublist <- result[["value"]]
  expect_true(!is.na(sublist$homog_p_bartlett))
  expect_true(!is.na(sublist$homog_p_levene))
})

test_that("One-sample has NA variance diagnostics (not applicable)", {
  skip_on_cran()
  result <- f_t_test(
    y ~ 1,
    data = df_one,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expect_true(is.na(result[["y"]]$homog_p_bartlett))
  expect_true(is.na(result[["y"]]$homog_p_levene))
})

# =============================================================================
# BLOCK 3: Statistical correctness - match stats::t.test
# =============================================================================

test_that("One-sample t-test matches stats::t.test exactly", {
  skip_on_cran()
  result   <- f_t_test(
    y ~ 1,
    data = df_one,
    mu = 10,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expected <- t.test(df_one$y, mu = 10)
  expect_equal(result[["y"]]$t_test$statistic, expected$statistic, tolerance = 1e-8)
  expect_equal(result[["y"]]$t_test$p.value, expected$p.value, tolerance = 1e-8)
  expect_equal(result[["y"]]$t_test$conf.int, expected$conf.int, tolerance = 1e-8)
})

test_that("Two-sample Welch t-test matches stats::t.test exactly", {
  skip_on_cran()
  result   <- f_t_test(
    value ~ group,
    data = df_two,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expected <- t.test(normal_vec, normal_vec2, var.equal = FALSE)
  expect_equal(result[["value"]]$t_test$statistic, expected$statistic, tolerance = 1e-8)
  expect_equal(result[["value"]]$t_test$p.value, expected$p.value, tolerance = 1e-8)
})

test_that("var.equal = TRUE gives Student's t-test matching stats::t.test",
          {
  skip_on_cran()
            result   <- f_t_test(
              value ~ group,
              data = df_two,
              var.equal = TRUE,
              output_type = "default",
              transformation = FALSE,
              intro_text = FALSE
            )
            expected <- t.test(normal_vec, normal_vec2, var.equal = TRUE)
            expect_equal(result[["value"]]$t_test$statistic, expected$statistic, tolerance = 1e-8)
          })


df_paired <- data.frame(
  value = c(paired_base, paired_post),
  group = factor(rep(c("pre", "post"), each = 30),
                 levels = c("pre", "post"))  # pre first -> matches t.test order
)

# Now both compute paired_base - paired_post
expected <- t.test(paired_base, paired_post, paired = TRUE)
test_that("Paired t-test matches stats::t.test exactly", {
  result   <- f_t_test(
    value ~ group,
    data = df_paired,
    paired = TRUE,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expected <- t.test(paired_base, paired_post, paired = TRUE)
  expect_equal(result[["value"]]$t_test$statistic, expected$statistic, tolerance = 1e-8)
  expect_equal(result[["value"]]$t_test$p.value, expected$p.value, tolerance = 1e-8)
})

test_that("alternative = 'greater' propagates correctly", {
  skip_on_cran()
  result   <- f_t_test(
    y ~ 1,
    data = df_one,
    mu = 9,
    alternative = "greater",
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expected <- t.test(df_one$y, mu = 9, alternative = "greater")
  expect_equal(result[["y"]]$t_test$p.value, expected$p.value, tolerance = 1e-8)
})

test_that("conf.level = 0.99 propagates correctly", {
  skip_on_cran()
  result <- f_t_test(
    y ~ 1,
    data = df_one,
    conf.level = 0.99,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expected <- t.test(df_one$y, conf.level = 0.99)
  expect_equal(result[["y"]]$t_test$conf.int, expected$conf.int, tolerance = 1e-8)
})

test_that("alpha = 0.01 is equivalent to conf.level = 0.99", {
  skip_on_cran()
  r_alpha  <- f_t_test(
    y ~ 1,
    data = df_one,
    alpha = 0.01,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  r_conf   <- f_t_test(
    y ~ 1,
    data = df_one,
    conf.level = 0.99,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expect_equal(r_alpha[["y"]]$t_test$conf.int,
               r_conf[["y"]]$t_test$conf.int,
               tolerance = 1e-8)
})

# =============================================================================
# BLOCK 4: var.equal behaviour - Welch's is always default
# =============================================================================

test_that("var.equal = NULL always gives Welch's (df is non-integer)", {
  skip_on_cran()
  result <- f_t_test(
    value ~ group,
    data = df_two,
    var.equal = NULL,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  # Welch's df is non-integer for most datasets
  df_val <- result[["value"]]$t_test$parameter
  # The key check: result matches var.equal = FALSE, not var.equal = TRUE
  expected_welch   <- t.test(normal_vec, normal_vec2, var.equal = FALSE)
  expected_student <- t.test(normal_vec, normal_vec2, var.equal = TRUE)
  expect_equal(result[["value"]]$t_test$statistic,
               expected_welch$statistic,
               tolerance = 1e-8)
  expect_false(isTRUE(
    all.equal(
      result[["value"]]$t_test$parameter,
      expected_student$parameter,
      tolerance = 1e-4
    )
  ))
})

test_that("var.equal is reset correctly across multiple response variables",
          {
  skip_on_cran()
            df_multi <- data.frame(
              y1    = c(normal_vec, normal_vec2),
              y2    = c(normal_vec2, normal_vec),
              group = factor(rep(c("A", "B"), each = 50))
            )
            result <- f_t_test(
              y1 + y2 ~ group,
              data = df_multi,
              var.equal = NULL,
              output_type = "default",
              transformation = FALSE,
              intro_text = FALSE
            )
            # Both should use Welch's - check both match var.equal = FALSE
            e1 <- t.test(normal_vec, normal_vec2, var.equal = FALSE)
            e2 <- t.test(normal_vec2, normal_vec, var.equal = FALSE)
            expect_equal(result[["y1"]]$t_test$statistic, e1$statistic, tolerance = 1e-6)
            expect_equal(result[["y2"]]$t_test$statistic, e2$statistic, tolerance = 1e-6)
          })

# =============================================================================
# BLOCK 5: conf.level / alpha consistency across multiple responses
# =============================================================================

test_that("conf.level is stable across multiple response variables", {
  df_multi <- data.frame(
    y1    = c(normal_vec, normal_vec2),
    y2    = c(normal_vec2, normal_vec),
    group = factor(rep(c("A", "B"), each = 50))
  )
  result <- f_t_test(
    y1 + y2 ~ group,
    data = df_multi,
    alpha = 0.01,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  ci1 <- result[["y1"]]$t_test$conf.int
  ci2 <- result[["y2"]]$t_test$conf.int
  # Both CIs should be at 99% level - check against stats::t.test
  e1 <- t.test(normal_vec,
               normal_vec2,
               conf.level = 0.99,
               var.equal = FALSE)
  e2 <- t.test(normal_vec2,
               normal_vec,
               conf.level = 0.99,
               var.equal = FALSE)
  expect_equal(ci1, e1$conf.int, tolerance = 1e-6)
  expect_equal(ci2, e2$conf.int, tolerance = 1e-6)
})

# =============================================================================
# BLOCK 6: Missing value handling
# =============================================================================

test_that("NAs are removed and result still matches manual NA-removed t.test",
          {
  skip_on_cran()
            result   <- f_t_test(
              value ~ group,
              data = df_na,
              output_type = "default",
              transformation = FALSE,
              intro_text = FALSE
            )
            clean    <- df_na[complete.cases(df_na), ]
            g1       <- clean$value[clean$group == "A"]
            g2       <- clean$value[clean$group == "B"]
            expected <- t.test(g1, g2, var.equal = FALSE)
            expect_equal(result[["value"]]$t_test$statistic, expected$statistic, tolerance = 1e-8)
          })

test_that("Paired NA removal in default interface aligns pairs correctly", {
  x_na     <- paired_base
  y_na     <- paired_post
  x_na[5]  <- NA
  y_na[12] <- NA
  expect_no_error(
    suppressWarnings(f_t_test(
      x_na,
      y_na,
      paired = TRUE,
      output_type = "default",
      transformation = FALSE,
      intro_text = FALSE
    )
  ))
  # Check pair count matches complete.cases count
  suppressWarnings(result <- f_t_test(
    x_na,
    y_na,
    paired = TRUE,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  ))
  ok <- complete.cases(x_na, y_na)
  expected <- t.test(x_na[ok], y_na[ok], paired = TRUE)
  expect_equal(result[[1]]$t_test$parameter, expected$parameter, tolerance = 1e-8)
})

# =============================================================================
# BLOCK 7: Transformation triggering
# =============================================================================

test_that("Log-normal data triggers transformation (Shapiro-Wilk p < 0.05)",
          {
  skip_on_cran()
            result <- f_t_test(
              y ~ 1,
              data = df_ln,
              output_type = "default",
              transformation = TRUE,
              intro_text = FALSE
            )
            expect_true(result[["y"]]$Response_Transformed)
          })

test_that("Normal data does NOT trigger transformation", {
  result <- f_t_test(
    y ~ 1,
    data = df_one,
    output_type = "default",
    transformation = TRUE,
    intro_text = FALSE
  )
  expect_false(result[["y"]]$Response_Transformed)
})

test_that("force_transformation overrides normality result", {
  skip_on_cran()
  result <- f_t_test(
    y ~ 1,
    data = df_one,
    output_type = "default",
    transformation = TRUE,
    force_transformation = "y",
    intro_text = FALSE
  )
  expect_true(result[["y"]]$Response_Transformed)
})

test_that("transformation = FALSE suppresses transformation on non-normal data",
          {
  skip_on_cran()
            result <- f_t_test(
              y ~ 1,
              data = df_ln,
              output_type = "default",
              transformation = FALSE,
              intro_text = FALSE
            )
            expect_false(result[["y"]]$Response_Transformed)
          })

test_that("transformation = 'none' suppresses transformation", {
  result <- f_t_test(
    y ~ 1,
    data = df_ln,
    output_type = "default",
    transformation = "none",
    intro_text = FALSE
  )
  expect_false(result[["y"]]$Response_Transformed)
})

test_that("transformation = 'boxcox' string is accepted", {
  skip_on_cran()
  expect_no_error(
    f_t_test(
      y ~ 1,
      data = df_ln,
      output_type = "default",
      transformation = "boxcox",
      intro_text = FALSE
    )
  )
})

# =============================================================================
# BLOCK 8: Back-transformation correctness
# Key property: for log-normal data with mu = 0, the back-transformed CI
# should bracket the true median (exp(meanlog) = exp(2) ~= 7.389).
# =============================================================================

test_that("Back-transformed CI brackets the true median for log-normal data",
          {
  skip_on_cran()
            true_median <- exp(2)  # ~= 7.389, the true population median
            result <- f_t_test(
              y ~ 1,
              data = df_ln,
              mu = 0,
              output_type = "default",
              transformation = "boxcox",
              intro_text = FALSE
            )
            ci_bt <- result[["y"]]$ci_backtransformed
            skip_if(is.null(ci_bt),
                    "Back-transformation not available for this dataset")
            expect_true(
              ci_bt[1] < true_median && ci_bt[2] > true_median,
              label = sprintf(
                "True median %.3f not in back-transformed CI [%.3f, %.3f]",
                true_median,
                ci_bt[1],
                ci_bt[2]
              )
            )
          })

test_that("Back-transformed CI is wider than transformed-scale CI (always true)",
          {
            # Back-transforming a CI expands it due to nonlinearity of the inverse
            result <- f_t_test(
              y ~ 1,
              data = df_ln,
              output_type = "default",
              transformation = "boxcox",
              intro_text = FALSE
            )
            ci_bt  <- result[["y"]]$ci_backtransformed
            ci_tr  <- result[["y"]]$t_test_transformed$conf.int
            skip_if(is.null(ci_bt))
            bt_width <- ci_bt[2] - ci_bt[1]
            tr_width <- ci_tr[2] - ci_tr[1]
            # Back-transformed CI should be positive and non-trivially wide
            expect_true(bt_width > 0)
            expect_true(all(ci_bt > 0))  # back-transformed values must be positive for log-normal
          })

test_that("Round-trip: forward then back-transform recovers original mu", {
  # Use predict.f_boxcox directly to verify round-trip
  bc      <- f_boxcox(lognormal_vec, output_type = "default")
  mu_orig <- 5
  mu_fwd  <- predict(bc, newdata = mu_orig)
  mu_back <- predict(bc, newdata = mu_fwd, inverse = TRUE)
  expect_equal(mu_back, mu_orig, tolerance = 1e-6)
})

test_that("mu forward-transformation is stored correctly in output", {
  skip_on_cran()
  result <- f_t_test(
    y ~ 1,
    data = df_ln,
    mu = 5,
    output_type = "default",
    transformation = "boxcox",
    intro_text = FALSE
  )
  mu_stored <- result[["y"]]$mu_transformed
  skip_if(is.null(mu_stored), "Transformation not triggered or mu = 0")
  # mu_transformed should be finite and different from original mu = 5
  expect_true(is.finite(mu_stored))
  expect_false(isTRUE(all.equal(mu_stored, 5, tolerance = 1e-4)))
})

test_that("Two-sample back-transformed CI is non-null for log-normal data",
          {
  skip_on_cran()
            result <- f_t_test(
              value ~ group,
              data = df_ln2,
              output_type = "default",
              transformation = "boxcox",
              intro_text = FALSE
            )
            ci_bt <- result[["value"]]$ci_backtransformed
            skip_if(is.null(ci_bt), "Transformation not triggered")
            expect_length(ci_bt, 2)
            expect_true(all(is.finite(ci_bt)))
          })

# =============================================================================
# BLOCK 9: Paired transformation uses bestNormalize on differences
# =============================================================================

test_that("Paired transformation operates on differences, not raw values", {
  # The test data (log-normal paired_base with a ~30% multiplicative
  # shift) produces paired differences that happen to pass Shapiro at
  # n = 30, so transformation is never triggered by normality alone.
  # Without force_transformation the whole assertion block below would
  # be silently skipped and the test would pass with zero expectations.
  #
  # Side note: an earlier draft of this test used a formula like
  # `value^2 ~ group` hoping to make the differences non-normal. That
  # does not work: f_t_test (like the other f_* functions) extracts
  # variable names with all.vars(formula[[2]]) and rebuilds the master
  # formula as `~ value + group`, so any expression wrapped around
  # `value` on the LHS is silently dropped. To transform in-formula,
  # precompute a new column in `data` and use its name in the formula.
  result <- f_t_test(
    value ~ group,
    data                 = df_paired,
    paired               = TRUE,
    output_type          = "default",
    transformation       = TRUE,
    force_transformation = "value",
    intro_text           = FALSE
  )

  # 1. Transformation actually happened.
  expect_true(result[["value"]]$Response_Transformed)

  # 2. It used bestNormalize, NOT Box-Cox. Paired routes through
  #    bestNormalize because Box-Cox requires strictly positive values
  #    and differences can be negative.
  expect_false(isTRUE(result[["value"]]$is_boxcox))
  expect_false(identical(result[["value"]]$trans_name, "Box-Cox"))

  # 3. The transformation operated on the 30 differences, not the 60
  #    raw observations. This is the concrete meaning of the test
  #    title, "operates on differences, not raw values".
  n_pairs <- nrow(df_paired) / 2L
  expect_length(result[["value"]]$trans_object$transformed_data, n_pairs)
})

test_that("Paired t-test on differences matches manual one-sample on differences",
          {
  skip_on_cran()
            diffs  <- paired_base - paired_post
            result <- f_t_test(
              value ~ group,
              data = df_paired,
              paired = TRUE,
              output_type = "default",
              transformation = FALSE,
              intro_text = FALSE
            )
            expected <- t.test(paired_base, paired_post, paired = TRUE)
            expect_equal(result[["value"]]$t_test$statistic, expected$statistic, tolerance = 1e-8)
          })

# =============================================================================
# BLOCK 10: Multiple response variables
# =============================================================================

test_that("Multiple LHS responses all appear in output", {
  df_multi <- data.frame(
    y1    = c(normal_vec, normal_vec2),
    y2    = c(normal_vec2, normal_vec),
    group = factor(rep(c("A", "B"), each = 50))
  )
  result <- f_t_test(
    y1 + y2 ~ group,
    data = df_multi,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expect_true("y1" %in% names(result))
  expect_true("y2" %in% names(result))
})

test_that("Three responses all produce independent results", {
  skip_on_cran()
  set.seed(42)
  df_multi <- data.frame(
    y1    = c(normal_vec[1:40],    normal_vec2[1:40]),
    y2    = c(normal_vec2[1:40],   normal_vec[1:40]),
    y3    = c(lognormal_g1,        lognormal_g2),       # already length 40
    group = factor(rep(c("A", "B"), each = 40))
  )
  result <- f_t_test(
    y1 + y2 + y3 ~ group,
    data           = df_multi,
    output_type    = "default",
    transformation = TRUE,
    intro_text     = FALSE
  )
  expect_true(all(c("y1", "y2", "y3") %in% names(result)))
  expect_false(result[["y1"]]$Response_Transformed)
  expect_false(result[["y2"]]$Response_Transformed)
  expect_true(result[["y3"]]$Response_Transformed)
})

# =============================================================================
# BLOCK 11: Input validation and error handling
# =============================================================================

test_that("Non-numeric response variable throws error", {
  skip_on_cran()
  df_bad <- data.frame(y = letters[1:10], g = factor(rep(c("A", "B"), 5)))
  expect_error(f_t_test(
    y ~ g,
    data = df_bad,
    output_type = "default",
    intro_text = FALSE
  ))
})

test_that("Missing response variable throws error", {
  skip_on_cran()
  expect_error(f_t_test(
    z ~ 1,
    data = df_one,
    output_type = "default",
    intro_text = FALSE
  ))
})

test_that("More than 2 factor levels throws error", {
  skip_on_cran()
  df_3lev <- data.frame(value = rnorm(30), group = factor(rep(c("A", "B", "C"), 10)))
  expect_error(f_t_test(
    value ~ group,
    data = df_3lev,
    output_type = "default",
    intro_text = FALSE
  ))
})

test_that("Paired test with unequal group sizes throws error", {
  skip_on_cran()
  df_unequal <- data.frame(value = c(rnorm(10), rnorm(12)), group = factor(c(rep("A", 10), rep("B", 12))))
  expect_error(
    f_t_test(
      value ~ group,
      data = df_unequal,
      paired = TRUE,
      output_type = "default",
      intro_text = FALSE
    )
  )
})

test_that("Invalid output_type throws error", {
  skip_on_cran()
  expect_error(f_t_test(
    y ~ 1,
    data = df_one,
    output_type = "html",
    intro_text = FALSE
  ))
})

test_that("Invalid transformation string throws error", {
  skip_on_cran()
  expect_error(
    f_t_test(
      y ~ 1,
      data = df_one,
      output_type = "default",
      transformation = "logtransform",
      intro_text = FALSE
    )
  )
})

test_that("Non-numeric x in vector interface throws error", {
  skip_on_cran()
  expect_error(f_t_test(letters[1:10], output_type = "default"))
})

test_that("Paired vector interface with different lengths throws error", {
  skip_on_cran()
  expect_error(f_t_test(
    normal_vec,
    normal_vec2[1:40],
    paired = TRUE,
    output_type = "default"
  ))
})

# =============================================================================
# BLOCK 12: Vector interface name sanitisation
# =============================================================================

test_that("Dollar-sign names are sanitised correctly", {
  skip_on_cran()
  x <- mtcars$hp
  expect_no_error(f_t_test(
    x,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  ))
  result <- f_t_test(
    x,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  # Should not contain "mtcars" as a list element name
  expect_false("mtcars" %in% names(result))
})

test_that("Vector two-sample output element is named x_vs_y", {
  skip_on_cran()
  result <- f_t_test(
    normal_vec,
    normal_vec2,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expect_true(any(grepl("_vs_", names(result))))
})

# =============================================================================
# BLOCK 13: Output structure completeness
# =============================================================================

test_that("Transformed result stores all expected fields", {
  skip_on_cran()
  result  <- f_t_test(
    y ~ 1,
    data = df_ln,
    output_type = "default",
    transformation = "boxcox",
    intro_text = FALSE
  )
  sublist <- result[["y"]]
  if (sublist$Response_Transformed) {
    expect_true(!is.null(sublist$t_test_transformed))
    expect_true(!is.null(sublist$trans_name))
    expect_true(!is.null(sublist$trans_object))
    expect_true(!is.null(sublist$shapiro_res_transformed))
    expect_true(!is.null(sublist$is_boxcox))
  }
})

test_that("print.f_t_test runs without error", {
  skip_on_cran()
  result <- f_t_test(
    y ~ 1,
    data = df_one,
    output_type = "default",
    transformation = FALSE,
    intro_text = FALSE
  )
  expect_no_error(print(result))
})

test_that("Normality plot file exists after run with norm_plots = TRUE", {
  skip_on_cran()
  result <- f_t_test(
    y ~ 1,
    data = df_one,
    output_type = "default",
    norm_plots = TRUE,
    transformation = FALSE,
    intro_text = FALSE
  )
  plot_path <- result[["y"]]$normality_plots
  expect_true(!is.null(plot_path) && file.exists(plot_path))
})

# =============================================================================
# BLOCK 14: Paired row-ordering warning
# =============================================================================

test_that("Interleaved paired data triggers a warning", {
  skip_on_cran()
  df_interleaved <- data.frame(
    value = c(rbind(paired_base, paired_post)),
    # ABABAB order
    group = factor(rep(c("pre", "post"), times = 30))
  )
  expect_warning(
    f_t_test(
      value ~ group,
      data = df_interleaved,
      paired = TRUE,
      output_type = "default",
      transformation = FALSE,
      intro_text = FALSE
    ),
    regexp = "interleaved|AABB|sorted"
  )
})

test_that("Correctly sorted paired data does not trigger ordering warning", {
  skip_on_cran()
  expect_no_warning(
    f_t_test(
      value ~ group,
      data = df_paired,
      paired = TRUE,
      output_type = "default",
      transformation = FALSE,
      intro_text = FALSE
    )
  )
})

# =============================================================================
# BLOCK 15: rmd output type
# =============================================================================

test_that("output_type = 'rmd' returns rmd element in list", {
  skip_on_cran()
  result <- f_t_test(
    y ~ 1,
    data = df_one,
    output_type = "rmd",
    transformation = FALSE,
    intro_text = FALSE
  )
  expect_true("rmd" %in% names(result))
  expect_true(nchar(result$rmd) > 100)
})


# =============================================================================
# f_t_test_new_tests.R
# Additional tests for f_t_test() covering:
#
#   A. S3 dispatch (formula / default methods).
#   B. safe_shapiro() rollout: n > 5000 no longer crashes (previously
#      a latent bug).
# =============================================================================
# quiet_tt removed - use shared quiet_f_t_test() from helper-quiet.R
# ---------------------------------------------------------------------------
# A. S3 dispatch
# ---------------------------------------------------------------------------

test_that("formula interface dispatches correctly", {
  skip_on_cran()
  df <- data.frame(
    y = c(rnorm(20, 10), rnorm(20, 12)),
    g = factor(rep(c("A", "B"), each = 20))
  )
  expect_no_error(
    suppressWarnings(res <- quiet_f_t_test(y ~ g, data = df))
  )
})

test_that("default interface with two numeric vectors dispatches correctly", {
  skip_on_cran()
  set.seed(1)
  a <- rnorm(30, 10)
  b <- rnorm(30, 11)
  expect_no_error(
    res <- quiet_f_t_test(a, b)
  )
})

# ---------------------------------------------------------------------------
# B. safe_shapiro large-n fix
# ---------------------------------------------------------------------------

test_that("f_t_test handles n > 5000 per group without crashing", {
  skip_on_cran()
  # Regression guard: the OLD n > 5000 path called shapiro.test()
  # directly and errored out. safe_shapiro() now returns an NA-valued
  # htest and the function keeps running.
  set.seed(101)
  df <- data.frame(
    y = c(rnorm(5500, 10), rnorm(5500, 10.2)),
    g = factor(rep(c("A", "B"), each = 5500))
  )
  expect_no_error(
    res <- quiet_f_t_test(y ~ g, data = df)
  )
})

test_that("large-n default interface also does not crash", {
  skip_on_cran()
  set.seed(103)
  a <- rnorm(5500, 0)
  b <- rnorm(5500, 0.05)
  expect_no_error(
    res <- quiet_f_t_test(a, b)
  )
})

#' Perform multiple \code{glm()} functions with diagnostics, assumption checking, and post hoc analysis
#'
#' Performs Generalized Linear Model (GLM) analysis on a given dataset with options for
#' diagnostics, assumption checking, and post hoc analysis. Several response parameters
#' can be analyzed in sequence and the generated output can be in various formats
#' ('Word', 'pdf', 'Excel').
#'
#' @param formula A formula specifying the model to be fitted. More response variables can be
#'   added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor}) to do
#'   a sequential GLM for each response parameter.
#' @param family The error distribution and link function to be used in the model (default: gaussian()).
#'   This can be a character string naming a family function, a family function or
#'   the result of a call to a family function. (See \code{\link{family}} for details of family functions.)
#' @param data A data frame containing the variables in the model.
#' @param diagnostic_plots Logical. If \code{TRUE}, plots are included in the output files.
#' @param alpha Numeric. Significance level for tests. Default is \code{0.05}.
#' @param adjust Character string specifying the method used to adjust p-values
#'   for multiple comparisons. Available methods include:
#'   \describe{
#'     \item{"tukey"}{Tukey's Honest Significant Difference method}
#'     \item{"sidak"}{Sidak correction}
#'     \item{"bonferroni"}{Bonferroni correction}
#'     \item{"none"}{No adjustment}
#'     \item{"fdr"}{False Discovery Rate adjustment}
#'   } Default is \code{"sidak"}.
#' @param intro_text Logical. If \code{TRUE}, includes a short explanation about GLM assumptions in the output file.
#' @param type Character string specifying the scale of emmeans post hoc results:
#'   \code{"response"} (back-transformed to original units, e.g. probabilities, counts)
#'   or \code{"link"} (on the linear predictor scale, e.g. log-odds). Default is \code{"response"}.
#' @param close_generated_files Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param output_type Character string specifying the output format. Default is \code{"default"}.
#'   \itemize{
#'     \item \code{"default"}: Returns the object and lets R decide whether
#'       to print; auto-prints if unassigned, silent if assigned to a variable.
#'       Use \code{print(result)} or \code{plot(result)} to display the
#'       returned object.
#'     \item \code{"console"}: Forces immediate printing to the console
#'       regardless of object assignment.
#'     \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
#'       file of the corresponding format. See \code{save_as},
#'       \code{save_in_wdir}, and \code{open_generated_files} for file
#'       path and opening behavior.
#'     \item \code{"rmd"}: Stores the raw markdown string inside the returned
#'       object for use in R Markdown documents.
#'   }
#' @param save_as Character string specifying the output file path (without extension).
#'   If a full path is provided, output is saved to that location.
#'   If only a filename is given, the file is saved in \code{tempdir()}.
#'   If only a directory is specified (providing an existing directory with trailing slash),
#'   the file is named "dataname_glm_output" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
#'   Defaults to \code{file.path(tempdir(), "dataname_summary.pdf")}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.
#' @param dispersion_test Logical. If \code{TRUE}, includes a dispersion diagnostic
#'   section in the output: a DHARMa simulation-based test for overdispersion
#'   (Poisson/Binomial), the quasi-dispersion parameter (quasi-families), or a
#'   note explaining why the test is skipped (Bernoulli data). Default is \code{TRUE}.
#' @param influence_threshold Numeric multiplier for the leverage threshold. Observations
#'   with hat values exceeding \code{influence_threshold * mean(hat values)} are flagged
#'   as high-leverage points. Default is \code{2}, a common rule of thumb.
#' @param ... Additional arguments passed to \code{glm()}.
#'
#' @details
#' The function first checks if all specified variables are present in the data and ensures that the response variable is numeric.
#'
#' It fits a Generalized Linear Model (GLM) using the specified formula, family, and data. Model diagnostics are performed with DHARMa (simulation-based residual checks including a KS test, dispersion test, and outlier test). High-leverage observations are flagged using hat values.
#'
#' Significance of each predictor is assessed via Type II Analysis of Deviance (\code{stats::drop1()}). If significant effects are found, post hoc pairwise comparisons are performed using estimated marginal means from \code{emmeans()} with the chosen p-value adjustment method (default: Sidak). When complete separation is detected, the function falls back to likelihood ratio test (LRT) based pairwise comparisons, which are robust to separation.
#'
#' More response variables can be added using \code{+} (e.g., \code{response1 + response2 ~ predictor}) to fit a sequential GLM for each response variable, captured in one output file.
#'
#' Outputs can be generated in multiple formats ("pdf", "word", "excel" and "rmd") as specified by \code{output_type}. The function also closes any open 'Word' files to avoid conflicts when generating 'Word' documents. If \code{output_type = "rmd"} is used it is advised to use it in a chunk with \{r, echo=FALSE, results='asis'\}
#'
#' This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
#'\itemize{
#' \item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
#' \item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
#' \item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
#'
#' \item If Pandoc is not found, this function may not work as intended.
#' }
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @return An object of class 'f_glm' (a named list, one entry per response variable) containing:
#'   \describe{
#'     \item{model}{The fitted \code{glm} object.}
#'     \item{summary}{Output of \code{summary(glm_fit)}.}
#'     \item{drop1}{Type II Analysis of Deviance table from \code{stats::drop1()}.}
#'     \item{diagnostics}{DHARMa residual checks and hat-value based leverage diagnostics.}
#'     \item{posthoc}{Estimated marginal means, pairwise comparisons, CLD letters, and summary table.}
#'     \item{sep_flag}{Logical indicating whether complete separation was detected.}
#'     \item{lrt_pct_explained}{McFadden's Pseudo-R\eqn{^2}.}
#'   }
#'   Using the option \code{output_type}, it can also generate output in the form of: R Markdown code, 'Word', 'pdf', or 'Excel' files. Includes print and plot methods for 'f_glm' objects.
#'
#' @examples
#' \donttest{
#' # GLM Binomial example with output to console
#' mtcars_mod <- mtcars
#' mtcars_mod$cyl <- as.factor(mtcars_mod$cyl)
#'
#' glm_bin <- f_glm(vs ~ cyl,
#'                  family = binomial,
#'                  data = mtcars_mod,
#'                  output_type = "default")
#' print(glm_bin)
#'
#' # GLM Binomial example with output to MS Word file
#' glm_bin_word <- f_glm(vs ~ cyl,
#'                  family = binomial,
#'                  data = mtcars_mod,
#'                  output_type = "word"
#'                  )
#'
#' # GLM Poisson example with output to rmd text
#' data(warpbreaks)
#'
#' glm_pos <- f_glm(breaks ~ wool + tension,
#'                  data = warpbreaks,
#'                  family = poisson(link = "log"),
#'                  intro_text = FALSE,
#'                  output_type = "rmd")
#' cat(glm_pos$rmd)
#' }
#'
#' @importFrom stats drop1
#' @importFrom utils combn
#' @export
#'
f_glm <- function(
    formula,                       # glm function formula
    family = gaussian(),           # family for the GLM
    data = NULL,                   # data.frame used for glm
    diagnostic_plots = TRUE,       # Show diagnostic plots in output files
    alpha = 0.05,                  # Significance level for tests
    adjust = "sidak",              # Method used to adjust p-values
    type = "response",             # Scale of emmeans posthoc results
    intro_text = TRUE,  # Print explanation about GLM assumptions
    dispersion_test = TRUE,        # Print dispersion test
    output_type = "default",           # Output type
    save_as = NULL,                # name of the output dir and file (name and type)
    save_in_wdir = FALSE,          # Save in working directory
    close_generated_files = FALSE, # Close open files
    open_generated_files = interactive(),   # Open files after creation
    influence_threshold = 2,       # Leverage threshold
    ...) {

  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state


  ##### Save dataframe name and Handle input from vectors (dataframe column) #####

  if(!is.null(data)){
    # Save dataframe name
    data_name <- deparse(substitute(data))

  } else if(is.null(data)){

    if(length(formula_extract_df_names(formula)) == 0){
      data_name <- "data"

    }
    else if(length(formula_extract_df_names(formula)) == 1){
      data_name <- formula_extract_df_names(formula)

    }
    else if(length(formula_extract_df_names(formula)) > 1){
      data_name <- paste(formula_extract_df_names(formula), collapse = "_")

    }

    # Make a data.frame based on the formula
    data <- formula_to_dataframe(formula)

    # Rewrite formula without data frame prefixes
    formula <- clean_formula(formula)

  }

  ##### File creation and output doc settings #####
  # Wrap lines in output document
  f_wrap_lines()

  # Output file handling
  temp_output_dir  <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "glm_output.Rmd")

  # Create the output file "output.Rmd" in tempdir()
  file.create(temp_output_file)
  # Create a file_extension switch
  file_extension <- NULL
  # Create a list to store all outputs in this function
  output_list <- list()


  ##### Error checking, converting input, closing files #####

  # match.arg() validates string args and gives clear error messages (CRAN standard)
  output_type <- match.arg(output_type,
                           choices = c("default", "console", "pdf", "word", "excel", "rmd"))
  adjust      <- match.arg(adjust,
                           choices = c("sidak", "bonferroni", "tukey", "none", "fdr"))
  type        <- match.arg(type, choices = c("response", "link"))

  # Parameter validation alpha
  if (!is.numeric(alpha) || length(alpha) != 1 || alpha <= 0 || alpha >= 1) {
    stop("'alpha' must be a single numeric value strictly between 0 and 1 (e.g. 0.05).")
  }

  ##### Handle option "save_as = " #####
  if(save_in_wdir == TRUE){
    save_dir <- getwd()
  }else{
    save_dir <- tempdir()
  }

  #map the output type to extensions
  output_type_map <- c(
    "pdf"  = ".pdf",
    "word" = ".docx",
    "excel"= ".xlsx",
    "rmd"  = ".rmd"
  )

  # If the user specifies a path, filename or save_in_wdir == TRUE an output file should be created
  if (!is.null(save_as) || save_in_wdir == TRUE) {

    if (!is.null(save_as)) {
      #Remove backslash in save_as if needed
      save_as <- gsub(pattern = "\\\\", replacement = "/", x = save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if(file_extension_save_as[1] != FALSE){
        file_extension <- file_extension_save_as
      }
    }

    if(is.null(file_extension) && output_type %in% c("console", "default")){
      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "glm_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = ".pdf"
      )
      #set output_type to default
      output_type <- "pdf"

    }
    else if(is.null(file_extension) && output_type %in% c("pdf", "word", "excel", "rmd")){

      #create extension based on input_type
      file.ext <- unname(output_type_map[output_type])

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "glm_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file.ext
      )


    }
    else if(!is.null(file_extension)) {

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "glm_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file_extension[1]
      )
      # reset the output type to match the user input extention in save_as
      output_type <- file_extension[2]
    }
  } else {

    #create extension based on input_type
    file.ext <- unname(output_type_map[output_type])

    # use helper get_save_path() to create output_path
    output_path <- get_save_path(save_as = save_as,
                                 default_name = paste(data_name, "glm_output", sep = "_"),
                                 default_dir = save_dir,
                                 file.ext = file.ext
    )
  }


  # Prevent output to console and keep files open when output is "rmd" format
  if (output_type == "rmd") close_generated_files <- FALSE

  # Cross-platform close_generated_files (was Windows-only taskkill)
  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    if (output_type == "excel") close_app("EXCEL.EXE",   "Microsoft Excel", "soffice")
  }



  ##### Evaluate family choice and Extract response variables #####
  # Family choice: make sure its correct input and not a function.
  # Handle character input like "gaussian"
  if (is.character(family)) {
    family <- get(family, mode = "function", envir = parent.frame())()
  }

  # Handle bare function input like gaussian()
  if (is.function(family)) {
    family <- family()
  }

  if(is.null(family$family)) {
    message(family)
    stop("'family' not recognized")
  }


  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(formula) #use helper_check_lhs.R

  # Extract response variables from the left-hand side of the formula
  lhs <- all.vars(formula[[2]])  # Get LHS variables (response)
  response_names <- lhs

  # Extract predictor variables from the right-hand side of the formula
  predictor_names <- all.vars(formula[[3]]) # Get RHS variables (predictors)

  # Extract the right-hand side (RHS) of the formula as a string
  rhs <- deparse(formula[[3]])  # Preserve the RHS structure

  # Ensure response and predictors are in the data
  for (response in response_names) {

    if (!(response %in% names(data))) {
      stop(paste("Response variable", response, "not found in the data."))
    }
    # Ensure the response variable is numeric
    response_var <- data[[response]]

    if (!is.numeric(response_var)) {
      stop("The response variable must be numeric.")
    }
  }

  for (predictor in predictor_names) {
    if (!(predictor %in% names(data))) {
      stop(paste("Predictor variable", predictor, "not found in the data."))
    }
    # Warn if a numeric predictor looks categorical -- emmeans will evaluate at
    # its mean rather than at each level, producing a single-row table.
    n_unique <- length(unique(stats::na.omit(data[[predictor]])))
    if (is.numeric(data[[predictor]]) && n_unique <= 10) {
      warning(paste0(
        "Predictor '", predictor, "' is numeric with only ", n_unique,
        " unique value(s). emmeans will evaluate at its marginal mean (",
        round(mean(data[[predictor]], na.rm = TRUE), 2),
        "), NOT at each level, resulting in a single-row post hoc table ",
        "instead of one row per group. ",
        "Convert to factor first if this is a categorical variable: ",
        "data$", predictor, " <- as.factor(data$", predictor, ")"
      ), call. = FALSE)
    }
  }

  #### Define Core GLM posthoc and diagnostic functions -----------------------

  check_residuals <- function(model, plot = TRUE) {

    # DHARMa cannot simulate from quasi-families because they lack a PDF.
    if(stats::family(model)$family %in% c("quasipoisson", "quasibinomial", "quasi")) {
      warning("DHARMa diagnostics skipped: Simulation not possible for quasi-families.")
      return(list(
        test_results = "Not performed (Quasi-family)",
        sim_res = NULL,
        path_qq_plot = NULL,
        path_residual_plot = NULL
      ))
    } else {

      sim_res <- DHARMa::simulateResiduals(model)

      temp_qq_plot <- tempfile(fileext = ".png")
      png(temp_qq_plot, width = 8, height = 5, units = "in", res = 600)
      plot(sim_res)
      dev.off()

      temp_residual_plot <- tempfile(fileext = ".png")
      png(temp_residual_plot, width = 8, height = 5, units = "in", res = 600)
      test_results <- DHARMa::testResiduals(sim_res)
      dev.off()


      return(list(
        test_results    = test_results,
        sim_res         = sim_res,
        path_qq_plot    = temp_qq_plot,
        path_residual_plot = temp_residual_plot
      ))
    }
  }

  check_influence <- function(model) {

    hat_values  <- stats::hatvalues(model)
    avg_hat     <- mean(hat_values, na.rm = TRUE)
    influential <- which(hat_values > influence_threshold * avg_hat)

    return(list(
      hat_values = hat_values,
      influential_points = influential
    ))
  }

  # LRT-based pairwise comparisons -- robust to complete separation.
  # For each pair of factor levels, fits a reduced model with those levels merged,
  # then takes the LRT vs the full model. Letters assigned via multcompLetters().
  # Returns a named character vector of letters (names = factor levels), or NULL.
  compute_pairwise_lrt_letters <- function(glm_fit, data, predictor_names, alpha, adjust) {

    # Only implemented for a single categorical predictor
    if (length(predictor_names) != 1) return(NULL)
    pred <- predictor_names[1]
    if (!is.factor(data[[pred]])) return(NULL)
    lvls <- levels(data[[pred]])
    if (length(lvls) < 2) return(NULL)

    is_quasi_lrt2 <- family$family %in% c("quasipoisson", "quasibinomial", "quasi")
    lrt_test2     <- if (is_quasi_lrt2) "F" else "Chisq"

    # All unique pairs
    pair_combns <- utils::combn(lvls, 2, simplify = FALSE)
    pair_names  <- vapply(pair_combns,
                          function(p) paste(sort(p), collapse = "-"),
                          character(1))
    raw_p <- stats::setNames(rep(NA_real_, length(pair_combns)), pair_names)

    full_formula <- stats::formula(glm_fit)

    for (i in seq_along(pair_combns)) {
      pair      <- pair_combns[[i]]
      data_temp <- data
      data_temp[[pred]] <- as.character(data[[pred]])
      data_temp[[pred]][data_temp[[pred]] %in% pair] <- "__merged__"
      data_temp[[pred]] <- factor(data_temp[[pred]])

      reduced <- tryCatch(
        stats::glm(full_formula, family = family, data = data_temp, ...),
        error = function(e) NULL
      )

      if (!is.null(reduced)) {
        lrt_res <- tryCatch(
          stats::anova(reduced, glm_fit, test = lrt_test2),
          error = function(e) NULL
        )
        if (!is.null(lrt_res) && nrow(lrt_res) >= 2) {
          p_col2 <- grep("^Pr|^p", names(as.data.frame(lrt_res)),
                         ignore.case = TRUE, value = TRUE)[1]
          if (!is.na(p_col2)) {
            raw_p[i] <- as.numeric(as.data.frame(lrt_res)[2, p_col2])
          }
        }
      }
    }

    # Conservative fallback for failed pairs
    raw_p[is.na(raw_p)] <- 1

    # Map adjust argument to a p.adjust method
    # NOTE: base R p.adjust() does not have a native Sidak method.
    # Bonferroni (alpha/k) is used as a conservative approximation for
    # Sidak (1-(1-alpha)^(1/k)). The difference is negligible for few comparisons.
    adj_method <- switch(adjust,
                         "sidak"      = "bonferroni",   # closest base-R equivalent
                         "tukey"      = "bonferroni",
                         "bonferroni" = "bonferroni",
                         "fdr"        = "fdr",
                         "none"       = "none",
                         "bonferroni"
    )
    adj_p <- stats::p.adjust(raw_p, method = adj_method)

    # Convert adjusted p-values to letters
    letter_result <- tryCatch(
      multcompView::multcompLetters(adj_p, threshold = alpha),
      error = function(e) NULL
    )
    if (is.null(letter_result)) return(NULL)

    # Return as named character vector: names = factor levels
    trimws(letter_result$Letters[lvls])
  }

  # post hoc analysis with emmeans
  perform_posthoc <- function(model, adjust, sig_effects, response_name, predictor_names, data,
                              sep_flag     = FALSE,
                              emm_is_cells = FALSE,
                              cat_preds    = character(0)) {

    # All formula predictors
    all_specs  <- all.vars(formula[[3]])

    # Categorical predictors only -- continuous ones are held at their mean
    # and must not appear as emmeans specs (CLD on a slope is meaningless)
    cat_specs  <- all_specs[vapply(all_specs, function(v) {
      is.factor(data[[v]]) || is.character(data[[v]])
    }, FUN.VALUE = logical(1))]
    cont_specs <- setdiff(all_specs, cat_specs)

    if (length(cont_specs) > 0) {
      warning(paste0(
        "f_glm: continuous predictor(s) [",
        paste(cont_specs, collapse = ", "),
        "] excluded from emmeans specs. ",
        "CLD letters are only assigned for categorical predictors. ",
        "Continuous predictors are held at their mean."
      ), call. = FALSE)
    }

    # Always use ALL categorical predictors as specs.
    # - emm_is_cells = TRUE  (significant interaction): returns cell means for
    #   every combination -- CLD compares all cells simultaneously.
    # - emm_is_cells = FALSE (no significant interaction): returns marginal means
    #   -- emmeans averages over non-significant factors appropriately.
    # Fall back to all predictors only if no categoricals exist (edge case).
    emm_specs <- if (length(cat_specs) > 0) cat_specs else all_specs

    emm   <- emmeans::emmeans(model,
                              specs = emm_specs,
                              type  = type)

    pairs_emm <- pairs(emm, adjust = adjust)

    mult_cld <- multcomp::cld(emm,
                              Letters = letters,
                              alpha   = alpha,
                              adjust  = adjust)

    # Convert the result to a data frame
    summary_table <- as.data.frame(mult_cld)
    rownames(summary_table) <- NULL
    names(summary_table)[names(summary_table) == ".group"] <- "Letter"

    if (!sig_effects) {
      # Overall effect not significant -- all groups in one class
      summary_table$Letter <- "ns"
    } else if (sep_flag) {
      # Separation detected: Wald-based pairwise tests are unreliable.
      # Attempt LRT-based pairwise comparisons instead.
      lrt_letters <- compute_pairwise_lrt_letters(
        model, data, predictor_names, alpha, adjust
      )
      if (!is.null(lrt_letters)) {
        pred <- predictor_names[1]
        summary_table$Letter <- lrt_letters[as.character(summary_table[[pred]])]
        summary_table$Letter[is.na(summary_table$Letter)] <- "\u2014"
      } else {
        summary_table$Letter <- "\u2014"
      }
    }

    # Data summary table groups by the same categorical specs as emmeans.
    # This keeps the raw-data table and the model-based table in sync:
    # - emm_is_cells = TRUE  -> cat_specs includes all factors -> cell-level counts
    # - emm_is_cells = FALSE -> cat_specs includes all factors -> counts per marginal group
    # Continuous predictors are excluded in both cases.
    #
    # Use model$model (the model.frame glm() actually fit on) instead
    # of raw `data` so that any subset / na.action / weights / NA
    # handling applied by glm() is reflected here too. Falls back to
    # `data` only if the user passed `model = FALSE` via `...`, which
    # suppresses model$model. Note: `model` is the first argument of
    # perform_posthoc() (the fitted glm); we do NOT reference
    # `glm_fit` here because that symbol only exists in the outer
    # generate_report() scope.
    summary_data <- if (!is.null(model$model)) model$model else data
    data_summary_table <- f_summary(summary_data,
                                    response_name,
                                    emm_specs,
                                    show_name = FALSE,
                                    digits    = NULL
    )$output_df

    # Identify the grouping key (e.g., "treatment")
    common_cols <- intersect(names(summary_table), names(data_summary_table))

    # Dynamically identify which "mean" column exists (emmean, rate, prob, or response)
    # and which stat columns exist (SE, confidence limits, etc.)
    target_cols <- c("emmean", "rate", "prob", "response",
                     "SE", "lower.CL", "upper.CL", "asymp.LCL", "asymp.UCL",
                     "Letter")

    # Select only the columns that are actually present in the summary_table
    existing_cols <- intersect(names(summary_table), target_cols)

    # Combine grouping columns with the detected data columns
    emm_cols <- c(common_cols, existing_cols)

    # Subset the Emmeans table using the correctly identified columns
    emm_subset <- summary_table[, emm_cols]

    # Getting n from data_summary_table
    n_df <- as.data.frame(data_summary_table[common_cols])
    n_df$n <- data_summary_table$n

    # Perform the Merge
    #    sort = FALSE -> CRITICAL! Prevents R from re-shuffling rows alphabetically
    post_hoc_summary_table <- merge(x = emm_subset,
                                    y = n_df,
                                    by = common_cols,
                                    all.x = TRUE,
                                    sort = FALSE)

    # Prevent breaking of emmean col title
    names(post_hoc_summary_table)[names(post_hoc_summary_table) == "emmean"] <- "emmean.."

    # Prepare table footnote CLD text string
    if (exists("mult_cld")) {

      if (sep_flag) {
        # Explain that LRT-based pairwise tests were used instead of Wald
        cld_text <- paste0(
          "\n**[!] Separation detected, LRT pairwise comparisons used:**  \n",
          "Complete separation was detected (at least one group perfectly predicts the outcome), ",
          "making Wald-based pairwise tests unreliable. ",
          "Letters were assigned using **likelihood ratio tests** (LRT): for each pair of groups, ",
          "a reduced model with those groups merged was compared against the full model via LRT. ",
          "This approach is robust to separation.  \n",
          "P-values were adjusted using the **",
          switch(adjust,
                 "sidak" = "Bonferroni method (conservative approximation; base R lacks native \u0160id\u00e1k)",
                 "bonferroni" = "Bonferroni method",
                 "fdr" = "FDR method",
                 "none" = "no adjustment (Fisher's LSD)",
                 "tukey" = "Bonferroni method (Tukey HSD not available for LRT; Bonferroni used instead)",
                 adjust
          ), "**.  \n",
          if (any(summary_table$Letter == "\u2014", na.rm = TRUE)) {
            paste0(
              "If letters show '\u2014', LRT comparisons could not be computed ",
              "(e.g. more than one predictor). ",
              "Refer to the **Type II Analysis of Deviance** table for the overall test."
            )
          } else {
            ""
          }
        )
      } else {




        # Extract the mult_cld text to adjust it
        msgs <- attr(mult_cld, "mesg")

        # Substitute 'alpha' with '\u03B1'
        msgs <- gsub("alpha", "\u03B1",  msgs)

        note_idx <- grep("^NOTE:", msgs, ignore.case = TRUE)
        if (length(note_idx)) {
          msgs[note_idx] <- paste0(
            "*Note: Groups in the \"Letters\" column sharing the same letter are ",
            "**not** significantly different (\u03B1 = ", alpha, "). Groups with ",
            "different letters are significantly different. Sharing a letter ",
            "indicates insufficient evidence to claim a difference; it does not ",
            "prove the groups are identical.*"
          )
        }

        cld_text <- paste(msgs, collapse = "  \n")


        # Extract min and max once to keep code clean and efficient
        min_df <- min(summary_table$df, na.rm = TRUE)
        max_df <- max(summary_table$df, na.rm = TRUE)

        # add the degrees of freedom
        cld_text <- paste0("Degrees of freedom: ",
                           if (min_df != max_df) {
                             paste(min_df, "-", max_df)
                           } else {
                             max_df
                           }, "  \n", cld_text
        )
      } # end else (no separation)
    } else {
      cld_text <- ""
    }


    return(
      list(
        emm = emm,
        pairs = pairs_emm,
        cld_text = cld_text,
        post_hoc_summary_table = post_hoc_summary_table,
        emmeans_table = summary_table,
        data_summary_table = data_summary_table
      )
    )

  }

  #### Output formatting functions -------------------------------------
  word_pdf_preamble <- function(){

    paste0("
---
title: \"f_glm Analysis Report\"
date: \"`r Sys.Date()`\"
output:
   word_document:
      reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
        latex_engine: pdflatex
header-includes:
  - \\usepackage[T1]{fontenc}
  - \\usepackage[utf8]{inputenc}
  - \\usepackage{textcomp}
  - \\DeclareUnicodeCharacter{03B1}{\\ensuremath{\\alpha}}
  - \\DeclareUnicodeCharacter{03B2}{\\ensuremath{\\beta}}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
  - \\DeclareUnicodeCharacter{03C6}{\\ensuremath{\\varphi}}
  - \\DeclareUnicodeCharacter{03C7}{\\ensuremath{\\chi}}
  - \\DeclareUnicodeCharacter{00B2}{\\ensuremath{^2}}
  - \\DeclareUnicodeCharacter{00D7}{\\ensuremath{\\times}}
  - \\DeclareUnicodeCharacter{2014}{\\textemdash}
  - \\DeclareUnicodeCharacter{2264}{\\ensuremath{\\leq}}
  - \\DeclareUnicodeCharacter{2265}{\\ensuremath{\\geq}}
  - \\DeclareUnicodeCharacter{2192}{\\ensuremath{\\rightarrow}}
  - \\DeclareUnicodeCharacter{221A}{\\ensuremath{\\surd}}
  - \\DeclareUnicodeCharacter{2212}{\\ensuremath{-}}
  - \\usepackage{titling}
  - \\setlength{\\droptitle}{-2.5cm} % Adjust vertical spacing
---
")
  }

  glm_assumptions_text <- function(){
    paste0("
# Generalized Linear Models (GLMs)

GLMs extend linear regression to response variables with non-normal distributions by specifying a link function and a probability distribution. The **link function** connects the mean of the response variable to the linear predictors. The choice of family and link function depends on the nature of your response variable.


| Family             | Typical Use Case               | Canonical Link     | Other Common Links        | R Syntax Example                      |
|--------------------|--------------------------------|--------------------|---------------------------|---------------------------------------|
| **Gaussian**       | Continuous, unbounded          | Identity           | Log, Inverse              | `gaussian(link = 'identity')`         |
| **Binomial**       | Binary or proportion           | Logit              | Probit, Cloglog, Log      | `binomial(link = 'logit')`            |
| **Poisson**        | Counts (no upper bound)        | Log                | Identity, Sqrt            | `poisson(link = 'log')`               |
| **Gamma**          | Positive, skewed continuous    | Inverse            | Log, Identity             | `Gamma(link = 'inverse')`             |
| **Inverse Gaussian** | Positive, highly skewed      | 1/mu$^2$           | Log, Identity, Inverse    | `inverse.gaussian(link = '1/mu^2')`   |
| **Quasi**          | Flexible (over/underdispersion)| Identity           | Log, others               | `quasi(link = 'identity')`            |
| **Quasibinomial**  | Flexible binomial              | Logit              | Probit, Cloglog, Log      | `quasibinomial(link = 'logit')`       |
| **Quasipoisson**   | Flexible Poisson               | Log                | Identity, Sqrt            | `quasipoisson(link = 'log')`          |

## Link Functions:\n

- *Identity*: \\( g(\\mu) = \\mu \\)
- *Log*: \\( g(\\mu) = \\log(\\mu) \\)
- *Logit*: \\( g(\\mu) = \\log\\left(\\frac{\\mu}{1-\\mu}\\right) \\)
- *Probit*: \\( g(\\mu) = \\Phi^{-1}(\\mu) \\) (inverse normal CDF)
- *Complementary log-log (cloglog)*: \\( g(\\mu) = \\log(-\\log(1-\\mu)) \\)
- *Inverse*: \\( g(\\mu) = 1/\\mu \\)
- *Inverse-square*: $g(\\mu) = 1/\\mu^2$
- *Square root*: $g(\\mu)= \\sqrt{\\mu}$


### How to choose:\n

- The *family* should match the distribution of your response variable.
- **The Link Function:** The **Canonical Link** is the standard **\"default\"** option for that family, this is usually the best place to start.
You might choose an alternative link for specific interpretation reasons (e.g., using a *log* link on Gaussian data to model percent changes).

## GLM Assumptions

1.  Correct distributional family.
2.  Appropriate link function.
3.  Independence of observations.
4.  Linear relationship between predictors and link-transformed response.
5.  **No overdispersion** (crucial for Poisson and grouped Binomial; not applicable to binary 0/1 data).

## DHARMa Diagnostics

The DHARMa package (short for *Diagnostics for HierArchical Regression Models*) provides simulation-based diagnostics to check the assumptions of your GLM.

## QQ Plot (Left Diagnostic Plot)

The QQ plot contains three specific statistical tests:

### 1. KS Test (Kolmogorov-Smirnov)
The KS test checks if your model's quantile residuals follow the expected **uniform** distribution.
* **Significant (low p-value):** Suggests the model does not fit the data well.
* *Note: With large datasets, this test is too sensitive. It often flags tiny, unimportant deviations. Always look at the plot yourself rather than relying solely on the p-value.*

### 2. Dispersion Test
This test checks for **overdispersion** (too much variance) or **underdispersion** (too little variance).

* **Where it matters:** This is most critical for **Poisson** (counts) and **Binomial** (proportions) families.
* **The Problem:** Real-world data is often \"clumped\" (e.g., disease cases within families), which creates more noise (variance) than the model expects, i.e. was assumed.
* **The Consequence:** If you ignore overdispersion, the model becomes \"overconfident.\" This leads to **false significance** (spurious significance); the model claims a result is statistically significant when it is actually just random noise.
* **The Fix:** If the dispersion test is significant (p < \u03b1), you should switch to a family that handles extra variance:
    * For Counts: **Negative Binomial** (e.g., package `MASS` or `glmmTMB`).
    * For Proportions: **Beta-Binomial** (e.g., package `glmmTMB`).

### 3. Outlier Test
This test checks if your data has more extreme values (outliers) than the model expects. A significant result means your data contains unexpected extremes.

## Residuals vs Model Predictions (Right Diagnostic Plot)

This plot shows your residuals (errors) on the Y-axis against the model's predictions on the X-axis.

* **What to look for:** In a good model, the dots should be **randomly scattered** around the middle line (0.5), with no clear patterns.
* **Red Stars:** These are outliers--observations that are more extreme than anything the model simulated.\n
&nbsp;\n  \n
**Common Problems to Spot:**

* *Funnel shape:* The variance is changing (heteroscedasticity).
* *U-shape or Hump:* You are missing a predictor or the relationship is not linear.
* *Patterns:* Any clear pattern suggests the model is failing to capture some structure in the data.
\n
<div style=\"page-break-after: always;\"></div>
\\newpage"
    )
  }

  #### Main analysis function and function combinations ------------------------------
  generate_report <- function(output = TRUE) {

    # Create a list to store all outputs in this function
    output_list <- list()


    # Multiple-response warning: only shown when > 1 response variable is analysed.
    # Fires regardless of intro_text so the user always sees it in this situation.
    if (length(lhs) > 1) {
      k          <- length(lhs)
      fwer_pct   <- round((1 - (1 - alpha)^k) * 100, 1)
      bonf_alpha <- round(alpha / k, 4)
      cat(paste0(
        "\n\n***\n\n",
        "**[!] NOTE: Multiple Testing Across ", k, " Response Variables**  \n\n",
        "This report runs ", k, " independent GLMs on the same dataset. ",
        "The **", adjust,"** correction keeps each individual test honest, it guards against ",
        "false positives among the pairwise group comparisons, but it offers no protection ",
        " against the accumulation of error across all ",k," tests combined. ",
        "\nAt \u03b1 = ", alpha, " per test, the probability of obtaining at least one ",
        "spurious significant result across all ", k, " responses is approximately ",
        "**", fwer_pct, "%** ($1 - (1 - ", alpha, ")^{", k, ",}$ assuming independence). ",
        "This risk is highest in exploratory studies; it is less of a concern when ",
        "each response has a clear a priori hypothesis.  \n\n",
        "**Possible remedies:**  \n",
        "\n- **Bonferroni** (conservative): re-run with `alpha = ", bonf_alpha,
        "` (\u03b1 / ", k, ").  \n",
        "\n- **False Discovery Rate (FDR)**: apply `p.adjust(p_values, method = \"fdr\")` to the ",
        k, " GLM p-values after the fact.  \n",
        "\n- **Pre-registration**: if each response was a pre-specified (primary) study ",
        "outcome, correction may not be required; document this decision ",
        "explicitly.  \n ",
        "\n\n***\n\n"
      ))
    }

    # Loop for several response parameters
    for (response_name in lhs) {

      # Create a new formula for each response, preserving interactions
      current_formula <- as.formula(paste0(response_name, "~", rhs))

      # GLM model fitting
      glm_fit <- stats::glm(current_formula,
                            family = family,
                            data = data,
                            ...)

      # Update the call info in glm_fit, did not succeed in family update.
      glm_fit$call$formula <- current_formula
      glm_fit$call$data    <- as.symbol(data_name)

      # Store the GLM model object in output_list
      output_list[[response_name]][["model"]] <- glm_fit

      # Store GLM model summary object in output_list
      output_list[[response_name]][["summary"]] <- summary(glm_fit)

      # Diagnostic checks
      output_list[[response_name]][["diagnostics"]] <- list(
        show_plot  = diagnostic_plots,
        residuals  = check_residuals(glm_fit, plot = diagnostic_plots),
        influence  = check_influence(glm_fit)
      )

      # Compute drop1 (Type II) and LRT FIRST -- needed for sig_effects below
      is_quasi_store <- family$family %in% c("quasipoisson", "quasibinomial", "quasi")
      lrt_test_store <- if (is_quasi_store) "F" else "Chisq"

      output_list[[response_name]][["drop1"]] <- tryCatch(
        stats::drop1(glm_fit, test = lrt_test_store),
        error = function(e) NULL
      )
      output_list[[response_name]][["lrt"]] <- tryCatch(
        stats::anova(glm_fit, test = lrt_test_store),
        error = function(e) NULL
      )
      output_list[[response_name]][["lrt_null_dev"]]      <- round(glm_fit$null.deviance, 3)
      output_list[[response_name]][["lrt_null_df"]]       <- glm_fit$df.null
      output_list[[response_name]][["lrt_resid_dev"]]     <- round(glm_fit$deviance, 3)
      output_list[[response_name]][["lrt_resid_df"]]      <- glm_fit$df.residual
      output_list[[response_name]][["lrt_pct_explained"]] <- round(
        1 - (glm_fit$deviance / glm_fit$null.deviance), 3
      )

      # sig_effects: based on LRT p-value from drop1 (not Wald z-tests).
      # Wald p-values are unreliable under complete separation -- LRT is robust.
      # If drop1 failed, fall back to Wald as a last resort.
      sig_effects <- {
        drop1_res_sig <- output_list[[response_name]][["drop1"]]
        if (!is.null(drop1_res_sig)) {
          drop1_df_sig <- as.data.frame(drop1_res_sig)
          p_col_sig    <- grep("^Pr|^p", names(drop1_df_sig), ignore.case = TRUE, value = TRUE)[1]
          if (!is.na(p_col_sig)) {
            p_vals_sig <- drop1_df_sig[[p_col_sig]]
            p_vals_sig <- p_vals_sig[!is.na(p_vals_sig)]
            length(p_vals_sig) > 0 && min(p_vals_sig) < alpha
          } else {
            FALSE
          }
        } else {
          # Fallback: Wald (minus intercept row)
          coef_pvals <- summary(glm_fit)$coefficients[-1, 4, drop = FALSE]
          nrow(coef_pvals) > 0 && min(coef_pvals) < alpha
        }
      }
      output_list[[response_name]][["sig_effects"]] <- sig_effects

      # Detect separation early so perform_posthoc can handle letter assignment correctly.
      # Heuristic: any predictor coefficient has SE > 100x its absolute estimate.
      coef_mat_early <- summary(glm_fit)$coefficients
      sep_flag_early <- if (nrow(coef_mat_early) > 1) {
        any(coef_mat_early[-1, "Std. Error"] > 100 * abs(coef_mat_early[-1, "Estimate"]),
            na.rm = TRUE)
      } else FALSE
      output_list[[response_name]][["sep_flag"]] <- sep_flag_early

      # ---- Interaction-aware emmeans mode (mirrors f_aov logic) ----
      # Parse drop1 term names to classify interactions vs main effects.
      # Determines whether emmeans returns cell means or marginal means,
      # and what note to show the user -- matching f_aov's emm_is_cells approach.
      drop1_for_int  <- output_list[[response_name]][["drop1"]]
      cat_preds      <- predictor_names[vapply(predictor_names, function(v)
        is.factor(data[[v]]) || is.character(data[[v]]),
        FUN.VALUE = logical(1))]

      emm_is_cells   <- FALSE
      sig_interactions  <- character(0)
      sig_main_effects  <- character(0)
      ns_main_effects   <- character(0)

      if (!is.null(drop1_for_int) && length(cat_preds) > 1) {
        d1_df      <- as.data.frame(drop1_for_int)
        term_names <- rownames(d1_df)
        p_col_int  <- grep("^Pr|^p", names(d1_df), ignore.case = TRUE, value = TRUE)[1]

        if (!is.na(p_col_int)) {
          p_vals_int <- d1_df[[p_col_int]]
          names(p_vals_int) <- term_names

          # Split into interaction terms (contain ":") and main effects
          int_terms  <- term_names[grepl(":", term_names, fixed = TRUE)]
          main_terms <- term_names[!grepl(":", term_names, fixed = TRUE) &
                                     term_names != "<none>"]

          # Keep only terms involving categorical predictors
          int_terms  <- int_terms[vapply(int_terms, function(t) {
            parts <- strsplit(t, ":")[[1]]
            all(parts %in% cat_preds)
          }, FUN.VALUE = logical(1))]

          sig_interactions <- int_terms[
            !is.na(p_vals_int[int_terms]) & p_vals_int[int_terms] < alpha]
          sig_main_effects <- main_terms[
            !is.na(p_vals_int[main_terms]) & p_vals_int[main_terms] < alpha &
              main_terms %in% cat_preds]
          ns_main_effects  <- main_terms[
            (is.na(p_vals_int[main_terms]) | p_vals_int[main_terms] >= alpha) &
              main_terms %in% cat_preds]

          emm_is_cells <- length(sig_interactions) > 0
        }
      }

      output_list[[response_name]][["emm_is_cells"]]      <- emm_is_cells
      output_list[[response_name]][["sig_interactions"]]  <- sig_interactions
      output_list[[response_name]][["sig_main_effects"]]  <- sig_main_effects
      output_list[[response_name]][["ns_main_effects"]]   <- ns_main_effects
      output_list[[response_name]][["cat_preds"]]         <- cat_preds

      output_list[[response_name]][["posthoc"]] <- perform_posthoc(
        glm_fit, adjust, sig_effects, response_name, predictor_names, data,
        sep_flag    = sep_flag_early,
        emm_is_cells = emm_is_cells,
        cat_preds   = cat_preds
      )


      # Create output and text for knitr
      cat("   \n  \n
# GLM of: ", response_name, "  \n"
      )
      cat("
## Model Diagnostics of: ", response_name, "\n   \n"
      )
      if( output_list[[response_name]][["diagnostics"]][["show_plot"]] == TRUE){

        path_qq_plot <-
          output_list[[response_name]][["diagnostics"]][["residuals"]][["path_qq_plot"]]

        if(!is.null(path_qq_plot)){
          cat(paste0("![](", path_qq_plot, ")"), "   \n  \n")
        } else {
          cat("\n**Note:** DHARMa diagnostic plots were skipped.\n")
          cat("Simulation-based residuals are not currently supported for quasi-families (e.g., Quasipoisson, Quasibinomial) because they lack a defined probability distribution for simulation.\n \n")
          # Pagebreak
          if(output_type != "rmd"){
            # Pagebreak
            cat("
<div style=\"page-break-after: always;\"></div>
\\newpage
        ")}
        }
      }

      # ---- Fix 5: Dispersion test text output (gated by dispersion_test param) ----
      if (isTRUE(dispersion_test)) {
        cat("\n## Dispersion Diagnostics of:", response_name, "\n\n")

        is_quasi    <- family$family %in% c("quasipoisson", "quasibinomial", "quasi")
        is_bernoulli <- family$family == "binomial" &&
          all(data[[response_name]] %in% c(0, 1, NA))

        if (is_bernoulli) {
          # Bernoulli (0/1) data cannot be overdispersed by definition --
          # variance is strictly p(1-p). Reporting a dispersion test would mislead students.
          cat(paste0(
            "*Dispersion test skipped: the response variable `", response_name,
            "` contains only 0s and 1s (Bernoulli data). ",
            "Overdispersion is mathematically impossible for binary outcomes;",
            "the variance is fixed at p(1\u2212p) and cannot exceed this. ",
            "Overdispersion in binomial models only arises with **grouped data** ",
            "(e.g., successes out of N trials per row, where N > 1).*  \n\n"
          ))
        } else if (is_quasi) {
          # For quasi-families, report the quasi-dispersion parameter from the model
          phi <- summary(glm_fit)$dispersion
          cat(paste0(
            "**Quasi-dispersion parameter (\u03c6):** ", round(phi, 4), "  \n\n",
            "The quasi-dispersion parameter estimates the degree of over- or underdispersion ",
            "relative to the nominal variance. A value close to **1** indicates no ",
            "extra-dispersion. Values **> 1** indicate overdispersion (more variance than ",
            "the model assumes); values **< 1** indicate underdispersion.  \n\n",
            "**What the quasi-family does mechanically:** standard errors are multiplied by ",
            "\u221a\u03c6 (the square root of \u03c6). When \u03c6 > 1, this inflates SEs, ",
            "making significance tests more conservative and correcting the overconfidence ",
            "that would arise from ignoring the extra variance.  \n\n",
            "*Note: DHARMa simulation-based tests are not available for quasi-families ",
            "because they lack a defined probability density function.*  \n\n"
          ))
        } else {
          # For proper non-Bernoulli families, extract DHARMa's dispersion test result
          disp_result <- output_list[[response_name]][["diagnostics"]][["residuals"]][["test_results"]][["dispersion"]]
          if (!is.null(disp_result)) {
            disp_stat <- round(as.numeric(disp_result$statistic), 4)
            disp_p    <- round(disp_result$p.value, 4)
            disp_sig  <- disp_p < alpha

            if (disp_stat > 1) {
              disp_direction <- "**overdispersion** (more variance than the model assumes)"
              disp_fix <- paste0(
                "  \n**Recommended action:** Switch to a family that accounts for extra variance:\n",
                "  - Counts (Poisson): use **Negative Binomial** (`MASS::glm.nb()` or `glmmTMB`).\n",
                "  - Proportions (Binomial): use **Beta-Binomial** (`glmmTMB`).\n",
                "  - Or use `quasipoisson` / `quasibinomial` to correct standard errors without changing the family."
              )
            } else {
              disp_direction <- "**underdispersion** (less variance than the model assumes)"
              disp_fix <- "  \n*Underdispersion is less common; consider whether the model or data are correctly specified.*"
            }

            disp_interp <- if (disp_sig) {
              paste0("The dispersion test is **significant** (p = ", disp_p, " \u2264 \u03b1 = ", alpha,
                     "), indicating ", disp_direction, ". ", disp_fix)
            } else {
              paste0("The dispersion test is **not significant** (p = ", disp_p, " > \u03b1 = ", alpha,
                     "), suggesting no strong evidence of extra-dispersion. The chosen family appears appropriate.")
            }

            cat(paste0(
              "**DHARMa Dispersion Test:**  ",
              "Ratio of simulated vs. observed variance: **", disp_stat, "**  \n",
              disp_interp, "  \n\n"
            ))
          } else {
            cat("*DHARMa dispersion test result not available.*  \n\n")
          }
        }
      }

      # ---- Fix 6: Influential points output ----
      {
        infl         <- output_list[[response_name]][["diagnostics"]][["influence"]]
        hat_vals     <- infl$hat_values
        inf_pts      <- infl$influential_points
        avg_hat      <- mean(hat_vals, na.rm = TRUE)
        threshold_val <- round(influence_threshold * avg_hat, 4)

        cat(paste0(
          "\n## High-Leverage Observations of: ", response_name, "\n\n",
          "Leverage (hat values) measures how far each observation's predictor values are ",
          "from the centroid of all predictors. High-leverage points *can* disproportionately ",
          "pull the fitted line, but only do so when they also have large residuals. ",
          "To assess actual **influence** (leverage \u00d7 residual), check Cook's distance ",
          "via `plot(glm_fit, which = 4)`.  \n\n",
          "**Threshold:** ", influence_threshold, " \u00d7 mean leverage = **", threshold_val, "**  \n"
        ))

        if (length(inf_pts) == 0) {
          cat(paste0(
            "**No influential observations detected** above the threshold. ",
            "All hat values are within acceptable range (max = ",
            round(max(hat_vals, na.rm = TRUE), 4), ").  \n\n"
          ))
        } else {
          cat(paste0(
            "**", length(inf_pts), " potentially influential observation(s) detected** ",
            "(row index: ", paste(inf_pts, collapse = ", "), "):  \n\n"
          ))

          inf_df <- data.frame(
            Row        = inf_pts,
            Hat_value  = round(hat_vals[inf_pts], 4)
          )
          f_pander(inf_df)

          cat(paste0(
            "**Recommended action:** Inspect these observations for data entry errors or ",
            "structural reasons why they are outlying. Use `plot(glm_fit)` and Cook's ",
            "distance to assess whether they materially affect coefficients.  \n\n"
          ))
        }
      }

      cat(paste("\n## Observed Descriptives Table of: ", deparse(current_formula), "  \n"))
      f_pander(f_conditional_round(output_list[[response_name]][['posthoc']][['data_summary_table']], digits = 3))
      # Add footnote
      cat(
        "**TIP:** These values represent your actual observed sample characteristics.
Use this table for *Methods* sections or Supplementary materials (to describe the sample).\n  \n
**CAUTION:** For statistical inference (significance letters and *p-values*) and reporting
main findings in the *Results* section, you **must** use the Emmeans table below.\n"
      )

      # Print the glm and posthoc
      cat("\n
## Model Summary of: ", response_name, "\n   \n")

      glm_sum <- output_list[[response_name]][["summary"]]

      # Print call and model info as plain text
      cat(paste0(
        "**Call:** `", deparse(current_formula), "`  \n",
        "**Family:** ", family$family, "  |  ",
        "**Link:** ", family$link, "  |  ",
        "**AIC:** ", round(AIC(glm_fit), 3), "  |  ",
        "**Null deviance:** ", round(glm_fit$null.deviance, 3),
        " (df = ", glm_fit$df.null, ")  |  ",
        "**Residual deviance:** ", round(glm_fit$deviance, 3),
        " (df = ", glm_fit$df.residual, ")  \n\n"
      ))

      # Convergence / separation warning
      if (!glm_fit$converged) {
        cat("**WARNING:** The model did not converge. Coefficient estimates are unreliable. ",
            "Check for complete separation or consider a different model specification.  \n\n")
      }
      if (any(is.na(coef(glm_fit)))) {
        cat("**WARNING:** One or more coefficients are NA, indicating perfect collinearity ",
            "or complete separation.  \n\n")
      }

      # Reuse the sep_flag already computed and stored above
      sep_flag <- output_list[[response_name]][["sep_flag"]]
      coef_mat <- glm_sum$coefficients

      # Coefficients as a pander table -- framed as effect estimates, not a significance test
      cat("**Coefficient Estimates** (direction and magnitude):  \n\n")
      coef_df <- as.data.frame(coef_mat)
      coef_df <- cbind(Term = rownames(coef_df), coef_df)
      rownames(coef_df) <- NULL
      stat_col <- if (family$family %in% c("quasipoisson","quasibinomial","quasi","gaussian"))
        "t value" else "z value"
      names(coef_df) <- c("Term", "Estimate", "Std. Error", stat_col, "Wald p")
      coef_df[["Estimate"]]      <- round(coef_df[["Estimate"]], 4)
      coef_df[["Std. Error"]]    <- round(coef_df[["Std. Error"]], 4)
      coef_df[[stat_col]]        <- round(coef_df[[stat_col]], 3)
      coef_df[["Wald p"]] <- ifelse(
        coef_df[["Wald p"]] < 0.001, "< 0.001",
        as.character(round(coef_df[["Wald p"]], 4))
      )
      f_pander(coef_df)

      # Note below coefficients table
      if (sep_flag) {
        cat(paste0(
          "\n**[!] Separation detected:** At least one predictor perfectly predicts the outcome ",
          "in a subset of the data (indicated by a very large Std. Error). ",
          "Wald p-values are unreliable in this situation; they will appear non-significant ",
          "even for strongly predictive terms. ",
          "**Use the Type II Analysis of Deviance table below for all significance decisions.**  \n\n"
        ))
      } else {
        cat(paste0(
          "\n*The 'Wald p' column is provided for completeness. ",
          "For significance testing use the **Type II Analysis of Deviance** table below, ",
          "which is robust to separation and more reliable for multi-predictor models.*  \n\n"
        ))
      }

      # ---- Fix 10: Type II Analysis of Deviance via stats::drop1() ----
      # Reuse the already-computed result stored in output_list (no recomputation).
      is_quasi_lrt  <- family$family %in% c("quasipoisson", "quasibinomial", "quasi")
      lrt_test_name <- if (is_quasi_lrt) "F-test" else "\u03c7\u00b2 likelihood ratio test"
      drop1_res     <- output_list[[response_name]][["drop1"]]

      cat("\n### Type II Analysis of Deviance of:", response_name, "\n\n")
      cat(paste0(
        "The table below tests the marginal significance of **each predictor term** ",
        "via `stats::drop1()` (Type II tests). Each term is dropped from the full model ",
        "in turn and tested against the model retaining all other terms; equivalent to ",
        "`car::Anova(type = 2)` but using only base R. ",
        "This is the GLM equivalent of the ANOVA F-table: it answers *\"does this predictor ",
        "improve the model?\"* after accounting for all other terms. ",
        "For single-predictor models this matches the coefficient", stat_col,
        "above; for multi-predictor models these per-term tests are the ones to report.  \n\n"
      ))

      if (!is.null(drop1_res)) {
        drop1_df  <- as.data.frame(drop1_res)
        drop1_df  <- cbind(Term = rownames(drop1_df), drop1_df)
        rownames(drop1_df) <- NULL
        # Round numeric columns
        num_cols <- vapply(drop1_df, is.numeric, FUN.VALUE = logical(1))
        drop1_df[num_cols] <- lapply(drop1_df[num_cols], function(x) round(x, 4))
        # Format p-value column if present
        p_col_name <- grep("^Pr|^p", names(drop1_df), ignore.case = TRUE, value = TRUE)[1]
        if (!is.na(p_col_name)) {
          drop1_df[[p_col_name]] <- ifelse(drop1_df[[p_col_name]] < 0.001, "< 0.001",
                                           as.character(round(as.numeric(drop1_df[[p_col_name]]), 4)))
        }
        #drop1_df <- drop1_df[drop1_df$Term != "<none>", ]
        drop1_df <- drop1_df[!is.na(drop1_df[[p_col_name]]), ]
        rownames(drop1_df) <- NULL
        f_pander(drop1_df)
      } else {
        cat("*Type II deviance table could not be computed.*  \n\n")
      }

      # ---- Fix 11: Likelihood ratio test vs. null model ----
      cat("\n### Model vs. Null Model Comparison of:", response_name, "\n\n")
      lrt <- output_list[[response_name]][["lrt"]]

      if (!is.null(lrt)) {
        # Extract the overall deviance reduction and p-value from the last row
        lrt_df    <- as.data.frame(lrt)
        dev_red   <- round(sum(lrt_df$Deviance, na.rm = TRUE), 4)
        p_col     <- grep("Pr|p.value|p-value", names(lrt_df), ignore.case = TRUE, value = TRUE)[1]
        lrt_p     <- if (!is.na(p_col)) min(lrt_df[[p_col]], na.rm = TRUE) else NA
        lrt_p_fmt <- if (!is.na(lrt_p) && lrt_p < 0.001) "< 0.001" else if (!is.na(lrt_p)) as.character(round(lrt_p, 4)) else NA

        cat(paste0(
          "The **null deviance** (intercept-only model) is ",
          round(output_list[[response_name]][["lrt_null_dev"]], 3), " on ",
          output_list[[response_name]][["lrt_null_df"]], " df. ",
          "The **residual deviance** (fitted model) is ",
          round(output_list[[response_name]][["lrt_resid_dev"]], 3), " on ",
          output_list[[response_name]][["lrt_resid_df"]], " df.  \n\n",
          "\n**McFadden's Pseudo-R\u00b2:** ",
          output_list[[response_name]][["lrt_pct_explained"]],
          "  \n\n",
          "*McFadden's Pseudo-R\u00b2 = 1 \u2212 (Residual deviance / Null deviance). ",
          "It measures how much the model improves over a null (intercept-only) model, ",
          "on a 0 to 1 scale. Unlike R\u00b2 in linear regression, it is **not** a proportion of ",
          "variance explained; values are typically lower: ",
          "0.2 to 0.4 is already considered an excellent fit for GLMs.*  \n\n",
          "&nbsp;  \n\n"
        ))

        if (!is.na(lrt_p)) {
          lrt_sig <- lrt_p < alpha
          cat(paste0(
            "**", lrt_test_name, "** vs. null model: p = **", lrt_p_fmt, "**. ",
            if (lrt_sig) {
              paste0("The model fits **significantly better** than the null model (\u03b1 = ", alpha,
                     "), meaning at least one predictor contributes to explaining the response.")
            } else {
              paste0("The model does **not** fit significantly better than the null model (\u03b1 = ", alpha,
                     "). Interpret results with caution.")
            }, "  \n\n"
          ))
        }

        lrt_df2 <- as.data.frame(lrt)
        lrt_df2 <- cbind(Term = rownames(lrt_df2), lrt_df2)
        rownames(lrt_df2) <- NULL
        # Round numeric columns
        num_cols2 <- vapply(lrt_df2, is.numeric, FUN.VALUE = logical(1))
        lrt_df2[num_cols2] <- lapply(lrt_df2[num_cols2], function(x) round(x, 4))
        # Format p-value column if present
        p_col2 <- grep("^Pr|^p", names(lrt_df2), ignore.case = TRUE, value = TRUE)[1]
        if (!is.na(p_col2)) {
          lrt_df2[[p_col2]] <- ifelse(lrt_df2[[p_col2]] < 0.001, "< 0.001",
                                      as.character(round(as.numeric(lrt_df2[[p_col2]]), 4)))
        }

        lrt_df2[] <- lapply(lrt_df2, function(x) ifelse(is.na(x), "", as.character(x)))
        f_pander(lrt_df2)

        # Warn about Type I (sequential) nature for multi-predictor models
        if (length(predictor_names) > 1) {
          cat(paste0(
            "\n*[!] This table uses **sequential (Type I) tests**, ",
            "per-term p-values depend on the order predictors enter the model. ",
            "For per-term significance, use the **Type II Analysis of Deviance** ",
            "(`drop1`) table above, which tests each term after accounting for all others.*  \n\n"
          ))
        }
      } else {
        cat("*Likelihood ratio test vs. null model could not be computed.*  \n\n")
      }

      # Pagebreak
      if(output_type != "rmd"){
        # Pagebreak
        cat("
<div style=\"page-break-after: always;\"></div>
\\newpage
        ")}
      cat("\n
\n## Model post hoc Analysis (Estimated Marginal Means) of: ", response_name, "\n   \n")

      # Retrieve interaction classification results
      emm_is_cells_out  <- output_list[[response_name]][["emm_is_cells"]]
      sig_ints_out      <- output_list[[response_name]][["sig_interactions"]]
      sig_main_out      <- output_list[[response_name]][["sig_main_effects"]]
      ns_main_out       <- output_list[[response_name]][["ns_main_effects"]]
      cat_preds_out     <- output_list[[response_name]][["cat_preds"]]

      # Interaction / marginal-means note -- mirrors f_aov's emm_is_cells logic
      if (isTRUE(emm_is_cells_out) && length(sig_ints_out) > 0) {
        sig_int_preds <- unique(unlist(strsplit(sig_ints_out, ":")))
        cat(paste0(
          "\n**NOTE: Significant interaction(s) detected: ",
          paste(sig_ints_out, collapse = ", "), "**  \n",
          "The post hoc table below shows **cell means**, the estimated value for ",
          "every combination of ", paste(sig_int_preds, collapse = " \u00d7 "), ".  \n",
          "Letters compare all cells simultaneously. ",
          "Interpretation should focus on the full interaction pattern, ",
          "not on individual factor effects in isolation.  \n\n"
        ))
      } else if (length(ns_main_out) > 0) {
        cat(paste0(
          "\n**NOTE:** The following term(s) were **not significant** (p \u2265 ", alpha, "): ",
          paste(ns_main_out, collapse = ", "), ".  \n",
          "The table below shows **marginal means**, the model averages over ",
          "non-significant factor(s), correcting for unbalanced designs. ",
          "Letter groups for non-significant terms are not meaningful but are ",
          "shown for completeness.  \n\n"
        ))
      }

      # --- DYNAMIC TEXT LOGIC START ---

      # 1. Get the table to check names
      ph_table_check <- output_list[[response_name]][['posthoc']][['post_hoc_summary_table']]

      # 2. Determine the Label (Rate vs Prob vs Mean)
      if (isTRUE(emm_is_cells_out)) {
        est_label <- if ("prob" %in% names(ph_table_check)) "Estimated Probabilities (Cell Means)" else
          if ("rate" %in% names(ph_table_check)) "Estimated Marginal Rates (Cell Means)" else
            "Estimated Cell Means"
      } else {
        if ("rate" %in% names(ph_table_check)) {
          est_label <- "Estimated Marginal Rates"
        } else if ("prob" %in% names(ph_table_check)) {
          est_label <- "Estimated Probabilities"
        } else if ("response" %in% names(ph_table_check)) {
          est_label <- "Estimated Responses"
        } else {
          est_label <- "Estimated Marginal Means"
        }
      }

      # 3. Determine the Scale Description based on 'type' and 'family'
      #    Accessing 'type' and 'family' from the main f_glm function environment
      if (type == "link") {
        scale_desc <- "model-based values on the linear link scale (e.g., log or logit)"
      } else {
        # If type == "response"
        if (family$family == "gaussian") {
          # Gaussian identity link doesn't really 'back-transform', so we simplify text
          scale_desc <- "model-based values on the response scale"
        } else {
          # Poisson, Binomial, Gamma, etc. are truly back-transformed
          scale_desc <- "back-transformed, model-based values on the response scale"
        }
      }

      # 4. Prepare Conditional Notes (New Logic)

      # Note on Asymmetry: Only needed for back-transformed GLMs (non-Gaussian response scale)
      note_asymmetry <- ""
      if (type == "response" && family$family != "gaussian") {
        note_asymmetry <- paste0(
          "\n* **Asymmetric Intervals:** You may notice that the Confidence Intervals are not symmetrical around the ",
          est_label, ". This is normal! Because of the non-linear link function, the uncertainty is often 'stretched' in one direction."
        )
      }

      # Note on Scale Interpretation
      if (type == "link") {
        note_scale <- "* **Interpretation:** These values are on the transformed scale (e.g., log-odds). You may need to convert them manually to interpret them in real-world units."
      } else {
        note_scale <- "* **Back-Transformation:** The values (Means and CIs) have been converted from the model's internal scale back to the original units."
      }

      # 5. Print the Combined Dynamic Text
      cat(
        "The table below shows the ", est_label, " (`emmeans` package). These are ", scale_desc, " for ", response_name, ". Unlike raw averages, these values correct
      for unbalanced designs and reflect the statistical model."
      )

      cat(
        "\n#### Publication & Reporting Tips\n")
      cat(
        "* For main results showing significant differences, prioritize reporting these", est_label, " and their *Confidence Intervals (CIs)* rather than raw means.\n",
        "* Figures should ideally overlay these ", est_label, " (and error bars) on top of the raw data points.\n"
      )
      cat(
        "\n#### Technical Notes (Explanations)\n",
        note_scale,
        note_asymmetry,
        "\n* **SE vs CI:** We recommend focusing on **Confidence Intervals** for inference rather than Standard Errors (SE).\n  \n"
      )

      cat("&nbsp;\n  \n")
      cat(paste("\n**Post Hoc Summary Table** of glm call: ", deparse(current_formula), "  \n"))
      f_pander(f_conditional_round(output_list[[response_name]][['posthoc']][['post_hoc_summary_table']], digits = 3))
      # Add footnote
      cat(output_list[[response_name]][['posthoc']][['cld_text']])
      # --- DYNAMIC TEXT LOGIC END ---
    }

    if (output == TRUE) {
      return(output_list)
    }
  }

  # Execute analysis ONCE: capture both the output_list and the markdown text.
  # Caching avoids running every GLM/DHARMa simulation a second time
  # (DHARMa uses random simulation, so two runs can yield different p-values).
  cached_markdown <- NULL
  suppressMessages(
    cached_markdown <- utils::capture.output(
      output_list <- generate_report()
    )
  )
  class(output_list) <- "f_glm"

  # Helper so word/pdf/rmd paths all use the same already-generated markdown.
  get_cached_markdown <- function() cached_markdown

  # Here the documents are constructed.
  if(output_type %in% c("word", "pdf")) {

    # Show save location before knitting else it will not display in console.
    message(paste0("Saving output in: ", output_path))

    # Prevent ## before printed output
    knitr::opts_chunk$set(comment = "")

    # Use the already-generated markdown (no second run of generate_report)
    generated_markdown <- get_cached_markdown()

    # Combine the preamble, assumptions, and the captured report into one string
    rmd_content <- paste(
      word_pdf_preamble(),
      if (intro_text) glm_assumptions_text() else "",
      # The captured output already contains the necessary markdown formatting and image links
      paste(generated_markdown, collapse = "\n"),
      sep = "\n"
    )

    # Write the complete Rmd content to the temp file
    writeLines(rmd_content, temp_output_file)

    # Create the RMarkdown file
    rmarkdown::render(
      temp_output_file,
      output_file = output_path,
      intermediates_dir = temp_output_dir,
      knit_root_dir = temp_output_dir,
      quiet = TRUE,
      output_format = paste0(output_type, "_document")
    )

    # Open files after creation
    if(open_generated_files == TRUE){
      # Open the file with default program
      f_open_file(output_path)
    }

    return(invisible(output_list))

  } else if(output_type == "excel") {

    # show the location were the file is saved
    message(paste0("Saving output in: ", output_path))

    # Extract all post_hoc_summary_table tables and keep their names
    post_hoc_tables <- lapply(output_list, function(obj)
      obj$posthoc$post_hoc_summary_table)

    # Assign names to the list for Excel sheet names based on response names
    names(post_hoc_tables) <- response_names

    # Write to an Excel file with each table in its own sheet
    writexl::write_xlsx(post_hoc_tables, path = output_path)

    # Open files after creation
    if(open_generated_files == TRUE){
      f_open_file(output_path)
    }

    return(invisible(output_list))
  }

  if(output_type == "rmd"){

    if (is.null(knitr::opts_knit$get("output.dir"))) {
      knitr::opts_knit$set(output.dir = tempdir())
    }

    # Use the already-generated markdown (no second run of generate_report)
    clean_rmd_output <- paste(
      if (intro_text) glm_assumptions_text() else "",
      paste(get_cached_markdown(), collapse = "\n"),
      sep = "\n"
    )
    output_list[["rmd"]] <- clean_rmd_output

    return(invisible(output_list))

  } else if(output_type %in% c("console")){
    #Print output list to the console (forced)
    print(output_list)

    return(output_list)

  } else if(output_type %in% c("default")){
    return(output_list)

  } else {
    warning("Invalid output format specified. No file generated.")

  }


  invisible(suppressWarnings(file.remove(temp_output_file)))

}

# Print method for f_glm objects
#' @export
print.f_glm <- function(x, ...) {

  # Loop over each response variable
  for (category in names(x)) {

    # Skip non-response entries (e.g. "rmd" character string)
    sublist <- x[[category]]
    if (!is.list(sublist)) next

    cat("==========================================\n")
    cat("   GLM of response variable:", category, "\n")
    cat("==========================================\n")

    # Model info header
    glm_sum <- sublist$summary
    cat("Family:", glm_sum$family$family,
        " | Link:", glm_sum$family$link,
        " | AIC:", round(sublist$model$aic, 3), "\n")
    cat("Null deviance:", round(sublist$lrt_null_dev, 4),
        "on", sublist$lrt_null_df, "df",
        " | Residual deviance:", round(sublist$lrt_resid_dev, 4),
        "on", sublist$lrt_resid_df, "df\n\n")

    # Convergence warning
    if (!isTRUE(sublist$model$converged)) {
      warning("Model did not converge, estimates unreliable.", call. = FALSE, immediate. = TRUE)
    }

    # Coefficients table
    cat("Coefficients:\n")
    print(glm_sum$coefficients)

    cat("\n--- Type II Analysis of Deviance ---\n")
    if (!is.null(sublist$drop1)) {
      print(sublist$drop1)
    } else {
      cat("(not available)\n")
    }

    cat("\n--- Model vs. Null Model ---\n")
    cat("Null deviance:     ", round(sublist$lrt_null_dev, 4),
        " on", sublist$lrt_null_df, "df\n")
    cat("Residual deviance: ", round(sublist$lrt_resid_dev, 4),
        " on", sublist$lrt_resid_df, "df\n")
    cat("McFadden's Pseudo-R\u00b2:", sublist$lrt_pct_explained, "\n")

    cat("\n--- Post hoc Comparisons of:", category, "---\n")
    print(sublist$posthoc$post_hoc_summary_table)
    cat("___________________________\n")
    cat(sublist$posthoc$cld_text)
    cat("\n   \n")
  }

}

#' @export
plot.f_glm <- function(x, which = 1:2, ...) {

  for (category in names(x)) {

    # Skip non-response entries (e.g. "rmd" character string)
    sublist <- x[[category]]
    if (!is.list(sublist)) next

    sim_res <- sublist$diagnostics$residuals$sim_res

    if (is.null(sim_res)) {
      message("plot.f_glm: DHARMa diagnostic plots skipped for '", category,
              "' (quasi-family or simulation not available).")
      next
    }

    # 1: QQ plot
    plot(sim_res)

    # 2: Residual tests
    DHARMa::testResiduals(sim_res)
  }
}

# =============================================================================
# helper-skips.R
# =============================================================================
# Centralised skip conditions for the rfriend test suite.
#
# Most rfriend functions that produce "pretty" output rely on external
# machinery (Pandoc for .docx / .pdf, RStudio for f_theme / f_setwd,
# Suggests-only packages for certain model comparisons). The skip helpers
# below let individual tests declare their dependencies in one line instead
# of duplicating the underlying checks in every file.
#
# Call pattern inside a test:
#
#     test_that("word output contains expected heading", {
#       skip_on_cran()
#       skip_if_no_pandoc()
#       ...
#     })
#
# Never call these outside a test_that() / it() block. They use
# testthat::skip() which is only meaningful inside a test.
# =============================================================================


# -----------------------------------------------------------------------------
# External binaries
# -----------------------------------------------------------------------------

# Skip when Pandoc is not available on PATH. rmarkdown::pandoc_available()
# is preferred over nzchar(Sys.which("pandoc")) because it also honours
# RSTUDIO_PANDOC on systems where Pandoc ships with RStudio but is not on
# the shell PATH.
skip_if_no_pandoc <- function(min_version = "3.2") {
  if (!requireNamespace("rmarkdown", quietly = TRUE)) {
    testthat::skip("rmarkdown not installed")
  }
  if (!rmarkdown::pandoc_available(min_version)) {
    testthat::skip(sprintf("Pandoc (>= %s) not available", min_version))
  }
}


# -----------------------------------------------------------------------------
# Soft dependencies declared in Suggests:
# -----------------------------------------------------------------------------
#
# DESCRIPTION declares MASS, nnet, pbkrtest, testthat in Suggests.
# testthat::skip_if_not_installed() is the canonical check for each.
# We do NOT wrap it; call it directly at the top of the test. The list
# below is a reminder of which functions depend on which suggest.

# f_boxcox():       MASS
# f_glm() (multinomial path): nnet
# f_lmer() (pbkrtest-based denominator DF): pbkrtest


# -----------------------------------------------------------------------------
# Interactive / GUI dependencies
# -----------------------------------------------------------------------------

# f_theme() and f_setwd() call rstudioapi::* and only make sense inside
# RStudio. Skip on non-interactive or non-RStudio sessions.
skip_if_not_rstudio <- function() {
  if (!requireNamespace("rstudioapi", quietly = TRUE)) {
    testthat::skip("rstudioapi not installed")
  }
  if (!rstudioapi::isAvailable()) {
    testthat::skip("not running inside RStudio")
  }
}


# -----------------------------------------------------------------------------
# Slow tests
# -----------------------------------------------------------------------------
#
# CRAN budgets ~10 minutes total for examples + tests on their check farms.
# Several rfriend tests exercise bootstrap simulation (DHARMa), large lmer
# fits, or bestNormalize iterating over every transform. Mark those with:
#
#     skip_on_cran()
#
# directly inside the test_that() block. No wrapper needed.
#
# For tests that are fast on developer laptops but slow in CI, use:
#
#     skip_if(isTRUE(as.logical(Sys.getenv("CI"))))
#
# if and only if CI is consistently too slow. Prefer optimising the test.

#' @import rmarkdown
#' @import knitr
#' @import ggplot2
#' @importFrom grDevices adjustcolor dev.off png rainbow rgb graphics.off  recordPlot replayPlot
#' @importFrom graphics abline arrows axis boxplot hist legend lines pairs par points polygon text stripchart strwidth layout mtext title
#' @importFrom stats aggregate aov as.formula cor kruskal.test drop1 median qchisq qqline qqnorm residuals sd shapiro.test spline chisq.test pnorm p.adjust AIC BIC anova coef deviance df.residual fitted formula gaussian glm hatvalues logLik model.frame nobs sigma terms update
#' @importFrom utils capture.output head tail combn
#' @importFrom multcompView multcompLetters
#' @importFrom pander pander panderOptions
#' @importFrom rstudioapi getActiveDocumentContext documentId documentPath
#' @importFrom stats ppoints qnorm quantile dnorm setNames family pchisq complete.cases predict
#' @importFrom rstatix levene_test dunn_test
#' @importFrom nortest ad.test
#' @importFrom writexl write_xlsx
#' @importFrom grid unit grid.newpage grid.draw grid.raster
#' @importFrom gridExtra grid.arrange arrangeGrob
#' @importFrom knitr knit_hooks opts_knit knit_child
#' @importFrom crayon red
#' @importFrom xfun split_lines dir_exists
#' @importFrom multcomp cld
#' @importFrom emmeans emmeans
#' @importFrom utils install.packages
#' @importFrom magick image_read image_scale
#' @importFrom bestNormalize bestNormalize
#' @importFrom stringr str_replace_all
#' @importFrom this.path this.dir
#' @importFrom MuMIn r.squaredGLMM
#' @importFrom dplyr select
#' @importFrom tidyr pivot_longer all_of
#' @importFrom magrittr %>%
#' @importFrom DHARMa plotResiduals testResiduals testDispersion simulateResiduals
#' @importFrom rlang .data
#' @importFrom lme4 ranef VarCorr ngrps isSingular lmerControl nobars
#' @importFrom lmerTest lmer
NULL

#' Rename Specific Columns in a Data Frame
#'
#' Renames specific columns in a data frame based on a named vector (name_map).
#' It ensures that only the specified columns are renamed, while others remain unchanged.
#'
#' @param df A data frame whose columns are to be renamed.
#' @param name_map A named vector where the names correspond to the current column names in \code{df}, and the values are the new names to assign. All names in \code{name_map} must exist in the column names of \code{df}. Yet, not all names in the data.frame have to be in \code{name_map}. This allows for selective renaming of just one or two columns.
#'
#' @return A data frame with updated column names. Columns not specified in \code{name_map} remain unchanged.
#'
#' @details
#' This function is particularly useful when you want to rename only a subset of columns in a
#' data frame. It performs input validation to ensure that:
#' \itemize{
#' \item \code{name_map} is a named vector.
#' \item All names in \code{name_map} exist as column names in \code{df}.
#' }
#' If these conditions are not met, the function will throw an error with an appropriate message.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#'
#' @examples
#' # Create a sample data frame.
#' df <- data.frame(a = 1:3, b = 4:6, c = 7:9)
#'
#' # Define a named vector for renaming specific columns.
#' name_map <- c(a = "alpha", c = "gamma")
#'
#' # Rename columns.
#' df <- f_rename_columns(df, name_map)
#'
#' # View updated data frame.
#' print(df)
#'
#' @export

f_rename_columns <- function(df, name_map) {
  # Check if input is a named vector
  if (is.null(names(name_map))) {
    stop("The name_map argument must be a named vector.")
  }

  # Check if all names in the vector exist in the data frame
  unmatched <- setdiff(names(name_map), colnames(df))

  if (length(unmatched) > 0) {
    stop(paste("The following columns are not found in the data frame:", paste(unmatched, collapse = ", ")))
  }

  # Replace column names based on the named vector
  colnames(df)[colnames(df) %in% names(name_map)] <- name_map[colnames(df)[colnames(df) %in% names(name_map)]]

  return(df)
}

# create a dataframe based on the vector input in a formula
formula_to_dataframe <- function(formula_obj) {

  # Get all variable names including data frame names
  all_variables <- formula_extract_clean_vars(formula_obj)

  # Dynamically evaluate each variable in the current environment
  df <- stats::setNames(
    lapply(all_variables, function(var) eval(parse(text = var))),
    all_variables
  )

  # Convert the list to a data frame
  df <- as.data.frame(df)

  names(df) <- sub(".*\\.", "", names(df))

  return(df)
}

#' Remove Outliers from Data
#'
#' @description
#' `f_remove_outliers()` removes specific rows from a dataframe based on a list of identifiers.
#' It is designed to work seamlessly with the output of \code{\link{f_outliers}}, but can also
#' accept a custom vector of IDs.
#'
#' @param data A data.frame, tibble, or data.table containing the original data.
#' @param outliers Either:
#'   \itemize{
#'     \item A dataframe returned by \code{\link{f_outliers}}.
#'     \item A vector of IDs/row numbers to remove.
#'   }
#' @param by A character string specifying the column to match on. Default is \code{"row_id"}.
#'   If the source \code{data} does not have a \code{row_id} column, the function effectively
#'   uses the row numbers (1, 2, 3...) to ensure safe deletion.
#' @param verbose Logical. If \code{TRUE} (default), prints a summary of how many rows were removed.
#'
#' @details
#' \strong{Safe Deletion Logic:}
#' This function performs a "anti-join" style filtering. It keeps rows where the identifier in
#' \code{by} is \strong{not} found in the \code{outliers} list.
#'
#' \strong{Handling Row IDs:}
#' If you use the default \code{by = "row_id"} and your original \code{data} does not have a
#' column named \code{"row_id"}, the function assumes you are referring to the intrinsic
#' row numbers of the data.frame, tibble, or data.table. It will temporarily generate IDs to
#' perform the deletion and then return the clean data with the original structure
#' (without adding a permanent \code{row_id} column to the result).
#'
#' @return An object of the same class as the input \code{data} (data.frame, tibble, or data.table)
#'   with the specified outlier rows removed.
#'
#' @seealso \code{\link{f_outliers}} to identify the rows to be removed.
#'
#' @examples
#' # --- Setup: Create Dummy Data ---
#' set.seed(42)
#' df <- data.frame(
#'   Team       = rep(c("A", "B"), each = 20),
#'   Department = rep(c("Sales", "IT"), each = 10, times = 2),
#'   Salary     = c(rnorm(19, 50000, 500), 100000,
#'                  rnorm(18, 50000, 500), 57000, 1000),
#'   Age        = c(rnorm(38, 35, 2), 90, 35),
#'   EmployeeID = paste0("E", sprintf("%03d", 1:40)),
#'   stringsAsFactors = FALSE
#' )
#' # row 20:  extreme high Salary (Team A)
#' # row 39:  mild Salary outlier at coef = 1.5 only
#' # row 40:  extreme low  Salary (Team B)
#' # row 39:  extreme high Age
#'
#' # --- Example 1: Basic two-step workflow (data.frame notation) ---
#' # The most common use case: find then remove in two lines.
#' bad_rows <- f_outliers(df, columns = "Salary")
#' clean_df <- f_remove_outliers(df, bad_rows)
#' nrow(df)       # 40
#' nrow(clean_df) # 40 minus flagged rows
#'
#' # --- Example 2: Basic two-step workflow (formula notation) ---
#' # Identical result to Example 1 using the formula interface.
#' bad_rows <- f_outliers(Salary ~ 1, data = df)
#' clean_df <- f_remove_outliers(df, bad_rows)
#' nrow(clean_df)
#'
#' # --- Example 3: Grouped detection then removal (both notations) ---
#' # Outliers are identified *within* each Team separately before removal.
#'
#' # data.frame notation:
#' bad_rows <- f_outliers(df, columns = "Salary", group_vars = "Team")
#' clean_df <- f_remove_outliers(df, bad_rows)
#'
#' # Formula notation (identical result):
#' bad_rows <- f_outliers(Salary ~ Team, data = df)
#' clean_df <- f_remove_outliers(df, bad_rows)
#' nrow(clean_df)
#'
#' # --- Example 4: Selective removal -- only act on a subset of outliers ---
#' # Find all flagged rows, but only remove the extreme high salaries.
#' # Step 1: Identify all Salary outliers grouped by Team
#' bad_rows    <- f_outliers(Salary ~ Team, data = df)
#' all_flagged <- bad_rows$output_df
#'
#' # Step 2: Filter to keep only the rows where Salary > 90000
#' really_bad  <- all_flagged[all_flagged$Salary > 90000, ]
#'
#' # Step 3: Remove only those rows -- low outlier (row 40) is preserved
#' clean_df <- f_remove_outliers(df, really_bad)
#' range(clean_df$Salary)  # low outlier still present, high one is gone
#'
#' # --- Example 5: Multi-column outlier removal ---
#' # f_outliers scans both Salary and Age; f_remove_outliers removes
#' # every row flagged by either column in one call.
#'
#' # Formula notation:
#' bad_rows <- f_outliers(Salary + Age ~ Team, data = df)
#' clean_df <- f_remove_outliers(df, bad_rows)
#'
#' # data.frame notation (identical result):
#' bad_rows <- f_outliers(df, columns = c("Salary", "Age"), group_vars = "Team")
#' clean_df <- f_remove_outliers(df, bad_rows)
#' nrow(clean_df)  # rows flagged by Salary OR Age are removed
#'
#' # --- Example 6: Strict detection + custom ID column ---
#' # coef = 3.0 flags only extreme outliers. EmployeeID is used
#' # as the matching key instead of the default row_id.
#'
#' # Formula notation:
#' bad_rows <- f_outliers(Salary ~ Team, data = df,
#'                        id_var = "EmployeeID", coef = 3.0)
#'
#' # data.frame notation (identical result):
#' bad_rows <- f_outliers(df, columns = "Salary", group_vars = "Team",
#'                        id_var = "EmployeeID", coef = 3.0)
#'
#' # Remove by EmployeeID rather than row position
#' clean_df <- f_remove_outliers(df, bad_rows$output_df, by = "EmployeeID")
#'
#' # Confirm the flagged employees are no longer in the clean data
#' bad_ids  <- bad_rows$output_df$EmployeeID
#' any(clean_df$EmployeeID %in% bad_ids)  # FALSE
#'
#' @export

f_remove_outliers <- function(data,
                              outliers,
                              by = "row_id",
                              verbose = TRUE) {

  # --- Check Input Data ---
  if (is.numeric(data) || is.integer(data)) {
    stop(
      "Vector input is not supported in f_remove_outliers.\n",
      "Wrap your vector in a data.frame first: \n  f_remove_outliers(data.frame(value = x), outliers)"
    )
  }

  # --- Check Input Data ---
  if (!is.data.frame(data)) {
    stop("Input 'data' must be a data.frame.")
  }

  # --- Check and Prepare Outlier IDs ---
  ids_to_remove <- NULL

  if (inherits(outliers, "f_outliers")) {
    # Extract all row_ids across every result table in the list
    ids_to_remove <- unique(unlist(lapply(outliers, function(df) df[[by]])))

  } else if (is.data.frame(outliers)) {
    if (!by %in% names(outliers)) {
      stop(paste0("The column '", by, "' was not found in the 'outliers' dataframe."))
    }
    ids_to_remove <- outliers[[by]]

  } else if (is.vector(outliers) && !is.logical(outliers)) {
    ids_to_remove <- outliers
  } else {
    stop("'outliers' must be a dataframe or a vector of IDs.")
  }

  # --- Check 'by' column in Source Data ---
  # Track if we added the ID so we can remove it later
  added_id_col <- FALSE

  if (!by %in% names(data)) {
    if (by == "row_id") {
      data$row_id <- seq_len(nrow(data))
      added_id_col <- TRUE # Mark that we added this
    } else {
      stop(paste0("The ID column '", by, "' was not found in the 'data'."))
    }
  }

  # --- The Filtering Logic ---
  rows_to_cut <- data[[by]] %in% ids_to_remove
  clean_data <- data[!rows_to_cut, ]

  # --- Cleanup ---
  # If 'row_id' was added remove it from the result
  # so the user gets back exactly what they put in (minus outliers).
  if (added_id_col) {
    clean_data$row_id <- NULL
  }

  # Reset the rownames to have continues numbers
  # (this prevents problems when using clean_data df)
  rownames(clean_data) <- NULL

  # --- User Feedback ---
  if (verbose) {
    n_removed <- sum(rows_to_cut)
    n_remaining <- nrow(clean_data)

    if (n_removed == 0) {
      message("rfriend: No outliers were removed (no IDs matched).")
    } else {
      message(paste0("rfriend: Removed ", n_removed, " outliers."))
      message(paste0("rfriend: Rows: ", nrow(data), " -> ", n_remaining))
    }
  }

  return(clean_data)
}

#' Conditional Rounding for Numeric Values
#'
#' @description
#' Conditionally formats numeric values based on their magnitude. Values that are very small
#' or very large are formatted using scientific notation, while other values are rounded
#' to a specified number of decimal places. Integers are preserved without decimal places.
#' When applied to a data frame, only numeric columns are processed. All output is character string.
#'
#' @param x A numeric vector or data frame containing numeric columns to be formatted.
#' @param threshold_small Numeric value. Values with absolute magnitude smaller than this
#'        threshold will be formatted using scientific notation. Default is \code{0.01}.
#' @param threshold_large Numeric value. Values with absolute magnitude larger than or equal
#'        to this threshold will be formatted using scientific notation. Default is \code{10000}.
#' @param digits Integer. Number of decimal digits to use in formatting. Default is \code{3}.
#' @param replace_na Logical. If \code{TRUE}, \code{NA} values in the output
#'   are replaced with the string specified by \code{na_string}. If
#'   \code{FALSE}, \code{NA} values are left as \code{NA}. Default is
#'   \code{TRUE}.
#' @param na_string Character string used to replace \code{NA} values in the
#'   output when \code{replace_na = TRUE}. Use \code{"-"} (default) for a
#'   standard "not available" dash, \code{""} for empty cells, or any other
#'   string as needed. Note that special characters such as the em dash
#'   (\code{"\u2014"}) may require additional LaTeX declarations when rendering
#'   to PDF via \code{pdflatex}. Default is \code{"-"}.
#' @param allow_integer_decimal_mix Logical. If \code{TRUE}, each individual cell is evaluated: integer values are displayed without decimal places, and non-integer values are displayed with the specified number of decimal places, i.e. \code{digits}. Default is \code{FALSE}, when a column contains a mix of integers and decimal values, all values are displayed with the specified number of decimal places. Note: columns containing only integers are **always** displayed without decimal places, regardless of \code{allow_integer_decimal_mix}.
#'
#' @return
#' \itemize{
#'   \item If input is a vector: A character vector of the same length as the input, with values
#' formatted according to the specified rules.
#'
#'   \item If input is a data frame: A data frame with the same structure as the input, but with
#' character columns formatted according to the specified rules.
#'  }
#'
#' @details
#' The function applies the following formatting rules:
#' \itemize{
#'   \item Values smaller than \code{threshold_small} or larger than \code{threshold_large}
#'         are formatted in scientific notation with decimal \code{digits}.
#'   \item Integer values are formatted without decimal places.
#'   \item Non-integer values that don't require scientific notation are rounded to
#'         \code{digits} decimal places.
#'   \item NA values are replaced with empty strings if \code{replace_na = TRUE}.
#'   \item Empty strings in the input are preserved.
#'   \item For data frames, only numeric columns are processed; other columns remain unchanged.
#' }
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # Vector examples.
#' f_conditional_round(c(0.0001, 0.5, 3, 10000))
#' # Returns: "1.000e-04" "0.500" "3" "1.000e+04".
#'
#' f_conditional_round(c(0.0001, 0.5, 3, 10000, NA), replace_na = TRUE)
#' # Returns: "1.000e-04" "0.500" "3" "1.000e+04" ""
#'
#' # Data frame example.
#' df <- data.frame(
#'   name = c("A", "B", "C"),
#'   small_val = c(0.0001, 0.002, 0.5),
#'   integer = c(1, 2, 3),
#'   integer_mix = c(10, 20, 30.1),
#'   large_val = c(10000, 5000, NA)
#' )
#'
#' # Show only two digits.
#' f_conditional_round(df, digits = 2)
#'
#' # To keep Integers as Integers (no digits)
#' # in columns with mixed data (Integers and digits)
#' # set allow_integer_decimal_mix = TRUE
#' f_conditional_round(df, allow_integer_decimal_mix = TRUE)
#'
#' # Custom NA replacement string.
#' f_conditional_round(c(0.5, NA, 3), replace_na = TRUE, na_string = "-")
#' # Returns: "0.500" "-" "3"
#'
#' f_conditional_round(c(0.5, NA, 3), replace_na = TRUE, na_string = "")
#' # Returns: "0.500" "" "3"
#'
#' @export
f_conditional_round <- function(x,
                                threshold_small = 0.01,
                                threshold_large = 10000,
                                digits = 3,
                                replace_na = TRUE,
                                na_string = "-",
                                allow_integer_decimal_mix = FALSE) {

  if(threshold_small >= threshold_large){
    stop("threshold_small > threshold_large.
    Threshold_small should be smaller than threshold_large")
  }

  # If digits is NULL, return x unchanged (no rounding)
  if (is.null(digits)) return(x)

  # Helper function to check if all non-NA values are integers
  is_integer_column <- function(vec) {
    non_na_vals <- vec[!is.na(vec) & vec != ""]
    if (length(non_na_vals) == 0) return(FALSE)
    all(abs(non_na_vals %% 1) < .Machine$double.eps^0.5)
  }

  # Check if input is a data frame
  if (is.data.frame(x)) {
    # Identify numeric columns
    numeric_cols <- sapply(x, is.numeric)

    if (any(numeric_cols)) {
      result <- x

      for (col in names(x)[numeric_cols]) {


        result[[col]] <- f_conditional_round(x[[col]],
                                             threshold_small = threshold_small,
                                             threshold_large = threshold_large,
                                             digits = digits,
                                             replace_na = replace_na,
                                             allow_integer_decimal_mix = allow_integer_decimal_mix)
      }
      return(result)
    } else {
      warning("No numeric columns found in the data frame.")
      return(x)
    }
  } else {
    # For vectors, apply the logic based on allow_integer_decimal_mix
    result <- x
    non_na <- !is.na(x)

    # Exact zeros get "0" -- the existing is_zero guard at the bottom handles this

    if (any(non_na)) {
      use_scientific <- abs(x[non_na]) < threshold_small | abs(x[non_na]) >= threshold_large


      # Scientific notation
      if (any(use_scientific)) {
        result[non_na][use_scientific] <- formatC(x[non_na][use_scientific],
                                                  format = "e", digits = digits)
      }

      # Regular values
      if (any(!use_scientific)) {
        non_sci_values <- x[non_na][!use_scientific]
        is_exact_zero  <- non_sci_values == 0
        rounds_to_zero <- !is_exact_zero & round(abs(non_sci_values), digits) == 0

        # Mirror data frame column logic exactly
        if (is_integer_column(non_sci_values)) {
          # All-integer: always no decimals regardless of allow_integer_decimal_mix
          result[non_na][!use_scientific] <-
            as.character(non_sci_values)
        } else if (allow_integer_decimal_mix == FALSE) {
          # Mixed but uniform formatting requested
          if (any(!rounds_to_zero & !is_exact_zero)) {
            result[non_na][!use_scientific][!rounds_to_zero & !is_exact_zero] <-
              formatC(non_sci_values[!rounds_to_zero & !is_exact_zero],
                      format = "f", digits = digits)
          }
          if (any(rounds_to_zero)) {
            result[non_na][!use_scientific][rounds_to_zero] <-
              formatC(0, format = "f", digits = digits)
          }
        } else {
          # Mixed with per-cell detection
          is_integer <- abs(non_sci_values %% 1) < .Machine$double.eps^0.5
          if (any(!is_integer & !rounds_to_zero & !is_exact_zero)) {
            result[non_na][!use_scientific][!is_integer & !rounds_to_zero & !is_exact_zero] <-
              formatC(non_sci_values[!is_integer & !rounds_to_zero & !is_exact_zero],
                      format = "f", digits = digits)
          }
          if (any(rounds_to_zero)) {
            result[non_na][!use_scientific][rounds_to_zero] <-
              formatC(0, format = "f", digits = digits)
          }
          if (any(is_integer)) {
            result[non_na][!use_scientific][is_integer] <-
              as.character(non_sci_values[is_integer])
          }
        }
      }
      # Special handling for zeros - always simple "0"
      is_zero <- !is.na(x) & x == 0
      if (any(is_zero)) {
        result[is_zero] <- "0"
      }
    }

    if (replace_na) {
      result[is.na(result) | (!is.na(result) & result == "")] <- na_string
    }

    return(result)
  }
}

#' Convert multiple columns to Factors in a data frame
#'
#' Converts multiple specified columns of a data frame into factors. If no columns are specified, it automatically detects and converts columns that are suitable to be factors. The function returns the entire data frame including non factor columns and can report the properties of this new data frame in the console (\code{properties = TRUE}).
#'
#' @param data A data frame containing the columns to be converted.
#' @param select A character vector specifying the names of the columns to convert into factors. If \code{NULL}, the function automatically detects columns that should be factors based on their data type and unique value count. Default is \code{NULL}.
#' @param exclude A character vector specifying the names of the columns NOT to convert into factors. If \code{NULL}, no columns are excluded. Default is \code{NULL}.
#' @param force_factors Logical. If \code{TRUE} all columns in the data.frame will be converted to factors except for the excluded columns using \code{exclude}.
#' @param properties Logical. If \code{TRUE}, prints a detailed table about the properties of the new data frame to the console. If \code{FALSE} no property table will be printed to the console. Default is \code{FALSE}.
#' @param unique_num_treshold  Numeric. A threshold of the amount of unique numbers a numeric column should have to keep it numeric, i.e. omit factor conversion. Default \code{8}.
#' @param repeats_threshold  Numeric. A threshold of the minimal number of repeats a numeric column should have to convert it to a factor. Default \code{2}.
#' @param ... Additional arguments passed to the \code{factor()} function of baseR.
#'
#' @details
#' \itemize{
#' \item If \code{select} is \code{NULL}, the function identifies columns with character data or numeric data with fewer than 8 unique values as candidates for conversion to factors.
#' \item The function checks if all specified columns exist in the data frame and stops execution if any are missing.
#' \item Converts specified columns into factors, applying any additional arguments provided.
#' \item Outputs a summary data frame with details about each column, including its type, class, number of observations, missing values, factor levels, and labels.
#'}
#' @return Returns the modified data frame with the specified (or all suitable) columns converted to factors. Can also force a print of a summary of the data frame's structure to the console (properties = TRUE).
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # Make a data.frame:
#' df <- data.frame(a = c("yes", "no", "yes", "yes", "no",
#'                        "yes", "yes", "no", "yes"),
#'                  b = c(1, 2, 3, 1, 2, 3, 1, 2, 3),
#'                  c = c("apple", "kiwi", "banana", "apple", "kiwi",
#'                         "banana", "apple", "kiwi", "banana"),
#'                  d = c(1.1, 1.1, 3.4, 4.5, 5.4, 6.7, 7.8, 8.1, 9.8)
#'                  )
#' str(df)
#'
#' # Convert specified columns to factors:
#' df1 <- f_factors(df, select = c("a", "c"))
#' str(df1)
#'
#'
#' # Convert all potential factor columns to factor but exclude column "b":
#' df2 <- f_factors(df, exclude = c("b"))
#' str(df2)
#'
#' # Convert all columns to factor but exclude column "b":
#' df3 <- f_factors(df, exclude = c("b"), force_factors = TRUE)
#' str(df3)
#'
#' # Or automatically detect and convert suitable columns to factors.
#' # Thus obtaining the same results as above automatically:
#' df4 <- f_factors(df)
#' str(df4)
#'
#' # In example above col b was converted to a factor as the number of repeats = 2
#' # and the amount of unique numbers < 8. In order to keep b numeric we can also
#' # adjust the unique_num_treshold and/or repeats_threshold:
#' df5 <- f_factors(df, unique_num_treshold = 2)
#' str(df5)
#'
#' # Use `properties = TRUE` to view the data frame's structure.
#' # This forces a printed output which is more insight than standard str() output.
#' df6 <- f_factors(df, properties = TRUE)
#'
#' @seealso
#'  \href{https://stat.ethz.ch/R-manual/R-devel/library/base/html/factor.html}{\code{factor}}
#'
#'
#' @export
#'
f_factors <- function(data,
                      select = NULL,
                      exclude = NULL,
                      properties = FALSE,
                      force_factors = FALSE,
                      unique_num_treshold = 8,
                      repeats_threshold = 2,
                      ...) {


    if (is.null(select) && force_factors == FALSE) {
      select <- names(data)[vapply(data, function(col) {
        is.character(col) ||
          (is.numeric(col) && length(unique(col)) < unique_num_treshold) && any(table(col) > repeats_threshold)
      }, logical(1))]
    }


  if(force_factors == TRUE){
    select <- names(data)
  }


    # If exclude is specified, remove those columns from select
    if (!is.null(exclude)) {
      select <- setdiff(select, exclude)
    }

  # Check if all specified columns exist in the data frame
  if (!all(select %in% names(data))) {
    stop("Some specified columns do not exist in the data frame.")
  }

  # Convert selected columns to factors
  data[select] <- lapply(data[select], function(col) factor(col, ...))


  if(properties == TRUE){
  # Create a data frame summarizing column names, classes, types, observations, and missing values
    summary <- data.frame(
      Column = names(data),
      Type = vapply(data, typeof, FUN.VALUE = character(1)),
      Class = vapply(data, class, FUN.VALUE = character(1)),
      Obs = vapply(data, function(col)
        sum(!is.na(col)), FUN.VALUE = integer(1)),
      Missing = vapply(data, function(col)
        sum(is.na(col)), FUN.VALUE = integer(1)),
      Factor_Levels = vapply(data, function(col)
        if (is.factor(col))
          nlevels(col)
        else
          0, FUN.VALUE = double(1)),
      Factor_Labels = vapply(data, function(col)
        if (is.factor(col))
          paste(levels(col), collapse = ", ")
        else
          "", FUN.VALUE = character(1))
    )

  # Print the data frame without row names
  # print(summary, row.names = FALSE)
  cat(paste(capture.output(summary), collapse = "\n"))
  cat("\n   \n")
  }

  return(data)
}

#Helper function to extract the extension provided by the user.
extract_extension <- function(filename) {

  # --- 1. Define Valid Extensions and their File Types (Lookup Table) ---
  ext_map <- c(
    "pdf"  = "pdf",
    "doc"  = "word",
    "docx" = "word",
    "xls"  = "excel",
    "xlsx" = "excel",
    "rmd"  = "rmd",
    "png"  = "png"
  )

  # Create the regex pattern dynamically from the map keys
  # This pattern MUST start with a literal dot: \. (ext1|ext2) $
  valid_ext_list <- paste(names(ext_map), collapse = "|")
  valid_regex <- paste0("\\.(", valid_ext_list, ")$")

  # --- 2. Check for Dot in the LAST 5 Characters ---

  # Check 2a: Get the last 5 characters of the filename
  last_5_chars <- substr(filename, nchar(filename) - 4, nchar(filename))

  # Check 2b: See if a dot exists in those last 5 characters
  dot_in_last_5 <- grepl("\\.", last_5_chars)

  # --- 3. Determine Status and Return Result ---

  # Check 1: Valid Extension Found (The filename ends with one of the valid extensions)
  if (grepl(valid_regex, filename, ignore.case = TRUE)) {

    # Extract the extension *without* the dot for the lookup
    extracted_ext_no_dot <- sub(paste0(".*\\."), "", filename, ignore.case = TRUE)
    ext_key <- tolower(extracted_ext_no_dot)
    file_type <- ext_map[ext_key]

    # Construct the output vector
    result <- c(
      "Extension" = paste0(".", ext_key),
      "FileType" = file_type
    )

    return(result)
  }

  # Check 2: Invalid Extension Found (A dot is present in the last 5 chars, but not a valid one)
  # NOTE: This only captures extensions 1-4 characters long (e.g., .f, .bak, .txt, .html)
  # that are NOT in the valid list. Longer invalid ones will be caught by the next check.
  else if (dot_in_last_5) {
    # Extract the full, invalid extension for the error message
    invalid_ext <- sub(paste0(".*\\."), ".", filename, ignore.case = TRUE)

    # Halt the function with a detailed error message
    stop(paste0("Invalid Extension '", invalid_ext,
                "' provided (or valid extension is > 5 chars). Must be one of: ",
                paste(names(ext_map), collapse = ", ")))
  }

  # Check 3: No Extension Found
  # This covers strings with no dot, or strings where the dot is more than 5 characters from the end.
  else {
    return(FALSE)
  }
}

 # foo <- unname(extract_extension("./hoh/kljkpf.lllll/xls"))
 # bar <- foo[1]
 # bar

#' Open a File with the Default Application
#'
#' Opens a specified file using the default application associated with its file type. It automatically detects the operating system (Windows, Linux, or macOS) and uses the appropriate command to open the file.
#'
#' @param filepath A character string specifying the path to the file to be opened. The path can be absolute or relative.
#'
#' @details
#' - On \bold{Windows}, the \code{f_open_file()} function uses \code{shell.exec()} to open the file.
#' - On \bold{Linux}, it uses \code{xdg-open} via the \code{system()} function.
#' - On \bold{macOS}, it uses \code{open} via the \code{system()} function.
#'
#' If an unsupported operating system is detected, the function will throw a message.
#'
#' @return Does not return a value; it is called for its side effect of opening a file.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # NOTE: The use of "if(interactive())" prevents this example from running
#' # during automated CRAN checks. This is necessary because the example
#' # opens a file, a behavior restricted by CRAN policies for automated
#' # testing.You don't need to use "if(interactive())" in your own scripts.
#'if(interactive()) {
#' # Open a PDF file.
#' f_open_file("example.pdf")
#'
#' # Open an image file.
#' f_open_file("image.png")
#'
#' # Open a text file.
#' f_open_file("document.txt")
#' }
#'
#' @seealso [shell.exec()], [system()]
#'
#' @export


f_open_file <- function(filepath) {
  # Detect operating system
  os <- Sys.info()["sysname"]

  # Use appropriate command based on OS
  if (os == "Windows") {
    shell.exec(filepath)

  } else if (os == "Linux") {
    # 1. use 'env LD_LIBRARY_PATH=' to clear RStudio's custom libraries so they
    #    don't conflict with system apps (like LibreOffice).
    # 2. use wait = FALSE so R doesn't freeze while the file is open.
    # 3. ignore stdout/stderr to keep the R console clean.
    system(paste("env LD_LIBRARY_PATH= xdg-open", shQuote(filepath)),
           wait = FALSE, ignore.stdout = TRUE, ignore.stderr = TRUE)

  } else if (os == "Darwin") { # macOS
    system(paste("open", shQuote(filepath)), wait = FALSE)

  } else {
    message("Unsupported operating system, the file will not be opened")
  }
}

#' Transform 'Wide' (Excel) data to 'Long' (R) format
#'
#' @description
#' This function converts "wide" data (e.g. Excel tables) into a "long" list format.
#' This is the essential first step to prepare your data for analysis and plotting in R.
#'
#' @details
#' Research data in Excel or output from lab instruments often contains measurements
#' side-by-side (in columns). Many R functions require measurements in a single
#' column (rows). `f_long` performs this translation for you.
#'
#' It performs three actions in one go:
#' 1. Selects your measurement columns (`measure_columns`).
#' 2. Keeps your important ID columns (`keep_cols`) and removes the rest.
#' 3. (Optional) Renames cryptic column headers into readable labels (`category_labels`).
#'
#' @param data The input data frame (e.g., from \code{read_excel}).
#' @param measure_columns (Optional) The columns containing your numeric measurements.
#'   These values are often the response variables, i.e. will end up on the Y-axis.
#'   \itemize{
#'     \item Use column names: \code{c("OD_t0", "OD_t1")}
#'     \item Use column numbers: \code{2:5}
#'     \item Use helpers: \code{starts_with("Measure")}
#'   }
#'   If NULL (default), the function will pivot ALL columns except those in \code{keep_cols}.
#' @param keep_cols (Optional) The columns that identify your samples (IDs).
#'   E.g., "SampleID", "PatientID", "Treatment", "Student number".
#'   These are repeated for every measurement.
#'   *If left empty, all non-measured columns are kept.*
#'   Important: If \code{measure_columns} is NULL, you MUST specify \code{keep_cols}.
#' @param category_name Name for the new column containing the headers.
#'   Default is "name". Choose something logical like "Timepoints", "Genes", or "Condition".
#' @param value_name Name for the new column containing the numbers.
#'   Default is "value". Choose something logical like "Absorbance", "Ct_Value", or "Weight".
#' @param category_labels (Optional) A character vector of new, readable names for your categories, i.e. the measure_columns that you entered.
#'   \strong{Note:} The order must match the order of \code{measure_columns} exactly.
#'   Useful for renaming "t0_raw" to "Start" instantly.
#' @param ... Additional arguments passed to \code{tidyr::pivot_longer}.
#'   E.g., \code{values_drop_na = TRUE} to remove empty cells immediately.
#'
#' @return A "Tidy" data frame (tibble) of class \code{f_long}.
#'
#' @note The custom class and attributes (\code{f_long_value}, \code{f_long_category})
#'   are used by the \code{plot} and \code{summary} methods. Be aware that most
#'   \pkg{dplyr} or \pkg{tidyr} operations (e.g., \code{filter}, \code{mutate})
#'   will silently strip these attributes. If that happens, use \code{f_scan} or
#'   \code{f_summary} directly with explicit column names instead.

#' @export
#' @examples
#' # --- Example 1: Using the 'iris' dataset ---
#' # Scenario: The iris dataset looks clean, but it is actually "Wide".
#' # It has 4 columns of measurements side-by-side.
#' # To compare Sepal Length vs Width in a plot, we must stack them.
#'
#' head(iris)
#'
#' # Reshape: Combine Length and Width into one column and plot the data.
#' iris_long <- f_long(
#'   data = iris,
#'   measure_columns = c("Sepal.Length", "Sepal.Width"),
#'   keep_cols = "Species",
#'   category_name = "Sepal_Dimension",    # Describes the grouping (What did we measure?)
#'   value_name = "Size_cm",               # Describes the value (What is the number?)
#'   category_labels=c("Length", "Width")  # New category labels
#'   )
#'
#'  head(iris_long)
#'
#'  # Plot the data using f_scan
#'  plot(iris_long)
#'
#'  # Make a f_summary table of iris_long
#'  summary(iris_long)
#'
#' # --- Example 2: Using the 'airquality' dataset ---
#' # Scenario: Pivot daily measurements of Wind and Temperature over time.
#'
#' head(airquality)
#'
#' weather_long <- f_long(
#'   data = airquality,
#'   measure_columns = c("Wind", "Temp"),
#'   keep_cols = c("Month", "Day"),
#'   category_name = "Climate_Parameter", # Descriptive name
#'   value_name = "Reading_Value",        # Generic name (since units differ: mph vs F)
#'   values_drop_na = TRUE
#' )
#'
#' head(weather_long)

f_long <- function(data,
                   measure_columns = NULL,
                   keep_cols = NULL,
                   category_name = "name",
                   value_name = "value",
                   category_labels = NULL,
                   ...) {

  if (!is.data.frame(data)) stop("rfriend error: Input 'data' must be a data frame.")

  # --- LOGIC SWITCH: MANUAL VS AUTOMATIC ---

  # Variable to store columns to be pivoted (for checking purposes later)
  target_cols <- NULL

  # Scenario 1: Lazy Mode (Pivot everything except keep_cols)
  if (is.null(measure_columns)) {

    if (is.null(keep_cols)) {
      stop("rfriend error: Lazy Mode active (no measure_columns selected); that is why you MUST specify 'keep_cols' so I know which columns NOT to pivot.")
    }

    # First calculate which columns these are for the safety check
    # Use dplyr::select to simulate what remains
    temp_data <- dplyr::select(data, -{{ keep_cols }})
    target_cols <- names(temp_data)

    message(paste0("rfriend: Lazy Mode active. Pivoting all columns EXCEPT: ", paste(names(dplyr::select(data, {{ keep_cols }})), collapse=", ")))

  } else {
    # Scenario 2: Manual Mode
    # Here we are sure what the user wants, but we retrieve names for safety
    temp_data <- dplyr::select(data, {{ measure_columns }})
    target_cols <- names(temp_data)

    if (!is.null(keep_cols)) {
      data <- dplyr::select(data, {{ keep_cols }}, all_of(target_cols))
    }

  }

  # --- THE SAFETY CHECK FOR LABELS ---
  if (!is.null(category_labels)) {

    # 1. Length check
    if (length(target_cols) != length(category_labels)) {
      stop(paste0("rfriend error: Count mismatch! Found ", length(target_cols), " columns to pivot, but you provided ", length(category_labels), " labels."))
    }

    # 2. Visual Check (Mapping)
    # This is crucial for the user
    message("---------------------------------------------------")
    message("rfriend SAFETY CHECK: \nPlease verify the order and names below.")
    message("\nColumn Name          -->  New Label")
    message("---------------------------------------------------")

    # Print the first 10 mappings (to avoid spamming with large datasets)
    limit <- min(10, length(target_cols))
    for(i in 1:limit) {
      # Ensure clean alignment
      col_fixed <- format(target_cols[i], width = 20)
      message(paste0(col_fixed, " -->  ", category_labels[i]))
    }
    if (length(target_cols) > 10) message("... (and ", length(target_cols) - 10, " others)")
    message("---------------------------------------------------")

    # If Manual Mode, we trust the user.
    # If Lazy Mode, we give an extra warning.
    if (is.null(measure_columns)) {
      warning("rfriend tip: You are using automatic selection with renaming. Check the table above carefully! R determines the column order.", call. = FALSE)
    }
  }

  # --- EXECUTION ---

  tryCatch({
    # Use target_cols here because we already calculated them.
    # This is safer than repeating the logic.
    result <- tidyr::pivot_longer(
      data = data,
      cols = all_of(target_cols), # Use all_of because we have names as strings now
      names_to = category_name,
      values_to = value_name,
      ...
    )

    # Apply Labels
    if (!is.null(category_labels)) {
      result[[category_name]] <- factor(result[[category_name]],
                                             levels = target_cols,
                                             labels = category_labels)
    }

    # --- ADD CLASS AND ATTRIBUTES  ---
    # Store the column names so plot() knows what to use later
    attr(result, "f_long_value") <- value_name
    attr(result, "f_long_category") <- category_name

    # Add "f_long" to the class list (keeping existing classes like tbl_df)
    class(result) <- c("f_long", class(result))

    return(result)

  }, error = function(e) {
    stop(paste0("rfriend technical error: ", e$message))
  })
}


#' Plot method for f_long objects
#'
#' @description
#' Automatically runs a \code{f_scan} diagnostic plot on data created by \code{f_long}.
#'
#' @param x An object of class \code{f_long} (output from \code{f_long}).
#' @param summary Logical. If \code{TRUE}, generates the summary table within the scan. Default is \code{TRUE}.
#' @param ... Additional arguments passed to \code{f_scan}.
#'
#' @return
#' Returns the output of \code{f_scan} (an object of class \code{f_scan}) invisibly.
#'
#' @export
#' @method plot f_long
plot.f_long <- function(x, summary = TRUE, ...) {

  # 1. Retrieve the hidden column names from attributes
  val_col <- attr(x, "f_long_value")
  cat_col <- attr(x, "f_long_category")

  # 2. Safety check (in case attributes are lost)
  if (is.null(val_col) || is.null(cat_col)) {
    stop("rfriend error: This data seems to have lost its f_long attributes. Please specify columns manually in f_scan().")
  }

  # 3. Call f_scan
  result <- f_scan(data = x,
                   column = val_col,
                   group_vars = cat_col,
                   summary = FALSE,
                   outliers = FALSE,
                   ...)

  # 4. Return the result
  return(result)
}


#' Summary method for f_long objects
#'
#' @description
#' Automatically runs the \code{f_summary} function on data created by \code{f_long}
#' using the attributes stored in the object.
#'
#' @param object An object of class \code{f_long} (output from \code{f_long}).
#' @param ... Additional arguments passed to \code{f_summary}.
#'
#' @return
#' Returns the summary table (usually a data frame or tibble) produced by \code{f_summary}.
#'
#' @export
#' @method summary f_long
summary.f_long <- function(object, ...) {

  # 1. Retrieve the hidden column names from attributes
  val_col <- attr(object, "f_long_value")
  cat_col <- attr(object, "f_long_category")

  # 2. Safety check (in case attributes are lost)
  if (is.null(val_col) || is.null(cat_col)) {
    stop("rfriend error: This data seems to have lost its f_long attributes. Please specify columns manually in f_summary().")
  }

  # 3. Call f_summary
  # We return the result visibly. When the user types summary(x),
  # R will automatically print the returned table.
  f_summary(object,
            val_col,
            cat_col,
            ...)
}

#' Plot a Histogram with an Overlaid Normal Curve
#'
#' This function creates a histogram of the provided data and overlays it with a normal distribution curve.
#'
#' @param data A numeric vector of data values to be plotted.
#' @param main A character string specifying the title of the histogram. Default is \code{"Histogram with Normal Curve"}.
#' @param xlab A character string specifying the label for the x-axis. Default is the name of the data variable.
#' @param probability A logical value indicating whether to plot a probability or frequency histogram. Default is \code{TRUE}.
#' @param col A character string specifying the fill color of the histogram bars. Default is \code{"white"}.
#' @param border A character string specifying the color of the histogram bar borders. Default is \code{"black"}.
#' @param line_col A character string specifying the color of the normal curve line. Default is \code{"red"}.
#' @param save_png A logical value default \code{FALSE}, if \code{TRUE} a png file is saved under the name of the data of under the specified file name.
#' @param open_png Logical. If \code{TRUE}, opens generated png files.
#' @param save_as Character string specifying the output file path (without extension).
#'   If a full path is provided, output is saved to that location.
#'   If only a filename is given, the file is saved in \code{tempdir()}.
#'   If only a directory is specified (providing an existing directory with trailing slash),
#'   the file is named "data_name_histogram.png" in that directory.
#'   Defaults to \code{file.path(tempdir(), "data_name_histogram.png")}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.
#' @param width Numeric, png figure width default \code{8} inch.
#' @param height Numeric, png figure height default \code{7} inch.
#' @param units Character string, png figure units default \code{"in"} = inch, other options are: \code{"px"} = Pixels, \code{"cm"} centimeters, \code{"mm"} millimeters.
#' @param res Numeric, png figure resolution default \code{300} dpi.
#' @param ... Additional arguments to be passed to the \code{hist} function.
#'
#' @details
#' The function first captures the name of the input variable for labeling purposes. It then calculates a sequence of x-values and corresponding y-values for a normal distribution based on the mean and standard deviation of the data. The histogram is plotted with specified aesthetics, and a normal curve is overlaid. To increase resolution you can use \code{png(...,res = 600)} or the 'RStudio' chunk setting, e.g. \code{dpi=600}.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @return A histogram plot is created and the function returns this as a \code{recordedplot}.
#'
#'
#' @examples
#' # Example usage:
#' set.seed(123)
#' sample_data <- rnorm(100)
#' f_hist(sample_data)
#'
#'
#' @seealso
#'  \href{https://stat.ethz.ch/R-manual/R-devel/library/graphics/html/hist.html}{\code{hist}}, \href{https://stat.ethz.ch/R-manual/R-devel/library/stats/html/Normal.html}{\code{dnorm}},  \href{https://stat.ethz.ch/R-manual/R-devel/library/graphics/html/lines.html}{\code{lines}}
#'
#'
#' @export
f_hist <- function(data,
                  main = NULL,
                  xlab = NULL,
                  probability = TRUE,
                  col = "white",
                  border = "black",
                  line_col = "red",
                  #ouput png settings
                  save_png = FALSE,
                  open_png = TRUE,
                  save_as = NULL,
                  save_in_wdir = FALSE,
                  width = 8,
                  height = 7,
                  units = "in",
                  res = 300,
                  ...) {
  # Capture the name of the input variable
  data_name <- deparse(substitute(data))
  data_name <- sub(".*\\$", "", data_name)  # Remove everything before the "$" symbol

  if (is.null(xlab)) xlab <- data_name  # Set x-axis label if not provided
  if (is.null(main)) main <- "Histogram with Normal Curve"

  # Define x values for the normal curve
  x_values <- seq(min(data, na.rm = TRUE), max(data, na.rm = TRUE), length = 100)

  # Calculate y values for the normal curve
  y_values <- dnorm(x_values, mean = mean(data, na.rm = TRUE), sd = sd(data, na.rm = TRUE))

  # Create histogram and capture density or count information
  hist_data <- hist(
    data,
    plot = FALSE # Do not plot yet, just calculate values
  )

  if (probability) {
    # Use density for y-axis
    max_hist_value <- max(hist_data$density, na.rm = TRUE)
    max_curve_value <- max(y_values, na.rm = TRUE)
    ylim_max <- max(max_hist_value, max_curve_value)
  } else {
    # Adjust y_values for count data
    y_values <- y_values * length(data) * diff(hist_data$breaks[1:2])
    max_hist_value <- max(hist_data$counts, na.rm = TRUE)
    max_curve_value <- max(y_values, na.rm = TRUE)
    ylim_max <- max(max_hist_value, max_curve_value)
  }

  # Plot histogram with adjusted ylim
  hist(
    data,
    probability = probability,
    col = col,
    border = border,
    main = main,
    xlab = xlab,
    ylim = c(0, ylim_max), # Use calculated maximum for ylim
    ...
  )

  # Overlay the normal curve
  lines(x_values, y_values, col = line_col, lwd = 2)

  saved_plot <- recordPlot()

  if (save_png == TRUE || !is.null(save_as) || save_in_wdir == TRUE){

    #### Handle option "save_as = " ###
    if(save_in_wdir == TRUE){
      save_dir <- getwd()
    }else{
      save_dir <- tempdir()
    }

    output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "histogram.png", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = ".png"
                                )



    png(
      paste0(output_path),
      width = width,
      height = height,
      units = units,
      res = res
    )

    replayPlot(saved_plot)

    # Close the png file
    dev.off()

    if (open_png == TRUE){
      f_open_file(output_path)
    }
  }



  return(invisible(saved_plot))
}

#' Apply a black or white 'RStudio' Theme and Zoom Level
#'
#' This comes in hand when teaching, the function allows users to apply a "black" or "white" 'RStudio' theme and adjust the zoom level in the 'RStudio' IDE. It includes error handling for invalid inputs.
#'
#' @param color A character string. The theme color to apply. Must be either \code{"black"} (dark theme) or \code{"white"} (light theme). Default is \code{"black"}.
#' @param zlevel A numeric value. The zoom level to apply, ranging from \code{0} (default size) to \code{4} (maximum zoom). Default is \code{0}.
#'
#' @details
#' The function performs the following actions:
#' \enumerate{
#' \item Applies the specified 'RStudio' theme:
#' \itemize{
#'   \item \code{"black"}: Applies the "Tomorrow Night 80s" dark theme.
#'   \item \code{"white"}: Applies the "Textmate (default)" light theme.
#'   }
#' \item Adjusts the zoom level in 'RStudio':
#' \itemize{
#'   \item \code{zlevel = 0}: Resets to default zoom level.
#'   \item \code{zlevel = 1}: Zooms in once.
#'   \item \code{zlevel = 2}: Zooms in twice.
#'   \item \code{zlevel = 3}: Zooms in three times.
#'   \item \code{zlevel = 4}: Zooms in four times.
#'   }
#'}
#' The function includes error handling to ensure valid inputs:
#'  \itemize{
#' \item \code{color} must be a character string and one of \code{"black"} or \code{"white"}.
#' \item \code{zlevel} must be a numeric value, an integer, and within the range of 0 to 4. If a non-integer is provided, it will be rounded to the nearest integer with a warning.
#' }
#'
#'@return None. The function is called for its side effects of changing the 'RStudio' theme or Zoomlevel.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @return This function does not return a value. It applies changes directly to the 'RStudio' IDE.
#'
#' @examples
#' # NOTE: This example will change your RStudio theme hence the dont run warning.
#' \dontrun{
#' # Apply a dark theme with with zoom level 2:
#' f_theme(color = "black", zlevel = 2)
#'
#' # Apply a black theme with maximum zoom level:
#' f_theme(color = "black", zlevel = 4)
#'
#' # Apply the default light theme default zoom level:
#' f_theme(color = "white", zlevel = 0)
#' }
#'
#' @export
#'
f_theme <- function(color = "black", zlevel = 0) {

# Use tryCatch to catch missing object errors
tryCatch({

  # Check if theme color is character
  if (!is.character(color)) {
    stop("Error: theme color must be a character value either:  \"black\" or \"white\"")
  }

  # Check if zlevel is numeric
  if (!is.numeric(zlevel)) {
    stop("Error: zlevel must be a numeric value!")
  }
  # Additional checks
  if (is.na(zlevel)) {
    stop("Error: zlevel cannot be NA!")
  }
  if (zlevel %% 1 != 0) {
    warning("Error: the zlevel value: ", zlevel, " has been rounded to the nearest Integer: ", round(zlevel),"!")
  }
  # Check for invalid zlevel
  if (zlevel > 4) {
    stop("Error: zlevel cannot be higher than 4!")
  }


  # Reset the zoom function to default to prevent additional zoom
  rstudioapi::executeCommand('zoomActualSize')

  # Match the 'color' argument
  color <- match.arg(color, choices = c("black", "white"))

  # Calculate the statistic
  switch(
    color,
    black = rstudioapi::applyTheme("Tomorrow Night 80s"),
    white = rstudioapi::applyTheme("Textmate (default)")
  )


  #the switch function starts at 1 not at 0
  zlevel <- round(zlevel) + 1

  switch(
    zlevel,
  #set zoom level 0 = default, 1 = zoom 1x, 2 = zoom 2x, 3 = zoom 3x, 4 = zoom 4x
  rstudioapi::executeCommand('zoomActualSize'),
  rstudioapi::executeCommand('zoomIn'),
  c(rstudioapi::executeCommand('zoomIn'),
  rstudioapi::executeCommand('zoomIn')),
  c(rstudioapi::executeCommand('zoomIn'),
    rstudioapi::executeCommand('zoomIn'),
    rstudioapi::executeCommand('zoomIn')),
  c(rstudioapi::executeCommand('zoomIn'),
    rstudioapi::executeCommand('zoomIn'),
    rstudioapi::executeCommand('zoomIn'),
    rstudioapi::executeCommand('zoomIn'))
  )
}, error = function(e) {
  # Handle case where object is not found
  if (grepl("object.*not found", e$message)) {
    cat(crayon::red("Error: Did you mean to use quotes? Try using '\"white\"' or '\"black\"'.\n"))
  } else if (grepl("*zlevel*", e$message)) {
    # Re-throw other errors as-is
    cat(crayon::red(e$message))
  } else {
    stop(e$message)
  }
})
}

#' Generate a Boxplot Report of a data.frame
#'
#' Generates boxplots for all numeric variables in a given dataset, grouped by factor variables. The function automatically detects numeric and factor variables. It allows two output formats ('pdf', 'Word') and includes an option to add a general explanation about interpreting boxplots.
#' @param x A data.frame or formula (dispatches to the right method).
#' @param formula A formula specifying the factor to be plotted. More response variables can be added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor}) to generate multiple boxplots. If the formula is omitted and only \code{data} is provided all data will be used for creating boxplots.
#' @param data A \code{data.frame} containing the data to be used for creating boxplots.
#' @param fancy_names An optional named vector mapping column names in \code{data} to more readable names for display in plots (name map). Defaults to \code{NULL}.
#' @param output_type Character string, specifying the output format: \code{"pdf"}, \code{"word"}, \code{"rmd"} or \code{"png"}. The option \code{"rmd"} saves rmd code in the output object not in a file. Default is \code{"pdf"}.
#' @param save_as Character string specifying the output file path (without extension).
#'   If a full path is provided, output is saved to that location.
#'   If only a filename is given, the file is saved in \code{tempdir()}.
#'   If only a directory is specified (providing an existing directory with trailing slash),
#'   the file is named "dataname_BoxPlot" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
#'   Defaults to \code{file.path(tempdir(), "dataname_BoxPlot.pdf")}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.
#' @param close_generated_files Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param boxplot_explanation A logical value indicating whether to include an explanation of how to interpret boxplots in the report. Defaults to \code{TRUE}.
#' @param detect_factors A logical value indicating whether to automatically detect factor variables in the dataset. Defaults to \code{TRUE}.
#' @param jitter A logical value, if \code{TRUE} all data per boxplot is shown, if \code{FALSE} (default) individual data points (except for outliers) are omitted.
#' @param width Numeric, png figure width default \code{8} inch
#' @param height Numeric, png figure height default \code{7} inch
#' @param units Character string, png figure units default \code{"in"} = inch, other options are: \code{"px"} = Pixels, \code{"cm"} = centimeters, \code{"mm"} = millimeters.
#' @param res Numeric, png figure resolution default 300 dpi
#' @param las An integer (\code{0} t/m \code{3}), \code{las = 0}: Axis labels are parallel to the axis. \code{las = 1}: Axis labels are always horizontal. \code{las = 2}: Axis labels are perpendicular to the axis. (default setting). \code{las = 3}: Axis labels are always vertical.
#' @param outliers Logical. If \code{TRUE}, scans for outliers using Tukey's fences and if they exist, adds them to the report using \code{f_outliers}. Default \code{TRUE}.
#' @param coef Numeric. The multiplier for the Interquartile Range (IQR) used for outlier detection. Default \code{1.5}.
#' @param limit_columns Integer or \code{NULL}. Defines the number of columns shown in the outlier table. Default = \code{7}. \code{NULL} = all columns are shown.
#' @param ... Further arguments forwarded to \code{f_boxplot_worker},
#'   such as \code{fancy_names}, \code{title}, \code{fill}, etc.
#'
#' @details
#' The function performs the following steps:
#' \itemize{
#' \item Detects numeric and factor variables in the dataset.
#' \item Generates boxplots for each numeric variable grouped by each factor variable.
#' \item Outputs the report in the specified format ('pdf', 'Word' or 'Rmd').
#'}
#'
#'If \code{output_type = "rmd"} is used it is adviced to use it in a chunk with \{r, echo=FALSE, results='asis'\}
#'
#' If no factor variables are detected, the function stops with an error message since factors are required for creating boxplots.
#'
#' This function will plot all numeric and factor candidates, use the function \code{subset()} to prepare a selection of columns before submitting to \code{f_boxplot()}.
#'
#' Note that there is an optional \code{jitter} option to plot all individual data points over the boxplots.
#'
#' This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
#'
#' \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
#'
#' \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
#'
#' \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
#'
#' If Pandoc is not found, this function may not work as intended.
#'
#'
#' @return The return value depends on \code{output_type}:
#' \itemize{
#'   \item \code{"pdf"} and \code{"word"}: Writes a report file to \code{save_as} (or \code{tempdir()} by default) and returns \code{NULL} invisibly. The file can optionally be opened with \code{open_generated_files = TRUE}.
#'   \item \code{"png"}: Writes one PNG file per response x factor combination into the directory given by \code{save_as} and returns \code{NULL} invisibly.
#'   \item \code{"rmd"}: Returns the generated R Markdown content as a single character string (invisibly). No file is written and nothing is printed to the console. The caller can \code{cat()} the string, assign it to a variable, or embed it in a larger report (see Examples).
#' }
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' \donttest{
#' # Example usage:
#' data(iris)
#'
#' new_names = c(
#'   "Sepal.Length" = "Sepal length (cm)" ,
#'   "Sepal.Width" = "Sepal width (cm)",
#'   "Petal.Length" = "Petal length (cm)",
#'   "Petal.Width" = "Petal width (cm)",
#'   "Species" = "Cultivar"
#' )
#'
#' # Use the whole data.frame to generate an MS Word report and don't open it.
#' f_boxplot(iris,
#'            fancy_names = new_names,
#'            output_type = "word"
#'            )
#'
#' # Use a formula to plot several response parameters (response 1 + response 2 etc)
#' # and generate a rmd output without boxplot_explanation.
#' data(mtcars)
#' f_boxplot(hp + disp ~ gear*cyl,
#'            data=mtcars,
#'            boxplot_explanation = FALSE,
#'            output_type = "word"
#'            )
#'
#' # Capture the R Markdown output as a string and render it inline.
#' # Use output_type = "rmd" to get the markdown back as a character value
#' # instead of writing a file. Useful for embedding in a larger knitr document.
#' rmd <- f_boxplot(iris,
#'                  output_type         = "rmd",
#'                  boxplot_explanation = FALSE,
#'                  outliers            = FALSE
#'                  )
#'
#' # Display it in the console
#' cat(rmd)
#'
#' # ...or splice it into a knitr child chunk with results = "asis":
#' #   ```{r, echo=FALSE, results='asis'}
#' #   cat(rmd)
#' #   ```
#' }
#'
#' @export
# Public generic -first argument drives dispatch
f_boxplot <- function(x, ...) {
  UseMethod("f_boxplot")
}

#' @export
#' @rdname f_boxplot
# Dispatch when first argument is a formula
f_boxplot.formula <- function(formula, data, ...) {
  f_boxplot_worker(formula = formula, data = data, ...)
}

#' @export
#' @rdname f_boxplot
# Dispatch when first argument is a data.frame
f_boxplot.data.frame <- function(x, ...) {
  data <- x  # alias for readability inside the body
  f_boxplot_worker(formula = NULL, data = data, ...)
}

#' @export
#' @rdname f_boxplot
# Private worker - all the real logic lives here
f_boxplot_worker <- function(formula = NULL, data, fancy_names = NULL,
                             output_type = "pdf", outliers = TRUE,
                             coef = 1.5,
                      limit_columns = 7,
                      save_as = NULL,
                      # Specify the name of the output dir and file (name and type).
                      save_in_wdir = FALSE,
                      # Save file output in the working directory.
                      close_generated_files = FALSE,
                      # Closes either open word files depending on the output format.
                      open_generated_files = interactive(),
                      # Open files after creation
                      boxplot_explanation = TRUE,
                      # This text reminds the user on how to read a boxplot.
                      detect_factors  = TRUE,
                      # Detect factors automatically (TRUE) or not (FALSE)
                      jitter = FALSE,
                      # show individual data points
                      #ouput png settings
                      width = 8,
                      height = 7,
                      units = "in",
                      res = 300,
                      las = 2

)
{

  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state


  if( !(output_type %in% c("pdf", "word", "rmd", "png")) ){
    stop("Character string specifying the output format (output_type = ) should be either: 'pdf', 'word', 'rmd', 'png' ")
  }

  # Generate a temporary file path for "output.Rmd"
  temp_output_dir <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")

  # Create the output file "output.Rmd" in tempdir()
  file.create(temp_output_file)
  file_extension <- NULL

  ####### Save dataframe name and Handle input from vectors (dataframe column) #####

  if(!is.null(data)){
    # Save dataframe name
    data_name <- deparse(substitute(data))

  } else if(is.null(data)){

    if(length(formula_extract_df_names(formula)) == 0){
      data_name <- "data"
    } else if(length(formula_extract_df_names(formula)) == 1){
      data_name <- formula_extract_df_names(formula)
    } else if(length(formula_extract_df_names(formula)) > 1){
      data_name <- paste(formula_extract_df_names(formula), collapse = "_")
    }

    # Make a data.frame based on the formula
    data <- formula_to_dataframe(formula)


    # Rewrite formula without data frame prefixes
    formula <- clean_formula(formula)

  }

  #### Handle option "save_as = " ###
  if (output_type != "rmd"){
    if(save_in_wdir == TRUE){
      save_dir <- getwd()
    }else{
      save_dir <- tempdir()
    }


  #map the output type to extensions
  output_type_map <- c(
    "pdf"  = ".pdf",
    "word" = ".docx",
    "excel"= ".xlsx",
    "png"  = ".png"
  )

  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(formula) #use helper_check_lhs.R



  # If the user specifies a path, filename or save_in_wdir == TRUE an output_file should be created
  if (!is.null(save_as) || save_in_wdir == TRUE) {

    if (!is.null(save_as)) {
      #Remove backslash in save_as if needed
      save_as <- gsub(pattern = "\\\\", replacement = "/", x = save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if(file_extension_save_as[1] != FALSE){
        file_extension <- file_extension_save_as
      }
    }

    if(is.null(file_extension) && output_type == "word"){
      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "BoxPlot", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = ".docx"
      )

      #set output_type to default
      output_type <- "word"

    }
    else if(is.null(file_extension) && output_type %in% c("pdf", "word", "excel", "rmd", "png")){

      #create extension based on input_type
      file.ext <- unname(output_type_map[output_type])

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "BoxPlot", sep = "_"),
                                   default_dir  = save_dir,
                                   file.ext     = file.ext
      )



    }
    else if(!is.null(file_extension) && length(file_extension) >= 2) {

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "BoxPlot", sep = "_"),
                                   default_dir  = save_dir,
                                   file.ext     = file_extension[1]
      )

      # Only override output_type if we actually got a valid type back
      if (!is.na(file_extension[2]) && nchar(file_extension[2]) > 0) {
        output_type <- file_extension[2]
      }
    }
  } else {

    #create extension based on input_type
    file.ext <- unname(output_type_map[output_type])

    # use helper get_save_path() to create output_path
    output_path <- get_save_path(save_as = save_as,
                                 default_name = paste(data_name, "BoxPlot", sep = "_"),
                                 default_dir  = save_dir,
                                 file.ext     = file.ext
    )

  }

  # Do not run explanation code when output is png
  if(output_type == "png"){
  boxplot_explanation <- FALSE
  }


  if (output_type == "rmd") close_generated_files <- FALSE

  # Cross-platform close_generated_files (was Windows-only taskkill)
  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    }
}

  # Wrap lines in rmd output document
  f_wrap_lines()

  # Create a counter for the number of factors and set it to zero
  factor_count <- 0



  # Automatically detect factor variables
  if (detect_factors == TRUE) {
    data <- f_factors(data)
  }

  # Rename the data.frame with fancy names if fancy_names are provided.
  if(!is.null(fancy_names)){
    data <- f_rename_columns(data, fancy_names)
  }


  # Count the number of factor variables
  factor_vars  <- vapply(data, is.factor, logical(1))
  factor_count <- sum(factor_vars)
  factor_vars  <- names(factor_vars)[factor_vars]

  if(factor_count == 0){
    stop("The data contains NO factor variable candidates, factors are required to make boxplots")
  }

  # Automatically detect numeric variables
  numeric_vars  <- vapply(data, is.numeric, logical(1)) # Identify numeric columns
  numeric_count <- sum(numeric_vars)
  numeric_vars  <- names(numeric_vars)[numeric_vars]


  if(!is.null(formula)){
    if(!is.null(fancy_names) ){
    formula <-  rename_formula_terms(formula, fancy_names)
    }
    # Extract and backtick-wrap RHS variable names that need it
    rhs_vars <- all.vars(formula[[3]])
    rhs_vars_backt <- ifelse(
      rhs_vars != make.names(rhs_vars),
      paste0("`", rhs_vars, "`"),
      rhs_vars
    )
    # Rebuild RHS string replacing bare names with backtick versions
    RHS <- deparse(formula[[3]])
    for (i in seq_along(rhs_vars)) {
      if (rhs_vars[i] != make.names(rhs_vars[i])) {  # only non-syntactic names
        backticked <- paste0("`", rhs_vars[i], "`")
        if (!grepl(backticked, RHS, fixed = TRUE)) {  # only if NOT already wrapped
          RHS <- gsub(rhs_vars[i], backticked, RHS, fixed = TRUE)
        }
      }
    }
    factor_vars <- all.vars(formula[[3]])  # bare names for data frame indexing
  }


generate_report <- function() {
    # This text reminds the user on how to read a boxplot.
    if(boxplot_explanation == TRUE){

      cat("  \n  \n
# Understanding Boxplots: A Visual Guide

A **boxplot** (or box-and-whisker plot) is a statistical tool for visualizing the distribution of a dataset. It offers a quick snapshot of the dataset's central tendency, spread, and potential outliers. Below is a guide to help you read and interpret a boxplot effectively.

## Anatomy of a Boxplot

1. **The Five-Number Summary**:
   - **Minimum**: The smallest value in the dataset (excluding outliers).
   - **First Quartile (Q1)**: The value below which 25% of the data falls.
   - **Median (Q2)**: The middle value of the dataset, splitting it into two equal halves.
   - **Third Quartile (Q3)**: The value below which 75% of the data lies.
   - **Maximum (Q4)**: The largest value in the dataset (excluding outliers).

&nbsp;  \n   \n

2. **The Box**:
   - The box represents the **interquartile range (IQR)**, spanning from Q1 to Q3, which contains the middle 50% of the data.
   - A vertical line inside the box marks the **median**, indicating the central value of the dataset.

&nbsp;  \n   \n

3. **The Whiskers**:
   - Whiskers extend from the box to the smallest and largest data points within **1.5 times the IQR range** from Q1 and Q3.
   - Values outside this range are considered **outliers**.

&nbsp;  \n   \n

4. **Outliers**:
   - These are individual points plotted beyond the whiskers. They highlight extreme values that may warrant further investigation.
&nbsp;  \n   \n

5. **Mean**:
   - Some boxplots include the **mean** (average) as a distinct marker, often represented by a cross or dot.

&nbsp;  \n   \n")

invisible(image_path <- system.file('extdata', 'boxplot_explained_rfriend.png',
                                    package = 'rfriend'))
# Include the saved plots in R Markdown
cat(paste0("![](", image_path, ")"), "   \n  \n")

cat("
## How to Interpret a Boxplot

- **Center**: The **median** represents the central tendency of the data and serves as the most reliable summary measure, especially in the presence of outliers or skewed distributions.
- **Spread**:
  - The **IQR** (length of the box) captures the variability of the middle 50% of the data.
  - The whiskers show the overall spread of values within the dataset.
- **Skewness**:
  - If the median is closer to one end of the box or if one whisker is longer, the dataset is skewed in that direction. Normal distribution should be evenanly distributed around the median and the median and mean should be similar.
- **Outliers**: Points outside the whiskers indicate unusual or extreme values that could influence the dataset's overall analysis.


## Why and When to Use Boxplots?

Boxplots are particularly valuable when comparing distributions across multiple groups or datasets. They allow you to quickly assess differences in central tendency, variability, and the presence of outliers, making them a powerful tool for exploratory data analysis. If data is not normally distributed and the mean is not a good summary measure, scientist resort to boxplot instead of barplot in their publications.")

    if(output_type != "rmd"){
      # Pagebreak
      cat("
<div style=\"page-break-after: always;\"></div>
\\newpage")
    }
    }


  #Main loop starts here
  for (response_name in numeric_vars) {

    if (response_name != make.names(response_name)) {
      response_name_backt <- paste0("`", response_name, "`")
    } else {
      response_name_backt <- response_name
    }

    if(output_type != "png"){
      cat("  \n  \n#  Boxplots of: ", response_name, "  \n")
    }

    if(!is.null(formula)){
        if(output_type != "png"){
        cat("##  Boxplot of: ", response_name, " as function of ", RHS, "  \n")
        }

        current_formula <- as.formula(paste(response_name_backt, "~", RHS))

        level_count <- NULL

        for(i in factor_vars){
          level_count[i] <- length(levels(data[[i]]))
        }

        factor_cols <- names(level_count)

        # Get the number of bars per boxplot and store in num.bars
        num.bars <- prod(level_count, na.rm = FALSE)

        # Define colors for boxplot
        c1 <- rainbow(num.bars)
        c2 <- rainbow(num.bars, alpha=0.2)
        c3 <- rainbow(num.bars, v=0.7)

        # Determine width of the boxes by making it relative to max here max boxes=9
        # this times two is 18, thus 9/18 = 0.5 the lower the num.bars the lower the relative width.
        box.width <- num.bars/18


        temp_file <- tempfile(fileext = ".png")
        png(temp_file, width = width, height = height, units = units, res = res)

        # Set margins where c(1=bottom, 2=left, 3=top and 4=right).
        # par(mar = c(5,7,4,2) + 0.1) ## default is c(5,4,4,2) + 0.1
        par(mar = c(8,5,4,2))
        par(cex.lab=0.9)  # is text size for y-axis
        par(cex.axis=0.9) # is text size for x-axis

        #Draw Boxplot (note gsub("(^')|('$)", "", resp$name[[i]]) removes the leading and tailing single quotes)
        boxp <- boxplot(current_formula,
                        data = data,
                        xlab = "",
                        ylab = response_name,
                        las  = las,
                        #Create nice colors
                        col=c2, medcol=c3, whiskcol=c1, staplecol=c3, boxcol=c3, outcol=c3,
                        #Set box width using relative number of boxes to keep them more or less equal
                        boxwex = box.width
                        #Let the width of the boxes depend on relative number of replicas: varwidth = TRUE
                        # varwidth = TRUE,

        )
        # Add jittered points
        if(jitter == TRUE){
          stripchart(current_formula,
                     data = data,
                     method = "jitter", # Adds random noise
                     pch = 19,          # Point style
                     vertical = TRUE,   # Align points vertically or not
                     cex = 0.7,         # Reduce point size
                     col = c1,          # Point color
                     add = TRUE)        # Overlay on the boxplot
        }

        # Calculate means for each group
        means <- tapply(data[[response_name]], interaction(data[factor_cols]), mean)

        # Add mean points to the plot
        points(seq_along(means), means, col = c3, pch = 10, cex = 1.5)


        # Allow plotting outside the plot region
        par(xpd = TRUE)

        # Get current plot limits
        usr <- par("usr")  # usr contains c(xmin, xmax, ymin, ymax)

        # Calculate a proportional position above the top of the plot
        y_position <- usr[4] + 0.05 * (usr[4] - usr[3])  # 5% above ymax

        #Get n
        x.pos.n <- rep(seq_along(boxp$n))
        text(x= x.pos.n, y = y_position, labels= paste0("n=", boxp$n))
        if(output_type != "png"){
        cat("   \n")
        }
        # Close the png file
        dev.off()

        if (output_type != "png"){
        # Include the saved plots in R Markdown
        cat(paste0("![](", temp_file, ")"), "   \n  \n")

          if(outliers == TRUE){
          suppressMessages(
            out <- f_outliers(
                data,
                columns = response_name,
                group_vars = factor_cols,
                coef = coef
              )
          )
            if (!is.null(out) &&
                is.data.frame(out) && nrow(out) > 0 && ncol(out) > 0) {
              cat("##  Outlier Table of: ",
                  response_name,
                  " as function of ",
                  RHS,
                  "  \n")
              cat(f_pander(out, limit_columns = limit_columns))
            } else {
              cat("*No outliers found in:* ",
                  response_name,
                  " as function of ",
                  RHS,
                  "  \n  \n")
            }
          }

        }

        if (output_type == "png"){

          # Define the new file name
          new_file_name <- paste0(dirname(output_path),"/",response_name,".png")

          # Rename the temporary file
          file.rename(temp_file, new_file_name)

          if (open_generated_files == TRUE){
            f_open_file(new_file_name)
          }
        }

    }
    else {
      for (factor_name in factor_vars) {

        if (factor_name != make.names(factor_name)) {
          factor_name_backt  <- paste0("`", factor_name, "`")

        } else {
          factor_name_backt   <- factor_name
          }

      # Create a new formula for each response
      current_formula <- as.formula(paste0(response_name_backt, "~", factor_name_backt))



      if(output_type != "png"){
      cat("##  Boxplot of: ", response_name, " as function of ", factor_name, "  \n")
      }
      # Get the number of bars per boxplot and store in num.bars
      num.bars <- length(levels(data[[factor_name]]))

      # Define colors for boxplot
      c1 <- rainbow(num.bars)
      c2 <- rainbow(num.bars, alpha=0.2)
      c3 <- rainbow(num.bars, v=0.7)

      # Determine width of the boxes by making it relative to max here max boxes=9
      # this times two is 18, thus 9/18 = 0.5 the lower the num.bars the lower the relative width.
      box.width <- num.bars/18



      temp_file <- tempfile(fileext = ".png")
      png(
        temp_file,
        width = width,
        height = height,
        units = units,
        res = res
      )

      # Set margins where c(1=bottom, 2=left, 3=top and 4=right).
      # par(mar = c(5,7,4,2) + 0.1) ## default is c(5,4,4,2) + 0.1
      par(mar = c(8,5,4,2))
      par(cex.lab=0.9) # is text size for y-axis
      par(cex.axis=0.9) # is text size for x-axis

      # Draw Boxplot (note gsub("(^')|('$)", "",
      # resp$name[[i]]) removes the leading and tailing single quotes)
      boxp <- boxplot(
        current_formula,
        data = data,
        xlab = "",
        ylab = response_name,
        las  = las,
        #Create nice colors
        col = c2,
        medcol = c3,
        whiskcol = c1,
        staplecol = c3,
        boxcol = c3,
        outcol = c3,
        #Set box width using relative number of boxes to keep them more or less equal
        boxwex = box.width
        #Let the width of the boxes depend on relative number of replicas: varwidth = TRUE
        # varwidth = TRUE,
      )
      # Add jittered points
      if (jitter == TRUE) {
        stripchart(
          current_formula,
          data = data,
          method = "jitter",
          # Adds random noise
          pch = 19,
          # Point style
          vertical = TRUE,
          # Align points vertically or not
          cex = 0.7,
          # Reduce point size
          col = c1,
          # Point color
          add = TRUE
        )        # Overlay on the boxplot
      }


      means <- tapply(data[[response_name]], data[[factor_name]], mean)

      # Add mean points to the plot
      points(seq_along(means), means, col = c3, pch = 10, cex = 1.5)


      # Allow plotting outside the plot region
      par(xpd = TRUE)

      # Get current plot limits
      usr <- par("usr")  # usr contains c(xmin, xmax, ymin, ymax)

      # Calculate a proportional position above the top of the plot
      y_position <- usr[4] + 0.05 * (usr[4] - usr[3])  # 5% above ymax

      #Get n
      x.pos.n <- rep(seq_along(boxp$n))
      text(x= x.pos.n, y = y_position, labels= paste0("n=", boxp$n))
      if(output_type != "png"){
        cat("   \n")
      }

      # Close the png file
      dev.off()

      if (output_type != "png"){
        # Include the saved plots in R Markdown
        cat(paste0("![](", temp_file, ")"), "   \n  \n")


        if (outliers == TRUE) {
          suppressMessages(
            out <- f_outliers(
              data,
              columns = response_name,
              group_vars = factor_name,
              coef = coef
            )
          )
          if (!is.null(out) &&
              is.data.frame(out) && nrow(out) > 0 && ncol(out) > 0) {
            cat("##  Outlier Table of: ",
                response_name,
                " as function of ",
                factor_name,
                "  \n")
            cat(f_pander(out, limit_columns = limit_columns))
          } else {
            cat("*No outliers found in:* ",
                response_name,
                " as function of ",
                factor_name,
                "  \n  \n")
          }
        }


      }

      if (output_type == "png"){
      # Define the new file name
      new_file_name <- paste0(dirname(output_path),"/",response_name,"_", factor_name, ".png")

      # Rename the temporary file
      file.rename(temp_file, new_file_name)

        if (open_generated_files == TRUE){
        f_open_file(new_file_name)
        }
      }

      } # End loop Factor variables
    } # End if statement
  } # End Loop Response variables
} # End generate_report function



# Here the documents are constructed.
if (output_type %in% c("word", "pdf")) {


  # Create a temporary R Markdown file
  word_pdf_preamble <- function(){ paste0( # Create a temporary R Markdown file
"
---
title: \"Boxplot Report\"
date: \"`r Sys.Date()`\"
output:
   word_document:
      reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
        latex_engine: pdflatex
header-includes:
  - \\usepackage[utf8]{inputenc}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
---
")}

  # Prevent ## before printed output
  knitr::opts_chunk$set(comment = "")

  # show the location were the file is saved
  message(paste0("Saving output in: ", output_path))

  # re-run generate_report, but this time capture its output to a string
  generated_markdown <- capture.output(generate_report())

  # Combine the preamble, assumptions, and the captured report into one string
  rmd_content <- paste(
    word_pdf_preamble(),
    # The captured output already contains the necessary markdown formatting and image links
    paste(generated_markdown, collapse = "\n"),
    sep = "\n"
  )

  # Write the complete Rmd content to the temp file
  writeLines(rmd_content, temp_output_file)

  # Create the RMarkdown file
  rmarkdown::render(
    temp_output_file,
    output_file = output_path,
    intermediates_dir = temp_output_dir,
    knit_root_dir = temp_output_dir,
    quiet = TRUE,
    output_format = paste0(output_type, "_document")
  )

  if(open_generated_files == TRUE){
  # Open the file with default program
    f_open_file(output_path)
  }

  # Remove the temporary R Markdown file
  invisible(suppressWarnings(file.remove(temp_output_file)))
  return(invisible(NULL))
  }
  else if (output_type == "rmd"){

    if (is.null(knitr::opts_knit$get("output.dir"))) {
      knitr::opts_knit$set(output.dir = tempdir())
    }

    # Re-capture the markdown text for the rmd output
    generated_markdown <- capture.output(generate_report())

    clean_rmd_output <- paste(generated_markdown, collapse = "\n")

    # Return the markdown as a character string (consistent with f_aov,
    # f_glm, f_summary, f_kruskal_test, etc., which all return their
    # rmd content as part of the output_list). Wrapping in invisible()
    # keeps the console quiet when the user calls f_boxplot(...) directly:
    # to display it they do `cat(result)` or `knitr::asis_output(result)`.
    return(invisible(clean_rmd_output))

  }
  else if (output_type == "png"){

    # Capture generate_report()'s cat() output so it does not leak to the
    # console. generate_report() does its real work (saving PNG files) as
    # a side effect, so we can discard the captured markdown string.
    invisible(capture.output(generate_report()))

    message(paste0("PNG files saved in: ", dirname(output_path), "\n   \n"))

    return(invisible(NULL))
  }
}

# Insert newline every X characters
insert_newline <- function(string, every_x_chars = 8, console = FALSE, keep_dots = FALSE) {
  # Pandoc-safe break: <br> works in PDF, Word, HTML
  # sep <- if (console) "  \n" else "  \\newline{}"
  # sep <- if (console) "  \n" else "  <br>"
  sep <- if (console) "  \n" else "  \n  \n"

  sapply(string, function(s) {

    parts <- unlist(strsplit(s, "\\."))

    parts <- sapply(parts, function(p) {

      # If the segment is empty (e.g. caused by double dots ".."), skip the math
      if (nchar(p) == 0) return("")

      chunks <- substring(
        p,
        seq(1, nchar(p), every_x_chars),
        pmin(seq(every_x_chars, nchar(p) + every_x_chars - 1, every_x_chars), nchar(p))
      )

      paste0(chunks, collapse = sep)
    })

    collapse_char <- if (keep_dots) paste0(".", sep) else sep

    paste(parts, collapse = collapse_char)

  }, USE.NAMES = FALSE)
}

formula_extract_df_names <- function(formula) {
  # Extract both sides of the formula
  rhs <- formula[[3]]
  lhs <- formula[[2]]

  # Function to recursively search for $ operators
  find_df_names <- function(expr) {
    if (!is.call(expr)) return(character(0))

    if (length(expr) >= 3 && expr[[1]] == "$") {
      # Found a $ operator, extract the data frame name
      return(as.character(expr[[2]]))
    } else {
      # Recursively search other parts of the expression
      unique(unlist(lapply(expr[-1], function(e) {
        if (is.call(e)) find_df_names(e) else character(0)
      })))
    }
  }

  # Search both sides of the formula
  df_names <- character(0)
  if (is.call(rhs)) df_names <- c(df_names, find_df_names(rhs))
  if (is.call(lhs)) df_names <- c(df_names, find_df_names(lhs))

  # Return unique data frame names
  unique(df_names)
}

# Generate a Valid Save File Path
#
# Constructs an absolute file path using user input with sensible defaults and robust path validation.
# It normalizes paths by replacing backslashes, handles directory inputs, verifies existence of specified directories,
# and ensures the resulting file path includes a file extension.
#
# @param save_as Optional character string specifying the desired save path or directory.
#   If a directory is specified (ends with a slash or exists as a directory), the returned path includes the default file name.
#   If a full file path is provided, it validates the directory part.
#   If a file name only is provided, the file is placed in the \code{default_dir}.
#   If \code{NULL}, returns path combining \code{default_dir} and \code{default_name}.
# @param default_name Character string specifying the default file name to use when \code{save_as} is \code{NULL} or a directory.
#   Defaults to \code{"output"}. If no extension is present, \code{file.ext} will be appended.
# @param default_dir Character string specifying the default directory to save files in when \code{save_as} is \code{NULL} or only a file name.
#   Defaults to \code{tempdir()}.
# @param file.ext Character string specifying the file extension to append if the filename does not already have one.
#   Defaults to \code{".pdf"}. Should include the leading dot (e.g., \code{".csv"}, \code{".txt"}).
#   If the filename already contains an extension, this parameter is ignored.
#
# @return A character string of a normalized save path with forward slashes and guaranteed file extension.
#
# @details
# - If \code{save_as} is \code{NULL}, the function returns the default directory combined with the default file name.
# - If \code{save_as} is a directory path (either ends with slash or exists), it returns the directory path combined with the default file name.
# - If \code{save_as} is a full file path, it verifies that the directory exists and returns a normalized full path.
# - If a file name only is given, the file will be placed in the \code{default_dir}.
# - The function checks if the final filename has an extension (pattern: one or more characters after a dot).
#   If no extension is detected, \code{file.ext} is automatically appended.
# - Extension checking works on the final filename component, not on directory names containing dots.
#
# throws Error if the specified directory in \code{save_as} does not exist.
#
# @examples
# # Default path with .pdf extension
# get_save_path()
#
# # Directory only - uses default name with extension
# get_save_path("mydata/")
#
# # Full path with extension specified - keeps original extension
# get_save_path("mydata/output.csv")
#
# # Filename without extension - adds default .pdf
# get_save_path("my_file")
#
# # Custom extension
# get_save_path("report", file.ext = ".pdf")
#
# # Filename with extension in custom directory
# get_save_path("my_file.txt", default_dir = "~/Documents")
#
get_save_path <- function(save_as = NULL,
                          default_name = "output",
                          default_dir = tempdir(),
                          file.ext = ".pdf") {

  # remove \\ and use / instead
  default_dir <- gsub("\\\\", "/", default_dir)

  # Function to ensure file extension
  ensure_extension <- function(filename, ext) {
    if (grepl("\\.[^\\.]+$", filename)) {
      # Has some extension, return as is
      filename
    } else {
      # Append given extension
      paste0(filename, ext)
    }
  }

  # 1. Default Case: If no input, use default path and name.
  if (is.null(save_as)) {
    file_with_ext <- ensure_extension(default_name, file.ext)
    return(file.path(default_dir, file_with_ext))
  }

  # Normalize input by removing trailing slashes for cleaner path resolution later
  save_as_normalized <- sub("/+$|\\\\+$", "", save_as)

  # Determine if the user intended a directory:
  # A. Input ends with a slash (e.g., "mydata/").
  # B. Input is the path to an existing directory (e.g., "mydata" where 'mydata' exists).
  is_directory_only_intent <- grepl("/$|\\\\$", save_as) || dir.exists(save_as_normalized)

  # --- Case Handling ---

  # 2. Case: Directory Only Intent (e.g., "mydata/" or "C:/reports")
  # Output: 'mydata/default_name'
  if (is_directory_only_intent) {

    # Check if the directory exists.
    if (!dir.exists(save_as_normalized)) {
      stop(paste0("Error: The specified directory does not exist: '", save_as_normalized, "'"))
    }

    # If it exists, normalize the path and apply the default name with extension.
    base_dir <- normalizePath(save_as_normalized, winslash = "/", mustWork = FALSE)
    file_with_ext <- ensure_extension(default_name, file.ext)
    return(file.path(base_dir, file_with_ext))
  }

  # 3. Case: Full Path or Filename

  # Extract the directory component
  dir_part <- dirname(save_as)

  # Check if the input is a full path (i.e., not just a filename, where dir_part == ".")
  if (dir_part != ".") {

    # A: Full Path Provided (e.g., "nonexistent/output.csv")

    # Check if the directory part of the path exists.
    if (!dir.exists(dir_part)) {

      # Stop execution with a clear error if the directory is missing.
      stop(paste0("Error: The directory part of the path does not exist: '", dir_part, "'"))
    }

    # Ensure the filename has the extension
    filename <- basename(save_as)
    file_with_ext <- ensure_extension(filename, file.ext)
    full_path <- file.path(dir_part, file_with_ext)

    # Return normalized full path
    return(normalizePath(full_path, winslash = "/", mustWork = FALSE))

  } else {
    # B: Only a Filename provided (e.g., "my_file" or "my_file.txt").
    file_with_ext <- ensure_extension(save_as, file.ext)
    # Output: 'default_dir/filename'
    return(file.path(default_dir, file_with_ext))
  }
}

#' Chi-squared Test with post hoc Analysis
#'
#' @description
#' Performs a chi-squared test  \code{\link[stats]{chisq.test}}, then automatically conducts post hoc analysis if the test is significant. The function provides adjusted p-values for each cell in the contingency table using a specified correction method.
#'
#' @param x  A numeric vector (or factor), or a contingency table in matrix or table form. If a data frame is entered the function will try to convert it to a table using \code{df_to_table()}.
#' @param y A numeric vector; ignored if x is a matrix, table or data.frame. If x is a factor, y should be a factor of the same length.
#' @param p A vector of probabilities of the same length as x. Default is \code{NULL}. An error is given if any entry of p is negative.
#' @param method Character string specifying the adjustment method for p-values. Default is \code{"bonferroni"}. Other options include \code{"holm", "hochberg", "hommel", "BH", "BY", "fdr", and "none"}.
#' @param digits Integer specifying the number of decimal places for rounding. Default is \code{3}.
#' @param alpha Numeric threshold for significance. Default is \code{0.05}.
#' @param force_posthoc Logical indicating whether to perform post hoc tests even if the chi-squared test is not significant. Default is \code{FALSE}.
#' @param ... Additional arguments passed to \code{\link[stats]{chisq.test}}.
#'
#' @return An object of class f_chisq_test containing:
#' \itemize{
#'   \item \code{chisq_test_output}: The output from \code{\link[stats]{chisq.test}}.
#'   \item \code{adjusted_p_values}: Matrix of adjusted p-values (for table/matrix input).
#'   \item \code{observed_vs_adj_p_value}: Interleaved table of observed values and adjusted p-values.
#'   \item \code{stdres_vs_adj_p_value}: Interleaved table of standardized residuals and adjusted p-values.
#'   \item \code{adj_p_values}: Vector of adjusted p-values (for vector input).
#'   \item \code{posthoc_output_table}: Data frame with observed values, expected values,
#'         standardized residuals, and adjusted p-values (for vector input).
#'   \item \code{observed_vs_adj_p_value}: Interleaved table of observed values and adjusted p-values (for table/matrix input).
#'   \item \code{stdres_vs_adj_p_value}: Interleaved table of standardized residuals and adjusted p-values (for table/matrix input).
#' }
#'
#' @details
#' The function first performs a chi-squared test using \code{\link[stats]{chisq.test}}. If the test is
#' significant (p < alpha) or if \code{force_posthoc = TRUE}, it conducts post hoc analysis by examining
#' the standardized residuals. The p-values for these residuals are adjusted using the specified method
#' to control for multiple comparisons.
#'
#' If the input is a data frame, the function attempts to convert it to a table and displays the
#' resulting table for verification.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # Chi.square on independence: Association between two variables.
#' # Create a contingency table.
#' my_table <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
#' dimnames(my_table) <- list(Gender = c("Male", "Female"),
#'                            Response = c("Agree", "Neutral", "Disagree"))
#'
#' # Perform chi-squared test with post hoc analysis.
#' f_chisq_test(my_table)
#'
#' # Use a different adjustment method.
#' f_chisq_test(my_table, method = "holm")
#'
#' # Other forms still work like Goodness-of-Fit: Match to theoretical distribution.
#' # Observed frequencies of rolling with a die 1 - 6.
#' observed <- c(2, 2, 10, 20, 15, 11)
#'
#' # Expected probabilities under a fair die.
#' expected_probs <- rep(1/6, 6)
#'
#' # Chi-Square Goodness-of-Fit Test.
#' f_chisq_test(x = observed, p = expected_probs)
#'
#' @references
#' This function implements a post hoc analysis for chi-squared tests inspired by the methodology in:
#'
#' Beasley, T. M., & Schumacker, R. E. (1995). Multiple Regression Approach to Analyzing Contingency Tables: Post Hoc and Planned Comparison Procedures. The Journal of Experimental Education, 64(1), 79-93.
#'
#' The implementation draws inspiration from the \href{https://CRAN.R-project.org/package=chisq.posthoc.test}{'chisq.posthoc.test'} package by Daniel Ebbert.
#'
#' @export


f_chisq_test <-
  function(x,
           y,
           p = NULL,
           method = "bonferroni",
           digits = 3,
           alpha = 0.05,
           force_posthoc = FALSE,
           ...) {

    # Create an output_list
    output_list <- list(p = p)

    # Store alpha for printing and reference
    output_list[["settings"]][["alpha"]]  <- alpha
    output_list[["settings"]][["force_posthoc"]] <- force_posthoc
    output_list[["settings"]][["method"]] <- method
    output_list[["settings"]][["digits"]] <- digits


    # Create a new environment to provide a name to the data.
    env <- new.env()

    # Get the name of the input object
    x_name <- deparse(substitute(x))


    if (is.data.frame(x) && is.null(p)) {
      suppressMessages(x <- df_to_table(x))
      env[[x_name]] <- x
      message(x_name, " is a data.frame, converted to table using df_to_table().\nPlease check if this table is as intended.\nIf not, consider using df_to_table(", x_name, ", label_col = ) to pick the correct column.\n")
      print(x)
      message("---")
    }

    if(!is.table(x) && !is.vector(x) && is.null(p)){
      x <- as.table(as.matrix(x))
      message(x_name, " is not a table, tried to convert data to a table.\nPlease check if this table is as intended: \n")
      print(x)
      message("---")
    }


    # Always store x in env AFTER potential conversion
    env[[x_name]] <- x


    # Do the Probabilities in 'p' sum up to 1
    if(!is.null(p) && abs(sum(p) - 1) > .Machine$double.eps^0.5){
      stop("Probabilities in 'p' must sum to 1. Got sum(p) = ", round(sum(p), 4))
    }

    # Do chisq.test()
    # Build the call, forwarding y and p correctly
    if (missing(y)) {
      if (!is.null(p)) {
        chisq.test.output <- eval(bquote(stats::chisq.test(.(as.name(x_name)), p = p, ...)), envir = env)
      } else {
        chisq.test.output <- eval(bquote(stats::chisq.test(.(as.name(x_name)), ...)), envir = env)
      }
    } else {
      env[["y"]] <- y
      chisq.test.output <- eval(bquote(stats::chisq.test(.(as.name(x_name)), y = y, ...)), envir = env)
    }

    output_list[["chisq_test_output"]] <- chisq.test.output


    if(chisq.test.output$p.value < alpha || force_posthoc == TRUE){
      #Do post hoc
      stdres <- chisq.test.output$stdres

      # Get the p values for the chi square values:
      # Testing whether standardized residuals (which are essentially z-scores),
      # are significantly different from zero.
      p_values <- 2 * pnorm(abs(stdres), lower.tail = FALSE)

      if(is.table(x) || is.matrix(x)){
        # Adjust the p values with the chosen method
        adjusted_p_values <- matrix(
          p.adjust(as.vector(p_values), method = method),
          nrow = nrow(p_values),
          ncol = ncol(p_values),
          dimnames = dimnames(p_values)
        )


        # name the row of the p value table
        rownames(adjusted_p_values) <- rep("p-value", times = nrow(adjusted_p_values))

        # Round and create tables with p values for merging procedure
        table_adj_p <- f_conditional_round(adjusted_p_values,
                                           digits = digits,
                                           allow_integer_decimal_mix = FALSE)

        # Save the adjusted p.values
        output_list[["adjusted_p_values"]] <- table_adj_p

        # Create tables with original data for merging procedure
        table_observed <- chisq.test.output$observed
        table_observed <- f_conditional_round(table_observed,
                                              digits = 0,
                                              allow_integer_decimal_mix = FALSE)

        # Create some space for readability in printed output
        table_empty <- table_observed
        table_empty[,] <- ""
        rownames(table_empty) <- rep("", times = nrow(table_empty))

        # Interleave the formatted tables for output with empty lines for readability
        n <- nrow(table_observed)
        idx <- rep(1:n, each=3) + rep(c(0, n, 2*n), times=n)
        print_obs_pvalue <- rbind(table_observed, table_adj_p, table_empty)[idx,]
        output_list[["spaced_observed_vs_adj_p_value"]] <- print_obs_pvalue

        # Interleave the formatted tables for output obs vs pvalue and store in output list
        n <- nrow(table_observed)
        idx <- rep(1:n, each=2) + rep(c(0, n), times=n)
        Posthoc_obs_pvalue <- rbind(table_observed, table_adj_p)[idx,]
        output_list[["observed_vs_adj_p_value"]] <- Posthoc_obs_pvalue

        # Interleave the formatted tables for output stdres vs pvalue
        stdres <- f_conditional_round(stdres, digits = digits, allow_integer_decimal_mix = FALSE)

        # Interleave the formatted tables for output and store in output list
        n <- nrow(table_observed)
        idx <- rep(1:n, each=2) + rep(c(0, n), times=n)
        Posthoc_stdres_pvalue <- rbind(stdres, table_adj_p)[idx,]
        # put this in output list file also the object of p values with out width.
        output_list[["stdres_vs_adj_p_value"]] <- Posthoc_stdres_pvalue


      } else if(!is.null(p)){

        # Adjust the p values with the chosen method
        adjusted_p_values <- p.adjust(p_values, method = method)

        # Round the p.values
        adjusted_p_values <- f_conditional_round(adjusted_p_values, digits = digits, allow_integer_decimal_mix = FALSE)

        # Save the adjusted p.values
        output_list[["adj_p_values"]] <- adjusted_p_values

        # Make a data.frame for output
        posthoc_out <- data.frame(
          "Observed"      = chisq.test.output$observed,
          "Expected"      = chisq.test.output$expected,
          "Std.Residuals" = stdres,
          "p-value"       = adjusted_p_values,
          check.names     = FALSE
        )


        output_list[["posthoc_output_table"]] <- posthoc_out

      }
    }

    class(output_list) <- "f_chisq_test"

    return(output_list)
  }

#' @export
print.f_chisq_test <- function(x, ...) {

  print(x$chisq_test_output)


  if(x$chisq_test_output$p.value < x$settings$alpha || x$settings$force_posthoc == TRUE){

    if(is.null(x$p)){

      cat("\nObserved data and corresponding", x$settings$method, "corrected p-values:\n   \n")
      print(x$spaced_observed_vs_adj_p_value, quote = FALSE)

    } else if(!is.null(x$p)){
      cat("\nObserved data and corresponding", x$settings$method, "corrected p-values:\n   \n")
      print(x$posthoc_output_table, row.names = FALSE, quote = FALSE)
    }

  } else {

    cat(paste0("No post hoc test was performed. \nChi-squared test p value = ", round(x$chisq_test_output$p.value, digits = x$settings$digits), " which is higher than alpha = ", x$settings$alpha, "\n(NOTE: a post hoc can be forced with option: 'force_posthoc = TRUE')"))
  }
}

# =============================================================================
# test-f_chisq_test.R
# =============================================================================
# Tests for f_chisq_test(). Uses inline contingency tables; no shared
# fixtures required.
# =============================================================================


# =============================================================================
# SECTION 1: Basic return structure
# =============================================================================

test_that("contingency table input: returns correct S3 class", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  dimnames(tbl) <- list(Gender = c("Male", "Female"),
                        Response = c("Agree", "Neutral", "Disagree"))
  result <- f_chisq_test(tbl)
  expect_s3_class(result, "f_chisq_test")
})

test_that("contingency table input: chisq_test_output is htest object", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl)
  expect_s3_class(result$chisq_test_output, "htest")
})

test_that("contingency table input: settings are stored correctly", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl, method = "holm", digits = 4, alpha = 0.01)
  expect_equal(result$settings$method, "holm")
  expect_equal(result$settings$digits, 4)
  expect_equal(result$settings$alpha, 0.01)
  expect_false(result$settings$force_posthoc)
})

# =============================================================================
# SECTION 2: Contingency table - significant result with post hoc
# =============================================================================

test_that("significant table: adjusted_p_values matrix is returned", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl)
  expect_true(!is.null(result$adjusted_p_values))
  expect_equal(dim(result$adjusted_p_values), dim(tbl))
})

test_that("significant table: observed_vs_adj_p_value has correct row count (2x nrow)", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl)
  expect_equal(nrow(result$observed_vs_adj_p_value), 2 * nrow(tbl))
})

test_that("significant table: stdres_vs_adj_p_value has correct row count (2x nrow)", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl)
  expect_equal(nrow(result$stdres_vs_adj_p_value), 2 * nrow(tbl))
})

test_that("significant table: spaced_observed_vs_adj_p_value has correct row count (3x nrow)", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl)
  expect_equal(nrow(result$spaced_observed_vs_adj_p_value), 3 * nrow(tbl))
})

test_that("significant table: column count of post hoc matrices equals ncol of input", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl)
  expect_equal(ncol(result$observed_vs_adj_p_value), ncol(tbl))
  expect_equal(ncol(result$stdres_vs_adj_p_value),   ncol(tbl))
})

# =============================================================================
# SECTION 3: Goodness-of-fit test (vector + p)
# =============================================================================

test_that("GOF: returns correct class with p argument", {
  observed      <- c(2, 2, 10, 20, 15, 11)
  expected_prob <- rep(1/6, 6)
  result <- f_chisq_test(x = observed, p = expected_prob)
  expect_s3_class(result, "f_chisq_test")
})

test_that("GOF: p is stored at top level (x$p), not in settings (BUG 1 check)", {
  observed      <- c(2, 2, 10, 20, 15, 11)
  expected_prob <- rep(1/6, 6)
  result <- f_chisq_test(x = observed, p = expected_prob)
  expect_equal(result$p, expected_prob)
})

test_that("GOF significant: posthoc_output_table is returned when significant", {
  observed      <- c(2, 2, 10, 20, 15, 11)
  expected_prob <- rep(1/6, 6)
  result <- f_chisq_test(x = observed, p = expected_prob)
  if (result$chisq_test_output$p.value < result$settings$alpha) {
    expect_true(!is.null(result$posthoc_output_table))
    expect_s3_class(result$posthoc_output_table, "data.frame")
    expect_true(all(c("Observed", "Expected", "Std.Residuals", "p-value") %in%
                      names(result$posthoc_output_table)))
  }
})

test_that("GOF: posthoc_output_table has same length as observed", {
  observed      <- c(2, 2, 10, 20, 15, 11)
  expected_prob <- rep(1/6, 6)
  result <- f_chisq_test(x = observed, p = expected_prob, force_posthoc = TRUE)
  expect_equal(nrow(result$posthoc_output_table), length(observed))
})

test_that("GOF: adj_p_values length equals length of input vector", {
  observed      <- c(2, 2, 10, 20, 15, 11)
  expected_prob <- rep(1/6, 6)
  result <- f_chisq_test(x = observed, p = expected_prob, force_posthoc = TRUE)
  expect_equal(length(result$adj_p_values), length(observed))
})

# =============================================================================
# SECTION 4: Two-vector input with y
# =============================================================================

test_that("two-vector input: y argument is not silently dropped (BUG 2 check)", {
  set.seed(42)
  x_vec <- sample(c("A", "B", "C"), 100, replace = TRUE)
  y_vec <- sample(c("X", "Y"),      100, replace = TRUE)

  result_f   <- f_chisq_test(x_vec, y = y_vec)
  result_ref <- chisq.test(x_vec, y = y_vec)

  expect_equal(result_f$chisq_test_output$statistic,
               result_ref$statistic,
               tolerance = 1e-6)
  expect_equal(result_f$chisq_test_output$parameter,
               result_ref$parameter)
})

# =============================================================================
# SECTION 5: Non-significant result - no post hoc
# =============================================================================

test_that("non-significant table: post hoc items are absent from output", {
  tbl_ns <- as.table(rbind(c(50, 50), c(50, 50)))
  result <- f_chisq_test(tbl_ns, alpha = 0.05)
  expect_true(result$chisq_test_output$p.value >= result$settings$alpha)
  expect_null(result$adjusted_p_values)
  expect_null(result$observed_vs_adj_p_value)
  expect_null(result$stdres_vs_adj_p_value)
})

test_that("non-significant: force_posthoc = TRUE overrides and returns post hoc", {
  tbl_ns <- as.table(rbind(c(50, 50), c(50, 50)))
  result  <- f_chisq_test(tbl_ns, alpha = 0.05, force_posthoc = TRUE)
  expect_true(!is.null(result$adjusted_p_values))
  expect_true(!is.null(result$observed_vs_adj_p_value))
  expect_true(result$settings$force_posthoc)
})

# =============================================================================
# SECTION 6: Correction method combinations
# =============================================================================

test_that("all p.adjust methods run without error on contingency table", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  methods <- c("bonferroni", "holm", "hochberg", "hommel", "BH", "BY", "fdr", "none")
  for (m in methods) {
    expect_no_error(f_chisq_test(tbl, method = m))
  }
})

test_that("bonferroni p-values are >= unadjusted p-values for contingency table", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result_bon  <- f_chisq_test(tbl, method = "bonferroni", force_posthoc = TRUE)
  result_none <- f_chisq_test(tbl, method = "none",       force_posthoc = TRUE)

  adj_bon  <- as.numeric(result_bon$adjusted_p_values)
  adj_none <- as.numeric(result_none$adjusted_p_values)

  expect_true(all(adj_bon >= adj_none - 1e-9))
})

test_that("correction method is preserved in output settings", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  for (m in c("bonferroni", "BH", "none")) {
    result <- f_chisq_test(tbl, method = m)
    expect_equal(result$settings$method, m)
  }
})

# =============================================================================
# SECTION 7: Data frame input conversion
# =============================================================================

test_that("data.frame input: converts to table without error", {
  df <- data.frame(Agree = c(100, 120), Neutral = c(150, 90), Disagree = c(50, 40))
  rownames(df) <- c("Male", "Female")
  expect_no_error(suppressMessages(f_chisq_test(df)))
})

test_that("data.frame input: result matches equivalent table input", {
  df  <- data.frame(Agree = c(100, 120), Neutral = c(150, 90), Disagree = c(50, 40))
  tbl <- as.table(as.matrix(df))
  result_df  <- suppressMessages(f_chisq_test(df))
  result_tbl <- f_chisq_test(tbl)
  expect_equal(result_df$chisq_test_output$statistic,
               result_tbl$chisq_test_output$statistic,
               tolerance = 1e-6)
})

# =============================================================================
# SECTION 8: Matrix input
# =============================================================================

test_that("matrix input: returns correct structure", {
  mat <- matrix(c(100, 120, 150, 90, 50, 40), nrow = 2)
  result <- f_chisq_test(mat)
  expect_s3_class(result, "f_chisq_test")
  expect_s3_class(result$chisq_test_output, "htest")
})

test_that("matrix and equivalent table give identical chi-square statistics", {
  mat <- matrix(c(100, 120, 150, 90, 50, 40), nrow = 2)
  tbl <- as.table(mat)
  result_mat <- suppressMessages(f_chisq_test(mat))
  result_tbl <- suppressMessages(f_chisq_test(tbl))
  expect_equal(result_mat$chisq_test_output$statistic,
               result_tbl$chisq_test_output$statistic,
               tolerance = 1e-6)
})

# =============================================================================
# SECTION 9: Edge cases
# =============================================================================

test_that("2x2 table: runs correctly", {
  tbl_2x2 <- as.table(matrix(c(30, 10, 20, 40), 2, 2))
  expect_no_error(f_chisq_test(tbl_2x2))
})

test_that("2x2 table: post hoc output has correct dimensions", {
  tbl_2x2 <- as.table(matrix(c(30, 10, 20, 40), 2, 2))
  result   <- f_chisq_test(tbl_2x2, force_posthoc = TRUE)
  expect_equal(dim(result$adjusted_p_values), c(2, 2))
  expect_equal(nrow(result$observed_vs_adj_p_value), 4)
})

test_that("large table (5x5): runs without error", {
  set.seed(7)
  mat    <- matrix(sample(10:200, 25), nrow = 5)
  result <- f_chisq_test(as.table(mat))
  expect_s3_class(result, "f_chisq_test")
})

test_that("single-column table raises an error or warning", {
  tbl_1col <- as.table(matrix(c(10, 20, 30), ncol = 1))
  expect_error(f_chisq_test(tbl_1col))
})

test_that("GOF: p not summing to 1 gives informative error", {
  expect_error(f_chisq_test(x = c(10, 20, 30), p = c(0.1, 0.1, 0.1)),
               regexp = "sum to 1")
})

# =============================================================================
# SECTION 10: Alpha threshold combinations
# =============================================================================

test_that("alpha = 1: post hoc always runs (everything significant)", {
  tbl_ns <- as.table(rbind(c(50, 50), c(50, 50)))
  result  <- f_chisq_test(tbl_ns, alpha = 1, force_posthoc = TRUE)
  expect_true(!is.null(result$adjusted_p_values))
})

test_that("alpha = 0: post hoc never runs (nothing ever significant)", {
  tbl <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl, alpha = 0)
  expect_null(result$adjusted_p_values)
})

test_that("force_posthoc = TRUE overrides alpha = 0", {
  tbl    <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl, alpha = 0, force_posthoc = TRUE)
  expect_true(!is.null(result$adjusted_p_values))
})

# =============================================================================
# SECTION 11: print method
# =============================================================================

test_that("print.f_chisq_test: runs without error for significant table", {
  tbl    <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
  result <- f_chisq_test(tbl)
  expect_no_error(capture.output(print(result)))
})

test_that("print.f_chisq_test: runs without error for non-significant table", {
  tbl_ns <- as.table(rbind(c(50, 50), c(50, 50)))
  result <- f_chisq_test(tbl_ns)
  output <- capture.output(print(result))
  expect_true(any(grepl("No post hoc", output)))
})

test_that("print.f_chisq_test: GOF output is actually printed", {
  observed      <- c(2, 2, 10, 20, 15, 11)
  expected_prob <- rep(1/6, 6)
  result <- f_chisq_test(x = observed, p = expected_prob, force_posthoc = TRUE)
  output <- capture.output(print(result))
  expect_true(any(grepl("Observed|p.value|p-value", output)))
})

# =============================================================================
# SECTION 12: Combined stress
# =============================================================================

test_that("combined: large table + BH + alpha=0.01 + force_posthoc", {
  set.seed(99)
  mat    <- matrix(sample(5:300, 20), nrow = 4)
  result <- f_chisq_test(as.table(mat),
                         method        = "BH",
                         alpha         = 0.01,
                         force_posthoc = TRUE,
                         digits        = 5)
  expect_s3_class(result, "f_chisq_test")
  expect_equal(result$settings$method, "BH")
  expect_equal(result$settings$digits, 5)
  expect_true(!is.null(result$adjusted_p_values))
  expect_equal(dim(result$adjusted_p_values), c(4, 5))
})

test_that("combined: GOF + holm + force_posthoc + simulate.p.value via ...", {
  observed      <- c(15, 25, 10, 30, 20)
  expected_prob <- c(0.1, 0.3, 0.1, 0.3, 0.2)
  result <- f_chisq_test(x             = observed,
                         p             = expected_prob,
                         method        = "holm",
                         force_posthoc = TRUE,
                         simulate.p.value = TRUE,
                         B             = 2000)
  expect_s3_class(result, "f_chisq_test")
  expect_true(!is.null(result$posthoc_output_table))
})

test_that("combined: 3x4 table + hommel + alpha=0.10: output dimensions correct", {
  set.seed(3)
  mat <- matrix(sample(20:100, 12), nrow = 3)
  result <- f_chisq_test(as.table(mat),
                         method        = "hommel",
                         alpha         = 0.10,
                         force_posthoc = TRUE)
  expect_equal(ncol(result$adjusted_p_values), 4)
  expect_equal(nrow(result$adjusted_p_values), 3)
  expect_equal(nrow(result$observed_vs_adj_p_value), 6)
})

#' f_bestNormalize: Automated Data Normalization with bestNormalize
#'
#' Applies optimal normalization transformations using 'bestNormalize',
#' provides diagnostic checks, and generates comprehensive reports.
#'
#' @param data Numeric vector or single-column data frame.
#' @param alpha Numeric. Significance level for normality tests (default = \code{0.05}).
#' @param data_name A character string to manually set the name of the data for plot axis and reporting. Default extracts name from input object. \code{data}.
#' @param plots Logical. If \code{TRUE}, plots Q-Q plots and Histograms of the original and transformed data. Default is \code{FALSE}.
#' @param output_type Character string specifying the output format. Default is \code{"default"}.
#'   \itemize{
#'     \item \code{"default"}: Returns the object and lets R decide whether
#'       to print; auto-prints if unassigned, silent if assigned to a variable.
#'       Use \code{print(result)} or \code{plot(result)} to display the
#'       returned object.
#'     \item \code{"console"}: Forces immediate printing to the console
#'       regardless of object assignment.
#'     \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
#'       file of the corresponding format. See \code{save_as},
#'       \code{save_in_wdir}, and \code{open_generated_files} for file
#'       path and opening behavior.
#'     \item \code{"rmd"}: Stores the raw markdown string inside the returned
#'       object for use in R Markdown documents.
#'   }
#' @param save_as Character string specifying the output file path (without extension).
#'   If a full path is provided, output is saved to that location.
#'   If only a filename is given, the file is saved in \code{tempdir()}.
#'   If only a directory is specified (providing an existing directory with trailing slash),
#'   the file is named "data_name_transformed" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
#'   Defaults to \code{file.path(tempdir(), "data_name_transformed.pdf")}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.
#' @param close_generated_files Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param ... Additional arguments passed to bestNormalize.
#'
#' @return Returns an object of class `f_bestNormalize` containing:
#' \itemize{
#'   \item \code{transformed_data} Normalized vector.
#'   \item \code{bestNormalize} Full bestNormalize object from original package.
#'   \item \code{data_name} Name of the analyzed dataset.
#'   \item \code{transformation_name} Name of selected transformation.
#'   \item \code{shapiro_original} Shapiro-Wilk test results for original data.
#'   \item \code{shapiro_transformed} Shapiro-Wilk test results for transformed data.
#'   \item \code{norm_stats} Data frame of normality statistics for all methods.
#'   \item \code{rmd} Rmd code if outputype = "rmd".
#'}
#' Also generates reports in 'Word', or 'pdf' files. When using output to console and plots = TRUE, the function prints QQ-plots, Histograms and a summary data transformation report. Includes print and plot methods for objects of class 'f_bestNormalize'.
#'
#' @details
#' This is a wrapper around the 'bestNormalize' package. Providing a fancy output and the settings  of 'bestNormalize' are tuned based on sample size n.
#' If n < 100, \code{loo = TRUE}, \code{allow_orderNorm = FALSE} and \code{r} doesn't matter as \code{loo = TRUE}.
#' If 100 <= n < 200, \code{loo = FALSE}, \code{allow_orderNorm = TRUE} and \code{r = 50}.
#' If n >= 200, \code{loo = FALSE}, \code{allow_orderNorm = TRUE}, \code{r = 10}. These setting can be overwritten by user options.
#'
#' This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
#'\itemize{
#' \item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
#' \item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
#' \item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
#'
#' \item If Pandoc is not found, this function may not work as intended.
#' }

#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @references
#' Peterson, C. (2025). \pkg{bestNormalize}: Flexibly calculate the best normalizing transformation for a vector.
#' Available at: \url{https://cran.r-project.org/package=bestNormalize}
#'
#' @examples
#' \donttest{
#' # Use set.seed to keep the outcome of bestNormalize stable.
#' set.seed(123)
#'
#' # Create some skewed data (e.g., using a log-normal distribution).
#' skewed_data <- rlnorm(100, meanlog = 0, sdlog = 1)
#'
#' # Basic usage: transform and store the full result object.
#' result <- f_bestNormalize(skewed_data, data_name = "Skewed log-normal data")
#'
#' # Print a summary of the transformation.
#' print(result)
#'
#' # Inspect normality statistics for all candidate transformations.
#' result$norm_stats
#'
#' # Plot histograms and QQ-plots for original vs. transformed data.
#' plot(result)
#'
#' # Use plots = TRUE to auto-plot when output_type = "default" (default).
#' result2 <- f_bestNormalize(skewed_data, plots = TRUE)
#'
#' # Extract only the transformed (data) vector directly.
#' transformed_data <- f_bestNormalize(skewed_data)$transformed_data
#'
#' # data.frame input: column name is used as data_name automatically.
#' df <- data.frame(measurement = skewed_data)
#' result_df <- f_bestNormalize(df)
#'
#' # Data with NAs: NAs are preserved at their original positions.
#' skewed_na <- skewed_data
#' skewed_na[c(5, 20)] <- NA
#' result_na <- f_bestNormalize(skewed_na)
#'
#' # Access a specific alternative transformation (first check what is available).
#' names(result$bestNormalize$other_transforms)
#' # Then extract the one you want, e.g.:
#' # result$bestNormalize$other_transforms$yeojohnson$x.t
#'
#' # Force output to console (prints report + plots automatically).
#' f_bestNormalize(skewed_data, output_type = "console")
#'
#' # Generate a PDF report saved to a custom path.
#' f_bestNormalize(skewed_data,
#'                 output_type          = "pdf",
#'                 save_as              = "my_report"
#'                 )
#'
#' # Generate R Markdown output for use inside a .Rmd chunk
#' # (set chunk option results = 'asis').
#' rmd_result <- f_bestNormalize(skewed_data, output_type = "rmd")
#' cat(rmd_result$rmd)
#' }
#'
#' @export
f_bestNormalize <- function(data,
                            alpha = 0.05,
                            plots = FALSE,
                            data_name = NULL,
                            output_type = "default",
                            save_as = NULL,
                            save_in_wdir = FALSE,
                            close_generated_files = FALSE,
                            open_generated_files = interactive(),
                            ...) {


  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state


  if( !(output_type %in% c("pdf", "word", "rmd", "console", "default")) ){
    stop("Character string specifying the output format (output_type = ) should be either: 'pdf', 'word', 'rmd', 'default', or 'console'")
  }

  ##############################################################################

  # Create object with transformed data as primary element
  output_list <- list()

  # Create a file_extension switch
  file_extension <- NULL

  # Capture the dots
  args <- list(...)


  # Input validation and initialization
  if (is.data.frame(data)) {
      if(is.null(data_name)){
      data_name <- colnames(data)[1]
      data_name <- sub(".*\\$", "", data_name)  # Remove everything before the "$" symbol
      y <- data[[1]]
      } else {
      #i.e. data_name = data_name
      y <- data[[1]]
      }
  } else if (is.vector(data)) {
      if(is.null(data_name)){
      data_name <- deparse(substitute(data))
      data_name <- sub(".*\\$", "", data_name)  # Remove everything before the "$" symbol
      y <- data
      } else {
      #i.e. data_name = data_name
      y <- data
      }
  } else stop("Input must be vector or data.frame")


  if (!is.numeric(y)) stop("Data must be numeric")

  #### Handle option "save_as = " ###
  if(save_in_wdir == TRUE){
    save_dir <- getwd()
  }else{
    save_dir <- tempdir()
  }

  #map the output type to extensions
  output_type_map <- c(
    "pdf"  = ".pdf",
    "word" = ".docx",
    "rmd"  = ".rmd"
  )

  # If the user specifies a path, filename or save_in_wdir == TRUE an output file should be created
  if (!is.null(save_as) || save_in_wdir == TRUE) {

    if (!is.null(save_as)) {
      #Remove backslash in save_as if needed
      save_as <- gsub(pattern = "\\\\", replacement = "/", x = save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if(file_extension_save_as[1] != FALSE){
        file_extension <- file_extension_save_as
      }
    }

    if(is.null(file_extension) && output_type %in% c("console", "default")){
      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "transformed", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = ".pdf"
      )
      #set output_type to default
      output_type <- "pdf"

    }
    else if(is.null(file_extension) && output_type %in% c("pdf", "word", "excel", "rmd")){

      #create extension based on input_type
      file.ext <- unname(output_type_map[output_type])

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "transformed", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file.ext
      )


    }
    else if(!is.null(file_extension)) {

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name, "transformed", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file_extension[1]
      )
      # reset the output type to match the user input extention in save_as
      output_type <- file_extension[2]
    }
  } else {

    #create extension based on input_type
    file.ext <- unname(output_type_map[output_type])

    # use helper get_save_path() to create output_path
    output_path <- get_save_path(save_as = save_as,
                                 default_name = paste(data_name, "transformed", sep = "_"),
                                 default_dir = save_dir,
                                 file.ext = file.ext
    )
  }


  # Prevent output to console and keep files open when output is "rmd" format
  if(output_type == "rmd"){
    close_generated_files <- FALSE
  }

  # Create clean data for stable tuning
  y_clean <- y[!is.na(y)]
  n <- length(y_clean)

  # Add the data to the arguments list
  args$x <- y_clean

  # Normality check on original data.
  # safe_shapiro() returns a shaped htest for all n regimes (real
  # result for n in [3, 5000], NA p-value with informative method
  # label otherwise), so downstream display code and stored output
  # are type-stable regardless of sample size.
  andersonD_original <- nortest::ad.test(y)
  shapiro_original <- safe_shapiro(y)


  # Tune setting of bestNormalize based on sample size. Can be overwritten by user options.

  # Tune bestNormalize settings based on valid sample size 'n'
  # Check and set conditional defaults
  if (!"loo" %in% names(args)) {
    args$loo <- (n < 500)
  }

  if (!"allow_orderNorm" %in% names(args)) {
    args$allow_orderNorm <- (n >= 100)
  }

  if (!"r" %in% names(args)) {
    # Assign a dummy value for n < 500 so 'r' always exists,
    # or 10 for larger samples.
    args$r <- ifelse(n < 500, 1, 10)
  }

  # Train bestNormalize on CLEAN data
  # This avoids crashes/instability and speeds up bestNormalize
  bn_obj <- do.call(bestNormalize::bestNormalize, args)

  # Get final vector matching original 'y' length using predict()
  # to automatically handle the NAs in 'newdata'
  transformed <- predict(bn_obj, newdata = y)

  # Patch the internal bn_obj with the full dataset
  # This ensures bn_obj$x.t matches the length of the input data 'y'
  bn_obj$x.t <- transformed
  # Patch the input data stored in the bestNormalize object for full consistency
  bn_obj$x <- y

  bn_obj$chosen_transform$x.t <- transformed
  bn_obj$chosen_transform$x   <- y

  #Do not patch the whole bn_obj as this will slow down the function.
  #leaving bn_obj$other_transforms alone, data will be without NAs


  # start code copy from bestNormalize.R
  prettynames <- c(
    "arcsinh_x" = "arcsinh(x)",
    "center_scale" = "Center+scale",
    "double_reverse_log" = "Double Reversed Log_b(x+a)",
    "boxcox" = "Box-Cox",
    "exp_x" = "Exp(x)",
    "lambert_h" = "Lambert W (h)",
    "lambert_s" = "Lambert W (s)",
    "log_x" = "Log-transform",
    "orderNorm" = "Quantile Normalization (ORQ)",
    "sqrt_x" = "sqrt(x + a)",
    "yeojohnson" = "Yeo-Johnson"
  )
  # Base R approach using match()
  normnames <- names(bn_obj$norm_stats)
  matches   <- match(normnames, names(prettynames))
  normnames <- ifelse(!is.na(matches), prettynames[matches], normnames)

  output <- data.frame(
    Transformation = normnames,
    Normality_Stat = round(bn_obj$norm_stats, 4),
    row.names = NULL
  )

  rownames(output) <- NULL
  Transf_name <- class(bn_obj$chosen_transform)[1]
  Transf_name <- f_rename_vector(Transf_name, prettynames)


  # Normality check on transformed data
  andersonD_transformed <- nortest::ad.test(transformed)
  shapiro_transformed <- safe_shapiro(transformed)






  if(output_type %in% c("word", "pdf", "rmd")){
    # png output mode
    temp_png <- tempfile(fileext = ".png")
    png(temp_png, width = 7.8, height = 7.8, units = "in", res = 600)

    par(mfrow = c(2, 2))
    f_hist(y, main = "Original Data", xlab = data_name)
    f_hist(transformed, main = "Transformed Data", xlab = paste(data_name, " Transformed"))

    f_qqnorm(y, main = "QQ Plot - Original", ylab = paste0("Quantiles of ", data_name))
    f_qqnorm(transformed,
             main = "QQ Plot - Transformed",
             ylab = paste0("Quantiles of transformed ", data_name))

    dev.off()
  }

  # Generate reports and plots
  generate_report <- function() {

    cat("\n   \n##  Data transformation of ", data_name, "using `bestNormalize`:", Transf_name,".  \n  \n")

    # Text report -- original data
    if (is.na(shapiro_original$p.value)) {
      cat("**Original Data Shapiro-Wilk Test:**&nbsp;&nbsp;&nbsp;&nbsp;",
          "skipped (", shapiro_original$method, "), see ",
          "Anderson-Darling and the plots below to assess normality.  \n",
          sep = "")
    } else {
      cat("**Original Data Shapiro-Wilk Test:**&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;")
      cat("W =", round(shapiro_original$statistic, 4))
      cat("&nbsp;&nbsp;&nbsp;&nbsp;p-value =", format.pval(shapiro_original$p.value), "\n")
    }


    # cat("**Applied Transformation:**&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;", Transf_name, "   \n   \n")

    # Text report -- transformed data
    if (is.na(shapiro_transformed$p.value)) {
      cat("\n**Transformed Data Shapiro-Wilk Test:** ",
          "skipped (", shapiro_transformed$method, "), see ",
          "Anderson-Darling and the plots below to assess normality.  \n  \n",
          sep = "")
    } else {
      cat("\n**Transformed Data Shapiro-Wilk Test:** ")
      cat("W =", round(shapiro_transformed$statistic, 4))
      cat("&nbsp;&nbsp;&nbsp;&nbsp;p-value =", format.pval(shapiro_transformed$p.value), "\n  \n")
    }
    cat("&nbsp;   \n  \n")

    cat("**Table.** All considered transformations: [Pearson P / df, lower => more normal]",
        paste0("  (n=", bn_obj$chosen_transform$n, ")\n"))
    f_pander(output)

    if(plots == TRUE){
    cat("&nbsp;\n   \n")
    cat("\nCheck the plots in the figure below to assess normality.  \n")

    cat(paste0("![](", temp_png, ")"), "   \n  \n")
    }
  } # Close generate report function


  # Generate a temporary file path for "output.Rmd"
  temp_output_dir  <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")

  # Attach metadata
  output_list[["plots"]]               <- plots
  output_list[["bestNormalize"]]       <- bn_obj
  output_list[["data_name"]]           <- data_name
  output_list[["transformation_name"]] <- Transf_name
  output_list[["original_data"]]       <- y
  output_list[["transformed_data"]]    <- transformed
  output_list[["shapiro_original"]]    <- shapiro_original
  output_list[["shapiro_transformed"]] <- shapiro_transformed
  output_list[["andersonD_original"]]    <- andersonD_original
  output_list[["andersonD_transformed"]] <- andersonD_transformed
  output_list[["norm_stats"]]          <- output
  if(output_type %in% c("word", "pdf", "rmd")){
    output_list[["normality_plots"]]  <- magick::image_scale(magick::image_read(temp_png), "600")
  }
  # Set class
  class(output_list) <- "f_bestNormalize"

  # Generate the pdf or word report, set save location and create markdown document
  if (output_type %in% c("word", "pdf")) {

    if (output_type != "rmd" && isTRUE(close_generated_files)) {
      close_app <- function(win_proc, mac_name, linux_name) {
        sysname <- Sys.info()[["sysname"]]
        if (.Platform$OS.type == "windows") {
          system(paste0("taskkill /im ", win_proc, " /f"),
                 ignore.stdout = TRUE, ignore.stderr = TRUE)
        } else if (sysname == "Darwin") {
          system(paste0("pkill -f '", mac_name, "'"),
                 ignore.stdout = TRUE, ignore.stderr = TRUE)
        } else {
          system(paste0("pkill -f ", linux_name),
                 ignore.stdout = TRUE, ignore.stderr = TRUE)
        }
      }
      if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    }


    # Show save location before knitting else it will not display in console.
    message(paste0("Saving output in: ", output_path))


          # Create a temporary R Markdown file
      word_pdf_preamble <- function(){ paste0("
---
title: \"f_bestNormalize Transformation Report\"
date: \"`r Sys.Date()`\"
output:
   word_document:
      reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
        latex_engine: pdflatex
header-includes:
  - \\usepackage[utf8]{inputenc}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
  - \\usepackage{titling}
  - \\setlength{\\droptitle}{-2.5cm} % Adjust vertical spacing
---
")}


      # Prevent ## before printed output
      knitr::opts_chunk$set(comment = "")

      # re-run generate_report, but this time capture its output to a string
      generated_markdown <- capture.output(generate_report())

      # Combine the preamble, assumptions, and the captured report into one string
      rmd_content <- paste(
        word_pdf_preamble(),
        # The captured output already contains the necessary markdown formatting and image links
        paste(generated_markdown, collapse = "\n"),
        sep = "\n"
      )

      # Write the complete Rmd content to the temp file
      writeLines(rmd_content, temp_output_file)

      # Create the RMarkdown file
      rmarkdown::render(
        temp_output_file,
        output_file = output_path,
        intermediates_dir = temp_output_dir,
        knit_root_dir = temp_output_dir,
        quiet = TRUE,
        output_format = paste0(output_type, "_document")
      )

      if(open_generated_files == TRUE){
      # Open the file with default program
      f_open_file(output_path)
      }

      return(invisible(output_list))

  } else if (output_type == "rmd"){

    if (is.null(knitr::opts_knit$get("output.dir"))) {
      knitr::opts_knit$set(output.dir = tempdir())
    }

    # Re-capture the markdown text for the rmd output
    generated_markdown <- capture.output(generate_report())

    clean_rmd_output <- paste(generated_markdown, collapse = "\n")

    output_list[["rmd"]] <- clean_rmd_output

    return(invisible(output_list))


  } else if (output_type == "console"){

    print(output_list)

    plot(output_list)

    return(invisible(output_list))

  } else if (output_type == "default"){

    if(plots == TRUE) plot(output_list)

    return(output_list)

  } else {
    warning("Invalid output format specified. No file generated.")

  }



} # Close


#' @export
print.f_bestNormalize <- function(x, ...) {

  cat("\nData transformation of", x$data_name, "using `bestNormalize`:", x$transformation_name,"\n")
  # Text report -- original
  if (is.na(x$shapiro_original$p.value)) {
    cat("Original Data Shapiro-Wilk Test: skipped (",
        x$shapiro_original$method, ")\n", sep = "")
  } else {
    cat("Original Data Shapiro-Wilk Test: ")
    cat("   W =", round(x$shapiro_original$statistic, 4),
        "  p-value =", format.pval(x$shapiro_original$p.value, digits = 4), "\n")
  }
  # Text report -- transformed
  if (is.na(x$shapiro_transformed$p.value)) {
    cat("Transformed Data Shapiro-Wilk Test: skipped (",
        x$shapiro_transformed$method, ")\n  \n", sep = "")
  } else {
    cat("Transformed Data Shapiro-Wilk Test: ")
    cat("W =", round(x$shapiro_transformed$statistic, 4),
        "  p-value =", format.pval(x$shapiro_transformed$p.value, digits = 4), "\n  \n")
  }
  cat("Below are all considered transformations: [Pearson P / df, lower => more normal]",
      paste0("  (n=", x$bestNormalize$chosen_transform$n, ")\n"))
  print(x$norm_stats, row.names = FALSE)
  cat("   \n")
  cat("\nCheck the normality plots, by using the plot() function or 'plots = TRUE' option\n")

  if(x$plots == TRUE) plot(x)

}


#' Plot method for f_bestNormalize objects
#'
#' @title Plot an f_bestNormalize object
#' @name plot.f_bestNormalize
#' @description Plots diagnostics for an object of class \code{f_bestNormalize}.
#' @param x An object of class \code{f_bestNormalize}.
#' @param which Integer determining which graph to plot. Default is \code{1:2}.
#' @param ask Logical. \code{TRUE} waits with plotting each graph until <Return> is pressed. Default is \code{FALSE}.
#' @param ... Further arguments passed to or from other methods.
#' @method plot f_bestNormalize
#'
#' @return This function is called for its side effect of generating plots
#'   and does not return a useful value. It invisibly returns `NULL`.

#' @export
plot.f_bestNormalize <- function(x, which = 1:2, ask = FALSE,...) {

  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state


  par(ask = ask)

   # Histograms
  if (1 %in% which) {
    # Set up a 2x2 plotting layout with adjusted margins mar = c(bottom, left, top, right)
    par(mfrow = c(1, 2),
        mar = c(3, 2.8, 4, 0.6), # Default is mar = c(5.1, 4.1, 4.1, 2.1)
        oma = c(0, 0, 2, 0),
        mgp = c(1.7, .5, 0)  # Default is par(mgp = c(3, 1, 0))
    )
    f_hist(x$original_data,
           main = paste0(main = "Original data: ", x$data_name),
           cex.main = 0.9, cex.lab = 0.8, cex.axis = 0.8)
    f_hist(x$transformed_data,
           main = paste0("Transformed data \nwith `bestNormalize`: ", x$transformation_name),
           cex.main = 0.9, cex.lab = 0.8, cex.axis = 0.8)
    mtext("Histograms", outer = TRUE, cex = 1.3)
    par(mfrow = c(1,1))
    layout(1)  # Clear layout matrix
  }

  if (2 %in% which) {
    # Set up a 2x2 plotting layout with adjusted margins mar = c(bottom, left, top, right)
    par(mfrow = c(1, 2),
        mar = c(3, 2.8, 4, 0.6), # Default is mar = c(5.1, 4.1, 4.1, 2.1)
        oma = c(0, 0, 2, 0),
        mgp = c(1.7, .5, 0)  # Default is par(mgp = c(3, 1, 0))
    )
    f_qqnorm(x$original_data, main = paste0("Original data: ", x$data_name),
             cex.main = 0.9, cex.lab = 0.8, cex.axis = 0.8)
    f_qqnorm(x$transformed_data,
             main = paste0("Transformed data \nwith `bestNormalize`: ", x$transformation_name),
             cex.main = 0.9, cex.lab = 0.8, cex.axis = 0.8)
    mtext("Normal Q-Q Plot with 95% Confidence Bands", outer = TRUE, cex = 1.3)
    par(mfrow = c(1,1))
    layout(1)  # Clear layout matrix
  }
}

# ==============================================================================
# Test suite for f_bestNormalize()
# Run with: testthat::test_file("test_f_bestNormalize.R")
# Or via devtools: devtools::test()
# ==============================================================================
# -- Helpers --------------------------------------------------------------------
set.seed(42)
skewed_small  <- rlnorm(50,  meanlog = 0, sdlog = 1)   # n < 100  (loo branch)
skewed_medium <- rlnorm(150, meanlog = 0, sdlog = 1)   # 100 <= n < 500
skewed_large  <- rlnorm(600, meanlog = 0, sdlog = 1)   # n >= 500 (boot branch)

skewed_with_na        <- skewed_small
skewed_with_na[c(3, 7, 15)] <- NA

skewed_df <- data.frame(my_col = skewed_small)

# ==============================================================================
# 1. INPUT VALIDATION
# ==============================================================================
test_that("non-numeric input is rejected", {
  skip_on_cran()
  expect_error(f_bestNormalize(letters), "numeric")
})

test_that("non-vector / non-data.frame input is rejected", {
  skip_on_cran()
  expect_error(f_bestNormalize(matrix(1:9, 3, 3)), "vector or data.frame")
})

test_that("invalid output_type is rejected", {
  skip_on_cran()
  expect_error(
    f_bestNormalize(skewed_small, output_type = "xlsx"),
    "output_type"
  )
})

test_that("single-column data.frame is accepted", {
  skip_on_cran()
  set.seed(1)
  result <- f_bestNormalize(skewed_df, output_type = "default")
  expect_s3_class(result, "f_bestNormalize")
})

# ==============================================================================
# 2. RETURN STRUCTURE
# ==============================================================================
test_that("output_type = 'off' returns correct class and list elements", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")

  expect_s3_class(out, "f_bestNormalize")

  expected_names <- c(
    "plots", "bestNormalize", "data_name", "transformation_name",
    "original_data", "transformed_data",
    "shapiro_original", "shapiro_transformed",
    "andersonD_original", "andersonD_transformed",
    "norm_stats"
  )
  expect_true(all(expected_names %in% names(out)))
})

test_that("transformed_data has the same length as input", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_equal(length(out$transformed_data), length(skewed_small))
})

test_that("original_data stored in output matches input", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_identical(out$original_data, skewed_small)
})

test_that("norm_stats is a data frame with expected columns", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_s3_class(out$norm_stats, "data.frame")
  expect_true("Transformation"   %in% names(out$norm_stats))
  expect_true("Normality_Stat"   %in% names(out$norm_stats))
})

test_that("bestNormalize slot contains a bestNormalize object", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_s3_class(out$bestNormalize, "bestNormalize")
})

# ==============================================================================
# 3. DATA_NAME HANDLING
# ==============================================================================
test_that("data_name is extracted from vector symbol when not supplied", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_equal(out$data_name, "skewed_small")
})

test_that("custom data_name overrides auto-detected name", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, data_name = "my_var", output_type = "default")
  expect_equal(out$data_name, "my_var")
})

test_that("data_name is extracted from data.frame column name", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_df, output_type = "default")
  expect_equal(out$data_name, "my_col")
})

test_that("data_name from $ accessor strips prefix (e.g. df$col -> col)", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_df$my_col, output_type = "default")
  # sub(".*\\$", "") should strip 'skewed_df$'
  expect_false(grepl("\\$", out$data_name))
})

# ==============================================================================
# 4. SAMPLE-SIZE BRANCHES  - critical r-variable bug check
# ==============================================================================
test_that("small n (< 100) runs without error  [catches undefined-r bug]", {
  skip_on_cran()
  set.seed(1)
  expect_no_error(
    f_bestNormalize(skewed_small, output_type = "default")
  )
})

test_that("medium n (100-499) runs without error", {
  skip_on_cran()
  set.seed(1)
  expect_no_error(
    f_bestNormalize(skewed_medium, output_type = "default")
  )
})

test_that("large n (>= 500) runs without error", {
  skip_on_cran()
  set.seed(1)
  expect_no_error(
    f_bestNormalize(skewed_large, output_type = "default")
  )
})

# ==============================================================================
# 5. NA HANDLING
# ==============================================================================
test_that("input with NAs does not crash", {
  skip_on_cran()
  set.seed(1)
  expect_no_error(
    f_bestNormalize(skewed_with_na, output_type = "default")
  )
})

test_that("NAs are preserved in transformed_data at same positions", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_with_na, output_type = "default")
  expect_equal(
    which(is.na(out$transformed_data)),
    which(is.na(skewed_with_na))
  )
})

test_that("transformed_data length matches input length when NAs present", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_with_na, output_type = "default")
  expect_equal(length(out$transformed_data), length(skewed_with_na))
})

# ==============================================================================
# 6. NORMALITY IMPROVEMENT
# ==============================================================================
test_that("transformation improves normality (p-value increases for skewed data)", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_medium, output_type = "default")
  p_orig  <- out$shapiro_original$p.value
  p_trans <- out$shapiro_transformed$p.value
  expect_gt(p_trans, p_orig)
})

test_that("shapiro_original and shapiro_transformed are htest objects", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_s3_class(out$shapiro_original,    "htest")
  expect_s3_class(out$shapiro_transformed, "htest")
})

test_that("andersonD objects are htest objects", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_s3_class(out$andersonD_original,    "htest")
  expect_s3_class(out$andersonD_transformed, "htest")
})

# ==============================================================================
# 7. OUTPUT TYPES
# ==============================================================================
test_that("output_type = 'console' returns invisible f_bestNormalize", {
  skip_on_cran()
  set.seed(1)
  out <- capture.output(
    result <- f_bestNormalize(skewed_small, output_type = "console",
                              open_generated_files = FALSE)
  )
  expect_s3_class(result, "f_bestNormalize")
})

test_that("output_type = 'rmd' adds $rmd character string to output", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "rmd",
                         open_generated_files = FALSE)
  expect_true("rmd" %in% names(out))
  expect_type(out$rmd, "character")
  expect_gt(nchar(out$rmd), 50)
})

test_that("rmd output contains transformation name", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "rmd",
                         open_generated_files = FALSE)
  expect_true(grepl(out$transformation_name, out$rmd, fixed = TRUE))
})

test_that("output_type = 'pdf' creates a file on disk", {
  skip_on_cran()
  set.seed(1)
  tmp_path <- file.path(tempdir(), "test_bn_output")
  suppressMessages(out <- f_bestNormalize(skewed_small,
                         output_type       = "pdf",
                         save_as           = tmp_path,
                         open_generated_files = FALSE))
  expect_true(file.exists(paste0(tmp_path, ".pdf")))
})

test_that("output_type = 'word' creates a .docx file on disk", {
  skip_on_cran()
  skip_if_no_pandoc()
  set.seed(1)
  tmp_path <- file.path(tempdir(), "test_bn_word")
  suppressMessages(out <- f_bestNormalize(skewed_small,
                         output_type          = "word",
                         save_as              = tmp_path,
                         open_generated_files = FALSE))
  expect_true(file.exists(paste0(tmp_path, ".docx")))
})

test_that("save_as with explicit .pdf extension overrides output_type", {
  skip_on_cran()
  skip_if_no_pandoc()
  set.seed(1)
  tmp_path <- file.path(tempdir(), "test_bn_ext.pdf")
  expect_no_error(
    suppressMessages(f_bestNormalize(skewed_small,
                    output_type          = "word",   # should be overridden
                    save_as              = tmp_path,
                    open_generated_files = FALSE))
  )
  expect_true(file.exists(tmp_path))
})

# ==============================================================================
# 8. PRINT & PLOT METHODS
# ==============================================================================
test_that("print.f_bestNormalize runs without error", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_no_error(capture.output(print(out)))
})

test_that("print output contains transformation name", {
  skip_on_cran()
  set.seed(1)
  out    <- f_bestNormalize(skewed_small, output_type = "default")
  printed <- capture.output(print(out))
  expect_true(any(grepl(out$transformation_name, printed, fixed = TRUE)))
})

test_that("plot.f_bestNormalize runs without error for both panels", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_no_error({
    pdf(NULL)          # suppress screen device
    plot(out, which = 1:2)
    dev.off()
  })
})

test_that("plot.f_bestNormalize accepts which = 1 only", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_no_error({
    pdf(NULL)
    plot(out, which = 1)
    dev.off()
  })
})

test_that("plot.f_bestNormalize accepts which = 2 only", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_no_error({
    pdf(NULL)
    plot(out, which = 2)
    dev.off()
  })
})

# ==============================================================================
# 9. GLOBAL ENVIRONMENT ISOLATION  (exists() without inherits = FALSE bug)
# ==============================================================================
test_that("global variable 'r' does not leak into tuning logic", {
  skip_on_cran()
  r <<- 999   # plant a global r
  on.exit(rm("r", envir = .GlobalEnv), add = TRUE)
  set.seed(1)
  # Should still complete without using the polluted r in an unexpected way
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_s3_class(out, "f_bestNormalize")
})

test_that("global variable 'loo' does not silently override internal logic", {
  skip_on_cran()
  loo <<- FALSE   # force loo = FALSE globally for small n
  on.exit(rm("loo", envir = .GlobalEnv), add = TRUE)
  set.seed(1)
  out <- f_bestNormalize(skewed_small, output_type = "default")
  expect_s3_class(out, "f_bestNormalize")
})

# ==============================================================================
# 10. PASSTHROUGH OF ... ARGUMENTS
# ==============================================================================
test_that("extra ... args are passed to bestNormalize without error", {
  skip_on_cran()
  set.seed(1)
  # standardize = TRUE is a valid bestNormalize arg
  expect_no_error(
    f_bestNormalize(skewed_small, output_type = "default", standardize = TRUE)
  )
})

# ==============================================================================
# 11. COMBINATION SCENARIOS
# ==============================================================================
test_that("plots = TRUE with output_type = 'off' produces plots silently", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_small, plots = TRUE, output_type = "default")
  # plots flag stored
  expect_true(out$plots)
})

test_that("data.frame input + custom name + NA data runs end-to-end", {
  skip_on_cran()
  set.seed(1)
  df_na <- data.frame(val = c(skewed_small[1:40], NA, NA))
  out   <- f_bestNormalize(df_na,
                           data_name   = "custom_name",
                           output_type = "default")
  expect_equal(out$data_name, "custom_name")
  expect_equal(length(out$transformed_data), nrow(df_na))
})

test_that("large n + rmd output returns rmd content", {
  skip_on_cran()
  set.seed(1)
  out <- f_bestNormalize(skewed_large, output_type = "rmd",
                         open_generated_files = FALSE)
  expect_type(out$rmd, "character")
})

test_that("small n + pdf output creates file", {
  skip_on_cran()
  set.seed(1)
  tmp <- file.path(tempdir(), "small_n_pdf")
  expect_no_error(
    suppressMessages(f_bestNormalize(skewed_small,
                    output_type          = "pdf",
                    save_as              = tmp,
                    open_generated_files = FALSE)
  ))
  expect_true(file.exists(paste0(tmp, ".pdf")))
})


#Gemini tests
# library(yourPackageName) # Make sure your function and helpers are loaded!

test_that("f_bestNormalize handles basic numeric vectors correctly", {
  skip_on_cran()
  set.seed(123)
  x <- rlnorm(100)

  # Test with 'off' to just get the object
  res <- f_bestNormalize(x, output_type = "default")

  # Check class and structure
  expect_s3_class(res, "f_bestNormalize")
  expect_true(is.list(res))
  expect_equal(length(res$original_data), 100)
  expect_equal(length(res$transformed_data), 100)

  # Check if Shapiro tests are included
  expect_true(!is.null(res$shapiro_original))
  expect_true(!is.null(res$shapiro_transformed))
})

test_that("f_bestNormalize handles data frames correctly", {
  skip_on_cran()
  set.seed(123)
  df <- data.frame(my_skewed_col = rlnorm(50))

  res <- f_bestNormalize(df, output_type = "default")

  # The function should extract the column name automatically
  expect_equal(res$data_name, "my_skewed_col")
})

test_that("f_bestNormalize generates Word and PDF files", {
  skip_on_cran()
  skip_if_no_pandoc()
  skip_if_not_installed("rmarkdown")
  skip_if_not_installed("knitr")

  set.seed(123)
  x <- rlnorm(50)

  # Test Word output
  word_file <- file.path(tempdir(), "test_output.docx")
  suppressMessages(res_word <- f_bestNormalize(x, output_type = "word", save_as = word_file, open_generated_files = FALSE))
  expect_true(file.exists(word_file))

  # Test RMD output
  res_rmd <- f_bestNormalize(x, output_type = "rmd")
  expect_true(!is.null(res_rmd$rmd))
  expect_true(is.character(res_rmd$rmd))
})

test_that("f_bestNormalize tunes settings based on sample size", {
  skip_on_cran()
  set.seed(123)

  # Small sample (n < 500) -> loo = TRUE
  x_small <- rlnorm(150)
  res_small <- f_bestNormalize(x_small, output_type = "default")

  # Large sample (n >= 500) -> loo = FALSE
  x_large <- rlnorm(600)
  res_large <- f_bestNormalize(x_large, output_type = "default")

  # Check if the underlying bestNormalize object reflects this
  expect_false(is.null(res_small$bestNormalize))
})



# =============================================================================
# f_bestnormalize_new_tests.R
# Additional tests for f_bestNormalize() covering the safe_shapiro
# rollout:
#
#   A. Stored Shapiro objects are always htest-classed.
#   B. n > 5000 no longer crashes.
# =============================================================================
# quiet_bn removed - use shared quiet_f_bestNormalize() from helper-quiet.R

test_that("f_bestNormalize does not crash with n > 5000", {
  skip_on_cran()
  set.seed(21)
  big <- rexp(6000, rate = 0.1)
  expect_no_error(
    res <- quiet_f_bestNormalize(big)
  )
})

test_that("Shapiro objects (if stored) are htest-classed", {
  skip_on_cran()
  set.seed(23)
  res <- quiet_f_bestNormalize(rexp(80, rate = 0.1))

  candidates <- c("shapiro_original", "Shapiro_original_data",
                  "shapiro_transformed", "Shapiro_transformed_data")
  present <- intersect(candidates, names(res))

  for (slot in present) {
    obj <- res[[slot]]
    if (inherits(obj, "htest")) {
      expect_true(all(c("statistic", "p.value", "method") %in% names(obj)))
    }
  }
})

test_that("large-n result is still a usable object, not an error", {
  skip_on_cran()
  set.seed(27)
  big <- rnorm(6500)
  res <- quiet_f_bestNormalize(big)
  expect_true(is.list(res) || inherits(res, "f_bestNormalize"))
})

#' Set Working Directory Based on Current File or Specified Path
#'
#' A wrapper around \code{setwd()} that sets the working directory to the location of the currently open file in 'RStudio' if no path is provided. If a path is specified, it sets the working directory to that path instead.
#'
#' @param path A character string specifying the desired working directory. If \code{NULL} (default), the function sets the working directory to the location of the currently open and saved file in 'RStudio'.
#'
#' @details
#' If \code{path} is not provided (\code{NULL}), this function uses the \code{this.path} package to determine the location of the currently open file and sets that as the working directory. The file must be saved for this to work properly.
#'
#' If a valid \code{path} is provided, it directly sets the working directory to that path.
#'
#' @note
#' \itemize{
#' \item The function checks whether the currently open file is saved before setting its location as the working directory.
#' \item If the function is called from an unsaved script or directly from the console, an error will be thrown.
#' }
#'
#' @return None. The function is called for its side effects of changing the working directory.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # NOTE: The use of "if(interactive())" prevents this example from running
#' # during automated CRAN checks. This is necessary because the example
#' # requires to be run from an R script. You don't need to use
#' # "if(interactive())" in your own scripts.
#'if(interactive()) {
#' # Store the current working directory, so we can reset it after the example.
#' current_wd <-  getwd()
#' print(current_wd)
#'
#' # Run this commando from a saved R script file, or R Notebook to set the working
#' # directory to scripts' file location
#' f_setwd()
#'
#' # Restore your current working directory
#' f_setwd(current_wd)
#'}
#'
#'
#' @export
f_setwd <- function(path = NULL){

  # Check if a specific path is provided
  if (is.null(path)) {
    # Getting the path of the currently running script
    script_dir <- this.path::sys.dir()
    if (!is.null(script_dir)) {
      setwd(script_dir)  # Set working directory to the script's location
      cat("Working directory has been set to:", "\n")
      cat(getwd(), "\n")
    } else {
      warning("Could not determine the script's directory. The working directory remains unchanged.")
    }
  } else {
    # Ensure that the file path is a character string before proceeding
    if (!is.character(path) || !dir.exists(path)) {
      stop("Invalid or non-existent directory path provided.")
    }
    setwd(path)
    # Check if the working dir is correct
    cat("Working directory has been set to:", "\n")
    cat(getwd(), "\n")
  }
}

# Function to rename terms in a formula
rename_formula_terms <- function(formula, name_map) {
  # Convert formula to character representation
  formula_str <- deparse(formula)

  # Replace terms in the formula string using the name map
  for (old_name in names(name_map)) {
    new_name    <- name_map[[old_name]]

    # Check if the name is not a valid syntactic name
    if (new_name != make.names(new_name)) {
      new_name_with_backticks <- paste0("`", new_name, "`")
    } else {
      new_name_with_backticks <- new_name
    }

    formula_str <- gsub(paste0("\\b", old_name, "\\b"), new_name_with_backticks, formula_str)
  }

  # Convert back to a formula
  as.formula(formula_str)

}

#' Correlation Plots with Factor Detection and Multiple Correlation Coefficients
#'
#' Creates correlation plots for numeric variables in a data frame. The upper
#' triangle displays Pearson \eqn{r}{r}, Spearman \eqn{\rho}{rho}, and Kendall
#' \eqn{\tau}{tau} simultaneously for each pair. Factor variables are automatically detected and
#' used for grouping, i.e. point colouring and shaping. Ordinal variables are supported via
#' \code{ordinal_vars}: their diagonal labels are italicised and Pearson \eqn{r}{r}
#' is greyed and bracketed for any pair that involves them.A separate legend file documents both
#' the grouping factors and the meaning of all three correlation symbols.
#'
#' @param data A \code{data.frame} containing the dataset. Must include at least
#'   two numeric variables.
#' @param detect_factors Logical. If \code{TRUE}, factor variables are
#'   automatically detected for colouring and shaping points. Default \code{TRUE}.
#' @param factor_table Logical. If \code{TRUE}, prints a detailed table of
#'   converted factors to the console. Default \code{FALSE}.
#' @param factor_select A character vector specifying the names of the columns to convert into factors. If \code{NULL}, the function automatically detects columns that should be factors based on their data type and unique value count. Default is \code{NULL}.
#' @param factor_exclude A character vector specifying the names of the columns NOT to convert into factors. If \code{NULL}, no columns are excluded. Default is \code{NULL}.
#' @param unique_num_treshold  Numeric. A threshold of the amount of unique numbers a numeric column should have to keep it numeric, i.e. omit factor conversion. Default \code{8}.
#' @param repeats_threshold  Numeric. A threshold of the minimal number of repeats a numeric column should have to convert it to a factor. Default \code{2}.
#' @param color_factor Character. Name of the factor variable used for point
#'   colours; \code{"auto"} selects automatically. Default \code{"auto"}.
#' @param shape_factor Character. Name of the factor variable used for point
#'   shapes; \code{"auto"} selects automatically. Default \code{"auto"}.
#' @param print_legend Logical. If \code{TRUE}, a separate legend file is
#'   created. Default \code{TRUE}.
#' @param fancy_names Named character vector or \code{NULL}. Maps column names
#'   to display names used in the plot and legend.
#' @param ordinal_vars Character vector or \code{NULL}. Names of variables to
#'   treat as ordinal. Ordered factors are coerced to integer ranks; other
#'   non-numeric types are coerced similarly. Their diagonal labels are
#'   italicised and Pearson \eqn{r}{r} is greyed and bracketed for any pair that
#'   involves them. Ordinal variables are included in the correlation panels but
#'   excluded from aesthetic factor detection. Default \code{NULL}.
#' @param width Numeric. Plot width in centimetres. Default 15.
#' @param height Numeric. Plot height in centimetres. Default 15.
#' @param res Numeric. Resolution in DPI. Default 600.
#' @param pointsize Numeric. Base font size. Default 8.
#' @param close_generated_files Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param output_type Character. One of \code{"pdf"}, \code{"word"},
#'   \code{"png"}, or \code{"rmd"}. Default \code{"word"}.
#' @param save_as Character or \code{NULL}. Output file path without extension.
#'   A full path, a filename, or a directory (with trailing slash) are all
#'   accepted. Providing an extension overrides \code{output_type}.
#'   Default saves to \code{tempdir()}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves to the working directory.
#'   Overridden by \code{save_as}. Default \code{FALSE}.
#'
#' @details
#' \itemize{
#'   \item \strong{Three correlations per panel:} Every upper-triangle panel
#'     shows \eqn{r}{r} (Pearson), \eqn{\rho}{rho} (Spearman), and \eqn{\tau}{tau}
#'     (Kendall) stacked vertically, so the reader can choose the most
#'     appropriate coefficient for each variable pair.
#'   \item \strong{Ordinal variables:} Specify column names with
#'     \code{ordinal_vars}. Those variables appear in italic on the diagonal.
#'     For any pair where at least one variable is ordinal, Pearson \eqn{r}{r} is
#'     shown greyed and in parentheses to signal it is technically inappropriate;
#'     Spearman and Kendall remain prominent.
#'   \item \strong{Factor detection:} Only unordered factors are used for
#'     colour/shape aesthetics. Ordered factors (\code{is.ordered()}) are
#'     treated as ordinal data, not as grouping variables.
#'   \item \strong{Legend:} The legend file documents the grouping factor levels
#'     (when present) and always includes an explanation of all three
#'     correlation symbols whenever a legend is generated.
#'   \item \strong{Constant columns:} Zero-variance columns produce \code{NA}
#'     in all correlation panels.
#' }
#'
#' This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
#'\itemize{
#' \item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
#' \item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
#' \item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
#'
#' \item If Pandoc is not found, this function may not work as intended.
#' }
#'
#' @return No value is returned to the R environment. Output files are saved
#'   and opened automatically.
#'
#' @author Sander H. van Delden \email{plantmind@proton.me}
#'
#' @examples
#' data(mtcars)
#' mtcars_sub <- subset(mtcars, select = -c(am, qsec, vs))
#' f_corplot(mtcars_sub,
#'           color_factor = "gear",
#'           shape_factor = "cyl",
#'           output_type  = "png"
#'           )
#'
#' # With ordinal variables
#' data(iris)
#' fancy_names <- c(Sepal.Length = "Sepal Length (cm)",
#'                  Sepal.Width  = "Sepal Width (cm)")
#' f_corplot(iris,
#'           fancy_names  = fancy_names,
#'           ordinal_vars = "Petal.Width",
#'           output_type  = "png",
#'           open_generated_files = FALSE)
#'
#' @export
f_corplot <- function(data,
                      detect_factors        = TRUE,
                      factor_table          = FALSE,
                      factor_exclude        = NULL,
                      factor_select         = NULL,
                      unique_num_treshold   = 8,
                      repeats_threshold     = 2,
                      color_factor          = "auto",
                      shape_factor          = "auto",
                      print_legend          = TRUE,
                      fancy_names           = NULL,
                      ordinal_vars          = NULL,
                      width                 = 15,
                      height                = 15,
                      res                   = 600,
                      pointsize             = 10,
                      close_generated_files = FALSE,
                      open_generated_files  = interactive(),
                      output_type           = "word",
                      save_as               = NULL,
                      save_in_wdir          = FALSE) {

  ##########################################################################
  # Save and restore settings on exit
  ##########################################################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state


  ##########################################################################
  # Validate output_type
  ##########################################################################
  if (!(output_type %in% c("pdf", "word", "png", "rmd")))
    stop('output_type must be one of: "pdf", "word", "png" or "rmd"')

  ##########################################################################
  # Extract data name
  ##########################################################################
  data_name      <- deparse(substitute(data))
  file_extension <- NULL   # must come before the save_as resolution block

  ##########################################################################
  # Resolve output path
  ##########################################################################
  save_dir <- if (isTRUE(save_in_wdir)) getwd() else tempdir()

  output_type_map <- c(pdf = ".pdf", word = ".docx", png = ".png")

  if (!is.null(save_as) || isTRUE(save_in_wdir)) {

    if (!is.null(save_as)) {
      save_as                <- gsub("\\\\", "/", save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if (!isFALSE(file_extension_save_as[1]))
        file_extension <- file_extension_save_as
    }

    if (is.null(file_extension) && output_type == "word") {
      output_path <- get_save_path(
        save_as      = save_as,
        default_name = paste(data_name, "CorPlot", sep = "_"),
        default_dir  = save_dir,
        file.ext     = ".docx")
      output_type <- "word"

    } else if (is.null(file_extension)) {
      file.ext    <- unname(output_type_map[output_type])
      output_path <- get_save_path(
        save_as      = save_as,
        default_name = paste(data_name, "CorPlot", sep = "_"),
        default_dir  = save_dir,
        file.ext     = file.ext)

    } else {
      output_path <- get_save_path(
        save_as      = save_as,
        default_name = paste(data_name, "CorPlot", sep = "_"),
        default_dir  = save_dir,
        file.ext     = file_extension[1])
      output_type <- file_extension[2]
    }

  } else {
    file.ext    <- unname(output_type_map[output_type])
    output_path <- get_save_path(
      save_as      = save_as,
      default_name = paste(data_name, "CorPlot", sep = "_"),
      default_dir  = save_dir,
      file.ext     = file.ext)
  }

  if (output_type == "rmd") close_generated_files <- FALSE

  # Cross-platform close_generated_files (was Windows-only taskkill)
  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
  }

  # guard dev.off() -- only close if a non-null device is active.
   if (grDevices::dev.cur() > 1L) grDevices::dev.off()

  ##########################################################################
  # Prepare temporary Rmd output file
  ##########################################################################
  temp_output_dir  <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")
  file.create(temp_output_file)

  ##########################################################################
  # Apply fancy_names (renames columns, factors, and ordinal_vars)
  ##########################################################################
  if (!is.null(fancy_names)) {
    data         <- f_rename_columns(data, fancy_names)
    color_factor <- f_rename_vector(color_factor, fancy_names)
    shape_factor <- f_rename_vector(shape_factor, fancy_names)
    if (!is.null(ordinal_vars) && ordinal_vars %in% names(fancy_names)) {
      ordinal_vars <- f_rename_vector(ordinal_vars, fancy_names)
    }
  }

  ##########################################################################
  # Coerce ordinal_vars to integer so they appear in the numeric matrix
  ##########################################################################
  if (!is.null(ordinal_vars)) {
    for (ov in ordinal_vars) {
      if (!ov %in% names(data)) {
        warning("ordinal_vars: '", ov, "' not found in data \u2014 ignored.")
        ordinal_vars <- setdiff(ordinal_vars, ov)
        next
      }
      if (is.ordered(data[[ov]])) {
        data[[ov]] <- as.integer(data[[ov]])
        message("Ordinal variable '", ov, "' coerced to integer ranks.")
      } else if (!is.numeric(data[[ov]])) {
        data[[ov]] <- as.integer(factor(data[[ov]]))
        message("Ordinal variable '", ov, "' coerced to integer.")
      }
      # Already numeric: keep as-is; annotation only
    }
    if (length(ordinal_vars) == 0L) ordinal_vars <- NULL
  }

  ##########################################################################
  # Factor detection and assignment
  ##########################################################################
  factor_count <- 0L

  if (color_factor == "auto" && shape_factor == "auto") {
    if (isTRUE(detect_factors)) {
      data <- f_factors(
        data,
        properties          = factor_table,
        exclude             = factor_exclude,
        select              = factor_select,
        unique_num_treshold = unique_num_treshold,
        repeats_threshold   = repeats_threshold
      )
    }


    # Exclude ordered factors -- those are ordinal data, not grouping aesthetics
    factor_vars  <- vapply(data,
                           function(x) is.factor(x) && !is.ordered(x),
                           logical(1))
    factor_count <- sum(factor_vars)

    if (factor_count > 2){
      warning("Data contains more than 2 factors; only the first 2 will be used: ",
              names(factor_vars[factor_vars])[1], " and ",
              names(factor_vars[factor_vars])[2],
              "\nFactors can be selected manually with ",
              "'color_factor =' and 'shape_factor ='.")
    }

    if (factor_count >= 2) {
      color_factor <- names(factor_vars[factor_vars])[1]
      shape_factor <- names(factor_vars[factor_vars])[2]
    } else if (factor_count == 1) {
      color_factor <- names(factor_vars[factor_vars])[1]
      shape_factor <- "auto"
    }

  } else {
    # Count specified factors so the legend is not
    # silently suppressed when the user sets color_factor/shape_factor by name.
    if (color_factor != "auto" && color_factor %in% names(data))
      factor_count <- factor_count + 1L
    if (shape_factor != "auto" && shape_factor %in% names(data))
      factor_count <- factor_count + 1L
  }

  ##########################################################################
  # Validate factor names and auto-convert if needed
  ##########################################################################
  if (color_factor != "auto" && !color_factor %in% names(data))
    stop("The defined color factor '", color_factor,
         "' was not found in the data.")
  if (shape_factor != "auto" && !shape_factor %in% names(data))
    stop("The defined shape factor '", shape_factor,
         "' was not found in the data.")

  if (color_factor != "auto" && !is.factor(data[[color_factor]])) {
    data[[color_factor]] <- factor(data[[color_factor]])
    message("\nVariable '", color_factor, "' was converted to a factor.")
  }
  if (shape_factor != "auto" && !is.factor(data[[shape_factor]])) {
    data[[shape_factor]] <- factor(data[[shape_factor]])
    message("Variable '", shape_factor, "' was converted to a factor.\n")
  }

  if (factor_count == 0L)
    message("No factors in data. ",
            "Factors can be added via 'color_factor =' and 'shape_factor ='.")

  ##########################################################################
  # Identify numeric columns (ordinal_vars are now integer, so included)
  ##########################################################################
  numeric_vars  <- vapply(data, is.numeric, logical(1))
  numeric_count <- sum(numeric_vars)
  if (numeric_count < 2L)
    stop("At least two numeric variables are required in data.")


  # calculate a dynamic cex for panel text
  # Scales from ~1.2 (2-3 vars, large panels) down to ~0.6 (10+ vars, small panels)
  panel_cex <- max(0.55, min(1.2, 2.8 / sqrt(numeric_count)))

  ##########################################################################
  # Colour and shape mappings
  ###########################################################################
  .make_shapes <- function(n) {
    shapes <- if (n <= 5L) {
      seq(21L, 21L + n - 1L)
    } else {
      c(21L:25L, seq(3L, 3L + n - 6L))
    }
    if (length(shapes) == 3L) shapes[shapes == 23L] <- 24L
    if (length(shapes) >  3L) shapes[c(1,2,3,4)]    <- shapes[c(1,2,4,3)]
    shapes
  }

  if (factor_count >= 2L) {
    color_levels       <- levels(data[[color_factor]])
    shape_levels       <- levels(data[[shape_factor]])
    color_levels_count <- length(color_levels)
    shape_levels_count <- length(shape_levels)

    colors                <- rainbow(color_levels_count, alpha = 0.07)
    color_mapping         <- colors[unclass(data[[color_factor]])]
    border_colors         <- rainbow(color_levels_count, v = 0.7)
    border_colors_mapping <- border_colors[unclass(data[[color_factor]])]

    shapes        <- .make_shapes(shape_levels_count)
    shape_mapping <- shapes[unclass(data[[shape_factor]])]

    legend_colors        <- rainbow(color_levels_count, alpha = 0.1)
    legend_border_colors <- rainbow(color_levels_count, v = 0.7)
    legend_              <- as.vector(outer(color_levels, shape_levels,
                                            FUN = function(c, s) paste(c, "-", s)))
    legend_title         <- paste("Levels of:", color_factor, "-", shape_factor)
    legend_col   <- rep(legend_border_colors, times = shape_levels_count)
    legend_pch   <- rep(shapes, each = color_levels_count)
    legend_pt.bg <- rep(legend_colors, times = shape_levels_count)
    legend_ncol           <- 4L

  } else if (factor_count == 1L) {

    if (shape_factor == "auto") {
      # Color factor only
      color_levels          <- levels(data[[color_factor]])
      color_levels_count    <- length(color_levels)
      colors                <- rainbow(color_levels_count, alpha = 0.07)
      color_mapping         <- colors[unclass(data[[color_factor]])]
      shape_mapping         <- 21L
      border_colors         <- rainbow(color_levels_count, v = 0.7)
      border_colors_mapping <- border_colors[unclass(data[[color_factor]])]
      legend_colors         <- rainbow(color_levels_count, alpha = 0.1)
      legend_border_colors  <- rainbow(color_levels_count, v = 0.7)
      legend_               <- as.vector(color_levels)
      legend_title          <- paste("Levels of:", color_factor)
      legend_col            <- legend_border_colors
      legend_pch            <- 21L
      legend_pt.bg          <- legend_colors
      legend_ncol           <- 4L

    } else if (color_factor == "auto") {
      # Shape factor only
      shape_levels       <- levels(data[[shape_factor]])
      shape_levels_count <- length(shape_levels)
      shapes             <- .make_shapes(shape_levels_count)
      shape_mapping      <- shapes[unclass(data[[shape_factor]])]
      color_mapping      <- "white"
      border_colors_mapping <- "black"
      legend_            <- as.vector(shape_levels)
      legend_title       <- paste("Levels of:", shape_factor)
      legend_col         <- "black"
      legend_pch         <- shapes
      legend_pt.bg       <- "white"
      legend_ncol        <- 4L
    }

  } else {
    color_mapping         <- "white"
    border_colors_mapping <- "black"
    shape_mapping         <- 16L
  }

  ##########################################################################
  # Precompute all three correlation matrices
  ##########################################################################
  data_num  <- data[, numeric_vars, drop = FALSE]
  num_names <- names(data_num)

  cor_p <- suppressWarnings(cor(data_num, method = "pearson",  use = "na.or.complete"))
  cor_s <- suppressWarnings(cor(data_num, method = "spearman", use = "na.or.complete"))
  cor_k <- suppressWarnings(cor(data_num, method = "kendall",  use = "na.or.complete"))

  cor_p[!is.finite(cor_p)] <- NA_real_
  cor_s[!is.finite(cor_s)] <- NA_real_
  cor_k[!is.finite(cor_k)] <- NA_real_

  fmt_r <- function(x)
    if (is.na(x)) "NA" else formatC(round(x, 2L), format = "f", digits = 2L)

  ##########################################################################
  # Panel functions
  ##########################################################################

  # Upper triangle: r, rho, tau stacked.
  # Uses a closure over data_num, num_names, cor_p/s/k, ordinal_vars.
  # Matching x/y back to column names by identity is reliable because
  # pairs() passes the original data columns without copying.
  panel.triple.cor <- function(x, y, ...) {
    usr <- par("usr")
    cx  <- mean(usr[1:2])
    cy  <- mean(usr[3:4])
    dy  <- diff(usr[3:4]) * 0.19   # vertical spacing between the three lines

    xi     <- which(vapply(data_num, identical, logical(1L), x))[1L]
    yi     <- which(vapply(data_num, identical, logical(1L), y))[1L]
    x_name <- if (!is.na(xi)) num_names[xi] else ""
    y_name <- if (!is.na(yi)) num_names[yi] else ""

    pair_has_ordinal <- !is.null(ordinal_vars) &&
                        (x_name %in% ordinal_vars || y_name %in% ordinal_vars)

    r_p <- if (!is.na(xi) && !is.na(yi)) cor_p[xi, yi] else NA_real_
    r_s <- if (!is.na(xi) && !is.na(yi)) cor_s[xi, yi] else NA_real_
    r_k <- if (!is.na(xi) && !is.na(yi)) cor_k[xi, yi] else NA_real_

    # Pearson: greyed and bracketed when either variable is ordinal
    p_col <- if (pair_has_ordinal) "grey60" else "black"
    p_lab <- if (pair_has_ordinal) paste0("(", fmt_r(r_p), ")") else fmt_r(r_p)

    text(cx, cy + dy, bquote(italic(r)   == .(p_lab)), cex = panel_cex, col = p_col)
    text(cx,
         cy,
         bquote(italic(rho) == .(fmt_r(r_s))),
         cex = panel_cex,
         col = "black")
    text(cx,
         cy - dy,
         bquote(italic(tau) == .(fmt_r(r_k))),
         cex = panel_cex,
         col = "black")
  }

  # Diagonal text: italic label for ordinal variables.
  # Overrides pairs()'s text.panel; receives the column label as 'txt'.
  text.panel.custom <- function(x = 0.5, y = 0.5, txt, cex, font, ...) {
    use_font <- if (!is.null(ordinal_vars) && txt %in% ordinal_vars) 3L else font
    text(x, y, txt, cex = cex, font = use_font)
  }

  ##########################################################################
  # PNG file paths
  ##########################################################################
  if (output_type != "png") {
    png_path_corplot <- file.path(temp_output_dir, "temp_CorPlot.png")
    png_path_legend  <- file.path(temp_output_dir, "temp_Legend.png")
  } else {
    png_path_corplot <- output_path
    png_path_legend  <- if (grepl("_CorPlot\\.png$", png_path_corplot)) {
      sub("_CorPlot\\.png$", "_Legend.png", png_path_corplot)
    } else {
      sub("\\.png$", "_Legend.png", png_path_corplot)
    }
    message("Saving output in: ", png_path_corplot,
            " and ", png_path_legend)
  }
  png_path_corplot <- gsub("\\\\", "/", png_path_corplot)
  png_path_legend  <- gsub("\\\\", "/", png_path_legend)

  ##########################################################################
  # Draw the correlation plot
  ##########################################################################
  png(png_path_corplot,
      width     = width,
      height    = height,
      units     = "cm",
      res       = res,
      pointsize = pointsize)

  pairs(
    data_num,
    bg          = color_mapping,
    col         = border_colors_mapping,
    pch         = shape_mapping,
    lwd         = 0.5,
    font.labels = 1L,
    cex.labels  = 1,
    cex.axis    = 0.9,
    upper.panel = panel.triple.cor,
    text.panel  = text.panel.custom
  )

  grDevices::dev.off()

  if (output_type == "png" && isTRUE(open_generated_files))
    f_open_file(png_path_corplot)

  ##########################################################################
  #  Legend logic
  #
  #  draw_group_legend : factors are present AND user wants the legend
  #  draw_legend_file  : either of the above OR ordinal_vars are specified
  #                      (the coefficient explanation block is always included
  #                      in the legend file when ordinal_vars are used)
  ##########################################################################
  draw_group_legend <- factor_count > 0L && isTRUE(print_legend)
  draw_legend_file  <- draw_group_legend || !is.null(ordinal_vars)

  if (draw_legend_file) {

    png(png_path_legend, width = 8, height = 8, units = "in", res = res)

    plot(1L, type = "n", xlab = "", ylab = "",
         xlim = c(0, 1), ylim = c(0, 1), axes = FALSE)

    y_cursor <- par("usr")[4L] - 0.05  # running y-position (top to bottom)

    # ---- Group legend (colour / shape levels) ----------------------------
    if (draw_group_legend) {
      text(-0.05, y_cursor, labels = legend_title,
           cex = 1, pos = 4L, font = 2L)
      y_cursor <- y_cursor - 0.06

      leg_params <- list(
        x          = -0.15,
        y          = y_cursor + 0.05,
        legend     = legend_,
        col        = legend_col,
        pch        = legend_pch,
        pt.bg      = legend_pt.bg,
        bty        = "n",
        cex        = 0.8,
        ncol       = legend_ncol,
        xpd        = TRUE,
        text.width = max(strwidth(legend_)) + 0.02
      )
      # Measure actual rendered height before drawing -- no more fixed guesses
      leg_dims <- do.call(legend, c(leg_params, list(plot = FALSE)))
      # Draw the real legend
      do.call(legend, leg_params)
      # Advance by the actual height plus a small breathing gap
      y_cursor <- y_cursor - leg_dims$rect$h - 0.04
    }

    # ---- Helper: draw one symbol + wrapped description -------------------------
    draw_cor_entry <- function(sym, desc, y_start,
                               sym_x   = -0.05,
                               text_x  =  0.03,
                               line_dy =  0.042,   # tighter line spacing
                               gap_dy  =  0.025,   # tighter gap between entries
                               wrap_w  =  85,
                               cex     =  0.80) {
      wrapped <- strwrap(desc, width = wrap_w)
      text(sym_x,  y_start, sym,        cex = cex, pos = 4L, xpd = TRUE)
      text(text_x, y_start, wrapped[1], cex = cex, pos = 4L, xpd = TRUE)
      if (length(wrapped) > 1L)
        for (i in seq(2L, length(wrapped)))
          text(text_x, y_start - (i - 1L) * line_dy,
               wrapped[i], cex = cex, pos = 4L, xpd = TRUE)
      y_start - length(wrapped) * line_dy - gap_dy
    }

    # ---- Helper: draw a plain wrapped text block --------------------------------
    draw_wrapped_text <- function(txt, y_start,
                                  text_x  = -0.05,
                                  line_dy =  0.042,
                                  gap_dy  =  0.025,
                                  wrap_w  =  92,
                                  cex     =  0.80,
                                  font    =  1L,
                                  col     =  "black") {
      wrapped <- strwrap(txt, width = wrap_w)
      for (i in seq_along(wrapped))
        text(text_x, y_start - (i - 1L) * line_dy,
             wrapped[i], cex = cex, pos = 4L, xpd = TRUE,
             font = font, col = col)
      y_start - length(wrapped) * line_dy - gap_dy
    }

    # ---- Correlation coefficient explanation block ------------------------------

    # Rule-of-thumb: split into two short lines manually so the break falls
    # at the "|" divider rather than mid-phrase
    y_cursor <- draw_wrapped_text(
      txt     = paste0("Rule of thumb:  ",
                       "continuous + normal \u2192 r",
                       "   |   ",
                       "ordinal or skewed \u2192 \u03c1 or \u03c4"),
      y_start = y_cursor,
      wrap_w  = 72,      # narrow enough to break at the | divider naturally
      font    = 3L,      # italic
      cex     = 0.78
    )

    # Section title
    text(-0.05, y_cursor, "Correlation coefficients:",
         cex = 0.85, pos = 4L, font = 2L, xpd = TRUE)
    y_cursor <- y_cursor - 0.055

    cor_entries <- list(
      list(sym  = expression(italic(r)),
           desc = paste0("= Pearson: measures the strength of a LINEAR relationship. ",
                         "Assumes both variables are continuous and approximately normally ",
                         "distributed. Sensitive to outliers.")),
      list(sym  = expression(italic(rho)),
           desc = paste0("= Spearman: measures the strength of a MONOTONIC relationship ",
                         "(one variable tends to increase as the other does, but not necessarily ",
                         "at a constant rate). Use instead of Pearson when data are ordinal, ",
                         "skewed, or contain outliers. Works on ranks, not raw values.")),
      list(sym  = expression(italic(tau)),
           desc = paste0("= Kendall: also rank-based like Spearman, but counts the proportion of CONCORDANT pairs (both variables increase together) minus DISCORDANT pairs (one increases, the other decreases). More robust than Spearman for small samples or many tied ranks (identical values that get the same rank). Preferred for strictly ordinal data.")
           )
    )

    for (entry in cor_entries)
      y_cursor <- draw_cor_entry(sym     = entry$sym,
                                 desc    = entry$desc,
                                 y_start = y_cursor)

    # ---- Ordinal annotation (if applicable) ------------------------------------
    if (!is.null(ordinal_vars)) {
      y_cursor <- y_cursor - 0.015   # small extra breath before this section
      y_cursor <- draw_wrapped_text(
        txt     = paste0("Italic variable names indicate ordinal variables: ",
                         paste(ordinal_vars, collapse = ", ")),
        y_start = y_cursor,
        font    = 3L
      )
      y_cursor <- draw_wrapped_text(
        txt     = paste0("(r) = Pearson is greyed and bracketed for pairs involving ",
                         "ordinal variables, as it is not appropriate for ordinal data. ",
                         "Use \u03c1 or \u03c4 instead."),
        y_start = y_cursor,
        col     = "grey50"
      )
    }

    grDevices::dev.off()

    if (output_type == "png" && isTRUE(open_generated_files))
      f_open_file(png_path_legend)
  }

  ##########################################################################
  # Generate Word / PDF via R Markdown, or emit Rmd code
  ##########################################################################
  legend_chunk <- if (draw_legend_file) {
    sprintf(paste0(
      "\n```{r, results='asis', fig.align = 'center', echo=FALSE}\n",
      "cat(paste0('![](%s)'), '   \\n  \\n')\n",
      "```\n"),
      png_path_legend)
  } else ""

  if (output_type %in% c("word", "pdf")) {

    cat(sprintf(
"---
title: \"Correlation Plots\"
date: \"`r Sys.Date()`\"
output:
   word_document:
      reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
        latex_engine: pdflatex
header-includes:
  - \\usepackage[utf8]{inputenc}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
  - \\usepackage{titling}
  - \\setlength{\\droptitle}{-2.5cm}
---
```{r, results='asis', fig.align = 'center', echo=FALSE}
cat(paste0('![](%s)'), '   \\n  \\n')
cat('   \\n    \\n&nbsp;  \\n  \\n')
```
%s",
      png_path_corplot, legend_chunk),
    file = temp_output_file)

  } else if (output_type == "rmd") {

    cat(sprintf(paste0(
      "\n```{r, results='asis', fig.align = 'center', echo=FALSE}\n",
      "cat(paste0('![](%s)'), '    \\n  \\n')\n",
      "cat('    \\n    \\n&nbsp;  \\n \\n')\n",
      "```\n%s"),
      png_path_corplot, legend_chunk))
  }

  if (output_type %in% c("pdf", "word")) {
    message("Saving output in: ", output_path)
    rmarkdown::render(
      temp_output_file,
      output_file       = output_path,
      intermediates_dir = temp_output_dir,
      knit_root_dir     = temp_output_dir,
      quiet             = TRUE,
      output_format     = paste0(output_type, "_document"))

    if (isTRUE(open_generated_files)) f_open_file(output_path)
    invisible(suppressWarnings(file.remove(temp_output_file)))
    invisible(suppressWarnings(file.remove(png_path_corplot)))
    invisible(suppressWarnings(file.remove(png_path_legend)))
  }
}

#' Normal Q-Q Plot with Confidence Bands
#'
#' This function creates a normal Q-Q plot for a given numeric vector and adds confidence bands to visualize the variability of the quantiles.
#'
#' @param x A numeric vector of data values.
#' @param main A character string specifying the title of the histogram. Default is "Histogram with Normal Curve".
#' @param conf_level Numeric, between 0 and 1. Confidence level for the confidence bands. Default is 0.95 (95\% confidence).
#' @param cex Numeric, optional parameter for graph cex with default \code{cex = 0.6}.
#' @param pch Numeric, optional parameter shape of points default \code{pch = 19}.
#' @param col Numeric, optional parameter for color of point with default 'black'.
#' @param save_png A logical value default \code{FALSE}, if \code{TRUE} a png file is saved under the name of the data of under the specified file name.
#' @param open_png Logical. If \code{TRUE}, opens generated png files.
#' @param save_as Character string specifying the output file path (without extension).
#'   If a full path is provided, output is saved to that location.
#'   If only a filename is given, the file is saved in \code{tempdir()}.
#'   If only a directory is specified (providing an existing directory with trailing slash),
#'   the file is named "data_name_QQplot.png" in that directory.
#'   Defaults to \code{file.path(tempdir(), "data_name_histogram.png")}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.
#' @param width Numeric, png figure width default \code{8} inch.
#' @param height Numeric, png figure height default \code{7} inch.
#' @param units Numeric, png figure units default inch.
#' @param res Numeric, png figure resolution default \code{300} dpi.
#' @param ylab A character string specifying the y-axsis label. Default name is  \code{"Quantiles of: data_name"}.
#' @param ... Additional graphical parameters to be passed to the \code{qqnorm} function.
#'
#' @details
#' The function calculates theoretical quantiles for a normal distribution and compares them with the sample quantiles of the input data.
#'
#' It also computes confidence intervals for the order statistics using the Blom approximation and displays these intervals as shaded bands on the plot.
#'
#' The reference line is fitted based on the first and third quartiles of both the sample data and theoretical quantiles.
#'
#' To increase resolution you can use \code{png(...,res = 600)} or the 'RStudio' chunck setting, e.g. \code{dpi = 600}.
#'
#' @return A Q-Q plot is created and the function returns this as a \code{recordedplot}.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # Generate random normal data
#' set.seed(123)
#' data <- rnorm(100)
#'
#' # Create a Q-Q plot with confidence bands
#' f_qqnorm(data)
#'
#' # Customize the plot with additional graphical parameters
#' f_qqnorm(data, conf_level = 0.99, pch = 16, col = "blue")
#'
#' @export

f_qqnorm <- function(x,
                    main = NULL,
                    ylab = NULL,
                    conf_level = 0.95,
                    col = NULL,
                    pch = NULL,
                    cex = NULL,
                    save_png = FALSE,
                    open_png = TRUE,
                    save_as = NULL,
                    save_in_wdir = FALSE,
                    width = 8,
                    height = 7,
                    units = "in",
                    res = 300,
                    ...
                    ){

  # Capture the name of the input variable
  data_name <- deparse(substitute(x))
  data_name <- sub(".*\\$", "", data_name)  # Remove everything before the "$" symbol

  # Default values if not supplied by user
  if (is.null(col)) col <- "black"
  if (is.null(cex)) cex <- 0.6
  if (is.null(pch)) pch <- 19
  if (is.null(main)) main <- paste0("Normal Q-Q Plot with ", conf_level*100, "% Confidence Bands")
  if (is.null(ylab)) ylab <- paste0("Quantiles of: ", data_name)  # Set y-axis label if not provided

  # Calculate theoretical quantiles
  n <- sum(!is.na(x))
  p <- ppoints(n)
  theoretical_q <- qnorm(p)

  # Parameters for the reference line
  q1_x <- quantile(x, 0.25, na.rm = TRUE)
  q3_x <- quantile(x, 0.75, na.rm = TRUE)
  q1_t <- qnorm(0.25)
  q3_t <- qnorm(0.75)
  slope <- (q3_x - q1_x) / (q3_t - q1_t)
  intercept <- q1_x - slope * q1_t


  # Calculate standard errors for order statistics
  # Using Blom approximation
  alpha <- 0.375 # Blom constant
  p_i <- (1:n - alpha) / (n - 2 * alpha + 1)
  phi_i <- dnorm(qnorm(p_i))
  se <- slope * sqrt(p_i * (1 - p_i) / (n * (phi_i^2)))

  # Calculate confidence intervals
  z <- qnorm(1 - (1 - conf_level) / 2)
  fitted <- slope * theoretical_q + intercept
  ci_lower <- fitted - z * se
  ci_upper <- fitted + z * se

  # Create the plot
  qqnorm(
    x,
    ylab = ylab,
    main = main,
    col = col,
    pch = pch,
    cex = cex,
    ...
  )

  # Add shaded confidence bands using polygon
  polygon(c(theoretical_q, rev(theoretical_q)),
          c(ci_lower, rev(ci_upper)),
          col = rgb(0.5, 0.5, 0.5, 0.3),  # Gray with 30% opacity
          border = "darkgray")  # darkgray border for the polygon

  # Add Q-Q line and points on top of shading
  qqline(x, col = "red", lwd = 2)
  points(theoretical_q, sort(x), col = col, pch = pch, cex = cex,...)
  saved_plot <- recordPlot()

  if (save_png == TRUE || !is.null(save_as) || save_in_wdir == TRUE){

    #### Handle option "save_as = " ###
    if(save_in_wdir == TRUE){
      save_dir <- getwd()
    }else{
      save_dir <- tempdir()
    }

    output_path <- get_save_path(save_as = save_as,
                                 default_name = paste(data_name, "QQplot.png", sep = "_"),
                                 default_dir = save_dir,
                                 file.ext = ".png"
    )



    png(
      paste0(output_path),
      width = width,
      height = height,
      units = units,
      res = res
    )

    replayPlot(saved_plot)

    # Close the png file
    dev.off()

    if (open_png == TRUE){
      f_open_file(output_path)
    }
  }



  return(invisible(saved_plot))
}

#' Fancy Pander Table Output
#'
#' Is a wrapper around the \code{pander} function from the 'pander' package, designed to produce a fancy table output with specific formatting options.
#'
#' @param table A data frame, matrix, or other table-like structure to be rendered.
#' @param col_width Integer. Specifies the maximum number of characters allowed in table header columns before a line break is inserted. Defaults to \code{10}. Note that latex will not break the table header if the col with is longer due to long names below the header.
#' @param table_width Integer or \code{NULL}. Defines the number of characters after which the table is split into separate sections. Defaults to \code{NULL}, meaning no break is applied.
#' @param limit_columns Integer or \code{NULL}. Defines the number of columns shown in a table.
#' @param style Character. Pander table style. Defaults to \code{"multiline"}.
#' @param console Logical. Whether to process headers for console output. Defaults to \code{TRUE}.
#' @param ... Additional arguments passed to the \code{pander} function.
#'
#' @details
#' This function sets several \code{pander} options to ensure that the table output is formatted in a visually appealing manner. The options set include:
#' \itemize{
#'   \item \code{table.alignment.default}: Aligns all columns to the left.
#'   \item \code{table.alignment.rownames}: Aligns row names to the left.
#'   \item \code{keep.trailing.zeros}: Keeps trailing zeros in numeric values.
#'   \item \code{knitr.auto.asis}: Ensures output is not automatically treated as 'asis'.
#'   \item \code{table.caption.prefix}: Removes the default "Table" prefix in captions.
#'   \item \code{keep.line.breaks}: Preserves line breaks in cell content.
#'   \item \code{table.split.table}: Controls table splitting (set to \code{Inf} if \code{table_width} is \code{NULL} or \code{FALSE}).
#'   \item \code{table.split.cells}: Inserts line breaks in headers every \code{col_width} characters.
#' }
#'
#' This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
#' \itemize{
#' \item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
#' \item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
#' \item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
#' \item If Pandoc is not found, this function may not work as intended.
#' }
#'
#' @return None. The function is called for its side effects of setting 'pander' options and creates a pander formatted table in R Markdown.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # Example usage of f_pander
#' df <- data.frame(
#'   Name = c("Alice", "Bob", "Charlie"),
#'   Age = c(25, 30, 35),
#'   Score = c(88.5, 92.3, 85.0)
#' )
#'
#' # Render the data frame as a fancy table
#' f_pander(df)
#' @export
f_pander <- function(table,
                     col_width = 10,
                     table_width = NULL,
                     limit_columns = NULL,
                     style = "multiline",
                     console = TRUE,
                     ...) {
  # 1. SAVE EXISTING OPTIONS
  old_options <- panderOptions()

  # 2. RESTORE OPTIONS
  # We must loop through the list and restore them one by one.
  on.exit({
    for (o in names(old_options)) {
      panderOptions(o, old_options[[o]])
    }
  })

  #Set Pander options for a fancy output tabel
  panderOptions('table.alignment.default', 'left')
  panderOptions('table.alignment.rownames', 'left')
  panderOptions('keep.trailing.zeros', TRUE)
  panderOptions('knitr.auto.asis', FALSE)
  panderOptions('table.caption.prefix', '')  # Remove prefix Table in caption
  panderOptions('keep.line.breaks', TRUE)

  if(is.null(table_width) || table_width == FALSE){
    panderOptions("table.split.table", Inf)
  } else {
    panderOptions("table.split.table", table_width)
  }

  if(!is.null(col_width)){
    # Insert newline every X characters
    panderOptions('table.split.cells', col_width)
    names(table) <- insert_newline(names(table), col_width, console = console)

  }

  if(!is.null(limit_columns)){
    if(limit_columns < ncol(table) ){
    table <- table[, 1:limit_columns, drop = FALSE]
    }
  }

  pander(table, style = style, ...)
}

#' Perform multiple Wilcoxon rank sum and signed rank tests with inspection and visualization.
#'
#' Performs One-sample (Wilcoxon signed rank), Two-sample independent (Wilcoxon rank sum /
#' Mann-Whitney U), or Paired (Wilcoxon signed rank) tests on a given dataset.
#' Several response parameters can be analysed in sequence (formula interface).
#' Additionally, a vector interface similar to stats::wilcox.test() is supported.
#'
#' @section Median vs Pseudo-median:
#' By default this function calls \code{stats::wilcox.test(conf.int = TRUE)},
#' which bases its confidence interval and hypothesis test on the
#' \strong{Hodges-Lehmann estimator}, not the raw sample median. This is
#' standard behaviour of the Wilcoxon test, not something specific to this
#' function. This function explicitly labels the estimator for what it is,
#' because it is commonly mislabelled as "CI for the median" in textbooks
#' and software output.
#'
#' The estimator works differently depending on the test type:
#'
#' \strong{One-sample:} The \emph{pseudo-median} is the middle value of all
#' possible pairwise averages of your data points (including each value
#' paired with itself). For a perfectly symmetric distribution it equals the sample
#' median; for skewed data the two can differ.
#'
#' \strong{Paired:} The paired differences (observation 1 minus observation 2
#' within each pair) are computed first, and the pseudo-median of those
#' differences is estimated. This is conceptually a one-sample problem
#' applied to the differences, not a comparison of two independent groups.
#' The CI is for the pseudo-median of the differences, \strong{not} for the difference
#' between the two separate sample medians.
#'
#' \strong{Two-sample independent:} The \emph{location shift} is the median
#' of all \eqn{n_1 \times n_2} pairwise differences (one value from Group 1
#' minus one from Group 2). It answers: \emph{by how much does a randomly
#' chosen value from Group 1 tend to exceed a randomly chosen value from
#' Group 2?} When both groups have the same distributional shape it equals
#' the raw difference in sample medians; when shapes differ, the two values can
#' diverge.
#'
#' In all three cases the sample median(s) are reported separately for
#' descriptive purposes only.
#'
#' @param x Numeric vector of data values (one-sample or first group for two-sample),
#'   or a formula of the form \code{response ~ group} or \code{response ~ 1}.
#' @param y Optional numeric vector (second group) for two-sample tests if using
#'   the vector interface. Ignored when a formula is supplied.
#' @param formula A formula specifying the model (alternative to using x/y).
#'   \itemize{
#'     \item \strong{Two-sample (Independent/Paired):} \code{response ~ group}
#'           (where group has exactly 2 levels).
#'     \item \strong{One-sample:} \code{response ~ 1} or \code{response ~ NULL}.
#'   }
#'   More response variables can be added using \code{+} (e.g., \code{y1 + y2 ~ group}).
#' @param data A data frame containing the variables when using the formula interface.
#' @param paired Logical. If \code{TRUE}, performs a paired Wilcoxon test.
#'   \strong{Note on row order:} For the formula interface, data must be
#'   sorted so that the observations in the two groups match row-for-row
#'   (i.e. row 1 of group 1 is paired with row 1 of group 2).
#'   For the vector interface, \code{x} and \code{y} must have the same length.
#'
#'   \strong{Note on factor level order:} The formula interface computes
#'   differences as \code{level1 - level2} based on factor level order,
#'   which defaults to alphabetical. To control the direction, set the
#'   reference level explicitly:
#'   \preformatted{
#'   # Set levels at creation
#'   group <- factor(group, levels = c("pre", "post"))
#'
#'   # Or relevel an existing factor
#'   group <- relevel(group, ref = "pre")
#'   }
#'   A reversed level order flips the sign of the estimate and CI but
#'   does not affect the W statistic or p-value.
#' @param mu Numeric. The hypothesised value of the pseudo-median (one-sample) or
#'   location shift (paired/two-sample) under H0. Default is 0.
#' @param alternative Character string. \code{"two.sided"} (default),
#'   \code{"greater"}, or \code{"less"}.
#' @param norm_plots Logical. If \code{TRUE}, descriptive diagnostic plots are
#'   included in the output. Default is \code{TRUE}.
#' @param alpha Numeric. Significance level. Default is \code{0.05}.
#' @param conf.level Numeric. Confidence level of the interval.
#'   Default is \code{1 - alpha}. If \code{conf.level} is specified, then
#'   \code{alpha <- 1 - conf.level}.
#' @param intro_text Logical. If \code{TRUE}, includes explanation about Wilcoxon
#'   test assumptions.
#' @param close_generated_files Logical. Closes open Excel/Word files before writing.
#'   Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice).
#'   Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param output_type Character string specifying the output format. Default is \code{"default"}.
#'   \itemize{
#'     \item \code{"default"}: Returns the object and lets R decide whether
#'       to print; auto-prints if unassigned, silent if assigned to a variable.
#'       Use \code{print(result)} or \code{plot(result)} to display the
#'       returned object.
#'     \item \code{"console"}: Forces immediate printing to the console
#'       regardless of object assignment.
#'     \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
#'       file of the corresponding format. See \code{save_as},
#'       \code{save_in_wdir}, and \code{open_generated_files} for file
#'       path and opening behavior.
#'     \item \code{"rmd"}: Stores the raw markdown string inside the returned
#'       object for use in R Markdown documents.
#'   }
#' @param save_as Character. Specific path/filename for output.
#' @param save_in_wdir Logical. Save in working directory.
#' @param ... For the formula method: additional arguments forwarded to
#'   the row-filtering step. The arguments \code{subset} and
#'   \code{na.action} are honored: when supplied, they are spliced
#'   (still unevaluated) into a \code{\link[stats]{model.frame}} call
#'   built once before the per-response loop, so the \code{subset}
#'   expression is evaluated in the data's column scope (e.g.
#'   \code{subset = cyl == 6} works). All responses in a multi-response
#'   call (\code{y1 + y2 ~ group}) are then tested on the identical row
#'   set. For the default (vector) method, \code{...} is currently
#'   unused.
#'
#' @return An object of class \code{'f_wilcox_test'}.
#'
#' @export
f_wilcox_test <- function(x, ...) {
  UseMethod("f_wilcox_test")
}

# ============================================================
#  Formula method
# ============================================================
#' @export
#' @rdname f_wilcox_test
f_wilcox_test.formula <- function(formula,
                                  data                 = NULL,
                                  paired               = FALSE,
                                  conf.level           = NULL,
                                  mu                   = 0,
                                  alternative          = "two.sided",
                                  norm_plots           = TRUE,
                                  alpha                = 0.05,
                                  intro_text           = TRUE,
                                  close_generated_files  = FALSE,
                                  open_generated_files   = TRUE,
                                  output_type          = "default",
                                  save_as              = NULL,
                                  save_in_wdir         = FALSE,
                                  ...) {

  # ------------------------------------------------------------------
  # 1.  Reset initial settings on exit
  # ------------------------------------------------------------------

  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state

  # ------------------------------------------------------------------
  # 2.  Data & formula parsing
  # ------------------------------------------------------------------
  if (!is.null(data)) {
    data_name <- deparse(substitute(data))
  } else {
    if (length(formula_extract_df_names(formula)) > 0) {
      data_name <- paste(formula_extract_df_names(formula), collapse = "_")
    } else {
      data_name <- "data"
    }
    data    <- formula_to_dataframe(formula)
    formula <- clean_formula(formula)
  }

  dots <- list(...)
  if (".original_name" %in% names(dots)) {
    data_name <- dots[[".original_name"]]
  }

  temp_output_dir  <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")
  file.create(temp_output_file)
  f_wrap_lines()

  output_list <- list()

  if (!(output_type %in% c("pdf", "word", "excel", "rmd", "console", "default"))) {
    stop("output_type should be: 'pdf', 'word', 'excel', 'console', 'rmd', 'default'")
  }

  # ------------------------------------------------------------------
  # 3.  File-path logic
  # ------------------------------------------------------------------

  file_extension <- NULL

  if (save_in_wdir) save_dir <- getwd() else save_dir <- tempdir()

  output_type_map <- c(
    "pdf"   = ".pdf",
    "word"  = ".docx",
    "excel" = ".xlsx",
    "rmd"   = ".rmd"
  )

  needs_output_file <- !(output_type %in% c("default", "console")) ||
    !is.null(save_as) || isTRUE(save_in_wdir)

  if (needs_output_file) {
    if (!is.null(save_as) || isTRUE(save_in_wdir)) {
      if (!is.null(save_as)) {
        save_as <- gsub("\\\\", "/", save_as)
        file_extension_save_as <- unname(extract_extension(save_as))
        if (!isFALSE(file_extension_save_as[1]))
          file_extension <- file_extension_save_as
      }
      if (is.null(file_extension) && output_type %in% c("console", "default")) {
        output_path <- get_save_path(
          save_as      = save_as,
          default_name = paste(data_name, "wilcox_output", sep = "_"),
          default_dir  = save_dir,
          file.ext     = ".pdf"
        )
        output_type <- "pdf"
      } else if (is.null(file_extension)) {
        file.ext <- unname(output_type_map[output_type])
        output_path <- get_save_path(
          save_as      = save_as,
          default_name = paste(data_name, "wilcox_output", sep = "_"),
          default_dir  = save_dir,
          file.ext     = file.ext
        )
      } else {
        output_path <- get_save_path(
          save_as      = save_as,
          default_name = paste(data_name, "wilcox_output", sep = "_"),
          default_dir  = save_dir,
          file.ext     = file_extension[1]
        )
        output_type <- file_extension[2]
      }
    } else {
      file.ext <- unname(output_type_map[output_type])
      output_path <- get_save_path(
        save_as      = save_as,
        default_name = paste(data_name, "wilcox_output", sep = "_"),
        default_dir  = save_dir,
        file.ext     = file.ext
      )
    }
  } else {
    output_path <- NULL
  }

  if (output_type == "rmd") close_generated_files <- FALSE

  # Cross-platform close_generated_files (was Windows-only taskkill)
  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    if (output_type == "excel") close_app("EXCEL.EXE",   "Microsoft Excel", "soffice")
  }

  # ------------------------------------------------------------------
  # 4.  Parse LHS and RHS of formula
  # ------------------------------------------------------------------
  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(formula) #use helper_check_lhs.R

  lhs <- all.vars(formula[[2]])

  if (length(lhs) < 1)
    stop("Formula must specify at least one response variable on the LHS.")

  if (length(formula) == 3) {
    rhs_vars   <- all.vars(formula[[3]])
    rhs_string <- deparse(formula[[3]])
    if (length(rhs_vars) == 0 || rhs_string %in% c("1", ".")) {
      is_one_sample  <- TRUE
      predictor_name <- NULL
    } else {
      is_one_sample  <- FALSE
      predictor_name <- rhs_vars[1]
      if (length(rhs_vars) > 1)
        warning("f_wilcox_test only supports one grouping variable. Using the first one.")
    }
  } else {
    is_one_sample  <- TRUE
    predictor_name <- NULL
  }

  # Validation
  for (response in lhs) {
    if (!(response %in% names(data)))
      stop(paste("Response variable", response, "not found in data."))
    if (!is.numeric(data[[response]]))
      stop(paste("Response variable", response, "must be numeric."))
  }
  if (!is_one_sample) {
    if (!(predictor_name %in% names(data)))
      stop(paste("Predictor", predictor_name, "not found in data."))
    data[[predictor_name]] <- as.factor(data[[predictor_name]])
    if (nlevels(data[[predictor_name]]) != 2)
      stop("Grouping variable must have exactly 2 levels for a Wilcoxon test.")
  }

  ########## Build master analysis frame (one row set, all responses) #######
  # Hoists row filtering out of the per-response loop so every response
  # in `y1 + y2 ~ group` is tested on the identical row set, and bakes
  # in any subset / na.action passed via `...`.
  #
  # Strategy: pre-evaluate subset eagerly against data first, falling
  # back to the user's calling environment. Then apply the filter
  # manually and pass plain values to stats::model.frame via do.call.
  # This avoids the fragile match.call/substitute dance where spliced
  # expressions inside a constructed call can end up containing `..N`
  # dots-index symbols that crash model.frame with "the ... list
  # contains fewer than 3 elements".
  master_vars <- if (is_one_sample) lhs else c(lhs, predictor_name)
  master_formula <- as.formula(
    paste("~", paste(master_vars, collapse = " + "))
  )

  mc          <- match.call(expand.dots = FALSE)
  dots_exprs  <- as.list(mc[["..."]])
  caller_env  <- parent.frame()

  # Resolve subset expression against data columns + caller env
  subset_vec <- NULL
  if (!is.null(dots_exprs$subset)) {
    subset_vec <- eval(dots_exprs$subset, envir = data, enclos = caller_env)
    if (is.logical(subset_vec)) subset_vec[is.na(subset_vec)] <- FALSE
  }
  n_before_master <- nrow(data)
  if (!is.null(subset_vec)) {
    data <- data[subset_vec, , drop = FALSE]
  }

  # Resolve na.action (a function, defaults to na.omit)
  na_action_fn <- stats::na.omit
  if (!is.null(dots_exprs$na.action)) {
    na_action_fn <- eval(dots_exprs$na.action, envir = caller_env)
  }

  mf_args <- list(
    formula            = master_formula,
    data               = data,
    drop.unused.levels = TRUE,
    na.action          = na_action_fn
  )

  data <- do.call(stats::model.frame, mf_args)
  n_after_master  <- nrow(data)
  ###########################################################################

  # Reconcile alpha / conf.level
  if (is.null(conf.level)) {
    conf.level <- 1 - alpha
  } else {
    alpha <- 1 - conf.level
  }

  # pre-compute rounded percentage label to avoid floating-point printing
  #         (e.g. 1 - 0.05 can print as 94.99999... without rounding)
  conf_pct <- round(conf.level * 100, 1)

  # ------------------------------------------------------------------
  # 5.  Inner report-generation function
  # ------------------------------------------------------------------
  generate_report <- function(output = TRUE) {

    if (isTRUE(intro_text)) {
      cat("
# Assumptions of Wilcoxon Rank-Sum / Signed-Rank Tests

## 1. Independence
- Observations must not influence each other. This is determined by study
  design, not statistics.
  + *Two-sample (Wilcoxon Rank-Sum):* Subjects should be distinct both
    *within* groups and *between* groups.
  + *Paired (Wilcoxon Signed-Rank):* Pairs are distinct, though observations
    within a pair are correlated.
- **Violation invalidates** the test; the solution is a change in experimental
  design.

## 2. Scale of Measurement
- Data must be at least Ordinal (test is based on ranks).
- Suitable for continuous (ratio) data, counts, or ordinal scales (Likert).
- **Normality is NOT required:** The test is robust to outliers and non-normal
  distributions.

## 3. Data Shape & Distribution
- **For Paired (Signed-Rank):** The *differences* between the pairs should be
  roughly symmetric around zero. If they are heavily skewed, consider the
  Sign Test instead.
- **For Two-Sample (Rank-Sum):**
  + **Ideal Case (Location shift):** If both groups have roughly the same
    distributional shape (just shifted), the test compares location parameters
    and the Hodges-Lehmann shift equals the difference in medians.
  + **General Case:** If the shapes differ (e.g. one wide and flat, the other
    tall and narrow), the test remains valid as a test of *stochastic
    dominance* (\"values in Group A tend to be higher\"), but the
    Hodges-Lehmann shift no longer equals the difference in medians.
- **Visual Check:** Inspect the side-by-side boxplots and density plots.

## 4. Handling Ties (Duplicate Values)
- A *tie* occurs when two or more observations share the same value.
- If you see a warning *'cannot compute exact p-value with ties,'* R has
  automatically switched to a normal approximation. The result is generally
  still valid.

## 5. Adequate Sample Size
- **Small samples (n < 20):** R uses an exact test (if no ties).
- **Large samples:** R uses a normal approximation.
- **Rule of Thumb:** $n \\geq 6$ (paired) or $n_1 + n_2 \\geq 10$
  (two-sample) for adequate power."
      )
      # if (output_type %in% c("pdf", "word")) {
      #   cat("<div style=\"page-break-after: always;\"></div>\n\\newpage")
      # }
    }

    i <- 0
    output_list <- list()

    # Fires regardless of intro_text so the user always sees it in this situation.
    if (length(lhs) > 1) {
      k          <- length(lhs)
      fwer_pct   <- round((1 - (1 - alpha)^k) * 100, 1)
      bonf_alpha <- round(alpha / k, 4)
      cat(paste0(
        "\n\n***\n\n",
        "**[!]  NOTE: Multiple Testing Across ", k, " Response Variables**  \n\n",
        "This report runs ", k, " independent Wilcoxon tests on the same dataset. ",
        "Each individual test is valid on its own, but running multiple tests ",
        "simultaneously inflates the risk of obtaining at least one spurious ",
        "significant result by chance.  \n\n",
        "At \u03b1 = ", alpha, " per test, the probability of at least one false positive ",
        "across all ", k, " responses is approximately ",
        "**", fwer_pct, "%** ($1-(1-", alpha, ")^{", k, "}$, assuming independence). ",
        "This risk is highest in exploratory studies; it is less of a concern when ",
        "each response variable has a clear a priori hypothesis.  \n\n",
        "**Possible remedies:**  \n",
        "\n-  **Bonferroni** (conservative): re-run with `alpha = ", bonf_alpha,
        "` (\u03b1 / ", k, ") to control the family-wise error rate.",
        "\n-  **False Discovery Rate (FDR)**: collect the ", k, " p-values and apply ",
        "`p.adjust(p_values, method = \"fdr\")` after the fact.",
        "\n-  **Pre-registration**: if each response was a pre-specified primary outcome, ",
        "correction may not be required; document this decision explicitly.",
        "\n\n***\n\n"
      ))
    }

    # ----------------------------------------------------------------
    # Loop through response variables
    # ----------------------------------------------------------------
    for (response_name in lhs) {

      output_list[[response_name]] <- list()

      # Data prep
      cols_needed <- if (is_one_sample) {
        c(response_name)
      } else {
        c(response_name, predictor_name)
      }
      current_formula <- if (is_one_sample) {
        as.formula(paste0(response_name, " ~ 1"))
      } else {
        as.formula(paste0(response_name, " ~ ", predictor_name))
      }

      n_before      <- n_before_master
      data_complete <- data[, cols_needed, drop = FALSE]
      n_after       <- n_after_master

      cat("  \n \n# Analysis of: ", response_name, "  \n")

      # Prepare group vectors
      y_vals <- data_complete[[response_name]]
      if (is_one_sample) {
        group1 <- y_vals
        group2 <- NULL
        levs   <- NULL
      } else {
        groups <- data_complete[[predictor_name]]
        levs   <- levels(groups)
        group1 <- y_vals[groups == levs[1]]
        group2 <- y_vals[groups == levs[2]]
      }

      # ------------------------------------------------------------
      # Keep mu, mu was silently dropped from two-sample wilcox.test()
      # ------------------------------------------------------------
      if (is_one_sample) {
        test_res <- wilcox.test(
          group1,
          mu          = mu,
          alternative = alternative,
          conf.int    = TRUE,
          conf.level  = conf.level,
          ...
        )
        call_str <- paste0("wilcox.test(", response_name, ", mu = ", mu, ")")
      } else {
        test_res <- wilcox.test(
          group1,
          group2,
          mu          = mu,          # FIX 2: now correctly forwarded
          paired      = paired,
          alternative = alternative,
          conf.int    = TRUE,
          conf.level  = conf.level
        )
        type_str <- if (paired) "Signed-Rank" else "Rank-Sum"
        call_str <- paste0(type_str, " Wilcoxon test(",
                           response_name, " by ", predictor_name, ")")
      }

      output_list[[response_name]][["wilcox_test"]] <- test_res
      output_list[[response_name]][["wilcox_call"]] <- call_str
      output_list[[response_name]][["alpha"]]       <- alpha

      # ------------------------------------------------------------
      # Descriptive / diagnostic plots
      # ------------------------------------------------------------
      temp_file <- tempfile(fileext = ".png")
      png(temp_file, width = 7.8, height = 7.8, units = "in", res = 300)
      par(mfrow = c(2, 2), mar = c(4, 4, 4, 1))

      if (is_one_sample) {
        boxplot(y_vals, main = "Boxplot", col = "lightblue")
        try(f_hist(y_vals, xlab = response_name), silent = TRUE)
        try(f_qqnorm(y_vals), silent = TRUE)
        plot(y_vals, main = "Data Index", ylab = response_name)
      } else {
        boxplot(current_formula, data = data_complete,
                main = "Data by Group", col = "lightgrey")
        try(f_hist(y_vals, xlab = response_name), silent = TRUE)
        stripchart(current_formula, data = data_complete,
                   method = "jitter", vertical = TRUE,
                   main = "Data by Group (Jittered)")
        if (paired) {
          plot(group1 - group2, main = "Paired Differences", ylab = "Differences")
        } else {
          plot(density(group1), col = "blue", main = "Group Densities")
          lines(density(group2), col = "red")
          legend("topright", legend = levs, col = c("blue", "red"), lty = 1)
        }
      }
      dev.off()

      if (isTRUE(norm_plots)) {
        cat("\n     \n&nbsp;  \n Check the diagnostic plots below.  \n")
        cat(paste0("![](", temp_file, ")"), "  \n \n")
        cat("&nbsp;\n  \n")
      }

      output_list[[response_name]][["descriptive_plots"]] <- temp_file

      if (output_type != "rmd") {
        cat("\n<div style=\"page-break-after: always;\"></div>\n\\newpage")
      }

      # ------------------------------------------------------------
      # Output text generation
      # ------------------------------------------------------------

      # Helper: descriptive_txt
      generate_descriptive_txt <- function(is_one_sample, console = FALSE) {
        if (is_one_sample) {
          sample_median  <- median(y_vals, na.rm = TRUE)
          hl_estimate    <- test_res$estimate   # pseudo-median from wilcox.test

          txt <- paste0(
            "\n-  **Sample Median of ",
            response_name,
            ":** ",
            round(sample_median, 3),
            "\n-  **Hypothesized Value:** ",
            f_conditional_round(mu),
            "\n"
          )
        } else {
          m1          <- median(group1, na.rm = TRUE)
          m2          <- median(group2, na.rm = TRUE)
          hl_shift    <- test_res$estimate   # HL location shift from wilcox.test

          txt <- paste0(
            "\n-  **Median ",
            levs[1],
            ":** ",
            round(m1, 3),
            "\n-  **Median ",
            levs[2],
            ":** ",
            round(m2, 3),
            "\n-  **Raw difference (",
            levs[1],
            " - ",
            levs[2],
            "):** ",
            round(m1 - m2, 3),
            "\n"
          )
        }

        # prepare text for console usage
        if (console == TRUE) {
          txt <- gsub("\\*\\*", "", txt)   # remove bold markers
          txt <- gsub("\n-(?= )", "\n", txt, perl = TRUE) # remove markdown bullets
        }

        return(txt)
      }

      # Helper: Hypothesis text
      generate_hypotheses <- function(name_a,
                                      name_b      = NULL,
                                      paired      = FALSE,
                                      mu          = 0,
                                      alternative = "two.sided",
                                      console     = FALSE) {

        # Describe the *parameter the test actually tests*
        if (is.null(name_b)) {
          param_desc <- sprintf("pseudo-median (Hodges-Lehmann) of %s", name_a)
        } else if (paired) {
          param_desc <- sprintf(
            "pseudo-median of paired differences (%s - %s)", name_a, name_b)
        } else {
          param_desc <- sprintf(
            "location shift (%s - %s, Hodges-Lehmann)", name_a, name_b)
        }

        switch(alternative,
               "two.sided" = { h0_op <- "is equal to";               ha_op <- "is not equal to" },
               "greater"   = { h0_op <- "is less than or equal to";  ha_op <- "is greater than" },
               "less"      = { h0_op <- "is greater than or equal to"; ha_op <- "is less than"  }
        )

        null_text <- sprintf("True %s %s %s", param_desc, h0_op, mu)
        alt_text  <- sprintf("True %s %s %s", param_desc, ha_op, mu)

        if (console) {
          txt <- paste0("\n  H0: ", null_text, ".\n  H1: ", alt_text,".\n")
        } else {
          txt <- paste0("\n  + $H_{0}$: ", null_text, ".\n  + $H_{1}$: ", alt_text,".\n")
        }

        return(txt)
      }

      # Helper: estimate txt
      generate_estimate_txt <- function(is_one_sample, paired, console = FALSE) {
        if (is_one_sample) {
          txt <- paste0(
            "\n-  **Pseudo-median** (Hodges-Lehmann): ",
            round(test_res$estimate, 3)
          )
        } else if (paired) {

          txt <- paste0(
            "\n-  **Pseudo-median of differences** (Hodges-Lehmann): ",
            round(test_res$estimate, 3)
          )
        } else {
          txt <- paste0(
            "\n-  **Location shift** (Hodges-Lehmann): ",
            round(test_res$estimate, 3)
          )
        }

        # prepare text for console usage
        if (console == TRUE) {
          txt <- gsub("\\*\\*", "", txt)   # remove bold markers
          txt <- gsub("\n-(?= )", "\n", txt, perl = TRUE) # remove markdown bullets
        }

        return(txt)
      }

      # Helper: sig_txt, show text on significance based on p and alpha
      generate_sig_txt <- function(console = FALSE) {
        if (console == TRUE) {
          if (test_res$p.value <= alpha) {
            txt <- paste0("*-> Significant, H0 is rejected (p \u2264 \u03b1 = ",
                          alpha,
                          ")")
          } else {
            txt <- paste0(" -> Not significant,\n  H0 is NOT rejected  (p > \u03b1 = ",
                          alpha,
                          ")")
          }
        } else if (console == FALSE){
          if (test_res$p.value <= alpha) {
            txt <- paste0("* -> **Significant**, $H_{0}$ is rejected  (p $\\leq \\alpha$ = ",
                          alpha,
                          ")")
          } else {
            txt <- paste0(" -> Not significant, $H_{0}$ is **NOT** rejected  (p > $\\alpha$ = ",
                          alpha,
                          ")")
          }
        }

        return(txt)
      }

      # Helper: ci_txt, print ready confidence interval
      generate_ci_txt <- function(console = FALSE) {
          txt <- paste0(
            "\n-  **",
            conf_pct,
            "% CI:** [ ",
            round(test_res$conf.int[1], 3),
            ", ",
            round(test_res$conf.int[2], 3),
            " ]"
          )
        if (console == TRUE) {
          txt <- gsub("\\*\\*", "", txt)   # remove bold markers
          txt <- gsub("\n-(?= )", "\n", txt, perl = TRUE) # remove markdown bullets
        }

        return(txt)
      }

      # Helper: CI Note (depends on test type)
      generate_ci_note <- function(is_one_sample, paired, console = FALSE) {
         if (is_one_sample) {
            txt <-paste0(
              "\n  \n**Note on the CI**: \nThis interval is for the **pseudo-median**",
              " (Hodges-Lehmann estimator); ",
              "the median of all pairwise averages of your data points. Both the ",
              "sample median and the pseudo-median are valid descriptions of your data; ",
              "they just measure slightly different things (see `?f_wilcox_test` for details)."
            )
          } else if (paired) {
            txt <-paste0(
              "\n  \n**Note on the CI:** \nIn a paired test, the within-pair differences ",
              "(observation 1 minus observation 2 for each pair) are computed first. ",
              "This interval is then for the **pseudo-median of those differences**",
              " (Hodges-Lehmann estimator);  ",
              "the middle value of all possible pairwise averages of the differences. ",
              "For symmetric differences it equals the median of the raw differences. ",
              "When the differences are skewed the two can diverge, which is why the ",
              "CI may not be centred on the median of the raw differences. ",
              "Both are valid descriptions; they just measure it slightly differently ",
              "(see `?f_wilcox_test` for details).\n"
            )
          } else {
            txt <-paste0(
              "\n  \n**Note on the CI:** \nThis interval is for the **location shift**",
              " (Hodges-Lehmann estimator); ",
              "the median of all possible pairwise differences between your two groups. ",
              "If both groups have the same distributional shape, this equals the raw ",
              "median difference. When shapes differ they can diverge, which is why ",
              "the CI may not be centred on the difference in sample medians. ",
              "Both the raw median difference and the location shift are valid ",
              "descriptions of the gap between groups; they just measure it ",
              "slightly differently (see `?f_wilcox_test` for details)."
            )
          }
          # prepare text for console usage
          if (console == TRUE) {
            txt <- gsub("\\*\\*", "", txt)   # remove bold markers
          }

          return(txt)
        }


      # Prepare creation of markdown/word/pdf output

      # SAMPLE STATISTICS:
      descriptive_txt <- generate_descriptive_txt(is_one_sample, console = FALSE)

      # HYPOTHESES:
      hypotheses_txt <- generate_hypotheses(
        name_a      = if (is_one_sample) response_name else levs[1],
        name_b      = if (is_one_sample) NULL else levs[2],
        paired      = paired,
        mu          = mu,
        alternative = alternative,
        console = FALSE
      )

      # TEST RESULTS:
      p_val_fmt    <- f_conditional_round(test_res$p.value)
      sig_txt      <- generate_sig_txt(console = FALSE)

      # ESTIMATE:
      estimate_txt <- generate_estimate_txt(is_one_sample, paired, console = FALSE)
      ci_txt       <- generate_ci_txt(console = FALSE)
      ci_note      <- generate_ci_note(is_one_sample, paired, console = FALSE)

      #-----------------------------------------------------
      # Write output to Markdown
      #-----------------------------------------------------
      cat(paste0("  \n## ", test_res$method," of:", response_name, "  \n"))
      cat(paste0("&nbsp;\n  \n**Method:** ",
                 test_res$method, " (", test_res$alternative, ")." ))
      cat(paste0("&nbsp;\n  \n&nbsp;\n  \n**Sample statistics:**\n"))
      cat(descriptive_txt)
      cat(paste0("&nbsp;\n  \n&nbsp;\n  \n**Hypotheses:**\n"))
      cat(hypotheses_txt)
      cat(paste0(
        "&nbsp;\n  \n&nbsp;\n  \n**Test Results:**\n",
        "\n-  **W:** ", round(test_res$statistic, 3),
        "\n-  **p-value:** ", p_val_fmt, " ", sig_txt, "   \n"))
      cat(paste0(
        "&nbsp;\n  \n**Estimate:**\n"))
      cat(estimate_txt)
      cat(ci_txt)
      cat("&nbsp;\n  \n&nbsp;  \n  \n")
      cat(ci_note)



      if (n_before != n_after)
        cat(paste0("**WARNING** Removed ", n_before - n_after,
                   " rows with missing values.\n"))


      #-----------------------------------------------------
      # Write output for console (for print.f_wilcox_test)
      #-----------------------------------------------------

      # Sample statistics (console)
      descriptive_txt_out <- generate_descriptive_txt(is_one_sample, console = TRUE)

      # HYPOTHESES: (console)
      hypotheses_txt_out <- generate_hypotheses(
        name_a      = if (is_one_sample) response_name else levs[1],
        name_b      = if (is_one_sample) NULL else levs[2],
        paired      = paired,
        mu          = mu,
        alternative = alternative,
        console     = TRUE
      )

      # TEST RESULTS: (console)
      sig_txt_out     <- generate_sig_txt(console = TRUE)

      # ESTIMATE: (console)
      estimate_txt_out<- generate_estimate_txt(is_one_sample, paired, console = TRUE)
      ci_txt_out      <- generate_ci_txt(console = TRUE)
      ci_note_out     <- generate_ci_note(is_one_sample, paired, console = TRUE)

      # store in output_list
      output_list[[response_name]][["descriptive_txt"]]<- descriptive_txt_out
      output_list[[response_name]][["hypotheses_txt"]] <- hypotheses_txt_out
      output_list[[response_name]][["p_val_fmt"]]      <- p_val_fmt
      output_list[[response_name]][["sig_txt"]]        <- sig_txt_out
      output_list[[response_name]][["estimate_txt"]]   <- estimate_txt_out
      output_list[[response_name]][["ci_txt"]]         <- ci_txt_out
      output_list[[response_name]][["ci_note"]]        <- ci_note_out

      # Pagebreak depending on doc length.
      i <- i + 1
      if (output_type != "rmd" && i < length(lhs))
        cat("\n \n<div style=\"page-break-after: always;\"></div>\n\\newpage\n")
    } # end lhs loop

    if (isTRUE(output)) return(output_list)
  } # end generate_report()

  # ------------------------------------------------------------------
  # 6.  Execute -- single call captures both output_list and markdown
  # ------------------------------------------------------------------

  md_lines   <- capture.output(output_list <- generate_report())
  class(output_list) <- "f_wilcox_test"

  # ------------------------------------------------------------------
  # 7.  File output
  # ------------------------------------------------------------------
  if (output_type %in% c("word", "pdf")) {

    message(paste0("Saving output in: ", output_path))

    word_pdf_preamble <- function() {
      paste0(
        "---\n",
        "title: \"f_wilcox_test Analysis Report\"\n",
        "date: \"`r Sys.Date()`\"\n",
        "output:\n",
        "   word_document:\n",
        "     reference_docx: !expr system.file(",
        "\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")\n",
        "   pdf_document:\n",
        "     latex_engine: pdflatex\n",
        "header-includes:\n",
        "  - \\usepackage[utf8]{inputenc}\n",
        "  - \\usepackage[T1]{fontenc}\n",
        "  - \\usepackage{textcomp}\n",
        "  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}\n",
        "  - \\DeclareUnicodeCharacter{2264}{\\ensuremath{\\leq}}\n",
        "  - \\DeclareUnicodeCharacter{03B1}{\\ensuremath{\\alpha}}\n",
        "  - \\usepackage{titling}\n",
        "  - \\setlength{\\droptitle}{-2.5cm}\n",
        "---\n"
      )
    }

    knitr::opts_chunk$set(comment = "")
    # Reuse md_lines from the single generate_report() call above
    rmd_content <- paste(word_pdf_preamble(),
                         paste(md_lines, collapse = "\n"),
                         sep = "\n")
    writeLines(rmd_content, temp_output_file)
    rmarkdown::render(
      temp_output_file,
      output_file       = output_path,
      intermediates_dir = temp_output_dir,
      knit_root_dir     = temp_output_dir,
      quiet             = TRUE,
      output_format     = paste0(output_type, "_document")
    )
    if (isTRUE(open_generated_files)) f_open_file(output_path)
    return(invisible(output_list))

  } else if (output_type == "excel") {

    message(paste0("Saving output in: ", output_path))
    summary_tables <- lapply(output_list, function(obj) {
      w_obj <- obj$wilcox_test
      data.frame(
        Statistic       = w_obj$statistic,
        p_value         = w_obj$p.value,
        LowerCI         = w_obj$conf.int[1],
        UpperCI         = w_obj$conf.int[2],
        HL_estimate     = w_obj$estimate,
        Method          = w_obj$method
      )
    })
    names(summary_tables) <- lhs
    write_xlsx(summary_tables, path = output_path)
    if (isTRUE(open_generated_files)) f_open_file(output_path)
    return(invisible(output_list))

  } else if (output_type == "rmd") {

    if (is.null(knitr::opts_knit$get("output.dir")))
      knitr::opts_knit$set(output.dir = tempdir())
    output_list[["rmd"]] <- paste(md_lines, collapse = "\n")
    return(invisible(output_list))

  } else if (output_type == "console") {

    print(output_list)
    return(invisible(output_list))

  }

  # "default" -- return list silently, clean up temp file
  invisible(suppressWarnings(file.remove(temp_output_file)))
  return(output_list)
}


# ============================================================
#  Default (vector) method
# ============================================================
#' @export
#' @rdname f_wilcox_test
f_wilcox_test.default <- function(x,
                                  y                    = NULL,
                                  paired               = FALSE,
                                  conf.level           = NULL,
                                  mu                   = 0,
                                  alternative          = "two.sided",
                                  norm_plots           = TRUE,
                                  alpha                = 0.05,
                                  intro_text           = TRUE,
                                  close_generated_files  = FALSE,
                                  open_generated_files   = TRUE,
                                  output_type          = "default",
                                  save_as              = NULL,
                                  save_in_wdir         = FALSE,
                                  ...) {

  if (!is.null(y) && paired && length(x) != length(y))
    stop("For a paired test, 'x' and 'y' must have the same length.")
  if (!is.numeric(x))
    stop("'x' must be numeric for the vector interface.")
  if (!is.null(y) && !is.numeric(y))
    stop("'y' must be numeric or NULL for the vector interface.")

  # Capture original variable names for labels
  x_name <- deparse(substitute(x))
  y_name <- if (!is.null(y)) deparse(substitute(y)) else NULL

  if (is.null(y)) {
    # One-sample
    col_name      <- make.names(x_name)
    response_name <- x_name
    data_list        <- setNames(list(x), col_name)
    df               <- as.data.frame(data_list)
    form             <- as.formula(paste0(col_name, " ~ 1"))
  } else {
    # Two-sample -- convert to long format
    col_name      <- make.names(paste0(x_name, "_vs_", y_name))
    response_name <- paste0(x_name, "_vs_", y_name)
    grp_levels    <- c(x_name, y_name)
    grp           <- factor(rep(grp_levels, times = c(length(x), length(y))),
                            levels = grp_levels)
    data_list     <- setNames(list(c(x, y), grp), c(col_name, "grp"))
    df            <- as.data.frame(data_list)
    form          <- as.formula(paste0(col_name, " ~ grp"))
  }

  out <- f_wilcox_test.formula(
    formula               = form,
    data                  = df,
    paired                = paired,
    conf.level            = conf.level,
    mu                    = mu,
    alternative           = alternative,
    norm_plots            = norm_plots,
    alpha                 = alpha,
    intro_text            = intro_text,
    close_generated_files = close_generated_files,
    open_generated_files  = open_generated_files,
    output_type           = output_type,
    save_as               = save_as,
    save_in_wdir          = save_in_wdir,
    ...
  )

  # Rename list key from make.names version back to the original name
  if (col_name %in% names(out)) {
    names(out)[names(out) == col_name] <- response_name
  }

  return(out)
}


# ============================================================
#  print method
# ============================================================
#' @export
print.f_wilcox_test <- function(x, ...) {
  for (nm in names(x)) {
    if (nm == "rmd") next
    sublist <- x[[nm]]
    w_obj   <- sublist$wilcox_test

    cat("\n==========================================================\n")
    cat(paste0(w_obj$method," (",w_obj$alternative,") of:", nm, "\n"))
    cat("==========================================================\n")
    cat("\nSAMPLE STATISTICS:")
    cat(sublist$descriptive_txt)
    cat("\nHYPOTHESES:")
    cat(sublist$hypotheses_txt)
    cat("\nTEST RESULTS:")
    cat(paste0("\n  W = ", w_obj$statistic,", p-value = ", sublist$p_val_fmt))
    cat(sublist$sig_txt)
    cat("\n  \nESTIMATE:")
    cat(sublist$estimate_txt)
    cat(sublist$ci_txt)
    cat(sublist$ci_note)

  }
  cat("\n \n")
}


# ============================================================
#  plot method
# ============================================================
#' @export
plot.f_wilcox_test <- function(x, ...) {

  response_names <- names(x)[names(x) != "rmd"]

  if (length(response_names) == 0) {
    message("No plot data available in this f_wilcox_test object.")
    return(invisible(NULL))
  }

  has_png  <- requireNamespace("png",  quietly = TRUE)
  has_grid <- requireNamespace("grid", quietly = TRUE)

  if (!has_png || !has_grid) {
    message("Install packages 'png' and 'grid' to display plots inline.")
    message("Stored diagnostic plot file(s):")
    for (nm in response_names) {
      path <- x[[nm]]$descriptive_plots
      if (!is.null(path)) message("  ", nm, ": ", path)
    }
    return(invisible(NULL))
  }

  for (nm in response_names) {
    path <- x[[nm]]$descriptive_plots
    if (!is.null(path) && file.exists(path)) {
      img <- png::readPNG(path)
      grid::grid.newpage()
      grid::grid.raster(img)
    } else {
      message("Plot file not found for variable: ", nm)
    }
  }

  invisible(NULL)
}

# This code is based on two packages MASS and rcompanion
# -------------------------
# 1.Some code to present the result was taken and modified from file:
# rcompanion/R/transformTukey.r
#
# @author Salvatore Mangiafico, \email{mangiafico@njaes.rutgers.edu}
#
# @references \url{https://rcompanion.org/handbook/I_12.html}
#-------------------------
# 2. The core of calculating lambda and the plotting was taken from:
# file MASS/R/boxcox.R
#
# copyright (C) 1994-2004 W. N. Venables and B. D. Ripley
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#
#  also see: https://r-coder.com/box-cox-transformation-r/
#--------------------------
#
#' f_boxcox: A User-Friendly Box-Cox Transformation
#'
#' Performs a Box-Cox transformation on a dataset to stabilize variance and make the data more normally distributed. It also provides diagnostic plots and tests for normality. The transformation is based on code of MASS/R/boxcox.R. The function prints \eqn{\lambda} to the console and returns (output) the transformed data set.
#'
#' @param data A numeric vector or a data frame with a single numeric column. The data to be transformed.
#' @param digits Numeric. Determines the accuracy of the estimate for lambda. Higher values increase computation time. Defaults to \code{3}.
#' @param range A numeric vector of length 2 defining the search interval for lambda. Defaults to \code{c(-2, 2)}.
#' @param plots Logical. If \code{TRUE}, plots log-likelihood of the Box-Cox transformation, Histograms and Q-Q plots of the original and transformed data. Default is \code{FALSE}.
#' @param close_generated_files Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param output_type Character string specifying the output format. Default is \code{"default"}.
#'   \itemize{
#'     \item \code{"default"}: Returns the object and lets R decide whether
#'       to print; auto-prints if unassigned, silent if assigned to a variable.
#'       Use \code{print(result)} or \code{plot(result)} to display the
#'       returned object.
#'     \item \code{"console"}: Forces immediate printing to the console
#'       regardless of object assignment.
#'     \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
#'       file of the corresponding format. See \code{save_as},
#'       \code{save_in_wdir}, and \code{open_generated_files} for file
#'       path and opening behavior.
#'     \item \code{"rmd"}: Stores the raw markdown string inside the returned
#'       object for use in R Markdown documents.
#'   }
#' @param save_as Character string specifying the output file path (without extension).
#'   If a full path is provided, output is saved to that location.
#'   If only a filename is given, the file is saved in \code{tempdir()}.
#'   If only a directory is specified (providing an existing directory with trailing slash),
#'   the file is named "data_name_aov_output" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
#'   Defaults to \code{file.path(tempdir(), "data_name_summary.pdf")}.
#' @param save_in_wdir Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.
#' @param transform.data Logical. If \code{TRUE}, returns the transformed data. Default is \code{TRUE}.
#' @param eps A small positive value used to determine when to switch from the power transformation to the log transformation for numerical stability. Default is \code{1/50}.
#' @param xlab Character string. Label for the x-axis in plots. Default is an expression object representing \eqn{\lambda}.
#' @param ylab Character string. Label for the y-axis in plots. Default is "log-Likelihood".
#' @param alpha Numeric. Significance level for the Shapiro-Wilk test of normality. Default is \code{0.05}.
#' @param ... Additional arguments passed to plotting functions.
#'
#'
#' @return An object of class 'f_boxcox' containing, among others, results from the boxcox transformation, lambda, the input data, transformed data, Shapiro-Wilk test on original and transformed data. Using the option "output_type", it can also generate output in the form of: R Markdown code, 'Word', or 'pdf' files. Includes print and plot methods for 'f_boxcox' objects.
#'
#' @details
#'
#'The function uses the following formula for transformation:
#'\deqn{
#'  y(\lambda) =
#'    \begin{cases}
#'  \frac{y^\lambda - 1}{\lambda}, & \lambda \neq 0 \\  \log(y), & \lambda = 0
#'  \end{cases}
#'}
#'
#' where (\eqn{y}) is the data being transformed, and (\eqn{\lambda}) the transformation parameter, which is estimated from the data using maximum likelihood. The function computes the Box-Cox transformation for a range of \eqn{\lambda} values and identifies the \eqn{\lambda} that maximizes the log-likelihood function. The beauty of this transformation is that, it checks suitability of many of the common transformations in one run. Examples of most common transformations and their \eqn{\lambda} value is given below:
#'\renewcommand{\arraystretch}{3} % Adjusts row height (default is 1)
#' \tabular{cc}{
#'   \strong{\eqn{\lambda}-Value} \tab \strong{Transformation} \cr
#'   \strong{-----------------------}\tab\strong{-----------------------}\cr
#'   -2          \tab \eqn{\frac{1}{x^2}}          \cr
#'   -1          \tab \eqn{\frac{1}{x}  }          \cr
#'   -0.5        \tab \eqn{\frac{1}{\sqrt{x}}}     \cr
#'   0           \tab \eqn{log(x)}                 \cr
#'   0.5         \tab \eqn{\sqrt{x}}               \cr
#'   1           \tab \eqn{x}                      \cr
#'   2           \tab \eqn{x^2}                    \cr
#'   \strong{-----------------------}\tab\strong{-----------------------}\cr
#' }
#'
#' If the estimated transformation parameter closely aligns with one of the values listed in the previous table, it is generally advisable to select the table value rather than the precise estimated value. This approach simplifies interpretation and practical application.
#'
#' The function provides diagnostic plots: a plot of log-likelihood against \eqn{\lambda} values and a Q-Q plot of the transformed data.It also performs a Shapiro-Wilk test for normality on the transformed data if the sample size is less than or equal to 5000.
#'
#' \strong{Note}: For sample sizes greater than 5000, Shapiro-Wilk test results are not provided due to limitations in its applicability.
#'
#' This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
#'\itemize{
#' \item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
#' \item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
#' \item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
#'
#' \item If Pandoc is not found, this function may not work as intended.
#' }
#'
#' @examples
#' # Create non-normal data in a data.frame or vector.
#' df   <- data.frame(values = rlnorm(100, meanlog = 0, sdlog = 1))
#'
#' # Store the transformation in object "bc".
#' bc <- f_boxcox(df$values)
#'
#' # Print lambda and Shaprio.
#' print(bc)
#'
#' # Plot the QQ plots, Histograms and Lambda Log-Likelihood estimation.
#' plot(bc)
#'
#' # Or Directly use the transformed data from the f_boxcox object.
#' df$values_transformed <- f_boxcox(df$values)$transformed_data
#' print(df$values_transformed)
#'
#' @seealso
#' \href{https://CRAN.R-project.org/package=MASS}{\code{boxcox}}
#'
#' @references
#'
#'The core of calculating \eqn{\lambda} and the plotting was taken from: \cr
#' file MASS/R/boxcox.R copyright (C) 1994-2004 W. N. Venables and B. D. Ripley
#' \itemize{
#' \item \url{https://r-coder.com/box-cox-transformation-r/}
#' \item \url{https://CRAN.R-project.org/package=MASS}
#' }
#'Some code to present the result was taken and modified from file: \cr
#'rcompanion/R/transformTukey.r. (Developed by Salvatore Mangiafico)
#' \itemize{
#' \item \url{https://rcompanion.org/handbook/I_12.html}
#' }
#' The explanation on BoxCox transformation provided here was provided by r-coder:
#' \itemize{
#' \item  \url{https://r-coder.com/box-cox-transformation-r/}
#' }

#' @author
#' \itemize{
#' \item Sander H. van Delden  \email{plantmind@proton.me}
#' \item Salvatore Mangiafico, \email{mangiafico@njaes.rutgers.edu}
#' \item W. N. Venables and B. D. Ripley
#' }
#'
#'
#' @export
f_boxcox <- function(
    data = data,                  # Vector or a data.frame column
    digits = 3,                   # Accuracy by how many digits lambda is estimated
    range = c(-2, 2),             # The search interval for lambda.
    plots = NULL,                # Show lambda est.histograms and QQ plots (TRUE) or not (FALSE)
    transform.data = TRUE,        # Specify the name of the file
    eps = 1 / 50,                 # Tolerance for lambda. Defaults to 0.02.
    xlab = expression(lambda),    # X-axis title of plot
    ylab = "log-Likelihood",      # Y-axis title of plot
    alpha = 0.05,                 # Significance level for shapiro test
    open_generated_files = interactive(),  # Open files after creation
    close_generated_files = FALSE,# Close open files to save a new one
    output_type = "default",          # Output type can be word, pdf, rmd, console
    save_as = NULL,               # Specify the name of the output dir and file (name and type).
    save_in_wdir = FALSE,         # Save file output in the working directory.
    ...                           # Additional arguments for model fitting
    ) {


  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state


  # Parameter validation
  if( !(output_type %in% c("pdf", "word", "excel", "rmd", "console" , "default")) ){
    stop("Character string specifying the output format (output_type = ) should be either: 'pdf', 'word', 'excel', 'console','rmd', 'default'")
  }

  if(is.null(plots) && output_type %in% c("pdf", "word")){
    plots <- TRUE
  } else if(is.null(plots)){
    plots <- FALSE
  }

  # Generate a temporary file path for "output.Rmd"
  temp_output_dir  <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")

  # Create the output file "output.Rmd" in tempdir()
  file.create(temp_output_file)
  # Create a file_extension switch
  file_extension <- NULL

  # Create an output list to store info
  output_list <- list()
  output_list[["plots"]] <- plots


  # Capture the name of the submitted data object and coerce to a
  # single-column data frame. The multi-column check MUST run before the
  # reduction below, otherwise it becomes dead code and a multi-column
  # data frame is silently truncated to its first column.
  if (is.data.frame(data)) {
    if (ncol(data) > 1) {
      stop("The data.frame has multiple columns please select one.")
    }
    data_name <- colnames(data)[[1]]
    data <- data.frame(y = data[[1]])
  } else if (is.vector(data)) {
    data_name <- deparse(substitute(data))
    data <- data.frame(y = data)
  } else {
    stop("Input must be a numeric vector or data frame.")
  }

  # At this point data is guaranteed to be a single-column data frame;
  # the only remaining type check is that the column is numeric.
  if (!is.numeric(data$y)) {
    stop(paste0("The ", data_name, " column in the data must be numeric."))
  }


  clean_data_name <- sub(".*\\$", "", data_name)  # Remove everything before the "$" symbol

  output_list[["data_name"]] <- clean_data_name

  #### Handle option "save_as = " ###
  if(save_in_wdir == TRUE){
    save_dir <- getwd()
  }else{
    save_dir <- tempdir()
  }

  #map the output type to extensions
  output_type_map <- c(
    "pdf"  = ".pdf",
    "word" = ".docx",
    "rmd"  = ".rmd",
    "excel" = ".xlsx"
  )

  # If the user specifies a path, filename or save_in_wdir == TRUE an output file should be created
  if (!is.null(save_as) || save_in_wdir == TRUE) {

    if (!is.null(save_as)) {
      #Remove backslash in save_as if needed
      save_as <- gsub(pattern = "\\\\", replacement = "/", x = save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if(file_extension_save_as[1] != FALSE){
        file_extension <- file_extension_save_as
      }
    }

    if(is.null(file_extension) && output_type %in% c("console", "default")){
      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(clean_data_name, "BoxCox_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = ".pdf"
      )
      #set output_type to default
      output_type <- "pdf"

    }
    else if(is.null(file_extension) && output_type %in% c("pdf", "word", "excel", "rmd")){

      #create extension based on input_type
      file.ext <- unname(output_type_map[output_type])

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(clean_data_name, "BoxCox_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file.ext
      )


    }
    else if(!is.null(file_extension)) {

      # use helper get_save_path() to create output_path
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(clean_data_name, "BoxCox_output", sep = "_"),
                                   default_dir = save_dir,
                                   file.ext = file_extension[1]
      )
      # reset the output type to match the user input extention in save_as
      output_type <- file_extension[2]
    }
  } else {

    #create extension based on input_type
    file.ext <- unname(output_type_map[output_type])

    # use helper get_save_path() to create output_path
    output_path <- get_save_path(save_as = save_as,
                                 default_name = paste(clean_data_name, "BoxCox_output", sep = "_"),
                                 default_dir = save_dir,
                                 file.ext = file.ext
    )
  }


  # Prevent output to console and keep files open when output is "rmd" format
  if (output_type == "rmd") close_generated_files <- FALSE

  # Cross-platform close_generated_files (was Windows-only taskkill)
  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    if (output_type == "excel") close_app("EXCEL.EXE",   "Microsoft Excel", "soffice")
  }

  #Create a Vector of potential values for lambda
  lambda <- seq(min(range, na.rm = TRUE), max(range, na.rm = TRUE), 1 / 10^digits)

  # Extract and validate the response variable
  y <- data$y
  y_clean <- y[!is.na(y)]


  if (any(y_clean <= 0)) stop("Response variable must be higher than zero.")

  # Scale y for numerical stability
  y_clean <- y_clean / exp(mean(log(y_clean)))  # Geometric mean scaling
  logy <- log(y_clean)
  n <- length(y_clean)
  output_list[["geom_mean"]] <- exp(mean(log(y[!is.na(y)])))

  # Initialize log-likelihood
  loglik <- numeric(length(lambda))
  for (i in seq_along(lambda)) {
    la <- lambda[i]
    yt <- if (abs(la) > eps) (y_clean^la - 1) / la else logy * (1 + la * logy / 2)
    loglik[i] <- -n / 2 * log(sum((yt - mean(yt))^2))
  }



  # Interp: whether to spline-interpolate for smoother plots. spline()
  # needs at least 2 points, so we also guard against degenerate lambda
  # vectors (e.g. range = c(-0.001, 0.001) with digits = 1 gives a
  # single lambda value).
  interp <- isTRUE(plots) && length(lambda) >= 2L && length(lambda) < 100L

  # Interpolation for smooth plotting
  if (interp) {
    sp <- spline(lambda, loglik, n = 100)
    lambda <- sp$x
    loglik <- sp$y
  }


  #Do a transformation of the entered data
  max_idx <- which.max(loglik)
  max_ll  <- loglik[max_idx]
  conf_limit <- max_ll - qchisq(0.95, 1) / 2
  # Apply the selected Box-Cox transformation to the (original, unscaled)
  # input data. Using `else` rather than `else if (lambda == 0)` guarantees
  # transformed_data is always assigned on any finite lambda, which avoids
  # a cryptic "object not found" error if lambda[max_idx] were ever NA.
  if (lambda[max_idx] != 0) {
    transformed_data <- (data ^ lambda[max_idx] - 1) / lambda[max_idx]
  } else {
    transformed_data <- log(data)
  }

  transformed_data <- data.frame(transformed_data)
  data_name_out <- paste0(data_name, "_transformed")
  colnames(transformed_data) <- data_name_out

  # Lambda for output
  lambda_out <- signif(lambda[max_idx], digits=4)


  n <- length(data$y)

  # Run Shapiro-Wilk on original and transformed data via the package
  # helper, which returns a shaped htest object for all n regimes:
  #   - n < 3    -> NA p-value, method "skipped: n < 3"
  #   - 3..5000  -> real Shapiro-Wilk result
  #   - n > 5000 -> NA p-value, method "skipped: n > 5000"
  # Downstream code therefore only needs to check is.na(p.value) once;
  # no separate n-guard branching is required here. Note: these local
  # data frames are named shapiro_original_df / shapiro_transformed_df
  # rather than df0 / df, because a local object named `df` shadows
  # stats::df (the F-distribution density) and produces cryptic
  # "closure is not subsettable" errors on any code path that fails
  # to assign to it.
  st0 <- safe_shapiro(data[[1]])
  W0               <- if (!is.na(st0$p.value)) signif(st0$statistic, digits = 4) else NA_real_
  Shapiro.p.value0 <- if (!is.na(st0$p.value)) signif(st0$p.value,   digits = 4) else NA_real_
  shapiro_original_df <- data.frame(W0, Shapiro.p.value0)

  st1 <- safe_shapiro(transformed_data[[1]])
  W                <- if (!is.na(st1$p.value)) signif(st1$statistic, digits = 4) else NA_real_
  Shapiro.p.value  <- if (!is.na(st1$p.value)) signif(st1$p.value,   digits = 4) else NA_real_
  shapiro_transformed_df <- data.frame("lambda" = lambda_out, W, Shapiro.p.value)

  output_list[["Shapiro_original_data"]]    <- shapiro_original_df
  output_list[["Shapiro_transformed_data"]] <- shapiro_transformed_df


    output_list[["transformed_data"]]         <- transformed_data[[1]]
    output_list[["original_data"]]            <- data[[1]]
    output_list[["lambda"]]                   <- lambda_out
    output_list[["n"]]                        <- n
    output_list[["missing values"]]           <- sum(is.na(y))
    output_list[["xlab"]]                     <- xlab
    output_list[["ylab"]]                     <- ylab
    output_list[["alpha"]]                    <- alpha
    output_list[["lambda_estimates"]]         <- lambda
    output_list[["loglik"]]                   <- loglik
    output_list[["conf_limit"]]               <- conf_limit


  # Make a class from the output to print and plot
  class(output_list) <- "f_boxcox"


  if(output_type != "console" && output_type != "default"){
  generate_report <- function(){
  # Return results
    if (is.na(Shapiro.p.value0)) {
      cat("Shapiro-Wilk test on untransformed **original data:** ",
          "skipped (", st0$method, "). ",
          "For very large samples Shapiro-Wilk is skipped because it is ",
          "overly sensitive to trivial deviations from normality; please ",
          "inspect the diagnostic plots instead.   \n")
    } else {
      cat("Shapiro-Wilk test on untransformed **original data:**")
      cat(" W =", W0, " p value =", Shapiro.p.value0, "   \n")
      if (Shapiro.p.value0 >= alpha) {
        cat(
          "According to the ShapiroW test (",
          Shapiro.p.value0,
          " > ",
          alpha,
          ") original data is already normally distributed. \nTransformation will be applied regardless...  \n"
        )
      } else {
        cat(
          "According to the Shapiro-Wilk test (",
          Shapiro.p.value0,
          " < ",
          alpha,
          ") original data is NOT normally distributed.  \nTransformation will be applied..."
        )
      }
    }
    cat("   \n   \n")
    cat("$$y(\\lambda) = \\begin{cases} \\frac{y^\\lambda - 1}{\\lambda}, & \\lambda \\neq 0 \\\\    \\log(y), & \\lambda = 0 \\end{cases}
            $$   \n")

    cat("   \n   \n**Box-Cox Transformation $\\lambda$ =", lambda_out, "**  \n")


    # Plot results of transformation
    if(plots == TRUE){
      temp_file <- tempfile(fileext = ".png")
      png(temp_file, width = 8, height = 5, units = "in", res = 300)
      plot(output_list, which= 1)
      dev.off()
      cat(paste0("![](", temp_file, ")"), "   \n  \n")
    }


    cat("   \n   \n**Interpretation:**   \n")
    if (is.na(Shapiro.p.value)) {
      cat("Shapiro-Wilk on transformed data was skipped (", st1$method,
          "). For very large samples Shapiro-Wilk is overly sensitive ",
          "to trivial deviations from normality; please inspect the ",
          "diagnostic plots below to assess normality of the ",
          "transformed data.  \n  \n", sep = "")
    } else if (Shapiro.p.value >= alpha) {
      cat("According to the Shapiro-Wilk test (", Shapiro.p.value, " > ",alpha,") data is normally distributed after transformation.  \n \n Inspect the plots to check normality and outliers:  \n  \n ")
    } else {
      cat("According to the Shapiro-Wilk test (", Shapiro.p.value, " < ",alpha,") data is still NOT normally distributed after transformation.  \n  \n Inspect the plots to check normality and outliers:  \n  \n ")
    }


    # Plot results of transformation histogram
    if(plots == TRUE){
      temp_file <- tempfile(fileext = ".png")
      png(temp_file, width = 7.8, height = 5, units = "in", res = 300)
      plot(output_list, which= 2)
      dev.off()
      cat(paste0("![](", temp_file, ")"), "   \n  \n")
    }

    # Plot results of transformation qqplots
    if(plots == TRUE){
      temp_file <- tempfile(fileext = ".png")
      png(temp_file, width = 7.8, height = 5, units = "in", res = 300)
      plot(output_list, which= 3)
      dev.off()
      cat(paste0("![](", temp_file, ")"), "   \n  \n")
    }
} # End generate report
  }

# Here the documents are constructed.
if (output_type %in% c("word", "pdf")) {


  # Create a temporary R Markdown file
  word_pdf_preamble <- function(){ paste0("
---
title: \"f_BoxCox Report\"
date: \"`r Sys.Date()`\"
output:
   word_document:
      reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
        latex_engine: pdflatex
header-includes:
  - \\usepackage[utf8]{inputenc}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
  - \\usepackage{titling}
  - \\setlength{\\droptitle}{-2.5cm} % Adjust vertical spacing
---
")}


  # Prevent ## before printed output
  knitr::opts_chunk$set(comment = "")

  # Show save location before knitting else it will not display in console.
  message(paste0("Saving output in: ", output_path))

  # re-run generate_report, but this time capture its output to a string
  generated_markdown <- capture.output(generate_report())

  # Combine the preamble, assumptions, and the captured report into one string
  rmd_content <- paste(
    word_pdf_preamble(),
    # The captured output already contains the necessary markdown formatting and image links
    paste(generated_markdown, collapse = "\n"),
    sep = "\n"
  )

  # Write the complete Rmd content to the temp file
  writeLines(rmd_content, temp_output_file)

  # Create the RMarkdown file
  rmarkdown::render(
    temp_output_file,
    output_file = output_path,
    intermediates_dir = temp_output_dir,
    knit_root_dir = temp_output_dir,
    quiet = TRUE,
    output_format = paste0(output_type, "_document")
  )

    if(open_generated_files == TRUE){
    # Open the file with default program
    f_open_file(output_path)
    }

  }
  else if (output_type == "rmd"){

    if (is.null(opts_knit$get("output.dir"))) {
      opts_knit$set(output.dir = tempdir())
    }

    # Re-capture the markdown text for the rmd output
    generated_markdown <- capture.output(generate_report())

    clean_rmd_output <- paste(generated_markdown, collapse = "\n")

    output_list[["rmd"]] <- clean_rmd_output

  }
  else if (output_type == "excel") {

    # show the location were the file is saved
    message(paste0("Saving output in: ", output_path))


    excel_out_list <- list(
      "transformed_data" = setNames(data.frame(transformed_data[[1]]), paste0("transformed_", clean_data_name)),
      "original_data"    = setNames(data.frame(data[[1]]), paste0("original_", clean_data_name ))
    )

    # Write to an Excel file with each table in its own sheet
    writexl::write_xlsx(excel_out_list, path = output_path)

    # Open files after creation
    if(open_generated_files == TRUE){
      f_open_file(output_path)
    }

    return(invisible(output_list))
  }
  else if (output_type == "console"){


    print(output_list)
    # Plot all to console
    plot(output_list)


  }
  else if (output_type == "default"){

 # Do not show anything unless the user want to see plots:
    # Plot results of transformation if the user wants to
    if (plots ==TRUE) plot(output_list)

   }
  else {

      warning("Invalid output format specified. No file generated.")


   }

  # Remove the temporary R Markdown file
  invisible(suppressWarnings(file.remove(temp_output_file)))

  return(output_list)

}

#' @export
print.f_boxcox <- function(x, ...) {

  cat("Box-Cox\n")
  cat("--------\n")
  .sp0 <- x$Shapiro_original_data$Shapiro.p.value0
  if (is.na(.sp0)) {
    cat("Shapiro-Wilk on original data was skipped (sample size outside ",
        "the 3..5000 range supported by shapiro.test). Transformation ",
        "will be applied regardless; inspect plots for assessment.   \n", sep = "")
  } else if (.sp0 >= x$alpha) {
    cat("According to the Shapiro-Wilk test (", .sp0, " > ", x$alpha,
        ") original data is:\n already normally distributed. Transformation will be applied regardless...  \n")
  } else {
    cat("According to the Shapiro-Wilk test (", .sp0, " < ", x$alpha,
        ") original data is:\n NOT normally distributed. Transformation will be applied...")
  }

  cat("\u00A0  \n")
  cat("  \nFormula used for transformation:   \n")
  cat("{ (x^\u03BB - 1) / \u03BB } if \u03BB != 0   \n")
  cat("{ log(x)\u00A0\u00A0\u00A0\u00A0\u00A0\u00A0\u00A0\u00A0} if \u03BB == 0   \n")
  cat("\u00A0  \n")
  cat("Box-Cox Transformation \u03BB =", x$lambda, "  \n")
  .sp1 <- x$Shapiro_transformed_data$Shapiro.p.value
  if (is.na(.sp1)) {
    cat("Shapiro-Wilk on transformed data was skipped (sample size ",
        "outside the 3..5000 range). \nInspect the diagnostic plots to ",
        "assess normality of the transformed data.\n",
        "Check the normality plots, by using the plot() function or ",
        "'plots = TRUE' option\n  \n", sep = "")
  } else if (.sp1 >= x$alpha) {
    cat("According to the Shapiro-Wilk test (", .sp1, " > ", x$alpha,
        ") data is\n normally distributed after transformation.\n  \n")
  } else {
    cat("According to the Shapiro-Wilk test (", .sp1, " < ", x$alpha,
        ") data is\n still NOT normally distributed after transformation.")
    cat("\nCheck the normality plots, by using the plot() function or 'plots = TRUE' option\n  \n")
  }

  if(x$plots == TRUE){

    plot(x)
  }

}

#' Plot method for f_boxcox objects
#'
#' @title Plot an f_boxcox object
#' @name plot.f_boxcox
#' @description Create diagnostic plots of an object of class \code{f_boxcox}.
#' @param x An object of class \code{f_boxcox}.
#' @param which Integer determining which graph to plot. Default is \code{1:2}.
#' @param ask Logical. \code{TRUE} waits with plotting each graph until <Return> is pressed. Default is \code{FALSE}.
#' @param ... Further arguments passed to or from other methods.
#' @method plot f_boxcox
#' @return This function is called for its side effect of generating plots
#' and does not return a useful value. It invisibly returns \code{1}.
#' @export
plot.f_boxcox <- function(x, which = 1:3, ask = FALSE, ...) {

  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state

par(ask = ask)

  # 1: Log-likelihood plot
if (1 %in% which) {
    plot(x$lambda_estimates, x$loglik, type = "l",
         xlab = x$xlab, ylab = x$ylab,
         main = paste("Log-Likelihood estimation\nLambda=", x$lambda))
    abline(h = x$conf_limit, lty = 2)
    abline(v = x$lambda, lty = 3)
}


  # 2: Histograms
if (2 %in% which) {
    # Set up a 2x2 plotting layout with adjusted margins mar = c(bottom, left, top, right)
    par(mfrow = c(1, 2),
        mar = c(3, 2.8, 4, 0.6), # Default is mar = c(5.1, 4.1, 4.1, 2.1)
        oma = c(0, 0, 2, 0),
        mgp = c(1.7, .5, 0)  # Default is par(mgp = c(3, 1, 0))
        )
    f_hist(x$original_data,
           main = paste0(main = "Original data: ", x$data_name),
           cex.main = 0.9, cex.lab = 0.8, cex.axis = 0.8)
    f_hist(x$transformed_data,
           main = paste0("Boxcox transformed data\n Lambda= ", x$lambda),
           cex.main = 0.9, cex.lab = 0.8, cex.axis = 0.8)
    mtext("Histograms", outer = TRUE, cex = 1.3)
    par(mfrow = c(1,1))
    layout(1)  # Clear layout matrix
}
  # 3: QQplots
if (3 %in% which) {
  par(mfrow = c(1, 2),
      mar = c(3, 2.8, 4, 0.6), # Default is mar = c(5.1, 4.1, 4.1, 2.1)
      oma = c(0, 0, 2, 0),
      mgp = c(1.7, .5, 0)  # Default is par(mgp = c(3, 1, 0))
  )
    f_qqnorm(x$original_data, paste0(main = "Original data: ", x$data_name),
             cex.main = 0.9, cex.lab = 0.8, cex.axis = 0.8)
    f_qqnorm(x$transformed_data,
             main = paste0("Boxcox transformed data\n Lambda= ", x$lambda),
             cex.main = 0.9, cex.lab = 0.8, cex.axis = 0.8)
    mtext("Normal Q-Q Plot with 95% Confidence Bands", outer = TRUE, cex = 1.3)
    par(mfrow = c(1,1))
    layout(1)  # Clear layout matrix
 }
}

#' Predict method for f_boxcox objects
#'
#' Applies the fitted Box-Cox transformation to new data (forward transform),
#' or reverses it back to the original scale (inverse transform). This is
#' useful for transforming hypothesis test parameters (e.g., \code{mu}) to the
#' transformed scale, or for back-transforming confidence intervals to the
#' original scale.
#'
#' @param object An object of class \code{f_boxcox}, as returned by
#'   \code{\link{f_boxcox}}.
#' @param newdata A numeric vector of values to transform. For the forward
#'   transform (\code{inverse = FALSE}), all values must be strictly positive
#'   (Box-Cox requires \eqn{y > 0}). For the inverse transform
#'   (\code{inverse = TRUE}), values are assumed to be on the Box-Cox
#'   transformed scale.
#' @param inverse Logical. If \code{FALSE} (default), applies the forward
#'   Box-Cox transformation to \code{newdata} using the estimated \eqn{\lambda}
#'   from the original fit. If \code{TRUE}, reverses the transformation, mapping
#'   values from the Box-Cox scale back to the original scale. Default is \code{FALSE}.
#' @param ... Further arguments passed to or from other methods (currently
#'   unused).
#'
#' @return A numeric vector of the same length as \code{newdata}, containing
#'   either the forward-transformed or back-transformed values.
#'
#' @details
#' The forward transformation applies the standard Box-Cox formula:
#' \deqn{y(\lambda) = \begin{cases}
#'   \frac{y^\lambda - 1}{\lambda}, & \lambda \neq 0 \\
#'   \log(y),                       & \lambda = 0
#' \end{cases}}
#'
#' The inverse transformation reverses this process to recover the original scale:
#' \deqn{y = \begin{cases}
#'   (y(\lambda) \cdot \lambda + 1)^{1/\lambda}, & \lambda \neq 0 \\
#'   \exp(y(\lambda)),                           & \lambda = 0
#' \end{cases}}
#'
#' \strong{Note on inverse validity:} When \eqn{\lambda > 0}, not all
#' transformed-scale values have a valid inverse. If
#' \eqn{y(\lambda) \cdot \lambda + 1 \leq 0}, the result is undefined and
#' \code{NaN} is returned with a warning.
#'
#' @examples
#' # Assuming mtcars is available and f_boxcox is loaded
#' bc <- f_boxcox(mtcars$hp)
#'
#' # Forward: transform a hypothesis value (mu) to the Box-Cox scale
#' mu <- 100
#' mu_transformed <- predict(bc, newdata = mu)
#'
#' # Inverse: back-transform a confidence interval to the original scale
#' ci_transformed <- c(5.5, 6.8)
#' predict(bc, newdata = ci_transformed, inverse = TRUE)
#'
#' # Round-trip sanity check should return exactly mu (e.g., 100)
#' predict(bc, newdata = mu_transformed, inverse = TRUE)
#'
#' @seealso \code{\link{f_boxcox}}
#'
#' @method predict f_boxcox
#' @export
predict.f_boxcox <- function(object, newdata, inverse = FALSE, ...) {

  # The geometric mean scaling applied internally in f_boxcox
  # during lambda estimation in is not part of
  # the stored transformation and is therefore not applied here.


  if (!is.numeric(newdata)) {
    stop("'newdata' must be numeric.")
  }

  lambda <- object$lambda

  if (!inverse) {
    # --- Forward transform: original scale -> Box-Cox scale ---
    if (any(newdata <= 0, na.rm = TRUE)) {
      stop("Box-Cox requires all values > 0. Found non-positive values in 'newdata'.")
    }

    # Must match the exact logic of the f_boxcox output generation
    if (lambda != 0) {
      transformed <- (newdata ^ lambda - 1) / lambda
    } else {
      transformed <- log(newdata)
    }
    return(transformed)

  } else {
    # --- Inverse transform: Box-Cox scale -> original scale ---
    if (lambda != 0) {

      # Statistical safety check: Ensure predictions are within the valid Box-Cox domain
      domain_check <- newdata * lambda + 1
      if (any(domain_check <= 0, na.rm = TRUE)) {
        warning("Some 'newdata' values are outside the valid inverse Box-Cox domain and will return NaN.")
      }

      # suppressWarnings safely ignores the NaNs generated by the domain check,
      # as we have already warned the user above.
      back_transformed <- suppressWarnings((domain_check) ^ (1 / lambda))

    } else {
      back_transformed <- exp(newdata)
    }

    return(back_transformed)
  }
}

# =============================================================================
# test_f_boxcox.R
# Comprehensive testthat stress tests for f_boxcox()
#
# Run with: testthat::test_file("f_boxcox_stress_tests.R")
#           or devtools::test() from the package root
#
# Test organisation (each describe() block is independent):
#   1.  Return object structure
#   2.  Input types (vector, data.frame, invalid)
#   3.  Edge cases in data values (NAs, zeros, negatives, n<3, n>5000)
#   4.  output_type variations ("default", "console", "rmd")
#   5.  File generation ("pdf", "excel") via save_as
#   6.  save_as combinations (full path, filename only, directory only, wdir)
#   7.  Lambda / digits / range parameters
#   8.  Output field completeness and naming
#   9.  print() and plot() methods
#  10.  Combined option stress
#  11.  predict.f_boxcox method (forward, inverse, round-trip, validation)
#  12.  Session state restoration (par, options)
#
# Style notes:
#  * Only ASCII characters are used in this file (CRAN preference).
#    Unicode is avoided entirely; use \uxxxx escapes in source strings
#    only when the content itself requires a symbol.
#  * output_type = "default" is the silent mode (NOT "off" - that value
#    would be rejected by the parameter validator).
# =============================================================================
# ---------------------------------------------------------------------------
# Helpers used across multiple describe() blocks
# ---------------------------------------------------------------------------

# Right-skewed, strictly positive data that Box-Cox can normalise.
make_skewed <- function(seed = 42, n = 100) {
  set.seed(seed)
  rlnorm(n, meanlog = 0, sdlog = 1)
}

# Approximately normal positive data (shifted to avoid zeros/negatives).
make_normal_pos <- function(seed = 42, n = 100) {
  set.seed(seed)
  abs(rnorm(n, mean = 10, sd = 2)) + 1
}

# Skewed data with injected NAs (trailing, so index-based checks are easy).
make_skewed_with_na <- function(seed = 42, n = 100, n_na = 5) {
  set.seed(seed)
  c(rlnorm(n - n_na, meanlog = 0, sdlog = 1), rep(NA_real_, n_na))
}

# Silence console / markdown output during tests. `plots`,
# `output_type`, and `open_generated_files` are accepted as named
# wrapper arguments rather than forwarded through ..., otherwise R
# raises "formal argument matched by multiple actual arguments" when
# a caller overrides one of them (e.g. plots = TRUE).
quiet_boxcox <- function(x,
                         plots                = FALSE,
                         output_type          = "default",
                         open_generated_files = FALSE,
                         ...) {
  suppressMessages(
    utils::capture.output(
      res <- f_boxcox(x,
                      plots                = plots,
                      output_type          = output_type,
                      open_generated_files = open_generated_files,
                      ...),
      file = nullfile()
    )
  )
  res
}

# For tests that need a specific output_type (e.g. "console", "rmd", "pdf").
# Does not wrap in capture.output because nested capture.output can leave
# the function's own markdown capture empty. Same named-wrapper pattern
# as quiet_boxcox() to avoid double-argument matching.
quiet_boxcox_typed <- function(x,
                               output_type,
                               plots                = FALSE,
                               open_generated_files = FALSE,
                               ...) {
  suppressMessages(
    f_boxcox(x,
             plots                = plots,
             output_type          = output_type,
             open_generated_files = open_generated_files,
             ...)
  )
}

# =============================================================================
# 1. Return object structure
# =============================================================================
# f_boxcox return object structure
x   <- make_skewed()
  res <- quiet_boxcox(x)

test_that("f_boxcox return object structure: returns an object of class 'f_boxcox'", {
  skip_on_cran()
    expect_s3_class(res, "f_boxcox")
    expect_true(is.list(res))
})

test_that("f_boxcox return object structure: contains every mandatory top-level field", {
  skip_on_cran()
    mandatory <- c("transformed_data", "original_data", "lambda", "n",
                   "missing values", "xlab", "ylab", "alpha",
                   "lambda_estimates", "loglik", "conf_limit",
                   "Shapiro_original_data", "Shapiro_transformed_data",
                   "plots", "data_name", "geom_mean")
    for (k in mandatory) {
      expect_true(k %in% names(res),
                  label = paste("mandatory field present:", k))
    }
})

test_that("f_boxcox return object structure: lambda is a finite numeric scalar", {
  skip_on_cran()
    expect_type(res$lambda, "double")
    expect_length(res$lambda, 1L)
    expect_true(is.finite(res$lambda))
})

test_that("f_boxcox return object structure: lambda_estimates and loglik have the same length", {
  skip_on_cran()
    expect_equal(length(res$lambda_estimates), length(res$loglik))
})

test_that("f_boxcox return object structure: transformed_data length equals the input length (NAs preserved)", {
  skip_on_cran()
    expect_length(res$transformed_data, length(x))
})

test_that("f_boxcox return object structure: Shapiro_original_data is a data frame with p-value column", {
  skip_on_cran()
    expect_s3_class(res$Shapiro_original_data, "data.frame")
    expect_true("Shapiro.p.value0" %in% names(res$Shapiro_original_data))
})

test_that("f_boxcox return object structure: Shapiro_transformed_data is a data frame with lambda and p-value", {
  skip_on_cran()
    expect_s3_class(res$Shapiro_transformed_data, "data.frame")
    expect_true(all(c("lambda", "W", "Shapiro.p.value") %in%
                    names(res$Shapiro_transformed_data)))
})

test_that("f_boxcox return object structure: data_name reflects the input object name", {
  skip_on_cran()
    expect_type(res$data_name, "character")
    expect_true(nzchar(res$data_name))
})

# =============================================================================
# 2. Input types
# =============================================================================
# f_boxcox input types
test_that("f_boxcox input types: accepts a plain numeric vector", {
  skip_on_cran()
    expect_s3_class(quiet_boxcox(make_skewed()), "f_boxcox")
})

test_that("f_boxcox input types: accepts a single-column data.frame and preserves the column name", {
  skip_on_cran()
    df  <- data.frame(values = make_skewed())
    res <- quiet_boxcox(df)
    expect_s3_class(res, "f_boxcox")
    expect_equal(res$data_name, "values")
})

test_that("f_boxcox input types: errors on a multi-column data.frame (regression test for the dead check)", {
  skip_on_cran()
    df_multi <- data.frame(a = make_skewed(),
                           b = make_skewed(seed = 2))
    expect_error(quiet_boxcox(df_multi), "multiple columns")
})

test_that("f_boxcox input types: errors on non-numeric input", {
  skip_on_cran()
    expect_error(quiet_boxcox(c("a", "b", "c")))
})

test_that("f_boxcox input types: errors on a matrix input (not a vector, not a data.frame)", {
  skip_on_cran()
    expect_error(quiet_boxcox(matrix(1:10, ncol = 2)))
})

# =============================================================================
# 3. Edge cases in data values
# =============================================================================
# f_boxcox data-value edge cases
test_that("f_boxcox data-value edge cases: handles NAs: records missing count and preserves length", {
  skip_on_cran()
    x   <- make_skewed_with_na(n = 100, n_na = 5)
    res <- quiet_boxcox(x)
    expect_equal(res[["missing values"]], 5L)
    expect_length(res$transformed_data, length(x))
})

test_that("f_boxcox data-value edge cases: errors informatively on zeros", {
  skip_on_cran()
    expect_error(quiet_boxcox(c(0, 1, 2, 3, 4)),
                 "higher than zero")
})

test_that("f_boxcox data-value edge cases: errors informatively on negative values", {
  skip_on_cran()
    expect_error(quiet_boxcox(c(-1, 2, 3, 4, 5)),
                 "higher than zero")
})

test_that("f_boxcox data-value edge cases: handles small samples (n = 10)", {
  skip_on_cran()
    res <- quiet_boxcox(make_skewed(n = 10))
    expect_s3_class(res, "f_boxcox")
    expect_true(is.finite(res$lambda))
})

test_that("f_boxcox data-value edge cases: handles already-normal positive data without error", {
  skip_on_cran()
    res <- quiet_boxcox(make_normal_pos())
    expect_s3_class(res, "f_boxcox")
    # Shapiro on original should not reject strongly (p > 0.001).
    p0 <- res$Shapiro_original_data$Shapiro.p.value0
    expect_true(is.na(p0) || p0 > 0.001)
})

test_that("f_boxcox data-value edge cases: does not crash at n > 5000 (safe_shapiro skip branch)", {
  skip_on_cran()
    set.seed(11)
    big <- rlnorm(5001, meanlog = 0, sdlog = 1)
    expect_no_error(res <- quiet_boxcox(big))
    expect_s3_class(res, "f_boxcox")
    # Shapiro is intentionally skipped; p-values should be NA.
    expect_true(is.na(res$Shapiro_original_data$Shapiro.p.value0))
    expect_true(is.na(res$Shapiro_transformed_data$Shapiro.p.value))
    expect_gt(res$n, 5000L)
})

# =============================================================================
# 4. output_type variations
# =============================================================================
# f_boxcox output_type variations
x <- make_skewed()

test_that("f_boxcox output_type variations: output_type = 'default' is silent and returns the object", {
  skip_on_cran()
    res <- quiet_boxcox(x)
    expect_s3_class(res, "f_boxcox")
})

test_that("f_boxcox output_type variations: output_type = 'console' prints to stdout and returns the object", {
  skip_on_cran()
    # console mode prints via cat(); capture it instead of nullfile().
    out <- capture.output(
      res <- suppressMessages(
        f_boxcox(x, plots = FALSE, output_type = "console",
                 open_generated_files = FALSE)
      )
    )
    expect_gt(length(out), 0L)
    expect_s3_class(res, "f_boxcox")
})

test_that("f_boxcox output_type variations: output_type = 'rmd' populates the rmd field with a non-empty string", {
  skip_on_cran()
    res <- quiet_boxcox_typed(x, "rmd")
    expect_s3_class(res, "f_boxcox")
    expect_type(res$rmd, "character")
    expect_true(nzchar(res$rmd))
})

test_that("f_boxcox output_type variations: errors informatively on an invalid output_type", {
  skip_on_cran()
    expect_error(
      f_boxcox(x, output_type = "html"),
      "output_type|output format"
    )
})

test_that("f_boxcox output_type variations: errors informatively on the common mis-value 'off'", {
  skip_on_cran()
    # 'off' is not a supported value; this is a guard so a future drift
    # back to the old broken stress-test style will fail loudly.
    expect_error(
      f_boxcox(x, output_type = "off"),
      "output_type|output format"
    )
})

# =============================================================================
# 5. File generation (pdf, excel)
#    Word generation is skipped by default because it requires LibreOffice
#    or MS Word at render time on many CI machines.
# =============================================================================
# f_boxcox file generation
test_that("f_boxcox file generation: output_type = 'pdf' produces a real PDF via save_as", {
  skip_on_cran()
    skip_if_not_installed("rmarkdown")
    out_path <- file.path(tempdir(),
                          paste0("test_bc_", as.integer(Sys.time()), ".pdf"))
    res <- suppressMessages(
      f_boxcox(make_skewed(),
               output_type          = "pdf",
               save_as              = out_path,
               open_generated_files = FALSE)
    )
    expect_true(file.exists(out_path))
    # Reasonable lower bound on PDF size; empty renders are a few hundred bytes.
    expect_gt(file.info(out_path)$size, 1000)
    expect_s3_class(res, "f_boxcox")
    unlink(out_path)
})

test_that("f_boxcox file generation: output_type = 'pdf' with plots = TRUE still produces a PDF", {
  skip_on_cran()
    skip_if_not_installed("rmarkdown")
    out_path <- file.path(tempdir(),
                          paste0("test_bc_plots_", as.integer(Sys.time()), ".pdf"))
    res <- suppressMessages(
      f_boxcox(make_skewed(),
               output_type          = "pdf",
               plots                = TRUE,
               save_as              = out_path,
               open_generated_files = FALSE)
    )
    expect_true(file.exists(out_path))
    expect_s3_class(res, "f_boxcox")
    unlink(out_path)
})

test_that("f_boxcox file generation: output_type = 'excel' produces a real xlsx via save_as", {
  skip_on_cran()
    skip_if_not_installed("writexl")
    out_path <- file.path(tempdir(),
                          paste0("test_bc_", as.integer(Sys.time()), ".xlsx"))
    res <- suppressMessages(
      f_boxcox(make_skewed(),
               output_type          = "excel",
               save_as              = out_path,
               open_generated_files = FALSE)
    )
    expect_true(file.exists(out_path))
    expect_gt(file.info(out_path)$size, 100)
    unlink(out_path)
})

# =============================================================================
# 6. save_as combinations
# =============================================================================
# f_boxcox save_as behaviour
test_that("f_boxcox save_as behaviour: accepts a full path with .pdf extension", {
  skip_on_cran()
    skip_if_not_installed("rmarkdown")
    out_path <- file.path(tempdir(),
                          paste0("full_path_", as.integer(Sys.time()), ".pdf"))
    res <- suppressMessages(
      f_boxcox(make_skewed(),
               save_as              = out_path,
               open_generated_files = FALSE)
    )
    expect_true(file.exists(out_path))
    unlink(out_path)
})

test_that("f_boxcox save_as behaviour: filename only -> saved in tempdir()", {
  skip_on_cran()
    skip_if_not_installed("rmarkdown")
    stem     <- paste0("name_only_", as.integer(Sys.time()))
    expected <- file.path(tempdir(), paste0(stem, ".pdf"))
    res <- suppressMessages(
      f_boxcox(make_skewed(),
               output_type          = "pdf",
               save_as              = stem,
               open_generated_files = FALSE)
    )
    expect_true(file.exists(expected))
    unlink(expected)
})

test_that("f_boxcox save_as behaviour: save_as with trailing slash (directory only) uses default filename", {
  skip_on_cran()
    skip_if_not_installed("rmarkdown")
    out_dir <- paste0(tempdir(), "/")
    res <- suppressMessages(
      f_boxcox(make_skewed(),
               output_type          = "pdf",
               save_as              = out_dir,
               open_generated_files = FALSE)
    )
    expect_s3_class(res, "f_boxcox")
})

test_that("f_boxcox save_as behaviour: save_in_wdir = TRUE writes to the working directory", {
  skip_on_cran()
    skip_if_not_installed("rmarkdown")
    wd_before <- getwd()
    res <- suppressMessages(
      f_boxcox(make_skewed(),
               output_type          = "pdf",
               save_in_wdir         = TRUE,
               open_generated_files = FALSE)
    )
    files_in_wd <- list.files(wd_before,
                              pattern     = "BoxCox_output\\.pdf$",
                              full.names  = TRUE)
    expect_gt(length(files_in_wd), 0L)
    unlink(files_in_wd)
    expect_s3_class(res, "f_boxcox")
})

test_that("f_boxcox save_as behaviour: extension in save_as overrides output_type (e.g. .xlsx over 'pdf')", {
  skip_on_cran()
    skip_if_not_installed("writexl")
    out_path <- file.path(tempdir(),
                          paste0("ext_override_", as.integer(Sys.time()), ".xlsx"))
    res <- suppressMessages(
      f_boxcox(make_skewed(),
               output_type          = "pdf",   # overridden by .xlsx
               save_as              = out_path,
               open_generated_files = FALSE)
    )
    expect_true(file.exists(out_path))
    unlink(out_path)
})

# =============================================================================
# 7. Lambda / digits / range parameters
# =============================================================================
# f_boxcox lambda / digits / range
x <- make_skewed()

test_that("f_boxcox lambda / digits / range: digits = 1 (coarse) still returns a finite lambda", {
  skip_on_cran()
    res <- quiet_boxcox(x, digits = 1)
    expect_true(is.finite(res$lambda))
})

test_that("f_boxcox lambda / digits / range: digits = 4 (fine) still returns a finite lambda", {
  skip_on_cran()
    res <- quiet_boxcox(x, digits = 4)
    expect_true(is.finite(res$lambda))
})

test_that("f_boxcox lambda / digits / range: narrow range constrains the selected lambda", {
  skip_on_cran()
    res <- quiet_boxcox(x, range = c(-0.5, 0.5))
    expect_true(abs(res$lambda) <= 0.5 + 1e-6)
})

test_that("f_boxcox lambda / digits / range: negative-only range forces a negative lambda", {
  skip_on_cran()
    res <- quiet_boxcox(x, range = c(-2, -0.1))
    expect_true(res$lambda < 0)
})

test_that("f_boxcox lambda / digits / range: degenerate range with plots = FALSE does not error", {
  skip_on_cran()
    # seq(-0.001, 0.001, by = 0.1) gives a single lambda value; the
    # spline branch must not run in this case (regression guard for the
    # length-1 lambda fix).
    expect_no_error(
      res <- quiet_boxcox(x,
                          range  = c(-0.001, 0.001),
                          digits = 1)
    )
    expect_s3_class(res, "f_boxcox")
})

test_that("f_boxcox lambda / digits / range: degenerate range with plots = TRUE still does not error", {
  skip_on_cran()
    # Same regression guard but with plots enabled; spline() would have
    # crashed before the length-1 guard was added.
    png_path <- tempfile(fileext = ".png")
    png(png_path, width = 600, height = 400)
    on.exit({ dev.off(); unlink(png_path) }, add = TRUE)
    expect_no_error(
      quiet_boxcox(x,
                   range  = c(-0.001, 0.001),
                   digits = 1,
                   plots  = TRUE)
    )
})

test_that("f_boxcox lambda / digits / range: lambda near zero produces log-like transformation", {
  skip_on_cran()
    # With a narrow range around 0 and digits = 3, lambda ~ 0 and the
    # transformation should track log(y) closely.
    res <- quiet_boxcox(x,
                        range  = c(-0.01, 0.01),
                        digits = 3)
    diff <- max(abs(res$transformed_data - log(res$original_data)),
                na.rm = TRUE)
    expect_lt(diff, 0.05)
})

# =============================================================================
# 8. Output field completeness
# =============================================================================
# f_boxcox output completeness and naming
x   <- make_skewed()
  res <- quiet_boxcox(x)

test_that("f_boxcox output completeness and naming: lambda_estimates is the correct field (not lambda_est)", {
  skip_on_cran()
    # Regression guard: plot.f_boxcox references 'lambda_estimates'. A
    # rename to 'lambda_est' would silently break plotting. We test this
    # via names() rather than `res$lambda_est`, because `$` does partial
    # matching in R: `res$lambda_est` would match `res$lambda_estimates`
    # and falsely report that the wrong name is present.
    expect_true("lambda_estimates" %in% names(res))
    expect_false("lambda_est" %in% names(res))
})

test_that("f_boxcox output completeness and naming: alpha defaults to 0.05 and is stored as-is", {
  skip_on_cran()
    expect_equal(res$alpha, 0.05)
    res2 <- quiet_boxcox(x, alpha = 0.01)
    expect_equal(res2$alpha, 0.01)
})

test_that("f_boxcox output completeness and naming: geom_mean is a positive finite scalar", {
  skip_on_cran()
    expect_length(res$geom_mean, 1L)
    expect_true(is.finite(res$geom_mean))
    expect_gt(res$geom_mean, 0)
})

test_that("f_boxcox output completeness and naming: xlab and ylab defaults are present", {
  skip_on_cran()
    expect_false(is.null(res$xlab))
    expect_equal(res$ylab, "log-Likelihood")
})

test_that("f_boxcox output completeness and naming: conf_limit is strictly less than max loglik", {
  skip_on_cran()
    expect_lt(res$conf_limit, max(res$loglik))
})

# =============================================================================
# 9. print() and plot() methods
# =============================================================================
# f_boxcox print and plot methods
x   <- make_skewed()
  res <- quiet_boxcox(x)

test_that("f_boxcox print and plot methods: print.f_boxcox runs without error and produces output", {
  skip_on_cran()
    out <- capture.output(print(res))
    expect_gt(length(out), 0L)
})

test_that("f_boxcox print and plot methods: print.f_boxcox handles NA Shapiro p-values (n > 5000)", {
  skip_on_cran()
    set.seed(12)
    big  <- rlnorm(5001)
    res2 <- quiet_boxcox(big)
    # Must not crash even though p-values are NA.
    expect_no_error(capture.output(print(res2)))
})

# =============================================================================
# 10. Combined option stress
# =============================================================================
# f_boxcox combined options
x <- make_skewed()

test_that("f_boxcox combined options: plots = TRUE with output_type = 'default' draws but stays silent", {
  skip_on_cran()
    png_path <- tempfile(fileext = ".png")
    png(png_path, width = 800, height = 500)
    on.exit({ dev.off(); unlink(png_path) }, add = TRUE)
    res <- quiet_boxcox(x, plots = TRUE)
    expect_s3_class(res, "f_boxcox")
})

test_that("f_boxcox combined options: digits = 2 with range = c(-1, 1) keeps lambda within range", {
  skip_on_cran()
    res <- quiet_boxcox(x, digits = 2, range = c(-1, 1))
    expect_s3_class(res, "f_boxcox")
    expect_true(abs(res$lambda) <= 1 + 1e-6)
})

test_that("f_boxcox combined options: data.frame input with output_type = 'rmd' populates rmd field", {
  skip_on_cran()
    df  <- data.frame(values = make_skewed())
    res <- quiet_boxcox_typed(df, "rmd")
    expect_true(nzchar(res$rmd))
})

test_that("f_boxcox combined options: large n (>5000) with plots = FALSE stays silent and succeeds", {
  skip_on_cran()
    set.seed(13)
    big <- rlnorm(5001)
    expect_no_error(res <- quiet_boxcox(big, plots = FALSE))
    expect_s3_class(res, "f_boxcox")
})

# =============================================================================
# 11. predict.f_boxcox method
# =============================================================================
# predict.f_boxcox forward and inverse transforms
x   <- make_skewed()
  res <- quiet_boxcox(x)

test_that("predict.f_boxcox forward and inverse transforms: forward transform on a single value is numeric and finite", {
  skip_on_cran()
    y <- predict(res, newdata = 10)
    expect_type(y, "double")
    expect_length(y, 1L)
    expect_true(is.finite(y))
})

test_that("predict.f_boxcox forward and inverse transforms: forward transform is vectorised", {
  skip_on_cran()
    y <- predict(res, newdata = c(1, 5, 10, 100))
    expect_length(y, 4L)
    expect_true(all(is.finite(y)))
})

test_that("predict.f_boxcox forward and inverse transforms: forward transform rejects non-positive values", {
  skip_on_cran()
    expect_error(predict(res, newdata = c(1, 0, 2)), "> 0")
    expect_error(predict(res, newdata = c(1, -1, 2)), "> 0")
})

test_that("predict.f_boxcox forward and inverse transforms: forward transform rejects non-numeric input", {
  skip_on_cran()
    expect_error(predict(res, newdata = c("a", "b")), "numeric")
})

test_that("predict.f_boxcox forward and inverse transforms: inverse transform undoes the forward transform (round-trip)", {
  skip_on_cran()
    mu      <- 100
    mu_fwd  <- predict(res, newdata = mu)
    mu_back <- predict(res, newdata = mu_fwd, inverse = TRUE)
    expect_equal(mu_back, mu, tolerance = 1e-8)
})

test_that("predict.f_boxcox forward and inverse transforms: inverse transform with lambda = 0 is exp()", {
  skip_on_cran()
    # Force lambda to 0 with a tight range around 0.
    res0 <- quiet_boxcox(x, range = c(-0.001, 0.001), digits = 3)
    if (isTRUE(res0$lambda == 0)) {
      y    <- c(0.5, 1, 2)
      back <- predict(res0, newdata = y, inverse = TRUE)
      expect_equal(back, exp(y), tolerance = 1e-12)
    } else {
      succeed()
    }
})

test_that("predict.f_boxcox forward and inverse transforms: inverse transform warns on out-of-domain values when lambda > 0", {
  skip_on_cran()
    # Pick a positive-lambda fit for the warning path.
    res_pos <- quiet_boxcox(x, range = c(0.1, 2))
    if (res_pos$lambda > 0) {
      # y * lambda + 1 <= 0  means  y <= -1 / lambda
      bad <- -1 / res_pos$lambda - 1
      expect_warning(
        out <- predict(res_pos, newdata = bad, inverse = TRUE),
        "outside|domain|NaN"
      )
      expect_true(any(is.nan(out)))
    } else {
      succeed()
    }
})

# =============================================================================
# 12. Session state restoration
# =============================================================================
# f_boxcox restores par() and options() on exit
test_that("f_boxcox restores par() and options() on exit: par('mfrow') is not clobbered by a normal call", {
  skip_on_cran()
    op <- par(no.readonly = TRUE)
    on.exit(par(op), add = TRUE)

    mfrow_before <- par("mfrow")
    invisible(quiet_boxcox(make_skewed()))
    mfrow_after  <- par("mfrow")

    expect_equal(mfrow_after, mfrow_before)
})

test_that("f_boxcox restores par() and options() on exit: par('mfrow') is not clobbered by a plot = TRUE call", {
  skip_on_cran()
    op <- par(no.readonly = TRUE)
    on.exit(par(op), add = TRUE)

    png_path <- tempfile(fileext = ".png")
    png(png_path, width = 800, height = 500)
    on.exit({ dev.off(); unlink(png_path) }, add = TRUE)

    mfrow_before <- par("mfrow")
    invisible(quiet_boxcox(make_skewed(), plots = TRUE))
    mfrow_after  <- par("mfrow")

    expect_equal(mfrow_after, mfrow_before)
})

#' Identify Outliers within Groups using Tukey's Fences
#'
#' @description
#' `f_outliers()` scans numerical column(s) for outliers based on the Interquartile Range (IQR) method.
#' It can detect outliers across the entire dataset or within specified subgroups.
#'
#' It returns a dataframe containing only the outlier rows, preserving the original data structure
#' and adding a \code{row_id} column for traceability.
#'
#' @param x A data.frame or formula (dispatches to the right method).
#' @param formula A formula specifying the columns (right hand side) to be checked per subgroup(s) (left hand side).
#' More columns or groups can be added using \code{-} or \code{+} (e.g., \code{col1 + col2 ~ group1 + group2})
#' to do a sequential analysis for each column parameter.
#' @param data A  \code{vector}, \code{data.frame},  \code{data.table}, or  \code{tibble}.
#' @param columns The numerical columns to analyze if no formula is used. Can be entered
#' as a single character string (e.g., \code{"weight"}) or as a character vector \code{c("weight", "length"}).
#' @param group_vars A character vector specifying the grouping variables in \code{data}
#'   If \code{NULL} (default), the entire dataset is treated as one group (e.g., \code{c("species", "fertilizer")}).
#' @param id_var (Optional) A character string naming a user-specific ID columns (e.g., \code{"EmployeeID"}).
#'   If provided, this columns is placed at the start of the output for easy identification.
#' @param coef A number indicating the IQR multiplier. Default is \code{1.5} (standard Tukey fence).
#'   Common values:
#'   \itemize{
#'     \item \code{1.5}: Standard outliers (approximately +/- 2.7 SD in normal distribution).
#'     \item \code{3.0}: Extreme outliers.
#'   }
#' @param digits Integer. Number of decimal places for the R console output.
#'   Default is \code{2}. If \code{NULL}, no rounding is applied.
#'   (Note: This does not affect the raw numbers exported to Excel).
#' @param export_to_excel Logical. If \code{TRUE}, exports results to an 'Excel' file. Default \code{FALSE}.
#' @param digits_excel Integer. Number of decimal places for the Excel file cells. Default \code{NULL} (no rounding). Defining \code{digits_excel}, sets \code{export_to_excel = TRUE} when excel output is not intended use: \code{digits} instead.
#' @param allow_integer_decimal_mix Logical. If \code{TRUE}, intergers in columns with a mix of integers and non-integers are displayed without decimals. Default \code{FALSE}, meaning if there are one or more numbers with decimals the whole column contains the number of decimals set by \code{digits}.
#' @param save_as Character string. Custom path or filename for the Excel export.
#'   \itemize{
#'     \item If full path: Saves to that location.
#'     \item If filename only: Saves to \code{tempdir()} (unless \code{save_in_wdir = TRUE}).
#'     \item If directory: Saves as "dataname_summary.xlsx" in that directory.
#'   }
#' @param save_in_wdir Logical. If \code{TRUE}, saves to the current working directory. Default \code{FALSE}.
#' @param close_generated_files Logical. If \code{TRUE}, forces Excel to close before saving (Windows only). Default \code{FALSE}.
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param check_input Logical. If \code{TRUE}, performs validation checks on inputs. Default \code{TRUE}.
#' @param ... Further arguments forwarded to \code{f_outliers.data.frame}.
#'
#' @details
#' \strong{The Outlier Logic (Tukey's Method):}
#' An observation is flagged as an outlier if it falls outside the calculated fences:
#' \itemize{
#'   \item \strong{Lower Fence:} \eqn{Q1 - (coef \times IQR)}
#'   \item \strong{Upper Fence:} \eqn{Q3 + (coef \times IQR)}
#' }
#' Where \eqn{Q1} is the 25th percentile, \eqn{Q3} is the 75th percentile, and \eqn{IQR = Q3 - Q1}.
#'
#' \strong{Output Structure:}
#' The function returns a subset of the original \code{data}. It automatically adds a \code{row_id}
#' columns, which corresponds to the row number in the original dataframe. This ensures you can
#' strictly map the outliers back to the source data.
#'
#' @return A data.frame containing the identified outlier rows. Returns \code{NULL} (with a message)
#'   if no outliers are found.
#'
#' @seealso \code{\link{f_remove_outliers}} to remove the rows identified by this function.
#'
#' @examples
#' # --- Setup: Create Dummy Data ---
#' set.seed(42)
#' df <- data.frame(
#'   Team       = rep(c("A", "B"), each = 20),
#'   Department = rep(c("Sales", "IT"), each = 10, times = 2),
#'   Salary     = rnorm(40, mean = 50000, sd = 2000),
#'   Age        = rnorm(40, mean = 35, sd = 3),
#'   EmployeeID = paste0("E", sprintf("%03d", 1:40))
#' )
#'
#' # Inject outliers
#' df[2,  "Salary"] <- 57000   # Mild outlier (between 1.5 and 3.0 fence)
#' df[1,  "Salary"] <- 100000  # Extreme high
#' df[35, "Salary"] <- 1000    # Extreme low
#'
#' # --- Example 1: Basic detection (data.frame notation) ---
#' # Scan the entire dataset for Salary outliers (no grouping)
#' out <- f_outliers(df, columns = "Salary")
#' print(out)
#'
#' # --- Example 2: Basic detection (formula notation) ---
#' # Equivalent to Example 1 using the formula interface
#' # LHS = column(s) to scan, RHS = grouping variable(s)
#' out <- f_outliers(Salary ~ 1, data = df)
#' print(out)
#'
#' # --- Example 3: Grouped detection (both notations) ---
#' # Outliers are now evaluated *within* each Team separately,
#' # making detection sensitive to group-level distributions
#'
#' # data.frame notation:
#' out <- f_outliers(df, columns = "Salary", group_vars = "Team")
#'
#' # Formula notation (identical result):
#' out <- f_outliers(Salary ~ Team, data = df)
#' print(out)
#'
#' # --- Example 4: Multi-column + multi-group (formula notation) ---
#' # Scan both Salary and Age for outliers, grouped by Team and Department.
#' # Returns a named list: one data.frame per column scanned.
#' out <- f_outliers(Salary + Age ~ Team + Department, data = df)
#' print(out)            # prints both result tables
#' out$Salary            # access Salary outliers directly
#' out$Age               # access Age outliers directly
#'
#' # --- Example 5: Strict detection with a custom ID column ---
#' # coef = 3.0 flags only extreme outliers (the "far out" Tukey fence).
#' # id_var places EmployeeID first in the output for easy identification.
#'
#' # data.frame notation:
#' out <- f_outliers(df,
#'                   columns   = "Salary",
#'                   group_vars = "Team",
#'                   id_var    = "EmployeeID",
#'                   coef      = 3.0)
#'
#' # Formula notation (identical result):
#' out <- f_outliers(Salary ~ Team, data = df,
#'                   id_var = "EmployeeID",
#'                   coef   = 3.0)
#' print(out)
#'
#' # --- Example 6: Sensitivity comparison ---
#' # Compare how coef = 1.5 (standard) vs coef = 3.0 (extreme-only)
#' # affects the number of flagged rows.
#'
#' out_standard <- f_outliers(Salary ~ Team, data = df, coef = 1.5)
#' out_extreme  <- f_outliers(Salary ~ Team, data = df, coef = 3.0)
#'
#' nrow(out_standard$output_df)  # 3 -- catches mild + extreme outliers
#' nrow(out_extreme$output_df)   # 2 -- catches extreme outliers only
#'
#' # --- Example 7: Vector input ---
#' # Pass a column directly as a vector -- no data.frame needed.
#' # The column name is captured automatically from the call.
#'
#' out <- f_outliers(df$Salary)
#' print(out)
#'
#' # Works with coef and other parameters too
#' out <- f_outliers(df$Salary, coef = 3.0)
#' print(out)
#'
#' # Inline vectors fall back to the column name "value"
#' out <- f_outliers(c(1, 2, 3, 4, 5, 100))
#' print(out)
#'
#' @export
f_outliers <- function(x, ...) {
  mc <- match.call()
  if (missing(x)) {
    if (!is.null(mc$data)) {
      x_val <- eval(mc$data, envir = parent.frame())
      dots <- list(...)
      dots[["data"]] <- NULL
      return(do.call(f_outliers.data.frame, c(list(x = x_val), dots)))
    } else {
      stop("Argument 'x' (or 'data') is missing.")
    }
  }
  # Capture name here -- deparse(substitute()) is lost after dispatch
  if (is.numeric(x) || is.integer(x)) {
    raw_name <- deparse(substitute(x))
    # "df$Salary" -> "Salary", "my_vec" -> "my_vec"
    col_name <- gsub(".*\\$", "", raw_name)
    # Fallback for complex expressions like c(1, 2, 100)
    if (nchar(col_name) > 40 || grepl("[^a-zA-Z0-9_.]", col_name)) {
      col_name <- "value"
    }
    return(f_outliers.numeric(x, ..., internal_col_name = col_name))
  }
  UseMethod("f_outliers")
}

#' @export
#' @rdname f_outliers
f_outliers.numeric <- function(x, ...) {
  # Extract hidden internal argument -- not exposed in the help file
  data <- x
  dots <- list(...)
  col_name <- if ("internal_col_name" %in% names(dots)) {
    dots[["internal_col_name"]]
  } else {
    "value"
  }
  # Strip internal arg before passing to data.frame method
  dots[["internal_col_name"]] <- NULL

  df <- setNames(data.frame(data), col_name)
  do.call(f_outliers.data.frame,
          c(list(x = df, columns = col_name), dots))
}


# Integer piggybacks on numeric -- no duplication needed
#' @export
#' @rdname f_outliers
f_outliers.integer <- f_outliers.numeric


#' @export
#' @rdname f_outliers
f_outliers.formula <- function(formula, data, ...) {
  # formula is the formula (e.g., y ~ B0 + B1)

  if (length(formula) < 3)
    stop("Formula must be two-sided: columns ~ group_vars (e.g., Salary ~ Team)")

  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(formula) #use helper_check_lhs.R

  # Parse LHS (Response Variable)
  lhs_vars <- all.vars(formula[[2]])



  # Parse RHS (Grouping Variables)
  rhs_vars  <- all.vars(formula[[3]])
  if (length(rhs_vars) == 0)
    rhs_vars <- NULL

  # Capture the name of the data object passed to the formula method
  data_name_str <- deparse(substitute(data))

  if(length(data_name_str) > 1) data_name_str <- "data"


  # Call the data.frame method
  f_outliers.data.frame(x = data,
                        columns = lhs_vars,
                        group_vars = rhs_vars,
                        internal_data_name = data_name_str,
                        ...)
}

#' @export
#' @rdname f_outliers
f_outliers.data.frame <- function(x,
                                  columns,
                                  group_vars = NULL,
                                  id_var = NULL,
                                  coef = 1.5,
                                  digits = NULL,
                                  # File Options
                                  export_to_excel = FALSE,
                                  close_generated_files = FALSE,
                                  open_generated_files = interactive(),
                                  save_as = NULL,
                                  save_in_wdir = FALSE,
                                  check_input = TRUE,
                                  digits_excel = NULL,
                                  allow_integer_decimal_mix = FALSE,
                                  ...) {
  data <- x

  # Input Validation & Setup
  if (!is.data.frame(data))
    stop("Input must be a data.frame.")

  # Check if there is one or multiple columns to summarize
  if(length(columns) == 1){
    single_col = TRUE
  } else if(length(columns) > 1){
    single_col = FALSE
  }

  # Validate Group Vars
  if (!is.null(group_vars)) {
    if (!all(group_vars %in% names(data))) {
      missing <- group_vars[!group_vars %in% names(data)]
      stop(paste("Group columns not found:", paste(missing, collapse = ", ")))
    }
  }

  # Validate Column Vars
  # Add this after the group_vars validation block:
  missing_cols <- columns[!columns %in% names(data)]
  if (length(missing_cols) > 0)
    stop(paste("Columns not found in data:", paste(missing_cols, collapse = ", ")))

  non_numeric <- columns[!sapply(data[columns], is.numeric)]
  if (length(non_numeric) > 0)
    stop(paste("Non-numeric columns specified:", paste(non_numeric, collapse = ", ")))

  # Safe filename logic
  # Default safety fallback
  data_name <- "data"

  # Check for the hidden internal name (passed from formula method)
  dots <- list(...)
  if ("internal_data_name" %in% names(dots)) {
    data_name <- dots[["internal_data_name"]]
  } else {
    # If no internal name, try to grab it from 'data' using:
    try_name <- try(deparse(substitute(x)), silent = TRUE)
    if (!inherits(try_name, "try-error") && length(try_name) == 1 && nchar(try_name) < 50) {
      data_name <- try_name
    }
  }

  # Create a list to store all outputs in this function
  output_list <- list()
  output_excel<- list()

  # Preserve Original Row Number
  work_data <- data
  work_data$row_id <- seq_len(nrow(work_data))

  # Create Grouping Factor
  if (is.null(group_vars)) {
    # If no groups, treat whole data as one group
    split_data <- list(all_data = work_data)
  } else {
    # Combine multiple groups (e.g. Team.Year)
    group_factor <- interaction(work_data[group_vars], drop = TRUE, sep = ".")
    split_data <- split(work_data, group_factor)
  }



  # --- The Logic (Tukey's Fences) ---
  find_outliers <- function(df) {
    values <- df[[target_col]]

    # Calculate Stats
    q1 <- quantile(values, 0.25, na.rm = TRUE)
    q3 <- quantile(values, 0.75, na.rm = TRUE)
    iqr_value <- q3 - q1

    lower_fence <- q1 - (coef * iqr_value)
    upper_fence <- q3 + (coef * iqr_value)

    # Filter
    is_outlier <- values < lower_fence | values > upper_fence
    is_outlier[is.na(is_outlier)] <- FALSE

    return(df[is_outlier, ])
  }


  for (target_col in columns) {

    # --- Execution ---
    outlier_list <- lapply(split_data, find_outliers)
    result <- do.call(rbind, outlier_list)

    # Handle empty results
    if (is.null(result) || nrow(result) == 0) {
      message(paste0("No outliers found in column: '", target_col, "'."))
      next
    }

    # Sort
    if (is.null(id_var)){
    result <- result[order(result$row_id), ]
    } else {
    result <- result[order(result[[id_var]]), ]
    }

    # --- columns Ordering ---
    # Define desired order: ID -> row_id -> Value -> Groups -> Rest
    priority_wishlist <- unique(c(id_var, "row_id", target_col, group_vars))

    # Filter this list to include ONLY columns that actually exist in 'result'
    priority_cols <- intersect(priority_wishlist, names(result))

    # Identify all other columns that are not in the priority list
    remaining_cols <- setdiff(names(result), priority_cols)

    # Reorder columns safely
    # drop = FALSE ensures it remains a dataframe even if only 1 column is selected
    final_result <- result[, c(priority_cols, remaining_cols), drop = FALSE]

    # Reset row names
    rownames(final_result) <- NULL

    # --- FORMATTING ---

    # For backward compatability use the name "output_df" when only one column is produced.
    if (single_col) {
      output_list[["output_df"]] <- final_result
    } else {
      output_list[[target_col]] <- final_result
    }

    # Use separate rounding for excel or default to no rounding.
    if (!is.null(digits_excel) ) {
      output_excel[[target_col]] <- f_conditional_round(final_result,
                                                    digits = digits_excel,
                                                    allow_integer_decimal_mix = allow_integer_decimal_mix)
    } else {
      output_excel[[target_col]] <- final_result
    }

  }

  class(output_list) <- "f_outliers"

  # --- EXCEL EXPORT ---
  if (!is.null(save_as) ||
      !is.null(digits_excel) || save_in_wdir == TRUE) {
    export_to_excel = TRUE
  }

  if (export_to_excel) {

    if (close_generated_files)
      system("taskkill /im EXCEL.EXE /f")

    if (save_in_wdir)
      save_dir <- getwd()
    else
      save_dir <- tempdir()

    output_path <- get_save_path(
      save_as = save_as,
      default_name = paste(data_name, "outliers.xlsx", sep = "_"),
      default_dir = save_dir,
      file.ext = ".xlsx"
    )

    message(paste0("Saved output in: ", output_path))

    write_xlsx(output_excel, path = output_path)

    if (open_generated_files){
      f_open_file(output_path)
    }
  }

  attr(output_list, "digits") <- digits

  if (length(output_list) == 0) {
    message("No outliers found in any column.")
    return(NULL)
  } else {
  return(output_list)
  }
}

#' Print method for f_outliers objects
#'
#' Prints a formatted summary table to the console.
#'
#' @param x Object of class f_outliers.
#' @param col_width Integer. Max characters in header before line break. Default \code{6}.
#' @param table_width Integer or \code{NULL}. Characters after which table splits. Default \code{90}.
#' @param digits Integer. Number of decimal digits to use in formatting. Default is \code{3}.
#' @param allow_integer_decimal_mix Logical. If \code{TRUE}, each individual cell is evaluated: integer values are displayed without decimal places, and non-integer values are displayed with the specified number of decimal places, i.e. \code{digits}. Default is \code{FALSE}, when a column contains a mix of integers and decimal values, all values are displayed with the specified number of decimal places. Note: columns containing only integers are **always** displayed without decimal places, regardless of \code{allow_integer_decimal_mix}.
#' @param ... Additional arguments passed to \code{pander}.
#' @return Invisibly returns \code{1}.
#' @export
print.f_outliers <- function(x,
                            col_width = 6,
                            table_width = 90,
                            digits = 2,
                            allow_integer_decimal_mix = FALSE,
                            ...) {

  # Get digits from f_outliers()
  if (missing(digits) && !is.null(attr(x, "digits"))) {
    digits <- attr(x, "digits")
  }

  # Loop over each category (a, b, etc.)
  for (category in names(x)) {
    # Get the sublist for this category
    sublist <- x[[category]]

    # Round numbers if digits is not NULL
    if (!is.null(digits)) {
      sublist <- f_conditional_round(sublist, digits = digits, allow_integer_decimal_mix = allow_integer_decimal_mix)
    }

    f_pander(sublist, col_width = col_width, table_width = table_width)
  }
}

formula_extract_clean_vars <- function(formula) {
  process_expr <- function(expr) {
    if (is.name(expr)) {
      return(as.character(expr))
    } else if (is.call(expr)) {
      if (expr[[1]] == "$") {
        return(deparse1(expr))
      } else {
        # Process only arguments, skip operator itself
        return(unlist(lapply(expr[-1], process_expr)))
      }
    }
    character(0)
  }

  unique(c(
    process_expr(formula[[2]]),  # LHS
    process_expr(formula[[3]])   # RHS
  ))
}

#' Fit a linear mixed model with \code{lme4::lmer()} including assumption checks, diagnostics, R-squared and post hoc tests.
#'
#' Fits a linear mixed-effects model using \code{lme4::lmer()} (with p-values
#' from \code{lmerTest}) and produces a fully-formatted report containing
#' the fixed-effects table, random-effects variance components, model-fit
#' indices (AIC, BIC, logLik, marginal & conditional R\eqn{^2}), residual and
#' BLUP diagnostics, convergence / singular-fit warnings, and post hoc
#' comparisons (\code{emmeans}) on factor fixed effects. Results can be
#' returned to the console or written to 'pdf', 'Word' or 'Excel'.
#'
#' @param formula A two-sided formula passed to \code{lme4::lmer()}, e.g.
#'   \code{y ~ treatment + time + (1 | subject)} or
#'   \code{y ~ treatment * time + (1 + time | subject)}. The right-hand
#'   side must contain at least one random-effects term in the
#'   \code{(varying | grouping)} syntax. See \emph{Details} for a guide to
#'   reading the random-effects syntax in study-design terms.
#'
#'   More than one response variable can be supplied on the left-hand side
#'   using \code{+} (e.g. \code{y1 + y2 ~ treatment + (1 | subject)}).
#'   A separate model is then fit for each response, sharing the same
#'   right-hand side, and a multiple-testing warning is added to the
#'   report. See the \emph{Multiple Testing Across Response Variables}
#'   section.
#' @param data A data frame containing the variables in the model.
#' @param REML Logical. If \code{TRUE} (default), the model is fit with
#'   restricted maximum likelihood, which gives less biased variance
#'   component estimates and is the appropriate choice for inference on
#'   fixed effects with Kenward-Roger or Satterthwaite degrees of freedom.
#'   Set to \code{FALSE} only when comparing nested models that differ in
#'   their fixed-effects structure.
#' @param ddf Character. Method for computing denominator degrees of
#'   freedom for fixed-effects p-values. One of:
#'   \describe{
#'     \item{\code{"Satterthwaite"}}{(default) Fast and accurate for most
#'       designs. Provided by \code{lmerTest}.}
#'     \item{\code{"Kenward-Roger"}}{Considered the gold standard, especially
#'       for small samples and unbalanced designs. Slower, and requires the
#'       \code{pbkrtest} package.}
#'     \item{\code{"lme4"}}{No p-values; only t-statistics are reported.
#'       Equivalent to a plain \code{lme4::lmer()} summary.}
#'   }
#' @param alpha Numeric. Significance level for the fixed-effects table and
#'   the post hoc tests. Default is \code{0.05}.
#' @param adjust Character. Method used to adjust p-values for multiple
#'   pairwise comparisons in the post hoc step (passed to
#'   \code{emmeans::emmeans()}). One of \code{"sidak"} (default),
#'   \code{"tukey"}, \code{"bonferroni"}, \code{"fdr"}, \code{"none"}.
#' @param norm_plots Logical. If \code{TRUE} (default), diagnostic plots
#'   (residuals vs fitted, Q-Q of level-1 residuals, Q-Q of random-effect
#'   BLUPs, scale-location) are included in the output.
#' @param post_hoc Logical. If \code{TRUE} (default), runs
#'   \code{emmeans::emmeans()} pairwise comparisons \strong{only when}
#'   the linear mixed model finds a significant fixed-effect term
#'   (ANOVA p-value below \code{alpha}). The post hoc is performed on
#'   each significant factor fixed-effect term separately. Numeric
#'   covariates are skipped because their slope is already reported in
#'   the fixed-effects coefficient table; pairwise contrasts are not
#'   meaningful for a continuous predictor. If no fixed-effect term is
#'   significant, no post hoc is run.
#' @param intro_text Logical. If \code{TRUE} (default), prepends an
#'   explanation of LMM assumptions and the random-effects syntax linked
#'   to study design.
#' @param close_generated_files Logical. Closes any open Word or Excel
#'   files before writing. Cross-platform (Windows taskkill, macOS / Linux
#'   pkill). Default \code{FALSE}. \strong{WARNING:} save your work first.
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param output_type Character. Output format. One of:
#'   \itemize{
#'     \item \code{"default"}: returns the \code{f_lmer} object; auto-prints
#'       if unassigned.
#'     \item \code{"console"}: forces immediate printing.
#'     \item \code{"pdf"}, \code{"word"}, \code{"excel"}: writes a file.
#'     \item \code{"rmd"}: stores the raw markdown string in the returned
#'       object for embedding in an R Markdown chunk with
#'       \code{\{r, echo=FALSE, results='asis'\}}.
#'   }
#' @param save_as Character. Output file path. See \code{\link{f_aov}}
#'   for the resolution rules; default is
#'   \code{file.path(tempdir(), "<dataname>_lmer_output.<ext>")}.
#' @param save_in_wdir Logical. If \code{TRUE}, save in the working
#'   directory instead of \code{tempdir()}. Default \code{FALSE}.
#' @param ... Additional arguments forwarded to \code{\link[lmerTest]{lmer}}.
#'   The arguments \code{subset}, \code{na.action}, and \code{weights}
#'   are handled specially: when supplied, they are applied via
#'   \code{\link[stats]{model.frame}} \strong{once} before the
#'   per-response loop, so every response in a multi-response call is
#'   fitted on the identical row set. Other \code{lmer()} arguments
#'   (e.g. \code{control = lmerControl(...)}, \code{contrasts},
#'   \code{offset}) are forwarded unchanged on every fit.
#'
#' @return An object of class \code{f_lmer}: a named list containing the
#'   fitted \code{lmerModLmerTest} model, the ANOVA-style fixed-effects
#'   table, the variance components and ICC, the R\eqn{^2} values, the
#'   observed descriptives table (raw-data n, mean, sd, se, min, Q1,
#'   median, Q3, max grouped by the categorical fixed-effect predictors),
#'   post hoc results (if any), diagnostic plots, and convergence
#'   diagnostics. When more than one response variable is supplied on the
#'   left-hand side, these elements are nested one level deep under each
#'   response name, e.g. \code{out$y1$fixed_effects},
#'   \code{out$y2$fixed_effects}. When \code{output_type = "rmd"} the
#'   markdown string is stored in \code{$rmd}.
#'
#' @details
#' \strong{What is a linear mixed model?}\cr
#' A linear mixed model (LMM) extends ordinary regression / ANOVA by
#' allowing two kinds of effects:
#' \itemize{
#'   \item \strong{Fixed effects} - factors you actively manipulated or
#'     whose specific levels you care about (treatment, dose, time,
#'     genotype). Reported as estimates with confidence intervals.
#'   \item \strong{Random effects} - grouping structure that creates
#'     non-independence in your data but whose levels are a random sample
#'     from a larger population (subjects measured repeatedly, plots
#'     within fields, observers, batches). Reported as variance
#'     components.
#' }
#' Use an LMM whenever observations share something that makes them more
#' alike than two random observations from the dataset. Ignoring such
#' grouping (running a plain \code{aov} or \code{lm}) is
#' \strong{pseudoreplication}, i.e. treating non-independent observations
#' as if they were independent: standard errors shrink, p-values shrink,
#' false positives explode.
#'
#' \strong{Vocabulary.}\cr
#' Before going further, a few terms used throughout the report:
#' \itemize{
#'   \item \strong{Subject} - the experimental unit that is measured
#'     repeatedly (a person, animal, pot and plot, cell line); in
#'     \code{lme4} syntax it is the grouping factor on the right of
#'     the \code{|}, e.g. \code{(1 | subject)}.
#'   \item \strong{Within-subject factor} - a predictor whose levels
#'     vary within the same subject (time in a longitudinal study,
#'     treatment in a cross-over study).
#'   \item \strong{Between-subject factor} - a predictor whose levels
#'     vary across subjects but are constant within a subject (sex,
#'     genotype, treatment arm in a parallel-groups trial). Both
#'     within- and between-subject factors are \strong{fixed} effects.
#'   \item \strong{BLUP} - \emph{Best Linear Unbiased Predictor}. The
#'     model's estimate of the random-effect value for each subject
#'     (e.g. how much a particular subject deviates from the population
#'     intercept). BLUPs are checked for normality just like residuals.
#'   \item \strong{ICC} - \emph{intraclass correlation coefficient}.
#'     The share of total variance attributable to between-group
#'     differences. ICC = 0 means the grouping factor is irrelevant;
#'     ICC = 1 means observations within a group are identical.
#'   \item \strong{REML} - \emph{restricted maximum likelihood}. The
#'     default fitting method for variance components; gives less biased
#'     estimates than ordinary maximum likelihood.
#'   \item \strong{Satterthwaite / Kenward-Roger} - methods to approximate
#'     the denominator degrees of freedom for fixed-effect p-values,
#'     since there is no exact df in an LMM.
#' }
#'
#' \strong{Reading the \code{(1 | group)} syntax.}\cr
#' Every random-effects term has the form \code{( <varying> | <group> )}.
#' The bar reads as "varies by". The grouping factor on the right is what
#' creates the non-independence. The left side is what is allowed to differ
#' between groups. Common patterns:
#' \itemize{
#'   \item \code{(1 | subject)} - random intercept per subject (each
#'     subject has its own baseline). Repeated measures, longitudinal data.
#'   \item \code{(1 | field)} - randomised block design or multi-site
#'     trial; one intercept per block.
#'   \item \code{(1 | field/plot)} - \code{plot} nested in \code{field};
#'     equivalent to \code{(1|field) + (1|field:plot)}. Split-plot or
#'     hierarchical sampling.
#'   \item \code{(1 + time | subject)} - random intercept and random slope
#'     of \code{time} per subject. Subjects differ both in baseline and
#'     in how fast they change. Growth curves.
#'   \item \code{(1 | subject) + (1 | observer)} - crossed random
#'     effects: every observer can rate every subject. Inter-rater designs.
#' }
#' \strong{Rule of thumb:} if you can answer "if I duplicated this
#' experiment, would I draw new levels of this factor?" with \emph{yes},
#' it belongs on the right of a \code{|}. If you would re-use the exact
#' same levels (e.g. control vs treated) it is a fixed effect.
#'
#' \strong{When to use a linear mixed model.}\cr
#' The most common reason is a \strong{repeated-measures design}, in
#' which the same experimental units are measured on more than one
#' occasion or under more than one treatment. Compared with a
#' between-groups design analysed by plain ANOVA this gives two real
#' advantages: fewer experimental units are needed (each subject acts
#' as its own control, removing between-subject variation from the
#' comparison) and individual differences cannot bias the treatment
#' groups (in a cross-over design every subject receives every
#' treatment). Two canonical examples:
#' \itemize{
#'   \item \strong{Longitudinal study} - same subjects measured at
#'     several time points: \code{y ~ time + (1 | subject)}. If
#'     subjects also differ in how fast they change, add a random
#'     slope: \code{y ~ time + (1 + time | subject)}.
#'   \item \strong{Cross-over design} - every subject receives every
#'     treatment in sequence: \code{y ~ treatment + (1 | subject)}. If
#'     carry-over between periods is a concern, add \code{period} as
#'     a fixed effect.
#' }
#' LMMs also apply to non-repeated structures that still create
#' non-independence: randomised block designs, split-plot trials,
#' multi-site studies, inter-rater designs.
#'
#' \strong{Assumptions of a linear mixed model:}
#' \enumerate{
#'   \item Linearity in the parameters of the fixed-effects part.
#'   \item Independence of observations \emph{conditional on} the random
#'     effects. If structure remains (e.g. temporal autocorrelation), more
#'     random effects or a correlation structure are needed.
#'   \item Normality of level-1 residuals (Q-Q plot of \code{residuals(m)}).
#'   \item Normality of the random-effect BLUPs
#'     (Q-Q plot of \code{ranef(m)}). \strong{This is the assumption most
#'     users forget.}
#'   \item Homoscedasticity: residual variance roughly constant across
#'     fitted values and across grouping levels.
#'   \item At least \code{~5} levels of each grouping factor; with
#'     \code{3-4} levels it is usually better to treat the factor as
#'     fixed.
#' }
#' If Levene's test or the Shapiro-Wilk tests on residuals or BLUPs
#' indicate a violation, the report adds a \emph{Recommendations for
#' Heteroscedasticity and/or non-normal residuals} section after the
#' diagnostics with concrete next steps (generalised mixed model,
#' transformation).
#'
#' \strong{Convergence and singular fits.}\cr
#' \code{f_lmer} surfaces \code{lme4} convergence warnings and the
#' "boundary (singular) fit" message prominently in the output. A singular
#' fit usually means the random-effects structure is too complex for the
#' data (often a random slope with too few levels) - simplify the model
#' before interpreting results.
#'
#' This function requires Pandoc (>= 1.12.3) for \code{pdf}, \code{word}
#' and \code{rmd} output. See \code{\link{f_aov}} for installation notes.
#'
#' @section Multiple Testing Across Response Variables:
#' When several response variables are analysed in a single call
#' (e.g. \code{y1 + y2 + y3 ~ treatment + (1 | subject)}), each linear
#' mixed model is an independent null-hypothesis test at level
#' \code{alpha}. The post hoc adjustments (\code{adjust = "sidak"},
#' \code{"tukey"}, etc.) only control the family-wise error rate
#' \strong{within} one model (across pairwise contrasts for that
#' response). They do \strong{not} protect against the inflation of
#' Type I error \strong{across} the set of responses.
#'
#' \strong{Practical implication:} With \eqn{k} independent response
#' variables all tested at \eqn{\alpha = 0.05}, the probability of
#' obtaining at least one false positive is
#' \eqn{1 - (1 - 0.05)^k}, which reaches ~40\% for \eqn{k = 10}.
#'
#' \strong{When this matters:} The risk is highest in exploratory studies
#' where many responses are screened simultaneously without a clear
#' a priori hypothesis for each one. It is less of a concern when each
#' response is a pre-specified primary outcome with its own biological
#' rationale.
#'
#' \strong{Possible remedies:}
#' \itemize{
#'   \item \strong{Bonferroni correction across responses:} use
#'     \code{alpha = 0.05 / k} where \code{k} is the number of response
#'     variables. Conservative but simple.
#'   \item \strong{False Discovery Rate (FDR):} apply
#'     \code{p.adjust(p_values, method = "fdr")} to the vector of
#'     per-response fixed-effect p-values after the fact.
#'   \item \strong{Multivariate model:} if the responses are correlated
#'     and you want a single omnibus test, fit a joint multivariate
#'     mixed model (e.g. \code{MCMCglmm}, \code{brms}) before
#'     interpreting individual responses.
#'   \item \strong{Pre-registration:} declare primary vs. exploratory
#'     responses before data collection to justify differential
#'     correction thresholds.
#' }
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # sleepstudy: reaction time vs days of sleep deprivation,
#' # repeated measures within Subject (ships with lme4).
#' data(sleepstudy, package = "lme4")
#'
#' # 1) Random intercept per subject - the simplest mixed model.
#' #    Each subject has its own baseline reaction time; the fixed
#' #    effect of Days is the average slope across subjects.
#' #    With output_type = "default" (the default), the result auto-
#' #    prints if not assigned, so no print() call is needed.
#' f_lmer_out <- f_lmer(Reaction ~ Days + (1 | Subject),
#'                      data = sleepstudy)
#'
#' # Re-print the stored result and show the diagnostic plots.
#' print(f_lmer_out)
#' plot(f_lmer_out)
#'
#' # 2) Random intercept AND random slope of Days per subject,
#' #    fitted with Kenward-Roger denominator df, saved to MS Word.
#' f_lmer(Reaction ~ Days + (1 + Days | Subject),
#'        data = sleepstudy,
#'        ddf  = "Kenward-Roger",
#'        output_type = "word"
#'        )
#'
#' # 3) A factor fixed effect triggers a post hoc test.
#' #    Bin Days into three sleep-deprivation phases so that the
#' #    fixed effect is categorical and emmeans pairwise comparisons
#' #    with a compact letter display are produced automatically.
#' sleepstudy$Phase <- cut(sleepstudy$Days,
#'                         breaks = c(-Inf, 2, 6, Inf),
#'                         labels = c("early", "mid", "late"))
#' f_lmer(Reaction ~ Phase + (1 | Subject),
#'        data = sleepstudy,
#'        adjust = "tukey")
#'
#' # 4) A minimal report: suppress the intro text and the diagnostic
#' #    plots, and save it directly to MS Word. Useful when embedding
#' #    many models in one document or when you only need the tables.
#' f_lmer(Reaction ~ Days + (1 | Subject),
#'        data = sleepstudy,
#'        intro_text = FALSE,
#'        norm_plots = FALSE,
#'        output_type = "word"
#'        )
#'
#' # 5) Get the raw markdown back for embedding in an R Markdown
#' #    document. Use it inside a chunk with results = 'asis'.
#' f_lmer_rmd_out <- f_lmer(Reaction ~ Days + (1 | Subject),
#'                          data = sleepstudy,
#'                          output_type = "rmd")
#' cat(f_lmer_rmd_out$rmd)
#'
#' # 6) Two response variables analysed in one call. A separate model
#' #    is fit for each, sharing the same right-hand side. The results
#' #    are nested under each response name.
#' sleepstudy$Reaction2 <- sleepstudy$Reaction + rnorm(nrow(sleepstudy), 0, 5)
#' multi_out <- f_lmer(Reaction + Reaction2 ~ Days + (1 | Subject),
#'                     data = sleepstudy,
#'                     intro_text = FALSE,
#'                     norm_plots = FALSE)
#' multi_out$Reaction$fixed_effects
#' multi_out$Reaction2$fixed_effects
#'
#' @export
f_lmer <- function(
    formula,
    data = NULL,
    REML = TRUE,
    ddf = "Satterthwaite",
    alpha = 0.05,
    adjust = "sidak",
    norm_plots = TRUE,
    post_hoc = TRUE,
    intro_text = TRUE,
    output_type = "default",
    save_as = NULL,
    save_in_wdir = FALSE,
    close_generated_files = FALSE,
    open_generated_files = interactive(),
    ...
    # Additional arguments forwarded to lme4::lmer() / lmerTest::lmer().
    # subset, na.action and weights are baked into a single master data
    # frame (built once before the response loop) so every response is
    # fitted on the identical row set. Other arguments such as
    # `control` or `contrasts` are forwarded unchanged.
) {

  ########## Reset initial settings on exit #################################
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state

  ########## Required packages ##############################################
  if (!requireNamespace("lme4", quietly = TRUE)) {
    stop("Package 'lme4' is required for f_lmer(). ",
         "Please run: install.packages('lme4')")
  }
  if (!requireNamespace("lmerTest", quietly = TRUE)) {
    stop("Package 'lmerTest' is required for f_lmer(). ",
         "Please run: install.packages('lmerTest')")
  }

  ########## Resolve data and formula like the rest of the package ##########
  if (!is.null(data)) {
    data_name <- deparse(substitute(data))
  } else {
    if (length(formula_extract_df_names(formula)) == 0) {
      data_name <- "data"
    } else if (length(formula_extract_df_names(formula)) == 1) {
      data_name <- formula_extract_df_names(formula)
    } else {
      data_name <- paste(formula_extract_df_names(formula), collapse = "_")
    }
    data <- formula_to_dataframe(formula)
    formula <- clean_formula(formula)
  }

  ########## Validate that the formula actually contains a random effect ####
  rhs_chars <- paste(deparse(formula[[3]]), collapse = " ")
  if (!grepl("\\|", rhs_chars)) {
    stop("f_lmer() requires at least one random-effects term in the formula, ",
         "e.g. 'y ~ x + (1 | subject)'. ",
         "For a model without random effects use f_aov() or f_glm().")
  }

  ########## Validate scalar arguments ######################################
  output_type <- match.arg(output_type,
                           choices = c("default", "console", "pdf",
                                       "word", "excel", "rmd"))
  ddf <- match.arg(ddf,
                   choices = c("Satterthwaite", "Kenward-Roger", "lme4"))
  adjust <- tolower(adjust)
  adjust <- match.arg(adjust,
                      choices = c("sidak", "tukey", "bonferroni",
                                  "fdr", "none"))

  if (!is.numeric(alpha) || length(alpha) != 1 || alpha <= 0 || alpha >= 1) {
    stop("'alpha' must be a single numeric value strictly between 0 and 1.")
  }

  if (ddf == "Kenward-Roger" && !requireNamespace("pbkrtest", quietly = TRUE)) {
    warning("ddf = 'Kenward-Roger' requires the 'pbkrtest' package. ",
            "Falling back to ddf = 'Satterthwaite'. ",
            "Run install.packages('pbkrtest') to enable Kenward-Roger.")
    ddf <- "Satterthwaite"
  }

  ########## Variable presence checks #######################################
  all_vars <- all.vars(formula)
  missing_vars <- setdiff(all_vars, names(data))
  if (length(missing_vars) > 0) {
    stop("The following variable(s) are not in the data: ",
         paste(missing_vars, collapse = ", "))
  }


  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(formula) #use helper_check_lhs.R

  # Extract response variables from the LHS of the formula. f_lmer now
  # supports multiple responses (y1 + y2 ~ ...), mirroring f_aov(). One
  # lmer() fit + report block is produced per response, sharing the same
  # RHS (fixed + random-effects) structure.
  lhs            <- all.vars(formula[[2]])
  response_names <- lhs



  if (length(lhs) == 0) {
    stop("f_lmer() requires at least one response variable on the ",
         "left-hand side of the formula.")
  }
  for (response in response_names) {
    if (!(response %in% names(data))) {
      stop("Response variable '", response, "' is not in the data.")
    }
    if (!is.numeric(data[[response]])) {
      stop("Response variable '", response,
           "' must be numeric for a linear mixed model. ",
           "For binary / count / proportion responses use lme4::glmer() ",
           "or glmmTMB::glmmTMB() directly.")
    }
  }

  # Cache the RHS as a string so we can rebuild a per-response formula
  # inside the loop. paste(deparse(...), collapse = " ") protects against
  # long random-effects expressions that deparse() may split over lines.
  rhs <- paste(deparse(formula[[3]]), collapse = " ")

  ########## Build master analysis frame (one row set, all responses) #######
  # Use a one-sided formula over ALL variables (responses + fixed
  # predictors + random-effect grouping variables, harvested via
  # all.vars()) so base model.frame() doesn't choke on the `|` of the
  # Build the analysis data set ONCE, before the response loop.
  # A single-sided formula listing only the non-bar-separated variables
  # (fixed effects + response + grouping factors) is used for
  # model.frame; lme4 handles random-effects terms via the original
  # random-effects bar. This guarantees every response in a multi-
  # response call is fitted on the identical row set, and bakes in any
  # subset / na.action / weights passed via `...`.
  #
  # Strategy: pre-evaluate subset/weights eagerly against data first,
  # falling back to the user's calling environment. Then apply the
  # filter manually and pass plain values to stats::model.frame via
  # do.call. This avoids the fragile match.call/substitute dance where
  # spliced expressions inside a constructed call can end up containing
  # `..N` dots-index symbols that crash model.frame with "the ... list
  # contains fewer than 3 elements".
  master_formula <- stats::as.formula(
    paste("~", paste(all_vars, collapse = " + "))
  )

  mc          <- match.call(expand.dots = FALSE)
  dots_exprs  <- as.list(mc[["..."]])
  caller_env  <- parent.frame()

  # Resolve subset expression against data columns + caller env
  subset_vec <- NULL
  if (!is.null(dots_exprs$subset)) {
    subset_vec <- eval(dots_exprs$subset, envir = data, enclos = caller_env)
    if (is.logical(subset_vec)) subset_vec[is.na(subset_vec)] <- FALSE
  }

  # Resolve weights against FULL data before subsetting
  weights_vec <- NULL
  if (!is.null(dots_exprs$weights)) {
    weights_vec <- eval(dots_exprs$weights, envir = data, enclos = caller_env)
  }

  # Apply the manual subset filter to data (and weights, if its length
  # matches the pre-subset row count)
  if (!is.null(subset_vec)) {
    if (!is.null(weights_vec) && length(weights_vec) == nrow(data)) {
      weights_vec <- weights_vec[subset_vec]
    }
    data <- data[subset_vec, , drop = FALSE]
  }

  # Resolve na.action (a function, defaults to na.omit)
  na_action_fn <- stats::na.omit
  if (!is.null(dots_exprs$na.action)) {
    na_action_fn <- eval(dots_exprs$na.action, envir = caller_env)
  }

  mf_args <- list(
    formula            = master_formula,
    data               = data,
    drop.unused.levels = TRUE,
    na.action          = na_action_fn
  )
  if (!is.null(weights_vec)) mf_args$weights <- weights_vec

  data <- do.call(stats::model.frame, mf_args)

  # Extract the weights vector once (NULL if none supplied) for the
  # lmer() calls below.
  mf_weights <- stats::model.weights(data)

  # Remaining ... args (control, contrasts, offset, ...) are evaluated
  # in the caller's frame  these are normal R objects, not column
  # references, so eager evaluation is safe and expected.
  extra_names <- setdiff(names(dots_exprs),
                         c("subset", "weights", "na.action"))
  lmer_extra  <- lapply(dots_exprs[extra_names], eval, envir = caller_env)

  ########## Save-path machinery (mirrors f_kruskal_test) ###################
  temp_output_dir  <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")
  file.create(temp_output_file)
  file_extension <- NULL

  f_wrap_lines()

  if (save_in_wdir == TRUE) {
    save_dir <- getwd()
  } else {
    save_dir <- tempdir()
  }

  output_type_map <- c("pdf"  = ".pdf",
                       "word" = ".docx",
                       "excel" = ".xlsx",
                       "rmd"  = ".rmd")

  if (!is.null(save_as) || save_in_wdir == TRUE) {
    if (!is.null(save_as)) {
      save_as <- gsub(pattern = "\\\\", replacement = "/", x = save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if (file_extension_save_as[1] != FALSE) {
        file_extension <- file_extension_save_as
      }
    }

    if (is.null(file_extension) && output_type %in% c("console", "default")) {
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name,
                                                        "lmer_output", sep = "_"),
                                   default_dir  = save_dir,
                                   file.ext     = ".pdf")
      output_type <- "pdf"
    } else if (is.null(file_extension) &&
               output_type %in% c("pdf", "word", "excel", "rmd")) {
      file.ext <- unname(output_type_map[output_type])
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name,
                                                        "lmer_output", sep = "_"),
                                   default_dir  = save_dir,
                                   file.ext     = file.ext)
    } else if (!is.null(file_extension)) {
      output_path <- get_save_path(save_as = save_as,
                                   default_name = paste(data_name,
                                                        "lmer_output", sep = "_"),
                                   default_dir  = save_dir,
                                   file.ext     = file_extension[1])
      output_type <- file_extension[2]
    }
  } else {
    file.ext <- unname(output_type_map[output_type])
    output_path <- get_save_path(save_as = save_as,
                                 default_name = paste(data_name,
                                                      "lmer_output", sep = "_"),
                                 default_dir  = save_dir,
                                 file.ext     = file.ext)
  }

  if (output_type == "rmd") close_generated_files <- FALSE

  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    if (output_type == "excel") close_app("EXCEL.EXE",   "Microsoft Excel", "soffice")
  }

  ########## Output accumulator #############################################
  output_list <- list()

  ########## Helper: format a p-value #######################################
  fmt_p <- function(p) {
    if (is.na(p)) return("NA")
    if (p < 0.001) "< 0.001" else format(round(p, 4), nsmall = 4)
  }

  ###########################################################################
  ## generate_report() - all side-effecting cat() calls live in here       ##
  ###########################################################################
  generate_report <- function(output = TRUE) {

    ## ---------- Intro / assumptions block --------------------------------
    if (intro_text == TRUE) {
      cat("
# Linear Mixed Models - what they are and when to use them

A linear mixed model (LMM) extends ordinary regression / ANOVA by allowing
**two kinds of effects**:

- **Fixed effects** - factors you actively manipulated or whose specific
  levels you care about (treatment, dose, time, genotype). You estimate them
  with a coefficient and a confidence interval.
- **Random effects** - grouping structure in your data that creates
  non-independence (subjects measured repeatedly, plots within fields, blocks,
  observers, batches, days). You treat the levels as a random sample from
  a larger population and you care about the *variance* they introduce,
  not the individual levels.

Use an LMM whenever observations are **not independent** - whenever some
observations share something that makes them more alike than two random
observations from the dataset. Ignoring this and running a plain `aov`
or `lm` instead is **pseudoreplication** (treating non-independent
observations as if they were independent): standard errors shrink,
p-values shrink, false positives increase.

# Vocabulary

A few terms used throughout this report:

- **Subject** - the experimental unit that is measured repeatedly
  (a person, animal, plot, pot, cell line). In `lme4` syntax this
  is the grouping factor on the right of the `|`: `(1 | subject)`.
- **Within-subject factor** - a predictor whose levels vary *within*
  the same subject. In a longitudinal study this is **time**. In a
  cross-over study it is the **treatment** that every subject receives
  in turn (e.g. diet A, B and C).
- **Between-subject factor** - a predictor whose levels vary *across*
  subjects but are constant within a subject (sex, genotype, treatment
  arm in a parallel-groups trial). Both within- and between-subject
  factors are **fixed** effects in the model.
- **BLUP** - *Best Linear Unbiased Predictor*. The model's estimate of
  the random-effect value for each subject (e.g. how much a particular
  subject deviates from the population intercept). BLUPs are checked
  for normality just like residuals.
- **ICC** - *intraclass correlation coefficient*. The share of total
  variance attributable to between-group differences, on a scale of
  0 to 1. ICC = 0 means the grouping factor is irrelevant;
  ICC = 1 means observations within a group are identical.
- **REML** - *restricted maximum likelihood*. The default fitting
  method for variance components; gives less biased estimates than
  ordinary maximum likelihood.
- **Satterthwaite / Kenward-Roger** - methods to approximate the
  denominator degrees of freedom for fixed-effect p-values, since
  there is no exact df in an LMM.

# Reading the `(1 | group)` syntax

The random-effects part of an `lme4` formula always has the form
`( <varying part> | <grouping factor> )`. The bar `|` reads as
*\"varies by\"*. The grouping factor on the right is what creates the
non-independence. The left side is what you let differ between groups.

+-----------------------------------+----------------------------------------------------------------------------------------+----------------------------------------------+
| Formula term                      | What it means                                                                          | Typical study design                         |
+===================================+========================================================================================+==============================================+
| `(1 | subject)`                   | Random **intercept** per subject. Each subject has its own baseline.                   | Repeated measures; longitudinal data.        |
+-----------------------------------+----------------------------------------------------------------------------------------+----------------------------------------------+
| `(1 | field)`                     | Random intercept per field / plot / block.                                             | Randomised block design; multi-site trial.   |
+-----------------------------------+----------------------------------------------------------------------------------------+----------------------------------------------+
| `(1 | field/plot)`                | `plot` **nested** in `field`. Equivalent to `(1|field) + (1|field:plot)`.              | Split-plot; hierarchical sampling.           |
+-----------------------------------+----------------------------------------------------------------------------------------+----------------------------------------------+
| `(1 + time | subject)`            | Random intercept **and** random slope of `time` per subject. Subjects differ in        | Growth curves with individual trajectories.  |
|                                   | baseline *and* trajectory.                                                             |                                              |
+-----------------------------------+----------------------------------------------------------------------------------------+----------------------------------------------+
| `(1 | subject) + (1 | observer)`  | **Crossed** random effects: every observer can rate every subject.                     | Inter-rater designs; items x raters.         |
+-----------------------------------+----------------------------------------------------------------------------------------+----------------------------------------------+

 **Rule of thumb:** ask yourself *\"if I ran this experiment again, would I get the same group labels, or would I get different ones?\"*
\n- If the labels would be **the same** (e.g. *control* vs *treated*, *male* vs *female*, *low* vs *high* dose) , the levels are the ones you specifically chose and care about; it is a **fixed effect**.
\n- If the labels would likely be **different** (e.g. a new set of subject IDs, different fields, blocks, different observers, different batches), the actual levels are interchangeable and you only care about the variation they introduce; it belongs on the right of a `|` as a **random effect**.


# When to use a Linear Mixed Model

The most common reason is a **repeated-measures design** in which the
same experimental units (people, animals, pots and plots) are measured
on more than one occasion or under more than one treatment. Compared
with a between-groups design analysed by plain ANOVA, this gives two
real advantages: fewer experimental units are needed (each subject acts
as its own control, removing between-subject variation from the
comparison of interest), and individual differences cannot bias the
treatment groups (in a cross-over design every subject receives every
treatment). The cost is that observations from the same subject are
**not independent**, which is exactly the problem an LMM solves by
entering the subject as a random effect (the *\"random intercept model\"*).

Two canonical examples:

- **Longitudinal study:** same subjects measured at several time
  points: `y ~ time + (1 | subject)`. If subjects also differ in *how
  fast* they change, add a random slope:
  `y ~ time + (1 + time | subject)`.
- **Cross-over design:** every subject receives every treatment in
  sequence (typically with a wash-out period in between):
  `y ~ treatment + (1 | subject)`. If carry-over between periods is a
  concern, add `period` as a fixed effect.

LMMs also apply to non-repeated structures that still create
non-independence: randomised block designs, split-plot trials,
multi-site studies, inter-rater designs.

# Assumptions of a Linear Mixed Model

1. **Linearity:** fixed effects are linear in the parameters.
2. **Independence conditional on the random effects** - once the grouping
   structure is accounted for, residuals should be independent. This is
   the whole point of including the random effects in the first place;
   if you still see structure (e.g. autocorrelation in time), you need
   more random effects or a correlation structure (`nlme`, `glmmTMB`).
3. **Normality of level-1 residuals:** checked with a Q-Q plot of
   `residuals(model)`.
4. **Normality of the random-effect BLUPs:** checked with a Q-Q plot of
   `ranef(model)`. This is *separate* from residual normality and is the
   assumption most users forget.
5. **Homoscedasticity:** residual variance is roughly constant across
   fitted values and across levels of the grouping factor (in the *Residuals
   vs Fitted* and *Scale-Location* plots, points should form a featureless
   horizontal band; no funnel, no curve).
6. **Enough levels of the grouping factor:** random-effect variance is
   poorly estimated with fewer than ~5 levels. With 3-4 levels you are
   often better off treating the factor as **fixed**.

If Levene's test or the Shapiro-Wilk tests on residuals or BLUPs
indicate a violation, the report adds a *Recommendations for
Heteroscedasticity and/or non-normal residuals* section after the
diagnostics with concrete next steps.
\n")

      if (output_type != "rmd") {
        cat("
<div style=\"page-break-after: always;\"></div>
\\newpage
")
      }
    }

    ## ---------- Multiple-response warning --------------------------------
    ## Mirrors f_aov(): only fires when > 1 response variable is supplied
    ## on the LHS. Shown regardless of intro_text so the user always sees
    ## it in this situation.
    if (length(lhs) > 1) {
      k          <- length(lhs)
      fwer_pct   <- round((1 - (1 - alpha)^k) * 100, 1)
      bonf_alpha <- round(alpha / k, 4)
      cat(paste0(
        "\n\n***\n\n",
        "**[!]  NOTE: Multiple Testing Across ", k, " Response Variables**  \n\n",
        "This report fits ", k, " independent linear mixed models on the same dataset. ",
        "The **", adjust, "** correction keeps each individual model honest, it guards against ",
        "false positives among the pairwise contrasts within a model, but it offers no protection ",
        " against the accumulation of error across all ", k, " tests combined. ",
        "\nAt \u03b1 = ", alpha, " per test, the probability of obtaining at least one ",
        "spurious significant result across all ", k, " responses is approximately ",
        "**", fwer_pct, "%** ($1-(1-", alpha, ")^{", k, "}$, assuming independence). ",
        "This risk is highest in exploratory studies; it is less of a concern when ",
        "each response has a clear a priori hypothesis.  \n\n",
        "**Possible remedies:**  \n",
        "\n-  **Bonferroni** (conservative): re-run with `alpha = ", bonf_alpha,
        "` (\u03b1 / ", k, ").",
        "\n-  **False Discovery Rate (FDR)**: apply `p.adjust(p_values, method = \"fdr\")` ",
        "to the ", k, " per-response fixed-effect p-values after the fact.",
        "\n-  **Multivariate model**: if responses are correlated, fit a joint multivariate ",
        "mixed model (e.g. `MCMCglmm`, `brms`) before interpreting individual responses.",
        "\n-  **Pre-registration**: if each response was a pre-specified (primary) study ",
        "outcome, correction may not be required; document this decision explicitly.  \n  \n",
        "\n\n***\n\n"
      ))
    }

    ## ---------- Per-response loop ----------------------------------------
    ## Each iteration fits its own lmer() with the same RHS but a
    ## different LHS, then writes its results into
    ## output_list[[response_name]][[key]]. This nesting is always used,
    ## regardless of the number of responses, so the returned structure
    ## is consistent with f_aov, f_kruskal_test, f_glm, f_t_test and
    ## f_wilcox_test: result[[response_name]]$<slot>.
    n_responses <- length(lhs)

    # set_out(): write a value to output_list[[response_name]][[key]].
    # Reads `response_name` from the enclosing loop frame via lexical
    # scope, so it MUST be defined inside generate_report (not at
    # f_lmer top level). Hoisted out of the loop so it isn't redefined
    # on every iteration.
    set_out <- function(key, value) {
      if (is.null(output_list[[response_name]])) {
        output_list[[response_name]] <<- list()
      }
      output_list[[response_name]][[key]] <<- value
    }

    for (resp_idx in seq_along(lhs)) {
      response_name <- lhs[[resp_idx]]

      # Build the per-response formula
      current_formula <- stats::as.formula(paste0(response_name, " ~ ", rhs))

    ## ---------- Header ----------------------------------------------------
    cat("\n# Linear Mixed Model: ", response_name, "\n\n")
    cat("**Model:** `", deparse(current_formula), "`  \n", sep = "")
    cat("**Method:** REML = ", REML,
        " &nbsp;&nbsp; **DF method:** ", ddf, "  \n\n", sep = "")

    ## ---------- Fit the model --------------------------------------------
    fit_warnings <- character(0)
    fit_messages <- character(0)
    fit <- withCallingHandlers(
      tryCatch(
        do.call(
          lmerTest::lmer,
          c(list(formula = current_formula, data = data,
                 REML = REML, weights = mf_weights),
            lmer_extra)
        ),
        error = function(e) {
          stop("lme4::lmer() failed to fit the model: ", conditionMessage(e))
        }
      ),
      warning = function(w) {
        fit_warnings <<- c(fit_warnings, conditionMessage(w))
        invokeRestart("muffleWarning")
      },
      message = function(m) {
        fit_messages <<- c(fit_messages, conditionMessage(m))
        invokeRestart("muffleMessage")
      }
    )

    is_singular <- isTRUE(lme4::isSingular(fit, tol = 1e-4))
    conv_msgs   <- fit@optinfo$conv$lme4$messages
    has_conv_warning <- length(conv_msgs) > 0 || length(fit_warnings) > 0

    set_out("model", fit)
    set_out("is_singular", is_singular)
    set_out("convergence_msgs", c(conv_msgs, fit_warnings))

    if (is_singular || has_conv_warning) {
      cat("\n***\n\n")
      if (is_singular) {
        cat("**[!] Singular fit detected.** One or more variance components ",
            "are estimated at (or very close to) zero. This usually means the ",
            "random-effects structure is too complex for the data - often a ",
            "random slope with too few levels, or a grouping factor that does ",
            "not actually create variance. **Simplify the random-effects ",
            "structure before interpreting results.**\n\n", sep = "")
      }
      if (has_conv_warning) {
        cat("**[!] Convergence / fitting messages from lme4:**\n\n")
        for (msg in unique(c(conv_msgs, fit_warnings))) {
          cat("- ", msg, "\n", sep = "")
        }
        cat("\n")
      }
      cat("\n***\n\n")
    }

    ## ---------- Sample size and grouping summary --------------------------
    n_obs    <- stats::nobs(fit)
    re_terms <- lme4::ngrps(fit)
    cat("\n## Sample size and grouping structure\n")
    cat("- **N observations:** ", n_obs, "  \n", sep = "")
    for (gnm in names(re_terms)) {
      cat("- **Levels of `", gnm, "`:** ", re_terms[[gnm]],
          if (re_terms[[gnm]] < 5)
            "  [!] *fewer than 5 levels - variance estimate is unreliable*"
          else "",
          "  \n", sep = "")
    }

    ## ---------- Fixed-effects ANOVA table --------------------------------
    cat("\n## Fixed-effects table (Type III)\n")

    if (ddf == "lme4") {
      coef_tbl <- summary(fit)$coefficients
      anova_tbl <- NULL
      cat("`ddf = \"lme4\"` was requested, so no p-values are reported. ",
          "Only t-statistics are shown.  \n\n", sep = "")
      coef_df <- as.data.frame(coef_tbl)
      coef_df$Term <- rownames(coef_df)
      coef_df <- coef_df[, c("Term", setdiff(names(coef_df), "Term"))]
      rownames(coef_df) <- NULL
      f_pander(f_conditional_round(coef_df, digits = 4))
    } else {
      anova_tbl <- tryCatch(
        as.data.frame(stats::anova(fit, ddf = ddf, type = "III")),
        error = function(e) {
          warning("anova(fit, ddf = '", ddf, "') failed: ",
                  conditionMessage(e),
                  ". Falling back to Satterthwaite.")
          as.data.frame(stats::anova(fit, ddf = "Satterthwaite", type = "III"))
        }
      )
      anova_tbl$Term  <- rownames(anova_tbl)
      anova_tbl$Sig   <- ifelse(anova_tbl[["Pr(>F)"]] < alpha, "*", "")
      # Pretty p-value column for display (numeric kept in output_list)
      anova_display <- anova_tbl
      anova_display[["Pr(>F)"]] <- vapply(anova_display[["Pr(>F)"]],
                                          fmt_p, character(1))
      # Reorder columns
      ord <- c("Term", "Sum Sq", "Mean Sq", "NumDF", "DenDF", "F value",
               "Pr(>F)", "Sig")
      ord <- ord[ord %in% names(anova_display)]
      anova_display <- anova_display[, c(ord, setdiff(names(anova_display), ord)),
                                     drop = FALSE]
      rownames(anova_display) <- NULL
      f_pander(f_conditional_round(anova_display, digits = 4))
      cat("\n`*` marks terms significant at \u03b1 = ", alpha,
          ". DenDF computed via **", ddf, "**.  \n\n", sep = "")
    }

    set_out("fixed_effects", if (ddf == "lme4") coef_df else anova_display)

    ## Coefficient (estimate) table for completeness
    coef_summary <- as.data.frame(summary(fit)$coefficients)
    coef_summary$Term <- rownames(coef_summary)
    coef_summary <- coef_summary[, c("Term",
                                     setdiff(names(coef_summary), "Term"))]
    rownames(coef_summary) <- NULL
    set_out("coefficients", coef_summary)

    cat("\n### Coefficient estimates\n")
    coef_display <- coef_summary
    if ("Pr(>|t|)" %in% names(coef_display)) {
      coef_display[["Pr(>|t|)"]] <- vapply(coef_display[["Pr(>|t|)"]],
                                           fmt_p, character(1))
    }
    f_pander(f_conditional_round(coef_display, digits = 4))

    ## ---------- Random-effects variance components + ICC -----------------
    cat("\n## Random-effects variance components\n")

    vc <- as.data.frame(lme4::VarCorr(fit))
    # vc has columns: grp, var1, var2, vcov, sdcor
    vc_print <- data.frame(
      Group   = vc$grp,
      Term    = ifelse(is.na(vc$var2),
                       ifelse(is.na(vc$var1), "(Intercept)", vc$var1),
                       paste(vc$var1, vc$var2, sep = " : ")),
      Variance = vc$vcov,
      Std.Dev. = vc$sdcor,
      stringsAsFactors = FALSE
    )
    # Residual row has no "term" - blank it out for clarity
    vc_print$Term[vc_print$Group == "Residual"] <- ""
    rownames(vc_print) <- NULL
    f_pander(f_conditional_round(vc_print, digits = 4))

    # ICC for the simplest case: random intercepts only
    # Values are computed here but printed in the combined Model Fit
    # table further down (together with R-squared).
    intercept_rows <- vc[is.na(vc$var2) &
                          (is.na(vc$var1) | vc$var1 == "(Intercept)" |
                             vc$var1 == ""), ]
    resid_row <- vc[vc$grp == "Residual", ]
    icc_table <- NULL
    between_var <- NA_real_
    resid_var   <- NA_real_
    icc_value   <- NA_real_
    if (nrow(intercept_rows) >= 1 && nrow(resid_row) == 1) {
      between_var <- sum(intercept_rows$vcov[intercept_rows$grp != "Residual"],
                         na.rm = TRUE)
      resid_var   <- resid_row$vcov[1]
      total_var   <- between_var + resid_var
      if (!is.na(total_var) && total_var > 0) {
        icc_value <- between_var / total_var
        # Stand-alone icc_table kept in output_list for back-compat /
        # programmatic access. Display happens in the combined block below.
        icc_table <- data.frame(
          var_group = unname(between_var),
          var_resid = unname(resid_var),
          icc       = unname(icc_value),
          stringsAsFactors = FALSE
        )
        names(icc_table) <- c("Between-group variance",
                              "Residual variance",
                              "ICC (between / total)")
        rownames(icc_table) <- NULL
      }
    }

    set_out("var_components", vc_print)
    set_out("icc", icc_table)

    ## ---------- Combined Model Fit table: ICC + R-squared ----------------
    ## Both have very few values; on a typical 78-char console they fit
    ## comfortably side by side as a single horizontal table.
    r2_table   <- NULL
    r2m <- r2c <- NA_real_
    if (requireNamespace("MuMIn", quietly = TRUE)) {
      r2 <- tryCatch(MuMIn::r.squaredGLMM(fit), error = function(e) NULL)
      if (!is.null(r2)) {
        r2m <- unname(r2[1, "R2m"])
        r2c <- unname(r2[1, "R2c"])
        # Stand-alone r2_table kept in output_list for back-compat /
        # programmatic access
        r2_table <- data.frame(marginal = r2m, conditional = r2c,
                               stringsAsFactors = FALSE)
        names(r2_table) <- c("Marginal R\u00b2 (fixed)",
                             "Conditional R\u00b2 (fixed + random)")
        rownames(r2_table) <- NULL
      }
    }
    set_out("r_squared", r2_table)

    # Build the combined display table only with the components that exist
    have_icc <- !is.null(icc_table)
    have_r2  <- !is.null(r2_table)
    if (have_icc || have_r2) {
      cat("\n## Model fit\n")

      fit_cols <- list()
      if (have_icc) {
        fit_cols[["Var(group)"]] <- unname(between_var)
        fit_cols[["Var(resid)"]] <- unname(resid_var)
        fit_cols[["ICC"]]        <- unname(icc_value)
      }
      if (have_r2) {
        fit_cols[["R\u00b2 marg."]] <- r2m
        fit_cols[["R\u00b2 cond."]] <- r2c
      }
      modelfit_tbl <- as.data.frame(fit_cols, check.names = FALSE,
                                    stringsAsFactors = FALSE)
      rownames(modelfit_tbl) <- NULL
      f_pander(f_conditional_round(modelfit_tbl, digits = 4),
               col_width = 14)

      # Compact combined legend
      cat(
        if (have_icc) paste0(
          "\n- **Var(group)** / **Var(resid):** between-group and residual
          variance components.",
          "\n- **ICC** = Var(group) / [Var(group) + Var(resid)]: share of total
          variance attributable to *between*-group differences (0 = grouping
          irrelevant; 1 = within-group observations identical)."
        ) else "",
        if (have_r2) paste0(
          "\n- **R\u00b2 marg.**: variance explained by the **fixed** effects
          alone (Nakagawa & Schielzeth).",
          "\n- **R\u00b2 cond.**: variance explained by **fixed + random**
          effects together. The gap is the variance absorbed by the
          random-effects structure.  \n"
        ) else "",
        sep = "")
    }

    ## ---------- Information criteria -------------------------------------
    dev_value <- if (isTRUE(REML)) {
      tryCatch(lme4::REMLcrit(fit), error = function(e) NA_real_)
    } else {
      tryCatch(stats::deviance(fit), error = function(e) NA_real_)
    }
    dev_label <- if (isTRUE(REML)) "REML criterion" else "deviance"

    fit_indices <- data.frame(
      AIC           = stats::AIC(fit),
      BIC           = stats::BIC(fit),
      logLik        = as.numeric(stats::logLik(fit)),
      `df.residual` = tryCatch(stats::df.residual(fit), error = function(e) NA_real_),
      check.names = FALSE,
      stringsAsFactors = FALSE
    )
    # Add the deviance / REML criterion column with the right label
    fit_indices[[dev_label]] <- dev_value
    # Reorder so the criterion sits next to logLik for readability
    fit_indices <- fit_indices[, c("AIC", "BIC", "logLik",
                                   dev_label, "df.residual")]
    cat("\n## Information criteria\n")
    f_pander(f_conditional_round(fit_indices, digits = 3),
             col_width = 16)
    cat("\n- **AIC** (Akaike Information Criterion): balances model fit ",
        "against complexity. *Lower is better*, but the absolute number ",
        "is meaningless on its own; use it only to compare models fit to ",
        "the *same* data.",
        "\n- **BIC** (Bayesian Information Criterion): same idea as AIC but ",
        "penalises extra parameters more strongly, so it tends to prefer ",
        "simpler models. *Lower is better*.",
        "\n- **logLik** (log-likelihood): how well the model fits the data. ",
        "*Higher (less negative) is better*. AIC and BIC are derived ",
        "from this.",
        sep = "")
    if (isTRUE(REML)) {
      cat("\n- **REML criterion:** the quantity `lmer()` minimises when ",
          "`REML = TRUE`. Use it to compare models that differ in their ",
          "random-effects structure only (same fixed effects). Do **not** ",
          "compare REML-fitted models that differ in fixed effects; refit ",
          "with `REML = FALSE` for that.  \n", sep = "")
    } else {
      cat("\n- **deviance:** (-2 * logLik): the quantity `lmer()` minimises ",
          "when `REML = FALSE`. Use it to compare nested models that differ ",
          "in fixed effects via likelihood-ratio tests.", sep = "")
    }
    cat("\n- **df residual:** residual degrees of freedom. In an LMM this ",
        "is approximate (no exact df exists), and is not used for ",
        "fixed-effect p-values; those use Satterthwaite or Kenward-Roger ",
        "instead.  \n\n", sep = "")
    set_out("fit_indices", fit_indices)

    if (output_type != "rmd") {
      cat("\n<div style=\"page-break-after: always;\"></div>\n\\newpage\n")
    }

    ## ---------- Diagnostic plots -----------------------------------------
    diag_plots <- list()
    if (norm_plots == TRUE) {
      cat("\n## Diagnostic plots\n")

      resids <- stats::residuals(fit)
      fits   <- stats::fitted(fit)

      # 4-panel diagnostic figure
      temp_diag <- tempfile(fileext = ".png")
      grDevices::png(temp_diag, width = 1800, height = 1400, res = 200)
      par(mfrow = c(2, 2), mar = c(4.2, 4.2, 3, 1))

      # 1: residuals vs fitted
      plot(fits, resids,
           xlab = "Fitted values", ylab = "Residuals",
           main = "Residuals vs Fitted",
           pch = 19, col = adjustcolor("steelblue", alpha.f = 0.6))
      abline(h = 0, col = "tomato", lwd = 2, lty = 2)

      # 2: QQ of level-1 residuals
      stats::qqnorm(resids, main = "Q-Q plot of residuals",
                    pch = 19, col = adjustcolor("steelblue", alpha.f = 0.6))
      stats::qqline(resids, col = "tomato", lwd = 2)

      # 3: QQ of random-effect BLUPs (first grouping factor, intercepts)
      re_list <- lme4::ranef(fit)
      first_grp <- re_list[[1]]
      blups_intercept <- if ("(Intercept)" %in% colnames(first_grp))
        first_grp[, "(Intercept)"] else first_grp[, 1]
      stats::qqnorm(blups_intercept,
                    main = paste0("Q-Q plot of BLUPs: ", names(re_list)[1]),
                    pch = 19, col = adjustcolor("seagreen", alpha.f = 0.7))
      stats::qqline(blups_intercept, col = "tomato", lwd = 2)

      # 4: scale-location
      plot(fits, sqrt(abs(resids)),
           xlab = "Fitted values",
           ylab = expression(sqrt(abs("residuals"))),
           main = "Scale-Location",
           pch = 19, col = adjustcolor("steelblue", alpha.f = 0.6))

      par(mfrow = c(1, 1))
      grDevices::dev.off()

      cat(paste0("![](", temp_diag, ")"), "  \n  \n")
      cat("\n*Top-left:* residuals should scatter randomly around zero with ",
          "no funnel shape. *Top-right:* level-1 residuals should fall on ",
          "the line. *Bottom-left:* the random-effect BLUPs should also be ",
          "approximately normal - this is the LMM-specific assumption most ",
          "users forget. *Bottom-right:* the spread of residuals should be ",
          "roughly constant across fitted values.\n\n", sep = "")

      diag_plots[["diagnostics_png"]] <- temp_diag

      # Shapiro-Wilk on residuals and BLUPs.
      # safe_shapiro() returns a shaped htest for all n regimes:
      # real result for n in [3, 5000], NA p-value with an informative
      # method label otherwise. This lets us tell the user *why* the
      # test was skipped instead of silently omitting the line, and
      # keeps the display code uniform.
      sw_res <- safe_shapiro(resids)
      if (!is.na(sw_res$p.value)) {
        cat("- Shapiro-Wilk on level-1 residuals: W = ",
            round(sw_res$statistic, 4),
            ", p = **", fmt_p(sw_res$p.value), "**", "  \n", sep = "")
      } else {
        cat("- Shapiro-Wilk on level-1 residuals: skipped (",
            sw_res$method, ")  \n", sep = "")
      }

      sw_blup <- safe_shapiro(blups_intercept)
      if (!is.na(sw_blup$p.value)) {
        cat("- Shapiro-Wilk on BLUPs of `", names(re_list)[1], "`: W = ",
            round(sw_blup$statistic, 4),
            ", p = **", fmt_p(sw_blup$p.value), "**", "  \n", sep = "")
      } else {
        cat("- Shapiro-Wilk on BLUPs of `", names(re_list)[1],
            "`: skipped (", sw_blup$method, ")  \n", sep = "")
      }

      # Levene's test on residuals against the first random-effects
      # grouping factor. This is the LMM analog of the Levene's test
      # in f_aov() / f_glm(): it asks whether residual variance is
      # roughly constant across the levels of the grouping factor,
      # which is the homoscedasticity assumption in an LMM.
      # Skipped silently if the grouping factor has < 2 levels with
      # >= 2 observations each, or if anything goes wrong.
      levene_p        <- NA_real_
      levene_stat     <- NA_real_
      levene_grp_name <- NA_character_
      levene_failed   <- FALSE
      tryCatch({
        mf_full  <- stats::model.frame(fit)
        grp_name <- names(re_list)[1]
        if (!is.null(grp_name) && grp_name %in% names(mf_full)) {
          grp_vec    <- mf_full[[grp_name]]
          grp_factor <- as.factor(grp_vec)
          tab_n      <- table(grp_factor)
          n_levels   <- length(tab_n[tab_n >= 2])
          if (n_levels >= 2 && length(resids) == length(grp_vec)) {
            lev_df  <- data.frame(res_lmer = resids, grp_lmer = grp_factor)
            lev_res <- rstatix::levene_test(res_lmer ~ grp_lmer, data = lev_df)
            if (!is.na(lev_res$p)) {
              levene_p        <- lev_res$p
              levene_stat     <- lev_res$statistic
              levene_grp_name <- grp_name
              levene_failed   <- lev_res$p < alpha
            }
          }
        }
      }, error = function(e) {
        # Anything unexpected: skip silently.
      })

      if (!is.na(levene_p)) {
        cat("- Levene's test on residuals (grouped by `", levene_grp_name,
            "`): F = ", round(levene_stat, 4),
            ", p = **", fmt_p(levene_p), "**",
            if (levene_failed)
              " (residuals do **NOT** have equal variance across groups)"
            else
              " (no evidence of heteroscedasticity across groups)",
            "  \n\n", sep = "")
      } else {
        cat("- Levene's test on residuals: skipped (grouping factor has ",
            "fewer than two levels with at least two observations each).  \n\n",
            sep = "")
      }

      cat("\n*Note: Shapiro-Wilk is sensitive to large samples and may flag ",
          "harmless deviations. Trust the Q-Q plot more than the p-value.*\n\n",
          sep = "")

      ## ---------- Conditional recommendations block --------------------
      ## Triggered when Levene's test or either Shapiro-Wilk test
      ## indicates a violation. Replaces the old "What if my residuals
      ## are not normal?" intro section by surfacing the same advice
      ## only when it is actually relevant.
      sw_res_failed  <- !is.na(sw_res$p.value)  && sw_res$p.value  < alpha
      sw_blup_failed <- !is.na(sw_blup$p.value) && sw_blup$p.value < alpha

      if (sw_res_failed || sw_blup_failed || levene_failed) {

        failed_items <- character(0)
        if (levene_failed)
          failed_items <- c(failed_items,
            paste0("Levene's test on residuals (grouped by `",
                   levene_grp_name, "`) p = ", fmt_p(levene_p),
                   " indicates **heteroscedasticity**"))
        if (sw_res_failed)
          failed_items <- c(failed_items,
            paste0("Shapiro-Wilk on level-1 residuals p = ",
                   fmt_p(sw_res$p.value),
                   " indicates **non-normal residuals**"))
        if (sw_blup_failed)
          failed_items <- c(failed_items,
            paste0("Shapiro-Wilk on BLUPs of `", names(re_list)[1],
                   "` p = ", fmt_p(sw_blup$p.value),
                   " indicates **non-normal random-effect BLUPs**"))

        cat("## Recommendations for Heteroscedasticity and/or non-normal residuals\n\n")
        cat("The following diagnostic test(s) flagged a violation at ",
            "\u03b1 = ", alpha, ":\n\n", sep = "")
        for (it in failed_items) cat("- ", it, ".  \n", sep = "")
        cat("\n")

        cat("LMMs are reasonably robust to mild non-normality because the ",
            "random effects absorb a lot of structure that would otherwise ",
            "show up as skew. If a problem remains, the **recommended** fix ",
            "is usually a model with a family that matches the response. ",
            "See `?f_glm` for the choice of family (Gamma / log-normal for ",
            "skewed positive data, Poisson or negative binomial for counts, ",
            "beta for proportions, etc.); the same families are available ",
            "with random effects via `lme4::glmer()` or `glmmTMB::glmmTMB()`.",
            "  \n\n", sep = "")
        cat("This keeps the response on its natural scale and the variance ",
            "components interpretable. As a **last resort** you may transform ",
            "the response manually with `f_boxcox()` or `f_bestNormalize()` ",
            "and refit with `f_lmer(transformed_y ~ ...)`, but be aware that ",
            "**variance components and ICCs computed on a transformed scale ",
            "do not back-transform to the original scale**. Report them with ",
            "care.\n\n", sep = "")
        cat("If only Levene's test is significant (heteroscedasticity ",
            "without non-normal residuals), a model that allows the residual ",
            "variance to differ across groups is also an option ",
            "(e.g. `glmmTMB` with the `dispformula` argument, or ",
            "`nlme::lme` with `weights = varIdent()`).\n\n", sep = "")
      }
    }
    set_out("diagnostics", diag_plots)

    if (output_type != "rmd") {
      cat("\n<div style=\"page-break-after: always;\"></div>\n\\newpage\n")
    }

    ## ---------- Observed Descriptives Table ------------------------------
    ## Raw-data summary (n, mean, sd, se, min/Q1/median/Q3/max) grouped by
    ## the categorical fixed-effect predictors only. Mirrors f_aov() /
    ## f_glm().
    ##
    ## We read column types from the **model frame** of the fitted lmer,
    ## not from the raw `data`, so that on-the-fly conversions in the
    ## formula (e.g. `factor(vs)`, `as.factor(gear)`, `ordered(dose)`)
    ## are honoured. Without this, a numeric column wrapped in `factor()`
    ## would be missed and the table would silently not print.
    ## `terms(fit, fixed.only = TRUE)` strips the random-effects part so
    ## random-effect grouping variables (subject / plot / block) do not
    ## end up as rows. Interaction terms like `"a:b"` are filtered out
    ## because they are not single columns in the model frame; the main
    ## effects of `a` and `b` remain and provide the grouping.
    ##
    ## `mf`, `cat_predictors`, and `numeric_predictors` are deliberately
    ## defined at this top level (with safe empty defaults) so that the
    ## post hoc section below can re-use them without re-parsing the
    ## formula.
    data_summary_table  <- NULL
    cat_predictors      <- character(0)
    numeric_predictors  <- character(0)
    mf <- tryCatch(stats::model.frame(fit), error = function(e) NULL)
    fixed_labels <- tryCatch(
      attr(stats::terms(fit, fixed.only = TRUE), "term.labels"),
      error = function(e) character(0)
    )
    if (!is.null(mf) && length(fixed_labels) > 0L) {
      fixed_labels <- fixed_labels[fixed_labels %in% names(mf)]
      if (length(fixed_labels) > 0L) {
        is_cat <- vapply(
          mf[fixed_labels],
          function(v) is.factor(v) || is.character(v) || is.logical(v),
          logical(1)
        )
        cat_predictors     <- fixed_labels[is_cat]
        numeric_predictors <- fixed_labels[!is_cat]
      }
    }

    ## Response column in the model frame (handles `log(y) ~ ...` too).
    ## Renamed to avoid shadowing the outer per-response loop counter
    ## `resp_idx` (line ~838), which is used later for page breaks.
    resp_col <- NA_character_
    if (!is.null(mf)) {
      mf_resp_idx <- attr(stats::terms(fit), "response")
      if (length(mf_resp_idx) && mf_resp_idx > 0L) resp_col <- names(mf)[mf_resp_idx]
    }

    if (length(cat_predictors) > 0L && !is.na(resp_col) &&
        is.numeric(mf[[resp_col]])) {
      data_summary_table <- tryCatch(
        f_summary(mf,
                  resp_col,
                  cat_predictors,
                  show_name = FALSE,
                  digits    = NULL)$output_df,
        error = function(e) NULL
      )

      if (!is.null(data_summary_table)) {
        cat(paste("\n## Observed Descriptives Table of: ",
                  deparse(current_formula), "  \n"))
        f_pander(f_conditional_round(data_summary_table, digits = 3,
                                     replace_na = FALSE))
        cat(
          "**TIP:** These values represent your actual observed sample characteristics.
Use this table for *Methods* sections or Supplementary materials (to describe the sample).\n  \n
**CAUTION:** For statistical inference (*p-values* and significance letters) and reporting
main findings in the *Results* section, you **must** use the fixed-effects (ANOVA) table
above and the Emmeans post hoc table below (when shown).\n"
        )
      }
    }
    set_out("data_summary_table", data_summary_table)

    ## ---------- Post hoc on factor fixed effects -------------------------
    ## Matches f_aov(): emmeans are shown for EVERY categorical fixed-effect
    ## term (not only the significant ones). Non-significant terms get their
    ## CLD letters collapsed to "ns" so the reader does not over-interpret
    ## them, but the estimates and CIs remain visible because they are often
    ## needed for manuscript tables.
    ##
    ## Categorical membership is read from `cat_predictors`, computed
    ## earlier from the model frame. This is essential: using the raw
    ## `data` here would miss on-the-fly conversions such as
    ## `factor(vs)` and `trm %in% names(data)` would be FALSE, causing
    ## the loop to silently skip every term (the bug present before).
    posthoc_results <- list()
    if (post_hoc == TRUE && !is.null(anova_tbl)) {
      # Per-term p-value lookup for the ns decision.
      p_lookup <- stats::setNames(anova_tbl[["Pr(>F)"]], anova_tbl$Term)

      # All non-interaction, non-intercept fixed-effect terms that have
      # a p-value. Numeric covariates are noted separately (no pairwise
      # comparisons for a slope).
      candidate_terms <- anova_tbl$Term[
        !is.na(anova_tbl[["Pr(>F)"]]) &
          !grepl(":", anova_tbl$Term) &
          anova_tbl$Term != "(Intercept)"
      ]
      posthoc_terms <- intersect(candidate_terms, cat_predictors)
      numeric_in_anova <- intersect(candidate_terms, numeric_predictors)

      # Overall significance: if no categorical fixed effect reaches
      # alpha, all letters collapse to "ns". We still print the table
      # (useful for manuscript tables and back-transformed responses).
      any_sig <- length(posthoc_terms) > 0L &&
        any(!is.na(p_lookup[posthoc_terms]) &
              p_lookup[posthoc_terms] < alpha)

      if (length(posthoc_terms) > 0L) {
        cat("\n## Post hoc comparisons (`emmeans`, adjust = `", adjust, "`)\n\n",
            sep = "")

        if (!any_sig) {
          cat("**NOTE:** No categorical fixed effect reached ",
              "\u03b1 = ", alpha, ". The emmeans below are shown for ",
              "completeness (estimates and CIs are still useful, ",
              "e.g. for manuscript tables), but all letters are ",
              "flagged `ns` and should not be interpreted as ",
              "significant pairwise differences.\n\n", sep = "")
        }
      }

      for (trm in posthoc_terms) {
        term_p  <- p_lookup[[trm]]
        term_ns <- is.na(term_p) || term_p >= alpha

        ## emmeans() needs the underlying variable name (`vs`), not the
        ## term label as written in the formula (`factor(vs)`).
        ## The reference grid stores variables by their bare name even
        ## when a wrapper is applied. `all.vars(parse(...))` strips the
        ## wrapper: factor(vs) -> "vs", as.factor(gear) -> "gear",
        ## ordered(dose) -> "dose". For already-bare labels ("vs") this
        ## is a no-op.
        emm_spec <- tryCatch(
          all.vars(parse(text = trm)[[1]]),
          error = function(e) trm
        )
        if (length(emm_spec) == 0L) emm_spec <- trm

        emm_df <- switch(ddf,
                         "Satterthwaite"  = "satterthwaite",
                         "Kenward-Roger"  = "kenward-roger",
                         "lme4"           = "asymptotic")
        emm <- tryCatch(
          emmeans::emmeans(fit, specs = emm_spec, lmer.df = emm_df),
          error = function(e) NULL
        )
        if (is.null(emm)) {
          cat("\n### ", trm, "\n\nemmeans failed for this term.\n\n", sep = "")
          next
        }

        pairs_tbl <- tryCatch(
          as.data.frame(pairs(emm, adjust = adjust)),
          error = function(e) NULL
        )
        emm_tbl <- as.data.frame(emm)

        # Compact letter display. When the term is not significant,
        # overwrite the .group column with "ns" so the reader does not
        # over-interpret letter clusters that fall out of an overall
        # non-significant test.
        cld_tbl <- tryCatch(
          as.data.frame(multcomp::cld(emm, adjust = adjust, Letters = letters)),
          error = function(e) NULL
        )
        if (!is.null(cld_tbl) && ".group" %in% names(cld_tbl)) {
          names(cld_tbl)[names(cld_tbl) == ".group"] <- "Letter"
          if (term_ns) cld_tbl$Letter <- "ns"
        }
        ## cld() reorders rows by mean, producing non-sequential
        ## row names (e.g. "2", "1"). f_pander renders these as an
        ## unnamed leading column -- strip them. Matches f_aov() /
        ## f_glm().
        if (!is.null(cld_tbl)) rownames(cld_tbl) <- NULL
        if (!is.null(pairs_tbl)) rownames(pairs_tbl) <- NULL
        rownames(emm_tbl) <- NULL

        cat("\n### ", trm,
            if (term_ns) "  (not significant)" else "",
            "\n\n", sep = "")

        if (term_ns) {
          cat("Term `", trm, "` was not significant (p = ",
              fmt_p(term_p), "). Letters collapsed to `ns`.  \n\n",
              sep = "")
        }

        if (!is.null(cld_tbl)) {
          cat("**Estimated marginal means with compact letter display.** \n",
              sep = "")
          f_pander(f_conditional_round(cld_tbl, digits = 4))
        } else {
          cat("**Estimated marginal means:**  \n\n")
          f_pander(f_conditional_round(emm_tbl, digits = 4))
        }

        n_groups <- if (!is.null(cld_tbl)) nrow(cld_tbl) else nrow(emm_tbl)
        n_pairs  <- if (!is.null(pairs_tbl)) nrow(pairs_tbl) else NA_integer_

        cld_text <- paste0("Confidence level used: ", 1 - alpha, "  \n",
            "Significance level used: \u03b1 = ", alpha, "  \n",
            "P-value and CI adjustment: ", adjust,
            " method for ", n_groups, " estimates",
            if (!is.na(n_pairs)) paste0(" / ", n_pairs, " tests") else "",
            ".  \n\n",
              "*Note: Groups in the \"Letters\" column sharing the same letter are ",
              "**not** significantly different (\u03B1 = ", alpha, "). Groups with ",
              "different letters are significantly different. Sharing a letter ",
              "indicates insufficient evidence to claim a difference; it does not ",
              "prove the groups are identical.*\n\n"
            )

        cat(cld_text)
        set_out("cld_text", cld_text)

        if (!is.null(pairs_tbl)) {
          cat("\n**Pairwise contrasts (", adjust, " adjusted):**  \n\n",
              sep = "")
          f_pander(f_conditional_round(pairs_tbl, digits = 4))
        }

        posthoc_results[[trm]] <- list(
          emmeans = emm_tbl,
          pairs   = pairs_tbl,
          cld     = cld_tbl
        )
      }

      # Note on numeric covariates in the fixed-effects table.
      if (length(numeric_in_anova) > 0L) {
        cat("\n**Note on numeric covariate(s):** ",
            paste(numeric_in_anova, collapse = ", "),
            ". Their slopes are reported in the coefficient table ",
            "above; no pairwise post hoc is performed.  \n\n", sep = "")
      }

      # Note about significant interactions (same behaviour as before).
      int_terms <- anova_tbl$Term[grepl(":", anova_tbl$Term) &
                                    !is.na(anova_tbl[["Pr(>F)"]]) &
                                    anova_tbl[["Pr(>F)"]] < alpha]
      if (length(int_terms) > 0) {
        cat("\n**[!] Significant interaction(s) detected:** ",
            paste(int_terms, collapse = ", "),
            ". Main-effect post hoc tests above should be interpreted with ",
            "care - the effect of one factor depends on the level of another. ",
            "Use `emmeans(model, ~ factor1 | factor2)` manually for ",
            "simple-effects contrasts.\n\n", sep = "")
      }
    }

    set_out("post_hoc", posthoc_results)

      ## ---------- Inter-response page break ------------------------------
      if (n_responses > 1L && resp_idx < n_responses && output_type != "rmd") {
        cat("\n<div style=\"page-break-after: always;\"></div>\n\\newpage\n")
      }
    } # end per-response loop

    if (output == TRUE) return(output_list)
  } # end generate_report

  ###########################################################################
  ## Run the report quietly to populate output_list                        ##
  ###########################################################################
  suppressMessages(
    utils::capture.output(
      output_list <- generate_report(),
      file = nullfile()
    )
  )
  class(output_list) <- "f_lmer"

  ###########################################################################
  ## Output dispatch                                                       ##
  ###########################################################################
  if (output_type %in% c("word", "pdf")) {

    message(paste0("Saving output in: ", output_path))

    word_pdf_preamble <- function() { paste0("
---
title: \"Linear Mixed Model Report\"
date: \"`r Sys.Date()`\"
output:
   word_document:
      reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
        latex_engine: pdflatex
header-includes:
  - \\usepackage[utf8]{inputenc}
  - \\usepackage{textcomp}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
  - \\DeclareUnicodeCharacter{03B1}{\\ensuremath{\\alpha}}
  - \\DeclareUnicodeCharacter{03C7}{\\ensuremath{\\chi}}
  - \\DeclareUnicodeCharacter{03B2}{\\ensuremath{\\beta}}
  - \\DeclareUnicodeCharacter{2212}{\\textminus}
  - \\DeclareUnicodeCharacter{00B2}{\\ensuremath{^2}}
  - \\DeclareUnicodeCharacter{2014}{\\textemdash}
  - \\DeclareUnicodeCharacter{00D7}{\\ensuremath{\\times}}
---
") }

    knitr::opts_chunk$set(comment = "")

    generated_markdown <- capture.output(generate_report(output = FALSE))

    rmd_content <- paste(
      word_pdf_preamble(),
      paste(generated_markdown, collapse = "\n"),
      sep = "\n"
    )

    writeLines(rmd_content, temp_output_file)

    rmarkdown::render(
      temp_output_file,
      output_file       = output_path,
      intermediates_dir = temp_output_dir,
      knit_root_dir     = temp_output_dir,
      quiet             = TRUE,
      output_format     = paste0(output_type, "_document")
    )

    if (open_generated_files == TRUE) f_open_file(output_path)
    return(invisible(output_list))

  } else if (output_type == "excel") {

    message(paste0("Saving output in: ", output_path))

    # Every response is nested under its name. Use the bare slot name
    # (no prefix) when there's only one response, otherwise prefix
    # sheet names with the response to keep them unique.
    sub_iter <- output_list[lhs]
    prefixes <- if (length(lhs) == 1L) "" else paste0(lhs, "_")

    sheets <- list()
    for (i in seq_along(sub_iter)) {
      sub    <- sub_iter[[i]]
      prefix <- prefixes[[i]]
      if (!is.null(sub$data_summary_table))
        sheets[[paste0(prefix, "Data_Summary")]] <- sub$data_summary_table
      if (!is.null(sub$fixed_effects))
        sheets[[paste0(prefix, "Fixed_Effects")]] <- sub$fixed_effects
      if (!is.null(sub$coefficients))
        sheets[[paste0(prefix, "Coefficients")]]  <- sub$coefficients
      if (!is.null(sub$var_components))
        sheets[[paste0(prefix, "Random_Effects")]] <- sub$var_components
      if (!is.null(sub$icc))
        sheets[[paste0(prefix, "ICC")]]           <- sub$icc
      if (!is.null(sub$r_squared))
        sheets[[paste0(prefix, "R_squared")]]     <- sub$r_squared
      if (!is.null(sub$fit_indices))
        sheets[[paste0(prefix, "Fit_Indices")]]   <- sub$fit_indices
      if (length(sub$post_hoc) > 0) {
        for (trm in names(sub$post_hoc)) {
          ph <- sub$post_hoc[[trm]]
          if (!is.null(ph$emmeans))
            sheets[[paste0(prefix, "emmeans_", trm)]] <- ph$emmeans
          if (!is.null(ph$pairs))
            sheets[[paste0(prefix, "pairs_",   trm)]] <- ph$pairs
          if (!is.null(ph$cld))
            sheets[[paste0(prefix, "cld_",     trm)]] <- ph$cld
        }
      }
    }
    # Excel sheet names: max 31 chars, no special chars
    names(sheets) <- substr(gsub("[^A-Za-z0-9_]", "_", names(sheets)), 1, 31)

    writexl::write_xlsx(sheets, path = output_path)

    if (open_generated_files == TRUE) f_open_file(output_path)
    return(invisible(output_list))

  } else if (output_type == "rmd") {

    if (is.null(knitr::opts_knit$get("output.dir"))) {
      knitr::opts_knit$set(output.dir = tempdir())
    }
    generated_markdown <- capture.output(generate_report(output = FALSE))
    output_list[["rmd"]] <- paste(generated_markdown, collapse = "\n")
    return(invisible(output_list))

  } else if (output_type == "default") {

    return(output_list)

  } else if (output_type == "console") {

    print(output_list)
    return(invisible(output_list))

  } else {
    warning("Invalid output format specified. No file generated.")
  }

  invisible(suppressWarnings(file.remove(temp_output_file)))
}


#' @export
print.f_lmer <- function(x, ...) {
  cat("\n==========================================================\n")
  cat("Linear Mixed Model (f_lmer)\n")
  cat("==========================================================\n")

  print_one <- function(sub) {
    if (!is.null(sub$model)) {
      cat("\nFormula: ", deparse(stats::formula(sub$model)), "\n", sep = "")
    }

    if (isTRUE(sub$is_singular)) {
      cat("\n!! SINGULAR FIT - simplify the random-effects structure ",
          "before interpreting results.\n", sep = "")
    }
    if (length(sub$convergence_msgs) > 0) {
      cat("\n!! Convergence messages:\n")
      for (m in unique(sub$convergence_msgs)) cat("   - ", m, "\n", sep = "")
    }

    if (!is.null(sub$fixed_effects)) {
      cat("\n--- Fixed-effects table ---\n")
      f_pander(f_conditional_round(sub$fixed_effects, digits = 4))
    }

    if (!is.null(sub$data_summary_table)) {
      cat("\n--- Observed descriptives (by fixed-effect factor levels) ---\n")
      f_pander(f_conditional_round(sub$data_summary_table, digits = 3,
                                   replace_na = FALSE))
    }

    if (!is.null(sub$var_components)) {
      cat("\n--- Random-effects variance components ---\n")
      f_pander(f_conditional_round(sub$var_components, digits = 4))
    }

    have_icc <- !is.null(sub$icc)
    have_r2  <- !is.null(sub$r_squared)
    if (have_icc || have_r2) {
      cat("\n--- Model fit ---\n")
      fit_cols <- list()
      if (have_icc) {
        fit_cols[["Var(group)"]] <- as.numeric(sub$icc[[1]])
        fit_cols[["Var(resid)"]] <- as.numeric(sub$icc[[2]])
        fit_cols[["ICC"]]        <- as.numeric(sub$icc[[3]])
      }
      if (have_r2) {
        fit_cols[["R\u00b2 marg."]] <- as.numeric(sub$r_squared[[1]])
        fit_cols[["R\u00b2 cond."]] <- as.numeric(sub$r_squared[[2]])
      }
      modelfit_tbl <- as.data.frame(fit_cols, check.names = FALSE,
                                    stringsAsFactors = FALSE)
      rownames(modelfit_tbl) <- NULL
      f_pander(f_conditional_round(modelfit_tbl, digits = 4),
               col_width = 14)

      legend_lines <- character(0)
      if (have_icc) {
        legend_lines <- c(legend_lines,
          "Var(group) / Var(resid): between-group and residual variance components.",
          "ICC = Var(group) / [Var(group) + Var(resid)]: share of total variance attributable to between-group differences (0 = grouping irrelevant; 1 = within-group observations identical).")
      }
      if (have_r2) {
        legend_lines <- c(legend_lines,
          "R\u00b2 marg.: variance explained by the fixed effects alone (Nakagawa & Schielzeth).",
          "R\u00b2 cond.: variance explained by fixed + random effects together. The gap is the variance absorbed by the random-effects structure.")
      }
      for (ln in legend_lines) {
        cat(paste(strwrap(paste("-", ln), width = 76, exdent = 2),
                  collapse = "\n"), "\n", sep = "")
      }
    }

    if (!is.null(sub$fit_indices)) {
      cat("\n--- Information criteria ---\n")
      f_pander(f_conditional_round(sub$fit_indices, digits = 3),
               col_width = 16)
    }

    if (length(sub$post_hoc) > 0) {
      cat("\n--- Post hoc comparisons ---\n")
      for (trm in names(sub$post_hoc)) {
        cat("\n* ", trm, " *\n", sep = "")
        ph <- sub$post_hoc[[trm]]
        if (!is.null(ph$cld)) {
          cat("Estimated marginal means with compact letter display:\n")
          f_pander(f_conditional_round(ph$cld, digits = 4))
          cat(sub$cld_text)
        } else if (!is.null(ph$emmeans)) {
          cat("Estimated marginal means:\n")
          f_pander(f_conditional_round(ph$emmeans, digits = 4))
        }
      }
    }
  }

  # Every f_lmer result is nested under response_name. Non-response
  # meta-slots (e.g. "rmd") are skipped.
  resp_names <- names(x)
  resp_names <- resp_names[!resp_names %in% c("rmd")]

  if (length(resp_names) == 1L) {
    print_one(x[[resp_names]])
  } else {
    for (rn in resp_names) {
      cat("\n----------------------------------------------------------\n")
      cat("Response: ", rn, "\n", sep = "")
      cat("----------------------------------------------------------\n")
      print_one(x[[rn]])
    }
  }

  invisible(x)
}


#' Plot method for f_lmer objects
#'
#' Replays the four-panel diagnostic figure (residuals vs fitted, Q-Q of
#' residuals, Q-Q of random-effect BLUPs, scale-location) produced by
#' \code{f_lmer()}.
#'
#' @param x An object of class \code{f_lmer}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return Returns \code{x} invisibly.
#'
#' @export
plot.f_lmer <- function(x, ...) {

  plot_one <- function(sub, label = NULL) {
    png_path <- sub$diagnostics$diagnostics_png
    if (is.null(png_path) || !file.exists(png_path)) {
      if (is.null(sub$model)) {
        message("No diagnostics available", if (!is.null(label)) paste0(" for '", label, "'") else "",
                " - plot was disabled with norm_plots = FALSE.")
        return(invisible(NULL))
      }
      fit <- sub$model
      resids <- stats::residuals(fit)
      fits   <- stats::fitted(fit)
      par(mfrow = c(2, 2), mar = c(4.2, 4.2, 3, 1))
      title_suffix <- if (!is.null(label)) paste0(" (", label, ")") else ""
      plot(fits, resids, xlab = "Fitted", ylab = "Residuals",
           main = paste0("Residuals vs Fitted", title_suffix),
           pch = 19, col = adjustcolor("steelblue", alpha.f = 0.6))
      abline(h = 0, col = "tomato", lwd = 2, lty = 2)
      stats::qqnorm(resids, main = paste0("Q-Q residuals", title_suffix),
                    pch = 19, col = adjustcolor("steelblue", alpha.f = 0.6))
      stats::qqline(resids, col = "tomato", lwd = 2)
      re_list <- lme4::ranef(fit)
      blups <- if ("(Intercept)" %in% colnames(re_list[[1]]))
        re_list[[1]][, "(Intercept)"] else re_list[[1]][, 1]
      stats::qqnorm(blups,
                    main = paste0("Q-Q BLUPs: ", names(re_list)[1], title_suffix),
                    pch = 19, col = adjustcolor("seagreen", alpha.f = 0.7))
      stats::qqline(blups, col = "tomato", lwd = 2)
      plot(fits, sqrt(abs(resids)), xlab = "Fitted",
           ylab = expression(sqrt(abs("residuals"))),
           main = paste0("Scale-Location", title_suffix),
           pch = 19, col = adjustcolor("steelblue", alpha.f = 0.6))
      par(mfrow = c(1, 1))
    } else {
      if (requireNamespace("magick", quietly = TRUE)) {
        img <- magick::image_read(png_path)
        grid::grid.newpage()
        grid::grid.raster(img)
      } else {
        message("Install 'magick' to redisplay diagnostic plots, ",
                "or open the PNG at: ", png_path)
      }
    }
  }

  resp_names <- names(x)
  resp_names <- resp_names[!resp_names %in% c("rmd")]

  if (length(resp_names) == 1L) {
    plot_one(x[[resp_names]])
  } else {
    for (rn in resp_names) plot_one(x[[rn]], label = rn)
  }
  invisible(x)
}

# test-f_glm.R
# Comprehensive stress tests for f_glm()
# Run with: testthat::test_file("test-f_glm.R")
# =============================================================================
# SHARED TEST DATA
# =============================================================================

# mtcars with categorical predictors and binary responses
mtcars_cat <- local({
  d        <- mtcars
  d$cyl    <- as.factor(d$cyl)
  d$gear   <- as.factor(d$gear)
  d$vs     <- as.integer(d$vs)
  d$am     <- as.integer(d$am)
  d
})

# Known non-significant dataset: perfect 50/50 split in every group
d_null <- data.frame(
  y   = c(rep(0L, 15), rep(1L, 15), rep(0L, 15), rep(1L, 15)),
  grp = factor(c(rep("A", 30), rep("B", 30)))
)

# Helper: call f_glm silently (use directly when testing warnings/messages)
run_glm <- function(...) {
  suppressMessages(suppressWarnings(f_glm(...)))
}

# =============================================================================
# 1. INPUT VALIDATION
# =============================================================================

test_that("alpha validation rejects values outside (0, 1)", {
  skip_on_cran()
  expect_error(f_glm(mpg ~ cyl, data = mtcars_cat, alpha = 0),
               regexp = "alpha")
  expect_error(f_glm(mpg ~ cyl, data = mtcars_cat, alpha = 1),
               regexp = "alpha")
  expect_error(f_glm(mpg ~ cyl, data = mtcars_cat, alpha = -0.1),
               regexp = "alpha")
  expect_error(f_glm(mpg ~ cyl, data = mtcars_cat, alpha = 1.1),
               regexp = "alpha")
  expect_error(f_glm(mpg ~ cyl, data = mtcars_cat, alpha = "0.05"),
               regexp = "alpha")
})

test_that("match.arg rejects invalid string arguments", {
  skip_on_cran()
  expect_error(
    f_glm(mpg ~ cyl, data = mtcars_cat, output_type = "INVALID"),
    regexp = "should be one of"
  )
  expect_error(
    f_glm(mpg ~ cyl, data = mtcars_cat, adjust = "INVALID"),
    regexp = "should be one of"
  )
  expect_error(
    f_glm(mpg ~ cyl, data = mtcars_cat, type = "INVALID"),
    regexp = "should be one of"
  )
})

test_that("missing variables in formula cause informative errors", {
  skip_on_cran()
  expect_error(
    f_glm(nonexistent ~ cyl, data = mtcars_cat),
    regexp = "not found in the data"
  )
  expect_error(
    f_glm(mpg ~ nonexistent, data = mtcars_cat),
    regexp = "not found in the data"
  )
})

test_that("non-numeric response variable is rejected", {
  skip_on_cran()
  # cyl is a factor - not numeric
  expect_error(
    f_glm(cyl ~ am, data = mtcars_cat),
    regexp = "must be numeric"
  )
})

test_that("invalid family causes an error", {
  skip_on_cran()
  expect_error(
    f_glm(mpg ~ cyl, data = mtcars_cat, family = "not_a_family")
  )
})

test_that("valid alpha values are accepted without error", {
  skip_on_cran()
  expect_error(
    run_glm(mpg ~ cyl, family = gaussian, data = mtcars_cat,
            alpha = 0.01, output_type = "default"),
    NA  # NA means expect no error
  )
  expect_error(
    run_glm(mpg ~ cyl, family = gaussian, data = mtcars_cat,
            alpha = 0.1, output_type = "default"),
    NA
  )
})

# =============================================================================
# 2. RETURN STRUCTURE
# =============================================================================

test_that("f_glm returns correct S3 class", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(mpg ~ cyl, family = gaussian, data = mtcars_cat,
                    output_type = "default")
  expect_s3_class(result, "f_glm")
})

test_that("f_glm return list has all expected elements per response", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(mpg ~ cyl, family = gaussian, data = mtcars_cat,
                    output_type = "default")

  expect_true("mpg" %in% names(result))
  resp <- result$mpg
  required_keys <- c("model", "summary", "diagnostics", "posthoc",
                     "drop1", "lrt", "sig_effects", "sep_flag",
                     "lrt_null_dev", "lrt_resid_dev", "lrt_pct_explained")
  expect_true(all(required_keys %in% names(resp)),
              info = paste("Missing:", paste(setdiff(required_keys, names(resp)),
                                             collapse = ", ")))
})

test_that("GLM model object embedded in result is correct class", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(mpg ~ cyl, family = gaussian, data = mtcars_cat,
                    output_type = "default")

  expect_s3_class(result$mpg$model, "glm")
  expect_s3_class(result$mpg$model, "lm")
  expect_equal(result$mpg$model$family$family, "gaussian")
})

test_that("deviance values satisfy model fit inequality", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(mpg ~ cyl, family = gaussian, data = mtcars_cat,
                    output_type = "default")

  # Residual deviance must be less than null for any predictor
  expect_lt(result$mpg$lrt_resid_dev, result$mpg$lrt_null_dev)
  # McFadden pseudo-R2 must be in (0, 100)
  expect_gt(result$mpg$lrt_pct_explained, 0)
  expect_lt(result$mpg$lrt_pct_explained, 100)
})

# =============================================================================
# 3. BINOMIAL FAMILY - SEPARATION AND LETTERS
# =============================================================================

test_that("complete separation in vs ~ cyl sets sep_flag to TRUE", {
  skip_on_cran()
  # All 8-cylinder cars in mtcars have vs = 0 - textbook complete separation
  expect_true(all(mtcars[mtcars$cyl == 8, "vs"] == 0))

  set.seed(42)
  result <- run_glm(vs ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  expect_true(result$vs$sep_flag)
})

test_that("am ~ cyl has no separation and sep_flag is FALSE", {
  skip_on_cran()
  # am has mix of 0s and 1s across all cyl levels
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  expect_false(result$am$sep_flag)
})

test_that("sig_effects is TRUE for vs ~ cyl despite Wald p ~ 1 on cyl8", {
  skip_on_cran()
  # Wald p for cyl8 is ~0.99 (separation), but LRT p << 0.05
  # sig_effects must be based on LRT, not Wald
  set.seed(42)
  result <- run_glm(vs ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  expect_true(result$vs$sig_effects)
})

test_that("with separation: LRT pairwise letters differ between groups", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(vs ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  letters_col <- result$vs$posthoc$post_hoc_summary_table$Letter

  # LRT should produce differentiation - not all the same letter
  expect_false(all(letters_col == letters_col[1]),
               info = "All letters identical - LRT pairwise comparison likely failed")

  # Should not fall back to all em-dashes (that means LRT computation failed)
  expect_false(all(letters_col == "\u2014"),
               info = "All dashes - LRT letters were not computed")

  # Should not be 'ns' (overall model IS significant)
  expect_false(any(letters_col == "ns"))
})

test_that("non-significant model gives 'ns' letters for all groups", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(y ~ grp, family = binomial, data = d_null,
                    output_type = "default")

  expect_false(result$y$sig_effects)
  letters_col <- result$y$posthoc$post_hoc_summary_table$Letter
  expect_true(all(letters_col == "ns"))
})

test_that("sig_effects is TRUE and normal letters for am ~ cyl", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  expect_true(result$am$sig_effects)
  letters_col <- result$am$posthoc$post_hoc_summary_table$Letter
  expect_false(all(letters_col == "ns"))
  expect_false(all(letters_col == "\u2014"))
})

# =============================================================================
# 4. POISSON FAMILY
# =============================================================================

test_that("Poisson model returns correct structure", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(breaks ~ wool + tension, family = poisson,
                    data = warpbreaks, output_type = "default")

  expect_s3_class(result, "f_glm")
  expect_true("breaks" %in% names(result))
  expect_equal(result$breaks$model$family$family, "poisson")
})

test_that("Poisson model: wool + tension IS significant", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(breaks ~ wool + tension, family = poisson,
                    data = warpbreaks, output_type = "default")

  expect_true(result$breaks$sig_effects)
  expect_lt(result$breaks$lrt_resid_dev, result$breaks$lrt_null_dev)
})

test_that("drop1 result for Poisson has correct structure", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(breaks ~ wool, family = poisson,
                    data = warpbreaks, output_type = "default")

  d1 <- result$breaks$drop1
  expect_false(is.null(d1))
  expect_true(inherits(d1, "data.frame"))
  expect_true("Df" %in% names(d1))
  expect_true("Deviance" %in% names(d1))
})

# =============================================================================
# 5. QUASI FAMILIES
# =============================================================================

test_that("quasipoisson model runs and has correct family", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(breaks ~ wool, family = quasipoisson,
                    data = warpbreaks, output_type = "default")

  expect_s3_class(result, "f_glm")
  expect_equal(result$breaks$model$family$family, "quasipoisson")
})

test_that("quasipoisson: DHARMa sim_res is NULL (quasi-families not simulatable)", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(breaks ~ wool, family = quasipoisson,
                    data = warpbreaks, output_type = "default")

  expect_null(result$breaks$diagnostics$residuals$sim_res)
})

test_that("plot.f_glm emits message and does not crash for quasi-family", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(breaks ~ wool, family = quasipoisson,
                    data = warpbreaks, output_type = "default")

  expect_message(
    plot(result),
    regexp = "skipped"
  )
})

test_that("quasibinomial model runs without error", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = quasibinomial,
                    data = mtcars_cat, output_type = "default")

  expect_s3_class(result, "f_glm")
  expect_equal(result$am$model$family$family, "quasibinomial")
  expect_null(result$am$diagnostics$residuals$sim_res)
})

# =============================================================================
# 6. MULTIPLE RESPONSE VARIABLES
# =============================================================================

test_that("multiple responses produce separate entries in result", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am + vs ~ cyl, family = binomial,
                    data = mtcars_cat, output_type = "default")

  expect_s3_class(result, "f_glm")
  expect_true("am" %in% names(result))
  expect_true("vs" %in% names(result))
})

test_that("multiple responses: each has its own independent model", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am + vs ~ cyl, family = binomial,
                    data = mtcars_cat, output_type = "default")

  # am: no separation; vs: complete separation
  expect_false(result$am$sep_flag)
  expect_true(result$vs$sep_flag)

  # Both should have sig_effects = TRUE (LRT confirms both)
  expect_true(result$am$sig_effects)
  expect_true(result$vs$sig_effects)
})

# =============================================================================
# 7. ALL ADJUST METHODS
# =============================================================================

test_that("all adjust methods produce valid post-hoc tables", {
  skip_on_cran()
  adjust_methods <- c("sidak", "bonferroni", "tukey", "none", "fdr")

  for (adj in adjust_methods) {
    set.seed(42)
    result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                      adjust = adj, output_type = "default")

    ph <- result$am$posthoc$post_hoc_summary_table
    expect_true(is.data.frame(ph),
                info = paste("adjust =", adj, "did not return data.frame"))
    expect_true("Letter" %in% names(ph),
                info = paste("adjust =", adj, "missing Letter column"))
    expect_true("n" %in% names(ph),
                info = paste("adjust =", adj, "missing n column"))
    expect_equal(nrow(ph), 3L,
                 info = paste("adjust =", adj, "wrong number of rows (expected 3 cyl levels)"))
  }
})

# =============================================================================
# 8. OUTPUT TYPES
# =============================================================================

test_that("output_type = 'rmd' stores markdown string at top level", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "rmd", intro_text = FALSE)

  expect_true("rmd" %in% names(result))
  expect_type(result$rmd, "character")
  expect_gt(nchar(result$rmd), 200)   # non-trivial content
})

test_that("output_type = 'rmd' still has response entries alongside rmd element", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "rmd", intro_text = FALSE)

  # rmd and am entries coexist
  expect_true("am" %in% names(result))
  expect_true("rmd" %in% names(result))
})

test_that("output_type = 'console' forces output to console and returns object", {
  skip_on_cran()
  set.seed(42)
  expect_output(
    result <- f_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "console"),
    regexp = "GLM"
  )
  expect_s3_class(result, "f_glm")
})

test_that("output_type = 'excel' creates a valid non-empty file", {
  skip_on_cran()
  set.seed(42)
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)

  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "excel", save_as = tmp,
                    open_generated_files = FALSE)

  expect_true(file.exists(tmp))
  expect_gt(file.size(tmp), 0)
  expect_s3_class(result, "f_glm")
})

test_that("output_type = 'word' creates a .docx file (requires pandoc)", {
  skip_on_cran()
  skip_if_no_pandoc()
  skip_if(!rmarkdown::pandoc_available(), "Pandoc not available")

  set.seed(42)
  tmp <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp), add = TRUE)

  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "word", save_as = tmp,
                    open_generated_files = FALSE, intro_text = FALSE)

  expect_true(file.exists(tmp))
  expect_gt(file.size(tmp), 0)
})

# =============================================================================
# 9. PRINT AND PLOT METHODS
# =============================================================================

test_that("print.f_glm works for Gaussian model", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(mpg ~ cyl, family = gaussian, data = mtcars_cat,
                    output_type = "default")

  expect_output(print(result), regexp = "GLM of response variable")
  expect_output(print(result), regexp = "Coefficients")
  expect_output(print(result), regexp = "Post hoc")
})

test_that("print.f_glm works for Binomial with separation", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(vs ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  expect_output(print(result), regexp = "GLM of response variable")
})

test_that("print.f_glm skips 'rmd' character entry without crash", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "rmd", intro_text = FALSE)

  # 'rmd' is a character string in the list - print must skip it gracefully
  expect_output(print(result), regexp = "GLM of response variable")
})

test_that("plot.f_glm runs without error for non-quasi binomial", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  expect_error(plot(result), NA)   # NA = expect no error
})

test_that("plot.f_glm skips 'rmd' element without crash", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "rmd", intro_text = FALSE)

  # Should not crash despite 'rmd' character element
  expect_error(
    suppressMessages(plot(result)),
    NA
  )
})

# =============================================================================
# 10. DISPERSION TEST PARAMETER
# =============================================================================

test_that("dispersion_test = FALSE omits Dispersion Diagnostics section", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(breaks ~ wool, family = poisson, data = warpbreaks,
                    output_type = "rmd", intro_text = FALSE,
                    dispersion_test = FALSE)

  expect_false(grepl("Dispersion Diagnostics", result$rmd, fixed = TRUE))
})

test_that("dispersion_test = TRUE includes Dispersion Diagnostics section for Poisson", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(breaks ~ wool, family = poisson, data = warpbreaks,
                    output_type = "rmd", intro_text = FALSE,
                    dispersion_test = TRUE)

  expect_true(grepl("Dispersion Diagnostics", result$rmd, fixed = TRUE))
})

test_that("Bernoulli 0/1 data skips dispersion test with Bernoulli explanation", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(vs ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "rmd", intro_text = FALSE,
                    dispersion_test = TRUE)

  # Must explain the skip, not run the DHARMa test
  expect_true(grepl("Bernoulli", result$rmd, fixed = TRUE))
  expect_false(grepl("DHARMa Dispersion Test", result$rmd, fixed = TRUE))
})

test_that("quasipoisson shows phi parameter, not DHARMa test", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(breaks ~ wool, family = quasipoisson, data = warpbreaks,
                    output_type = "rmd", intro_text = FALSE,
                    dispersion_test = TRUE)

  expect_true(grepl("Quasi-dispersion parameter", result$rmd, fixed = TRUE))
  expect_false(grepl("DHARMa Dispersion Test", result$rmd, fixed = TRUE))
})

# =============================================================================
# 11. NUMERIC PREDICTOR WARNING
# =============================================================================

test_that("numeric predictor with <= 10 unique values triggers a warning", {
  skip_on_cran()
  d <- mtcars
  d$vs <- as.integer(d$vs)
  # cyl is numeric with 3 unique values (4, 6, 8) - should warn

  expect_warning(
    suppressMessages(
      f_glm(vs ~ cyl, family = binomial, data = d, output_type = "default")
    ),
    regexp = "numeric"
  )
})

test_that("factor predictor does NOT trigger a numeric predictor warning", {
  skip_on_cran()
  # cyl IS a factor - must call f_glm directly since run_glm() suppresses warnings
  expect_warning(
    suppressMessages(
      f_glm(vs ~ cyl, family = binomial, data = mtcars_cat,
            output_type = "default")
    ),
    NA   # NA = expect no warning
  )
})

# =============================================================================
# 12. CONTINUOUS PREDICTOR EXCLUDED FROM EMMEANS
# =============================================================================

test_that("continuous covariate is excluded from emmeans specs with a warning", {
  skip_on_cran()
  d <- mtcars
  d$cyl <- as.factor(d$cyl)

  # Uses warning() not message() so it propagates through suppressMessages()
  # inside f_glm's internal caching - suppressMessages() would swallow message()
  expect_warning(
    suppressMessages(
      f_glm(mpg ~ cyl + hp, family = gaussian, data = d,
            output_type = "default")
    ),
    regexp = "continuous predictor"
  )
})

test_that("with continuous covariate: post-hoc table rows = factor levels only", {
  skip_on_cran()
  d <- mtcars
  d$cyl <- as.factor(d$cyl)

  result <- suppressMessages(
    run_glm(mpg ~ cyl + hp, family = gaussian, data = d,
            output_type = "default")
  )

  ph <- result$mpg$posthoc$post_hoc_summary_table
  # emmeans over cyl only (3 levels), hp held at mean
  expect_equal(nrow(ph), 3L)
})

# =============================================================================
# 13. TYPE ARGUMENT (response vs link scale)
# =============================================================================

test_that("type = 'response' gives probability column for binomial", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    type = "response", output_type = "default")

  ph <- result$am$posthoc$post_hoc_summary_table
  expect_true("prob" %in% names(ph))
  # Probabilities must lie in [0, 1]
  expect_true(all(ph$prob >= 0 & ph$prob <= 1, na.rm = TRUE))
})

test_that("type = 'link' gives emmean column on logit scale for binomial", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    type = "link", output_type = "default")

  ph <- result$am$posthoc$post_hoc_summary_table
  # Should have an emmean-like column (possibly renamed "emmean..")
  emm_present <- any(grepl("emmean", names(ph), ignore.case = TRUE))
  expect_true(emm_present)
})

# =============================================================================
# 14. POST-HOC TABLE STRUCTURE
# =============================================================================

test_that("post-hoc table has required columns", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  ph <- result$am$posthoc$post_hoc_summary_table
  expect_true(is.data.frame(ph))
  expect_true("Letter" %in% names(ph))
  expect_true("n"      %in% names(ph))
  expect_true("cyl"    %in% names(ph))
})

test_that("post-hoc table has one row per factor level", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  ph <- result$am$posthoc$post_hoc_summary_table
  expect_equal(nrow(ph), nlevels(mtcars_cat$cyl))  # 3
})

test_that("cld_text is a non-empty string", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  cld <- result$am$posthoc$cld_text
  expect_type(cld, "character")
  expect_true(nchar(cld) > 0)
})

# =============================================================================
# 15. DROP1 AND LRT STRUCTURE
# =============================================================================

test_that("drop1 result has anova class and key columns", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  d1 <- result$am$drop1
  expect_false(is.null(d1))
  expect_true(inherits(d1, "data.frame"))
  expect_true("Df"       %in% names(d1))
  expect_true("Deviance" %in% names(d1))
})

test_that("lrt result is a data.frame with NULL and predictor rows", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  lrt <- result$am$lrt
  expect_false(is.null(lrt))
  expect_true(inherits(lrt, "data.frame"))
  # Sequential anova: NULL row + cyl row = 2 rows
  expect_equal(nrow(lrt), 2L)
})

test_that("lrt_pct_explained is between 0 and 100 for significant model", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  pct <- result$am$lrt_pct_explained
  expect_true(is.numeric(pct))
  expect_gt(pct, 0)
  expect_lt(pct, 100)
})

# =============================================================================
# 16. FAMILY AS STRING VS FUNCTION
# =============================================================================

test_that("family passed as string gives same model as function", {
  skip_on_cran()
  set.seed(42)
  result_fn  <- run_glm(am ~ cyl, family = binomial,
                        data = mtcars_cat, output_type = "default")
  set.seed(42)
  result_str <- run_glm(am ~ cyl, family = "binomial",
                        data = mtcars_cat, output_type = "default")

  expect_equal(coef(result_fn$am$model),
               coef(result_str$am$model))
})

# =============================================================================
# 17. MISC PARAMETERS
# =============================================================================

test_that("diagnostic_plots = FALSE stores show_plot = FALSE in diagnostics", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    diagnostic_plots = FALSE, output_type = "default")

  expect_false(result$am$diagnostics$show_plot)
})

test_that("intro_text = FALSE produces shorter rmd output", {
  skip_on_cran()
  set.seed(42)
  result_with <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                         output_type = "rmd", intro_text = TRUE)
  result_without <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                            output_type = "rmd", intro_text = FALSE)

  expect_gt(nchar(result_with$rmd), nchar(result_without$rmd))
})

test_that("sig_effects and sep_flag are stored as logical scalars", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  expect_type(result$am$sig_effects, "logical")
  expect_length(result$am$sig_effects, 1L)
  expect_type(result$am$sep_flag, "logical")
  expect_length(result$am$sep_flag, 1L)
})

test_that("influential points structure is present and correct type", {
  skip_on_cran()
  set.seed(42)
  result <- run_glm(am ~ cyl, family = binomial, data = mtcars_cat,
                    output_type = "default")

  infl <- result$am$diagnostics$influence
  expect_true(!is.null(infl))
  expect_true("hat_values" %in% names(infl))
  expect_true("influential_points" %in% names(infl))
  expect_true(is.numeric(infl$hat_values))
  expect_equal(length(infl$hat_values), nrow(mtcars_cat))
})

test_that("higher influence_threshold detects fewer influential points", {
  skip_on_cran()
  set.seed(42)
  result_strict <- run_glm(mpg ~ cyl, family = gaussian, data = mtcars_cat,
                           influence_threshold = 1,
                           output_type = "default")
  result_lax    <- run_glm(mpg ~ cyl, family = gaussian, data = mtcars_cat,
                           influence_threshold = 10,
                           output_type = "default")

  n_strict <- length(result_strict$mpg$diagnostics$influence$influential_points)
  n_lax    <- length(result_lax$mpg$diagnostics$influence$influential_points)

  expect_gte(n_strict, n_lax)
})



# =============================================================================
# f_glm_new_tests.R
# Additional tests for f_glm() covering the fix where the descriptive
# f_summary() table now reads from glm_fit$model so it stays in sync
# with the rows actually fitted when subset / na.action / weights /
# offset are used via `...`.
# =============================================================================
# Deterministic 2-group binomial data
make_bin_data <- function(seed = 2026, n_per_group = 40) {
  set.seed(seed)
  data.frame(
    y   = rbinom(2 * n_per_group, 1,
                 prob = rep(c(0.3, 0.7), each = n_per_group)),
    grp = factor(rep(c("ctrl", "trt"), each = n_per_group)),
    x   = rnorm(2 * n_per_group)
  )
}

test_that("f_glm runs with subset passed through dots", {
  skip_on_cran()
  df <- make_bin_data()
  keep <- seq_len(nrow(df)) %% 2L == 0L

  expect_no_error(
    res <- suppressMessages(
      f_glm(y ~ grp, data = df, family = binomial,
            subset               = keep,
            output_type          = "default",
            open_generated_files = FALSE,
            intro_text           = FALSE)
    )
  )
  # Fitted model saw only the subset rows.
  expect_equal(nobs(res[[1]]$model), sum(keep))
})

test_that("descriptive summary table reflects fitted rows, not original data", {
  df   <- make_bin_data()
  keep <- seq_len(nrow(df)) %% 2L == 0L

  res <- suppressWarnings(suppressMessages(
    f_glm(y ~ grp, data = df, family = binomial,
          subset               = keep,
          output_type          = "default",
          open_generated_files = FALSE,
          intro_text           = FALSE)
  ))

  # Sanity: the model saw half the rows.
  expect_equal(nobs(res[[1]]$model), sum(keep))

  # Primary assertion: the per-group n column in the post-hoc summary
  # table sums to nobs(model), not nrow(df). If the table were built
  # from the raw input data, sum(ph$n) would equal nrow(df) (= 80).
  ph <- res[[1]]$posthoc$post_hoc_summary_table
  expect_false(is.null(ph))
  expect_true("n" %in% names(ph))
  expect_equal(sum(ph$n, na.rm = TRUE), sum(keep))

  # Both factor levels remain in the subset, so both should still be
  # listed in the post-hoc table.
  expect_setequal(as.character(ph$grp), c("ctrl", "trt"))
})


test_that("na.action = na.omit is honored by glm via dots", {
  skip_on_cran()
  df <- make_bin_data()
  df$x[c(1, 2, 3, 80)] <- NA

  res <- suppressWarnings(
    f_glm(y ~ grp + x, data = df, family = binomial,
          na.action = na.omit,
          output_type = "default",
          open_generated_files = FALSE,
          intro_text = FALSE)
  )
  expect_equal(nobs(res[[1]]$model), 76L)      # 80 - 4
})

#' Perform multiple t-tests with optional data transformation, inspection and visualization.
#'
#' Performs One-sample, Two-sample (Independent), or Paired t-tests on a given dataset
#' with options for (Box-Cox/BestNormalize) transformations, normality tests, and
#' visualization. Several response parameters can be analysed in sequence
#' (formula interface). Additionally, a vector interface similar to \code{stats::t.test()}
#' is supported.
#'
#' @param x Numeric vector of data values (one-sample or first group for two-sample),
#'   or a formula of the form \code{response ~ group} or \code{response ~ 1}.
#' @param y Optional numeric vector (second group) for two-sample tests if using
#'   the vector interface. Ignored when a formula is supplied.
#' @param formula A formula specifying the model (alternative to using x/y).
#'   \itemize{
#'     \item \strong{Two-sample (Independent/Paired):} \code{response ~ group}
#'           (where group has exactly 2 levels).
#'     \item \strong{One-sample:} \code{response ~ 1} or \code{response ~ NULL}.
#'   }
#'   More response variables can be added using \code{+} (e.g., \code{y1 + y2 ~ group}).
#' @param data A data frame containing the variables when using the formula interface.
#' @param paired Logical. If \code{TRUE}, performs a paired t-test.
#'   \strong{Note:} For the formula interface, data must be sorted so that all
#'   observations of group 1 appear before group 2 (AABB order). For the vector
#'   interface, \code{x} and \code{y} must have the same length.
#' @param var.equal Logical or \code{NULL}. If \code{TRUE}, forces Student's t-test
#'   (equal variances). If \code{FALSE} or \code{NULL} (default), Welch's t-test is
#'   used. Bartlett's and Levene's tests are always reported as diagnostics but do
#'   \strong{not} affect this choice. See Delacre, Lakens & Leys (2017).
#' @param mu Numeric. The true value to test against: the mean (one-sample), the mean
#'   of differences (paired), or the difference in means (two-sample). Default is 0.
#'   For transformed analyses, \code{mu} is forward-transformed for one-sample and
#'   paired tests. For two-sample tests with \code{mu != 0} a warning is issued.
#' @param alternative Character string. \code{"two.sided"} (default),
#'   \code{"greater"}, or \code{"less"}.
#' @param norm_plots Logical. If \code{TRUE}, diagnostic plots are included in the
#'   output. Default is \code{TRUE}.
#' @param transformation Logical or character string. If \code{TRUE} or
#'   \code{"boxcox"}, applies \code{f_boxcox()} when Shapiro-Wilk indicates
#'   non-normality. If \code{"bestnormalize"}, applies \code{f_bestNormalize()}.
#'   If \code{FALSE} or \code{"none"}, no transformation is applied.
#'   \strong{Note:} For paired tests, \code{bestNormalize} (Yeo-Johnson) is always
#'   used on the differences, since Box-Cox requires strictly positive values.
#'   Default is \code{TRUE}.
#' @param force_transformation Character vector. Names of variables to transform
#'   regardless of normality results.
#' @param alpha Numeric. Significance level. Default is \code{0.05}.
#' @param conf.level Numeric. Confidence level. Default is \code{1 - alpha}.
#'   If \code{conf.level} is specified, \code{alpha} is set to \code{1 - conf.level}.
#' @param intro_text Logical. If \code{TRUE}, includes explanation of t-test
#'   assumptions. Default is \code{TRUE}.
#' @param close_generated_files Logical. Closes open Excel/Word files before writing.
#'   Default \code{FALSE}. \strong{Windows only.}
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param output_type Character string specifying the output format. Default is \code{"default"}.
#'   \itemize{
#'     \item \code{"default"}: Returns the object and lets R decide whether
#'       to print; auto-prints if unassigned, silent if assigned to a variable.
#'       Use \code{print(result)} or \code{plot(result)} to display the
#'       returned object.
#'     \item \code{"console"}: Forces immediate printing to the console
#'       regardless of object assignment.
#'     \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
#'       file of the corresponding format. See \code{save_as},
#'       \code{save_in_wdir}, and \code{open_generated_files} for file
#'       path and opening behavior.
#'     \item \code{"rmd"}: Stores the raw markdown string inside the returned
#'       object for use in R Markdown documents.
#'   }
#' @param save_as Character. Specific path/filename for output.
#' @param save_in_wdir Logical. Save in working directory. Default \code{FALSE}.
#' @param ... For the formula method: additional arguments forwarded to
#'   the row-filtering step. The arguments \code{subset} and
#'   \code{na.action} are honored: when supplied, they are spliced
#'   (still unevaluated) into a \code{\link[stats]{model.frame}} call
#'   built once before the per-response loop, so the \code{subset}
#'   expression is evaluated in the data's column scope (e.g.
#'   \code{subset = cyl == 6} works). All responses in a
#'   multi-response call (\code{y1 + y2 ~ group}) are then tested on
#'   the identical row set. For the default (vector) method, \code{...}
#'   is currently unused.
#'
#' @return An object of class \code{'f_t_test'}, a named list with one element per
#'   response variable. Each element contains the t-test result, normality test
#'   results, variance diagnostic results, transformation object (if applied),
#'   and back-transformed confidence interval (if applicable).
#'
#' @references
#' Delacre, M., Lakens, D., & Leys, C. (2017). Why psychologists should by default
#' use Welch's t-test instead of Student's t-test.
#' \emph{International Review of Social Psychology}, 30(1), 92-101.
#' \doi{10.5334/irsp.82}
#'
#' @examples
#' \donttest{
#' # 1. Two-sample independent Welch's t-test (default)
#' f_t_test(mpg ~ am, data = mtcars, output_type = "console", norm_plots = FALSE)
#'
#' # 2. Multiple response variables in one call
#' f_t_test(mpg + hp ~ am, data = mtcars, output_type = "console", norm_plots = FALSE)
#'
#' # 3. One-sample t-test: test if mean mpg equals 20
#' f_t_test(mpg ~ 1, data = mtcars, mu = 20,
#'          output_type = "console", norm_plots = FALSE)
#'
#' # 4. Paired t-test (sleep dataset is already in AABB order)
#' f_t_test(extra ~ group, data = sleep, paired = TRUE,
#'          output_type = "console", norm_plots = FALSE)
#'
#' # 5. Vector interface: two-sample independent
#' group_auto   <- mtcars$mpg[mtcars$am == 0]
#' group_manual <- mtcars$mpg[mtcars$am == 1]
#' f_t_test(group_auto, group_manual, output_type = "console", norm_plots = FALSE)
#'
#' # 6. Vector interface: one-sample
#' f_t_test(mtcars$mpg, mu = 20, output_type = "console", norm_plots = FALSE)
#'
#' # 7. Force Student's t-test (equal variances assumed)
#' f_t_test(mpg ~ am, data = mtcars, var.equal = TRUE,
#'          output_type = "console", norm_plots = FALSE)
#'
#' # 8. One-sided test
#' f_t_test(mpg ~ am, data = mtcars, alternative = "greater",
#'          output_type = "console", norm_plots = FALSE)
#'
#' # 9. Custom significance level (alpha = 0.01 is equivalent to conf.level = 0.99)
#' f_t_test(mpg ~ am, data = mtcars, alpha = 0.01,
#'          output_type = "console", norm_plots = FALSE)
#'
#' # 10. Box-Cox transformation with back-transformed CI
#' # The back-transformed CI estimates the MEDIAN, not the arithmetic mean.
#' result <- f_t_test(hp ~ am, data = mtcars, transformation = TRUE,
#'                    output_type = "console", norm_plots = FALSE)
#' result[["hp"]]$ci_backtransformed
#'
#' # 11. One-sample with non-zero mu and back-transformation
#' f_t_test(hp ~ 1, data = mtcars, mu = 100, transformation = TRUE,
#'          output_type = "console", norm_plots = FALSE)
#'
#' # 12. BestNormalize transformation (set seed for reproducibility)
#' set.seed(123)
#' f_t_test(hp ~ am, data = mtcars, transformation = "bestnormalize",
#'          output_type = "console", norm_plots = FALSE)
#'
#' # 13. Force transformation regardless of normality
#' f_t_test(mpg + hp ~ am, data = mtcars, force_transformation = "mpg",
#'          output_type = "console", norm_plots = FALSE)
#'
#' # 14. Suppress transformation (diagnostic mode)
#' f_t_test(hp ~ am, data = mtcars, transformation = FALSE,
#'          output_type = "console", norm_plots = FALSE)
#'
#' # 15. Access return object fields directly
#' result <- f_t_test(mpg + hp ~ am, data = mtcars,
#'                    output_type = "default", norm_plots = FALSE, intro_text = FALSE)
#' result[["mpg"]]$t_test          # standard htest object
#' result[["hp"]]$shapiro_res      # Shapiro-Wilk result
#' result[["hp"]]$homog_p_bartlett # Bartlett p-value (diagnostic only)
#' result[["hp"]]$homog_p_levene   # Levene p-value (diagnostic only)
#' result[["hp"]]$ci_backtransformed # back-transformed CI if transformed
#' }
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#' @export
# =============================================================================
#' @export
# =============================================================================
f_t_test <- function(x, ...) {
  UseMethod("f_t_test")
}
# =============================================================================
#' @export
#' @rdname f_t_test
# =============================================================================
f_t_test.formula <- function(formula,
                             data = NULL,
                             paired = FALSE,
                             var.equal = NULL,
                             conf.level = NULL,
                             mu = 0,
                             alternative = "two.sided",
                             norm_plots = TRUE,
                             transformation = TRUE,
                             force_transformation = NULL,
                             alpha = 0.05,
                             intro_text = TRUE,
                             close_generated_files = FALSE,
                             open_generated_files = interactive(),
                             output_type = "default",
                             save_as = NULL,
                             save_in_wdir = FALSE,
                             ...) {
  # ---------------------------------------------------------------------------
  # 1. Reset initial settings on exit
  # ---------------------------------------------------------------------------
  .session_state <- save_session_state()  # Helper function: helper_session_state
  on.exit(restore_session_state(.session_state), add = TRUE) # Helper function: helper_session_state

  # ---------------------------------------------------------------------------
  # 2. Data & Formula Parsing
  # ---------------------------------------------------------------------------
  if (!is.null(data)) {
    data_name <- deparse(substitute(data))
  } else {
    if (length(formula_extract_df_names(formula)) > 0) {
      data_name <- paste(formula_extract_df_names(formula), collapse = "_")
    } else {
      data_name <- "data"
    }
    data    <- formula_to_dataframe(formula)
    formula <- clean_formula(formula)
  }

  temp_output_dir  <- tempdir()
  temp_output_file <- file.path(temp_output_dir, "output.Rmd")
  file.create(temp_output_file)
  try(f_wrap_lines(), silent = TRUE)

  file_extension <- NULL
  output_list    <- list()

  if (!(output_type %in% c("pdf", "word", "excel", "rmd", "console", "default")))
    stop("output_type should be: 'pdf', 'word', 'excel', 'console', 'rmd', 'default'")

  # ---------------------------------------------------------------------------
  # 3. File Path Logic
  # ---------------------------------------------------------------------------
  save_dir <- if (isTRUE(save_in_wdir))
    getwd()
  else
    tempdir()
  output_type_map <- c(
    "pdf" = ".pdf",
    "word" = ".docx",
    "excel" = ".xlsx",
    "rmd" = ".rmd"
  )

  if (!is.null(save_as) || isTRUE(save_in_wdir)) {
    if (!is.null(save_as)) {
      save_as <- gsub("\\\\", "/", save_as)
      file_extension_save_as <- unname(extract_extension(save_as))
      if (file_extension_save_as[1] != FALSE)
        file_extension <- file_extension_save_as
    }

    if (is.null(file_extension) &&
        output_type %in% c("console", "default")) {
      output_path <- get_save_path(
        save_as = save_as,
        default_name = paste(data_name, "ttest_output", sep = "_"),
        default_dir = save_dir,
        file.ext = ".pdf"
      )
      output_type <- "pdf"
    } else if (is.null(file_extension)) {
      output_path <- get_save_path(
        save_as = save_as,
        default_name = paste(data_name, "ttest_output", sep = "_"),
        default_dir = save_dir,
        file.ext = unname(output_type_map[output_type])
      )
    } else {
      output_path <- get_save_path(
        save_as = save_as,
        default_name = paste(data_name, "ttest_output", sep = "_"),
        default_dir = save_dir,
        file.ext = file_extension[1]
      )
      output_type <- file_extension[2]
    }
  } else if (output_type %in% names(output_type_map)) {
    output_path <- get_save_path(
      save_as = save_as,
      default_name = paste(data_name, "ttest_output", sep = "_"),
      default_dir = save_dir,
      file.ext = unname(output_type_map[output_type])
    )
  } else {
    output_path <- NULL
  }

  if (output_type == "rmd") close_generated_files <- FALSE

  # Cross-platform close_generated_files (was Windows-only taskkill)
  if (output_type != "rmd" && isTRUE(close_generated_files)) {
    close_app <- function(win_proc, mac_name, linux_name) {
      sysname <- Sys.info()[["sysname"]]
      if (.Platform$OS.type == "windows") {
        system(paste0("taskkill /im ", win_proc, " /f"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else if (sysname == "Darwin") {
        system(paste0("pkill -f '", mac_name, "'"),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      } else {
        system(paste0("pkill -f ", linux_name),
               ignore.stdout = TRUE, ignore.stderr = TRUE)
      }
    }
    if (output_type == "word")  close_app("WINWORD.EXE", "Microsoft Word",  "soffice")
    if (output_type == "excel") close_app("EXCEL.EXE",   "Microsoft Excel", "soffice")
  }

  # ---------------------------------------------------------------------------
  # 4. Transformation Argument Parsing
  # ---------------------------------------------------------------------------
  if (is.character(transformation)) {
    trans_input   <- tolower(transformation)
    valid_options <- c("bestnormalize", "boxcox", "true", "false", "none")
    matched_index <- pmatch(trans_input, valid_options)
    if (is.na(matched_index))
      stop("Invalid transformation option!")
    matched_option <- valid_options[matched_index]
    transformation <- switch(
      matched_option,
      "true"  = TRUE,
      "false" = ,
      "none"  = FALSE,
      matched_option
    )
  }

  # ---------------------------------------------------------------------------
  # 5. Parse LHS and RHS of Formula
  # ---------------------------------------------------------------------------
  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(formula) #use helper_check_lhs.R

  lhs <- all.vars(formula[[2]])


  if (length(lhs) < 1)
    stop("Formula must specify at least one response variable on the LHS.")

  if (length(formula) == 3) {
    rhs_vars   <- all.vars(formula[[3]])
    rhs_string <- deparse(formula[[3]])
    if (length(rhs_vars) == 0 || rhs_string %in% c("1", ".")) {
      is_one_sample  <- TRUE
      predictor_name <- NULL
    } else {
      is_one_sample  <- FALSE
      predictor_name <- rhs_vars[1]
      if (length(rhs_vars) > 1)
        warning("f_t_test only supports one grouping variable. Using the first one.")
    }
  } else {
    is_one_sample  <- TRUE
    predictor_name <- NULL
  }

  for (response in lhs) {
    if (!(response %in% names(data)))
      stop(paste("Response", response, "not found."))
    if (!is.numeric(data[[response]]))
      stop(paste("Response variable", response, "must be numeric."))
  }

  if (!is_one_sample) {
    if (!(predictor_name %in% names(data)))
      stop(paste("Predictor", predictor_name, "not found."))
    data[[predictor_name]] <- as.factor(data[[predictor_name]])
    if (nlevels(data[[predictor_name]]) != 2)
      stop("Grouping variable must have exactly 2 levels for a t-test.")
  }

  ########## Build master analysis frame (one row set, all responses) #######
  # Hoists row filtering out of the per-response loop so every response
  # in `mpg + hp ~ am` is tested on the identical row set, and bakes in
  # any subset / na.action passed via `...`.
  #
  # Strategy: pre-evaluate subset eagerly against data first, falling
  # back to the user's calling environment. Then apply the filter
  # manually and pass plain values to stats::model.frame via do.call.
  # This avoids the fragile match.call/substitute dance where spliced
  # expressions inside a constructed call can end up containing `..N`
  # dots-index symbols that crash model.frame with "the ... list
  # contains fewer than 3 elements".
  master_vars <- if (is_one_sample) lhs else c(lhs, predictor_name)
  master_formula <- as.formula(
    paste("~", paste(master_vars, collapse = " + "))
  )

  mc          <- match.call(expand.dots = FALSE)
  dots_exprs  <- as.list(mc[["..."]])
  caller_env  <- parent.frame()

  # Resolve subset expression against data columns + caller env
  subset_vec <- NULL
  if (!is.null(dots_exprs$subset)) {
    subset_vec <- eval(dots_exprs$subset, envir = data, enclos = caller_env)
    if (is.logical(subset_vec)) subset_vec[is.na(subset_vec)] <- FALSE
  }
  n_before_master <- nrow(data)            # before any filtering
  if (!is.null(subset_vec)) {
    data <- data[subset_vec, , drop = FALSE]
  }

  # Resolve na.action (a function, defaults to na.omit)
  na_action_fn <- stats::na.omit
  if (!is.null(dots_exprs$na.action)) {
    na_action_fn <- eval(dots_exprs$na.action, envir = caller_env)
  }

  mf_args <- list(
    formula            = master_formula,
    data               = data,
    drop.unused.levels = TRUE,
    na.action          = na_action_fn
  )

  data <- do.call(stats::model.frame, mf_args)
  n_after_master  <- nrow(data)            # after subset + NA drop
  ###########################################################################

  # ---------------------------------------------------------------------------
  # 6. Helper Functions
  # ---------------------------------------------------------------------------
  back_transform <- function(values, trans_obj, is_boxcox) {
    tryCatch({
      if (is_boxcox)
        predict(trans_obj, newdata = values, inverse = TRUE)
      else
        predict(trans_obj$bestNormalize,
                newdata = values,
                inverse = TRUE)
    }, error = function(e) {
      warning("Back-transformation failed. CI reported on transformed scale only.")
      NULL
    })
  }

  forward_transform <- function(value, trans_obj, is_boxcox) {
    tryCatch({
      if (is_boxcox)
        predict(trans_obj, newdata = value)
      else
        predict(trans_obj$bestNormalize,
                newdata = value,
                inverse = FALSE)
    }, error = function(e) {
      warning("Forward transformation of mu failed. Original mu used on transformed scale.")
      NULL
    })
  }

  generate_ci_label <- function(is_one_sample, paired, console = FALSE) {
    if (console) {
      if (is_one_sample)
        return(" for the population mean (\u03bc) ")
      if (paired)
        return(" for population mean of differences (\u03bc_diff) ")
      return(" for the difference in population means (\u03bc1 - \u03bc2) ")
    } else {
      if (is_one_sample)
        return(" for the population mean ($\\mu$)")
      if (paired)
        return(" of population mean of differences ($\\mu_{diff}$) ")
      return(" for the difference in population means ($\\mu_1 - \\mu_2$) ")
    }
  }

  generate_hypotheses <- function(name_a,
                                  name_b = NULL,
                                  paired = FALSE,
                                  mu = 0,
                                  alternative = "two.sided",
                                  console = FALSE) {
    param_desc <- if (is.null(name_b))
      sprintf("mean of %s", name_a)
    else if (paired)
      sprintf("mean difference (%s - %s)", name_a, name_b)
    else
      sprintf("difference in means (%s - %s)", name_a, name_b)

    ops <- switch(
      alternative,
      "two.sided" = list(h0 = "is equal to", ha = "is not equal to"),
      "greater"   = list(h0 = "is less than or equal to", ha = "is greater than"),
      "less"      = list(h0 = "is greater than or equal to", ha = "is less than")
    )

    null_text <- sprintf("True %s %s %s", param_desc, ops$h0, mu)
    alt_text  <- sprintf("True %s %s %s", param_desc, ops$ha, mu)

    if (console)
      paste0("\n  H0: ",
             null_text,
             "\n  H1: ",
             alt_text)
    else
      paste0("\n  + $H_{0}$: ", null_text, "\n  + $H_{1}$: ", alt_text)
  }

  # ===========================================================================
  # 7. Report Generation Function
  # ===========================================================================
  generate_report <- function(output = TRUE) {
    if (isTRUE(intro_text)) {
      cat(
        "
# Assumptions of t-tests

##  1. Continuous Scale of Measurement
- The dependent variable must be continuous (interval or ratio scale, e.g., weight, length).
- For discrete or ordinal data (e.g., counts, Likert scales), use non-parametric tests like Wilcoxon or Mann-Whitney.

##  2. Independence
- Observations must not influence each other. This is determined by study design, not statistics.
  + *Two-sample*: Subjects should be distinct both *within* and *between* groups.
  + *Paired*: Pairs should be distinct (though observations *within* a pair are correlated by design).
- **Violation invalidates** the test; the solution is a change in experimental design.

##  3. Normality
- Data must follow a normal (Gaussian) distribution. T-tests are robust to minor deviations if n > 30 (Central Limit Theorem).
  + *One/Two-sample:* The data within each group (or residuals) should be normal.
  + *Paired:* The **differences** between pairs (not raw values) should be normal.
- Normality is assessed *visually* (Q-Q plots, Histograms) and *formally* (Shapiro-Wilk test).
  + **Decision to transform is based on Shapiro-Wilk only** -- the most powerful omnibus normality test.
  + Anderson-Darling is shown as additional diagnostic information only.

##  4. Homogeneity of Variance (Homoscedasticity) -- Two-sample t-tests only.
- **Welch's t-test is the default** (`var.equal = FALSE`). Current best practice ([Delacre, Lakens & Leys, 2017](https://doi.org/10.5334/irsp.82)) shows Welch's performs as well as Student's when variances are equal, and substantially better when they are not.
- **Pre-testing is avoided.** Using Bartlett's or Levene's test to decide between t-tests inflates Type I error, and variance tests have low power with small samples.
- **Diagnostic only:** Bartlett's and Levene's (Brown-Forsythe) tests are reported to help you understand your data, but do not affect the choice of t-test.
- **Override:** To force Student's t-test (equal variances assumed), set `var.equal = TRUE`.

##  5. No Significant Outliers
- T-tests use the mean, making them sensitive to extreme values. Check with boxplots.

##  6. Hypothesis Direction (Design Choice)
- *Two-sided (Default):* Tests for any difference ($A \\neq B$).
- *One-sided:* Tests a specific direction ($A > B$). Only use if an opposite effect is theoretically impossible or irrelevant."
      )
      if (output_type %in% c("pdf", "word"))
        cat("\n<div style=\"page-break-after: always;\"></div>\n\\newpage")
    }

    i <- 0
    original_var.equal <- var.equal
    org.var.equal.null  <- is.null(var.equal)

    if (is.null(conf.level)) {
      conf.level <- 1 - alpha
    } else {
      alpha <- 1 - conf.level
    }

    # Fires regardless of intro_text so the user always sees it in this situation.
    if (length(lhs) > 1) {
      k          <- length(lhs)
      fwer_pct   <- round((1 - (1 - alpha)^k) * 100, 1)
      bonf_alpha <- round(alpha / k, 4)
      cat(paste0(
        "\n\n***\n\n",
        "**[!] NOTE: Multiple Testing Across ", k, " Response Variables**  \n\n",
        "This report runs ", k, " independent t-tests on the same dataset. ",
        "Each individual test is valid on its own, but running multiple tests ",
        "simultaneously inflates the risk of obtaining at least one spurious ",
        "significant result by chance.  \n\n",
        "At \u03b1 = ", alpha, " per test, the probability of at least one false positive ",
        "across all ", k, " responses is approximately ",
        "**", fwer_pct, "%** ($1-(1-", alpha, ")^{", k, "}$, assuming independence). ",
        "This risk is highest in exploratory studies; it is less of a concern when ",
        "each response variable has a clear a priori hypothesis.  \n\n",
        "**Possible remedies:**  \n",
        "\n-  **Bonferroni** (conservative): re-run with `alpha = ", bonf_alpha,
        "` (\u03b1 / ", k, ") to control the family-wise error rate.",
        "\n-  **False Discovery Rate (FDR)**: collect the ", k, " p-values and apply ",
        "`p.adjust(p_values, method = \"fdr\")` after the fact.",
        "\n-  **MANOVA**: if the response variables are correlated, consider `manova()` ",
        "as a single omnibus test before interpreting individual t-tests.",
        "\n-  **Pre-registration**: if each response was a pre-specified primary outcome, ",
        "correction may not be required; document this decision explicitly.",
        "\n\n***\n\n"
      ))
    }

    # Pre-compute once -- avoids floating point printing (e.g. 94.9999...)
    conf_pct <- round(conf.level * 100, 1)

    # =========================================================================
    # LOOP THROUGH RESPONSE VARIABLES
    # =========================================================================
    for (response_name in lhs) {
      var.equal <- if (org.var.equal.null)
        FALSE
      else
        original_var.equal

      Response_Transformed <- FALSE
      mu_transformed       <- NULL
      ci_backtransformed   <- NULL
      is_boxcox_trans      <- FALSE

      output_list[[response_name]][["transformation_option"]] <- transformation

      if (is_one_sample) {
        current_formula <- as.formula(paste0(response_name, " ~ 1"))
        cols_needed     <- response_name
      } else {
        current_formula <- as.formula(paste0(response_name, " ~ ", predictor_name))
        cols_needed     <- c(response_name, predictor_name)
      }

      # Row filtering already happened once at the master level (above
      # the loop), with subset / na.action from `...` baked in. Every
      # response in a multi-response call therefore sees the identical
      # row set. We just take the relevant columns here. data_complete
      # is a per-iteration copy because the transformation step below
      # mutates its response column in place.
      n_before      <- n_before_master
      data_complete <- data[, cols_needed, drop = FALSE]
      n_after       <- n_after_master

      cat("  \n\n# Analysis of: ", response_name, "  \n")
      cat("\n## Normality and Assumptions for: ", response_name, "  \n")

      y_vals <- data_complete[[response_name]]

      if (is_one_sample) {
        group1 <- y_vals
        group2 <- NULL
        levs   <- NULL
        normality_values<- y_vals
      } else {
        groups <- data_complete[[predictor_name]]
        levs   <- levels(groups)
        group1 <- y_vals[groups == levs[1]]
        group2 <- y_vals[groups == levs[2]]

        if (paired) {
          if (length(group1) != length(group2)) {
            stop(
              paste0(
                "Paired t-test requires equal group sizes. '",
                response_name,
                "' has unequal groups after removing NAs. Please clean your pairs."
              )
            )
          }

          group_runs <- rle(as.character(data_complete[[predictor_name]]))
          if (length(group_runs$lengths) != 2) {
            warning(
              "Paired t-test expects data sorted so all observations of '",
              levs[1],
              "' come before '",
              levs[2],
              "' (AABB order). ",
              "Your data appears interleaved. Please verify your data is sorted correctly."
            )
          }

          normality_values<- group1 - group2

        } else {
          lm_temp            <- lm(current_formula, data = data_complete)
          normality_values<- residuals(lm_temp)
        }
      }

      if (!is_one_sample && !paired) {
        bart_res  <- stats::bartlett.test(list(group1, group2))
        homog_p   <- bart_res$p.value

        lev_res   <- rstatix::levene_test(data_complete, current_formula, center = median)
        levene_p  <- lev_res$p[1]

        cat(
          paste0(
            "\n  **Variance tests**",
            "\n- *These results are informational only as Welch's t-test is used regardless.*",
            "\n- Bartlett's test: p = ",
            round(homog_p, 4),
            ifelse(
              homog_p > alpha,
              " (no evidence of unequal variance)",
              " (suggests unequal variance)"
            ),
            "\n- Levene's / Brown-Forsythe: p = ",
            round(levene_p, 4),
            ifelse(
              levene_p > alpha,
              " (no evidence of unequal variance)",
              " (suggests unequal variance)"
            ),
            "\n&nbsp;  \n  ",
            "\n  - These two tests can disagree near the alpha threshold.",
            "\n&nbsp;  \n  "
          )
        )

        output_list[[response_name]][["homog_p_bartlett"]] <- homog_p
        output_list[[response_name]][["homog_p_levene"]]   <- levene_p
      } else {
        output_list[[response_name]][["homog_p_bartlett"]] <- NA
        output_list[[response_name]][["homog_p_levene"]]   <- NA
      }

      if (is_one_sample) {
        test_res <- t.test(
          group1,
          mu = mu,
          alternative = alternative,
          conf.level = conf.level
        )
        call_str <- paste0("t.test(", response_name, ", mu = ", mu, ")")
      } else {
        test_res <- t.test(
          group1,
          group2,
          paired = paired,
          var.equal = var.equal,
          alternative = alternative,
          mu = mu,
          conf.level = conf.level
        )
        type_str <- if (paired)
          "Paired"
        else if (var.equal)
          "Student's"
        else
          "Welch's"
        call_str <- paste0(type_str,
                           " t.test(",
                           response_name,
                           " by ",
                           predictor_name,
                           ")")
      }

      output_list[[response_name]][["t_test"]] <- test_res
      output_list[[response_name]][["t_call"]] <- call_str

      # Use safe_shapiro() so edge cases (n < 3, n > 5000) return a
      # shaped htest with NA p-value instead of erroring. Downstream
      # code must handle NA in $p.value and $statistic.
      shapiro_res <- safe_shapiro(normality_values)
      adt_res     <- nortest::ad.test(normality_values)
      data_label  <- if (paired)
        "Differences"
      else if (is_one_sample)
        "Data"
      else
        "Residuals"

      if (is.na(shapiro_res$p.value)) {
        cat(
          paste0(
            "\n- **Shapiro-Wilk** skipped (", shapiro_res$method, "): ",
            data_label, " normality cannot be tested at this sample ",
            "size, the transformation decision falls back to the ",
            "Anderson-Darling result below.",
            "\n- Anderson-Darling (A = ",
            round(adt_res$statistic, 4),
            ", p = ",
            round(adt_res$p.value, 4),
            "): ",
            data_label,
            ifelse(adt_res$p.value > alpha, " are Normal", " are NOT Normal"),
            ", *drives transformation decision (Shapiro skipped)* \n"
          )
        )
      } else {
        cat(
          paste0(
            "\n- **Shapiro-Wilk** (W = ",
            round(shapiro_res$statistic, 4),
            ", p = ",
            round(shapiro_res$p.value, 4),
            "): ",
            data_label,
            ifelse(
              shapiro_res$p.value > alpha,
              paste0(" **are Normal** (p > ", alpha, ")"),
              paste0(" are **NOT Normal** (p \u2264 ", alpha, ")")
            ),
            ", *drives transformation decision*",
            "\n- Anderson-Darling (A = ",
            round(adt_res$statistic, 4),
            ", p = ",
            round(adt_res$p.value, 4),
            "): ",
            data_label,
            ifelse(adt_res$p.value > alpha, " are Normal", " are NOT Normal, "),
            "*diagnostic only* \n"
          )
        )
      }

      output_list[[response_name]][["shapiro_res"]] <- shapiro_res
      output_list[[response_name]][["adt_res"]]     <- adt_res
      output_list[[response_name]][["alpha"]]       <- alpha

      temp_file <- tempfile(fileext = ".png")
      png(
        temp_file,
        width = 7.8,
        height = 7.8,
        units = "in",
        res = 300
      )
      par(mfrow = c(2, 2), mar = c(4, 4, 4, 1))

      boxplot(
        normality_values,
        main = paste0("Boxplot (", data_label, ")"),
        col = "lightblue"
      )
      try(f_hist(normality_values, xlab = data_label), silent = TRUE)
      try(f_qqnorm(normality_values), silent = TRUE)

      if (!is_one_sample && !paired) {
        boxplot(
          current_formula,
          data = data_complete,
          main = "Raw Data by Group",
          col = "lightgrey"
        )
      } else {
        plot(
          normality_values,
          main = paste0(data_label, ", Index Plot"),
          ylab = data_label
        )
      }
      dev.off()

      if (isTRUE(norm_plots)) {
        cat("\n     \n&nbsp;   \nCheck the plots below to assess assumptions.  \n")
        cat(paste0("![](", temp_file, ")"), "  \n\n")
        cat("&nbsp;\n  \n")
      }
      output_list[[response_name]][["normality_plots"]] <- temp_file

      if (output_type %in% c("pdf", "word"))
        cat("\n<div style=\"page-break-after: always;\"></div>\n\\newpage")

      # Decide whether normality is violated. Prefer Shapiro-Wilk when
      # available (most powerful for n <= 5000); fall back to
      # Anderson-Darling when Shapiro was skipped (n outside 3..5000)
      # so the decision is still made on test evidence rather than
      # silently defaulting to FALSE. If both are unavailable, leave
      # the trigger FALSE and let the user inspect the qq-plot.
      normality_violated <- if (!is.na(shapiro_res$p.value)) {
        shapiro_res$p.value < alpha
      } else if (!is.na(adt_res$p.value)) {
        adt_res$p.value < alpha
      } else {
        FALSE
      }

      needs_trans <- normality_violated ||
        response_name %in% force_transformation

      if (needs_trans) {
        if (isFALSE(transformation)) {
          cat(
            paste0(
              "  \n\n**WARNING** Normality assumption violated for '",
              response_name,
              "'. Consider enabling `transformation = TRUE`.  \n\n"
            )
          )
        } else {
          if (paired && !is_one_sample) {
            values_to_transform <- group1 - group2
            effective_trans     <- "bestnormalize"
            trans_subject_label <- paste0("differences (", levs[1], " - ", levs[2], ")")
            if (!identical(transformation, "bestnormalize")) {
              cat(
                "\n*Note: For paired tests, **bestNormalize** (Yeo-Johnson) is applied to the differences, as Box-Cox requires strictly positive values.* \n"
              )
            }
          } else {
            values_to_transform <- data_complete[[response_name]]
            effective_trans     <- transformation
            trans_subject_label <- response_name
          }

          if (identical(effective_trans, "bestnormalize")) {
            cat("\n  \n## BestNormalize transformation of: ",
                trans_subject_label,
                "  \n  \n")
            transformed_var <- f_bestNormalize(
              values_to_transform,
              data_name   = trans_subject_label,
              output_type = "rmd",
              plots       = isTRUE(norm_plots),
              alpha       = alpha
            )
            cat(transformed_var$rmd)
            new_vals        <- transformed_var$transformed_data
            trans_name      <- transformed_var$transformation_name
            is_boxcox_trans <- FALSE
          } else {
            cat("\n  \n## Box-Cox transformation of: ",
                trans_subject_label,
                "  \n  \n")
            capture.output(
              transformed_var <- f_boxcox(
                values_to_transform,
                output_type = "rmd",
                alpha = alpha
              )
            )
            cat(transformed_var$rmd)
            new_vals        <- transformed_var$transformed_data
            trans_name      <- "Box-Cox"
            is_boxcox_trans <- TRUE
          }

          output_list[[response_name]][["trans_object"]] <- transformed_var
          output_list[[response_name]][["is_boxcox"]]    <- is_boxcox_trans

          mu_for_test <- mu
          if (mu != 0) {
            if (!is_one_sample && !paired) {
              cat(
                paste0(
                  "\n**Note:** `mu = ",
                  mu,
                  "` is a difference of means and cannot be meaningfully forward-transformed. The original `mu = ",
                  mu,
                  "` is used on the transformed scale as an approximation. Interpret with caution.  \n"
                )
              )
            } else {
              mu_t <- forward_transform(mu, transformed_var, is_boxcox_trans)
              if (!is.null(mu_t)) {
                mu_for_test    <- mu_t
                mu_transformed <- mu_t
                cat(
                  paste0(
                    "\n- *mu forward-transformed: ",
                    mu,
                    " \u2192 ",
                    round(mu_t, 4),
                    " (",
                    trans_name,
                    " scale)* \n"
                  )
                )
              }
            }
          }

          if ((paired && !is_one_sample) || is_one_sample) {
            group1_t   <- new_vals
            group2_t   <- NULL
            res_t      <- new_vals
            test_res_t <- t.test(new_vals, mu = mu_for_test,
                                 alternative = alternative, conf.level = conf.level)
          } else {
            data_complete[[response_name]] <- new_vals
            group1_t   <- new_vals[groups == levs[1]]
            group2_t   <- new_vals[groups == levs[2]]
            lm_t       <- lm(current_formula, data = data_complete)
            res_t      <- residuals(lm_t)

            bart_t <- stats::bartlett.test(list(group1_t, group2_t))
            lev_t  <- rstatix::levene_test(data_complete, current_formula, center = median)
            cat(
              paste0(
                "\n- Variance diagnostics on transformed data (diagnostic only):",
                "\n- Bartlett's: p = ",
                round(bart_t$p.value, 4),
                "\n- Levene's / Brown-Forsythe: p = ",
                round(lev_t$p, 4),
                "  \n",
                "\n- These two tests can disagree near the alpha threshold."
              )
            )

            test_res_t <- t.test(
              group1_t,
              group2_t,
              paired = FALSE,
              var.equal = var.equal,
              alternative = alternative,
              mu = mu_for_test,
              conf.level = conf.level
            )
          }

          ci_bt <- back_transform(test_res_t$conf.int,
                                  transformed_var,
                                  is_boxcox_trans)
          if (!is.null(ci_bt))
            ci_backtransformed <- ci_bt

          shapiro_res_t <- safe_shapiro(res_t)
          if (is.na(shapiro_res_t$p.value)) {
            cat(
              paste0(
                "\n## Assumptions of **TRANSFORMED** Data\n\n",
                "- Shapiro-Wilk (Transformed): skipped (",
                shapiro_res_t$method,
                "). Inspect the plots below to assess normality of ",
                "the transformed data.  \n"
              )
            )
          } else {
            cat(
              paste0(
                "\n## Assumptions of **TRANSFORMED** Data\n\n",
                "- Shapiro-Wilk (Transformed): p = ",
                round(shapiro_res_t$p.value, 4),
                " ",
                ifelse(
                  shapiro_res_t$p.value > alpha,
                  "(OK, normality achieved)",
                  "(**Violation, consider a non-parametric test**)"
                ),
                "  \n"
              )
            )
          }

          temp_file_t <- tempfile(fileext = ".png")
          png(
            temp_file_t,
            width = 7.8,
            height = 7.8,
            units = "in",
            res = 300
          )
          par(mfrow = c(2, 2), mar = c(4, 4, 4, 1))
          boxplot(res_t, main = "Boxplot (Transformed)", col = "lightgreen")
          try(f_hist(res_t, xlab = "Transformed"), silent = TRUE)
          try(f_qqnorm(res_t), silent = TRUE)
          if (!is_one_sample && !paired) {
            boxplot(
              current_formula,
              data = data_complete,
              main = "Transformed Data by Group",
              col = "lightgrey"
            )
          } else {
            plot(res_t, main = "Transformed, Index Plot", ylab = "Transformed")
          }
          dev.off()

          if (isTRUE(norm_plots)) {
            cat("\n   \nAssumption plots after transformation:  \n")
            cat(paste0("![](", temp_file_t, ")"), "  \n\n")
          }

          output_list[[response_name]][["t_test_transformed"]]          <- test_res_t
          output_list[[response_name]][["Response_Transformed"]]        <- TRUE
          output_list[[response_name]][["trans_name"]]                  <- trans_name
          output_list[[response_name]][["shapiro_res_transformed"]]     <- shapiro_res_t
          output_list[[response_name]][["transformed_normality_plots"]] <- temp_file_t
          output_list[[response_name]][["mu_transformed"]]              <- mu_transformed
          output_list[[response_name]][["ci_backtransformed"]]          <- ci_backtransformed

          t_test_final         <- test_res_t
          Response_Transformed <- TRUE

          if (output_type %in% c("pdf", "word"))
            cat("\n<div style=\"page-break-after: always;\"></div>\n\\newpage")
        }
      }

      if (!Response_Transformed) {
        t_test_final <- test_res
        output_list[[response_name]][["Response_Transformed"]] <- FALSE
        output_list[[response_name]][["mu_transformed"]]       <- NULL
        output_list[[response_name]][["ci_backtransformed"]]   <- NULL
      }

      # -----------------------------------------------------------------------
      # G. HELPER FUNCTIONS FOR OUTPUT TEXT
      # -----------------------------------------------------------------------

      # Helper: sample statistics text
      generate_sample_stats_txt <- function(is_one_sample, paired, console = FALSE) {

        scale_note <- if (Response_Transformed) {
          if (console) " (transformed scale)" else " *(transformed scale)*"
        } else ""

        if (is_one_sample) {
          m_est <- round(t_test_final$estimate, 3)

          txt <- paste0(
            if (console) "\n  Mean " else "\n-  **Mean ",
            response_name,
            if (console) ": " else ":** ",
            m_est,
            scale_note,
            if (!console && Response_Transformed) "" else ""
          )

          if (Response_Transformed && !is.null(ci_backtransformed)) {
            m_est_bt <- round(back_transform(t_test_final$estimate,
                                             transformed_var,
                                             is_boxcox_trans), 3)
            med_raw  <- round(median(y_vals, na.rm = TRUE), 3)
            txt <- paste0(
              txt,
              if (console)
                paste0("\n  --- back-transformed (original scale) ---",
                       "\n  Back-transformed mean: ", m_est_bt,
                       "\n  Sample median (raw data): ", med_raw)
              else
                paste0("\n\n-  **Back-transformed mean:** ", m_est_bt,
                       " *(original scale)*",
                       "\n-  **Sample median (raw data):** ", med_raw)
            )
          }

        } else if (paired) {
          m_est <- round(t_test_final$estimate, 3)

          txt <- paste0(
            if (console)
              paste0("\n  Mean of differences", scale_note, ": ", m_est)
            else
              paste0("\n-  **Mean of differences", scale_note, ":** ", m_est)
          )

          if (Response_Transformed && !is.null(ci_backtransformed)) {
            m_est_bt <- round(back_transform(t_test_final$estimate,
                                             transformed_var,
                                             is_boxcox_trans), 3)
            med_raw  <- round(median(group1 - group2, na.rm = TRUE), 3)
            txt <- paste0(
              txt,
              if (console)
                paste0("\n  --- back-transformed (original scale) ---",
                       "\n  Back-transformed mean of differences: ", m_est_bt,
                       "\n  Median of raw differences: ", med_raw)
              else
                paste0("\n\n-  **Back-transformed mean of differences:** ", m_est_bt,
                       " *(original scale)*",
                       "\n-  **Median of raw differences:** ", med_raw)
            )
          }

        } else {
          # Two-sample independent
          vec_a <- if (Response_Transformed) group1_t else group1
          vec_b <- if (Response_Transformed) group2_t else group2
          m1    <- mean(vec_a, na.rm = TRUE)
          m2    <- mean(vec_b, na.rm = TRUE)

          txt <- paste0(
            if (console) {
              paste0("\n  Mean ", levs[1], scale_note, ": ", round(m1, 3),
                     "\n  Mean ", levs[2], scale_note, ": ", round(m2, 3),
                     "\n  Difference (", levs[1], " - ", levs[2], "): ",
                     round(m1 - m2, 3))
            } else {
              paste0("\n-  **Mean ", levs[1], scale_note, ":** ", round(m1, 3),
                     "\n-  **Mean ", levs[2], scale_note, ":** ", round(m2, 3),
                     "\n-  **Difference (", levs[1], " - ", levs[2], "):** ",
                     round(m1 - m2, 3))
            }
          )

          if (Response_Transformed && !is.null(ci_backtransformed)) {
            m1_bt   <- round(back_transform(m1, transformed_var, is_boxcox_trans), 3)
            m2_bt   <- round(back_transform(m2, transformed_var, is_boxcox_trans), 3)
            med1_raw <- round(median(group1, na.rm = TRUE), 3)
            med2_raw <- round(median(group2, na.rm = TRUE), 3)

            txt <- paste0(
              txt,
              if (console)
                paste0("\n  --- back-transformed (original scale) ---",
                       "\n  Back-transformed mean ", levs[1], ": ", m1_bt,
                       "\n  Back-transformed mean ", levs[2], ": ", m2_bt,
                       "\n  Sample median ", levs[1], " (raw data): ", med1_raw,
                       "\n  Sample median ", levs[2], " (raw data): ", med2_raw)
              else
                paste0("\n\n-  **Back-transformed mean ", levs[1], ":** ", m1_bt,
                       " *(original scale)*",
                       "\n-  **Back-transformed mean ", levs[2], ":** ", m2_bt,
                       " *(original scale)*",
                       "\n-  **Sample median ", levs[1], " (raw data):** ", med1_raw,
                       "\n-  **Sample median ", levs[2], " (raw data):** ", med2_raw)
            )
          }
        }

        return(txt)
      }

      # Helper: significance text
      generate_sig_txt <- function(console = FALSE) {
        if (t_test_final$p.value <= alpha) {
          if (console)
            paste0("* -> Significant, H0 is rejected  (p \u2264 \u03b1 = ", alpha, ")")
          else
            paste0("* -> **Significant**, $H_{0}$ is rejected  (p $\\leq \\alpha$ = ", alpha, ")")
        } else {
          if (console)
            paste0("-> Not significant, H0 is NOT rejected  (p > \u03b1 = ", alpha, ")")
          else
            paste0("-> Not significant, $H_{0}$ is **NOT** rejected  (p > $\\alpha$ = ", alpha, ")")
        }
      }


      generate_ci_txt <- function(console = FALSE) {

        scale_note <- if (Response_Transformed) {
          if (console) " (transformed scale)" else "transformed scale"
        } else {
          generate_ci_label(is_one_sample, paired, console = console)
        }

        txt <- paste0(
          if (console)
            paste0("\n  ", conf_pct, "% CI", scale_note, ": [ ",
                   round(t_test_final$conf.int[1], 3), ", ",
                   round(t_test_final$conf.int[2], 3), " ]")
          else
            paste0("\n-  **", conf_pct, "% CI** (", scale_note, "):  [ ",
                   round(t_test_final$conf.int[1], 3), ", ",
                   round(t_test_final$conf.int[2], 3), " ]")
        )

        if (Response_Transformed && !is.null(ci_backtransformed)) {

          # Option 4: "back-transformed" label, interpret carefully only for two-sample
          careful_note <- if (!is_one_sample && !paired) {
            if (console) "  *interpret carefully*" else "  *(interpret carefully)*"
          } else {
            ""
          }

          txt <- paste0(
            txt,
            if (console)
              paste0("\n  ", conf_pct, "% CI (back-transformed):  [ ",
                     round(ci_backtransformed[1], 3), ", ",
                     round(ci_backtransformed[2], 3), " ]",
                     careful_note, "\n")
            else
              paste0("\n-  **", conf_pct, "% CI** (back-transformed):  [ ",
                     round(ci_backtransformed[1], 3), ", ",
                     round(ci_backtransformed[2], 3), " ]",
                     careful_note)
          )
        }

        return(txt)
      }


      # Helper: transformation note
      generate_trans_note <- function(console = FALSE) {
        if (!Response_Transformed) return(NULL)

        if (console) {
          paste0(
            "\nNote on transformation:",
            "\nThe t-test was conducted on the ", trans_name, "-transformed scale. ",
            "The back-transformed mean and the sample median of the raw data will ",
            "differ when the transformation is not perfectly normalizing. ",
            "For non-normal data consider f_wilcox_test() which tests the median ",
            "directly without transformation assumptions."
          )
        } else {
          paste0(
            "\n**Note on transformation:**",
            "\n The t-test was conducted on the *", trans_name, "*-transformed scale. ",
            "The back-transformed mean and the sample median of the raw data will ",
            "differ when the transformation is not perfectly normalizing. ",
            "For non-normal data consider `f_wilcox_test()` which tests the median ",
            "directly without transformation assumptions."
          )
        }
      }

      # -----------------------------------------------------------------------
      # G. FINAL SUMMARY PRINTING
      # -----------------------------------------------------------------------

      # Determine title
      method_title <- if (Response_Transformed) {
        paste0(t_test_final$method, " of (", trans_name, " transformed): ", response_name)
      } else {
        paste0(t_test_final$method, " of: ", response_name)
      }

      # Determine levs_for_hyp
      if (is_one_sample) {
        levs_for_hyp <- response_name
        levs2        <- NULL
      } else {
        levs_for_hyp <- levs[1]
        levs2        <- levs[2]
      }

      # p-value formatting
      p_val_fmt <- f_conditional_round(t_test_final$p.value)

      # Generate all text components
      sample_stats_txt  <- generate_sample_stats_txt(is_one_sample, paired, console = FALSE)
      hypotheses_txt    <- generate_hypotheses(
        levs_for_hyp, levs2,
        paired = paired, mu = mu, alternative = alternative,
        console = FALSE
      )
      sig_txt           <- generate_sig_txt(console = FALSE)
      ci_txt            <- generate_ci_txt(console = FALSE)
      trans_note        <- generate_trans_note(console = FALSE)

      # Scale note for test statistics header
      scale_note_hdr <- if (Response_Transformed) " *(transformed scale)*" else ""

      # -------  Write markdown output  ----------------------------------------
      cat(paste0("  \n## ", method_title, "  \n"))
      cat(paste0("\n**Method:** ", t_test_final$method,
                 " (", t_test_final$alternative, ")"))

      cat(paste0("&nbsp;\n  \n&nbsp;\n  \n**SAMPLE STATISTICS:**\n"))
      cat(sample_stats_txt)

      cat(paste0("&nbsp;\n  \n&nbsp;\n  \n**HYPOTHESES:**\n"))
      cat(hypotheses_txt)

      if (!is.null(mu_transformed)) {
        cat(paste0("\n  + *(mu forward-transformed to ",
                   round(mu_transformed, 4), " on ", trans_name, " scale)*"))
      }

      cat(paste0(
        "&nbsp;\n  \n&nbsp;\n  \n**TEST RESULTS", scale_note_hdr, ":**\n",
        "\n-  **t:** ", round(t_test_final$statistic, 3),
        "\n-  **df:** ", round(t_test_final$parameter, 3), "   \n",
        "\n-  **p-value:** ", p_val_fmt, " ", sig_txt, "   \n"
      ))

      cat(paste0("&nbsp;\n  \n**ESTIMATE:**\n"))
      cat(ci_txt)
      cat("&nbsp;\n  \n&nbsp;  \n  \n")
      if (!is.null(trans_note)) cat(trans_note)



      if (n_before != n_after)
        cat(paste0("**WARNING** Removed ", n_before - n_after,
                   " rows with missing values.\n"))

      # -------  Store console versions in output_list  ------------------------
      sample_stats_txt_out <- generate_sample_stats_txt(is_one_sample, paired, console = TRUE)
      hypotheses_txt_out   <- generate_hypotheses(
        levs_for_hyp, levs2,
        paired = paired, mu = mu, alternative = alternative,
        console = TRUE
      )
      sig_txt_out          <- generate_sig_txt(console = TRUE)
      ci_txt_out           <- generate_ci_txt(console = TRUE)
      trans_note_out       <- generate_trans_note(console = TRUE)

      output_list[[response_name]][["sample_stats_txt"]]  <- sample_stats_txt_out
      output_list[[response_name]][["hypotheses_txt"]]     <- hypotheses_txt_out
      output_list[[response_name]][["sig_txt"]]            <- sig_txt_out
      output_list[[response_name]][["ci_txt"]]             <- ci_txt_out
      output_list[[response_name]][["trans_note"]]         <- trans_note_out
      output_list[[response_name]][["alpha"]]              <- alpha
      output_list[[response_name]][["Response_Transformed"]] <- Response_Transformed
      output_list[[response_name]][["trans_name"]]<- if (Response_Transformed) trans_name else NULL
      output_list[[response_name]][["mu_transformed"]]     <- mu_transformed

      cat("&nbsp;\n  \n&nbsp;  \n  \n")

      if (n_before != n_after)
        cat(paste0(
          "**WARNING** Removed ",
          n_before - n_after,
          " rows with missing values.\n"
        ))

      i <- i + 1
      if (output_type %in% c("pdf", "word") && i < length(lhs))
        cat("\n\n<div style=\"page-break-after: always;\"></div>\n\\newpage\n")
    }

    if (output)
      return(output_list)
  }

  # ===========================================================================
  # 8. Execution & File Saving
  # ===========================================================================
  suppressMessages(
    utils::capture.output(
      output_list <- generate_report(),
      file = nullfile()
    )
  )
  class(output_list) <- "f_t_test"


  if (output_type %in% c("word", "pdf")) {
    message(paste0("Saving output in: ", output_path))
    word_pdf_preamble <- function() {
      paste0(
        "
---
title: \"f_t_test Analysis Report\"
date: \"`r Sys.Date()`\"
output:
   word_document:
     reference_docx: !expr system.file(\"rmarkdown/templates/MS_word_style.docx\", package = \"rfriend\")
   pdf_document:
     latex_engine: pdflatex
header-includes:
  - \\usepackage[T1]{fontenc}
  - \\usepackage[utf8]{inputenc}
  - \\usepackage{textcomp}
  - \\DeclareUnicodeCharacter{03BB}{\\ensuremath{\\lambda}}
  - \\DeclareUnicodeCharacter{2264}{\\ensuremath{\\leq}}
  - \\DeclareUnicodeCharacter{2265}{\\ensuremath{\\geq}}
  - \\DeclareUnicodeCharacter{2192}{\\ensuremath{\\rightarrow}}
  - \\DeclareUnicodeCharacter{00D7}{\\ensuremath{\\times}}
  - \\DeclareUnicodeCharacter{2014}{\\textemdash}
  - \\DeclareUnicodeCharacter{03B1}{\\ensuremath{\\alpha}}
  - \\DeclareUnicodeCharacter{2019}{\\textquoteright}
  - \\DeclareUnicodeCharacter{0160}{\\v{S}}
  - \\DeclareUnicodeCharacter{00E1}{\\'{a}}
  - \\usepackage{titling}
  - \\setlength{\\droptitle}{-2.5cm} % Adjust vertical spacing
---
"
      )
    }
    knitr::opts_chunk$set(comment = "")
    generated_markdown <- capture.output(generate_report(output = FALSE))
    rmd_content <- paste(word_pdf_preamble(),
                         paste(generated_markdown, collapse = "\n"),
                         sep = "\n")
    writeLines(rmd_content, temp_output_file)
    rmarkdown::render(
      temp_output_file,
      output_file = output_path,
      intermediates_dir = temp_output_dir,
      knit_root_dir = temp_output_dir,
      quiet = TRUE,
      output_format = paste0(output_type, "_document")
    )
    if (open_generated_files)
      try(f_open_file(output_path), silent = TRUE)
    return(invisible(output_list))

  } else if (output_type == "excel") {
    message(paste0("Saving output in: ", output_path))
    summary_tables <- lapply(output_list, function(obj) {
      t_obj <- if (obj$Response_Transformed)
        obj$t_test_transformed
      else
        obj$t_test
      m_str <- paste(round(t_obj$estimate, 3), collapse = " vs ")
      est_names <- names(t_obj$estimate)
      ci_type <- if ("mean of x" %in% est_names &&
                     length(est_names) == 1)
        "Mean"
      else if ("mean of the differences" %in% est_names)
        "Mean Difference"
      else
        "Difference in Means"

      df_out <- data.frame(
        Statistic = t_obj$statistic,
        df = t_obj$parameter,
        p_value = t_obj$p.value,
        Mean = m_str,
        CI_Type = ci_type,
        LowerCI = t_obj$conf.int[1],
        UpperCI = t_obj$conf.int[2],
        Method = t_obj$method,
        Transformed = obj$Response_Transformed
      )
      if (isTRUE(obj$Response_Transformed) &&
          !is.null(obj$ci_backtransformed)) {
        df_out$LowerCI_original_scale <- obj$ci_backtransformed[1]
        df_out$UpperCI_original_scale <- obj$ci_backtransformed[2]
      }
      df_out
    })
    names(summary_tables) <- lhs
    writexl::write_xlsx(summary_tables, path = output_path)
    if (open_generated_files)
      try(f_open_file(output_path), silent = TRUE)
    return(invisible(output_list))

  } else if (output_type == "rmd") {
    if (is.null(knitr::opts_knit$get("output.dir")))
      knitr::opts_knit$set(output.dir = tempdir())
    generated_markdown <- capture.output(generate_report(output = FALSE))
    output_list[["rmd"]] <- paste(generated_markdown, collapse = "\n")
    return(invisible(output_list))

  } else if (output_type == "console") {
    print(output_list)
    return(invisible(output_list))
  }

  invisible(suppressWarnings(file.remove(temp_output_file)))
  return(output_list)
}

# =============================================================================
#' @export
#' @rdname f_t_test
# =============================================================================
f_t_test.default <- function(x,
                             y = NULL,
                             paired = FALSE,
                             var.equal = NULL,
                             conf.level = NULL,
                             mu = 0,
                             alternative = "two.sided",
                             norm_plots = TRUE,
                             transformation = TRUE,
                             force_transformation = NULL,
                             alpha = 0.05,
                             intro_text = TRUE,
                             close_generated_files = FALSE,
                             open_generated_files = interactive(),
                             output_type = "default",
                             save_as = NULL,
                             save_in_wdir = FALSE,
                             ...) {
  if (!is.numeric(x))
    stop("'x' must be numeric for the vector interface.")
  if (!is.null(y) &&
      !is.numeric(y))
    stop("'y' must be numeric or NULL for the vector interface.")
  if (!is.null(y) &&
      paired &&
      length(x) != length(y))
    stop("For a paired test, 'x' and 'y' must have the same length.")

  # Sanitise names: strip data frame prefix (e.g. "mtcars$hp" -> "hp")
  x_name       <- make.names(sub(".*\\$", "", deparse(substitute(x))))
  y_name       <- if (!is.null(y)) {
    make.names(sub(".*\\$", "", deparse(substitute(y))))
  } else{
    NULL
  }

  response_col <- if (is.null(y)) {
    x_name
  } else{
    "y"
  }

  if (!is.null(force_transformation) && !is.null(y)) {

    if (force_transformation %in% c(x_name, y_name, "x", "y")) {
      # Valid group name -- silently redirect to the response column
      force_transformation <- response_col

    } else {
      # Genuinely unrecognised name -- warn and correct
      warning("'force_transformation = \"", force_transformation, "\"' was not ",
              "recognised. In the vector interface, valid values are '",
              x_name, "' or '", y_name, "'. ",
              "Transformation will be forced on the entire response column.")
      force_transformation <- response_col
    }
  }

  # Pairwise deletion of NAs before melting to long format
  if (!is.null(y) && paired) {
    ok <- complete.cases(x, y)
    if (sum(!ok) > 0) {
      warning(sprintf(
        "Removed %d incomplete pairs containing NAs in 'x' or 'y'.",
        sum(!ok)
      ))
      x <- x[ok]
      y <- y[ok]
    }
  }

  if (is.null(y)) {
    df   <- setNames(data.frame(x), x_name)
    form <- as.formula(paste0("`", x_name, "` ~ 1"))
  } else {
    grp_levels <- c(x_name, y_name)
    grp <- factor(rep(grp_levels, times = c(length(x), length(y))), levels = grp_levels)
    df   <- data.frame(y = c(x, y), grp = grp)
    form <- y ~ grp
  }

  out <- f_t_test.formula(
    formula               = form,
    data                  = df,
    paired                = paired,
    var.equal             = var.equal,
    conf.level            = conf.level,
    mu                    = mu,
    alternative           = alternative,
    norm_plots            = norm_plots,
    transformation        = transformation,
    force_transformation  = if (is.null(force_transformation))
      NULL
    else
      response_col,
    alpha                 = alpha,
    intro_text            = intro_text,
    close_generated_files = close_generated_files,
    open_generated_files  = open_generated_files,
    output_type           = output_type,
    save_as               = save_as,
    save_in_wdir          = save_in_wdir,
    ...
  )

  if (!is.null(y)) {
    new_name <- paste0(x_name, "_vs_", y_name)
    names(out)[names(out) == "y"] <- new_name
  }

  out
}

# ============================================================
#  print method
# ============================================================
#' @export
print.f_t_test <- function(x, ...) {
  for (nm in names(x)) {
    if (nm == "rmd") next
    sublist <- x[[nm]]

    t_obj <- if (isTRUE(sublist$Response_Transformed)) {
      sublist$t_test_transformed
    } else {
      sublist$t_test
    }

    # Header
    cat("\n==========================================================\n")
    if (isTRUE(sublist$Response_Transformed)) {
      cat(paste0(t_obj$method, " (", sublist$trans_name,
                 " transformed) of: ", nm, "\n"))
    } else {
      cat(paste0(t_obj$method, " (", t_obj$alternative, ") of: ", nm, "\n"))
    }
    cat("==========================================================\n")

    # Sample statistics
    cat("\nSAMPLE STATISTICS:")
    cat(sublist$sample_stats_txt)

    # Hypotheses
    cat("\n\nHYPOTHESES:")
    cat(sublist$hypotheses_txt)
    if (!is.null(sublist$mu_transformed))
      cat(paste0("\n  (mu on transformed scale: ",
                 round(sublist$mu_transformed, 4), ")"))

    # Test results
    scale_note <- if (isTRUE(sublist$Response_Transformed))
      " (transformed scale)" else ""
    cat(paste0("\n\nTEST RESULTS", scale_note, ":"))
    cat(sprintf(
      "\n  t = %.3f,  df = %.3f,  p-value = %.4f  %s\n",
      t_obj$statistic,
      t_obj$parameter,
      t_obj$p.value,
      # FIX: uses stored alpha, not hardcoded 0.05
      ifelse(t_obj$p.value <= sublist$alpha, "*", "")
    ))
    cat(sublist$sig_txt)

    # Estimate / CI
    cat("\n\nESTIMATE:")
    cat(sublist$ci_txt)

    # Transformation note
    if (!is.null(sublist$trans_note))
      cat(sublist$trans_note)

    cat("\n")
  }
  cat("\n\n")
}

# =============================================================================
#' @export
# =============================================================================
plot.f_t_test <- function(x, ...) {
  plots_shown <- 0
  for (nm in names(x)) {
    if (nm == "rmd")
      next
    sublist <- x[[nm]]
    for (plot_key in c("normality_plots", "transformed_normality_plots")) {
      if (!is.null(sublist[[plot_key]]) &&
          file.exists(sublist[[plot_key]])) {
        if (requireNamespace("png", quietly = TRUE) &&
            requireNamespace("grid", quietly = TRUE)) {
          img <- png::readPNG(sublist[[plot_key]])
          grid::grid.newpage()
          grid::grid.raster(img)
          plots_shown <- plots_shown + 1
        } else {
          warning("Packages 'png' and 'grid' are required to render these plots.")
        }
      }
    }
  }
  if (plots_shown == 0)
    cat("No plots found. Run with norm_plots = TRUE and output_type != 'default'.\n")
  invisible(x)
}

# Convert formula to text and clean using regex
clean_formula <- function(formula) {
  formula_text <- deparse(formula)
  cleaned <- gsub("\\w+\\$", "", formula_text)  # Remove text before $
  output <- as.formula(paste(cleaned, collapse = ""))
  return(output)
}

# =============================================================================
# helper_check_lhs.R
# =============================================================================
# Internal helper. Warns when the LHS of a formula contains expressions
# (function calls, arithmetic) instead of bare variable names.
#
# Why this guard exists
# ---------------------
# rfriend's formula-based functions (f_aov, f_glm, f_kruskal_test, f_lmer,
# f_t_test, f_wilcox_test, f_summary, f_outliers, f_scan, f_stat_wizard,
# f_boxplot) all parse the LHS via:
#
#     lhs <- all.vars(formula[[2]])
#
# all.vars() silently strips function calls. That means:
#
#   log(y)  ~ group   -> LHS extracted as "y" (untransformed!)
#   I(y^2)  ~ group   -> LHS extracted as "y"
#   sqrt(y) ~ group   -> LHS extracted as "y"
#
# In every case the function then operates on the untransformed column,
# producing a correct-looking analysis of the wrong data. This warning
# alerts the user so they can precompute a column and use its name.
#
# Allowed syntax (no warning)
# ---------------------------
#   y ~ group                        bare name
#   y1 + y2 ~ group                  multiple responses (rfriend convention)
#   y1 + y2 + y3 ~ group             any number of responses
#   `my column` ~ group              backtick-quoted non-syntactic name
#   y1 + `my column` ~ group         mix of bare and backticked names
#
# Disallowed syntax (warning fires, naming the offending term)
# -----------------------------------------------------------
#   log(y) ~ group                   function call on LHS
#   I(y^2) ~ group                   I() wrapper on LHS
#   sqrt(y1) + y2 ~ group            one call + one bare name -> warns on sqrt(y1)
#   y * 2 ~ group                    arithmetic on LHS
#
# The warning is emitted with call. = FALSE so that users see a clean
# message without an unhelpful "Error in check_lhs_is_names(...)" prefix.
# =============================================================================

check_lhs_is_names <- function(formula) {

  # Deparse the LHS back to source text. Split on '+' to recover the
  # individual response terms as rfriend would see them.
  lhs_deparsed <- deparse(formula[[2]])
  lhs_terms    <- trimws(strsplit(lhs_deparsed, "\\+")[[1]])

  # Strip surrounding backticks from a term, if present.
  # A backtick-quoted name like `my column` is syntactically a valid
  # reference, so it counts as a bare name for the purpose of this check.
  strip_backticks <- function(t) {
    if (startsWith(t, "`") && endsWith(t, "`")) {
      substr(t, 2L, nchar(t) - 1L)
    } else {
      t
    }
  }

  # A term qualifies as a bare (or backticked) name if:
  #   - after stripping backticks the inner text is a syntactically
  #     valid R name (make.names() does not alter it), OR
  #   - the term as written already matches make.names() unchanged.
  #
  # The two-path check means both plain names ("y1") and backticked
  # non-syntactic names ("`my column`") pass cleanly, while function
  # calls like "log(y)" fail because make.names("log(y)") returns
  # "log.y." which differs from the original.
  is_bare <- vapply(lhs_terms, function(t) {
    if (startsWith(t, "`") && endsWith(t, "`")) {
      # Backtick-wrapped: the wrapping itself is syntactic; the inner
      # contents are allowed to be anything, so this always passes.
      return(TRUE)
    }
    t == make.names(t)
  }, logical(1))

  if (!all(is_bare)) {
    bad <- lhs_terms[!is_bare]
    warning(
      "Expressions on the LHS of the formula are ignored: ",
      paste(bad, collapse = ", "),
      ". \nOnly variable names are used by rfriend. \nPrecompute a column in `data` ",
      "instead (e.g. `data$log_value <- log(data$value)`).",
      call. = FALSE
    )
  }

  invisible(NULL)
}

#' Install and Load Multiple R Packages
#'
#' Checks if the specified packages are installed. If not, it installs them and then loads them into the global R session.
#'
#' @param ... Unquoted or quoted names of packages to be installed and loaded. These should be valid package names available on CRAN.
#'
#' @details The function takes a list or vector indicating package names, installs any that are missing, and loads all specified packages into the global environment of the R session. It uses \code{requireNamespace()} to check for installation and \code{library()} to load the packages.
#'
#' @return None. The function is called for its side effects of installing and loading packages.
#'
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#'
#' @export
# Load multiple packages
f_load_packages <- function(...) {
  packages <- as.character(match.call(expand.dots = TRUE)[-1])
  for(package in packages) {
    if (!requireNamespace(package, quietly = TRUE)) {
      install.packages(package)
    }
    library(package, character.only = TRUE)
  }
}

is_nested <- function(model1, model2) {
  # Initialize result
  result <- list(
    nested = FALSE,
    reason = "Models could not be compared",
    direction = NA
  )

  # reason_message <- "Automatically determined if models were nested and found:\n"

  # Try to extract formulas
  f1 <- tryCatch(formula(model1), error = function(e) NULL)
  f2 <- tryCatch(formula(model2), error = function(e) NULL)

  if (is.null(f1) || is.null(f2)) {
    result$reason <- "that the formula could not be extracted from one or both models."
    return(result)
  }

  # Extract response variables
  resp1 <- as.character(f1)[2]
  resp2 <- as.character(f2)[2]

  # Check if response variables are the same
  if (resp1 != resp2) {
    result$reason <- "that there is a different number of response variables."
    return(result)
  }


  get_model_data <- function(model) {
    # Try various common ways models store data
    data <- NULL

    # Try model.frame first
    data <- tryCatch(model.frame(model), error = function(e) NULL)
    if (!is.null(data)) return(data)

    # Try model$model
    if (!is.null(model$model)) return(model$model)

    # Try model$data
    if (!is.null(model$data)) {
      # Check if model$data is a data frame or a character string
      if (is.data.frame(model$data)) {
        return(model$data)
      } else if (is.character(model$data)) {
        # If it's a character string, try to get the data from global environment
        data_obj <- tryCatch(get(model$data, envir = .GlobalEnv), error = function(e) NULL)
        if (!is.null(data_obj) && is.data.frame(data_obj)) return(data_obj)
      }
    }

    # Try model$call$data
    if (!is.null(model$call$data)) {
      data_name <- tryCatch(as.character(model$call$data), error = function(e) NULL)
      if (!is.null(data_name)) {
        data_obj <- tryCatch(get(data_name, envir = .GlobalEnv), error = function(e) NULL)
        if (!is.null(data_obj) && is.data.frame(data_obj)) return(data_obj)
      }
    }

    # Try environment(formula(model))
    env_data <- tryCatch({
      env <- environment(formula(model))
      if (exists("data", env)) {
        data_obj <- get("data", env)
        if (is.data.frame(data_obj)) {
          return(data_obj)
        } else if (is.character(data_obj)) {
          # If it's a character string, try to get the data from global environment
          data_frame <- tryCatch(get(data_obj, envir = .GlobalEnv), error = function(e) NULL)
          if (!is.null(data_frame) && is.data.frame(data_frame)) return(data_frame)
        }
      }
      NULL
    }, error = function(e) NULL)
    if (!is.null(env_data)) return(env_data)

    # Try to extract data name from call and get it from global environment
    data_name <- tryCatch({
      call_obj <- model$call
      if (is.null(call_obj)) call_obj <- attr(model, "call")
      if (!is.null(call_obj) && "data" %in% names(call_obj)) {
        as.character(call_obj$data)
      } else {
        NULL
      }
    }, error = function(e) NULL)

    if (!is.null(data_name)) {
      data_obj <- tryCatch(get(data_name, envir = .GlobalEnv), error = function(e) NULL)
      if (!is.null(data_obj) && is.data.frame(data_obj)) return(data_obj)
    }

    return(NULL)
  }


  data1 <- get_model_data(model1)
  data2 <- get_model_data(model2)


  if (is.null(is.null(data1)) || is.null(data2)) {
    result$reason <- "the data could not be extracted from one or both models."
    return(result)
  } else if(nrow(data1) != nrow(data2)) {

    # Check if data dimensions match
    result$reason <- "that the number of observations differs between models."
    return(result)
  }

  # Extract terms using multiple approaches
  get_model_terms <- function(model) {
    # Try terms() first
    terms_labels <- tryCatch(attr(terms(model), "term.labels"),
                             error = function(e) NULL)
    if (!is.null(terms_labels)) return(terms_labels)

    # Try extracting from formula
    f <- tryCatch(formula(model), error = function(e) NULL)
    if (!is.null(f)) {
      rhs <- as.character(f)[3]
      if (rhs == "1" || rhs == "0") return(character(0))
      terms_str <- unlist(strsplit(rhs, " \\+ "))
      return(terms_str)
    }

    return(NULL)
  }

  terms1 <- get_model_terms(model1)
  terms2 <- get_model_terms(model2)

  if (is.null(terms1) || is.null(terms2)) {
    result$reason <- "that the predictors could not be extracted from one or both models."
    return(result)
  }

  # Check if one model's terms are a subset of the other
  if (all(terms1 %in% terms2)) {
    result$nested <- TRUE
    result$reason <- "Model 1 is nested within Model 2"
    result$direction <- "1 in 2"
    return(result)
  } else if (all(terms2 %in% terms1)) {
    result$nested <- TRUE
    result$reason <- "Model 2 is nested within Model 1"
    result$direction <- "2 in 1"
    return(result)
  } else {
    result$reason <- "that neither model is nested within the other"
    return(result)
  }
}

#' Summarize a Data Frame with Grouping Variables
#'
#' @description
#' Computes summary statistics (n, mean, sd, etc.) for a specified numerical columns
#' in a data frame. The data can be analyzed as a whole or split by one or more
#' grouping variables.
#'
#' The function returns a formatted data frame and includes options to export
#' the results directly to an 'Excel' file.
#'
#' @param x A data.frame or formula (dispatches to the right method).
#' @param formula A formula specifying the columns (right hand side) to be summarized by groups (left hand side). More columns or groups can be added using \code{-} or \code{+} (e.g., \code{col1 + col2 ~ group1 + group2}) to do a sequential summary for each column parameter.
#' @param data A 'data.frame', 'data.table', or 'tibble'.
#' @param columns The numerical column(s) to summarize if no formula is used. Can be entered as a single character string (e.g., \code{"weight"}) or as a character vector \code{c("weight", "length"}).
#' @param group_vars A character vector specifying the grouping variables in \code{data}
#'   (e.g., \code{c("species", "fertilizer")}) if no formula is used. If \code{NULL}, the entire dataset is summarized.
#' @param show_name Logical. Include variable name. Default \code{TRUE}.
#' @param show_n Logical. Include count (\code{n}). Default \code{TRUE}.
#' @param show_mean Logical. Include mean. Default \code{TRUE}.
#' @param show_sd Logical. Include standard deviation. Default \code{TRUE}.
#' @param show_se Logical. Include standard error. Default \code{TRUE}.
#' @param show_min Logical. Include minimum value. Default \code{TRUE}.
#' @param show_max Logical. Include maximum value. Default \code{TRUE}.
#' @param show_median Logical. Include median. Default \code{TRUE}.
#' @param show_Q1 Logical. Include first quartile (25th percentile). Default \code{TRUE}.
#' @param show_Q3 Logical. Include third quartile (75th percentile). Default \code{TRUE}.
#' @param show_skew Logical. Include Skewness (measure of asymmetry). Default \code{FALSE}.
#' @param show_kurtosis Logical. Include Excess Kurtosis (measure of "tailedness"). Default \code{FALSE}.
#' @param digits Integer. Number of decimal places for the R console output.
#'   Default is \code{2}. If \code{NULL}, no rounding is applied.
#'   (Note: This does not affect the raw numbers exported to Excel).
#' @param export_to_excel Logical. If \code{TRUE}, exports results to an 'Excel' file. Default \code{FALSE}.
#' @param digits_excel Integer. Number of decimal places for the Excel file cells. Default \code{NULL} (no rounding). Defining \code{digits_excel}, sets \code{export_to_excel = TRUE} when excel output is not intended use: \code{digits} instead.
#' @param allow_integer_decimal_mix Logical. If \code{TRUE}, intergers in columns with a mix of integers and non-integers are displayed without decimals. Default \code{FALSE}, meaning if there are one or more numbers with decimals the whole column contains the number of decimals set by \code{digits}.
#' @param save_as Character string. Custom path or filename for the Excel export.
#'   \itemize{
#'     \item If full path: Saves to that location.
#'     \item If filename only: Saves to \code{tempdir()} (unless \code{save_in_wdir = TRUE}).
#'     \item If directory: Saves as "dataname_summary.xlsx" in that directory.
#'   }
#' @param save_in_wdir Logical. If \code{TRUE}, saves to the current working directory. Default \code{FALSE}.
#' @param close_generated_files Logical. If \code{TRUE}, forces Excel to close before saving (Windows only). Default \code{FALSE}.
#' @param open_generated_files Logical. Whether to open the generated output
#'   files after creation. Defaults to \code{TRUE} in an interactive R session
#'   and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
#'   Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.
#' @param check_input Logical. If \code{TRUE}, performs validation checks on inputs. Default \code{TRUE}.
#' @param ... Further arguments forwarded to \code{f_summary.data.frame}.
#'
#'
#' @details
#' The function computes the following statistics:
#' \itemize{
#'   \item \code{n}: number of observations
#'   \item \code{mean}: arithmetic mean
#'   \item \code{sd}: standard deviation
#'   \item \code{se}: standard error (\eqn{sd / \sqrt{n}})
#'   \item \code{min}: minimum value
#'   \item \code{max}: maximum value
#'   \item \code{median}: median value
#'   \item \code{Q1}: 25th percentile
#'   \item \code{Q3}: 75th percentile
#'   \item \code{skew}: Sample skewness (if requested).
#'   \item \code{kurt}: Sample excess kurtosis (if requested).
#' }
#'
#' \code{skew} stands for Skewness which is a measure of asymmetry of a distribution around its mean. Where \code{skew} values near \bold{0} indicate approximate symmetry, while large positive or negative values indicate noticeable asymmetry.
#' \itemize{
#'   \item \code{> 0}: Right-skewed (long or heavier tail to the right).
#'   \item \code{< 0}: Left-skewed  (long or heavier tail to the left).
#'  }
#'
#' \code{kurt} stands for Excess Kurtosis: Tells you about the "tails" and the peak.
#'   \itemize{
#'     \item \code{0}: Same tail heaviness as the normal distribution (mesokurtic).
#'     \item \code{> 0}: Heavier tails than normal (Leptokurtic) -- indicates frequent outliers.
#'     \item \code{< 0}: Lighter tails than normal (Platykurtic) -- indicates fewer (or less extreme) outliers than a normal distribution.
#'  }
#' If \code{group_vars} are provided, the statistics are calculated for each group combination.
#' When \code{export_to_excel = TRUE}, the file is automatically generated.
#'
#' @return A list of class \code{f_summary} containing the results data frame.
#'
#' @author
#' Sander H. van Delden \email{plantmind@proton.me}
#'
#' @examples
#'
#' # --- Example 1: Basic Usage (data.frame notation) ---
#' # Summarize "hp" grouped by "cyl"; columns and group_vars can be positional
#' summary_mtcars <- f_summary(mtcars, columns = "hp", group_vars = "cyl")
#' summary_mtcars <- f_summary(mtcars, "hp", "cyl")  # shorthand equivalent
#' print(summary_mtcars)
#'
#' # --- Example 2: Multiple Columns & Groups with Custom Toggles ---
#' # Summarize "hp" and "disp", grouped by "cyl" and "gear", hide Q1/Q3
#' summary_custom <- f_summary(mtcars,
#'                             columns    = c("hp", "disp"),
#'                             group_vars = c("cyl", "gear"),
#'                             show_Q1    = FALSE,
#'                             show_Q3    = FALSE)
#' print(summary_custom)
#'
#' # --- Example 3: Formula Notation ---
#' # Identical result to Example 2 using formula interface
#' # and export output to excel
#' summary_formula <- f_summary(hp + disp ~ cyl + gear,
#'                              data    = mtcars,
#'                              show_Q1 = FALSE,
#'                              show_Q3 = FALSE,
#'                              export_to_excel = TRUE)
#' print(summary_formula)
#'
#' # --- Example 4: Distributional Stats & Digits ---
#' # Add skewness and kurtosis, control rounding
#' summary_dist <- f_summary(Sepal.Length + Petal.Length ~ Species,
#'                           data          = iris,
#'                           show_skew     = TRUE,
#'                           show_kurtosis = TRUE,
#'                           digits        = 3)
#' print(summary_dist)
#'
#' # --- Example 5: Custom Print Formatting ---
#' summary_iris <- f_summary(iris, "Sepal.Length", group_vars = "Species")
#' print(summary_iris, col_width = 10, table_width = 70)

#' @export
f_summary <- function(x, ...) {
  # Use match.call() to check for 'data' WITHOUT evaluating the arguments.
  # This prevents the "object x not found" error.
  mc <- match.call()

  if (missing(x)) {
    # Check if 'data' was provided in the call (e.g. f_summary(data = mtcars))
    if (!is.null(mc$data)) {
      # Manually retrieve the data object
      x_val <- eval(mc$data, envir = parent.frame())
      # Dispatch manually to the data.frame method
      return(f_summary.data.frame(x_val, ...))
    } else {
      stop("Argument 'x' (or 'data') is missing.")
    }
  }

  # Standard S3 Dispatch
  UseMethod("f_summary")
}

#' @export
#' @rdname f_summary
f_summary.formula <- function(x, data, ...) {
  # x is the formula (e.g., y ~ B0 + B1)

  # Warn if LHS has expressions like log(y) before silently stripping them
  check_lhs_is_names(x) #use helper_check_lhs.R

  # Parse LHS (Response Variable)
  lhs_vars <- all.vars(x[[2]])

  # Parse RHS (Grouping Variables)
  rhs_vars  <- all.vars(x[[3]])
  if (length(rhs_vars) == 0)
    rhs_vars <- NULL

  # Capture the name of the data object passed to the formula method
  data_name_str <- deparse(substitute(data))

  if(length(data_name_str) > 1) data_name_str <- "data"

  # Call the data.frame method
  f_summary.data.frame(x = data,
                       columns = lhs_vars,
                       group_vars = rhs_vars,
                       internal_data_name = data_name_str,
                       ...)
}

#' @export
#' @rdname f_summary
f_summary.data.frame <- function(x,
                                 columns,
                                 group_vars = NULL,
                                 show_name = TRUE,
                                 # Statistics Toggles
                                 show_n = TRUE,
                                 show_mean = TRUE,
                                 show_sd = TRUE,
                                 show_se = TRUE,
                                 show_min = TRUE,
                                 show_max = TRUE,
                                 show_median = TRUE,
                                 show_Q1 = TRUE,
                                 show_Q3 = TRUE,
                                 show_skew = FALSE,
                                 show_kurtosis = FALSE,
                                 digits = NULL,
                                 # File Options
                                 export_to_excel = FALSE,
                                 close_generated_files = FALSE,
                                 open_generated_files = interactive(),
                                 save_as = NULL,
                                 save_in_wdir = FALSE,
                                 check_input = TRUE,
                                 digits_excel = NULL,
                                 allow_integer_decimal_mix = FALSE,
                                 ...) {
  # Map 'x' (S3 standard) back to 'data' (Internal logic)
  data <- x

  # Input Validation & Setup
  if (!is.data.frame(data))
    stop("Input must be a data.frame.")

  # Check if there is one or multiple columns to summarize
  if(length(columns) == 1){
    single_col = TRUE
  } else if(length(columns) > 1){
    single_col = FALSE
  }

  # Validate Group Vars
  if (!is.null(group_vars)) {
    if (!all(group_vars %in% names(data))) {
      missing <- group_vars[!group_vars %in% names(data)]
      stop(paste("Group columns not found:", paste(missing, collapse = ", ")))
    }
  }

  # Safe filename logic
  # Default safety fallback
  data_name <- "data"

  # Check for the hidden internal name (passed from formula method)
  dots <- list(...)
  if ("internal_data_name" %in% names(dots)) {
    data_name <- dots[["internal_data_name"]]
  } else {
    # If no internal name, try to grab it from 'x' using:
    try_name <- try(deparse(substitute(x)), silent = TRUE)
    if (!inherits(try_name, "try-error") && length(try_name) == 1 && nchar(try_name) < 50) {
      data_name <- try_name
    }
  }

  # Create a list to store all outputs in this function
  output_list <- list()
  output_excel<- list()

  # --- WORKER FUNCTION ---
  summary_fun <- function(vec) {
    result <- c()
    if (show_n)
      result["n"] <- sum(!is.na(vec))
    if (show_mean)
      result["mean"] <- mean(vec, na.rm = TRUE)
    if (show_sd)
      result["sd"] <- sd(vec, na.rm = TRUE)
    if (show_se)
      result["se"] <- sd(vec, na.rm = TRUE) / sqrt(length(vec))
    if (show_min)
      result["min"] <- min(vec, na.rm = TRUE)
    if (show_Q1)
      result["Q1"] <- quantile(vec, probs = 0.25, na.rm = TRUE)
    if (show_median)
      result["median"] <- median(vec, na.rm = TRUE)
    if (show_Q3)
      result["Q3"] <- quantile(vec, probs = 0.75, na.rm = TRUE)
    if (show_max)
      result["max"] <- max(vec, na.rm = TRUE)

    if (show_skew || show_kurtosis) {
      n <- length(vec)
      s <- sd(vec, na.rm = TRUE)
      m <- mean(vec, na.rm = TRUE)
      if (show_skew) {
        if (n < 3 || s == 0)
          result["skew"] <- NA
        else {
          m3 <- sum((vec - m)^3) / n
          result["skew"] <- (n * (n - 1)) / (n - 2) * (m3 / s^3)
        }
      }
      if (show_kurtosis) {
        if (n < 4 || s == 0)
          result["kurt"] <- NA
        else {
          m4 <- sum((vec - m)^4) / n
          kurt_val <- ((n + 1) * n * (n - 1)) / ((n - 2) * (n - 3)) * (m4 / s^4)
          result["kurt"] <- kurt_val - (3 * (n - 1)^2) / ((n - 2) * (n - 3))
        }
      }
    }
    return(result)
  }



  for (column in columns) {

    # --- AGGREGATION ---
    if (is.null(group_vars)) {
      raw_result <- summary_fun(data[[column]])
      result <- as.data.frame(t(raw_result))
    } else {
      result <- do.call(data.frame, aggregate(data[[column]], by = data[group_vars], FUN = summary_fun))

      base_cols <- group_vars
      stat_cols <- names(result)[(length(base_cols) + 1):ncol(result)]
      stat_cols <- gsub("^x\\.", "", stat_cols)
      if (length(stat_cols) == 1 && stat_cols == "x") {
        stat_cols <- names(summary_fun(data[[column]]))
      }
      if (show_name == TRUE) {
        names(result) <- c(base_cols, paste(column, stat_cols, sep = "."))
      } else {
        names(result) <- c(base_cols, stat_cols)
      }
    }

    # --- FORMATTING ---

    # For backward compatability use the name "output_df" when only one column is produced.
    if (single_col) {
      output_list[["output_df"]] <- result
    } else {
      output_list[[column]] <- result
    }

    # Use separate rounding for excel or default to no rounding.
    if (!is.null(digits_excel) ) {
      output_excel[[column]] <- f_conditional_round(result,
                                         digits = digits_excel,
                                         allow_integer_decimal_mix = allow_integer_decimal_mix)
    } else {
      output_excel[[column]] <- result
    }

  }

  class(output_list) <- "f_summary"

  # --- EXCEL EXPORT ---
  if (!is.null(save_as) ||
      !is.null(digits_excel) || save_in_wdir == TRUE) {
    export_to_excel = TRUE
  }

  if (export_to_excel) {

    if (close_generated_files)
      system("taskkill /im EXCEL.EXE /f")

    if (save_in_wdir)
      save_dir <- getwd()
    else
      save_dir <- tempdir()

    output_path <- get_save_path(
      save_as = save_as,
      default_name = paste(data_name, "summary.xlsx", sep = "_"),
      default_dir = save_dir,
      file.ext = ".xlsx"
    )

    message(paste0("Saved output in: ", output_path))

    write_xlsx(output_excel, path = output_path)

    if (open_generated_files){
      f_open_file(output_path)
    }
  }

  attr(output_list, "digits") <- digits
  return(output_list)
}


#' Print method for f_summary objects
#'
#' Prints a formatted summary table to the console.
#'
#' @param x Object of class f_summary.
#' @param col_width Integer. Max characters in header before line break. Default \code{6}.
#' @param table_width Integer or \code{NULL}. Characters after which table splits. Default \code{90}.
#' @param digits Integer. Number of decimal digits to use in formatting. Default is \code{3}.
#' @param allow_integer_decimal_mix Logical. If \code{TRUE}, each individual cell is evaluated: integer values are displayed without decimal places, and non-integer values are displayed with the specified number of decimal places, i.e. \code{digits}. Default is \code{FALSE}, when a column contains a mix of integers and decimal values, all values are displayed with the specified number of decimal places. Note: columns containing only integers are **always** displayed without decimal places, regardless of \code{allow_integer_decimal_mix}.
#' @param ... Additional arguments passed to \code{pander}.
#' @return Invisibly returns \code{1}.
#' @export
print.f_summary <- function(x,
                            col_width = 6,
                            table_width = 90,
                            digits = 2,
                            allow_integer_decimal_mix = FALSE,
                            ...) {

  # Get digits from f_summary()
  if (missing(digits) && !is.null(attr(x, "digits"))) {
    digits <- attr(x, "digits")
  }

  # Loop over each category (a, b, etc.)
  for (category in names(x)) {
    # Get the sublist for this category
    sublist <- x[[category]]

    # Round numbers if digits is not NULL
    if (!is.null(digits)) {
      sublist <- f_conditional_round(sublist, digits = digits, allow_integer_decimal_mix = allow_integer_decimal_mix)
    }

    f_pander(sublist, col_width = col_width, table_width = table_width)
  }
}

# =============================================================================
# test-f_summary.R
# =============================================================================
# Covers: basic usage, formula notation, multiple columns/groups, toggle
# combinations, edge cases, input validation, output structure,
# and S3 dispatch cleanup.
#
# Fixtures: local (test_data, na_data, tiny_data, flat_data) - specific to
# this file's assertions about exact statistic values.
# =============================================================================

# --- Shared test data --------------------------------------------------------

test_data <- data.frame(
  value      = c(10, 20, 30, 40, 50, 60, 70, 80, 90, 100),
  weight     = c(1.1, 2.2, 3.3, 4.4, 5.5, 6.6, 7.7, 8.8, 9.9, 11.0),
  group_a    = rep(c("A", "B"), each = 5),
  group_b    = rep(c("X", "Y"), times = 5),
  int_col    = as.integer(c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)),
  stringsAsFactors = FALSE
)

na_data <- data.frame(
  value   = c(1, 2, NA, 4, 5, NA, 7, 8, 9, NA),
  group   = rep(c("A", "B"), each = 5),
  stringsAsFactors = FALSE
)

tiny_data <- data.frame(
  value = c(10, 20),
  group = c("A", "B"),
  stringsAsFactors = FALSE
)

flat_data <- data.frame(
  value = rep(5, 10),
  group = rep(c("A", "B"), each = 5),
  stringsAsFactors = FALSE
)


# =============================================================================
# 1. RETURN STRUCTURE
# =============================================================================
test_that("returns list of class f_summary", {
  res <- f_summary(test_data, columns = "value")
  expect_s3_class(res, "f_summary")
  expect_type(res, "list")
})

test_that("single-column result stored under 'output_df'", {
  res <- f_summary(test_data, columns = "value")
  expect_true("output_df" %in% names(res))
})

test_that("multi-column results stored under each column name", {
  res <- f_summary(test_data, columns = c("value", "weight"))
  expect_true("value"  %in% names(res))
  expect_true("weight" %in% names(res))
  expect_false("output_df" %in% names(res))
})

test_that("result data frame has correct number of rows (no groups)", {
  res <- f_summary(test_data, columns = "value")
  expect_equal(nrow(res$output_df), 1L)
})

test_that("result data frame has correct number of rows (one group)", {
  res <- f_summary(test_data, columns = "value", group_vars = "group_a")
  expect_equal(nrow(res$output_df), 2L)
})

test_that("result data frame has correct number of rows (two groups)", {
  res <- f_summary(test_data, columns = "value", group_vars = c("group_a", "group_b"))
  expect_equal(nrow(res$output_df), 4L)
})


# =============================================================================
# 2. FORMULA INTERFACE
# =============================================================================
test_that("formula notation matches direct call (single column, single group)", {
  res_formula <- f_summary(value ~ group_a, data = test_data)
  res_direct  <- f_summary(test_data, columns = "value", group_vars = "group_a")
  expect_equal(res_formula$output_df, res_direct$output_df)
})

test_that("formula notation works for multiple LHS columns", {
  res <- f_summary(value + weight ~ group_a, data = test_data)
  expect_true("value"  %in% names(res))
  expect_true("weight" %in% names(res))
})

test_that("formula notation works for multiple RHS groups", {
  res <- f_summary(value ~ group_a + group_b, data = test_data)
  expect_equal(nrow(res$output_df), 4L)
})

test_that("formula notation: multiple columns AND multiple groups", {
  res <- f_summary(value + weight ~ group_a + group_b, data = test_data)
  expect_true("value"  %in% names(res))
  expect_true("weight" %in% names(res))
  expect_equal(nrow(res$value), 4L)
  expect_equal(nrow(res$weight), 4L)
})


# =============================================================================
# 3. STATISTIC TOGGLES - ALL ON (default)
# =============================================================================
test_that("default output contains all expected statistic columns (no groups)", {
  res  <- f_summary(test_data, columns = "value")
  cols <- names(res$output_df)
  for (stat in c("n", "mean", "sd", "se", "min", "Q1", "median", "Q3", "max")) {
    expect_true(stat %in% cols, info = paste("Missing stat column:", stat))
  }
})

test_that("skew and kurt absent by default", {
  res  <- f_summary(test_data, columns = "value")
  cols <- names(res$output_df)
  expect_false("skew" %in% cols)
  expect_false("kurt" %in% cols)
})


# =============================================================================
# 4. STATISTIC TOGGLES - INDIVIDUAL ON/OFF
# =============================================================================
toggle_params <- list(
  list(param = "show_n",        col = "n"),
  list(param = "show_mean",     col = "mean"),
  list(param = "show_sd",       col = "sd"),
  list(param = "show_se",       col = "se"),
  list(param = "show_min",      col = "min"),
  list(param = "show_max",      col = "max"),
  list(param = "show_median",   col = "median"),
  list(param = "show_Q1",       col = "Q1"),
  list(param = "show_Q3",       col = "Q3")
)

for (tp in toggle_params) {
  local({
    p <- tp$param
    c <- tp$col
    test_that(paste("setting", p, "= FALSE removes column", c), {
      args      <- list(test_data, columns = "value")
      args[[p]] <- FALSE
      res       <- do.call(f_summary, args)
      expect_false(c %in% names(res$output_df))
    })
  })
}

test_that("show_skew = TRUE adds skew column", {
  res <- f_summary(test_data, columns = "value", show_skew = TRUE)
  expect_true("skew" %in% names(res$output_df))
})

test_that("show_kurtosis = TRUE adds kurt column", {
  res <- f_summary(test_data, columns = "value", show_kurtosis = TRUE)
  expect_true("kurt" %in% names(res$output_df))
})

test_that("show_skew + show_kurtosis = TRUE both present", {
  res <- f_summary(test_data, columns = "value", show_skew = TRUE, show_kurtosis = TRUE)
  expect_true("skew" %in% names(res$output_df))
  expect_true("kurt" %in% names(res$output_df))
})

test_that("turning off ALL optional stats leaves only n", {
  res <- f_summary(test_data, columns = "value",
                   show_mean = FALSE, show_sd = FALSE, show_se = FALSE,
                   show_min = FALSE,  show_max = FALSE, show_median = FALSE,
                   show_Q1 = FALSE,   show_Q3 = FALSE)
  expect_equal(names(res$output_df), "n")
})


# =============================================================================
# 5. TOGGLE COMBINATIONS (grouped)
# =============================================================================
test_that("Q1/Q3 hidden, mean/sd shown, grouped", {
  res  <- f_summary(test_data, columns = "value", group_vars = "group_a",
                    show_Q1 = FALSE, show_Q3 = FALSE)
  cols <- names(res$output_df)
  expect_true("value.mean" %in% cols)
  expect_true("value.sd"   %in% cols)
  expect_false("value.Q1"  %in% cols)
  expect_false("value.Q3"  %in% cols)
})

test_that("only median returned when all other stats off (grouped)", {
  res <- f_summary(test_data, columns = "value", group_vars = "group_a",
                   show_n = FALSE, show_mean = FALSE, show_sd = FALSE,
                   show_se = FALSE, show_min = FALSE, show_max = FALSE,
                   show_Q1 = FALSE, show_Q3 = FALSE)
  stat_cols <- setdiff(names(res$output_df), "group_a")
  expect_equal(stat_cols, "value.median")
})

test_that("skew + kurtosis with groups returns correct columns", {
  res  <- f_summary(test_data, columns = "value", group_vars = "group_a",
                    show_skew = TRUE, show_kurtosis = TRUE)
  cols <- names(res$output_df)
  expect_true(any(grepl("value.skew", cols)))
  expect_true(any(grepl("value.kurt", cols)))
})


# =============================================================================
# 6. CORRECTNESS OF COMPUTED VALUES
# =============================================================================
test_that("n is correct (no groups)", {
  res <- f_summary(test_data, columns = "value")
  expect_equal(res$output_df$n, nrow(test_data))
})

test_that("mean is correct (no groups)", {
  res <- f_summary(test_data, columns = "value")
  expect_equal(res$output_df$mean, mean(test_data$value))
})

test_that("sd is correct (no groups)", {
  res <- f_summary(test_data, columns = "value")
  expect_equal(res$output_df$sd, sd(test_data$value))
})

test_that("se equals sd / sqrt(n) (no groups)", {
  res <- f_summary(test_data, columns = "value")
  expected_se <- sd(test_data$value) / sqrt(nrow(test_data))
  expect_equal(res$output_df$se, expected_se)
})

test_that("min and max are correct (no groups)", {
  res <- f_summary(test_data, columns = "value")
  expect_equal(res$output_df$min, min(test_data$value))
  expect_equal(res$output_df$max, max(test_data$value))
})

test_that("median is correct (no groups)", {
  res <- f_summary(test_data, columns = "value")
  expect_equal(res$output_df$median, median(test_data$value))
})

test_that("Q1 and Q3 are correct (no groups)", {
  res <- f_summary(test_data, columns = "value")
  expect_equal(res$output_df$Q1, unname(quantile(test_data$value, 0.25)))
  expect_equal(res$output_df$Q3, unname(quantile(test_data$value, 0.75)))
})

test_that("group means are correct (one group)", {
  res <- f_summary(test_data, columns = "value", group_vars = "group_a")
  a_mean <- mean(test_data$value[test_data$group_a == "A"])
  b_mean <- mean(test_data$value[test_data$group_a == "B"])
  row_a  <- res$output_df[res$output_df$group_a == "A", ]
  row_b  <- res$output_df[res$output_df$group_a == "B", ]
  expect_equal(row_a$value.mean, a_mean)
  expect_equal(row_b$value.mean, b_mean)
})


# =============================================================================
# 7. DIGITS / ROUNDING
# =============================================================================
test_that("digits = 0 rounds mean to whole number", {
  res <- f_summary(test_data, columns = "value", digits = 0)
  res$output_df$mean <- as.numeric(res$output_df$mean)
  expect_equal(res$output_df$mean %% 1, 0)
})

test_that("digits = 4 preserves at least 4 decimal places for non-integer values", {
  res <- f_summary(data.frame(value = c(1, 2, 3, 7)), columns = "value", digits = 4)
  res$output_df$mean <- as.numeric(res$output_df$mean)
  expect_equal(res$output_df$mean, round(mean(c(1, 2, 3, 7)), 4))
})

test_that("digits = NULL applies no rounding", {
  res <- f_summary(test_data, columns = "value", digits = NULL)
  expect_equal(res$output_df$mean, mean(test_data$value))
})


# =============================================================================
# 8. NA HANDLING
# =============================================================================
test_that("n reflects only non-NA values", {
  res <- f_summary(na_data, columns = "value")
  expect_equal(res$output_df$n, sum(!is.na(na_data$value)))
})

test_that("mean ignores NAs", {
  res <- f_summary(na_data, columns = "value")
  expect_equal(res$output_df$mean, mean(na_data$value, na.rm = TRUE))
})

test_that("grouped summary with NAs: group means are correct", {
  res    <- f_summary(na_data, columns = "value", group_vars = "group")
  a_mean <- mean(na_data$value[na_data$group == "A"], na.rm = TRUE)
  row_a  <- res$output_df[res$output_df$group == "A", ]
  expect_equal(row_a$value.mean, a_mean)
})


# =============================================================================
# 9. EDGE CASES
# =============================================================================
test_that("single-row data returns without error", {
  single <- data.frame(value = 42, group = "A", stringsAsFactors = FALSE)
  expect_no_error(f_summary(single, columns = "value"))
})

test_that("flat (zero-variance) data: sd = 0, skew = NA (guarded)", {
  res <- f_summary(flat_data, columns = "value",
                   show_skew = TRUE, show_kurtosis = TRUE)
  expect_equal(res$output_df$sd, 0)
})

test_that("flat grouped data: skew and kurt are NA (s = 0 guard)", {
  res <- f_summary(flat_data, columns = "value", group_vars = "group",
                   show_skew = TRUE, show_kurtosis = TRUE)
  expect_true(all(is.na(res$output_df[, grep("skew", names(res$output_df))])))
  expect_true(all(is.na(res$output_df[, grep("kurt", names(res$output_df))])))
})

test_that("two-group combo uses both group columns as identifiers", {
  res   <- f_summary(test_data, columns = "value",
                     group_vars = c("group_a", "group_b"))
  cols  <- names(res$output_df)
  expect_true("group_a" %in% cols)
  expect_true("group_b" %in% cols)
})


# =============================================================================
# 10. INPUT VALIDATION
# =============================================================================
test_that("non-existent group column raises error", {
  expect_error(
    f_summary(test_data, columns = "value", group_vars = "nonexistent_col"),
    regexp = "not found"
  )
})

test_that("non-data.frame input raises error", {
  expect_error(f_summary(list(a = 1:5), columns = "a"))
})


# =============================================================================
# 11. MULTIPLE COLUMNS x MULTIPLE GROUPS x TOGGLES (combined stress tests)
# =============================================================================
test_that("stress: 2 columns x 2 groups x skew + kurtosis on", {
  res <- f_summary(test_data,
                   columns    = c("value", "weight"),
                   group_vars = c("group_a", "group_b"),
                   show_skew  = TRUE, show_kurtosis = TRUE)
  for (col in c("value", "weight")) {
    df   <- res[[col]]
    cols <- names(df)
    expect_true("group_a" %in% cols)
    expect_true("group_b" %in% cols)
    expect_true(any(grepl("skew", cols)))
    expect_true(any(grepl("kurt", cols)))
    expect_equal(nrow(df), 4L)
  }
})

test_that("stress: formula 2 cols x 2 groups x Q1/Q3 off x digits = 3", {
  res <- f_summary(value + weight ~ group_a + group_b,
                   data    = test_data,
                   show_Q1 = FALSE, show_Q3 = FALSE,
                   digits  = 3)
  for (col in c("value", "weight")) {
    df   <- res[[col]]
    cols <- names(df)
    expect_false(any(grepl("Q1", cols)))
    expect_false(any(grepl("Q3", cols)))
    expect_equal(nrow(df), 4L)
  }
})

test_that("stress: all toggles on simultaneously", {
  res <- f_summary(test_data,
                   columns      = c("value", "weight"),
                   group_vars   = "group_a",
                   show_n       = TRUE, show_mean     = TRUE, show_sd   = TRUE,
                   show_se      = TRUE, show_min      = TRUE, show_max  = TRUE,
                   show_median  = TRUE, show_Q1       = TRUE, show_Q3   = TRUE,
                   show_skew    = TRUE, show_kurtosis = TRUE,
                   digits       = 2)
  expect_equal(length(res), 2L)
  for (col in c("value", "weight")) {
    expect_equal(nrow(res[[col]]), 2L)
  }
})

test_that("stress: all non-essential stats turned off across 2 groups", {
  res <- f_summary(test_data,
                   columns    = c("value", "weight"),
                   group_vars = c("group_a", "group_b"),
                   show_mean  = FALSE, show_sd     = FALSE, show_se  = FALSE,
                   show_min   = FALSE, show_max    = FALSE, show_Q1  = FALSE,
                   show_Q3    = FALSE, show_median = FALSE)
  for (col in c("value")) {
    stat_cols <- setdiff(names(res[[col]]), c("group_a", "group_b"))
    expect_equal(stat_cols, "value.n")
  }
})

test_that("stress: built-in iris dataset, grouped by Species, all stats", {
  res <- f_summary(iris,
                   columns       = c("Sepal.Length", "Sepal.Width",
                                     "Petal.Length", "Petal.Width"),
                   group_vars    = "Species",
                   show_skew     = TRUE,
                   show_kurtosis = TRUE,
                   digits        = 3)
  expect_equal(length(res), 4L)
  for (col in c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")) {
    expect_equal(nrow(res[[col]]), 3L)
  }
})

test_that("stress: mtcars, 3 columns, 2 groups, formula, digits = 1", {
  res <- f_summary(hp + disp + wt ~ cyl + gear,
                   data   = mtcars,
                   digits = 1)
  expect_true(all(c("hp", "disp", "wt") %in% names(res)))
})


# =============================================================================
# 12. PRINT METHOD SMOKE TEST
# =============================================================================
test_that("print.f_summary runs without error", {
  res <- f_summary(test_data, columns = "value", group_vars = "group_a")
  expect_no_error(print(res))
})

test_that("print.f_summary with custom col_width and table_width", {
  res <- f_summary(iris, "Sepal.Length", group_vars = "Species")
  expect_no_error(print(res, col_width = 10, table_width = 70))
})


# =============================================================================
# 13. S3 dispatch cleanup
# =============================================================================


# ---------------------------------------------------------------------------
# A. Formula interface
# ---------------------------------------------------------------------------
test_that("formula interface works with positional formula", {
  expect_no_error(
    res <- suppressMessages(f_summary(Sepal.Length ~ Species, data = iris))
  )
})

test_that("formula interface works with no grouping (y ~ 1)", {
  expect_no_error(
    res <- suppressMessages(f_summary(Sepal.Length ~ 1, data = iris))
  )
})

# ---------------------------------------------------------------------------
# B. data.frame interface
# ---------------------------------------------------------------------------
test_that("data.frame interface via positional x works", {
  expect_no_error(
    res <- suppressMessages(f_summary(iris, columns = "Sepal.Length",
                           group_vars = "Species"))
  )
})

test_that("data.frame interface via named x works", {
  expect_no_error(
    res <- suppressMessages(f_summary(x = iris, columns = "Sepal.Length",
                           group_vars = "Species"))
  )
})

# ---------------------------------------------------------------------------
# C. match.call shim: f_summary(data = iris, ...)
# ---------------------------------------------------------------------------
test_that("named-only call f_summary(data = iris, columns = ...) works", {
  expect_no_error(
    res <- suppressMessages(f_summary(data = iris, columns = "Sepal.Length",
                           group_vars = "Species"))
  )
})

test_that("missing x AND data errors with informative message", {
  expect_error(
    suppressMessages(f_summary(columns = "Sepal.Length"),
    "missing"
  ))
})

#' Convert a data frame to a contingency table
#'
#' @param df A data frame. Either (a) one column contains row labels and the rest are
#'   numeric, (b) a fully numeric data frame with meaningful \code{rownames()} set, or
#'   (c) a fully numeric data frame with no row labels at all (in which case the data
#'   frame is coerced directly to a table without row labels).
#' @param label_col Index or name of the column containing row labels. If NULL (default),
#'   the function auto-detects the first character/factor column. If no such column is
#'   found, the function falls back to using \code{rownames(df)} when these are
#'   meaningful, otherwise it coerces the data frame to a table as-is.
#' @return A contingency table.
#' @export

df_to_table <- function(df, label_col = NULL) {

  if (is.null(label_col)) {
    label_col <- which(sapply(df, function(col) is.character(col) || is.factor(col)))[1]

    if (is.na(label_col)) {
      # No character/factor column: try to fall back on rownames().
      rn <- rownames(df)
      default_rn <- is.null(rn) || identical(rn, as.character(seq_len(nrow(df))))

      mat <- as.matrix(df)

      if (!default_rn) {
        rownames(mat) <- rn
        message("Auto-detected rownames(df) as the label source.\n",
                "If not correct, use label_col = to pick the correct column.")
      } else {
        message("No label column or rownames() detected; ",
                "coerced data frame to a table as-is.\n",
                "If not correct, use label_col = to pick the correct column.")
      }

      return(as.table(mat))
    }

    message("Auto-detected column '", names(df)[label_col], "' as the label column.\n",
            "If not correct, use label_col = to pick the correct column.")
  }

  labels <- df[[label_col]]
  mat    <- as.matrix(df[, -label_col, drop = FALSE])
  rownames(mat) <- labels
  as.table(mat)
}

#' Statistical Test Wizard
#'
#' @description
#' Analyzes your data structure based on a formula and recommends the appropriate statistical test.
#' Checks variable types, normality of residuals, homogeneity of variance, and checks if f_boxcox
#' transformation can fix non-normality. Recommends rfriend functions as primary code, with
#' base R alternatives shown as fallback.
#'
#' Supports standard formulas including \code{y ~ .}, \code{y ~ as.factor(x)}, and interaction
#' terms. Formulas with random effects (e.g. \code{(1|ID)}) are detected and handled separately.
#' Multivariate responses (e.g. \code{cbind(y1, y2) ~ x}) and transformed responses
#' (e.g. \code{log(y) ~ x}) are not supported.
#'
#' @param x A formula (e.g., \code{y ~ x}) or a data frame. When a formula is
#'   provided, \code{data} must also be supplied. When a data frame is provided,
#'   \code{formula} must be supplied as the second argument.
#' @param formula A formula specifying the relationship (used with the data.frame method).
#' @param data A data frame containing the variables referenced in the formula.
#' @param id_col Character string. Name of the column identifying subjects/blocks
#'   for paired or repeated-measures designs. When supplied, the wizard (a) verifies
#'   the pairing structure (each subject should appear in every group exactly once),
#'   (b) treats the design as paired/repeated measures, and (c) embeds the real column
#'   name into the generated code. Omit for independent-samples designs. Default \code{NULL}.
#' @param data_name Character string to name the data base used. Default \code{NULL},
#' which automatically derives the data name from the data frame used as input.
#' @param plots Logical. If \code{TRUE}, generates diagnostic plots using
#'   \code{f_hist()} (histogram of the response) and \code{f_qqnorm()} (QQ-plot
#'   of model residuals). Plots are stored in the result as \code{$histogram}
#'   and \code{$qqplot} (\code{recordedplot} objects) and displayed by the
#'   print method. Default \code{FALSE}.
#' @param output_type Character string specifying the output format of the
#'   recommended rfriend function (when \code{run=TRUE}) and the displayed code
#'   strings. Passed through to \code{f_aov()}, \code{f_t_test()}, \code{f_glm()},
#'   etc. Valid values match those of the underlying function (\code{"word"},
#'   \code{"pdf"}, \code{"excel"}, \code{"console"}, \code{"default"}, \code{"rmd"}).
#'   Default \code{"word"}.
#' @param run Logical. If \code{TRUE}, the wizard attempts to execute the recommended
#'   rfriend function and stores the result in \code{$run_result}. Only works for
#'   unambiguous single-function recommendations (not multi-step or external packages).
#'   Default \code{FALSE}.
#' @param interactive Logical. If \code{TRUE}, asks the user questions about study design. Default \code{FALSE}.
#' @param ... Additional arguments (currently unused).
#'
#' @return An object of class \code{"f_stat_wizard"}: a list containing:
#'   \describe{
#'     \item{formula}{The formula used.}
#'     \item{formula_text}{Character string of the formula.}
#'     \item{data_name}{Name of the data object as passed by the user.}
#'     \item{n}{Effective sample size (after NA removal).}
#'     \item{n_dropped}{Number of rows removed due to missing values.}
#'     \item{paired}{Logical. Whether a paired/repeated-measures design was detected (via \code{id_col}).}
#'     \item{id_col}{Character. Name of the subject/block column supplied, or \code{NULL}.}
#'     \item{y_var}{Name of the response variable.}
#'     \item{y_type}{Detected type of the response: \code{"binary"}, \code{"count"},
#'       \code{"multinomial"}, \code{"ratio_normal"}, \code{"ratio_non_normal"},
#'       \code{"ratio_unknown"}, or \code{"unsupported"}.}
#'     \item{x_vars}{Character vector of explanatory variable names.}
#'     \item{x_types}{Character vector of detected types (\code{"nominal"},
#'       \code{"ordinal"}, \code{"ratio"}).}
#'     \item{n_groups}{Number of groups (for single categorical X), or \code{NULL}.}
#'     \item{group_sizes}{Table of per-group sample sizes, or \code{NULL}.}
#'     \item{is_ancova}{Logical. \code{TRUE} if the model mixes nominal and ratio predictors.}
#'     \item{has_interaction}{Logical. \code{TRUE} if interaction terms were detected.}
#'     \item{normality}{A list with \code{p_value} (Shapiro-Wilk) and \code{is_normal} (logical or \code{NA}).}
#'     \item{variance}{A list with \code{test_used} (\code{"Levene"} or \code{"Bartlett"}),
#'       \code{p_value}, and \code{is_equal} (logical).}
#'     \item{boxcox}{A list with \code{attempted} (logical), \code{can_fix} (logical),
#'       and \code{p_value_after} (numeric or \code{NA}).}
#'     \item{overdispersion}{A list with \code{is_overdispersed} (logical, from
#'       DHARMa dispersion test) and \code{p_value}. Only meaningful for count data.}
#'     \item{recommended_call}{A language object representing the rfriend function call,
#'       or \code{NULL} if no single function could be determined.}
#'     \item{run_result}{The result of executing the recommended test (when \code{run=TRUE}),
#'       or \code{NULL}.}
#'     \item{histogram}{A \code{recordedplot} from \code{f_hist()} (when \code{plots=TRUE}),
#'       or \code{NULL}.}
#'     \item{qqplot}{A \code{recordedplot} from \code{f_qqnorm()} of model residuals
#'       (when \code{plots=TRUE} and Y is continuous), or \code{NULL}.}
#'     \item{report}{Character vector of the human-readable report lines (used by
#'       \code{print.f_stat_wizard}).}
#'   }
#'
#' @export
#' @import stats
#' @examples
#' # Formula interface (recommended)
#' f_stat_wizard(Sepal.Length ~ Species, data = iris)
#'
#' # Data-first interface (backward compatible)
#' f_stat_wizard(iris, Sepal.Length ~ Species)
#'
#' # Paired design -- supply the id_col that identifies matched subjects
#' f_stat_wizard(extra ~ group, data = sleep, id_col = "ID")
#'
#' # With diagnostic plots
#' f_stat_wizard(Sepal.Length ~ Species, data = iris, plots = TRUE)
#'
#' # Run the recommended test directly
#' result <- f_stat_wizard(Sepal.Length ~ Species, data = iris, run = TRUE)
#' result$run_result
#'
#' # Inspect metadata
#' result <- f_stat_wizard(Sepal.Length ~ Species, data = iris)
#' result$y_type
#' result$normality
#' result$group_sizes
# =============================================================================
#' @export
# =============================================================================
f_stat_wizard <- function(x, ...) {
  mc <- match.call()

  if (missing(x)) {
    # Handle f_stat_wizard(data = iris, formula = y ~ x)
    if (!is.null(mc$data)) {
      x_val <- eval(mc$data, envir = parent.frame())
      # Strip data/formula from ... to avoid "matched by multiple actual arguments"
      # (they got swept into ... because the generic only declares `x`)
      dots <- list(...)
      dots$data    <- NULL
      dots$formula <- NULL
      if (!is.null(mc$formula)) {
        frm <- eval(mc$formula, envir = parent.frame())
        return(do.call(f_stat_wizard.data.frame,
                       c(list(x = x_val, formula = frm), dots)))
      }
      return(do.call(f_stat_wizard.data.frame,
                     c(list(x = x_val), dots)))
    }
    stop("Argument 'x' (formula or data) is missing.")
  }

  UseMethod("f_stat_wizard")
}

# =============================================================================
#' @export
#' @rdname f_stat_wizard
# =============================================================================
f_stat_wizard.formula <- function(formula, data, id_col = NULL, run = FALSE,
                                  plots = FALSE, output_type = "word",
                                  interactive = FALSE, data_name = NULL, ...) {

  if (missing(data) || !is.data.frame(data)) {
    stop("When using the formula interface, 'data' must be a data frame.")
  }

  if (is.null(data_name)) {
    data_name <- deparse(substitute(data))
    if (length(data_name) > 1) data_name <- "data"
  }

  # Delegate to the internal workhorse
  .f_stat_wizard_engine(
    formula     = formula,
    data        = data,
    data_name   = data_name,
    id_col      = id_col,
    run         = run,
    plots       = plots,
    output_type = output_type,
    interactive = interactive
  )
}

# =============================================================================
#' @export
#' @rdname f_stat_wizard
# =============================================================================
f_stat_wizard.data.frame <- function(x, formula, id_col = NULL, run = FALSE,
                                     plots = FALSE, output_type = "word",
                                     interactive = FALSE, data_name = NULL, ...) {
  data <- x
  if (missing(formula) || !inherits(formula, "formula")) {
    stop("When passing a data frame, you must also provide a formula (e.g., y ~ x).")
  }

  if (is.null(data_name)) {
    data_name <- deparse(substitute(x))
    if (length(data_name) > 1) data_name <- "data"
  }

  .f_stat_wizard_engine(
    formula     = formula,
    data        = data,
    data_name   = data_name,
    id_col      = id_col,
    run         = run,
    plots       = plots,
    output_type = output_type,
    interactive = interactive
  )
}


# =============================================================================
# INTERNAL ENGINE -- all logic lives here
# =============================================================================
.f_stat_wizard_engine <- function(formula, data, data_name, id_col = NULL,
                                  run = FALSE, plots = FALSE,
                                  output_type = "word", interactive = FALSE) {

  # --- UTILITIES ---
  safe_deparse <- function(expr) paste(deparse(expr), collapse = " ")

  # --- DERIVE PAIRED STATUS FROM id_col ---
  # Presence of id_col = paired/repeated measures design.
  paired <- !is.null(id_col)
  if (paired) {
    if (!is.character(id_col) || length(id_col) != 1) {
      stop("'id_col' must be a single character string naming the subject/block column.")
    }
    if (!id_col %in% names(data)) {
      stop("id_col '", id_col, "' not found in data.")
    }
  }

  # --- 1. INPUT CHECKS ---
  if (!is.data.frame(data)) stop("Input 'data' must be a data frame.")
  if (!inherits(formula, "formula")) stop("You must provide a valid formula (e.g., y ~ x).")

  # --- 2. FORMULA PARSING ---
  formula_text <- safe_deparse(formula)
  y_col <- all.vars(formula[[2]])
  if (length(y_col) != 1) {
    stop("This wizard supports a single response variable only. Found: ",
         paste(y_col, collapse = ", "))
  }

  # Guard: transformed response (log(y), sqrt(y), etc.)
  y_lhs_text <- safe_deparse(formula[[2]])
  if (y_lhs_text != y_col) {
    stop("This wizard does not support transformed responses (e.g. log(y) ~ x). ",
         "Found: ", y_lhs_text, ". Transform your data first, then use a simple formula.")
  }

  if (!y_col %in% names(data)) stop(paste("Response variable", y_col, "not found in data."))

  # Detect random effects EARLY -- exit before further parsing
  has_re_in_formula <- grepl("|", formula_text, fixed = TRUE)

  # Use terms() to expand y ~ . and get proper term labels
  if (has_re_in_formula) {
    x_cols_vars <- setdiff(all.vars(formula[[3]]), y_col)
    x_cols_vars <- x_cols_vars[x_cols_vars %in% names(data)]
  } else {
    tf <- tryCatch(terms(formula, data = data), error = function(e) NULL)
    if (!is.null(tf)) {
      tl <- attr(tf, "term.labels")
      x_cols_vars <- unique(unlist(lapply(tl, function(t) {
        all.vars(as.formula(paste("~", t)))
      })))
    } else {
      x_cols_vars <- all.vars(formula[[3]])
    }
  }

  # Check columns exist
  missing_x <- x_cols_vars[!x_cols_vars %in% names(data)]
  if (length(missing_x) > 0) stop(paste("Explanatory variable(s) not found:", paste(missing_x, collapse = ", ")))

  # --- 3. NA HANDLING ---
  used_cols <- c(y_col, x_cols_vars)
  if (paired) used_cols <- c(used_cols, id_col)
  complete_rows <- complete.cases(data[, used_cols, drop = FALSE])
  n_dropped <- sum(!complete_rows)
  data <- data[complete_rows, , drop = FALSE]
  y_data <- data[[y_col]]

  # --- INITIALISE METADATA (populated throughout, returned in list) ---
  y_type           <- "unknown"
  x_types          <- character()
  n_groups         <- NULL
  is_ancova        <- FALSE
  has_interaction  <- FALSE
  can_be_normalized <- FALSE
  shapiro_p        <- NA_real_
  is_normal        <- NA
  variance_test    <- NA_character_
  variance_p       <- NA_real_
  variance_equal   <- TRUE
  boxcox_attempted <- FALSE
  boxcox_can_fix   <- FALSE
  boxcox_p_after   <- NA_real_
  is_overdispersed <- FALSE         # DHARMa-based overdispersion check
  dispersion_p     <- NA_real_      # p-value from DHARMa dispersion test
  recommended_call <- NULL          # stores the primary rfriend call expression
  run_result       <- NULL          # stores result of run=TRUE execution
  group_sizes      <- NULL          # stores per-group sample sizes
  wizard_histogram <- NULL          # stores f_hist plot of response
  wizard_qqplot    <- NULL          # stores f_qqnorm plot of residuals

  # --- HELPER: build result list ---
  build_result <- function(report_lines) {
    result <- list(
      formula        = formula,
      formula_text   = formula_text,
      data_name      = data_name,
      n              = nrow(data),
      n_dropped      = n_dropped,
      paired         = paired,
      id_col         = id_col,
      y_var          = y_col,
      y_type         = y_type,
      x_vars         = x_cols_vars,
      x_types        = x_types,
      n_groups       = n_groups,
      group_sizes    = group_sizes,
      is_ancova      = is_ancova,
      has_interaction = has_interaction,
      normality      = list(
        p_value   = shapiro_p,
        is_normal = is_normal
      ),
      variance       = list(
        test_used = variance_test,
        p_value   = variance_p,
        is_equal  = variance_equal
      ),
      boxcox         = list(
        attempted    = boxcox_attempted,
        can_fix      = boxcox_can_fix,
        p_value_after = boxcox_p_after
      ),
      overdispersion = list(
        is_overdispersed = is_overdispersed,
        p_value          = dispersion_p
      ),
      recommended_call = recommended_call,
      run_result     = run_result,
      histogram      = wizard_histogram,
      qqplot         = wizard_qqplot,
      report         = report_lines
    )
    class(result) <- "f_stat_wizard"
    result
  }

  # --- TEXT BUILDER ---
  report_lines <- character()
  add_txt <- function(...) { report_lines <<- c(report_lines, paste0(...)) }

  add_txt("---------------------------------------------------")
  add_txt("          rfriend STATISTICAL WIZARD               ")
  add_txt("---------------------------------------------------")
  add_txt(paste0("Model: ", formula_text))
  if (n_dropped > 0) {
    add_txt(paste0("Note: Removed ", n_dropped, " row(s) with missing values (NA)."))
  }
  add_txt(paste0("Effective sample size: n = ", nrow(data)))
  if (paired) {
    add_txt(paste0("Design: PAIRED / Repeated Measures (id_col = '", id_col, "')"))
  }

  # --- PAIRING STRUCTURE VERIFICATION ---
  # Only runs when id_col is supplied AND we have a single categorical predictor.
  # Checks that each subject appears in every group exactly once.
  pairing_verified <- FALSE
  if (paired && length(x_cols_vars) >= 1) {
    tryCatch({
      grp_col <- x_cols_vars[1]
      id_vec  <- data[[id_col]]
      grp_vec <- data[[grp_col]]
      tab <- table(id_vec, grp_vec)
      n_subjects  <- nrow(tab)
      n_grp_levels <- ncol(tab)

      # Check 1: every subject x group cell == 1
      if (all(tab == 1)) {
        add_txt(paste0("   OK: Pairing structure verified (",
                       n_subjects, " subjects \u00d7 ", n_grp_levels, " groups, one observation per cell)."))
        pairing_verified <- TRUE
      } else {
        # Check what's wrong
        missing_cells <- sum(tab == 0)
        multi_cells   <- sum(tab > 1)
        if (missing_cells > 0) {
          add_txt(paste0("   WARN: Unbalanced pairing \u2014 ", missing_cells,
                         " subject\u00d7group combination(s) are missing."))
          add_txt("   Not every subject appears in every group. Pairing may be incomplete.")
        }
        if (multi_cells > 0) {
          add_txt(paste0("   WARN: ", multi_cells,
                         " subject\u00d7group combination(s) have multiple observations."))
          add_txt("   True repeated measures may need averaging or a mixed model (f_lmer).")
        }
      }
    }, error = function(e) {
      add_txt(paste0("   Note: Could not verify pairing structure (", conditionMessage(e), ")."))
    })
  }

  # --- EARLY EXIT: Random effects in formula ---
  if (has_re_in_formula) {
    add_txt(" ")
    add_txt("-> Random effect syntax detected in formula (e.g. '(1|ID)').")
    add_txt("---------------------------------------------------")
    add_txt(" ")
    add_txt("RECOMMENDATION:")
    add_txt("Test: Linear Mixed-Effects Model (LMM)")
    add_txt(paste0("Code: f_lmer(", formula_text, ", data=", data_name, ", output_type=\"", output_type, "\")"))
    add_txt("   (Requires lme4 package. Use f_lmer() or lme4::lmer() directly.)")
    add_txt(" ")
    add_txt("---------------------------------------------------")
    add_txt("The wizard cannot fully analyze mixed-model formulas.")
    add_txt("Verify your random-effects structure and check model diagnostics manually.")
    return(build_result(report_lines))
  }

  # --- INTERACTIVE DESIGN CHECK ---
  has_random_effects <- FALSE
  random_var <- NULL

  if (interactive && base::interactive()) {
    message(">>> INTERACTIVE CHECK <<<")
    ans <- readline(prompt = "Are your observations independent? (y/n): ")
    if (tolower(substr(ans, 1, 1)) == "n") {
      message("   (You indicated dependency, e.g. repeated measures or blocking)")
      ans2 <- readline(prompt = "Do you want to use a Mixed Model? (y/n): ")
      if (tolower(substr(ans2, 1, 1)) == "y") {
        has_random_effects <- TRUE
        random_var <- readline(prompt = "What is your random grouping variable name (e.g. SubjectID)? ")
      }
    }
    message(">>> END CHECK <<<")
  }

  # --- CLASSIFY X (PREDICTORS) ---
  # Use model.frame() to evaluate as.factor(), poly(), etc. as the model sees them
  mf <- tryCatch(model.frame(formula, data = data, na.action = na.pass), error = function(e) NULL)

  if (!is.null(mf) && ncol(mf) > 1) {
    for (i in 2:ncol(mf)) {
      val <- mf[[i]]
      if (is.ordered(val))                                      x_types <- c(x_types, "ordinal")
      else if (is.factor(val) || is.character(val))              x_types <- c(x_types, "nominal")
      else if (is.numeric(val) && length(unique(val)) > 10)     x_types <- c(x_types, "ratio")
      else                                                       x_types <- c(x_types, "nominal")
    }
  } else {
    for (col in x_cols_vars) {
      val <- data[[col]]
      if (is.ordered(val))                                      x_types <- c(x_types, "ordinal")
      else if (is.factor(val) || is.character(val))              x_types <- c(x_types, "nominal")
      else if (is.numeric(val) && length(unique(val)) > 10)     x_types <- c(x_types, "ratio")
      else                                                       x_types <- c(x_types, "nominal")
    }
  }

  n_x <- length(x_types)
  has_ratio_x <- "ratio" %in% x_types
  has_nominal_x <- any(x_types %in% c("nominal", "ordinal"))

  # Detect interaction terms
  has_interaction <- grepl("[*:]", formula_text)
  if (has_interaction) {
    add_txt("   Note: Interaction term(s) detected. Check interaction plots manually.")
  }

  # Detect ANCOVA pattern -- mixed ratio + nominal predictors
  is_ancova <- n_x > 1 && has_ratio_x && has_nominal_x

  # Note: intercept-only formula (y ~ 1) -- continues to Y classification below
  if (n_x == 0) {
    add_txt("-> Explanatory: None (intercept-only model).")
  }

  if (n_x == 1) {
    add_txt(paste0("-> Explanatory: Single variable (", x_types[1], ")."))
    if (x_types[1] %in% c("nominal", "ordinal")) {
      grp_col <- if (!is.null(mf) && ncol(mf) > 1) mf[[2]] else data[[x_cols_vars[1]]]
      n_groups <- length(unique(grp_col))
      group_sizes <- table(grp_col)
      if (n_groups < 2) add_txt(paste0("   WARN: Only ", n_groups, " group(s) found. Cannot compare."))
    } else {
      n_groups <- 0L
    }
  } else if (n_x > 1) {
    label <- if (is_ancova) "ANCOVA pattern (nominal + ratio predictors)" else paste(x_types, collapse = ", ")
    add_txt(paste0("-> Explanatory: Multiple variables (", label, ")."))
  }

  # --- SAMPLE SIZE WARNINGS ---
  if (!is.null(group_sizes) && length(group_sizes) >= 2) {
    min_n <- min(group_sizes)
    max_n <- max(group_sizes)
    if (min_n < 5) {
      add_txt(paste0("   WARN: Very small group detected (min n=", min_n,
                     "). Results may be unreliable."))
    }
    if (max_n / min_n > 3) {
      add_txt(paste0("   WARN: Unbalanced design (group sizes: ",
                     paste(group_sizes, collapse = ", "),
                     "). Consider Welch-type tests."))
    }
  }
  if (nrow(data) < 10) {
    add_txt(paste0("   WARN: Total sample (n=", nrow(data),
                   ") is very small. Consider non-parametric tests."))
  }

  # --- Shapiro-Wilk wrapper ---
  # Uses the package-level safe_shapiro() helper from
  # helper_safe_shapiro.R, which returns a shaped htest with
  # p.value = NA for n < 3 or n > 5000 (outside shapiro.test()'s
  # valid range). Call sites below must handle is.na(p.value);
  # they already do via the `"Normality could not be assessed."`
  # branch, which covers both small-n and large-n cases uniformly.
  #
  # Historical note: this function previously had an inline
  # safe_shapiro() that (a) called add_txt() as a side effect for
  # the n < 3 case and (b) subsampled to 5000 with a fixed seed
  # for n > 5000. Both have been removed for consistency with the
  # rest of the package: every Shapiro call now goes through the
  # same helper and produces NA for out-of-range input, so the
  # wizard will not contradict what f_aov / f_t_test / f_lmer
  # report on the same dataset.

  # --- CLASSIFY Y (RESPONSE) ---

  if (is.factor(y_data) || is.character(y_data)) {
    n_levels <- length(unique(y_data))
    if (n_levels < 2) {
      y_type <- "unsupported"
      add_txt(paste0("-> Response: Categorical but only ", n_levels, " level(s). Cannot analyze."))
    } else {
      y_type <- if (n_levels == 2) "binary" else "multinomial"
      add_txt(paste0("-> Response: Categorical (", y_type, ")."))
    }

  } else if (is.logical(y_data)) {
    y_type <- "binary"
    add_txt("-> Response: Logical (treated as binary).")

  } else if (is.numeric(y_data)) {

    unique_vals <- sort(unique(y_data))

    # Binary 0/1 BEFORE count
    if (length(unique_vals) == 2 && all(unique_vals %in% c(0, 1))) {
      y_type <- "binary"
      add_txt("-> Response: Numeric 0/1 (treated as binary).")

    # Count data: non-negative integers
    } else if (length(unique_vals) > 0 && all(y_data >= 0) && all(y_data == floor(y_data)) &&
               max(y_data) < 1000 && length(unique_vals) >= 3) {
      y_type <- "count"
      add_txt("-> Response: Numeric (detected as COUNT data: non-negative integers).")

      # Skip overdispersion check for intercept-only models -- there's nothing to fit a GLM against
      if (n_x == 0) {
        add_txt("   (Overdispersion check skipped \u2014 no predictors in intercept-only model.)")
      } else {
      # Model-based overdispersion check using DHARMa
      tryCatch({
        poisson_fit <- glm(formula, data = data, family = poisson())
        sim_res <- DHARMa::simulateResiduals(poisson_fit, plot = FALSE)
        disp_test <- DHARMa::testDispersion(sim_res, plot = FALSE)
        dispersion_p <- disp_test$p.value
        if (!is.na(dispersion_p) && dispersion_p < 0.05) {
          is_overdispersed <- TRUE
          add_txt(paste0("   Overdispersion detected (DHARMa dispersion test p=",
                         round(dispersion_p, 3), ")."))
          add_txt("   -> Negative Binomial GLM recommended over Poisson.")
        } else {
          add_txt(paste0("   No significant overdispersion (DHARMa p=",
                         round(dispersion_p, 3), ")."))
          add_txt("   Poisson GLM is appropriate.")
        }
      }, error = function(e) {
        # Fallback: rough var/mean check if DHARMa or glm fails
        y_mean <- mean(y_data)
        y_var <- var(y_data)
        if (y_var > 2 * y_mean) {
          is_overdispersed <<- TRUE
          add_txt(paste0("   Overdispersion likely (var/mean = ", round(y_var / y_mean, 1),
                         "; DHARMa check failed: ", conditionMessage(e), ")."))
          add_txt("   -> Negative Binomial GLM recommended over Poisson.")
        } else {
          add_txt("   Poisson GLM is appropriate (no strong overdispersion).")
          add_txt(paste0("   Note: DHARMa check failed: ", conditionMessage(e)))
        }
      })
      } # end if (n_x > 0)

    } else {
      # Continuous numeric -- check normality
      add_txt("-> Response: Numeric. Checking normality of residuals...")
      if (paired) {
        add_txt("   CAVEAT: For paired tests, normality should be checked on within-pair")
        add_txt("   differences, not model residuals. Verify manually after pairing.")
      }

      n_obs <- nrow(data)
      if (n_obs > 500) {
        add_txt(paste0("   Note: Large sample (n=", n_obs, "). Shapiro-Wilk is very sensitive at this size."))
        add_txt("   Consider also: qqnorm(residuals(model)); qqline(residuals(model))")
      } else if (n_obs < 20) {
        add_txt(paste0("   Note: Small sample (n=", n_obs, "). Shapiro-Wilk has low power."))
      }

      is_normal <- tryCatch({
        temp_model <- lm(formula, data = data)
        resids <- residuals(temp_model)

        # Shapiro-then-AD fallback: prefer Shapiro-Wilk (most powerful
        # for n in [3, 5000]), fall back to Anderson-Darling when
        # Shapiro was skipped (safe_shapiro returned NA), so that the
        # wizard produces a definitive decision on datasets of any
        # size rather than silently falling through to "unknown".
        # Returns list(p = numeric, label = "Shapiro"/"Anderson-Darling"/NA).
        normality_p <- function(vals) {
          sh <- safe_shapiro(vals)
          if (!is.na(sh$p.value)) {
            list(p = sh$p.value, label = "Shapiro-Wilk", short = "Shapiro")
          } else {
            ad <- tryCatch(nortest::ad.test(vals),
                           error = function(e) NULL)
            if (!is.null(ad) && !is.na(ad$p.value)) {
              list(p = ad$p.value, label = "Anderson-Darling", short = "AD")
            } else {
              list(p = NA_real_, label = NA_character_, short = NA_character_)
            }
          }
        }

        np        <- normality_p(resids)
        p_val     <- np$p
        shapiro_p <- p_val  # legacy name kept for downstream compatibility

        if (is.na(p_val)) {
          add_txt("   Normality could not be assessed.")
          NA
        } else if (p_val > 0.05) {
          add_txt(paste0("   OK: Residuals appear Normal (",
                         np$short, " p=", round(p_val, 3), ")."))
          TRUE
        } else {
          add_txt(paste0("   WARN: Residuals NOT Normal (",
                         np$short, " p=", round(p_val, 3), ")."))
          # QQ-plot suggestion for borderline cases
          if (p_val > 0.01 && n_obs > 30) {
            add_txt("   Note: p is close to 0.05 with moderate n. Visual QQ-plot check recommended.")
            add_txt(paste0("   Code: qqnorm(residuals(lm(", formula_text, ", data=", data_name, "))); qqline(...)"))
          }
          # Box-Cox check
          if (all(y_data > 0)) {
            add_txt("   ... Checking if f_boxcox() can fix this ...")
            boxcox_attempted <- TRUE
            if (exists("f_boxcox", mode = "function")) {
              tryCatch({
                bc_res <- suppressMessages(f_boxcox(y_data, plots = FALSE))
                y_trans <- bc_res$transformed_data
                if (!is.null(y_trans) && length(y_trans) == length(y_data)) {
                  data_temp <- data
                  data_temp$y_trans_internal <- y_trans
                  rhs <- safe_deparse(formula[[3]])
                  new_model <- lm(as.formula(paste("y_trans_internal ~", rhs)), data = data_temp)
                  # Same Shapiro-then-AD fallback on the post-transform residuals
                  np_new  <- normality_p(residuals(new_model))
                  new_p   <- np_new$p
                  boxcox_p_after <- new_p
                  if (!is.na(new_p) && new_p > 0.05) {
                    add_txt(paste0("   TIP: f_boxcox fixes normality! (New ",
                                   np_new$short, " p=", round(new_p, 3), ")"))
                    can_be_normalized <- TRUE
                    boxcox_can_fix <- TRUE
                  } else if (!is.na(new_p)) {
                    add_txt(paste0("   Note: f_boxcox did not fix normality (",
                                   np_new$short, " p=", round(new_p, 3), ")."))
                  }
                } else {
                  add_txt("   Note: f_boxcox returned unexpected output, skipping.")
                }
              }, error = function(e) {
                add_txt(paste0("   Note: f_boxcox failed (", conditionMessage(e), "). Skipping."))
              })
            } else {
              add_txt("   (f_boxcox not found, skipping transformation check)")
            }
          }
          FALSE
        }
      }, error = function(e) {
        add_txt(paste0("   Note: Could not calculate residuals (", conditionMessage(e), ")."))
        NA
      })

      if (isTRUE(is_normal))      y_type <- "ratio_normal"
      else if (is.na(is_normal))   y_type <- "ratio_unknown"
      else                         y_type <- "ratio_non_normal"

    } # end continuous numeric
  } else {
    add_txt(paste0("-> Response: Unsupported type (", class(y_data)[1], ")."))
    y_type <- "unsupported"
  }

  # --- DIAGNOSTIC PLOTS (optional) ---
  if (plots && is.numeric(y_data)) {
    tryCatch({
      # Histogram of response variable
      wizard_histogram <- f_hist(y_data, main = paste("Distribution of", y_col))
      add_txt(paste0("   Plot: Histogram of '", y_col, "' generated."))

      # QQ-plot of residuals (if a model can be fitted)
      if (y_type %in% c("ratio_normal", "ratio_non_normal", "ratio_unknown")) {
        temp_model_qq <- tryCatch(lm(formula, data = data), error = function(e) NULL)
        if (!is.null(temp_model_qq)) {
          wizard_qqplot <- f_qqnorm(residuals(temp_model_qq),
                                   main = paste("QQ-plot residuals:", formula_text))
          add_txt("   Plot: QQ-plot of residuals generated.")
        }
      }
    }, error = function(e) {
      add_txt(paste0("   Note: Could not generate plots (", conditionMessage(e), ")."))
    })
  }

  # --- HOMOSCEDASTICITY CHECK ---
  if (n_x == 1 && x_types[1] %in% c("nominal", "ordinal") &&
      y_type %in% c("ratio_normal", "ratio_non_normal", "ratio_unknown") &&
      !is.null(n_groups) && n_groups >= 2) {
    tryCatch({
      grp <- as.factor(if (!is.null(mf) && ncol(mf) > 1) mf[[2]] else data[[x_cols_vars[1]]])
      if (requireNamespace("rstatix", quietly = TRUE)) {
        lev_df <- data.frame(.y = y_data, .group = grp)
        lev <- rstatix::levene_test(lev_df, .y ~ .group)
        lev_p <- lev$p[1]
        variance_test <- "Levene"
        variance_p <- lev_p
        if (lev_p > 0.05) {
          add_txt(paste0("   OK: Variances appear equal (Levene p=", round(lev_p, 3), ")."))
        } else {
          add_txt(paste0("   WARN: Variances UNEQUAL (Levene p=", round(lev_p, 3), ")."))
          variance_equal <- FALSE
        }
      } else {
        bart_df <- data.frame(.y = y_data, .g = grp)
        bart <- bartlett.test(.y ~ .g, data = bart_df)
        variance_test <- "Bartlett"
        variance_p <- bart$p.value
        if (bart$p.value > 0.05) {
          add_txt(paste0("   OK: Variances appear equal (Bartlett p=", round(bart$p.value, 3), ")."))
        } else {
          add_txt(paste0("   WARN: Variances UNEQUAL (Bartlett p=", round(bart$p.value, 3), ")."))
          add_txt("   Note: Bartlett is sensitive to non-normality. Install 'rstatix' for Levene's test.")
          variance_equal <- FALSE
        }
      }
    }, error = function(e) {
      add_txt("   Note: Could not perform variance equality test.")
    })
  }

  add_txt("---------------------------------------------------")
  add_txt(" ")
  add_txt("RECOMMENDATION:")

  # --- HELPER: Code template with user's data name ---
  # Automatically appends output_type="..." to all rfriend function calls
  cf <- function(fn, ...) {
    extra <- paste0(list(...), collapse = "")
    paste0(fn, "(", safe_deparse(formula), ", data=", data_name, extra,
           ", output_type=\"", output_type, "\")")
  }

  # --- RECOMMENDATIONS ---

  if (y_type == "unsupported") {
    add_txt("No recommendation available for this response type.")

  } else if (has_random_effects) {
    add_txt("Test: Linear Mixed-Effects Model (LMM)")
    rhs <- safe_deparse(formula[[3]])
    re <- if (!is.null(random_var) && nzchar(random_var)) random_var else "SubjectID"
    add_txt(paste0("Code: f_lmer(", y_col, " ~ ", rhs, " + (1|", re, "), data=", data_name,
                   ", output_type=\"", output_type, "\")"))

  # === INTERCEPT-ONLY (y ~ 1) ===
  } else if (n_x == 0) {
    add_txt("This is a null/intercept-only model with no explanatory variables.")
    if (y_type %in% c("ratio_normal", "ratio_unknown")) {
      add_txt("Test: One-sample T-test (test whether the mean differs from a value)")
      add_txt(paste0("Code: f_t_test(", y_col, " ~ 1, data=", data_name, ", output_type=\"", output_type, "\")"))
      add_txt(paste0("  (Base R: t.test(", data_name, "$", y_col, "))"))
      recommended_call <- bquote(f_t_test(.(formula), data = data, output_type = .(output_type)))
    } else if (y_type == "ratio_non_normal") {
      add_txt("Test: One-sample Wilcoxon Signed-Rank (non-normal response)")
      add_txt(paste0("Code: wilcox.test(", data_name, "$", y_col, ")"))
      add_txt("   Alternative: f_t_test() with transformation=TRUE")
    } else if (y_type == "binary") {
      add_txt("Test: One-sample proportion test")
      add_txt(paste0("Code: prop.test(sum(", data_name, "$", y_col, "), length(", data_name, "$", y_col, "))"))
    } else if (y_type == "count") {
      add_txt("Note: Descriptive statistics only (no comparison without explanatory variable).")
      add_txt(paste0("Code: f_summary(", data_name, ", columns='", y_col, "')"))
    } else {
      add_txt("Note: No meaningful test without an explanatory variable for this response type.")
    }

  # === MULTIPLE X ===
  } else if (n_x > 1) {
    if (y_type == "binary") {
      add_txt("Test: Multiple Logistic Regression")
      add_txt(paste0("Code: ", cf("f_glm", ", family='binomial'")))
      recommended_call <- bquote(f_glm(.(formula), data = data, family = "binomial", output_type = .(output_type)))
    } else if (y_type == "count") {
      if (is_overdispersed) {
        add_txt("Test: Negative Binomial GLM (overdispersion detected)")
        add_txt(paste0("Code: MASS::glm.nb(", safe_deparse(formula), ", data=", data_name, ")"))
        add_txt("   (No rfriend wrapper yet \u2014 requires MASS package)")
      } else {
        add_txt("Test: Poisson GLM")
        add_txt(paste0("Code: ", cf("f_glm", ", family='poisson'")))
      }
    } else if (y_type == "multinomial") {
      add_txt("Test: Multinomial Logistic Regression")
      add_txt(paste0("Code: nnet::multinom(", safe_deparse(formula), ", data=", data_name, ")"))
      add_txt("   (No rfriend wrapper yet \u2014 requires nnet package)")
    } else if (y_type == "ratio_normal") {
      if (is_ancova) {
        add_txt("Test: ANCOVA (Analysis of Covariance)")
        add_txt("   Your model combines categorical and continuous predictors.")
        add_txt(paste0("Code: ", cf("f_aov", ", ANCOVA=TRUE")))
        recommended_call <- bquote(f_aov(.(formula), data = data, ANCOVA = TRUE, output_type = .(output_type)))
      } else {
        add_txt("Test: Multi-way ANOVA or Linear Regression")
        add_txt(paste0("Code: ", cf("f_aov")))
        recommended_call <- bquote(f_aov(.(formula), data = data, output_type = .(output_type)))
      }
      if (has_nominal_x) {
        add_txt("   Post-hoc: f_aov() includes post-hoc tests (adjust='sidak' by default)")
        add_txt("   Alternative: emmeans::emmeans() for custom contrasts")
      }
    } else if (can_be_normalized) {
      add_txt("Test: ANOVA with automatic Box-Cox transformation")
      add_txt(paste0("Code: ", cf("f_aov")))
      add_txt("   Note: f_aov() auto-applies Box-Cox when residuals are non-normal.")
      add_txt("Alternative: GLM")
      if (all(y_data > 0, na.rm = TRUE)) {
        add_txt(paste0("   Code: ", cf("f_glm", ", family=Gamma(link='log')")))
      } else {
        add_txt("   Consider robust regression or a GLM family suited to your data.")
      }
      recommended_call <- bquote(f_aov(.(formula), data = data, output_type = .(output_type)))
    } else if (y_type == "ratio_unknown") {
      add_txt("Note: Normality could not be assessed (too few observations).")
      add_txt("Option 1: Linear Regression (if normality is plausible)")
      add_txt(paste0("   Code: ", cf("f_aov")))
      add_txt("Option 2: Non-parametric or GLM approach (safer)")
      add_txt(paste0("   Code: ", cf("f_glm")))
      recommended_call <- bquote(f_aov(.(formula), data = data, output_type = .(output_type)))
    } else {
      add_txt("Test: Generalized Linear Model (GLM)")
      if (all(y_data > 0, na.rm = TRUE)) {
        add_txt(paste0("Code: ", cf("f_glm", ", family=Gamma(link='log')")))
        recommended_call <- bquote(f_glm(.(formula), data = data, family = Gamma(link = "log"), output_type = .(output_type)))
      } else {
        add_txt("Consider: Robust regression, rank-based methods, or a suitable GLM family.")
      }
    }

  # === SINGLE X ===
  } else {
    # X is Ratio
    if (x_types[1] == "ratio") {
      if (y_type == "binary") {
        add_txt("Test: Logistic Regression")
        add_txt(paste0("Code: ", cf("f_glm", ", family='binomial'")))
      } else if (y_type == "count") {
        if (is_overdispersed) {
          add_txt("Test: Negative Binomial Regression (overdispersion detected)")
          add_txt(paste0("Code: MASS::glm.nb(", safe_deparse(formula), ", data=", data_name, ")"))
          add_txt("   (No rfriend wrapper yet \u2014 requires MASS package)")
        } else {
          add_txt("Test: Poisson Regression")
          add_txt(paste0("Code: ", cf("f_glm", ", family='poisson'")))
        }
      } else if (y_type == "multinomial") {
        add_txt("Test: Multinomial Logistic Regression")
        add_txt(paste0("Code: nnet::multinom(", safe_deparse(formula), ", data=", data_name, ")"))
        add_txt("   (No rfriend wrapper yet \u2014 requires nnet package)")
      } else if (y_type == "ratio_normal" || can_be_normalized) {
        if (can_be_normalized) {
          add_txt("Note: f_glm() does NOT auto-transform like f_t_test/f_aov. Apply Box-Cox manually:")
          add_txt(paste0("   Step 1: ", data_name, "$", y_col, "_t <- f_boxcox(", data_name, "$", y_col, ")$transformed_data"))
          add_txt(paste0("   Step 2: f_glm(", y_col, "_t ~ ", safe_deparse(formula[[3]]), ", data=", data_name, ")"))
          add_txt("Otherwise, untransformed:")
        }
        add_txt("Test: Linear Regression / Pearson Correlation")
        add_txt(paste0("Code: ", cf("f_glm")))
        recommended_call <- bquote(f_glm(.(formula), data = data, output_type = .(output_type)))
        add_txt(paste0("  (Base R: lm(", safe_deparse(formula), ", data=", data_name, "))"))
        add_txt(paste0("   Visualise: f_corplot(", data_name, ")"))
        add_txt(paste0("   Test: cor.test(", data_name, "$", x_cols_vars[1], ", ", data_name, "$", y_col, ", method='pearson')"))
      } else if (y_type == "ratio_unknown") {
        add_txt("Note: Normality could not be assessed.")
        add_txt("Test: Linear Regression (if plausible) or Spearman Correlation (non-parametric)")
        add_txt(paste0("Code: ", cf("f_glm")))
        add_txt(paste0("   Visualise: f_corplot(", data_name, ")"))
        add_txt(paste0("   Test: cor.test(", data_name, "$", x_cols_vars[1], ", ", data_name, "$", y_col, ", method='spearman')"))
      } else {
        add_txt("Test: Spearman Correlation or GLM")
        add_txt(paste0("Code: ", cf("f_glm")))
        add_txt(paste0("   Visualise: f_corplot(", data_name, ")"))
        add_txt(paste0("   Test: cor.test(", data_name, "$", x_cols_vars[1], ", ", data_name, "$", y_col, ", method='spearman')"))
      }

    # X is Ordinal
    } else if (x_types[1] == "ordinal") {
      if (y_type == "binary") {
        add_txt("Test: Logistic Regression with ordinal contrast")
        add_txt(paste0("Code: ", cf("f_glm", ", family='binomial'")))
      } else if (y_type == "count") {
        if (is_overdispersed) {
          add_txt("Test: Negative Binomial GLM with ordinal predictor")
          add_txt(paste0("Code: MASS::glm.nb(", safe_deparse(formula), ", data=", data_name, ")"))
          add_txt("   (No rfriend wrapper yet \u2014 requires MASS package)")
        } else {
          add_txt("Test: Poisson GLM with ordinal predictor")
          add_txt(paste0("Code: ", cf("f_glm", ", family='poisson'")))
        }
      } else if (y_type == "multinomial") {
        add_txt("Test: Multinomial Logistic Regression")
        add_txt(paste0("Code: nnet::multinom(", safe_deparse(formula), ", data=", data_name, ")"))
        add_txt("   (No rfriend wrapper yet \u2014 requires nnet package)")
      } else if (y_type == "ratio_normal") {
        add_txt("Test: Linear Regression with polynomial contrasts (ordinal X)")
        add_txt(paste0("Code: ", cf("f_glm")))
        add_txt(paste0("  (Base R: lm(", safe_deparse(formula), ", data=", data_name, "))"))
        add_txt("   Alternatively: f_kruskal_test() treating groups as unordered")
      } else if (y_type == "ratio_non_normal") {
        add_txt("Test: Kruskal-Wallis Test (non-normal Y with ordinal X)")
        add_txt(paste0("Code: f_kruskal_test(", safe_deparse(formula), ", data=", data_name, ", output_type=\"", output_type, "\")"))
        add_txt("   Alternative: Jonckheere-Terpstra trend test (if order matters)")
      } else if (y_type == "ratio_unknown") {
        add_txt("Note: Normality could not be assessed.")
        add_txt("Option 1: Linear Regression with polynomial contrasts")
        add_txt(paste0("   Code: ", cf("f_glm")))
        add_txt("Option 2: Kruskal-Wallis (safer)")
        add_txt(paste0("   Code: f_kruskal_test(", safe_deparse(formula), ", data=", data_name, ", output_type=\"", output_type, "\")"))
      }

    # X is Nominal
    } else {
      if (is.null(n_groups) || n_groups < 2) {
        add_txt("   Note: < 2 groups found. Cannot compare groups.")

      # --- GUARD: paired but pairing failed verification ---
      } else if (paired && !pairing_verified) {
        add_txt("RECOMMENDATION SUPPRESSED: pairing structure failed verification.")
        add_txt("   The id_col was supplied but the data does not form a clean")
        add_txt("   one-row-per-(subject \u00d7 group) layout required by paired tests.")
        add_txt("   Options:")
        add_txt("   1. Fix the data so each subject appears exactly once per group, then re-run.")
        add_txt("   2. Drop id_col to run an independent-samples test instead.")
        add_txt("   3. If you have multiple measurements per subject per group,")
        add_txt("      use a mixed model that models the dependency directly:")
        add_txt(paste0("      Code: f_lmer(", y_col, " ~ ", x_cols_vars[1],
                       " + (1|", id_col, "), data=", data_name,
                       ", output_type=\"", output_type, "\")"))
        recommended_call <- NULL  # block run=TRUE from executing a broken test

      # --- 2 GROUPS ---
      } else if (n_groups == 2) {
        paired_str <- if (paired) ", paired=TRUE" else ""

        if (y_type == "binary") {
          if (paired) {
            add_txt("Test: Paired binary data requires McNemar's Test")
            add_txt("   McNemar's test needs a 2x2 table of before/after outcomes per subject.")
            add_txt("   This requires wide-format data with subject IDs \u2014 the wizard cannot")
            add_txt("   construct this automatically from a formula.")
            add_txt("   See: ?mcnemar.test for usage once your data is structured.")
          } else {
            add_txt("Test: Chi-square / Fisher's Exact")
            add_txt(paste0("Code: f_chisq_test(", data_name, ", '", y_col, "', '", x_cols_vars[1], "', output_type=\"", output_type, "\")"))
          }
        } else if (y_type == "count") {
          if (is_overdispersed) {
            add_txt("Test: Negative Binomial GLM (overdispersion detected)")
            add_txt(paste0("Code: MASS::glm.nb(", safe_deparse(formula), ", data=", data_name, ")"))
            add_txt("   (No rfriend wrapper yet \u2014 requires MASS package)")
          } else {
            add_txt("Test: Poisson GLM")
            add_txt(paste0("Code: ", cf("f_glm", ", family='poisson'")))
          }
          if (paired) add_txt("   Note: For paired count data, consider a mixed model approach.")
        } else if (y_type == "multinomial") {
          add_txt("Test: Chi-square / Fisher's Exact")
          add_txt(paste0("Code: f_chisq_test(", data_name, ", '", y_col, "', '", x_cols_vars[1], "', output_type=\"", output_type, "\")"))
          if (paired) add_txt("   Note: For paired multinomial data, consider marginal homogeneity test.")
        } else if (y_type == "ratio_normal") {
          if (paired) {
            add_txt("Test: Paired T-test")
            add_txt(paste0("Code: f_t_test(", safe_deparse(formula), ", data=", data_name, ", paired=TRUE, output_type=\"", output_type, "\")"))
            recommended_call <- bquote(f_t_test(.(formula), data = data, paired = TRUE, output_type = .(output_type)))
          } else {
            add_txt("Test: Welch's T-test (default; robust to unequal variances)")
            add_txt(paste0("Code: f_t_test(", safe_deparse(formula), ", data=", data_name, ", output_type=\"", output_type, "\")"))
            recommended_call <- bquote(f_t_test(.(formula), data = data, output_type = .(output_type)))
            if (variance_equal) {
              add_txt("   (Variances appear equal; you may also use f_t_test(..., var.equal=TRUE) for Student's t-test)")
            }
          }
          add_txt(paste0("  (Base R: t.test(", safe_deparse(formula), ", data=", data_name, paired_str, "))"))
          add_txt("   Effect size: rstatix::cohens_d() for Cohen's d")
        } else if (can_be_normalized) {
          add_txt("Test: T-test with automatic Box-Cox transformation")
          add_txt(paste0("Code: f_t_test(", safe_deparse(formula), ", data=", data_name, paired_str, ", output_type=\"", output_type, "\")"))
          add_txt("   Note: f_t_test() auto-applies Box-Cox when residuals are non-normal.")
          if (!variance_equal) {
            add_txt("   Note: Variances are also unequal. Verify after transformation;")
            add_txt("   if still unequal, prefer the non-parametric alternative.")
          }
          add_txt("Alternative: Wilcoxon (Non-parametric)")
          add_txt(paste0("   Code: f_wilcox_test(", safe_deparse(formula), ", data=", data_name, paired_str, ", output_type=\"", output_type, "\")"))
          if (paired) {
            recommended_call <- bquote(f_t_test(.(formula), data = data, paired = TRUE, output_type = .(output_type)))
          } else {
            recommended_call <- bquote(f_t_test(.(formula), data = data, output_type = .(output_type)))
          }
        } else if (y_type == "ratio_unknown") {
          add_txt("Note: Normality could not be assessed.")
          if (paired) {
            add_txt("Option 1: Paired T-test (if normality is plausible)")
            add_txt(paste0("   Code: f_t_test(", safe_deparse(formula), ", data=", data_name, ", paired=TRUE, output_type=\"", output_type, "\")"))
            add_txt("Option 2: Wilcoxon Signed-Rank Test (safer)")
            add_txt(paste0("   Code: f_wilcox_test(", safe_deparse(formula), ", data=", data_name, ", paired=TRUE, output_type=\"", output_type, "\")"))
            recommended_call <- bquote(f_t_test(.(formula), data = data, paired = TRUE, output_type = .(output_type)))
          } else {
            add_txt("Option 1: Welch's T-test (if normality is plausible)")
            add_txt(paste0("   Code: f_t_test(", safe_deparse(formula), ", data=", data_name, ", output_type=\"", output_type, "\")"))
            add_txt("Option 2: Mann-Whitney U / Wilcoxon (safer)")
            add_txt(paste0("   Code: f_wilcox_test(", safe_deparse(formula), ", data=", data_name, ", output_type=\"", output_type, "\")"))
            recommended_call <- bquote(f_t_test(.(formula), data = data, output_type = .(output_type)))
          }
        } else {
          if (paired) {
            add_txt("Test: Wilcoxon Signed-Rank Test (paired, non-parametric)")
            add_txt(paste0("Code: f_wilcox_test(", safe_deparse(formula), ", data=", data_name, ", paired=TRUE, output_type=\"", output_type, "\")"))
            recommended_call <- bquote(f_wilcox_test(.(formula), data = data, paired = TRUE, output_type = .(output_type)))
          } else {
            add_txt("Test: Mann-Whitney U / Wilcoxon (Non-parametric)")
            add_txt(paste0("Code: f_wilcox_test(", safe_deparse(formula), ", data=", data_name, ", output_type=\"", output_type, "\")"))
            recommended_call <- bquote(f_wilcox_test(.(formula), data = data, output_type = .(output_type)))
          }
        }

      # --- >2 GROUPS ---
      } else if (n_groups > 2) {
        if (paired && y_type %in% c("ratio_normal", "ratio_non_normal", "ratio_unknown")) {
          # Paired / repeated measures with >2 groups
          id_name <- id_col  # guaranteed non-NULL when paired==TRUE
          if (y_type == "ratio_normal") {
            add_txt("Test: Repeated Measures ANOVA")
            add_txt("   Your data has >2 paired groups with normal residuals.")
            add_txt("   Consider a mixed model for flexibility:")
            add_txt(paste0("   Code: f_lmer(", y_col, " ~ ", x_cols_vars[1], " + (1|", id_name, "), data=", data_name,
                           ", output_type=\"", output_type, "\")"))
            add_txt("   Alternative (rfriend): f_aov with Error() term")
            add_txt(paste0("   Code: f_aov(", safe_deparse(formula), " + Error(", id_name, "/", x_cols_vars[1], "), data=", data_name, ", output_type=\"", output_type, "\")"))
            add_txt("   Alternative (base R): aov() with Error() term")
            add_txt(paste0("   Code: aov(", safe_deparse(formula), " + Error(", id_name, "/", x_cols_vars[1], "), data=", data_name, ")"))
          } else {
            add_txt("Test: Friedman Rank-Sum Test (non-parametric repeated measures)")
            add_txt(paste0("Code: friedman.test(", y_col, " ~ ", x_cols_vars[1],
                           " | ", id_name, ", data=", data_name, ")"))
            add_txt("   Post-hoc: rstatix::wilcox_test() with pairwise paired comparisons")
          }
        } else if (y_type == "binary") {
          add_txt("Test: Chi-square Test")
          add_txt(paste0("Code: f_chisq_test(", data_name, ", '", y_col, "', '", x_cols_vars[1], "', output_type=\"", output_type, "\")"))
        } else if (y_type == "count") {
          if (is_overdispersed) {
            add_txt("Test: Negative Binomial GLM (overdispersion detected)")
            add_txt(paste0("Code: MASS::glm.nb(", safe_deparse(formula), ", data=", data_name, ")"))
            add_txt("   (No rfriend wrapper yet \u2014 requires MASS package)")
          } else {
            add_txt("Test: Poisson GLM")
            add_txt(paste0("Code: ", cf("f_glm", ", family='poisson'")))
          }
        } else if (y_type == "multinomial") {
          add_txt("Test: Chi-square Test")
          add_txt(paste0("Code: f_chisq_test(", data_name, ", '", y_col, "', '", x_cols_vars[1], "', output_type=\"", output_type, "\")"))
        } else if (y_type == "ratio_normal") {
          if (variance_equal) {
            add_txt("Test: One-Way ANOVA")
            add_txt(paste0("Code: ", cf("f_aov")))
          } else {
            add_txt("Test: One-Way ANOVA (variances unequal \u2014 see note)")
            add_txt(paste0("Code: ", cf("f_aov")))
            add_txt("   Note: Levene flagged unequal variances. f_aov() is reasonably robust")
            add_txt("   for balanced designs; for strict correctness use Welch's ANOVA:")
            add_txt(paste0("   Alternative: oneway.test(", safe_deparse(formula), ", data=", data_name, ", var.equal=FALSE)"))
          }
          recommended_call <- bquote(f_aov(.(formula), data = data, output_type = .(output_type)))
          add_txt("   Post-hoc: f_aov() includes post-hoc tests (adjust='sidak' by default)")
          add_txt("   Effect size: rstatix::eta_squared() for eta-squared")
        } else if (can_be_normalized) {
          add_txt("Test: One-Way ANOVA with automatic Box-Cox transformation")
          add_txt(paste0("Code: ", cf("f_aov")))
          add_txt("   Note: f_aov() auto-applies Box-Cox when residuals are non-normal.")
          if (!variance_equal) {
            add_txt("   Note: Variances are also unequal. Verify after transformation;")
            add_txt("   if still unequal, prefer Kruskal-Wallis below.")
          }
          add_txt("Alternative: Kruskal-Wallis Test (Non-parametric)")
          add_txt(paste0("   Code: f_kruskal_test(", safe_deparse(formula), ", data=", data_name, ", output_type=\"", output_type, "\")"))
          recommended_call <- bquote(f_aov(.(formula), data = data, output_type = .(output_type)))
        } else if (y_type == "ratio_unknown") {
          add_txt("Note: Normality could not be assessed.")
          add_txt("Option 1: ANOVA (if normality is plausible)")
          add_txt(paste0("   Code: ", cf("f_aov")))
          add_txt("Option 2: Kruskal-Wallis Test (safer)")
          add_txt(paste0("   Code: f_kruskal_test(", safe_deparse(formula), ", data=", data_name, ", output_type=\"", output_type, "\")"))
          recommended_call <- bquote(f_aov(.(formula), data = data, output_type = .(output_type)))
        } else {
          add_txt("Test: Kruskal-Wallis Test (Non-parametric)")
          add_txt(paste0("Code: f_kruskal_test(", safe_deparse(formula), ", data=", data_name, ", output_type=\"", output_type, "\")"))
          recommended_call <- bquote(f_kruskal_test(.(formula), data = data, output_type = .(output_type)))
          add_txt("   Post-hoc: rstatix::dunn_test() for pairwise comparisons")
        }
      }
    }
  }

  add_txt(" ")
  add_txt("---------------------------------------------------")
  add_txt("Disclaimer:")
  add_txt("- This wizard checks data properties, not experimental design.")
  add_txt("- Always verify if your data is Paired or Independent!")
  add_txt("- BETA: f_stat_wizard is new and under active development \u2014 verify recommendations.")
  add_txt("- Other tests than the recommended one could be more suitable.")
  add_txt("The WIZARD recommends exclusively from these rfriend functions:")
  add_txt("   f_t_test, f_aov, f_glm, f_chisq_test,")
  add_txt("   f_kruskal_test, f_wilcox_test, f_corplot,")
  add_txt("   f_boxcox (transformation), f_scan (diagnostics)")
  add_txt("And these external packages (when no rfriend wrapper exists):")
  add_txt("   oneway.test (Welch ANOVA), MASS::glm.nb (Neg. Binomial),")
  add_txt("   nnet::multinom (Multinomial), cor.test (correlation test).")

  # Check recommended packages
  report_text <- paste(report_lines, collapse = " ")
  missing_pkgs <- unique(c(
    if (grepl("nnet::multinom", report_text, fixed = TRUE) && !requireNamespace("nnet", quietly = TRUE)) "nnet",
    if (grepl("MASS::glm.nb", report_text, fixed = TRUE) && !requireNamespace("MASS", quietly = TRUE)) "MASS",
    if (grepl("emmeans::", report_text, fixed = TRUE) && !requireNamespace("emmeans", quietly = TRUE)) "emmeans",
    if (grepl("rstatix::", report_text, fixed = TRUE) && !requireNamespace("rstatix", quietly = TRUE)) "rstatix"
  ))
  if (length(missing_pkgs) > 0) {
    add_txt(" ")
    add_txt(paste0("WARNING: Package(s) not installed: ", paste(missing_pkgs, collapse = ", ")))
    add_txt(paste0("   Install with: install.packages(c('", paste(missing_pkgs, collapse = "', '"), "'))"))
  }

  # --- RUN THE RECOMMENDED TEST ---
  if (run && !is.null(recommended_call)) {
    add_txt(" ")
    add_txt("===================================================")
    add_txt("RUNNING RECOMMENDED TEST...")
    add_txt("===================================================")
    run_result <- tryCatch({
      res <- suppressMessages(suppressWarnings(eval(recommended_call)))
      add_txt(paste0("   Test executed successfully (class: ",
                     paste(class(res), collapse = ", "), ")."))
      add_txt("   Access via $run_result in the returned object.")
      res
    }, error = function(e) {
      add_txt(paste0("   Could not run test: ", conditionMessage(e)))
      add_txt("   Try running the suggested code manually.")
      NULL
    })
  } else if (run && is.null(recommended_call)) {
    add_txt(" ")
    add_txt("Note: run=TRUE was set but no single rfriend function could be auto-executed.")
    add_txt("   This can happen with multi-step recommendations or external packages.")
    add_txt("   Please run the suggested code manually.")
  }

  return(build_result(report_lines))
}


# =============================================================================
# S3 PRINT METHOD
# =============================================================================

#' Print method for f_stat_wizard
#'
#' @param x An object of class \code{f_stat_wizard}.
#' @param plots Logical. If \code{TRUE}, display diagnostic plots (histogram and
#'   QQ-plot) if they were generated with \code{plots=TRUE}. Default \code{TRUE}.
#' @param ... Additional arguments (ignored).
#' @export
print.f_stat_wizard <- function(x, plots = TRUE, ...) {
  cat(x$report, sep = "\n")

  # Display diagnostic plots if available
  if (plots) {
    if (!is.null(x$histogram) && inherits(x$histogram, "recordedplot")) {
      grDevices::replayPlot(x$histogram)
    }
    if (!is.null(x$qqplot) && inherits(x$qqplot, "recordedplot")) {
      grDevices::replayPlot(x$qqplot)
    }
  }

  invisible(x)
}

#' Rename Elements of a Vector Based on a Mapping
#'
#' Renames elements of a vector based on a named mapping vector. Elements that match the names in the mapping vector are replaced with their corresponding values, while elements not found in the mapping remain unchanged.
#'
#' @param vector A character vector containing the elements to be renamed.
#' @param name_map A named vector where the names correspond to the elements in \code{vector} that should be renamed, and the values are the new names to assign.
#'
#' @return A character vector with updated element names. Elements not found in \code{name_map} remain unchanged.
#'
#' @details
#' This function iterates through each element of \code{vector} and checks if it exists in the names of \code{name_map}. If a match is found, the element is replaced with the corresponding value from \code{name_map}. If no match is found, the original element is retained. The result is returned as an unnamed character vector.
#'
#'
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # Define a vector and a name map.
#' vector   <- c("Species", "Weight", "L")
#' name_map <- c(Species = "New_species_name", L = "Length_cm")
#'
#' # Rename elements of the vector.
#' updated_vector <- f_rename_vector(vector, name_map)
#'
#' # View updated vector
#' print(updated_vector)
#'
#' @export

f_rename_vector <- function(vector, name_map) {
  updated_names <- vapply(vector, function(x) {
    if (x %in% names(name_map)) {
      name_map[x]
    } else {
      x
    }
  }, FUN.VALUE = character(1))

  updated_names_out <- as.vector(updated_names)
  return(updated_names_out)
}

#' f_clear: Clear Various Aspects of the R Environment
#'
#' Provides a convenient way to clear different components of the R environment, including the console, memory, graphics, and more. It also offers the option to restart the R session. This can come in handy at the start of an R script.
#'
#' @param env Logical. If \code{TRUE}, all objects in the global environment are removed. Default is \code{TRUE}.
#' @param gc Logical. If \code{TRUE}, garbage collection is performed to free up memory. Default is \code{TRUE}.
#' @param console Logical. If \code{TRUE}, the R console is cleared. Default is \code{TRUE}.
#' @param graph Logical. If \code{TRUE}, all open graphics devices are closed. Default is \code{TRUE}.
#' @param restart Logical. If \code{TRUE}, the R session is restarted using 'RStudio's' API. Default is \code{FALSE}.
#'
#' @details
#'\itemize{
#'\item Console Clearing: Clears the console output.
#'\item Garbage Collection: Performs garbage collection to free memory from unreferenced objects.
#'\item Graph Clearing: Closes all open graphics devices.
#'\item Environment Clearing: Removes all objects from the global environment.
#'\item Session Restart: Restarts the R session (only available in 'RStudio').
#' }
#'
#' @note The \code{restart} parameter requires 'RStudio' and its API package ('rstudioapi') to be installed and available.
#'
#' @return No return value, called for side effects, see details.
#'
#' @author
#' Sander H. van Delden  \email{plantmind@proton.me} \cr
#'
#' @examples
#' # Clear console, memory, graphs, and for example NOT the environment.
#' f_clear(env = FALSE)
#'
#'
#' @export


f_clear <- function(env = TRUE, gc = TRUE, console = TRUE, graph = TRUE, restart = FALSE) {

    # Console clearing
    if (console) {
      cat("\014")  # Clears console in R
      cat("Cleaned up console...\n")
    }

    # Garbage collection
    if (gc) {
      # Perform garbage collection, i.e. cleaning up unreferenced objects
      gc()
      cat("Freed up memory...\n")
    }

    # Graph generation
    if (graph) {
      # Clearing all open plots.
      graphics.off()
      cat("Closed all graphs...\n")
    }

    # Delete all objects from data environment
    if (env) {
      cat("Deleted all objects from data environment...\n")
      # Remove all objects
      rm(list = ls(envir = .GlobalEnv), envir = .GlobalEnv)
    }

    # Restart session
    if (restart) {
      # Restarting R session
      rstudioapi::restartSession()
      } else {
  message("To also restart R use: f_clear(restart = TRUE)")
    }
  }

# =============================================================================
# setup.R
# =============================================================================
# Runs ONCE at the very start of the test suite. Anything that should be
# reset between individual tests belongs in a helper-*.R or in the
# test_that() block itself via withr::local_*(), not here.
#
# testthat sources setup.R before any test-*.R file.
# =============================================================================


# -----------------------------------------------------------------------------
# Make output deterministic
# -----------------------------------------------------------------------------

# Many rfriend functions print numeric tables. Lock the formatting options
# so a developer's personal options() cannot influence test output.
old_opts <- options(
  OutDec          = ".",
  scipen          = 0,
  digits          = 7,
  warn            = 1,       # print warnings as they occur, do not defer
  warnPartialMatchArgs = FALSE,
  stringsAsFactors = FALSE
)


# -----------------------------------------------------------------------------
# Graphical device hygiene
# -----------------------------------------------------------------------------

# Any plotting done during the suite goes to a throwaway PDF device opened
# on tempfile(). This prevents tests from popping up RStudio graphics
# windows or leaving active devices behind that later tests then inherit.
#
# The device is closed in teardown.R.
grDevices::pdf(file = tempfile(fileext = ".pdf"))


# -----------------------------------------------------------------------------
# Teardown registration
# -----------------------------------------------------------------------------
# The paired teardown lives in teardown.R. We intentionally do NOT use
# withr::defer(envir = teardown_env()) here because teardown.R is already
# the canonical place for suite-wide cleanup and mixing both styles makes
# the sequence hard to trace.
#
# `old_opts` is left in the test-runner environment so teardown.R can
# restore it.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ffactors.R
\name{f_factors}
\alias{f_factors}
\title{Convert multiple columns to Factors in a data frame}
\usage{
f_factors(
  data,
  select = NULL,
  exclude = NULL,
  properties = FALSE,
  force_factors = FALSE,
  unique_num_treshold = 8,
  repeats_threshold = 2,
  ...
)
}
\arguments{
\item{data}{A data frame containing the columns to be converted.}

\item{select}{A character vector specifying the names of the columns to convert into factors. If \code{NULL}, the function automatically detects columns that should be factors based on their data type and unique value count. Default is \code{NULL}.}

\item{exclude}{A character vector specifying the names of the columns NOT to convert into factors. If \code{NULL}, no columns are excluded. Default is \code{NULL}.}

\item{properties}{Logical. If \code{TRUE}, prints a detailed table about the properties of the new data frame to the console. If \code{FALSE} no property table will be printed to the console. Default is \code{FALSE}.}

\item{force_factors}{Logical. If \code{TRUE} all columns in the data.frame will be converted to factors except for the excluded columns using \code{exclude}.}

\item{unique_num_treshold}{Numeric. A threshold of the amount of unique numbers a numeric column should have to keep it numeric, i.e. omit factor conversion. Default \code{8}.}

\item{repeats_threshold}{Numeric. A threshold of the minimal number of repeats a numeric column should have to convert it to a factor. Default \code{2}.}

\item{...}{Additional arguments passed to the \code{factor()} function of baseR.}
}
\value{
Returns the modified data frame with the specified (or all suitable) columns converted to factors. Can also force a print of a summary of the data frame's structure to the console (properties = TRUE).
}
\description{
Converts multiple specified columns of a data frame into factors. If no columns are specified, it automatically detects and converts columns that are suitable to be factors. The function returns the entire data frame including non factor columns and can report the properties of this new data frame in the console (\code{properties = TRUE}).
}
\details{
\itemize{
\item If \code{select} is \code{NULL}, the function identifies columns with character data or numeric data with fewer than 8 unique values as candidates for conversion to factors.
\item The function checks if all specified columns exist in the data frame and stops execution if any are missing.
\item Converts specified columns into factors, applying any additional arguments provided.
\item Outputs a summary data frame with details about each column, including its type, class, number of observations, missing values, factor levels, and labels.
}
}
\examples{
# Make a data.frame:
df <- data.frame(a = c("yes", "no", "yes", "yes", "no",
                       "yes", "yes", "no", "yes"),
                 b = c(1, 2, 3, 1, 2, 3, 1, 2, 3),
                 c = c("apple", "kiwi", "banana", "apple", "kiwi",
                        "banana", "apple", "kiwi", "banana"),
                 d = c(1.1, 1.1, 3.4, 4.5, 5.4, 6.7, 7.8, 8.1, 9.8)
                 )
str(df)

# Convert specified columns to factors:
df1 <- f_factors(df, select = c("a", "c"))
str(df1)


# Convert all potential factor columns to factor but exclude column "b":
df2 <- f_factors(df, exclude = c("b"))
str(df2)

# Convert all columns to factor but exclude column "b":
df3 <- f_factors(df, exclude = c("b"), force_factors = TRUE)
str(df3)

# Or automatically detect and convert suitable columns to factors.
# Thus obtaining the same results as above automatically:
df4 <- f_factors(df)
str(df4)

# In example above col b was converted to a factor as the number of repeats = 2
# and the amount of unique numbers < 8. In order to keep b numeric we can also
# adjust the unique_num_treshold and/or repeats_threshold:
df5 <- f_factors(df, unique_num_treshold = 2)
str(df5)

# Use `properties = TRUE` to view the data frame's structure.
# This forces a printed output which is more insight than standard str() output.
df6 <- f_factors(df, properties = TRUE)

}
\seealso{
\href{https://stat.ethz.ch/R-manual/R-devel/library/base/html/factor.html}{\code{factor}}
}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fconditional_round.R
\name{f_conditional_round}
\alias{f_conditional_round}
\title{Conditional Rounding for Numeric Values}
\usage{
f_conditional_round(
  x,
  threshold_small = 0.01,
  threshold_large = 10000,
  digits = 3,
  replace_na = TRUE,
  na_string = "-",
  allow_integer_decimal_mix = FALSE
)
}
\arguments{
\item{x}{A numeric vector or data frame containing numeric columns to be formatted.}

\item{threshold_small}{Numeric value. Values with absolute magnitude smaller than this
threshold will be formatted using scientific notation. Default is \code{0.01}.}

\item{threshold_large}{Numeric value. Values with absolute magnitude larger than or equal
to this threshold will be formatted using scientific notation. Default is \code{10000}.}

\item{digits}{Integer. Number of decimal digits to use in formatting. Default is \code{3}.}

\item{replace_na}{Logical. If \code{TRUE}, \code{NA} values in the output
are replaced with the string specified by \code{na_string}. If
\code{FALSE}, \code{NA} values are left as \code{NA}. Default is
\code{TRUE}.}

\item{na_string}{Character string used to replace \code{NA} values in the
output when \code{replace_na = TRUE}. Use \code{"-"} (default) for a
standard "not available" dash, \code{""} for empty cells, or any other
string as needed. Note that special characters such as the em dash
(\code{"\u2014"}) may require additional LaTeX declarations when rendering
to PDF via \code{pdflatex}. Default is \code{"-"}.}

\item{allow_integer_decimal_mix}{Logical. If \code{TRUE}, each individual cell is evaluated: integer values are displayed without decimal places, and non-integer values are displayed with the specified number of decimal places, i.e. \code{digits}. Default is \code{FALSE}, when a column contains a mix of integers and decimal values, all values are displayed with the specified number of decimal places. Note: columns containing only integers are **always** displayed without decimal places, regardless of \code{allow_integer_decimal_mix}.}
}
\value{
\itemize{
  \item If input is a vector: A character vector of the same length as the input, with values
formatted according to the specified rules.

  \item If input is a data frame: A data frame with the same structure as the input, but with
character columns formatted according to the specified rules.
 }
}
\description{
Conditionally formats numeric values based on their magnitude. Values that are very small
or very large are formatted using scientific notation, while other values are rounded
to a specified number of decimal places. Integers are preserved without decimal places.
When applied to a data frame, only numeric columns are processed. All output is character string.
}
\details{
The function applies the following formatting rules:
\itemize{
  \item Values smaller than \code{threshold_small} or larger than \code{threshold_large}
        are formatted in scientific notation with decimal \code{digits}.
  \item Integer values are formatted without decimal places.
  \item Non-integer values that don't require scientific notation are rounded to
        \code{digits} decimal places.
  \item NA values are replaced with empty strings if \code{replace_na = TRUE}.
  \item Empty strings in the input are preserved.
  \item For data frames, only numeric columns are processed; other columns remain unchanged.
}
}
\examples{
# Vector examples.
f_conditional_round(c(0.0001, 0.5, 3, 10000))
# Returns: "1.000e-04" "0.500" "3" "1.000e+04".

f_conditional_round(c(0.0001, 0.5, 3, 10000, NA), replace_na = TRUE)
# Returns: "1.000e-04" "0.500" "3" "1.000e+04" ""

# Data frame example.
df <- data.frame(
  name = c("A", "B", "C"),
  small_val = c(0.0001, 0.002, 0.5),
  integer = c(1, 2, 3),
  integer_mix = c(10, 20, 30.1),
  large_val = c(10000, 5000, NA)
)

# Show only two digits.
f_conditional_round(df, digits = 2)

# To keep Integers as Integers (no digits)
# in columns with mixed data (Integers and digits)
# set allow_integer_decimal_mix = TRUE
f_conditional_round(df, allow_integer_decimal_mix = TRUE)

# Custom NA replacement string.
f_conditional_round(c(0.5, NA, 3), replace_na = TRUE, na_string = "-")
# Returns: "0.500" "-" "3"

f_conditional_round(c(0.5, NA, 3), replace_na = TRUE, na_string = "")
# Returns: "0.500" "" "3"

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fboxcox.R
\name{plot.f_boxcox}
\alias{plot.f_boxcox}
\title{Plot an f_boxcox object}
\usage{
\method{plot}{f_boxcox}(x, which = 1:3, ask = FALSE, ...)
}
\arguments{
\item{x}{An object of class \code{f_boxcox}.}

\item{which}{Integer determining which graph to plot. Default is \code{1:2}.}

\item{ask}{Logical. \code{TRUE} waits with plotting each graph until <Return> is pressed. Default is \code{FALSE}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
This function is called for its side effect of generating plots
and does not return a useful value. It invisibly returns \code{1}.
}
\description{
Create diagnostic plots of an object of class \code{f_boxcox}.
}
\details{
Plot method for f_boxcox objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flmer.R
\name{f_lmer}
\alias{f_lmer}
\title{Fit a linear mixed model with \code{lme4::lmer()} including assumption checks, diagnostics, R-squared and post hoc tests.}
\usage{
f_lmer(
  formula,
  data = NULL,
  REML = TRUE,
  ddf = "Satterthwaite",
  alpha = 0.05,
  adjust = "sidak",
  norm_plots = TRUE,
  post_hoc = TRUE,
  intro_text = TRUE,
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  ...
)
}
\arguments{
\item{formula}{A two-sided formula passed to \code{lme4::lmer()}, e.g.
  \code{y ~ treatment + time + (1 | subject)} or
  \code{y ~ treatment * time + (1 + time | subject)}. The right-hand
  side must contain at least one random-effects term in the
  \code{(varying | grouping)} syntax. See \emph{Details} for a guide to
  reading the random-effects syntax in study-design terms.

  More than one response variable can be supplied on the left-hand side
  using \code{+} (e.g. \code{y1 + y2 ~ treatment + (1 | subject)}).
  A separate model is then fit for each response, sharing the same
  right-hand side, and a multiple-testing warning is added to the
  report. See the \emph{Multiple Testing Across Response Variables}
  section.}

\item{data}{A data frame containing the variables in the model.}

\item{REML}{Logical. If \code{TRUE} (default), the model is fit with
restricted maximum likelihood, which gives less biased variance
component estimates and is the appropriate choice for inference on
fixed effects with Kenward-Roger or Satterthwaite degrees of freedom.
Set to \code{FALSE} only when comparing nested models that differ in
their fixed-effects structure.}

\item{ddf}{Character. Method for computing denominator degrees of
freedom for fixed-effects p-values. One of:
\describe{
  \item{\code{"Satterthwaite"}}{(default) Fast and accurate for most
    designs. Provided by \code{lmerTest}.}
  \item{\code{"Kenward-Roger"}}{Considered the gold standard, especially
    for small samples and unbalanced designs. Slower, and requires the
    \code{pbkrtest} package.}
  \item{\code{"lme4"}}{No p-values; only t-statistics are reported.
    Equivalent to a plain \code{lme4::lmer()} summary.}
}}

\item{alpha}{Numeric. Significance level for the fixed-effects table and
the post hoc tests. Default is \code{0.05}.}

\item{adjust}{Character. Method used to adjust p-values for multiple
pairwise comparisons in the post hoc step (passed to
\code{emmeans::emmeans()}). One of \code{"sidak"} (default),
\code{"tukey"}, \code{"bonferroni"}, \code{"fdr"}, \code{"none"}.}

\item{norm_plots}{Logical. If \code{TRUE} (default), diagnostic plots
(residuals vs fitted, Q-Q of level-1 residuals, Q-Q of random-effect
BLUPs, scale-location) are included in the output.}

\item{post_hoc}{Logical. If \code{TRUE} (default), runs
\code{emmeans::emmeans()} pairwise comparisons \strong{only when}
the linear mixed model finds a significant fixed-effect term
(ANOVA p-value below \code{alpha}). The post hoc is performed on
each significant factor fixed-effect term separately. Numeric
covariates are skipped because their slope is already reported in
the fixed-effects coefficient table; pairwise contrasts are not
meaningful for a continuous predictor. If no fixed-effect term is
significant, no post hoc is run.}

\item{intro_text}{Logical. If \code{TRUE} (default), prepends an
explanation of LMM assumptions and the random-effects syntax linked
to study design.}

\item{output_type}{Character. Output format. One of:
\itemize{
  \item \code{"default"}: returns the \code{f_lmer} object; auto-prints
    if unassigned.
  \item \code{"console"}: forces immediate printing.
  \item \code{"pdf"}, \code{"word"}, \code{"excel"}: writes a file.
  \item \code{"rmd"}: stores the raw markdown string in the returned
    object for embedding in an R Markdown chunk with
    \code{\{r, echo=FALSE, results='asis'\}}.
}}

\item{save_as}{Character. Output file path. See \code{\link{f_aov}}
for the resolution rules; default is
\code{file.path(tempdir(), "<dataname>_lmer_output.<ext>")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, save in the working
directory instead of \code{tempdir()}. Default \code{FALSE}.}

\item{close_generated_files}{Logical. Closes any open Word or Excel
files before writing. Cross-platform (Windows taskkill, macOS / Linux
pkill). Default \code{FALSE}. \strong{WARNING:} save your work first.}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{...}{Additional arguments forwarded to \code{\link[lmerTest]{lmer}}.
The arguments \code{subset}, \code{na.action}, and \code{weights}
are handled specially: when supplied, they are applied via
\code{\link[stats]{model.frame}} \strong{once} before the
per-response loop, so every response in a multi-response call is
fitted on the identical row set. Other \code{lmer()} arguments
(e.g. \code{control = lmerControl(...)}, \code{contrasts},
\code{offset}) are forwarded unchanged on every fit.}
}
\value{
An object of class \code{f_lmer}: a named list containing the
  fitted \code{lmerModLmerTest} model, the ANOVA-style fixed-effects
  table, the variance components and ICC, the R\eqn{^2} values, the
  observed descriptives table (raw-data n, mean, sd, se, min, Q1,
  median, Q3, max grouped by the categorical fixed-effect predictors),
  post hoc results (if any), diagnostic plots, and convergence
  diagnostics. When more than one response variable is supplied on the
  left-hand side, these elements are nested one level deep under each
  response name, e.g. \code{out$y1$fixed_effects},
  \code{out$y2$fixed_effects}. When \code{output_type = "rmd"} the
  markdown string is stored in \code{$rmd}.
}
\description{
Fits a linear mixed-effects model using \code{lme4::lmer()} (with p-values
from \code{lmerTest}) and produces a fully-formatted report containing
the fixed-effects table, random-effects variance components, model-fit
indices (AIC, BIC, logLik, marginal & conditional R\eqn{^2}), residual and
BLUP diagnostics, convergence / singular-fit warnings, and post hoc
comparisons (\code{emmeans}) on factor fixed effects. Results can be
returned to the console or written to 'pdf', 'Word' or 'Excel'.
}
\details{
\strong{What is a linear mixed model?}\cr
A linear mixed model (LMM) extends ordinary regression / ANOVA by
allowing two kinds of effects:
\itemize{
  \item \strong{Fixed effects} - factors you actively manipulated or
    whose specific levels you care about (treatment, dose, time,
    genotype). Reported as estimates with confidence intervals.
  \item \strong{Random effects} - grouping structure that creates
    non-independence in your data but whose levels are a random sample
    from a larger population (subjects measured repeatedly, plots
    within fields, observers, batches). Reported as variance
    components.
}
Use an LMM whenever observations share something that makes them more
alike than two random observations from the dataset. Ignoring such
grouping (running a plain \code{aov} or \code{lm}) is
\strong{pseudoreplication}, i.e. treating non-independent observations
as if they were independent: standard errors shrink, p-values shrink,
false positives explode.

\strong{Vocabulary.}\cr
Before going further, a few terms used throughout the report:
\itemize{
  \item \strong{Subject} - the experimental unit that is measured
    repeatedly (a person, animal, pot and plot, cell line); in
    \code{lme4} syntax it is the grouping factor on the right of
    the \code{|}, e.g. \code{(1 | subject)}.
  \item \strong{Within-subject factor} - a predictor whose levels
    vary within the same subject (time in a longitudinal study,
    treatment in a cross-over study).
  \item \strong{Between-subject factor} - a predictor whose levels
    vary across subjects but are constant within a subject (sex,
    genotype, treatment arm in a parallel-groups trial). Both
    within- and between-subject factors are \strong{fixed} effects.
  \item \strong{BLUP} - \emph{Best Linear Unbiased Predictor}. The
    model's estimate of the random-effect value for each subject
    (e.g. how much a particular subject deviates from the population
    intercept). BLUPs are checked for normality just like residuals.
  \item \strong{ICC} - \emph{intraclass correlation coefficient}.
    The share of total variance attributable to between-group
    differences. ICC = 0 means the grouping factor is irrelevant;
    ICC = 1 means observations within a group are identical.
  \item \strong{REML} - \emph{restricted maximum likelihood}. The
    default fitting method for variance components; gives less biased
    estimates than ordinary maximum likelihood.
  \item \strong{Satterthwaite / Kenward-Roger} - methods to approximate
    the denominator degrees of freedom for fixed-effect p-values,
    since there is no exact df in an LMM.
}

\strong{Reading the \code{(1 | group)} syntax.}\cr
Every random-effects term has the form \code{( <varying> | <group> )}.
The bar reads as "varies by". The grouping factor on the right is what
creates the non-independence. The left side is what is allowed to differ
between groups. Common patterns:
\itemize{
  \item \code{(1 | subject)} - random intercept per subject (each
    subject has its own baseline). Repeated measures, longitudinal data.
  \item \code{(1 | field)} - randomised block design or multi-site
    trial; one intercept per block.
  \item \code{(1 | field/plot)} - \code{plot} nested in \code{field};
    equivalent to \code{(1|field) + (1|field:plot)}. Split-plot or
    hierarchical sampling.
  \item \code{(1 + time | subject)} - random intercept and random slope
    of \code{time} per subject. Subjects differ both in baseline and
    in how fast they change. Growth curves.
  \item \code{(1 | subject) + (1 | observer)} - crossed random
    effects: every observer can rate every subject. Inter-rater designs.
}
\strong{Rule of thumb:} if you can answer "if I duplicated this
experiment, would I draw new levels of this factor?" with \emph{yes},
it belongs on the right of a \code{|}. If you would re-use the exact
same levels (e.g. control vs treated) it is a fixed effect.

\strong{When to use a linear mixed model.}\cr
The most common reason is a \strong{repeated-measures design}, in
which the same experimental units are measured on more than one
occasion or under more than one treatment. Compared with a
between-groups design analysed by plain ANOVA this gives two real
advantages: fewer experimental units are needed (each subject acts
as its own control, removing between-subject variation from the
comparison) and individual differences cannot bias the treatment
groups (in a cross-over design every subject receives every
treatment). Two canonical examples:
\itemize{
  \item \strong{Longitudinal study} - same subjects measured at
    several time points: \code{y ~ time + (1 | subject)}. If
    subjects also differ in how fast they change, add a random
    slope: \code{y ~ time + (1 + time | subject)}.
  \item \strong{Cross-over design} - every subject receives every
    treatment in sequence: \code{y ~ treatment + (1 | subject)}. If
    carry-over between periods is a concern, add \code{period} as
    a fixed effect.
}
LMMs also apply to non-repeated structures that still create
non-independence: randomised block designs, split-plot trials,
multi-site studies, inter-rater designs.

\strong{Assumptions of a linear mixed model:}
\enumerate{
  \item Linearity in the parameters of the fixed-effects part.
  \item Independence of observations \emph{conditional on} the random
    effects. If structure remains (e.g. temporal autocorrelation), more
    random effects or a correlation structure are needed.
  \item Normality of level-1 residuals (Q-Q plot of \code{residuals(m)}).
  \item Normality of the random-effect BLUPs
    (Q-Q plot of \code{ranef(m)}). \strong{This is the assumption most
    users forget.}
  \item Homoscedasticity: residual variance roughly constant across
    fitted values and across grouping levels.
  \item At least \code{~5} levels of each grouping factor; with
    \code{3-4} levels it is usually better to treat the factor as
    fixed.
}
If Levene's test or the Shapiro-Wilk tests on residuals or BLUPs
indicate a violation, the report adds a \emph{Recommendations for
Heteroscedasticity and/or non-normal residuals} section after the
diagnostics with concrete next steps (generalised mixed model,
transformation).

\strong{Convergence and singular fits.}\cr
\code{f_lmer} surfaces \code{lme4} convergence warnings and the
"boundary (singular) fit" message prominently in the output. A singular
fit usually means the random-effects structure is too complex for the
data (often a random slope with too few levels) - simplify the model
before interpreting results.

This function requires Pandoc (>= 1.12.3) for \code{pdf}, \code{word}
and \code{rmd} output. See \code{\link{f_aov}} for installation notes.
}
\section{Multiple Testing Across Response Variables}{

When several response variables are analysed in a single call
(e.g. \code{y1 + y2 + y3 ~ treatment + (1 | subject)}), each linear
mixed model is an independent null-hypothesis test at level
\code{alpha}. The post hoc adjustments (\code{adjust = "sidak"},
\code{"tukey"}, etc.) only control the family-wise error rate
\strong{within} one model (across pairwise contrasts for that
response). They do \strong{not} protect against the inflation of
Type I error \strong{across} the set of responses.

\strong{Practical implication:} With \eqn{k} independent response
variables all tested at \eqn{\alpha = 0.05}, the probability of
obtaining at least one false positive is
\eqn{1 - (1 - 0.05)^k}, which reaches ~40\% for \eqn{k = 10}.

\strong{When this matters:} The risk is highest in exploratory studies
where many responses are screened simultaneously without a clear
a priori hypothesis for each one. It is less of a concern when each
response is a pre-specified primary outcome with its own biological
rationale.

\strong{Possible remedies:}
\itemize{
  \item \strong{Bonferroni correction across responses:} use
    \code{alpha = 0.05 / k} where \code{k} is the number of response
    variables. Conservative but simple.
  \item \strong{False Discovery Rate (FDR):} apply
    \code{p.adjust(p_values, method = "fdr")} to the vector of
    per-response fixed-effect p-values after the fact.
  \item \strong{Multivariate model:} if the responses are correlated
    and you want a single omnibus test, fit a joint multivariate
    mixed model (e.g. \code{MCMCglmm}, \code{brms}) before
    interpreting individual responses.
  \item \strong{Pre-registration:} declare primary vs. exploratory
    responses before data collection to justify differential
    correction thresholds.
}
}

\examples{
# sleepstudy: reaction time vs days of sleep deprivation,
# repeated measures within Subject (ships with lme4).
data(sleepstudy, package = "lme4")

# 1) Random intercept per subject - the simplest mixed model.
#    Each subject has its own baseline reaction time; the fixed
#    effect of Days is the average slope across subjects.
#    With output_type = "default" (the default), the result auto-
#    prints if not assigned, so no print() call is needed.
f_lmer_out <- f_lmer(Reaction ~ Days + (1 | Subject),
                     data = sleepstudy)

# Re-print the stored result and show the diagnostic plots.
print(f_lmer_out)
plot(f_lmer_out)

# 2) Random intercept AND random slope of Days per subject,
#    fitted with Kenward-Roger denominator df, saved to MS Word.
f_lmer(Reaction ~ Days + (1 + Days | Subject),
       data = sleepstudy,
       ddf  = "Kenward-Roger",
       output_type = "word"
       )

# 3) A factor fixed effect triggers a post hoc test.
#    Bin Days into three sleep-deprivation phases so that the
#    fixed effect is categorical and emmeans pairwise comparisons
#    with a compact letter display are produced automatically.
sleepstudy$Phase <- cut(sleepstudy$Days,
                        breaks = c(-Inf, 2, 6, Inf),
                        labels = c("early", "mid", "late"))
f_lmer(Reaction ~ Phase + (1 | Subject),
       data = sleepstudy,
       adjust = "tukey")

# 4) A minimal report: suppress the intro text and the diagnostic
#    plots, and save it directly to MS Word. Useful when embedding
#    many models in one document or when you only need the tables.
f_lmer(Reaction ~ Days + (1 | Subject),
       data = sleepstudy,
       intro_text = FALSE,
       norm_plots = FALSE,
       output_type = "word"
       )

# 5) Get the raw markdown back for embedding in an R Markdown
#    document. Use it inside a chunk with results = 'asis'.
f_lmer_rmd_out <- f_lmer(Reaction ~ Days + (1 | Subject),
                         data = sleepstudy,
                         output_type = "rmd")
cat(f_lmer_rmd_out$rmd)

# 6) Two response variables analysed in one call. A separate model
#    is fit for each, sharing the same right-hand side. The results
#    are nested under each response name.
sleepstudy$Reaction2 <- sleepstudy$Reaction + rnorm(nrow(sleepstudy), 0, 5)
multi_out <- f_lmer(Reaction + Reaction2 ~ Days + (1 | Subject),
                    data = sleepstudy,
                    intro_text = FALSE,
                    norm_plots = FALSE)
multi_out$Reaction$fixed_effects
multi_out$Reaction2$fixed_effects

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flong.R
\name{summary.f_long}
\alias{summary.f_long}
\title{Summary method for f_long objects}
\usage{
\method{summary}{f_long}(object, ...)
}
\arguments{
\item{object}{An object of class \code{f_long} (output from \code{f_long}).}

\item{...}{Additional arguments passed to \code{f_summary}.}
}
\value{
Returns the summary table (usually a data frame or tibble) produced by \code{f_summary}.
}
\description{
Automatically runs the \code{f_summary} function on data created by \code{f_long}
using the attributes stored in the object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fpander.R
\name{f_pander}
\alias{f_pander}
\title{Fancy Pander Table Output}
\usage{
f_pander(
  table,
  col_width = 10,
  table_width = NULL,
  limit_columns = NULL,
  style = "multiline",
  console = TRUE,
  ...
)
}
\arguments{
\item{table}{A data frame, matrix, or other table-like structure to be rendered.}

\item{col_width}{Integer. Specifies the maximum number of characters allowed in table header columns before a line break is inserted. Defaults to \code{10}. Note that latex will not break the table header if the col with is longer due to long names below the header.}

\item{table_width}{Integer or \code{NULL}. Defines the number of characters after which the table is split into separate sections. Defaults to \code{NULL}, meaning no break is applied.}

\item{limit_columns}{Integer or \code{NULL}. Defines the number of columns shown in a table.}

\item{style}{Character. Pander table style. Defaults to \code{"multiline"}.}

\item{console}{Logical. Whether to process headers for console output. Defaults to \code{TRUE}.}

\item{...}{Additional arguments passed to the \code{pander} function.}
}
\value{
None. The function is called for its side effects of setting 'pander' options and creates a pander formatted table in R Markdown.
}
\description{
Is a wrapper around the \code{pander} function from the 'pander' package, designed to produce a fancy table output with specific formatting options.
}
\details{
This function sets several \code{pander} options to ensure that the table output is formatted in a visually appealing manner. The options set include:
\itemize{
  \item \code{table.alignment.default}: Aligns all columns to the left.
  \item \code{table.alignment.rownames}: Aligns row names to the left.
  \item \code{keep.trailing.zeros}: Keeps trailing zeros in numeric values.
  \item \code{knitr.auto.asis}: Ensures output is not automatically treated as 'asis'.
  \item \code{table.caption.prefix}: Removes the default "Table" prefix in captions.
  \item \code{keep.line.breaks}: Preserves line breaks in cell content.
  \item \code{table.split.table}: Controls table splitting (set to \code{Inf} if \code{table_width} is \code{NULL} or \code{FALSE}).
  \item \code{table.split.cells}: Inserts line breaks in headers every \code{col_width} characters.
}

This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
\itemize{
\item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
\item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
\item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.
\item If Pandoc is not found, this function may not work as intended.
}
}
\examples{
# Example usage of f_pander
df <- data.frame(
  Name = c("Alice", "Bob", "Charlie"),
  Age = c(25, 30, 35),
  Score = c(88.5, 92.3, 85.0)
)

# Render the data frame as a fancy table
f_pander(df)
}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fhist.R
\name{f_hist}
\alias{f_hist}
\title{Plot a Histogram with an Overlaid Normal Curve}
\usage{
f_hist(
  data,
  main = NULL,
  xlab = NULL,
  probability = TRUE,
  col = "white",
  border = "black",
  line_col = "red",
  save_png = FALSE,
  open_png = TRUE,
  save_as = NULL,
  save_in_wdir = FALSE,
  width = 8,
  height = 7,
  units = "in",
  res = 300,
  ...
)
}
\arguments{
\item{data}{A numeric vector of data values to be plotted.}

\item{main}{A character string specifying the title of the histogram. Default is \code{"Histogram with Normal Curve"}.}

\item{xlab}{A character string specifying the label for the x-axis. Default is the name of the data variable.}

\item{probability}{A logical value indicating whether to plot a probability or frequency histogram. Default is \code{TRUE}.}

\item{col}{A character string specifying the fill color of the histogram bars. Default is \code{"white"}.}

\item{border}{A character string specifying the color of the histogram bar borders. Default is \code{"black"}.}

\item{line_col}{A character string specifying the color of the normal curve line. Default is \code{"red"}.}

\item{save_png}{A logical value default \code{FALSE}, if \code{TRUE} a png file is saved under the name of the data of under the specified file name.}

\item{open_png}{Logical. If \code{TRUE}, opens generated png files.}

\item{save_as}{Character string specifying the output file path (without extension).
If a full path is provided, output is saved to that location.
If only a filename is given, the file is saved in \code{tempdir()}.
If only a directory is specified (providing an existing directory with trailing slash),
the file is named "data_name_histogram.png" in that directory.
Defaults to \code{file.path(tempdir(), "data_name_histogram.png")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.}

\item{width}{Numeric, png figure width default \code{8} inch.}

\item{height}{Numeric, png figure height default \code{7} inch.}

\item{units}{Character string, png figure units default \code{"in"} = inch, other options are: \code{"px"} = Pixels, \code{"cm"} centimeters, \code{"mm"} millimeters.}

\item{res}{Numeric, png figure resolution default \code{300} dpi.}

\item{...}{Additional arguments to be passed to the \code{hist} function.}
}
\value{
A histogram plot is created and the function returns this as a \code{recordedplot}.
}
\description{
This function creates a histogram of the provided data and overlays it with a normal distribution curve.
}
\details{
The function first captures the name of the input variable for labeling purposes. It then calculates a sequence of x-values and corresponding y-values for a normal distribution based on the mean and standard deviation of the data. The histogram is plotted with specified aesthetics, and a normal curve is overlaid. To increase resolution you can use \code{png(...,res = 600)} or the 'RStudio' chunk setting, e.g. \code{dpi=600}.
}
\examples{
# Example usage:
set.seed(123)
sample_data <- rnorm(100)
f_hist(sample_data)


}
\seealso{
\href{https://stat.ethz.ch/R-manual/R-devel/library/graphics/html/hist.html}{\code{hist}}, \href{https://stat.ethz.ch/R-manual/R-devel/library/stats/html/Normal.html}{\code{dnorm}},  \href{https://stat.ethz.ch/R-manual/R-devel/library/graphics/html/lines.html}{\code{lines}}
}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsummary.R
\name{f_summary}
\alias{f_summary}
\alias{f_summary.formula}
\alias{f_summary.data.frame}
\title{Summarize a Data Frame with Grouping Variables}
\usage{
f_summary(x, ...)

\method{f_summary}{formula}(x, data, ...)

\method{f_summary}{data.frame}(
  x,
  columns,
  group_vars = NULL,
  show_name = TRUE,
  show_n = TRUE,
  show_mean = TRUE,
  show_sd = TRUE,
  show_se = TRUE,
  show_min = TRUE,
  show_max = TRUE,
  show_median = TRUE,
  show_Q1 = TRUE,
  show_Q3 = TRUE,
  show_skew = FALSE,
  show_kurtosis = FALSE,
  digits = NULL,
  export_to_excel = FALSE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  save_as = NULL,
  save_in_wdir = FALSE,
  check_input = TRUE,
  digits_excel = NULL,
  allow_integer_decimal_mix = FALSE,
  ...
)
}
\arguments{
\item{x}{A data.frame or formula (dispatches to the right method).}

\item{...}{Further arguments forwarded to \code{f_summary.data.frame}.}

\item{data}{A 'data.frame', 'data.table', or 'tibble'.}

\item{columns}{The numerical column(s) to summarize if no formula is used. Can be entered as a single character string (e.g., \code{"weight"}) or as a character vector \code{c("weight", "length"}).}

\item{group_vars}{A character vector specifying the grouping variables in \code{data}
(e.g., \code{c("species", "fertilizer")}) if no formula is used. If \code{NULL}, the entire dataset is summarized.}

\item{show_name}{Logical. Include variable name. Default \code{TRUE}.}

\item{show_n}{Logical. Include count (\code{n}). Default \code{TRUE}.}

\item{show_mean}{Logical. Include mean. Default \code{TRUE}.}

\item{show_sd}{Logical. Include standard deviation. Default \code{TRUE}.}

\item{show_se}{Logical. Include standard error. Default \code{TRUE}.}

\item{show_min}{Logical. Include minimum value. Default \code{TRUE}.}

\item{show_max}{Logical. Include maximum value. Default \code{TRUE}.}

\item{show_median}{Logical. Include median. Default \code{TRUE}.}

\item{show_Q1}{Logical. Include first quartile (25th percentile). Default \code{TRUE}.}

\item{show_Q3}{Logical. Include third quartile (75th percentile). Default \code{TRUE}.}

\item{show_skew}{Logical. Include Skewness (measure of asymmetry). Default \code{FALSE}.}

\item{show_kurtosis}{Logical. Include Excess Kurtosis (measure of "tailedness"). Default \code{FALSE}.}

\item{digits}{Integer. Number of decimal places for the R console output.
Default is \code{2}. If \code{NULL}, no rounding is applied.
(Note: This does not affect the raw numbers exported to Excel).}

\item{export_to_excel}{Logical. If \code{TRUE}, exports results to an 'Excel' file. Default \code{FALSE}.}

\item{close_generated_files}{Logical. If \code{TRUE}, forces Excel to close before saving (Windows only). Default \code{FALSE}.}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{save_as}{Character string. Custom path or filename for the Excel export.
\itemize{
  \item If full path: Saves to that location.
  \item If filename only: Saves to \code{tempdir()} (unless \code{save_in_wdir = TRUE}).
  \item If directory: Saves as "dataname_summary.xlsx" in that directory.
}}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves to the current working directory. Default \code{FALSE}.}

\item{check_input}{Logical. If \code{TRUE}, performs validation checks on inputs. Default \code{TRUE}.}

\item{digits_excel}{Integer. Number of decimal places for the Excel file cells. Default \code{NULL} (no rounding). Defining \code{digits_excel}, sets \code{export_to_excel = TRUE} when excel output is not intended use: \code{digits} instead.}

\item{allow_integer_decimal_mix}{Logical. If \code{TRUE}, intergers in columns with a mix of integers and non-integers are displayed without decimals. Default \code{FALSE}, meaning if there are one or more numbers with decimals the whole column contains the number of decimals set by \code{digits}.}

\item{formula}{A formula specifying the columns (right hand side) to be summarized by groups (left hand side). More columns or groups can be added using \code{-} or \code{+} (e.g., \code{col1 + col2 ~ group1 + group2}) to do a sequential summary for each column parameter.}
}
\value{
A list of class \code{f_summary} containing the results data frame.
}
\description{
Computes summary statistics (n, mean, sd, etc.) for a specified numerical columns
in a data frame. The data can be analyzed as a whole or split by one or more
grouping variables.

The function returns a formatted data frame and includes options to export
the results directly to an 'Excel' file.
}
\details{
The function computes the following statistics:
\itemize{
  \item \code{n}: number of observations
  \item \code{mean}: arithmetic mean
  \item \code{sd}: standard deviation
  \item \code{se}: standard error (\eqn{sd / \sqrt{n}})
  \item \code{min}: minimum value
  \item \code{max}: maximum value
  \item \code{median}: median value
  \item \code{Q1}: 25th percentile
  \item \code{Q3}: 75th percentile
  \item \code{skew}: Sample skewness (if requested).
  \item \code{kurt}: Sample excess kurtosis (if requested).
}

\code{skew} stands for Skewness which is a measure of asymmetry of a distribution around its mean. Where \code{skew} values near \bold{0} indicate approximate symmetry, while large positive or negative values indicate noticeable asymmetry.
\itemize{
  \item \code{> 0}: Right-skewed (long or heavier tail to the right).
  \item \code{< 0}: Left-skewed  (long or heavier tail to the left).
 }

\code{kurt} stands for Excess Kurtosis: Tells you about the "tails" and the peak.
  \itemize{
    \item \code{0}: Same tail heaviness as the normal distribution (mesokurtic).
    \item \code{> 0}: Heavier tails than normal (Leptokurtic) -- indicates frequent outliers.
    \item \code{< 0}: Lighter tails than normal (Platykurtic) -- indicates fewer (or less extreme) outliers than a normal distribution.
 }
If \code{group_vars} are provided, the statistics are calculated for each group combination.
When \code{export_to_excel = TRUE}, the file is automatically generated.
}
\examples{

# --- Example 1: Basic Usage (data.frame notation) ---
# Summarize "hp" grouped by "cyl"; columns and group_vars can be positional
summary_mtcars <- f_summary(mtcars, columns = "hp", group_vars = "cyl")
summary_mtcars <- f_summary(mtcars, "hp", "cyl")  # shorthand equivalent
print(summary_mtcars)

# --- Example 2: Multiple Columns & Groups with Custom Toggles ---
# Summarize "hp" and "disp", grouped by "cyl" and "gear", hide Q1/Q3
summary_custom <- f_summary(mtcars,
                            columns    = c("hp", "disp"),
                            group_vars = c("cyl", "gear"),
                            show_Q1    = FALSE,
                            show_Q3    = FALSE)
print(summary_custom)

# --- Example 3: Formula Notation ---
# Identical result to Example 2 using formula interface
# and export output to excel
summary_formula <- f_summary(hp + disp ~ cyl + gear,
                             data    = mtcars,
                             show_Q1 = FALSE,
                             show_Q3 = FALSE,
                             export_to_excel = TRUE)
print(summary_formula)

# --- Example 4: Distributional Stats & Digits ---
# Add skewness and kurtosis, control rounding
summary_dist <- f_summary(Sepal.Length + Petal.Length ~ Species,
                          data          = iris,
                          show_skew     = TRUE,
                          show_kurtosis = TRUE,
                          digits        = 3)
print(summary_dist)

# --- Example 5: Custom Print Formatting ---
summary_iris <- f_summary(iris, "Sepal.Length", group_vars = "Species")
print(summary_iris, col_width = 10, table_width = 70)
}
\author{
Sander H. van Delden \email{plantmind@proton.me}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fremove_outliers.R
\name{f_remove_outliers}
\alias{f_remove_outliers}
\title{Remove Outliers from Data}
\usage{
f_remove_outliers(data, outliers, by = "row_id", verbose = TRUE)
}
\arguments{
\item{data}{A data.frame, tibble, or data.table containing the original data.}

\item{outliers}{Either:
\itemize{
  \item A dataframe returned by \code{\link{f_outliers}}.
  \item A vector of IDs/row numbers to remove.
}}

\item{by}{A character string specifying the column to match on. Default is \code{"row_id"}.
If the source \code{data} does not have a \code{row_id} column, the function effectively
uses the row numbers (1, 2, 3...) to ensure safe deletion.}

\item{verbose}{Logical. If \code{TRUE} (default), prints a summary of how many rows were removed.}
}
\value{
An object of the same class as the input \code{data} (data.frame, tibble, or data.table)
  with the specified outlier rows removed.
}
\description{
`f_remove_outliers()` removes specific rows from a dataframe based on a list of identifiers.
It is designed to work seamlessly with the output of \code{\link{f_outliers}}, but can also
accept a custom vector of IDs.
}
\details{
\strong{Safe Deletion Logic:}
This function performs a "anti-join" style filtering. It keeps rows where the identifier in
\code{by} is \strong{not} found in the \code{outliers} list.

\strong{Handling Row IDs:}
If you use the default \code{by = "row_id"} and your original \code{data} does not have a
column named \code{"row_id"}, the function assumes you are referring to the intrinsic
row numbers of the data.frame, tibble, or data.table. It will temporarily generate IDs to
perform the deletion and then return the clean data with the original structure
(without adding a permanent \code{row_id} column to the result).
}
\examples{
# --- Setup: Create Dummy Data ---
set.seed(42)
df <- data.frame(
  Team       = rep(c("A", "B"), each = 20),
  Department = rep(c("Sales", "IT"), each = 10, times = 2),
  Salary     = c(rnorm(19, 50000, 500), 100000,
                 rnorm(18, 50000, 500), 57000, 1000),
  Age        = c(rnorm(38, 35, 2), 90, 35),
  EmployeeID = paste0("E", sprintf("\%03d", 1:40)),
  stringsAsFactors = FALSE
)
# row 20:  extreme high Salary (Team A)
# row 39:  mild Salary outlier at coef = 1.5 only
# row 40:  extreme low  Salary (Team B)
# row 39:  extreme high Age

# --- Example 1: Basic two-step workflow (data.frame notation) ---
# The most common use case: find then remove in two lines.
bad_rows <- f_outliers(df, columns = "Salary")
clean_df <- f_remove_outliers(df, bad_rows)
nrow(df)       # 40
nrow(clean_df) # 40 minus flagged rows

# --- Example 2: Basic two-step workflow (formula notation) ---
# Identical result to Example 1 using the formula interface.
bad_rows <- f_outliers(Salary ~ 1, data = df)
clean_df <- f_remove_outliers(df, bad_rows)
nrow(clean_df)

# --- Example 3: Grouped detection then removal (both notations) ---
# Outliers are identified *within* each Team separately before removal.

# data.frame notation:
bad_rows <- f_outliers(df, columns = "Salary", group_vars = "Team")
clean_df <- f_remove_outliers(df, bad_rows)

# Formula notation (identical result):
bad_rows <- f_outliers(Salary ~ Team, data = df)
clean_df <- f_remove_outliers(df, bad_rows)
nrow(clean_df)

# --- Example 4: Selective removal -- only act on a subset of outliers ---
# Find all flagged rows, but only remove the extreme high salaries.
# Step 1: Identify all Salary outliers grouped by Team
bad_rows    <- f_outliers(Salary ~ Team, data = df)
all_flagged <- bad_rows$output_df

# Step 2: Filter to keep only the rows where Salary > 90000
really_bad  <- all_flagged[all_flagged$Salary > 90000, ]

# Step 3: Remove only those rows -- low outlier (row 40) is preserved
clean_df <- f_remove_outliers(df, really_bad)
range(clean_df$Salary)  # low outlier still present, high one is gone

# --- Example 5: Multi-column outlier removal ---
# f_outliers scans both Salary and Age; f_remove_outliers removes
# every row flagged by either column in one call.

# Formula notation:
bad_rows <- f_outliers(Salary + Age ~ Team, data = df)
clean_df <- f_remove_outliers(df, bad_rows)

# data.frame notation (identical result):
bad_rows <- f_outliers(df, columns = c("Salary", "Age"), group_vars = "Team")
clean_df <- f_remove_outliers(df, bad_rows)
nrow(clean_df)  # rows flagged by Salary OR Age are removed

# --- Example 6: Strict detection + custom ID column ---
# coef = 3.0 flags only extreme outliers. EmployeeID is used
# as the matching key instead of the default row_id.

# Formula notation:
bad_rows <- f_outliers(Salary ~ Team, data = df,
                       id_var = "EmployeeID", coef = 3.0)

# data.frame notation (identical result):
bad_rows <- f_outliers(df, columns = "Salary", group_vars = "Team",
                       id_var = "EmployeeID", coef = 3.0)

# Remove by EmployeeID rather than row position
clean_df <- f_remove_outliers(df, bad_rows$output_df, by = "EmployeeID")

# Confirm the flagged employees are no longer in the clean data
bad_ids  <- bad_rows$output_df$EmployeeID
any(clean_df$EmployeeID \%in\% bad_ids)  # FALSE

}
\seealso{
\code{\link{f_outliers}} to identify the rows to be removed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/foutliers.R
\name{print.f_outliers}
\alias{print.f_outliers}
\title{Print method for f_outliers objects}
\usage{
\method{print}{f_outliers}(
  x,
  col_width = 6,
  table_width = 90,
  digits = 2,
  allow_integer_decimal_mix = FALSE,
  ...
)
}
\arguments{
\item{x}{Object of class f_outliers.}

\item{col_width}{Integer. Max characters in header before line break. Default \code{6}.}

\item{table_width}{Integer or \code{NULL}. Characters after which table splits. Default \code{90}.}

\item{digits}{Integer. Number of decimal digits to use in formatting. Default is \code{3}.}

\item{allow_integer_decimal_mix}{Logical. If \code{TRUE}, each individual cell is evaluated: integer values are displayed without decimal places, and non-integer values are displayed with the specified number of decimal places, i.e. \code{digits}. Default is \code{FALSE}, when a column contains a mix of integers and decimal values, all values are displayed with the specified number of decimal places. Note: columns containing only integers are **always** displayed without decimal places, regardless of \code{allow_integer_decimal_mix}.}

\item{...}{Additional arguments passed to \code{pander}.}
}
\value{
Invisibly returns \code{1}.
}
\description{
Prints a formatted summary table to the console.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flong.R
\name{plot.f_long}
\alias{plot.f_long}
\title{Plot method for f_long objects}
\usage{
\method{plot}{f_long}(x, summary = TRUE, ...)
}
\arguments{
\item{x}{An object of class \code{f_long} (output from \code{f_long}).}

\item{summary}{Logical. If \code{TRUE}, generates the summary table within the scan. Default is \code{TRUE}.}

\item{...}{Additional arguments passed to \code{f_scan}.}
}
\value{
Returns the output of \code{f_scan} (an object of class \code{f_scan}) invisibly.
}
\description{
Automatically runs a \code{f_scan} diagnostic plot on data created by \code{f_long}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frenamevector.R
\name{f_rename_vector}
\alias{f_rename_vector}
\title{Rename Elements of a Vector Based on a Mapping}
\usage{
f_rename_vector(vector, name_map)
}
\arguments{
\item{vector}{A character vector containing the elements to be renamed.}

\item{name_map}{A named vector where the names correspond to the elements in \code{vector} that should be renamed, and the values are the new names to assign.}
}
\value{
A character vector with updated element names. Elements not found in \code{name_map} remain unchanged.
}
\description{
Renames elements of a vector based on a named mapping vector. Elements that match the names in the mapping vector are replaced with their corresponding values, while elements not found in the mapping remain unchanged.
}
\details{
This function iterates through each element of \code{vector} and checks if it exists in the names of \code{name_map}. If a match is found, the element is replaced with the corresponding value from \code{name_map}. If no match is found, the original element is retained. The result is returned as an unnamed character vector.
}
\examples{
# Define a vector and a name map.
vector   <- c("Species", "Weight", "L")
name_map <- c(Species = "New_species_name", L = "Length_cm")

# Rename elements of the vector.
updated_vector <- f_rename_vector(vector, name_map)

# View updated vector
print(updated_vector)

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fclear.R
\name{f_clear}
\alias{f_clear}
\title{f_clear: Clear Various Aspects of the R Environment}
\usage{
f_clear(env = TRUE, gc = TRUE, console = TRUE, graph = TRUE, restart = FALSE)
}
\arguments{
\item{env}{Logical. If \code{TRUE}, all objects in the global environment are removed. Default is \code{TRUE}.}

\item{gc}{Logical. If \code{TRUE}, garbage collection is performed to free up memory. Default is \code{TRUE}.}

\item{console}{Logical. If \code{TRUE}, the R console is cleared. Default is \code{TRUE}.}

\item{graph}{Logical. If \code{TRUE}, all open graphics devices are closed. Default is \code{TRUE}.}

\item{restart}{Logical. If \code{TRUE}, the R session is restarted using 'RStudio's' API. Default is \code{FALSE}.}
}
\value{
No return value, called for side effects, see details.
}
\description{
Provides a convenient way to clear different components of the R environment, including the console, memory, graphics, and more. It also offers the option to restart the R session. This can come in handy at the start of an R script.
}
\details{
\itemize{
\item Console Clearing: Clears the console output.
\item Garbage Collection: Performs garbage collection to free memory from unreferenced objects.
\item Graph Clearing: Closes all open graphics devices.
\item Environment Clearing: Removes all objects from the global environment.
\item Session Restart: Restarts the R session (only available in 'RStudio').
}
}
\note{
The \code{restart} parameter requires 'RStudio' and its API package ('rstudioapi') to be installed and available.
}
\examples{
# Clear console, memory, graphs, and for example NOT the environment.
f_clear(env = FALSE)


}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fchisq_test.R
\name{f_chisq_test}
\alias{f_chisq_test}
\title{Chi-squared Test with post hoc Analysis}
\usage{
f_chisq_test(
  x,
  y,
  p = NULL,
  method = "bonferroni",
  digits = 3,
  alpha = 0.05,
  force_posthoc = FALSE,
  ...
)
}
\arguments{
\item{x}{A numeric vector (or factor), or a contingency table in matrix or table form. If a data frame is entered the function will try to convert it to a table using \code{df_to_table()}.}

\item{y}{A numeric vector; ignored if x is a matrix, table or data.frame. If x is a factor, y should be a factor of the same length.}

\item{p}{A vector of probabilities of the same length as x. Default is \code{NULL}. An error is given if any entry of p is negative.}

\item{method}{Character string specifying the adjustment method for p-values. Default is \code{"bonferroni"}. Other options include \code{"holm", "hochberg", "hommel", "BH", "BY", "fdr", and "none"}.}

\item{digits}{Integer specifying the number of decimal places for rounding. Default is \code{3}.}

\item{alpha}{Numeric threshold for significance. Default is \code{0.05}.}

\item{force_posthoc}{Logical indicating whether to perform post hoc tests even if the chi-squared test is not significant. Default is \code{FALSE}.}

\item{...}{Additional arguments passed to \code{\link[stats]{chisq.test}}.}
}
\value{
An object of class f_chisq_test containing:
\itemize{
  \item \code{chisq_test_output}: The output from \code{\link[stats]{chisq.test}}.
  \item \code{adjusted_p_values}: Matrix of adjusted p-values (for table/matrix input).
  \item \code{observed_vs_adj_p_value}: Interleaved table of observed values and adjusted p-values.
  \item \code{stdres_vs_adj_p_value}: Interleaved table of standardized residuals and adjusted p-values.
  \item \code{adj_p_values}: Vector of adjusted p-values (for vector input).
  \item \code{posthoc_output_table}: Data frame with observed values, expected values,
        standardized residuals, and adjusted p-values (for vector input).
  \item \code{observed_vs_adj_p_value}: Interleaved table of observed values and adjusted p-values (for table/matrix input).
  \item \code{stdres_vs_adj_p_value}: Interleaved table of standardized residuals and adjusted p-values (for table/matrix input).
}
}
\description{
Performs a chi-squared test  \code{\link[stats]{chisq.test}}, then automatically conducts post hoc analysis if the test is significant. The function provides adjusted p-values for each cell in the contingency table using a specified correction method.
}
\details{
The function first performs a chi-squared test using \code{\link[stats]{chisq.test}}. If the test is
significant (p < alpha) or if \code{force_posthoc = TRUE}, it conducts post hoc analysis by examining
the standardized residuals. The p-values for these residuals are adjusted using the specified method
to control for multiple comparisons.

If the input is a data frame, the function attempts to convert it to a table and displays the
resulting table for verification.
}
\examples{
# Chi.square on independence: Association between two variables.
# Create a contingency table.
my_table <- as.table(rbind(c(100, 150, 50), c(120, 90, 40)))
dimnames(my_table) <- list(Gender = c("Male", "Female"),
                           Response = c("Agree", "Neutral", "Disagree"))

# Perform chi-squared test with post hoc analysis.
f_chisq_test(my_table)

# Use a different adjustment method.
f_chisq_test(my_table, method = "holm")

# Other forms still work like Goodness-of-Fit: Match to theoretical distribution.
# Observed frequencies of rolling with a die 1 - 6.
observed <- c(2, 2, 10, 20, 15, 11)

# Expected probabilities under a fair die.
expected_probs <- rep(1/6, 6)

# Chi-Square Goodness-of-Fit Test.
f_chisq_test(x = observed, p = expected_probs)

}
\references{
This function implements a post hoc analysis for chi-squared tests inspired by the methodology in:

Beasley, T. M., & Schumacker, R. E. (1995). Multiple Regression Approach to Analyzing Contingency Tables: Post Hoc and Planned Comparison Procedures. The Journal of Experimental Education, 64(1), 79-93.

The implementation draws inspiration from the \href{https://CRAN.R-project.org/package=chisq.posthoc.test}{'chisq.posthoc.test'} package by Daniel Ebbert.
}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ftheme.R
\name{f_theme}
\alias{f_theme}
\title{Apply a black or white 'RStudio' Theme and Zoom Level}
\usage{
f_theme(color = "black", zlevel = 0)
}
\arguments{
\item{color}{A character string. The theme color to apply. Must be either \code{"black"} (dark theme) or \code{"white"} (light theme). Default is \code{"black"}.}

\item{zlevel}{A numeric value. The zoom level to apply, ranging from \code{0} (default size) to \code{4} (maximum zoom). Default is \code{0}.}
}
\value{
None. The function is called for its side effects of changing the 'RStudio' theme or Zoomlevel.

This function does not return a value. It applies changes directly to the 'RStudio' IDE.
}
\description{
This comes in hand when teaching, the function allows users to apply a "black" or "white" 'RStudio' theme and adjust the zoom level in the 'RStudio' IDE. It includes error handling for invalid inputs.
}
\details{
The function performs the following actions:
\enumerate{
\item Applies the specified 'RStudio' theme:
\itemize{
  \item \code{"black"}: Applies the "Tomorrow Night 80s" dark theme.
  \item \code{"white"}: Applies the "Textmate (default)" light theme.
  }
\item Adjusts the zoom level in 'RStudio':
\itemize{
  \item \code{zlevel = 0}: Resets to default zoom level.
  \item \code{zlevel = 1}: Zooms in once.
  \item \code{zlevel = 2}: Zooms in twice.
  \item \code{zlevel = 3}: Zooms in three times.
  \item \code{zlevel = 4}: Zooms in four times.
  }
}
The function includes error handling to ensure valid inputs:
 \itemize{
\item \code{color} must be a character string and one of \code{"black"} or \code{"white"}.
\item \code{zlevel} must be a numeric value, an integer, and within the range of 0 to 4. If a non-integer is provided, it will be rounded to the nearest integer with a warning.
}
}
\examples{
# NOTE: This example will change your RStudio theme hence the dont run warning.
\dontrun{
# Apply a dark theme with with zoom level 2:
f_theme(color = "black", zlevel = 2)

# Apply a black theme with maximum zoom level:
f_theme(color = "black", zlevel = 4)

# Apply the default light theme default zoom level:
f_theme(color = "white", zlevel = 0)
}

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fstat_wizard.R
\name{f_stat_wizard}
\alias{f_stat_wizard}
\alias{f_stat_wizard.formula}
\alias{f_stat_wizard.data.frame}
\title{Statistical Test Wizard}
\usage{
f_stat_wizard(x, ...)

\method{f_stat_wizard}{formula}(
  formula,
  data,
  id_col = NULL,
  run = FALSE,
  plots = FALSE,
  output_type = "word",
  interactive = FALSE,
  data_name = NULL,
  ...
)

\method{f_stat_wizard}{data.frame}(
  x,
  formula,
  id_col = NULL,
  run = FALSE,
  plots = FALSE,
  output_type = "word",
  interactive = FALSE,
  data_name = NULL,
  ...
)
}
\arguments{
\item{x}{A formula (e.g., \code{y ~ x}) or a data frame. When a formula is
provided, \code{data} must also be supplied. When a data frame is provided,
\code{formula} must be supplied as the second argument.}

\item{...}{Additional arguments (currently unused).}

\item{formula}{A formula specifying the relationship (used with the data.frame method).}

\item{data}{A data frame containing the variables referenced in the formula.}

\item{id_col}{Character string. Name of the column identifying subjects/blocks
for paired or repeated-measures designs. When supplied, the wizard (a) verifies
the pairing structure (each subject should appear in every group exactly once),
(b) treats the design as paired/repeated measures, and (c) embeds the real column
name into the generated code. Omit for independent-samples designs. Default \code{NULL}.}

\item{run}{Logical. If \code{TRUE}, the wizard attempts to execute the recommended
rfriend function and stores the result in \code{$run_result}. Only works for
unambiguous single-function recommendations (not multi-step or external packages).
Default \code{FALSE}.}

\item{plots}{Logical. If \code{TRUE}, generates diagnostic plots using
\code{f_hist()} (histogram of the response) and \code{f_qqnorm()} (QQ-plot
of model residuals). Plots are stored in the result as \code{$histogram}
and \code{$qqplot} (\code{recordedplot} objects) and displayed by the
print method. Default \code{FALSE}.}

\item{output_type}{Character string specifying the output format of the
recommended rfriend function (when \code{run=TRUE}) and the displayed code
strings. Passed through to \code{f_aov()}, \code{f_t_test()}, \code{f_glm()},
etc. Valid values match those of the underlying function (\code{"word"},
\code{"pdf"}, \code{"excel"}, \code{"console"}, \code{"default"}, \code{"rmd"}).
Default \code{"word"}.}

\item{interactive}{Logical. If \code{TRUE}, asks the user questions about study design. Default \code{FALSE}.}

\item{data_name}{Character string to name the data base used. Default \code{NULL},
which automatically derives the data name from the data frame used as input.}
}
\value{
An object of class \code{"f_stat_wizard"}: a list containing:
  \describe{
    \item{formula}{The formula used.}
    \item{formula_text}{Character string of the formula.}
    \item{data_name}{Name of the data object as passed by the user.}
    \item{n}{Effective sample size (after NA removal).}
    \item{n_dropped}{Number of rows removed due to missing values.}
    \item{paired}{Logical. Whether a paired/repeated-measures design was detected (via \code{id_col}).}
    \item{id_col}{Character. Name of the subject/block column supplied, or \code{NULL}.}
    \item{y_var}{Name of the response variable.}
    \item{y_type}{Detected type of the response: \code{"binary"}, \code{"count"},
      \code{"multinomial"}, \code{"ratio_normal"}, \code{"ratio_non_normal"},
      \code{"ratio_unknown"}, or \code{"unsupported"}.}
    \item{x_vars}{Character vector of explanatory variable names.}
    \item{x_types}{Character vector of detected types (\code{"nominal"},
      \code{"ordinal"}, \code{"ratio"}).}
    \item{n_groups}{Number of groups (for single categorical X), or \code{NULL}.}
    \item{group_sizes}{Table of per-group sample sizes, or \code{NULL}.}
    \item{is_ancova}{Logical. \code{TRUE} if the model mixes nominal and ratio predictors.}
    \item{has_interaction}{Logical. \code{TRUE} if interaction terms were detected.}
    \item{normality}{A list with \code{p_value} (Shapiro-Wilk) and \code{is_normal} (logical or \code{NA}).}
    \item{variance}{A list with \code{test_used} (\code{"Levene"} or \code{"Bartlett"}),
      \code{p_value}, and \code{is_equal} (logical).}
    \item{boxcox}{A list with \code{attempted} (logical), \code{can_fix} (logical),
      and \code{p_value_after} (numeric or \code{NA}).}
    \item{overdispersion}{A list with \code{is_overdispersed} (logical, from
      DHARMa dispersion test) and \code{p_value}. Only meaningful for count data.}
    \item{recommended_call}{A language object representing the rfriend function call,
      or \code{NULL} if no single function could be determined.}
    \item{run_result}{The result of executing the recommended test (when \code{run=TRUE}),
      or \code{NULL}.}
    \item{histogram}{A \code{recordedplot} from \code{f_hist()} (when \code{plots=TRUE}),
      or \code{NULL}.}
    \item{qqplot}{A \code{recordedplot} from \code{f_qqnorm()} of model residuals
      (when \code{plots=TRUE} and Y is continuous), or \code{NULL}.}
    \item{report}{Character vector of the human-readable report lines (used by
      \code{print.f_stat_wizard}).}
  }
}
\description{
Analyzes your data structure based on a formula and recommends the appropriate statistical test.
Checks variable types, normality of residuals, homogeneity of variance, and checks if f_boxcox
transformation can fix non-normality. Recommends rfriend functions as primary code, with
base R alternatives shown as fallback.

Supports standard formulas including \code{y ~ .}, \code{y ~ as.factor(x)}, and interaction
terms. Formulas with random effects (e.g. \code{(1|ID)}) are detected and handled separately.
Multivariate responses (e.g. \code{cbind(y1, y2) ~ x}) and transformed responses
(e.g. \code{log(y) ~ x}) are not supported.
}
\examples{
# Formula interface (recommended)
f_stat_wizard(Sepal.Length ~ Species, data = iris)

# Data-first interface (backward compatible)
f_stat_wizard(iris, Sepal.Length ~ Species)

# Paired design -- supply the id_col that identifies matched subjects
f_stat_wizard(extra ~ group, data = sleep, id_col = "ID")

# With diagnostic plots
f_stat_wizard(Sepal.Length ~ Species, data = iris, plots = TRUE)

# Run the recommended test directly
result <- f_stat_wizard(Sepal.Length ~ Species, data = iris, run = TRUE)
result$run_result

# Inspect metadata
result <- f_stat_wizard(Sepal.Length ~ Species, data = iris)
result$y_type
result$normality
result$group_sizes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fopen_file.R
\name{f_open_file}
\alias{f_open_file}
\title{Open a File with the Default Application}
\usage{
f_open_file(filepath)
}
\arguments{
\item{filepath}{A character string specifying the path to the file to be opened. The path can be absolute or relative.}
}
\value{
Does not return a value; it is called for its side effect of opening a file.
}
\description{
Opens a specified file using the default application associated with its file type. It automatically detects the operating system (Windows, Linux, or macOS) and uses the appropriate command to open the file.
}
\details{
- On \bold{Windows}, the \code{f_open_file()} function uses \code{shell.exec()} to open the file.
- On \bold{Linux}, it uses \code{xdg-open} via the \code{system()} function.
- On \bold{macOS}, it uses \code{open} via the \code{system()} function.

If an unsupported operating system is detected, the function will throw a message.
}
\examples{
# NOTE: The use of "if(interactive())" prevents this example from running
# during automated CRAN checks. This is necessary because the example
# opens a file, a behavior restricted by CRAN policies for automated
# testing.You don't need to use "if(interactive())" in your own scripts.
if(interactive()) {
# Open a PDF file.
f_open_file("example.pdf")

# Open an image file.
f_open_file("image.png")

# Open a text file.
f_open_file("document.txt")
}

}
\seealso{
[shell.exec()], [system()]
}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fboxcox.R
\name{f_boxcox}
\alias{f_boxcox}
\title{f_boxcox: A User-Friendly Box-Cox Transformation}
\usage{
f_boxcox(
  data = data,
  digits = 3,
  range = c(-2, 2),
  plots = NULL,
  transform.data = TRUE,
  eps = 1/50,
  xlab = expression(lambda),
  ylab = "log-Likelihood",
  alpha = 0.05,
  open_generated_files = interactive(),
  close_generated_files = FALSE,
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  ...
)
}
\arguments{
\item{data}{A numeric vector or a data frame with a single numeric column. The data to be transformed.}

\item{digits}{Numeric. Determines the accuracy of the estimate for lambda. Higher values increase computation time. Defaults to \code{3}.}

\item{range}{A numeric vector of length 2 defining the search interval for lambda. Defaults to \code{c(-2, 2)}.}

\item{plots}{Logical. If \code{TRUE}, plots log-likelihood of the Box-Cox transformation, Histograms and Q-Q plots of the original and transformed data. Default is \code{FALSE}.}

\item{transform.data}{Logical. If \code{TRUE}, returns the transformed data. Default is \code{TRUE}.}

\item{eps}{A small positive value used to determine when to switch from the power transformation to the log transformation for numerical stability. Default is \code{1/50}.}

\item{xlab}{Character string. Label for the x-axis in plots. Default is an expression object representing \eqn{\lambda}.}

\item{ylab}{Character string. Label for the y-axis in plots. Default is "log-Likelihood".}

\item{alpha}{Numeric. Significance level for the Shapiro-Wilk test of normality. Default is \code{0.05}.}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{close_generated_files}{Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!}

\item{output_type}{Character string specifying the output format. Default is \code{"default"}.
\itemize{
  \item \code{"default"}: Returns the object and lets R decide whether
    to print; auto-prints if unassigned, silent if assigned to a variable.
    Use \code{print(result)} or \code{plot(result)} to display the
    returned object.
  \item \code{"console"}: Forces immediate printing to the console
    regardless of object assignment.
  \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
    file of the corresponding format. See \code{save_as},
    \code{save_in_wdir}, and \code{open_generated_files} for file
    path and opening behavior.
  \item \code{"rmd"}: Stores the raw markdown string inside the returned
    object for use in R Markdown documents.
}}

\item{save_as}{Character string specifying the output file path (without extension).
If a full path is provided, output is saved to that location.
If only a filename is given, the file is saved in \code{tempdir()}.
If only a directory is specified (providing an existing directory with trailing slash),
the file is named "data_name_aov_output" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
Defaults to \code{file.path(tempdir(), "data_name_summary.pdf")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.}

\item{...}{Additional arguments passed to plotting functions.}
}
\value{
An object of class 'f_boxcox' containing, among others, results from the boxcox transformation, lambda, the input data, transformed data, Shapiro-Wilk test on original and transformed data. Using the option "output_type", it can also generate output in the form of: R Markdown code, 'Word', or 'pdf' files. Includes print and plot methods for 'f_boxcox' objects.
}
\description{
Performs a Box-Cox transformation on a dataset to stabilize variance and make the data more normally distributed. It also provides diagnostic plots and tests for normality. The transformation is based on code of MASS/R/boxcox.R. The function prints \eqn{\lambda} to the console and returns (output) the transformed data set.
}
\details{
The function uses the following formula for transformation:
\deqn{
 y(\lambda) =
   \begin{cases}
 \frac{y^\lambda - 1}{\lambda}, & \lambda \neq 0 \\  \log(y), & \lambda = 0
 \end{cases}
}

where (\eqn{y}) is the data being transformed, and (\eqn{\lambda}) the transformation parameter, which is estimated from the data using maximum likelihood. The function computes the Box-Cox transformation for a range of \eqn{\lambda} values and identifies the \eqn{\lambda} that maximizes the log-likelihood function. The beauty of this transformation is that, it checks suitability of many of the common transformations in one run. Examples of most common transformations and their \eqn{\lambda} value is given below:
\renewcommand{\arraystretch}{3} % Adjusts row height (default is 1)
\tabular{cc}{
  \strong{\eqn{\lambda}-Value} \tab \strong{Transformation} \cr
  \strong{-----------------------}\tab\strong{-----------------------}\cr
  -2          \tab \eqn{\frac{1}{x^2}}          \cr
  -1          \tab \eqn{\frac{1}{x}  }          \cr
  -0.5        \tab \eqn{\frac{1}{\sqrt{x}}}     \cr
  0           \tab \eqn{log(x)}                 \cr
  0.5         \tab \eqn{\sqrt{x}}               \cr
  1           \tab \eqn{x}                      \cr
  2           \tab \eqn{x^2}                    \cr
  \strong{-----------------------}\tab\strong{-----------------------}\cr
}

If the estimated transformation parameter closely aligns with one of the values listed in the previous table, it is generally advisable to select the table value rather than the precise estimated value. This approach simplifies interpretation and practical application.

The function provides diagnostic plots: a plot of log-likelihood against \eqn{\lambda} values and a Q-Q plot of the transformed data.It also performs a Shapiro-Wilk test for normality on the transformed data if the sample size is less than or equal to 5000.

\strong{Note}: For sample sizes greater than 5000, Shapiro-Wilk test results are not provided due to limitations in its applicability.

This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
\itemize{
\item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
\item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
\item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.

\item If Pandoc is not found, this function may not work as intended.
}
}
\examples{
# Create non-normal data in a data.frame or vector.
df   <- data.frame(values = rlnorm(100, meanlog = 0, sdlog = 1))

# Store the transformation in object "bc".
bc <- f_boxcox(df$values)

# Print lambda and Shaprio.
print(bc)

# Plot the QQ plots, Histograms and Lambda Log-Likelihood estimation.
plot(bc)

# Or Directly use the transformed data from the f_boxcox object.
df$values_transformed <- f_boxcox(df$values)$transformed_data
print(df$values_transformed)

}
\references{
The core of calculating \eqn{\lambda} and the plotting was taken from: \cr
file MASS/R/boxcox.R copyright (C) 1994-2004 W. N. Venables and B. D. Ripley
\itemize{
\item \url{https://r-coder.com/box-cox-transformation-r/}
\item \url{https://CRAN.R-project.org/package=MASS}
}
Some code to present the result was taken and modified from file: \cr
rcompanion/R/transformTukey.r. (Developed by Salvatore Mangiafico)
\itemize{
\item \url{https://rcompanion.org/handbook/I_12.html}
}
The explanation on BoxCox transformation provided here was provided by r-coder:
\itemize{
\item  \url{https://r-coder.com/box-cox-transformation-r/}
}
}
\seealso{
\href{https://CRAN.R-project.org/package=MASS}{\code{boxcox}}
}
\author{
\itemize{
\item Sander H. van Delden  \email{plantmind@proton.me}
\item Salvatore Mangiafico, \email{mangiafico@njaes.rutgers.edu}
\item W. N. Venables and B. D. Ripley
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flong.R
\name{f_long}
\alias{f_long}
\title{Transform 'Wide' (Excel) data to 'Long' (R) format}
\usage{
f_long(
  data,
  measure_columns = NULL,
  keep_cols = NULL,
  category_name = "name",
  value_name = "value",
  category_labels = NULL,
  ...
)
}
\arguments{
\item{data}{The input data frame (e.g., from \code{read_excel}).}

\item{measure_columns}{(Optional) The columns containing your numeric measurements.
These values are often the response variables, i.e. will end up on the Y-axis.
\itemize{
  \item Use column names: \code{c("OD_t0", "OD_t1")}
  \item Use column numbers: \code{2:5}
  \item Use helpers: \code{starts_with("Measure")}
}
If NULL (default), the function will pivot ALL columns except those in \code{keep_cols}.}

\item{keep_cols}{(Optional) The columns that identify your samples (IDs).
E.g., "SampleID", "PatientID", "Treatment", "Student number".
These are repeated for every measurement.
*If left empty, all non-measured columns are kept.*
Important: If \code{measure_columns} is NULL, you MUST specify \code{keep_cols}.}

\item{category_name}{Name for the new column containing the headers.
Default is "name". Choose something logical like "Timepoints", "Genes", or "Condition".}

\item{value_name}{Name for the new column containing the numbers.
Default is "value". Choose something logical like "Absorbance", "Ct_Value", or "Weight".}

\item{category_labels}{(Optional) A character vector of new, readable names for your categories, i.e. the measure_columns that you entered.
\strong{Note:} The order must match the order of \code{measure_columns} exactly.
Useful for renaming "t0_raw" to "Start" instantly.}

\item{...}{Additional arguments passed to \code{tidyr::pivot_longer}.
E.g., \code{values_drop_na = TRUE} to remove empty cells immediately.}
}
\value{
A "Tidy" data frame (tibble) of class \code{f_long}.
}
\description{
This function converts "wide" data (e.g. Excel tables) into a "long" list format.
This is the essential first step to prepare your data for analysis and plotting in R.
}
\details{
Research data in Excel or output from lab instruments often contains measurements
side-by-side (in columns). Many R functions require measurements in a single
column (rows). `f_long` performs this translation for you.

It performs three actions in one go:
1. Selects your measurement columns (`measure_columns`).
2. Keeps your important ID columns (`keep_cols`) and removes the rest.
3. (Optional) Renames cryptic column headers into readable labels (`category_labels`).
}
\note{
The custom class and attributes (\code{f_long_value}, \code{f_long_category})
  are used by the \code{plot} and \code{summary} methods. Be aware that most
  \pkg{dplyr} or \pkg{tidyr} operations (e.g., \code{filter}, \code{mutate})
  will silently strip these attributes. If that happens, use \code{f_scan} or
  \code{f_summary} directly with explicit column names instead.
}
\examples{
# --- Example 1: Using the 'iris' dataset ---
# Scenario: The iris dataset looks clean, but it is actually "Wide".
# It has 4 columns of measurements side-by-side.
# To compare Sepal Length vs Width in a plot, we must stack them.

head(iris)

# Reshape: Combine Length and Width into one column and plot the data.
iris_long <- f_long(
  data = iris,
  measure_columns = c("Sepal.Length", "Sepal.Width"),
  keep_cols = "Species",
  category_name = "Sepal_Dimension",    # Describes the grouping (What did we measure?)
  value_name = "Size_cm",               # Describes the value (What is the number?)
  category_labels=c("Length", "Width")  # New category labels
  )

 head(iris_long)

 # Plot the data using f_scan
 plot(iris_long)

 # Make a f_summary table of iris_long
 summary(iris_long)

# --- Example 2: Using the 'airquality' dataset ---
# Scenario: Pivot daily measurements of Wind and Temperature over time.

head(airquality)

weather_long <- f_long(
  data = airquality,
  measure_columns = c("Wind", "Temp"),
  keep_cols = c("Month", "Day"),
  category_name = "Climate_Parameter", # Descriptive name
  value_name = "Reading_Value",        # Generic name (since units differ: mph vs F)
  values_drop_na = TRUE
)

head(weather_long)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fcorplot.R
\name{f_corplot}
\alias{f_corplot}
\title{Correlation Plots with Factor Detection and Multiple Correlation Coefficients}
\usage{
f_corplot(
  data,
  detect_factors = TRUE,
  factor_table = FALSE,
  factor_exclude = NULL,
  factor_select = NULL,
  unique_num_treshold = 8,
  repeats_threshold = 2,
  color_factor = "auto",
  shape_factor = "auto",
  print_legend = TRUE,
  fancy_names = NULL,
  ordinal_vars = NULL,
  width = 15,
  height = 15,
  res = 600,
  pointsize = 10,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  output_type = "word",
  save_as = NULL,
  save_in_wdir = FALSE
)
}
\arguments{
\item{data}{A \code{data.frame} containing the dataset. Must include at least
two numeric variables.}

\item{detect_factors}{Logical. If \code{TRUE}, factor variables are
automatically detected for colouring and shaping points. Default \code{TRUE}.}

\item{factor_table}{Logical. If \code{TRUE}, prints a detailed table of
converted factors to the console. Default \code{FALSE}.}

\item{factor_exclude}{A character vector specifying the names of the columns NOT to convert into factors. If \code{NULL}, no columns are excluded. Default is \code{NULL}.}

\item{factor_select}{A character vector specifying the names of the columns to convert into factors. If \code{NULL}, the function automatically detects columns that should be factors based on their data type and unique value count. Default is \code{NULL}.}

\item{unique_num_treshold}{Numeric. A threshold of the amount of unique numbers a numeric column should have to keep it numeric, i.e. omit factor conversion. Default \code{8}.}

\item{repeats_threshold}{Numeric. A threshold of the minimal number of repeats a numeric column should have to convert it to a factor. Default \code{2}.}

\item{color_factor}{Character. Name of the factor variable used for point
colours; \code{"auto"} selects automatically. Default \code{"auto"}.}

\item{shape_factor}{Character. Name of the factor variable used for point
shapes; \code{"auto"} selects automatically. Default \code{"auto"}.}

\item{print_legend}{Logical. If \code{TRUE}, a separate legend file is
created. Default \code{TRUE}.}

\item{fancy_names}{Named character vector or \code{NULL}. Maps column names
to display names used in the plot and legend.}

\item{ordinal_vars}{Character vector or \code{NULL}. Names of variables to
treat as ordinal. Ordered factors are coerced to integer ranks; other
non-numeric types are coerced similarly. Their diagonal labels are
italicised and Pearson \eqn{r}{r} is greyed and bracketed for any pair that
involves them. Ordinal variables are included in the correlation panels but
excluded from aesthetic factor detection. Default \code{NULL}.}

\item{width}{Numeric. Plot width in centimetres. Default 15.}

\item{height}{Numeric. Plot height in centimetres. Default 15.}

\item{res}{Numeric. Resolution in DPI. Default 600.}

\item{pointsize}{Numeric. Base font size. Default 8.}

\item{close_generated_files}{Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{output_type}{Character. One of \code{"pdf"}, \code{"word"},
\code{"png"}, or \code{"rmd"}. Default \code{"word"}.}

\item{save_as}{Character or \code{NULL}. Output file path without extension.
A full path, a filename, or a directory (with trailing slash) are all
accepted. Providing an extension overrides \code{output_type}.
Default saves to \code{tempdir()}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves to the working directory.
Overridden by \code{save_as}. Default \code{FALSE}.}
}
\value{
No value is returned to the R environment. Output files are saved
  and opened automatically.
}
\description{
Creates correlation plots for numeric variables in a data frame. The upper
triangle displays Pearson \eqn{r}{r}, Spearman \eqn{\rho}{rho}, and Kendall
\eqn{\tau}{tau} simultaneously for each pair. Factor variables are automatically detected and
used for grouping, i.e. point colouring and shaping. Ordinal variables are supported via
\code{ordinal_vars}: their diagonal labels are italicised and Pearson \eqn{r}{r}
is greyed and bracketed for any pair that involves them.A separate legend file documents both
the grouping factors and the meaning of all three correlation symbols.
}
\details{
\itemize{
  \item \strong{Three correlations per panel:} Every upper-triangle panel
    shows \eqn{r}{r} (Pearson), \eqn{\rho}{rho} (Spearman), and \eqn{\tau}{tau}
    (Kendall) stacked vertically, so the reader can choose the most
    appropriate coefficient for each variable pair.
  \item \strong{Ordinal variables:} Specify column names with
    \code{ordinal_vars}. Those variables appear in italic on the diagonal.
    For any pair where at least one variable is ordinal, Pearson \eqn{r}{r} is
    shown greyed and in parentheses to signal it is technically inappropriate;
    Spearman and Kendall remain prominent.
  \item \strong{Factor detection:} Only unordered factors are used for
    colour/shape aesthetics. Ordered factors (\code{is.ordered()}) are
    treated as ordinal data, not as grouping variables.
  \item \strong{Legend:} The legend file documents the grouping factor levels
    (when present) and always includes an explanation of all three
    correlation symbols whenever a legend is generated.
  \item \strong{Constant columns:} Zero-variance columns produce \code{NA}
    in all correlation panels.
}

This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
\itemize{
\item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
\item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
\item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.

\item If Pandoc is not found, this function may not work as intended.
}
}
\examples{
data(mtcars)
mtcars_sub <- subset(mtcars, select = -c(am, qsec, vs))
f_corplot(mtcars_sub,
          color_factor = "gear",
          shape_factor = "cyl",
          output_type  = "png"
          )

# With ordinal variables
data(iris)
fancy_names <- c(Sepal.Length = "Sepal Length (cm)",
                 Sepal.Width  = "Sepal Width (cm)")
f_corplot(iris,
          fancy_names  = fancy_names,
          ordinal_vars = "Petal.Width",
          output_type  = "png",
          open_generated_files = FALSE)

}
\author{
Sander H. van Delden \email{plantmind@proton.me}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fqqnorm.R
\name{f_qqnorm}
\alias{f_qqnorm}
\title{Normal Q-Q Plot with Confidence Bands}
\usage{
f_qqnorm(
  x,
  main = NULL,
  ylab = NULL,
  conf_level = 0.95,
  col = NULL,
  pch = NULL,
  cex = NULL,
  save_png = FALSE,
  open_png = TRUE,
  save_as = NULL,
  save_in_wdir = FALSE,
  width = 8,
  height = 7,
  units = "in",
  res = 300,
  ...
)
}
\arguments{
\item{x}{A numeric vector of data values.}

\item{main}{A character string specifying the title of the histogram. Default is "Histogram with Normal Curve".}

\item{ylab}{A character string specifying the y-axsis label. Default name is  \code{"Quantiles of: data_name"}.}

\item{conf_level}{Numeric, between 0 and 1. Confidence level for the confidence bands. Default is 0.95 (95\% confidence).}

\item{col}{Numeric, optional parameter for color of point with default 'black'.}

\item{pch}{Numeric, optional parameter shape of points default \code{pch = 19}.}

\item{cex}{Numeric, optional parameter for graph cex with default \code{cex = 0.6}.}

\item{save_png}{A logical value default \code{FALSE}, if \code{TRUE} a png file is saved under the name of the data of under the specified file name.}

\item{open_png}{Logical. If \code{TRUE}, opens generated png files.}

\item{save_as}{Character string specifying the output file path (without extension).
If a full path is provided, output is saved to that location.
If only a filename is given, the file is saved in \code{tempdir()}.
If only a directory is specified (providing an existing directory with trailing slash),
the file is named "data_name_QQplot.png" in that directory.
Defaults to \code{file.path(tempdir(), "data_name_histogram.png")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.}

\item{width}{Numeric, png figure width default \code{8} inch.}

\item{height}{Numeric, png figure height default \code{7} inch.}

\item{units}{Numeric, png figure units default inch.}

\item{res}{Numeric, png figure resolution default \code{300} dpi.}

\item{...}{Additional graphical parameters to be passed to the \code{qqnorm} function.}
}
\value{
A Q-Q plot is created and the function returns this as a \code{recordedplot}.
}
\description{
This function creates a normal Q-Q plot for a given numeric vector and adds confidence bands to visualize the variability of the quantiles.
}
\details{
The function calculates theoretical quantiles for a normal distribution and compares them with the sample quantiles of the input data.

It also computes confidence intervals for the order statistics using the Blom approximation and displays these intervals as shaded bands on the plot.

The reference line is fitted based on the first and third quartiles of both the sample data and theoretical quantiles.

To increase resolution you can use \code{png(...,res = 600)} or the 'RStudio' chunck setting, e.g. \code{dpi = 600}.
}
\examples{
# Generate random normal data
set.seed(123)
data <- rnorm(100)

# Create a Q-Q plot with confidence bands
f_qqnorm(data)

# Customize the plot with additional graphical parameters
f_qqnorm(data, conf_level = 0.99, pch = 16, col = "blue")

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fscan.R
\name{f_scan}
\alias{f_scan}
\alias{f_scan.formula}
\alias{f_scan.data.frame}
\title{Perform a visual check on your data}
\usage{
f_scan(x, ...)

\method{f_scan}{formula}(formula, data, ...)

\method{f_scan}{data.frame}(
  x,
  columns,
  group_vars = NULL,
  summary = TRUE,
  outliers = TRUE,
  coef = 1.5,
  limit_columns = 7,
  fancy_names = NULL,
  advice = FALSE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  digits = NULL,
  ...
)
}
\arguments{
\item{x}{A data.frame or formula (dispatches to the right method).}

\item{...}{Further arguments forwarded to \code{f_scan.data.frame}.}

\item{formula}{A formula specifying the columns (right hand side) to be summarized by maximal 3 groups (left hand side). More columns or groups can be added using \code{-} or \code{+} (e.g., \code{col1 + col2 ~ group1 + group2}) to do a sequential summary for each column parameter.}

\item{data}{A 'data.frame', 'data.table', or 'tibble'.}

\item{columns}{The numerical column(s) to summarize if no formula is used. Can be entered as a single character string (e.g., \code{"weight"}) or as a character vector \code{c("weight", "length"}).}

\item{group_vars}{Character vector of up to 3 grouping variables (e.g., \code{c("species", "fertilizer")}).}

\item{summary}{Logical. Show a summary table of the data. Default is \code{TRUE}.}

\item{outliers}{Logical. If \code{TRUE}, scans for outliers using Tukey's fences and if they exist, adds them to the result object. Default \code{TRUE}.}

\item{coef}{Numeric. The multiplier for the Interquartile Range (IQR) used for outlier detection. Default \code{1.5}.}

\item{limit_columns}{Integer or \code{NULL}. Defines the number of columns shown in the outlier table. Default = \code{7}. \code{NULL} = all columns are shown.}

\item{fancy_names}{Named character vector or \code{NULL}. Optional mapping of column names to more readable names for display in plots and legends.}

\item{advice}{Logical. If \code{TRUE}, runs \code{f_stat_wizard()} on each response
column and appends the recommendation to the result. The advice is accessible
via \code{result[["column_name"]]$advice} and is printed automatically.
Default \code{FALSE}.}

\item{close_generated_files}{Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{output_type}{Character string specifying the output format. Default is \code{"default"}.
\itemize{
  \item \code{"default"}: Returns the object and lets R decide whether
    to print; auto-prints if unassigned, silent if assigned to a variable.
    Use \code{print(result)} or \code{plot(result)} to display the
    returned object.
  \item \code{"console"}: Forces immediate printing to the console
    regardless of object assignment.
  \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
    file of the corresponding format. See \code{save_as},
    \code{save_in_wdir}, and \code{open_generated_files} for file
    path and opening behavior.
  \item \code{"rmd"}: Stores the raw markdown string inside the returned
    object for use in R Markdown documents.
}}

\item{save_as}{Character string specifying the output file path (without extension).
If a full path is provided, output is saved to that location.
If only a filename is given, the file is saved in \code{tempdir()}.
If only a directory is specified (providing an existing directory with trailing slash),
the file is named "dataname_fscan" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
Defaults to \code{file.path(tempdir(), "dataname_fscan.pdf")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.}

\item{digits}{Integer. Decimal places for printed tables in 'pdf' and 'Word' output files. Default \code{3}.}
}
\value{
A list of class \code{f_scan} containing plots, the summary table, and the outlier table. Using the option "output_type", it can also generate output in the form of: R Markdown code, 'Word', 'pdf', or 'Excel' files. Includes print, summary and plot methods for 'f_scan' objects.
}
\description{
Creates a 3-panel diagnostic dashboard to check data distribution and assumptions. It can also output a data summary table and identify outliers.
}
\details{
\code{f_scan} automatically adapts the visualization based on the number of grouping variables provided:
\itemize{
  \item \strong{0 groups:} Univariate analysis (Single density/boxplot).
  \item \strong{1 group :} Main grouping variable (X-axis and Color).
  \item \strong{2 groups:} Adds Facet Wrapping.
  \item \strong{3 groups:} Adds Facet Grid (Row vs Column).
}


This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
\itemize{
\item \bold{Windows:} Install Pandoc and ensure the installation folder.
\cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
\item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
\item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.

\item If Pandoc is not found, this function may not work as intended.
}
}
\examples{
# 1. Non-formula | No groups | Default output (default)
result <- f_scan(iris, columns = "Sepal.Length")
print(result)

# 2. Non-formula | 1 group | Console output
result <- f_scan(
  mtcars,
  columns   = "mpg",
  group_vars = "cyl",
  output_type = "console"
)

\donttest{
# 3. Non-formula | 2 groups | Multiple columns | Excel output
 result <- f_scan(
   mtcars,
   columns    = c("mpg", "hp"),
   group_vars = c("cyl", "am"),
   outliers   = TRUE,
   coef       = 1.5,
   output_type = "excel",
   save_as    = "mtcars_scan"
 )

# 4. Formula | 1 group | Strict outlier detection | Word output
result <- f_scan(
  Sepal.Width ~ Species,
  data        = iris,
  outliers    = TRUE,
  coef        = 3.0,
  output_type = "word",
  save_as     = "iris_scan"
 )

# 5. Formula | 2 groups | Multiple columns | Fancy names
result <- f_scan(
 mpg + hp + wt ~ vs + am,
 data        = mtcars,
 fancy_names = c(mpg = "Fuel Efficiency", hp = "Horsepower",
                 wt  = "Weight",          vs = "Engine Type",
                 am  = "Transmission"),
 summary     = TRUE
)
print(result)


#Create a small reproducible dataset with 3 grouping variables
set.seed(42)
plant_data <- data.frame(
  weight    = c(rnorm(60, 10, 2), rnorm(60, 14, 2)),
  species   = rep(c("A", "B"), each = 60),
  treatment = rep(rep(c("control", "treated"), each = 30), 2),
  batch     = factor(rep(c("1", "2", "3"), 40))
 )

# 6. Formula | 3 groups | Facet Grid
result <- f_scan(
  weight ~ species + treatment + batch,
  data         = plant_data,
  coef         = 2.0,
  digits       = 2,
  output_type  = "word"
)
print(result)

# 7. With statistical advice
result <- f_scan(
  Sepal.Length ~ Species,
  data    = iris,
  advice  = TRUE
)
#' print(result)
result[["Sepal.Length"]]$advice$y_type
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/df_to_table.R
\name{df_to_table}
\alias{df_to_table}
\title{Convert a data frame to a contingency table}
\usage{
df_to_table(df, label_col = NULL)
}
\arguments{
\item{df}{A data frame. Either (a) one column contains row labels and the rest are
numeric, (b) a fully numeric data frame with meaningful \code{rownames()} set, or
(c) a fully numeric data frame with no row labels at all (in which case the data
frame is coerced directly to a table without row labels).}

\item{label_col}{Index or name of the column containing row labels. If NULL (default),
the function auto-detects the first character/factor column. If no such column is
found, the function falls back to using \code{rownames(df)} when these are
meaningful, otherwise it coerces the data frame to a table as-is.}
}
\value{
A contingency table.
}
\description{
Convert a data frame to a contingency table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsetwd.R
\name{f_setwd}
\alias{f_setwd}
\title{Set Working Directory Based on Current File or Specified Path}
\usage{
f_setwd(path = NULL)
}
\arguments{
\item{path}{A character string specifying the desired working directory. If \code{NULL} (default), the function sets the working directory to the location of the currently open and saved file in 'RStudio'.}
}
\value{
None. The function is called for its side effects of changing the working directory.
}
\description{
A wrapper around \code{setwd()} that sets the working directory to the location of the currently open file in 'RStudio' if no path is provided. If a path is specified, it sets the working directory to that path instead.
}
\details{
If \code{path} is not provided (\code{NULL}), this function uses the \code{this.path} package to determine the location of the currently open file and sets that as the working directory. The file must be saved for this to work properly.

If a valid \code{path} is provided, it directly sets the working directory to that path.
}
\note{
\itemize{
\item The function checks whether the currently open file is saved before setting its location as the working directory.
\item If the function is called from an unsaved script or directly from the console, an error will be thrown.
}
}
\examples{
# NOTE: The use of "if(interactive())" prevents this example from running
# during automated CRAN checks. This is necessary because the example
# requires to be run from an R script. You don't need to use
# "if(interactive())" in your own scripts.
if(interactive()) {
# Store the current working directory, so we can reset it after the example.
current_wd <-  getwd()
print(current_wd)

# Run this commando from a saved R script file, or R Notebook to set the working
# directory to scripts' file location
f_setwd()

# Restore your current working directory
f_setwd(current_wd)
}


}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faov.R
\name{f_aov}
\alias{f_aov}
\title{Perform multiple \code{aov()} functions with optional data transformation, inspection and Post Hoc test.}
\usage{
f_aov(
  formula,
  data = NULL,
  norm_plots = TRUE,
  interaction_plots = TRUE,
  ANCOVA = FALSE,
  transformation = TRUE,
  force_transformation = NULL,
  force_aov = FALSE,
  alpha = 0.05,
  adjust = "sidak",
  intro_text = TRUE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  ...
)
}
\arguments{
\item{formula}{A formula specifying the model to be fitted. More response variables can be added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor}) to do a sequential \code{aov()} for each response parameter.}

\item{data}{A data frame containing the variables in the model.}

\item{norm_plots}{Logical. If \code{TRUE}, diagnostic residual plots are included in the output files. Default is \code{TRUE}.}

\item{interaction_plots}{Logical. If \code{TRUE}, estimated means / interaction plots are included in the output files after the post hoc table. Default is \code{TRUE}.}

\item{ANCOVA}{Logical. If \code{TRUE}, prevents automatic conversion of predictors to factors, allowing for Analysis of Covariance (ANCOVA). Default is \code{FALSE}.}

\item{transformation}{Logical or character string. If \code{TRUE}, or if \code{"boxcox"} applies a \code{f_boxcox()} transformation if residuals are not normal. If \code{"bestnormalize"}, applies \code{f_bestNormalize()} transformation. If \code{FALSE} no transformation will be applied. Default is \code{TRUE}.}

\item{force_transformation}{Character string. A vector containing the names of response variables that should be transformed regardless of the normality test. Default is \code{NULL}}

\item{force_aov}{Logical. If \code{TRUE}, runs the ANOVA even when at least
one cell has \eqn{n = 1} (saturated model). By default (\code{FALSE}),
such responses are skipped with a warning because F-statistics and
p-values are undefined for saturated models. Set to \code{TRUE} only for
diagnostic purposes -- results should \strong{not} be reported or
interpreted as valid. Default is \code{FALSE}.}

\item{alpha}{Numeric. Significance level for ANOVA, post hoc tests, and Shapiro-Wilk test. Default is \code{0.05}.}

\item{adjust}{Character string specifying the method used to adjust p-values
for multiple comparisons. Available methods include:
\describe{
  \item{"tukey"}{Tukey's Honest Significant Difference method, appropriate for
                all pairwise comparisons. Controls family-wise error rate.}
  \item{"sidak"}{Sidak correction that controls the family-wise error rate.
                Less conservative than Bonferroni.}
  \item{"bonferroni"}{Conservative adjustment that multiplies p-values by
                     the number of comparisons.}
  \item{"none"}{No adjustment. Equivalent to Fisher's LSD method.}
  \item{"fdr"}{False Discovery Rate adjustment, controls the expected proportion
              of false positives among significant results.}
} Default is \code{"sidak"}.}

\item{intro_text}{Logical. If \code{TRUE}, includes a short explanation about ANOVA assumptions in the output file. Default is \code{TRUE}.}

\item{close_generated_files}{Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{output_type}{Character string specifying the output format. Default is \code{"default"}.
\itemize{
  \item \code{"default"}: Returns the object and lets R decide whether
    to print; auto-prints if unassigned, silent if assigned to a variable.
    Use \code{print(result)} or \code{plot(result)} to display the
    returned object.
  \item \code{"console"}: Forces immediate printing to the console
    regardless of object assignment.
  \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
    file of the corresponding format. See \code{save_as},
    \code{save_in_wdir}, and \code{open_generated_files} for file
    path and opening behavior.
  \item \code{"rmd"}: Stores the raw markdown string inside the returned
    object for use in R Markdown documents.
}}

\item{save_as}{Character string specifying the output file path (without extension).
If a full path is provided, output is saved to that location.
If only a filename is given, the file is saved in \code{tempdir()}.
If only a directory is specified (providing an existing directory with trailing slash),
the file is named "dataname_aov_output" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
Defaults to \code{file.path(tempdir(), "dataname_summary.pdf")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.}

\item{...}{Additional arguments forwarded to \code{\link[stats]{aov}}.
The arguments \code{subset}, \code{na.action}, and \code{weights} are
handled specially: when supplied, they are applied via
\code{\link[stats]{model.frame}} so that the n=1 cell check,
Shapiro-Wilk test, Levene test, optional transformations, residual
diagnostics, and \code{emmeans} post hoc tests all see the exact
same row set as \code{aov()} itself. Any other \code{aov()}
arguments (e.g. \code{contrasts}, \code{projections},
\code{qr}, \code{contrasts.arg}) are passed through unchanged.}
}
\value{
An object of class 'f_aov' containing results from \code{aov()}, normality tests, transformations, and post hoc tests. Using the option "output_type", it can also generate output in the form of: R Markdown code, 'Word', 'pdf', or 'Excel' files. Includes print and plot methods for 'f_aov' objects.
}
\description{
Performs an Analysis of Variance (ANOVA) on a given dataset with options for (Box-Cox)
transformations, normality tests, and post hoc analysis. Several response parameters can be analysed in sequence and the generated output can be in various formats ('Word', 'pdf', 'Excel').
}
\details{
The function performs the following steps:
\itemize{
\item Check if all specified variables are present in the data.
\item Ensure that the response variable is numeric.
\item Perform Analysis of Variance (ANOVA) using the specified formula and data.
\item If \code{shapiro = TRUE}, check for normality of residuals using the Shapiro-Wilk test.
\item If residuals are not normal and \code{transformation = TRUE} apply a data transformation.
\item If significant differences are found in ANOVA, proceed with post hoc tests using estimated marginal means from \code{emmeans()} and Sidak adjustment (or another option of \code{adjust =}.
}

More response variables can be added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor}) to do a sequential \code{aov()} for each response parameter captured in one output file.

Outputs can be generated in multiple formats ("pdf", "word", "excel" and "rmd") as specified by \code{output_type}. The function also closes any open 'Word' files to avoid conflicts when generating 'Word' documents. If \code{output_type = "rmd"} is used it is adviced to use it in a chunk with \{r, echo=FALSE, results='asis'\}

*Non-significant ANOVA results*: When the overall F-test is not significant, f_aov still reports the estimated marginal means table, but with all pairwise comparison letters replaced by *"ns"*. The numeric estimates (and their confidence intervals) are provided because they are often needed for manuscript tables, especially when the response was back-transformed from a Box-Cox or bestNormalize scale - the raw descriptive means and the emmeans values can differ, and it is the emmeans values that correspond to the actual model. The *"ns"* labels signal that pairwise differences should not be interpreted.

This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
\itemize{
\item \bold{Windows:} Install Pandoc and ensure the installation folder.
\cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
\item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
\item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.

\item If Pandoc is not found, this function may not work as intended.
}
}
\section{Multiple Testing Across Response Variables}{

When several response variables are analysed in a single call
(e.g. \code{y1 + y2 + y3 ~ treatment}), each ANOVA is an independent
null-hypothesis test at level \code{alpha}. The post hoc adjustments
(\code{adjust = "sidak"}, \code{"tukey"}, etc.) only control the
family-wise error rate \strong{within} one ANOVA (across pairwise group
comparisons for that response). They do \strong{not} protect against
the inflation of Type I error \strong{across} the set of responses.

\strong{Practical implication:} With \eqn{k} independent response
variables all tested at \eqn{\alpha = 0.05}, the probability of
obtaining at least one false positive is
\eqn{1 - (1 - 0.05)^k}, which reaches ~40\% for \eqn{k = 10}.

\strong{When this matters:} The risk is highest in exploratory studies
where many responses are screened simultaneously without a clear
a priori hypothesis for each one. It is less of a concern when
each response is a pre-specified primary outcome with its own
biological rationale.

\strong{Possible remedies:}
\itemize{
  \item \strong{Bonferroni correction across responses:} use
    \code{alpha = 0.05 / k} where \code{k} is the number of
    response variables. Conservative but simple.
  \item \strong{False Discovery Rate (FDR):} apply
    \code{p.adjust(p_values, method = "fdr")} to the vector of
    per-response ANOVA p-values after the fact.
  \item \strong{MANOVA:} if the responses are correlated and you
    want a single omnibus test across all of them, use
    \code{manova()} before interpreting individual ANOVAs.
  \item \strong{Pre-registration:} declare primary vs. exploratory
    responses before data collection to justify differential
    correction thresholds.
}
}

\examples{
# Make a factor of Species.
iris$Species <- factor(iris$Species)

# The left hand side contains two response variables,
# so two aov's will be conducted, i.e. "Sepal.Width"
# and "Sepal.Length" in response to the explanatory variable: "Species".
f_aov_out <- f_aov(Sepal.Width + Sepal.Length ~ Species,
                   data = iris,
                   # Save output in MS Word file (Default is console)
                   output_type = "word",
                   # Do bestNormalize transformation for non-normal residual (Default is boxcox)
                   transformation = "bestnormalize"
                   )

# Print output to the console.
print(f_aov_out)

# Plot residual plots.
plot(f_aov_out)

#To print rmd output set chunck option to results = 'asis' and use cat().
f_aov_rmd_out <- f_aov(Sepal.Width ~ Species, data = iris, output_type = "rmd")
cat(f_aov_rmd_out$rmd)

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fboxplot.R
\name{f_boxplot}
\alias{f_boxplot}
\alias{f_boxplot.formula}
\alias{f_boxplot.data.frame}
\alias{f_boxplot_worker}
\title{Generate a Boxplot Report of a data.frame}
\usage{
f_boxplot(x, ...)

\method{f_boxplot}{formula}(formula, data, ...)

\method{f_boxplot}{data.frame}(x, ...)

f_boxplot_worker(
  formula = NULL,
  data,
  fancy_names = NULL,
  output_type = "pdf",
  outliers = TRUE,
  coef = 1.5,
  limit_columns = 7,
  save_as = NULL,
  save_in_wdir = FALSE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  boxplot_explanation = TRUE,
  detect_factors = TRUE,
  jitter = FALSE,
  width = 8,
  height = 7,
  units = "in",
  res = 300,
  las = 2
)
}
\arguments{
\item{x}{A data.frame or formula (dispatches to the right method).}

\item{...}{Further arguments forwarded to \code{f_boxplot_worker},
such as \code{fancy_names}, \code{title}, \code{fill}, etc.}

\item{formula}{A formula specifying the factor to be plotted. More response variables can be added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor}) to generate multiple boxplots. If the formula is omitted and only \code{data} is provided all data will be used for creating boxplots.}

\item{data}{A \code{data.frame} containing the data to be used for creating boxplots.}

\item{fancy_names}{An optional named vector mapping column names in \code{data} to more readable names for display in plots (name map). Defaults to \code{NULL}.}

\item{output_type}{Character string, specifying the output format: \code{"pdf"}, \code{"word"}, \code{"rmd"} or \code{"png"}. The option \code{"rmd"} saves rmd code in the output object not in a file. Default is \code{"pdf"}.}

\item{outliers}{Logical. If \code{TRUE}, scans for outliers using Tukey's fences and if they exist, adds them to the report using \code{f_outliers}. Default \code{TRUE}.}

\item{coef}{Numeric. The multiplier for the Interquartile Range (IQR) used for outlier detection. Default \code{1.5}.}

\item{limit_columns}{Integer or \code{NULL}. Defines the number of columns shown in the outlier table. Default = \code{7}. \code{NULL} = all columns are shown.}

\item{save_as}{Character string specifying the output file path (without extension).
If a full path is provided, output is saved to that location.
If only a filename is given, the file is saved in \code{tempdir()}.
If only a directory is specified (providing an existing directory with trailing slash),
the file is named "dataname_BoxPlot" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
Defaults to \code{file.path(tempdir(), "dataname_BoxPlot.pdf")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.}

\item{close_generated_files}{Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{boxplot_explanation}{A logical value indicating whether to include an explanation of how to interpret boxplots in the report. Defaults to \code{TRUE}.}

\item{detect_factors}{A logical value indicating whether to automatically detect factor variables in the dataset. Defaults to \code{TRUE}.}

\item{jitter}{A logical value, if \code{TRUE} all data per boxplot is shown, if \code{FALSE} (default) individual data points (except for outliers) are omitted.}

\item{width}{Numeric, png figure width default \code{8} inch}

\item{height}{Numeric, png figure height default \code{7} inch}

\item{units}{Character string, png figure units default \code{"in"} = inch, other options are: \code{"px"} = Pixels, \code{"cm"} = centimeters, \code{"mm"} = millimeters.}

\item{res}{Numeric, png figure resolution default 300 dpi}

\item{las}{An integer (\code{0} t/m \code{3}), \code{las = 0}: Axis labels are parallel to the axis. \code{las = 1}: Axis labels are always horizontal. \code{las = 2}: Axis labels are perpendicular to the axis. (default setting). \code{las = 3}: Axis labels are always vertical.}
}
\value{
The return value depends on \code{output_type}:
\itemize{
  \item \code{"pdf"} and \code{"word"}: Writes a report file to \code{save_as} (or \code{tempdir()} by default) and returns \code{NULL} invisibly. The file can optionally be opened with \code{open_generated_files = TRUE}.
  \item \code{"png"}: Writes one PNG file per response x factor combination into the directory given by \code{save_as} and returns \code{NULL} invisibly.
  \item \code{"rmd"}: Returns the generated R Markdown content as a single character string (invisibly). No file is written and nothing is printed to the console. The caller can \code{cat()} the string, assign it to a variable, or embed it in a larger report (see Examples).
}
}
\description{
Generates boxplots for all numeric variables in a given dataset, grouped by factor variables. The function automatically detects numeric and factor variables. It allows two output formats ('pdf', 'Word') and includes an option to add a general explanation about interpreting boxplots.
}
\details{
The function performs the following steps:
\itemize{
\item Detects numeric and factor variables in the dataset.
\item Generates boxplots for each numeric variable grouped by each factor variable.
\item Outputs the report in the specified format ('pdf', 'Word' or 'Rmd').
}

If \code{output_type = "rmd"} is used it is adviced to use it in a chunk with \{r, echo=FALSE, results='asis'\}

If no factor variables are detected, the function stops with an error message since factors are required for creating boxplots.

This function will plot all numeric and factor candidates, use the function \code{subset()} to prepare a selection of columns before submitting to \code{f_boxplot()}.

Note that there is an optional \code{jitter} option to plot all individual data points over the boxplots.

This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.

\bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.

\bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.

\bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.

If Pandoc is not found, this function may not work as intended.
}
\examples{
\donttest{
# Example usage:
data(iris)

new_names = c(
  "Sepal.Length" = "Sepal length (cm)" ,
  "Sepal.Width" = "Sepal width (cm)",
  "Petal.Length" = "Petal length (cm)",
  "Petal.Width" = "Petal width (cm)",
  "Species" = "Cultivar"
)

# Use the whole data.frame to generate an MS Word report and don't open it.
f_boxplot(iris,
           fancy_names = new_names,
           output_type = "word"
           )

# Use a formula to plot several response parameters (response 1 + response 2 etc)
# and generate a rmd output without boxplot_explanation.
data(mtcars)
f_boxplot(hp + disp ~ gear*cyl,
           data=mtcars,
           boxplot_explanation = FALSE,
           output_type = "word"
           )

# Capture the R Markdown output as a string and render it inline.
# Use output_type = "rmd" to get the markdown back as a character value
# instead of writing a file. Useful for embedding in a larger knitr document.
rmd <- f_boxplot(iris,
                 output_type         = "rmd",
                 boxplot_explanation = FALSE,
                 outliers            = FALSE
                 )

# Display it in the console
cat(rmd)

# ...or splice it into a knitr child chunk with results = "asis":
#   ```{r, echo=FALSE, results='asis'}
#   cat(rmd)
#   ```
}

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fglm.R
\name{f_glm}
\alias{f_glm}
\title{Perform multiple \code{glm()} functions with diagnostics, assumption checking, and post hoc analysis}
\usage{
f_glm(
  formula,
  family = gaussian(),
  data = NULL,
  diagnostic_plots = TRUE,
  alpha = 0.05,
  adjust = "sidak",
  type = "response",
  intro_text = TRUE,
  dispersion_test = TRUE,
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  influence_threshold = 2,
  ...
)
}
\arguments{
\item{formula}{A formula specifying the model to be fitted. More response variables can be
added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor}) to do
a sequential GLM for each response parameter.}

\item{family}{The error distribution and link function to be used in the model (default: gaussian()).
This can be a character string naming a family function, a family function or
the result of a call to a family function. (See \code{\link{family}} for details of family functions.)}

\item{data}{A data frame containing the variables in the model.}

\item{diagnostic_plots}{Logical. If \code{TRUE}, plots are included in the output files.}

\item{alpha}{Numeric. Significance level for tests. Default is \code{0.05}.}

\item{adjust}{Character string specifying the method used to adjust p-values
for multiple comparisons. Available methods include:
\describe{
  \item{"tukey"}{Tukey's Honest Significant Difference method}
  \item{"sidak"}{Sidak correction}
  \item{"bonferroni"}{Bonferroni correction}
  \item{"none"}{No adjustment}
  \item{"fdr"}{False Discovery Rate adjustment}
} Default is \code{"sidak"}.}

\item{type}{Character string specifying the scale of emmeans post hoc results:
\code{"response"} (back-transformed to original units, e.g. probabilities, counts)
or \code{"link"} (on the linear predictor scale, e.g. log-odds). Default is \code{"response"}.}

\item{intro_text}{Logical. If \code{TRUE}, includes a short explanation about GLM assumptions in the output file.}

\item{dispersion_test}{Logical. If \code{TRUE}, includes a dispersion diagnostic
section in the output: a DHARMa simulation-based test for overdispersion
(Poisson/Binomial), the quasi-dispersion parameter (quasi-families), or a
note explaining why the test is skipped (Bernoulli data). Default is \code{TRUE}.}

\item{output_type}{Character string specifying the output format. Default is \code{"default"}.
\itemize{
  \item \code{"default"}: Returns the object and lets R decide whether
    to print; auto-prints if unassigned, silent if assigned to a variable.
    Use \code{print(result)} or \code{plot(result)} to display the
    returned object.
  \item \code{"console"}: Forces immediate printing to the console
    regardless of object assignment.
  \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
    file of the corresponding format. See \code{save_as},
    \code{save_in_wdir}, and \code{open_generated_files} for file
    path and opening behavior.
  \item \code{"rmd"}: Stores the raw markdown string inside the returned
    object for use in R Markdown documents.
}}

\item{save_as}{Character string specifying the output file path (without extension).
If a full path is provided, output is saved to that location.
If only a filename is given, the file is saved in \code{tempdir()}.
If only a directory is specified (providing an existing directory with trailing slash),
the file is named "dataname_glm_output" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
Defaults to \code{file.path(tempdir(), "dataname_summary.pdf")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.}

\item{close_generated_files}{Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{influence_threshold}{Numeric multiplier for the leverage threshold. Observations
with hat values exceeding \code{influence_threshold * mean(hat values)} are flagged
as high-leverage points. Default is \code{2}, a common rule of thumb.}

\item{...}{Additional arguments passed to \code{glm()}.}
}
\value{
An object of class 'f_glm' (a named list, one entry per response variable) containing:
  \describe{
    \item{model}{The fitted \code{glm} object.}
    \item{summary}{Output of \code{summary(glm_fit)}.}
    \item{drop1}{Type II Analysis of Deviance table from \code{stats::drop1()}.}
    \item{diagnostics}{DHARMa residual checks and hat-value based leverage diagnostics.}
    \item{posthoc}{Estimated marginal means, pairwise comparisons, CLD letters, and summary table.}
    \item{sep_flag}{Logical indicating whether complete separation was detected.}
    \item{lrt_pct_explained}{McFadden's Pseudo-R\eqn{^2}.}
  }
  Using the option \code{output_type}, it can also generate output in the form of: R Markdown code, 'Word', 'pdf', or 'Excel' files. Includes print and plot methods for 'f_glm' objects.
}
\description{
Performs Generalized Linear Model (GLM) analysis on a given dataset with options for
diagnostics, assumption checking, and post hoc analysis. Several response parameters
can be analyzed in sequence and the generated output can be in various formats
('Word', 'pdf', 'Excel').
}
\details{
The function first checks if all specified variables are present in the data and ensures that the response variable is numeric.

It fits a Generalized Linear Model (GLM) using the specified formula, family, and data. Model diagnostics are performed with DHARMa (simulation-based residual checks including a KS test, dispersion test, and outlier test). High-leverage observations are flagged using hat values.

Significance of each predictor is assessed via Type II Analysis of Deviance (\code{stats::drop1()}). If significant effects are found, post hoc pairwise comparisons are performed using estimated marginal means from \code{emmeans()} with the chosen p-value adjustment method (default: Sidak). When complete separation is detected, the function falls back to likelihood ratio test (LRT) based pairwise comparisons, which are robust to separation.

More response variables can be added using \code{+} (e.g., \code{response1 + response2 ~ predictor}) to fit a sequential GLM for each response variable, captured in one output file.

Outputs can be generated in multiple formats ("pdf", "word", "excel" and "rmd") as specified by \code{output_type}. The function also closes any open 'Word' files to avoid conflicts when generating 'Word' documents. If \code{output_type = "rmd"} is used it is advised to use it in a chunk with \{r, echo=FALSE, results='asis'\}

This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
\itemize{
\item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
\item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
\item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.

\item If Pandoc is not found, this function may not work as intended.
}
}
\examples{
\donttest{
# GLM Binomial example with output to console
mtcars_mod <- mtcars
mtcars_mod$cyl <- as.factor(mtcars_mod$cyl)

glm_bin <- f_glm(vs ~ cyl,
                 family = binomial,
                 data = mtcars_mod,
                 output_type = "default")
print(glm_bin)

# GLM Binomial example with output to MS Word file
glm_bin_word <- f_glm(vs ~ cyl,
                 family = binomial,
                 data = mtcars_mod,
                 output_type = "word"
                 )

# GLM Poisson example with output to rmd text
data(warpbreaks)

glm_pos <- f_glm(breaks ~ wool + tension,
                 data = warpbreaks,
                 family = poisson(link = "log"),
                 intro_text = FALSE,
                 output_type = "rmd")
cat(glm_pos$rmd)
}

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fkruskal_test.R
\name{plot.f_kruskal_test}
\alias{plot.f_kruskal_test}
\title{Plot method for f_kruskal_test objects}
\usage{
\method{plot}{f_kruskal_test}(x, which = c("distributions", "Boxplot"), ...)
}
\arguments{
\item{x}{An object of class \code{f_kruskal_test}.}

\item{which}{Character vector indicating which plots to show. Options are
\code{"distributions"} (density plot), \code{"Boxplot"}, or both (default).}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Returns \code{x} invisibly.
}
\description{
Displays the density plot and/or boxplot stored in an \code{f_kruskal_test}
object. Plots are only available when the original call used \code{plot = TRUE}.
}
\examples{
result <- f_kruskal_test(Sepal.Width ~ Species, data = iris,
                         output_type = "default")
plot(result)                          # both plots
plot(result, which = "Boxplot")       # boxplot only

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fmodel_compare.R
\name{f_model_compare}
\alias{f_model_compare}
\title{Compare Two Statistical Models}
\usage{
f_model_compare(
  model1,
  model2,
  nested = NULL,
  model1_name = NULL,
  model2_name = NULL,
  digits = 3
)
}
\arguments{
\item{model1}{The first model object. Supported classes include: \code{"lm", "glm", "aov", "lmerMod", "glmerMod", and "nls"}.}

\item{model2}{The second model object. Supported classes include: \code{"lm", "glm", "aov", "lmerMod", "glmerMod", and "nls"}.}

\item{nested}{Logical. If \code{TRUE}, assumes the models are nested and performs an ANOVA comparison.
If \code{NULL} (default), the function attempts to automatically determine if the models are nested.}

\item{model1_name}{Optional character string. A custom name for model1 in the output. If \code{NULL} (default),
the function uses \code{deparse(substitute(model1))}.}

\item{model2_name}{Optional character string. A custom name for model2 in the output. If \code{NULL} (default),
the function uses \code{deparse(substitute(model2))}.}

\item{digits}{Integer. The number of decimal places to round the output metrics. Defaults to \code{3}.}
}
\value{
A list of class "f_model_comparison" containing:
  \item{model1_name}{The name of the first model (always the simpler model when nested).}
  \item{model2_name}{The name of the second model (always the more complex model when nested).}
  \item{model1_class}{The class of the first model.}
  \item{model2_class}{The class of the second model.}
  \item{metrics_table}{A data frame summarizing metrics for both models, their differences, and (if applicable) the ANOVA p-value.}
  \item{formatted_metrics_table}{A formatted version of the metrics table for printing.}
  \item{anova_comparison}{The ANOVA comparison results if the models are nested and an ANOVA test was performed.}
  \item{nested}{Logical indicating whether the models were treated as nested.}
  \item{swapped}{Logical indicating whether the model order was swapped to ensure model1 is the simpler model.}
}
\description{
Compares two statistical models by calculating key metrics such as AIC, BIC, log-likelihood, \eqn{R^2},
and others. Supports comparison of nested models using ANOVA tests.
}
\details{
Calculate various metrics to assess model fit:
  \itemize{
    \item \bold{AIC/BIC:} Lower values indicate better fit.
    \item \bold{Log-Likelihood:} Higher values (less negative) indicate better fit.
    \item \bold{\eqn{R^2}:} Proportion of variance explained by the model.
    \item \bold{Adjusted \eqn{R^2}:} \eqn{R^2} penalized for the number of parameters (for linear models).
    \item \bold{Nagelkerke \eqn{R^2}:} A pseudo-\eqn{R^2} for generalized linear models (GLMs).
    \item \bold{Marginal/Conditional \eqn{R^2}:} For mixed models, marginal \eqn{R^2} reflects fixed effects, while conditional \eqn{R^2} includes random effects.
    \item \bold{Sigma:} Residual standard error.
    \item \bold{Deviance:} Model deviance.
    \item \bold{SSE:} Sum of squared errors.
    \item \bold{Parameters (df):} Number of model parameters.
    \item \bold{Residual df:} Residual degrees of freedom.
  }

When nested models are detected or specified, model1 is always treated as the simpler
model (fewer parameters). If the user passes the complex model first, the function
automatically swaps them and issues a message.

If the models are nested, an ANOVA test is performed to compare them, and a p-value
is provided to assess whether the more complex model significantly improves fit.
}
\note{
\itemize{
    \item The function supports a variety of model types but may issue warnings if unsupported or partially supported classes are used.
    \item For GLMs, Nagelkerke's \eqn{R^2} is used as a pseudo-\eqn{R^2} approximation, computed
      from the model's null deviance to avoid refitting a null model.
    \item For mixed models, the function relies on the 'r.squaredGLMM' function from the 'MuMIn' package for \eqn{R^2} calculation.
    \item For NLS models, \eqn{R^2} is provided for convenience but should be interpreted with caution
      as it does not have the same statistical properties as in linear models.
    \item The idea of this function (not the code), I got from Dustin Fife's function \href{https://github.com/dustinfife/flexplot/blob/master/R/model.comparison.R}{'model.comparison'} in the super cool \href{https://github.com/dustinfife/flexplot/}{'flexplot package'}.
  }
}
\section{Supported Model Classes}{

The function supports the following model classes:
  \itemize{
    \item Linear models ("lm")
    \item Generalized linear models ("glm")
    \item Analysis of variance models ("aov")
    \item Linear mixed models ("lmerMod")
    \item Generalized linear mixed models ("glmerMod")
    \item Nonlinear least squares models ("nls")
  }

Note: Multi-stratum AOV models (fitted with \code{Error()}) are not supported
and will produce a warning.
}

\examples{
# Example with linear models.
model1 <- lm(mpg ~ wt, data = mtcars)
model2 <- lm(mpg ~ wt + hp, data = mtcars)
comparison <- f_model_compare(model1, model2)
print(comparison)

# Example with GLMs.
\donttest{
model1 <- glm(am ~ wt, data = mtcars, family = binomial)
model2 <- glm(am ~ wt + hp, data = mtcars, family = binomial)
comparison <- f_model_compare(model1, model2)
print(comparison)
}

# Models can be passed in any order - the function auto-swaps if needed.
complex <- lm(mpg ~ wt + hp + qsec, data = mtcars)
simple  <- lm(mpg ~ wt, data = mtcars)
comparison <- f_model_compare(complex, simple)
# model1 will be "simple", model2 will be "complex" in the output

# Example with custom model names (useful when calling from wrapper functions).
comparison <- f_model_compare(model1, model2,
                              model1_name = "Weight only",
                              model2_name = "Weight + Horsepower")
print(comparison)

}
\seealso{
\code{\link{AIC}}, \code{\link{BIC}}, \code{\link{anova}}, \code{\link{logLik}}, \code{\link[MuMIn]{r.squaredGLMM}}
}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fbestnormalize.R
\name{f_bestNormalize}
\alias{f_bestNormalize}
\title{f_bestNormalize: Automated Data Normalization with bestNormalize}
\usage{
f_bestNormalize(
  data,
  alpha = 0.05,
  plots = FALSE,
  data_name = NULL,
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  ...
)
}
\arguments{
\item{data}{Numeric vector or single-column data frame.}

\item{alpha}{Numeric. Significance level for normality tests (default = \code{0.05}).}

\item{plots}{Logical. If \code{TRUE}, plots Q-Q plots and Histograms of the original and transformed data. Default is \code{FALSE}.}

\item{data_name}{A character string to manually set the name of the data for plot axis and reporting. Default extracts name from input object. \code{data}.}

\item{output_type}{Character string specifying the output format. Default is \code{"default"}.
\itemize{
  \item \code{"default"}: Returns the object and lets R decide whether
    to print; auto-prints if unassigned, silent if assigned to a variable.
    Use \code{print(result)} or \code{plot(result)} to display the
    returned object.
  \item \code{"console"}: Forces immediate printing to the console
    regardless of object assignment.
  \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
    file of the corresponding format. See \code{save_as},
    \code{save_in_wdir}, and \code{open_generated_files} for file
    path and opening behavior.
  \item \code{"rmd"}: Stores the raw markdown string inside the returned
    object for use in R Markdown documents.
}}

\item{save_as}{Character string specifying the output file path (without extension).
If a full path is provided, output is saved to that location.
If only a filename is given, the file is saved in \code{tempdir()}.
If only a directory is specified (providing an existing directory with trailing slash),
the file is named "data_name_transformed" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
Defaults to \code{file.path(tempdir(), "data_name_transformed.pdf")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.}

\item{close_generated_files}{Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{...}{Additional arguments passed to bestNormalize.}
}
\value{
Returns an object of class `f_bestNormalize` containing:
\itemize{
  \item \code{transformed_data} Normalized vector.
  \item \code{bestNormalize} Full bestNormalize object from original package.
  \item \code{data_name} Name of the analyzed dataset.
  \item \code{transformation_name} Name of selected transformation.
  \item \code{shapiro_original} Shapiro-Wilk test results for original data.
  \item \code{shapiro_transformed} Shapiro-Wilk test results for transformed data.
  \item \code{norm_stats} Data frame of normality statistics for all methods.
  \item \code{rmd} Rmd code if outputype = "rmd".
}
Also generates reports in 'Word', or 'pdf' files. When using output to console and plots = TRUE, the function prints QQ-plots, Histograms and a summary data transformation report. Includes print and plot methods for objects of class 'f_bestNormalize'.
}
\description{
Applies optimal normalization transformations using 'bestNormalize',
provides diagnostic checks, and generates comprehensive reports.
}
\details{
This is a wrapper around the 'bestNormalize' package. Providing a fancy output and the settings  of 'bestNormalize' are tuned based on sample size n.
If n < 100, \code{loo = TRUE}, \code{allow_orderNorm = FALSE} and \code{r} doesn't matter as \code{loo = TRUE}.
If 100 <= n < 200, \code{loo = FALSE}, \code{allow_orderNorm = TRUE} and \code{r = 50}.
If n >= 200, \code{loo = FALSE}, \code{allow_orderNorm = TRUE}, \code{r = 10}. These setting can be overwritten by user options.

This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
\itemize{
\item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
\item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
\item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.

\item If Pandoc is not found, this function may not work as intended.
}
}
\examples{
\donttest{
# Use set.seed to keep the outcome of bestNormalize stable.
set.seed(123)

# Create some skewed data (e.g., using a log-normal distribution).
skewed_data <- rlnorm(100, meanlog = 0, sdlog = 1)

# Basic usage: transform and store the full result object.
result <- f_bestNormalize(skewed_data, data_name = "Skewed log-normal data")

# Print a summary of the transformation.
print(result)

# Inspect normality statistics for all candidate transformations.
result$norm_stats

# Plot histograms and QQ-plots for original vs. transformed data.
plot(result)

# Use plots = TRUE to auto-plot when output_type = "default" (default).
result2 <- f_bestNormalize(skewed_data, plots = TRUE)

# Extract only the transformed (data) vector directly.
transformed_data <- f_bestNormalize(skewed_data)$transformed_data

# data.frame input: column name is used as data_name automatically.
df <- data.frame(measurement = skewed_data)
result_df <- f_bestNormalize(df)

# Data with NAs: NAs are preserved at their original positions.
skewed_na <- skewed_data
skewed_na[c(5, 20)] <- NA
result_na <- f_bestNormalize(skewed_na)

# Access a specific alternative transformation (first check what is available).
names(result$bestNormalize$other_transforms)
# Then extract the one you want, e.g.:
# result$bestNormalize$other_transforms$yeojohnson$x.t

# Force output to console (prints report + plots automatically).
f_bestNormalize(skewed_data, output_type = "console")

# Generate a PDF report saved to a custom path.
f_bestNormalize(skewed_data,
                output_type          = "pdf",
                save_as              = "my_report"
                )

# Generate R Markdown output for use inside a .Rmd chunk
# (set chunk option results = 'asis').
rmd_result <- f_bestNormalize(skewed_data, output_type = "rmd")
cat(rmd_result$rmd)
}

}
\references{
Peterson, C. (2025). \pkg{bestNormalize}: Flexibly calculate the best normalizing transformation for a vector.
Available at: \url{https://cran.r-project.org/package=bestNormalize}
}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fttest.R
\name{f_t_test}
\alias{f_t_test}
\alias{f_t_test.formula}
\alias{f_t_test.default}
\title{Perform multiple t-tests with optional data transformation, inspection and visualization.}
\usage{
f_t_test(x, ...)

\method{f_t_test}{formula}(
  formula,
  data = NULL,
  paired = FALSE,
  var.equal = NULL,
  conf.level = NULL,
  mu = 0,
  alternative = "two.sided",
  norm_plots = TRUE,
  transformation = TRUE,
  force_transformation = NULL,
  alpha = 0.05,
  intro_text = TRUE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  ...
)

\method{f_t_test}{default}(
  x,
  y = NULL,
  paired = FALSE,
  var.equal = NULL,
  conf.level = NULL,
  mu = 0,
  alternative = "two.sided",
  norm_plots = TRUE,
  transformation = TRUE,
  force_transformation = NULL,
  alpha = 0.05,
  intro_text = TRUE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  ...
)
}
\arguments{
\item{x}{Numeric vector of data values (one-sample or first group for two-sample),
or a formula of the form \code{response ~ group} or \code{response ~ 1}.}

\item{...}{For the formula method: additional arguments forwarded to
the row-filtering step. The arguments \code{subset} and
\code{na.action} are honored: when supplied, they are spliced
(still unevaluated) into a \code{\link[stats]{model.frame}} call
built once before the per-response loop, so the \code{subset}
expression is evaluated in the data's column scope (e.g.
\code{subset = cyl == 6} works). All responses in a
multi-response call (\code{y1 + y2 ~ group}) are then tested on
the identical row set. For the default (vector) method, \code{...}
is currently unused.}

\item{formula}{A formula specifying the model (alternative to using x/y).
\itemize{
  \item \strong{Two-sample (Independent/Paired):} \code{response ~ group}
        (where group has exactly 2 levels).
  \item \strong{One-sample:} \code{response ~ 1} or \code{response ~ NULL}.
}
More response variables can be added using \code{+} (e.g., \code{y1 + y2 ~ group}).}

\item{data}{A data frame containing the variables when using the formula interface.}

\item{paired}{Logical. If \code{TRUE}, performs a paired t-test.
\strong{Note:} For the formula interface, data must be sorted so that all
observations of group 1 appear before group 2 (AABB order). For the vector
interface, \code{x} and \code{y} must have the same length.}

\item{var.equal}{Logical or \code{NULL}. If \code{TRUE}, forces Student's t-test
(equal variances). If \code{FALSE} or \code{NULL} (default), Welch's t-test is
used. Bartlett's and Levene's tests are always reported as diagnostics but do
\strong{not} affect this choice. See Delacre, Lakens & Leys (2017).}

\item{conf.level}{Numeric. Confidence level. Default is \code{1 - alpha}.
If \code{conf.level} is specified, \code{alpha} is set to \code{1 - conf.level}.}

\item{mu}{Numeric. The true value to test against: the mean (one-sample), the mean
of differences (paired), or the difference in means (two-sample). Default is 0.
For transformed analyses, \code{mu} is forward-transformed for one-sample and
paired tests. For two-sample tests with \code{mu != 0} a warning is issued.}

\item{alternative}{Character string. \code{"two.sided"} (default),
\code{"greater"}, or \code{"less"}.}

\item{norm_plots}{Logical. If \code{TRUE}, diagnostic plots are included in the
output. Default is \code{TRUE}.}

\item{transformation}{Logical or character string. If \code{TRUE} or
\code{"boxcox"}, applies \code{f_boxcox()} when Shapiro-Wilk indicates
non-normality. If \code{"bestnormalize"}, applies \code{f_bestNormalize()}.
If \code{FALSE} or \code{"none"}, no transformation is applied.
\strong{Note:} For paired tests, \code{bestNormalize} (Yeo-Johnson) is always
used on the differences, since Box-Cox requires strictly positive values.
Default is \code{TRUE}.}

\item{force_transformation}{Character vector. Names of variables to transform
regardless of normality results.}

\item{alpha}{Numeric. Significance level. Default is \code{0.05}.}

\item{intro_text}{Logical. If \code{TRUE}, includes explanation of t-test
assumptions. Default is \code{TRUE}.}

\item{close_generated_files}{Logical. Closes open Excel/Word files before writing.
Default \code{FALSE}. \strong{Windows only.}}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{output_type}{Character string specifying the output format. Default is \code{"default"}.
\itemize{
  \item \code{"default"}: Returns the object and lets R decide whether
    to print; auto-prints if unassigned, silent if assigned to a variable.
    Use \code{print(result)} or \code{plot(result)} to display the
    returned object.
  \item \code{"console"}: Forces immediate printing to the console
    regardless of object assignment.
  \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
    file of the corresponding format. See \code{save_as},
    \code{save_in_wdir}, and \code{open_generated_files} for file
    path and opening behavior.
  \item \code{"rmd"}: Stores the raw markdown string inside the returned
    object for use in R Markdown documents.
}}

\item{save_as}{Character. Specific path/filename for output.}

\item{save_in_wdir}{Logical. Save in working directory. Default \code{FALSE}.}

\item{y}{Optional numeric vector (second group) for two-sample tests if using
the vector interface. Ignored when a formula is supplied.}
}
\value{
An object of class \code{'f_t_test'}, a named list with one element per
  response variable. Each element contains the t-test result, normality test
  results, variance diagnostic results, transformation object (if applied),
  and back-transformed confidence interval (if applicable).
}
\description{
Performs One-sample, Two-sample (Independent), or Paired t-tests on a given dataset
with options for (Box-Cox/BestNormalize) transformations, normality tests, and
visualization. Several response parameters can be analysed in sequence
(formula interface). Additionally, a vector interface similar to \code{stats::t.test()}
is supported.
}
\examples{
\donttest{
# 1. Two-sample independent Welch's t-test (default)
f_t_test(mpg ~ am, data = mtcars, output_type = "console", norm_plots = FALSE)

# 2. Multiple response variables in one call
f_t_test(mpg + hp ~ am, data = mtcars, output_type = "console", norm_plots = FALSE)

# 3. One-sample t-test: test if mean mpg equals 20
f_t_test(mpg ~ 1, data = mtcars, mu = 20,
         output_type = "console", norm_plots = FALSE)

# 4. Paired t-test (sleep dataset is already in AABB order)
f_t_test(extra ~ group, data = sleep, paired = TRUE,
         output_type = "console", norm_plots = FALSE)

# 5. Vector interface: two-sample independent
group_auto   <- mtcars$mpg[mtcars$am == 0]
group_manual <- mtcars$mpg[mtcars$am == 1]
f_t_test(group_auto, group_manual, output_type = "console", norm_plots = FALSE)

# 6. Vector interface: one-sample
f_t_test(mtcars$mpg, mu = 20, output_type = "console", norm_plots = FALSE)

# 7. Force Student's t-test (equal variances assumed)
f_t_test(mpg ~ am, data = mtcars, var.equal = TRUE,
         output_type = "console", norm_plots = FALSE)

# 8. One-sided test
f_t_test(mpg ~ am, data = mtcars, alternative = "greater",
         output_type = "console", norm_plots = FALSE)

# 9. Custom significance level (alpha = 0.01 is equivalent to conf.level = 0.99)
f_t_test(mpg ~ am, data = mtcars, alpha = 0.01,
         output_type = "console", norm_plots = FALSE)

# 10. Box-Cox transformation with back-transformed CI
# The back-transformed CI estimates the MEDIAN, not the arithmetic mean.
result <- f_t_test(hp ~ am, data = mtcars, transformation = TRUE,
                   output_type = "console", norm_plots = FALSE)
result[["hp"]]$ci_backtransformed

# 11. One-sample with non-zero mu and back-transformation
f_t_test(hp ~ 1, data = mtcars, mu = 100, transformation = TRUE,
         output_type = "console", norm_plots = FALSE)

# 12. BestNormalize transformation (set seed for reproducibility)
set.seed(123)
f_t_test(hp ~ am, data = mtcars, transformation = "bestnormalize",
         output_type = "console", norm_plots = FALSE)

# 13. Force transformation regardless of normality
f_t_test(mpg + hp ~ am, data = mtcars, force_transformation = "mpg",
         output_type = "console", norm_plots = FALSE)

# 14. Suppress transformation (diagnostic mode)
f_t_test(hp ~ am, data = mtcars, transformation = FALSE,
         output_type = "console", norm_plots = FALSE)

# 15. Access return object fields directly
result <- f_t_test(mpg + hp ~ am, data = mtcars,
                   output_type = "default", norm_plots = FALSE, intro_text = FALSE)
result[["mpg"]]$t_test          # standard htest object
result[["hp"]]$shapiro_res      # Shapiro-Wilk result
result[["hp"]]$homog_p_bartlett # Bartlett p-value (diagnostic only)
result[["hp"]]$homog_p_levene   # Levene p-value (diagnostic only)
result[["hp"]]$ci_backtransformed # back-transformed CI if transformed
}
}
\references{
Delacre, M., Lakens, D., & Leys, C. (2017). Why psychologists should by default
use Welch's t-test instead of Student's t-test.
\emph{International Review of Social Psychology}, 30(1), 92-101.
\doi{10.5334/irsp.82}
}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frenamecolumns.R
\name{f_rename_columns}
\alias{f_rename_columns}
\title{Rename Specific Columns in a Data Frame}
\usage{
f_rename_columns(df, name_map)
}
\arguments{
\item{df}{A data frame whose columns are to be renamed.}

\item{name_map}{A named vector where the names correspond to the current column names in \code{df}, and the values are the new names to assign. All names in \code{name_map} must exist in the column names of \code{df}. Yet, not all names in the data.frame have to be in \code{name_map}. This allows for selective renaming of just one or two columns.}
}
\value{
A data frame with updated column names. Columns not specified in \code{name_map} remain unchanged.
}
\description{
Renames specific columns in a data frame based on a named vector (name_map).
It ensures that only the specified columns are renamed, while others remain unchanged.
}
\details{
This function is particularly useful when you want to rename only a subset of columns in a
data frame. It performs input validation to ensure that:
\itemize{
\item \code{name_map} is a named vector.
\item All names in \code{name_map} exist as column names in \code{df}.
}
If these conditions are not met, the function will throw an error with an appropriate message.
}
\examples{
# Create a sample data frame.
df <- data.frame(a = 1:3, b = 4:6, c = 7:9)

# Define a named vector for renaming specific columns.
name_map <- c(a = "alpha", c = "gamma")

# Rename columns.
df <- f_rename_columns(df, name_map)

# View updated data frame.
print(df)

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flmer.R
\name{plot.f_lmer}
\alias{plot.f_lmer}
\title{Plot method for f_lmer objects}
\usage{
\method{plot}{f_lmer}(x, ...)
}
\arguments{
\item{x}{An object of class \code{f_lmer}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Returns \code{x} invisibly.
}
\description{
Replays the four-panel diagnostic figure (residuals vs fitted, Q-Q of
residuals, Q-Q of random-effect BLUPs, scale-location) produced by
\code{f_lmer()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fload_packages.R
\name{f_load_packages}
\alias{f_load_packages}
\title{Install and Load Multiple R Packages}
\usage{
f_load_packages(...)
}
\arguments{
\item{...}{Unquoted or quoted names of packages to be installed and loaded. These should be valid package names available on CRAN.}
}
\value{
None. The function is called for its side effects of installing and loading packages.
}
\description{
Checks if the specified packages are installed. If not, it installs them and then loads them into the global R session.
}
\details{
The function takes a list or vector indicating package names, installs any that are missing, and loads all specified packages into the global environment of the R session. It uses \code{requireNamespace()} to check for installation and \code{library()} to load the packages.
}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fboxcox.R
\name{predict.f_boxcox}
\alias{predict.f_boxcox}
\title{Predict method for f_boxcox objects}
\usage{
\method{predict}{f_boxcox}(object, newdata, inverse = FALSE, ...)
}
\arguments{
\item{object}{An object of class \code{f_boxcox}, as returned by
\code{\link{f_boxcox}}.}

\item{newdata}{A numeric vector of values to transform. For the forward
transform (\code{inverse = FALSE}), all values must be strictly positive
(Box-Cox requires \eqn{y > 0}). For the inverse transform
(\code{inverse = TRUE}), values are assumed to be on the Box-Cox
transformed scale.}

\item{inverse}{Logical. If \code{FALSE} (default), applies the forward
Box-Cox transformation to \code{newdata} using the estimated \eqn{\lambda}
from the original fit. If \code{TRUE}, reverses the transformation, mapping
values from the Box-Cox scale back to the original scale. Default is \code{FALSE}.}

\item{...}{Further arguments passed to or from other methods (currently
unused).}
}
\value{
A numeric vector of the same length as \code{newdata}, containing
  either the forward-transformed or back-transformed values.
}
\description{
Applies the fitted Box-Cox transformation to new data (forward transform),
or reverses it back to the original scale (inverse transform). This is
useful for transforming hypothesis test parameters (e.g., \code{mu}) to the
transformed scale, or for back-transforming confidence intervals to the
original scale.
}
\details{
The forward transformation applies the standard Box-Cox formula:
\deqn{y(\lambda) = \begin{cases}
  \frac{y^\lambda - 1}{\lambda}, & \lambda \neq 0 \\
  \log(y),                       & \lambda = 0
\end{cases}}

The inverse transformation reverses this process to recover the original scale:
\deqn{y = \begin{cases}
  (y(\lambda) \cdot \lambda + 1)^{1/\lambda}, & \lambda \neq 0 \\
  \exp(y(\lambda)),                           & \lambda = 0
\end{cases}}

\strong{Note on inverse validity:} When \eqn{\lambda > 0}, not all
transformed-scale values have a valid inverse. If
\eqn{y(\lambda) \cdot \lambda + 1 \leq 0}, the result is undefined and
\code{NaN} is returned with a warning.
}
\examples{
# Assuming mtcars is available and f_boxcox is loaded
bc <- f_boxcox(mtcars$hp)

# Forward: transform a hypothesis value (mu) to the Box-Cox scale
mu <- 100
mu_transformed <- predict(bc, newdata = mu)

# Inverse: back-transform a confidence interval to the original scale
ci_transformed <- c(5.5, 6.8)
predict(bc, newdata = ci_transformed, inverse = TRUE)

# Round-trip sanity check should return exactly mu (e.g., 100)
predict(bc, newdata = mu_transformed, inverse = TRUE)

}
\seealso{
\code{\link{f_boxcox}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fwilcox_test.R
\name{f_wilcox_test}
\alias{f_wilcox_test}
\alias{f_wilcox_test.formula}
\alias{f_wilcox_test.default}
\title{Perform multiple Wilcoxon rank sum and signed rank tests with inspection and visualization.}
\usage{
f_wilcox_test(x, ...)

\method{f_wilcox_test}{formula}(
  formula,
  data = NULL,
  paired = FALSE,
  conf.level = NULL,
  mu = 0,
  alternative = "two.sided",
  norm_plots = TRUE,
  alpha = 0.05,
  intro_text = TRUE,
  close_generated_files = FALSE,
  open_generated_files = TRUE,
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  ...
)

\method{f_wilcox_test}{default}(
  x,
  y = NULL,
  paired = FALSE,
  conf.level = NULL,
  mu = 0,
  alternative = "two.sided",
  norm_plots = TRUE,
  alpha = 0.05,
  intro_text = TRUE,
  close_generated_files = FALSE,
  open_generated_files = TRUE,
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  ...
)
}
\arguments{
\item{x}{Numeric vector of data values (one-sample or first group for two-sample),
or a formula of the form \code{response ~ group} or \code{response ~ 1}.}

\item{...}{For the formula method: additional arguments forwarded to
the row-filtering step. The arguments \code{subset} and
\code{na.action} are honored: when supplied, they are spliced
(still unevaluated) into a \code{\link[stats]{model.frame}} call
built once before the per-response loop, so the \code{subset}
expression is evaluated in the data's column scope (e.g.
\code{subset = cyl == 6} works). All responses in a multi-response
call (\code{y1 + y2 ~ group}) are then tested on the identical row
set. For the default (vector) method, \code{...} is currently
unused.}

\item{formula}{A formula specifying the model (alternative to using x/y).
\itemize{
  \item \strong{Two-sample (Independent/Paired):} \code{response ~ group}
        (where group has exactly 2 levels).
  \item \strong{One-sample:} \code{response ~ 1} or \code{response ~ NULL}.
}
More response variables can be added using \code{+} (e.g., \code{y1 + y2 ~ group}).}

\item{data}{A data frame containing the variables when using the formula interface.}

\item{paired}{Logical. If \code{TRUE}, performs a paired Wilcoxon test.
  \strong{Note on row order:} For the formula interface, data must be
  sorted so that the observations in the two groups match row-for-row
  (i.e. row 1 of group 1 is paired with row 1 of group 2).
  For the vector interface, \code{x} and \code{y} must have the same length.

  \strong{Note on factor level order:} The formula interface computes
  differences as \code{level1 - level2} based on factor level order,
  which defaults to alphabetical. To control the direction, set the
  reference level explicitly:
  \preformatted{
  # Set levels at creation
  group <- factor(group, levels = c("pre", "post"))

  # Or relevel an existing factor
  group <- relevel(group, ref = "pre")
  }
  A reversed level order flips the sign of the estimate and CI but
  does not affect the W statistic or p-value.}

\item{conf.level}{Numeric. Confidence level of the interval.
Default is \code{1 - alpha}. If \code{conf.level} is specified, then
\code{alpha <- 1 - conf.level}.}

\item{mu}{Numeric. The hypothesised value of the pseudo-median (one-sample) or
location shift (paired/two-sample) under H0. Default is 0.}

\item{alternative}{Character string. \code{"two.sided"} (default),
\code{"greater"}, or \code{"less"}.}

\item{norm_plots}{Logical. If \code{TRUE}, descriptive diagnostic plots are
included in the output. Default is \code{TRUE}.}

\item{alpha}{Numeric. Significance level. Default is \code{0.05}.}

\item{intro_text}{Logical. If \code{TRUE}, includes explanation about Wilcoxon
test assumptions.}

\item{close_generated_files}{Logical. Closes open Excel/Word files before writing.
Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice).
Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{output_type}{Character string specifying the output format. Default is \code{"default"}.
\itemize{
  \item \code{"default"}: Returns the object and lets R decide whether
    to print; auto-prints if unassigned, silent if assigned to a variable.
    Use \code{print(result)} or \code{plot(result)} to display the
    returned object.
  \item \code{"console"}: Forces immediate printing to the console
    regardless of object assignment.
  \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
    file of the corresponding format. See \code{save_as},
    \code{save_in_wdir}, and \code{open_generated_files} for file
    path and opening behavior.
  \item \code{"rmd"}: Stores the raw markdown string inside the returned
    object for use in R Markdown documents.
}}

\item{save_as}{Character. Specific path/filename for output.}

\item{save_in_wdir}{Logical. Save in working directory.}

\item{y}{Optional numeric vector (second group) for two-sample tests if using
the vector interface. Ignored when a formula is supplied.}
}
\value{
An object of class \code{'f_wilcox_test'}.
}
\description{
Performs One-sample (Wilcoxon signed rank), Two-sample independent (Wilcoxon rank sum /
Mann-Whitney U), or Paired (Wilcoxon signed rank) tests on a given dataset.
Several response parameters can be analysed in sequence (formula interface).
Additionally, a vector interface similar to stats::wilcox.test() is supported.
}
\section{Median vs Pseudo-median}{

By default this function calls \code{stats::wilcox.test(conf.int = TRUE)},
which bases its confidence interval and hypothesis test on the
\strong{Hodges-Lehmann estimator}, not the raw sample median. This is
standard behaviour of the Wilcoxon test, not something specific to this
function. This function explicitly labels the estimator for what it is,
because it is commonly mislabelled as "CI for the median" in textbooks
and software output.

The estimator works differently depending on the test type:

\strong{One-sample:} The \emph{pseudo-median} is the middle value of all
possible pairwise averages of your data points (including each value
paired with itself). For a perfectly symmetric distribution it equals the sample
median; for skewed data the two can differ.

\strong{Paired:} The paired differences (observation 1 minus observation 2
within each pair) are computed first, and the pseudo-median of those
differences is estimated. This is conceptually a one-sample problem
applied to the differences, not a comparison of two independent groups.
The CI is for the pseudo-median of the differences, \strong{not} for the difference
between the two separate sample medians.

\strong{Two-sample independent:} The \emph{location shift} is the median
of all \eqn{n_1 \times n_2} pairwise differences (one value from Group 1
minus one from Group 2). It answers: \emph{by how much does a randomly
chosen value from Group 1 tend to exceed a randomly chosen value from
Group 2?} When both groups have the same distributional shape it equals
the raw difference in sample medians; when shapes differ, the two values can
diverge.

In all three cases the sample median(s) are reported separately for
descriptive purposes only.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/foutliers.R
\name{f_outliers}
\alias{f_outliers}
\alias{f_outliers.numeric}
\alias{f_outliers.integer}
\alias{f_outliers.formula}
\alias{f_outliers.data.frame}
\title{Identify Outliers within Groups using Tukey's Fences}
\usage{
f_outliers(x, ...)

\method{f_outliers}{numeric}(x, ...)

\method{f_outliers}{integer}(x, ...)

\method{f_outliers}{formula}(formula, data, ...)

\method{f_outliers}{data.frame}(
  x,
  columns,
  group_vars = NULL,
  id_var = NULL,
  coef = 1.5,
  digits = NULL,
  export_to_excel = FALSE,
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  save_as = NULL,
  save_in_wdir = FALSE,
  check_input = TRUE,
  digits_excel = NULL,
  allow_integer_decimal_mix = FALSE,
  ...
)
}
\arguments{
\item{x}{A data.frame or formula (dispatches to the right method).}

\item{...}{Further arguments forwarded to \code{f_outliers.data.frame}.}

\item{formula}{A formula specifying the columns (right hand side) to be checked per subgroup(s) (left hand side).
More columns or groups can be added using \code{-} or \code{+} (e.g., \code{col1 + col2 ~ group1 + group2})
to do a sequential analysis for each column parameter.}

\item{data}{A  \code{vector}, \code{data.frame},  \code{data.table}, or  \code{tibble}.}

\item{columns}{The numerical columns to analyze if no formula is used. Can be entered
as a single character string (e.g., \code{"weight"}) or as a character vector \code{c("weight", "length"}).}

\item{group_vars}{A character vector specifying the grouping variables in \code{data}
If \code{NULL} (default), the entire dataset is treated as one group (e.g., \code{c("species", "fertilizer")}).}

\item{id_var}{(Optional) A character string naming a user-specific ID columns (e.g., \code{"EmployeeID"}).
If provided, this columns is placed at the start of the output for easy identification.}

\item{coef}{A number indicating the IQR multiplier. Default is \code{1.5} (standard Tukey fence).
Common values:
\itemize{
  \item \code{1.5}: Standard outliers (approximately +/- 2.7 SD in normal distribution).
  \item \code{3.0}: Extreme outliers.
}}

\item{digits}{Integer. Number of decimal places for the R console output.
Default is \code{2}. If \code{NULL}, no rounding is applied.
(Note: This does not affect the raw numbers exported to Excel).}

\item{export_to_excel}{Logical. If \code{TRUE}, exports results to an 'Excel' file. Default \code{FALSE}.}

\item{close_generated_files}{Logical. If \code{TRUE}, forces Excel to close before saving (Windows only). Default \code{FALSE}.}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{save_as}{Character string. Custom path or filename for the Excel export.
\itemize{
  \item If full path: Saves to that location.
  \item If filename only: Saves to \code{tempdir()} (unless \code{save_in_wdir = TRUE}).
  \item If directory: Saves as "dataname_summary.xlsx" in that directory.
}}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves to the current working directory. Default \code{FALSE}.}

\item{check_input}{Logical. If \code{TRUE}, performs validation checks on inputs. Default \code{TRUE}.}

\item{digits_excel}{Integer. Number of decimal places for the Excel file cells. Default \code{NULL} (no rounding). Defining \code{digits_excel}, sets \code{export_to_excel = TRUE} when excel output is not intended use: \code{digits} instead.}

\item{allow_integer_decimal_mix}{Logical. If \code{TRUE}, intergers in columns with a mix of integers and non-integers are displayed without decimals. Default \code{FALSE}, meaning if there are one or more numbers with decimals the whole column contains the number of decimals set by \code{digits}.}
}
\value{
A data.frame containing the identified outlier rows. Returns \code{NULL} (with a message)
  if no outliers are found.
}
\description{
`f_outliers()` scans numerical column(s) for outliers based on the Interquartile Range (IQR) method.
It can detect outliers across the entire dataset or within specified subgroups.

It returns a dataframe containing only the outlier rows, preserving the original data structure
and adding a \code{row_id} column for traceability.
}
\details{
\strong{The Outlier Logic (Tukey's Method):}
An observation is flagged as an outlier if it falls outside the calculated fences:
\itemize{
  \item \strong{Lower Fence:} \eqn{Q1 - (coef \times IQR)}
  \item \strong{Upper Fence:} \eqn{Q3 + (coef \times IQR)}
}
Where \eqn{Q1} is the 25th percentile, \eqn{Q3} is the 75th percentile, and \eqn{IQR = Q3 - Q1}.

\strong{Output Structure:}
The function returns a subset of the original \code{data}. It automatically adds a \code{row_id}
columns, which corresponds to the row number in the original dataframe. This ensures you can
strictly map the outliers back to the source data.
}
\examples{
# --- Setup: Create Dummy Data ---
set.seed(42)
df <- data.frame(
  Team       = rep(c("A", "B"), each = 20),
  Department = rep(c("Sales", "IT"), each = 10, times = 2),
  Salary     = rnorm(40, mean = 50000, sd = 2000),
  Age        = rnorm(40, mean = 35, sd = 3),
  EmployeeID = paste0("E", sprintf("\%03d", 1:40))
)

# Inject outliers
df[2,  "Salary"] <- 57000   # Mild outlier (between 1.5 and 3.0 fence)
df[1,  "Salary"] <- 100000  # Extreme high
df[35, "Salary"] <- 1000    # Extreme low

# --- Example 1: Basic detection (data.frame notation) ---
# Scan the entire dataset for Salary outliers (no grouping)
out <- f_outliers(df, columns = "Salary")
print(out)

# --- Example 2: Basic detection (formula notation) ---
# Equivalent to Example 1 using the formula interface
# LHS = column(s) to scan, RHS = grouping variable(s)
out <- f_outliers(Salary ~ 1, data = df)
print(out)

# --- Example 3: Grouped detection (both notations) ---
# Outliers are now evaluated *within* each Team separately,
# making detection sensitive to group-level distributions

# data.frame notation:
out <- f_outliers(df, columns = "Salary", group_vars = "Team")

# Formula notation (identical result):
out <- f_outliers(Salary ~ Team, data = df)
print(out)

# --- Example 4: Multi-column + multi-group (formula notation) ---
# Scan both Salary and Age for outliers, grouped by Team and Department.
# Returns a named list: one data.frame per column scanned.
out <- f_outliers(Salary + Age ~ Team + Department, data = df)
print(out)            # prints both result tables
out$Salary            # access Salary outliers directly
out$Age               # access Age outliers directly

# --- Example 5: Strict detection with a custom ID column ---
# coef = 3.0 flags only extreme outliers (the "far out" Tukey fence).
# id_var places EmployeeID first in the output for easy identification.

# data.frame notation:
out <- f_outliers(df,
                  columns   = "Salary",
                  group_vars = "Team",
                  id_var    = "EmployeeID",
                  coef      = 3.0)

# Formula notation (identical result):
out <- f_outliers(Salary ~ Team, data = df,
                  id_var = "EmployeeID",
                  coef   = 3.0)
print(out)

# --- Example 6: Sensitivity comparison ---
# Compare how coef = 1.5 (standard) vs coef = 3.0 (extreme-only)
# affects the number of flagged rows.

out_standard <- f_outliers(Salary ~ Team, data = df, coef = 1.5)
out_extreme  <- f_outliers(Salary ~ Team, data = df, coef = 3.0)

nrow(out_standard$output_df)  # 3 -- catches mild + extreme outliers
nrow(out_extreme$output_df)   # 2 -- catches extreme outliers only

# --- Example 7: Vector input ---
# Pass a column directly as a vector -- no data.frame needed.
# The column name is captured automatically from the call.

out <- f_outliers(df$Salary)
print(out)

# Works with coef and other parameters too
out <- f_outliers(df$Salary, coef = 3.0)
print(out)

# Inline vectors fall back to the column name "value"
out <- f_outliers(c(1, 2, 3, 4, 5, 100))
print(out)

}
\seealso{
\code{\link{f_remove_outliers}} to remove the rows identified by this function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fscan.R
\name{print.f_scan}
\alias{print.f_scan}
\alias{summary.f_scan}
\alias{plot.f_scan}
\title{Print method for f_scan objects}
\usage{
\method{print}{f_scan}(
  x,
  summary = TRUE,
  outliers = TRUE,
  boxplot = TRUE,
  histogram = TRUE,
  qqplot = TRUE,
  main_plot = TRUE,
  advice = TRUE,
  digits = 3,
  ...
)

\method{summary}{f_scan}(object, digits = 3, ...)

\method{plot}{f_scan}(x, boxplot = TRUE, histogram = TRUE, qqplot = TRUE, main_plot = TRUE, ...)
}
\arguments{
\item{x}{An \code{f_scan} object.}

\item{summary}{Logical. Print summary statistics table? Default \code{TRUE}.}

\item{outliers}{Logical. Print outlier table? Default \code{TRUE}.}

\item{boxplot, histogram, qqplot, main_plot}{Logical. Which plots to render?}

\item{advice}{Logical. Print statistical test recommendations? Default \code{TRUE} (shown only if \code{advice=TRUE} was used during \code{f_scan}).}

\item{digits}{Integer. Decimal places for printed tables. Default \code{3}.}

\item{...}{Further arguments passed to or from other methods. Currently
unused by the \code{f_scan} methods themselves, but accepted so the
methods remain consistent with the base generics \code{print},
\code{summary}, and \code{plot}.}

\item{object}{f_scan object to make a summary table from.}
}
\description{
Print method for f_scan objects

Summary method for f_scan objects

Plot method for f_scan objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fstat_wizard.R
\name{print.f_stat_wizard}
\alias{print.f_stat_wizard}
\title{Print method for f_stat_wizard}
\usage{
\method{print}{f_stat_wizard}(x, plots = TRUE, ...)
}
\arguments{
\item{x}{An object of class \code{f_stat_wizard}.}

\item{plots}{Logical. If \code{TRUE}, display diagnostic plots (histogram and
QQ-plot) if they were generated with \code{plots=TRUE}. Default \code{TRUE}.}

\item{...}{Additional arguments (ignored).}
}
\description{
Print method for f_stat_wizard
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fkruskal_test.R
\name{f_kruskal_test}
\alias{f_kruskal_test}
\title{Perform multiple Kruskal-Wallis tests with a user-friendly output file, do data inspection and Dunn's test (of 'rstatix') as post hoc.}
\usage{
f_kruskal_test(
  formula,
  data = NULL,
  plot = TRUE,
  alpha = 0.05,
  output_type = "default",
  save_as = NULL,
  save_in_wdir = FALSE,
  intro_text = TRUE,
  adjust = "bonferroni",
  close_generated_files = FALSE,
  open_generated_files = interactive(),
  ...
)
}
\arguments{
\item{formula}{A formula specifying the response and predictor variable (e.g., \code{response ~ predictor)}.
more response variables and predictors can be added using \code{-} or \code{+} (e.g., \code{response1 + response2 ~ predictor1 + predictor2)}. The function iterates through these combinations or response and predictors, because the Kruskal-Wallis test itself only allows one response and one predictor combination to be tested simultaneously.}

\item{data}{A \code{data.frame} containing the variables referenced in the formula.}

\item{plot}{Logical. If \code{TRUE}, generates plots (e.g., density plots and boxplots)
in the output files. Default is \code{TRUE}.}

\item{alpha}{Numeric. The significance level for the Kruskal-Wallis test and Dunn's
test. Default is  \code{0.05}.}

\item{output_type}{Character string specifying the output format. Default is \code{"default"}.
\itemize{
  \item \code{"default"}: Returns the object and lets R decide whether
    to print; auto-prints if unassigned, silent if assigned to a variable.
    Use \code{print(result)} or \code{plot(result)} to display the
    returned object.
  \item \code{"console"}: Forces immediate printing to the console
    regardless of object assignment.
  \item \code{"pdf"}, \code{"word"}, \code{"excel"}: Saves results to a
    file of the corresponding format. See \code{save_as},
    \code{save_in_wdir}, and \code{open_generated_files} for file
    path and opening behavior.
  \item \code{"rmd"}: Stores the raw markdown string inside the returned
    object for use in R Markdown documents.
}}

\item{save_as}{Character string specifying the output file path (without extension).
If a full path is provided, output is saved to that location.
If only a filename is given, the file is saved in \code{tempdir()}.
If only a directory is specified (providing an existing directory with trailing slash),
the file is named "dataname_Kruskal_Wallis_output" in that directory. If an extension is provided the output format specified with option "output_type" will be overruled.
Defaults to \code{file.path(tempdir(), "dataname_summary.pdf")}.}

\item{save_in_wdir}{Logical. If \code{TRUE}, saves the file in the working directory. Default is \code{FALSE}, this avoid unintended changes to the global environment. If \code{save_as} location is specified \code{save_in_wdir} is overwritten by \code{save_as}.}

\item{intro_text}{Logical. If \code{TRUE}, includes a section about Kruskal-Wallis test assumptions in the output document. Default is \code{TRUE}.}

\item{adjust}{Character string. Adjustment method for pairwise comparisons in Dunn's test. Options include \code{"holm", "hommel", "bonferroni", "hochberg", "bh", "by", "fdr"} or \code{"none"}. Default is \code{"bonferroni"}, if you don't want to adjust the p value (not recommended), use \code{adjust = "none"}.}

\item{close_generated_files}{Logical. Closes open Excel or Word (NOT pdf) files before writing, depending on the output format. Works on Windows (taskkill), macOS (pkill) and Linux (pkill/soffice). Default \code{FALSE}. \strong{WARNING:} Always save your work before using this option!!}

\item{open_generated_files}{Logical. Whether to open the generated output
files after creation. Defaults to \code{TRUE} in an interactive R session
and \code{FALSE} otherwise (e.g. in scripts or automated pipelines).
Set to \code{TRUE} or \code{FALSE} to override this behaviour explicitly.}

\item{...}{Additional arguments forwarded to \code{\link[stats]{kruskal.test}}.
The arguments \code{subset} and \code{na.action} are honored: when
supplied, they are applied via \code{\link[stats]{model.frame}} so
that the descriptive summary table, density plot, boxplot, Dunn's
post hoc test and the Kruskal-Wallis test itself all see the exact
same row set.}
}
\value{
An object of class 'f_kruskal_test' (a named list, one entry per
  response-predictor combination) containing:
  \describe{
    \item{kruskal.test}{The \code{htest} object from \code{kruskal.test()}.}
    \item{dunn_test}{Data frame of pairwise Dunn's test results from \code{rstatix::dunn_test()}.}
    \item{summary_table}{Descriptive statistics with compact letter display (Letters column).}
    \item{alpha}{The significance level used.}
    \item{DunnTest_adjust}{The p-value adjustment method used.}
    \item{distributions}{ggplot density plot (if \code{plot = TRUE}).}
    \item{Boxplot}{ggplot boxplot with CLD letters (if \code{plot = TRUE}).}
  }

Using the option \code{output_type}, it can also generate output in the form of: R Markdown code, 'Word', 'pdf', or 'Excel' files. Includes print and plot methods for 'f_kruskal_test' objects.
}
\description{
Performs the Kruskal-Wallis rank sum test to assess whether there are statistically significant differences in the distributions (mean ranks) of three or more independent groups. It provides detailed outputs, including plots, assumption checks, and post hoc analyses using Dunn's test. Results can be saved in various formats ('pdf', 'Word', 'Excel', or console only) with customizable output options.
}
\details{
This function offers a comprehensive workflow for non-parametric analysis using the Kruskal-Wallis test:
\itemize{
\item Assumption Checks: Optionally includes a summary of assumptions in the output.
\item Visualization: Generates density plots and boxplots to visualize group distributions.
\item Post hoc Analysis: Conducts Dunn's test with specified correction methods if significant differences are found.
}
-----------\cr

Output files are generated in the format specified by \code{output_type =} and saved to the working directory, options are \code{"pdf", "word"} or \code{"excel"}. If \code{output_type = "rmd"} is used it is advised to use it in a chunk with \{r, echo=FALSE, results='asis'\}

This function requires [Pandoc](https://github.com/jgm/pandoc/releases/tag) (version 1.12.3 or higher), a universal document converter.
\itemize{
\item \bold{Windows:} Install Pandoc and ensure the installation folder \cr (e.g., "C:/Users/your_username/AppData/Local/Pandoc") is added to your system PATH.
\item \bold{macOS:} If using Homebrew, Pandoc is typically installed in "/usr/local/bin". Alternatively, download the .pkg installer and verify that the binary's location is in your PATH.
\item \bold{Linux:} Install Pandoc through your distribution's package manager (commonly installed in "/usr/bin" or "/usr/local/bin") or manually, and ensure the directory containing Pandoc is in your PATH.

\item If Pandoc is not found, this function may not work as intended.
}
}
\section{Multiple Testing Across Response Variables}{

When several response variables are analysed in a single call
(e.g. \code{y1 + y2 + y3 ~ treatment}), each Kruskal-Wallis test is an
independent null-hypothesis test at level \code{alpha}. The post hoc
adjustment (e.g. \code{adjust = "bonferroni"}) only controls the
family-wise error rate \strong{within} one test (across pairwise Dunn
comparisons for that response). It does \strong{not} protect against
the inflation of Type I error \strong{across} the set of responses.

\strong{Practical implication:} With \eqn{k} independent response
variables all tested at \eqn{\alpha = 0.05}, the probability of
obtaining at least one false positive is
\eqn{1-(1-0.05)^k}, which reaches ~40\% for \eqn{k = 10}.
}

\examples{
# Example usage:
data(iris)

# Perform Kruskal-Wallis test on Sepal.Length and Sepal.Width by Species
# with "holm" correction for posthoc dunn_test, without showing the output.
output <- f_kruskal_test(
               Sepal.Width + Sepal.Length ~ Species,
               data = iris,
               plot = FALSE,
               output_type = "word",
               adjust = "holm"
               )

# Save Kruskal-Wallis test and posthoc to Excel sheets: Sepal.Width and Sepal.Length.
f_kruskal_out <- f_kruskal_test(
                     Sepal.Width + Sepal.Length ~ Species,
                     data = iris,
                     plot = FALSE,
                     output_type = "excel",
                     adjust = "holm"
                     )

}
\author{
Sander H. van Delden  \email{plantmind@proton.me} \cr
}
